
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header$

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

      SUBROUTINE HRPRODLOSS

C***********************************************************************
C
C  FUNCTION: To compute the production and loss rates for EBI species
C
C  PRECONDITIONS: For the RACM2 family of mechanisms
C
C  RETURN VALUES: None
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, March 4, 2013
C
C***********************************************************************
      USE HRDATA

      IMPLICIT NONE


C.....INCLUDES: NONE


C.....ARGUMENTS: NONE


C.....PARAMETERS: NONE


C.....EXTERNAL FUNCTIONS: NONE


C.....SAVED VARIABLES: NONE
      CHARACTER( 16 ), SAVE  ::  PNAME = 'HRPRODLOSS'   ! Program name


C.....LOCAL VARIABLES:

      INTEGER ESP

C***********************************************************************


      PROD( H2O2    ) =           RXRAT(  45 )      ! HO2+HO2=H2O2
     &                 +          RXRAT(  46 )      ! HO2+HO2=H2O2
     &                 +  0.068 * RXRAT( 127 )      ! OLT+O3=0.068*H2O2+0.32*HO2+...
     &                 +  0.026 * RXRAT( 128 )      ! OLI+O3=0.026*H2O2+0.07*HO2+...
     &                 +  0.001 * RXRAT( 129 )      ! DIEN+O3=0.001*H2O2+0.28*HO+...
     &                 +  0.090 * RXRAT( 130 )      ! ISO+O3=0.09*H2O2+0.25*HO2+...
     &                 +  0.020 * RXRAT( 131 )      ! API+O3=0.02*H2O2+0.1*HO2+...
     &                 +  0.020 * RXRAT( 132 )      ! LIM+O3=0.02*H2O2+0.1*HO2+...
      LOSS( H2O2    ) =           RXRAT(   3 )      ! H2O2=2*HO
     &                 +          RXRAT(  47 )      ! H2O2+HO=HO2

      PROD( HNO3    ) =           RXRAT(  51 )      ! NO+HO2=HNO3
     &                 +          RXRAT(  56 )      ! NO2+HO=HNO3
     &                 +  0.300 * RXRAT(  59 )      ! NO3+HO2=0.3*HNO3+0.7*NO2+...
     &                 +  2.000 * RXRAT(  65 )      ! N2O5=2*HNO3
     &                 +          RXRAT( 148 )      ! HCHO+NO3=HNO3+CO+HO2
     &                 +          RXRAT( 149 )      ! ACD+NO3=HNO3+ACO3
     &                 +          RXRAT( 150 )      ! ALD+NO3=HNO3+RCO3
     &                 +  0.320 * RXRAT( 151 )      ! MACR+NO3=0.32*HNO3+...
     &                 +          RXRAT( 153 )      ! GLY+NO3=HNO3+2*CO+HO2
     &                 +          RXRAT( 154 )      ! MGLY+NO3=HNO3+CO+ACO3
     &                 +  0.500 * RXRAT( 155 )      ! PHEN+NO3=0.5*HNO3+0.1*ADDC+...
     &                 +  0.500 * RXRAT( 156 )      ! CSL+NO3=0.5*HNO3+0.1*ADDC+...
     &                 +  0.500 * RXRAT( 157 )      ! EPX+NO3=0.5*HNO3+1.5*HO2+...
     &                 +          RXRAT( 158 )      ! MCT+NO3=HNO3+MCTO
      LOSS( HNO3    ) =           RXRAT(   8 )      ! HNO3=HO+NO2
     &                 +          RXRAT(  57 )      ! HNO3+HO=NO3

      PROD( HCHO    ) =   0.350 * RXRAT(  15 )      ! UALD=0.35*HCHO+0.784*ACO3+...
     &                 +          RXRAT(  18 )      ! HKET=HCHO+ACO3+HO2
     &                 +  0.670 * RXRAT(  19 )      ! MACR=0.67*HCHO+0.66*HO2+...
     &                 +          RXRAT(  22 )      ! GLY=HCHO+CO
     &                 +          RXRAT(  28 )      ! OP1=HCHO+HO2+HO
     &                 +          RXRAT( 109 )      ! MOH+HO=HCHO+HO2
     &                 +  0.350 * RXRAT( 113 )      ! OP1+HO=0.35*HCHO+0.65*MO2+...
     &                 +  0.350 * RXRAT( 119 )      ! PAA+HO=0.35*HCHO+0.65*ACO3+...
     &                 +          RXRAT( 120 )      ! PAN+HO=HCHO+NO3+XO2
     &                 +          RXRAT( 121 )      ! PPN+HO=HCHO+NO3+XO2
     &                 +  0.070 * RXRAT( 125 )      ! ISON+HO=0.07*HCHO+...
     &                 +          RXRAT( 126 )      ! ETE+O3=HCHO+0.15*HO2+...
     &                 +  0.560 * RXRAT( 127 )      ! OLT+O3=0.56*HCHO+0.32*HO2+...
     &                 +  0.090 * RXRAT( 128 )      ! OLI+O3=0.09*HCHO+0.07*HO2+...
     &                 +  0.900 * RXRAT( 129 )      ! DIEN+O3=0.9*HCHO+0.28*HO+...
     &                 +  0.580 * RXRAT( 130 )      ! ISO+O3=0.58*HCHO+0.25*HO2+...
     &                 +  0.040 * RXRAT( 132 )      ! LIM+O3=0.04*HCHO+0.1*HO2+...
     &                 +  0.100 * RXRAT( 134 )      ! MVK+O3=0.1*HCHO+0.11*HO2+...
     &                 +  0.080 * RXRAT( 135 )      ! UALD+O3=0.08*HCHO+...
     &                 +  0.050 * RXRAT( 136 )      ! DCB1+O3=0.05*HCHO+HO2+...
     &                 +  0.050 * RXRAT( 137 )      ! DCB2+O3=0.05*HCHO+HO2+...
     &                 +  0.680 * RXRAT( 151 )      ! MACR+NO3=0.68*HCHO+...
     &                 +  0.332 * RXRAT( 152 )      ! UALD+NO3=0.332*HCHO+XO2+...
     &                 +          RXRAT( 172 )      ! MO2+NO=HCHO+NO2+HO2
     &                 +  0.018 * RXRAT( 175 )      ! HC5P+NO=0.018*HCHO+...
     &                 +  1.600 * RXRAT( 177 )      ! ETEP+NO=1.6*HCHO+NO2+HO2+...
     &                 +  0.780 * RXRAT( 178 )      ! OLTP+NO=0.78*HCHO+0.97*NO2+...
     &                 +  0.200 * RXRAT( 188 )      ! ISOP+NO=0.2*HCHO+0.88*NO2+...
     &                 +  0.230 * RXRAT( 189 )      ! APIP+NO=0.23*HCHO+0.82*NO2+...
     &                 +  0.430 * RXRAT( 190 )      ! LIMP+NO=0.43*HCHO+...
     &                 +          RXRAT( 193 )      ! ACTP+NO=HCHO+NO2+ACO3
     &                 +  0.330 * RXRAT( 194 )      ! MEKP+NO=0.33*HCHO+NO2+...
     &                 +  0.650 * RXRAT( 196 )      ! MACP+NO=0.65*HCHO+...
     &                 +  0.500 * RXRAT( 197 )      ! MCP+NO=0.5*HCHO+0.5*HO2+...
     &                 +  0.300 * RXRAT( 198 )      ! MVKP+NO=0.3*HCHO+0.7*ACO3+...
     &                 +  0.030 * RXRAT( 199 )      ! UALP+NO=0.03*HCHO+NO2+...
     &                 +  0.287 * RXRAT( 206 )      ! OLND+NO=0.287*HCHO+2*NO2+...
     &                 +  0.150 * RXRAT( 233 )      ! ACTP+HO2=0.15*HCHO+...
     &                 +  1.370 * RXRAT( 248 )      ! MO2+MO2=1.37*HCHO+0.74*HO2+...
     &                 +  0.750 * RXRAT( 249 )      ! ETHP+MO2=0.75*HCHO+HO2+...
     &                 +  0.827 * RXRAT( 250 )      ! HC3P+MO2=0.827*HCHO+...
     &                 +  0.777 * RXRAT( 251 )      ! HC5P+MO2=0.777*HCHO+...
     &                 +  0.750 * RXRAT( 252 )      ! HC8P+MO2=0.75*HCHO+...
     &                 +  1.950 * RXRAT( 253 )      ! ETEP+MO2=1.95*HCHO+HO2+...
     &                 +  1.500 * RXRAT( 254 )      ! OLTP+MO2=1.5*HCHO+HO2+...
     &                 +  0.750 * RXRAT( 255 )      ! OLIP+MO2=0.75*HCHO+HO2+...
     &                 +          RXRAT( 256 )      ! BENP+MO2=HCHO+0.459*DCB3+...
     &                 +          RXRAT( 257 )      ! TLP1+MO2=HCHO+HO2+BALD
     &                 +          RXRAT( 258 )      ! TOLP+MO2=HCHO+2*HO2+...
     &                 +          RXRAT( 259 )      ! PER1+MO2=HCHO+2*HO2+MGLY+DCB1
     &                 +          RXRAT( 260 )      ! XYL1+MO2=HCHO+HO2+BALD
     &                 +          RXRAT( 261 )      ! XYLP+MO2=HCHO+2*HO2+DCB2
     &                 +          RXRAT( 262 )      ! PER2+MO2=HCHO+2*HO2+MGLY+...
     &                 +          RXRAT( 263 )      ! XYOP+MO2=HCHO+2*HO2+...
     &                 +  1.310 * RXRAT( 264 )      ! ISOP+MO2=1.31*HCHO+HO2+...
     &                 +  0.750 * RXRAT( 265 )      ! APIP+MO2=0.75*HCHO+HO2+...
     &                 +  1.040 * RXRAT( 266 )      ! LIMP+MO2=1.04*HCHO+HO2+...
     &                 +          RXRAT( 267 )      ! ACO3+MO2=HCHO+0.9*MO2+...
     &                 +          RXRAT( 268 )      ! RCO3+MO2=HCHO+0.9*MO2+...
     &                 +  1.500 * RXRAT( 269 )      ! ACTP+MO2=1.5*HCHO+0.5*ACO3+...
     &                 +          RXRAT( 270 )      ! MEKP+MO2=HCHO+0.834*HO2+...
     &                 +  0.750 * RXRAT( 271 )      ! KETP+MO2=0.75*HCHO+HO2+...
     &                 +  1.660 * RXRAT( 272 )      ! MACP+MO2=1.66*HCHO+...
     &                 +  1.500 * RXRAT( 273 )      ! MCP+MO2=1.5*HCHO+HO2+NO2+...
     &                 +  1.500 * RXRAT( 274 )      ! MVKP+MO2=1.5*HCHO+...
     &                 +  0.773 * RXRAT( 275 )      ! UALP+MO2=0.773*HCHO+...
     &                 +          RXRAT( 276 )      ! BALP+MO2=HCHO+BAL1+HO2
     &                 +          RXRAT( 277 )      ! BAL1+MO2=HCHO+BAL2+HO2
     &                 +          RXRAT( 278 )      ! ADDC+MO2=HCHO+2*HO2+...
     &                 +          RXRAT( 279 )      ! MCTP+MO2=HCHO+MCTO+HO2
     &                 +          RXRAT( 280 )      ! ORAP+MO2=HCHO+HO2+GLY
     &                 +          RXRAT( 281 )      ! OLNN+MO2=HCHO+2*HO2+ONIT
     &                 +  0.965 * RXRAT( 282 )      ! OLND+MO2=0.965*HCHO+...
     &                 +          RXRAT( 283 )      ! ADCN+MO2=HCHO+0.7*NO2+HO2+...
     &                 +          RXRAT( 284 )      ! XO2+MO2=HCHO+HO2
     &                 +  0.130 * RXRAT( 286 )      ! HC3P+ACO3=0.13*HCHO+...
     &                 +  0.042 * RXRAT( 287 )      ! HC5P+ACO3=0.042*HCHO+...
     &                 +  1.600 * RXRAT( 289 )      ! ETEP+ACO3=1.6*HCHO+0.5*MO2+...
     &                 +          RXRAT( 290 )      ! OLTP+ACO3=HCHO+0.5*MO2+...
     &                 +  1.048 * RXRAT( 300 )      ! ISOP+ACO3=1.048*HCHO+...
     &                 +  0.385 * RXRAT( 302 )      ! LIMP+ACO3=0.385*HCHO+...
     &                 +          RXRAT( 305 )      ! ACTP+ACO3=HCHO+0.5*ACO3+...
     &                 +  0.330 * RXRAT( 306 )      ! MEKP+ACO3=0.33*HCHO+...
     &                 +          RXRAT( 308 )      ! MACP+ACO3=HCHO+0.5*MO2+...
     &                 +          RXRAT( 309 )      ! MCP+ACO3=HCHO+0.5*HO2+NO2+...
     &                 +          RXRAT( 310 )      ! MVKP+ACO3=HCHO+0.5*MO2+...
     &                 +  0.030 * RXRAT( 311 )      ! UALP+ACO3=0.03*HCHO+...
     &                 +  0.287 * RXRAT( 318 )      ! OLND+ACO3=0.287*HCHO+NO2+...
     &                 +          RXRAT( 322 )      ! MO2+NO3=HCHO+HO2+NO2
     &                 +  0.024 * RXRAT( 325 )      ! HC5P+NO3=0.024*HCHO+...
     &                 +  1.600 * RXRAT( 327 )      ! ETEP+NO3=1.6*HCHO+NO2+HO2+...
     &                 +  0.790 * RXRAT( 328 )      ! OLTP+NO3=0.79*HCHO+...
     &                 +  0.750 * RXRAT( 338 )      ! ISOP+NO3=0.75*HCHO+NO2+HO2+...
     &                 +  0.385 * RXRAT( 340 )      ! LIMP+NO3=0.385*HCHO+NO2+...
     &                 +          RXRAT( 343 )      ! ACTP+NO3=HCHO+NO2+ACO3
     &                 +  0.330 * RXRAT( 344 )      ! MEKP+NO3=0.33*HCHO+NO2+...
     &                 +          RXRAT( 346 )      ! MACP+NO3=HCHO+0.538*ACO3+...
     &                 +          RXRAT( 347 )      ! MCP+NO3=HCHO+HO2+NO2+HKET
     &                 +  0.300 * RXRAT( 348 )      ! MVKP+NO3=0.3*HCHO+0.7*ACO3+...
     &                 +  0.030 * RXRAT( 349 )      ! UALP+NO3=0.03*HCHO+NO2+...
     &                 +  0.287 * RXRAT( 356 )      ! OLND+NO3=0.287*HCHO+2*NO2+...
     &                 +  0.202 * RXRAT( 359 )      ! OLNN+OLND=0.202*HCHO+...
     &                 +  0.504 * RXRAT( 360 )      ! OLND+OLND=0.504*HCHO+NO2+...
      LOSS( HCHO    ) =           RXRAT(  10 )      ! HCHO=CO
     &                 +          RXRAT(  11 )      ! HCHO=HO2+HO2+CO
     &                 +          RXRAT(  89 )      ! HCHO+HO=HO2+CO
     &                 +          RXRAT( 148 )      ! HCHO+NO3=HO2+CO+HNO3

      PROD( CO      ) =           RXRAT(  10 )      ! HCHO=CO
     &                 +          RXRAT(  11 )      ! HCHO=CO+HO2+HO2
     &                 +          RXRAT(  12 )      ! ACD=CO+MO2+HO2
     &                 +          RXRAT(  13 )      ! ALD=CO+ETHP+HO2
     &                 +  1.220 * RXRAT(  15 )      ! UALD=1.22*CO+0.784*ACO3+...
     &                 +  0.670 * RXRAT(  19 )      ! MACR=0.67*CO+0.66*HO2+...
     &                 +  0.700 * RXRAT(  20 )      ! MVK=0.7*CO+0.3*MACP+...
     &                 +  2.000 * RXRAT(  21 )      ! GLY=2*CO
     &                 +          RXRAT(  22 )      ! GLY=CO+HCHO
     &                 +  2.000 * RXRAT(  23 )      ! GLY=2*CO+2*HO2
     &                 +          RXRAT(  24 )      ! MGLY=CO+ACO3+HO2
     &                 +          RXRAT(  25 )      ! DCB1=CO+0.25*ACO3+0.2*XO2+...
     &                 +          RXRAT(  26 )      ! DCB2=CO+0.25*ACO3+0.2*XO2+...
     &                 +          RXRAT(  27 )      ! BALD=CO+HO2+CHO
     &                 +  0.350 * RXRAT(  80 )      ! ACE+HO=0.35*CO+0.35*HO2+...
     &                 +          RXRAT(  89 )      ! HCHO+HO=CO+HO2
     &                 +  2.000 * RXRAT(  99 )      ! GLY+HO=2*CO+HO2
     &                 +          RXRAT( 100 )      ! MGLY+HO=CO+ACO3
     &                 +  0.330 * RXRAT( 101 )      ! DCB1+HO=0.33*CO+0.52*HO2+...
     &                 +  0.330 * RXRAT( 102 )      ! DCB2+HO=0.33*CO+0.52*HO2+...
     &                 +  0.110 * RXRAT( 103 )      ! DCB3+HO=0.11*CO+0.21*MACP+...
     &                 +          RXRAT( 107 )      ! EPX+HO=CO+XO2+HO2+ALD
     &                 +  0.430 * RXRAT( 126 )      ! ETE+O3=0.43*CO+0.15*HO2+...
     &                 +  0.430 * RXRAT( 127 )      ! OLT+O3=0.43*CO+0.32*HO2+...
     &                 +  0.370 * RXRAT( 128 )      ! OLI+O3=0.37*CO+0.07*HO2+...
     &                 +  0.360 * RXRAT( 129 )      ! DIEN+O3=0.36*CO+0.28*HO+...
     &                 +  0.140 * RXRAT( 130 )      ! ISO+O3=0.14*CO+0.25*HO2+...
     &                 +  0.140 * RXRAT( 131 )      ! API+O3=0.14*CO+0.1*HO2+...
     &                 +  0.140 * RXRAT( 132 )      ! LIM+O3=0.14*CO+0.1*HO2+...
     &                 +  0.220 * RXRAT( 133 )      ! MACR+O3=0.22*CO+0.14*HO2+...
     &                 +  0.560 * RXRAT( 134 )      ! MVK+O3=0.56*CO+0.11*HO2+...
     &                 +  0.243 * RXRAT( 135 )      ! UALD+O3=0.243*CO+0.072*HO2+...
     &                 +  1.500 * RXRAT( 136 )      ! DCB1+O3=1.5*CO+HO2+...
     &                 +  1.500 * RXRAT( 137 )      ! DCB2+O3=1.5*CO+HO2+...
     &                 +  1.500 * RXRAT( 138 )      ! DCB3+O3=1.5*CO+HO2+0.05*HO+...
     &                 +  1.500 * RXRAT( 139 )      ! EPX+O3=1.5*CO+1.5*HO2+...
     &                 +          RXRAT( 148 )      ! HCHO+NO3=CO+HO2+HNO3
     &                 +  0.668 * RXRAT( 152 )      ! UALD+NO3=0.668*CO+XO2+HO2+...
     &                 +  2.000 * RXRAT( 153 )      ! GLY+NO3=2*CO+HO2+HNO3
     &                 +          RXRAT( 154 )      ! MGLY+NO3=CO+ACO3+HNO3
     &                 +  1.500 * RXRAT( 157 )      ! EPX+NO3=1.5*CO+1.5*HO2+...
     &                 +  0.650 * RXRAT( 196 )      ! MACP+NO=0.65*CO+0.35*ACO3+...
     &                 +  0.610 * RXRAT( 199 )      ! UALP+NO=0.61*CO+NO2+HO2+...
     &                 +  0.500 * RXRAT( 272 )      ! MACP+MO2=0.5*CO+0.269*ACO3+...
     &                 +  0.305 * RXRAT( 275 )      ! UALP+MO2=0.305*CO+HO2+...
     &                 +  0.500 * RXRAT( 308 )      ! MACP+ACO3=0.5*CO+0.5*MO2+...
     &                 +  0.500 * RXRAT( 311 )      ! UALP+ACO3=0.5*CO+0.5*MO2+...
     &                 +          RXRAT( 346 )      ! MACP+NO3=CO+0.538*ACO3+...
     &                 +  0.610 * RXRAT( 349 )      ! UALP+NO3=0.61*CO+NO2+HO2+...
      LOSS( CO      ) =           RXRAT(  70 )      ! CO+HO=HO2

      PROD( ACD     ) =           RXRAT( 110 )      ! EOH+HO=ACD+HO2
     &                 +  0.184 * RXRAT( 111 )      ! ROH+HO=0.184*ACD+0.719*ALD+HO2
     &                 +  0.010 * RXRAT( 127 )      ! OLT+O3=0.01*ACD+0.32*HO2+...
     &                 +  0.457 * RXRAT( 128 )      ! OLI+O3=0.457*ACD+0.07*HO2+...
     &                 +  0.420 * RXRAT( 135 )      ! UALD+O3=0.42*ACD+0.072*HO2+...
     &                 +          RXRAT( 173 )      ! ETHP+NO=ACD+NO2+HO2
     &                 +  0.504 * RXRAT( 174 )      ! HC3P+NO=0.504*ACD+...
     &                 +  0.045 * RXRAT( 175 )      ! HC5P+NO=0.045*ACD+...
     &                 +  0.012 * RXRAT( 178 )      ! OLTP+NO=0.012*ACD+0.97*NO2+...
     &                 +  0.810 * RXRAT( 179 )      ! OLIP+NO=0.81*ACD+0.95*NO2+...
     &                 +  0.750 * RXRAT( 249 )      ! ETHP+MO2=0.75*ACD+...
     &                 +          RXRAT( 285 )      ! ETHP+ACO3=ACD+0.5*MO2+...
     &                 +          RXRAT( 323 )      ! ETHP+NO3=ACD+NO2+HO2
     &                 +  0.519 * RXRAT( 324 )      ! HC3P+NO3=0.519*ACD+...
     &                 +  0.048 * RXRAT( 325 )      ! HC5P+NO3=0.048*ACD+...
     &                 +  0.020 * RXRAT( 328 )      ! OLTP+NO3=0.02*ACD+...
     &                 +  0.850 * RXRAT( 329 )      ! OLIP+NO3=0.85*ACD+0.72*ALD+...
      LOSS( ACD     ) =           RXRAT(  12 )      ! ACD=HO2+MO2+CO
     &                 +          RXRAT(  90 )      ! ACD+HO=ACO3
     &                 +          RXRAT( 149 )      ! ACD+NO3=ACO3+HNO3

      PROD( MO2     ) =           RXRAT(  12 )      ! ACD=MO2+HO2+CO
     &                 +          RXRAT(  14 )      ! ACT=MO2+ACO3
     &                 +  0.500 * RXRAT(  16 )      ! MEK=0.5*MO2+0.5*ETHP+ACO3
     &                 +  0.300 * RXRAT(  20 )      ! MVK=0.3*MO2+0.3*MACP+...
     &                 +          RXRAT(  30 )      ! PAA=MO2+HO
     &                 +          RXRAT(  33 )      ! PAN=MO2+NO3
     &                 +          RXRAT(  71 )      ! HO=MO2
     &                 +  0.650 * RXRAT( 113 )      ! OP1+HO=0.65*MO2+0.35*HO+...
     &                 +  0.640 * RXRAT( 118 )      ! ORA2+HO=0.64*MO2+0.36*ORAP
     &                 +  0.080 * RXRAT( 127 )      ! OLT+O3=0.08*MO2+0.32*HO2+...
     &                 +  0.320 * RXRAT( 128 )      ! OLI+O3=0.32*MO2+0.07*HO2+...
     &                 +  0.030 * RXRAT( 129 )      ! DIEN+O3=0.03*MO2+0.28*HO+...
     &                 +  0.080 * RXRAT( 130 )      ! ISO+O3=0.08*MO2+0.25*HO2+...
     &                 +  0.008 * RXRAT( 135 )      ! UALD+O3=0.008*MO2+...
     &                 +  0.131 * RXRAT( 174 )      ! HC3P+NO=0.131*MO2+0.66*HO2+...
     &                 +  0.051 * RXRAT( 175 )      ! HC5P+NO=0.051*MO2+0.2*HO2+...
     &                 +          RXRAT( 191 )      ! ACO3+NO=MO2+NO2
     &                 +  0.650 * RXRAT( 196 )      ! MACP+NO=0.65*MO2+0.35*ACO3+...
     &                 +  0.440 * RXRAT( 231 )      ! ACO3+HO2=0.44*MO2+0.44*HO+...
     &                 +  0.500 * RXRAT( 285 )      ! ETHP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  0.580 * RXRAT( 286 )      ! HC3P+ACO3=0.58*MO2+...
     &                 +  0.518 * RXRAT( 287 )      ! HC5P+ACO3=0.518*MO2+...
     &                 +  0.500 * RXRAT( 288 )      ! HC8P+ACO3=0.5*MO2+...
     &                 +  0.500 * RXRAT( 289 )      ! ETEP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  0.500 * RXRAT( 290 )      ! OLTP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  0.500 * RXRAT( 291 )      ! OLIP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +          RXRAT( 292 )      ! BENP+ACO3=MO2+0.6*HO2+...
     &                 +          RXRAT( 293 )      ! TLP1+ACO3=MO2+BALD
     &                 +          RXRAT( 294 )      ! TOLP+ACO3=MO2+HO2+DCB2
     &                 +          RXRAT( 295 )      ! PER1+ACO3=MO2+HO2+MGLY+DCB1
     &                 +          RXRAT( 296 )      ! XYL1+ACO3=MO2+BALD
     &                 +          RXRAT( 297 )      ! XYLP+ACO3=MO2+HO2+DCB2
     &                 +          RXRAT( 298 )      ! PER2+ACO3=MO2+HO2+MGLY+...
     &                 +          RXRAT( 299 )      ! XYOP+ACO3=MO2+HO2+...
     &                 +  0.500 * RXRAT( 300 )      ! ISOP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  0.500 * RXRAT( 301 )      ! APIP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  0.500 * RXRAT( 302 )      ! LIMP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  2.000 * RXRAT( 303 )      ! ACO3+ACO3=2*MO2
     &                 +          RXRAT( 304 )      ! RCO3+ACO3=MO2+ETHP
     &                 +  0.500 * RXRAT( 305 )      ! ACTP+ACO3=0.5*MO2+0.5*ACO3+...
     &                 +  0.500 * RXRAT( 306 )      ! MEKP+ACO3=0.5*MO2+0.33*HO2+...
     &                 +  0.500 * RXRAT( 307 )      ! KETP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  0.500 * RXRAT( 308 )      ! MACP+ACO3=0.5*MO2+...
     &                 +  0.500 * RXRAT( 309 )      ! MCP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  0.500 * RXRAT( 310 )      ! MVKP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +  0.500 * RXRAT( 311 )      ! UALP+ACO3=0.5*MO2+0.5*HO2+...
     &                 +          RXRAT( 312 )      ! BALP+ACO3=MO2+BAL1
     &                 +          RXRAT( 313 )      ! BAL1+ACO3=MO2+BAL2
     &                 +          RXRAT( 314 )      ! ADDC+ACO3=MO2+2*HO2+...
     &                 +          RXRAT( 315 )      ! MCTP+ACO3=MO2+HO2+MCTO
     &                 +          RXRAT( 316 )      ! ORAP+ACO3=MO2+GLY
     &                 +          RXRAT( 317 )      ! OLNN+ACO3=MO2+HO2+ONIT
     &                 +  0.500 * RXRAT( 318 )      ! OLND+ACO3=0.5*MO2+NO2+...
     &                 +          RXRAT( 319 )      ! ADCN+ACO3=MO2+HO2+0.7*NO2+...
     &                 +          RXRAT( 320 )      ! XO2+ACO3=MO2
     &                 +  0.140 * RXRAT( 324 )      ! HC3P+NO3=0.14*MO2+...
     &                 +  0.055 * RXRAT( 325 )      ! HC5P+NO3=0.055*MO2+...
     &                 +          RXRAT( 341 )      ! ACO3+NO3=MO2+NO2
      LOSS( MO2     ) =           RXRAT( 172 )      ! MO2+NO=HO2+NO2+HCHO
     &                 +          RXRAT( 212 )      ! MO2+HO2=OP1
     &                 +  2.000 * RXRAT( 248 )      ! MO2+MO2=0.74*HO2+1.37*HCHO+...
     &                 +          RXRAT( 249 )      ! MO2+ETHP=HO2+0.75*HCHO+...
     &                 +  0.920 * RXRAT( 250 )      ! MO2+HC3P=0.08*MO2+...
     &                 +  0.982 * RXRAT( 251 )      ! MO2+HC5P=0.018*MO2+...
     &                 +          RXRAT( 252 )      ! MO2+HC8P=0.91*HO2+...
     &                 +          RXRAT( 253 )      ! MO2+ETEP=HO2+1.95*HCHO+...
     &                 +          RXRAT( 254 )      ! MO2+OLTP=HO2+1.5*HCHO+...
     &                 +          RXRAT( 255 )      ! MO2+OLIP=HO2+0.75*HCHO+...
     &                 +          RXRAT( 256 )      ! MO2+BENP=1.6*HO2+...
     &                 +          RXRAT( 257 )      ! MO2+TLP1=HO2+HCHO+BALD
     &                 +          RXRAT( 258 )      ! MO2+TOLP=2*HO2+HCHO+...
     &                 +          RXRAT( 259 )      ! MO2+PER1=2*HO2+HCHO+MGLY+DCB1
     &                 +          RXRAT( 260 )      ! MO2+XYL1=HO2+HCHO+BALD
     &                 +          RXRAT( 261 )      ! MO2+XYLP=2*HO2+HCHO+DCB2
     &                 +          RXRAT( 262 )      ! MO2+PER2=2*HO2+HCHO+MGLY+...
     &                 +          RXRAT( 263 )      ! MO2+XYOP=2*HO2+HCHO+...
     &                 +          RXRAT( 264 )      ! MO2+ISOP=HO2+1.31*HCHO+...
     &                 +          RXRAT( 265 )      ! MO2+APIP=HO2+0.75*HCHO+...
     &                 +          RXRAT( 266 )      ! MO2+LIMP=HO2+1.04*HCHO+...
     &                 +  0.100 * RXRAT( 267 )      ! MO2+ACO3=0.9*MO2+0.9*HO2+...
     &                 +  0.100 * RXRAT( 268 )      ! MO2+RCO3=0.9*MO2+0.9*HO2+...
     &                 +          RXRAT( 269 )      ! MO2+ACTP=0.5*HO2+0.5*ACO3+...
     &                 +          RXRAT( 270 )      ! MO2+MEKP=0.834*HO2+HCHO+...
     &                 +          RXRAT( 271 )      ! MO2+KETP=HO2+0.75*HCHO+...
     &                 +  0.750 * RXRAT( 272 )      ! MO2+MACP=0.25*MO2+...
     &                 +          RXRAT( 273 )      ! MO2+MCP=NO2+HO2+1.5*HCHO+...
     &                 +          RXRAT( 274 )      ! MO2+MVKP=HO2+1.16*ACO3+...
     &                 +          RXRAT( 275 )      ! MO2+UALP=HO2+0.305*CO+...
     &                 +          RXRAT( 276 )      ! MO2+BALP=HO2+BAL1+HCHO
     &                 +          RXRAT( 277 )      ! MO2+BAL1=HO2+BAL2+HCHO
     &                 +          RXRAT( 278 )      ! MO2+ADDC=2*HO2+HCHO+...
     &                 +          RXRAT( 279 )      ! MO2+MCTP=HO2+MCTO+HCHO
     &                 +          RXRAT( 280 )      ! MO2+ORAP=HCHO+HO2+GLY
     &                 +          RXRAT( 281 )      ! MO2+OLNN=2*HO2+HCHO+ONIT
     &                 +          RXRAT( 282 )      ! MO2+OLND=0.5*HO2+0.5*NO2+...
     &                 +          RXRAT( 283 )      ! MO2+ADCN=HO2+0.7*NO2+HCHO+...
     &                 +          RXRAT( 284 )      ! MO2+XO2=HO2+HCHO
     &                 +          RXRAT( 322 )      ! MO2+NO3=HO2+HCHO+NO2

      PROD( ALD     ) =   0.434 * RXRAT(  15 )      ! UALD=0.434*ALD+0.784*ACO3+...
     &                 +          RXRAT(  29 )      ! OP2=ALD+HO2+HO
     &                 +  0.200 * RXRAT(  31 )      ! ONIT=0.2*ALD+NO2+HO2+0.8*KET
     &                 +  0.025 * RXRAT(  75 )      ! HC8+HO=0.025*ALD+...
     &                 +  0.400 * RXRAT( 101 )      ! DCB1+HO=0.4*ALD+0.33*CO+...
     &                 +          RXRAT( 107 )      ! EPX+HO=ALD+XO2+CO+HO2
     &                 +  0.719 * RXRAT( 111 )      ! ROH+HO=0.719*ALD+HO2+0.184*ACD
     &                 +          RXRAT( 112 )      ! ETEG+HO=ALD+HO2
     &                 +  0.080 * RXRAT( 114 )      ! OP2+HO=0.08*ALD+0.44*HC3P+...
     &                 +  0.440 * RXRAT( 127 )      ! OLT+O3=0.44*ALD+0.32*HO2+...
     &                 +  0.730 * RXRAT( 128 )      ! OLI+O3=0.73*ALD+0.07*HO2+...
     &                 +  0.650 * RXRAT( 131 )      ! API+O3=0.65*ALD+0.1*HO2+...
     &                 +  0.100 * RXRAT( 134 )      ! MVK+O3=0.1*ALD+0.11*HO2+...
     &                 +  0.332 * RXRAT( 152 )      ! UALD+NO3=0.332*ALD+XO2+...
     &                 +  0.132 * RXRAT( 174 )      ! HC3P+NO=0.132*ALD+...
     &                 +  0.203 * RXRAT( 175 )      ! HC5P+NO=0.203*ALD+...
     &                 +  0.150 * RXRAT( 176 )      ! HC8P+NO=0.15*ALD+...
     &                 +  0.200 * RXRAT( 177 )      ! ETEP+NO=0.2*ALD+NO2+...
     &                 +  0.440 * RXRAT( 178 )      ! OLTP+NO=0.44*ALD+0.97*NO2+...
     &                 +  0.680 * RXRAT( 179 )      ! OLIP+NO=0.68*ALD+0.95*NO2+...
     &                 +  0.027 * RXRAT( 188 )      ! ISOP+NO=0.027*ALD+0.88*NO2+...
     &                 +  0.430 * RXRAT( 189 )      ! APIP+NO=0.43*ALD+0.82*NO2+...
     &                 +  0.460 * RXRAT( 195 )      ! KETP+NO=0.46*ALD+0.23*ACO3+...
     &                 +  0.700 * RXRAT( 198 )      ! MVKP+NO=0.7*ALD+0.7*ACO3+...
     &                 +  0.270 * RXRAT( 199 )      ! UALP+NO=0.27*ALD+NO2+...
     &                 +  1.240 * RXRAT( 206 )      ! OLND+NO=1.24*ALD+...
     &                 +  0.198 * RXRAT( 250 )      ! HC3P+MO2=0.198*ALD+...
     &                 +  0.251 * RXRAT( 251 )      ! HC5P+MO2=0.251*ALD+...
     &                 +  0.197 * RXRAT( 252 )      ! HC8P+MO2=0.197*ALD+...
     &                 +  0.150 * RXRAT( 253 )      ! ETEP+MO2=0.15*ALD+...
     &                 +  0.705 * RXRAT( 254 )      ! OLTP+MO2=0.705*ALD+...
     &                 +  1.280 * RXRAT( 255 )      ! OLIP+MO2=1.28*ALD+...
     &                 +  0.023 * RXRAT( 264 )      ! ISOP+MO2=0.023*ALD+...
     &                 +  0.750 * RXRAT( 265 )      ! APIP+MO2=0.75*ALD+...
     &                 +  1.750 * RXRAT( 274 )      ! MVKP+MO2=1.75*ALD+...
     &                 +  0.203 * RXRAT( 275 )      ! UALP+MO2=0.203*ALD+...
     &                 +  0.930 * RXRAT( 282 )      ! OLND+MO2=0.93*ALD+0.5*NO2+...
     &                 +  0.273 * RXRAT( 286 )      ! HC3P+ACO3=0.273*ALD+...
     &                 +  0.381 * RXRAT( 287 )      ! HC5P+ACO3=0.381*ALD+...
     &                 +  0.217 * RXRAT( 288 )      ! HC8P+ACO3=0.217*ALD+...
     &                 +  0.200 * RXRAT( 289 )      ! ETEP+ACO3=0.2*ALD+0.5*MO2+...
     &                 +  0.940 * RXRAT( 290 )      ! OLTP+ACO3=0.94*ALD+0.5*MO2+...
     &                 +  1.710 * RXRAT( 291 )      ! OLIP+ACO3=1.71*ALD+0.5*MO2+...
     &                 +          RXRAT( 301 )      ! APIP+ACO3=ALD+0.5*MO2+...
     &                 +  2.300 * RXRAT( 310 )      ! MVKP+ACO3=2.3*ALD+0.5*MO2+...
     &                 +  0.270 * RXRAT( 311 )      ! UALP+ACO3=0.27*ALD+0.5*MO2+...
     &                 +  1.240 * RXRAT( 318 )      ! OLND+ACO3=1.24*ALD+NO2+...
     &                 +  0.147 * RXRAT( 324 )      ! HC3P+NO3=0.147*ALD+...
     &                 +  0.241 * RXRAT( 325 )      ! HC5P+NO3=0.241*ALD+...
     &                 +  0.203 * RXRAT( 326 )      ! HC8P+NO3=0.203*ALD+...
     &                 +  0.200 * RXRAT( 327 )      ! ETEP+NO3=0.2*ALD+NO2+...
     &                 +  0.470 * RXRAT( 328 )      ! OLTP+NO3=0.47*ALD+...
     &                 +  0.720 * RXRAT( 329 )      ! OLIP+NO3=0.72*ALD+0.86*HO2+...
     &                 +  0.031 * RXRAT( 338 )      ! ISOP+NO3=0.031*ALD+NO2+...
     &                 +          RXRAT( 339 )      ! APIP+NO3=ALD+NO2+HO2+KET
     &                 +  0.700 * RXRAT( 348 )      ! MVKP+NO3=0.7*ALD+0.7*ACO3+...
     &                 +  0.270 * RXRAT( 349 )      ! UALP+NO3=0.27*ALD+NO2+...
     &                 +  1.240 * RXRAT( 356 )      ! OLND+NO3=1.24*ALD+...
     &                 +  0.640 * RXRAT( 359 )      ! OLNN+OLND=0.64*ALD+0.5*NO2+...
     &                 +  1.210 * RXRAT( 360 )      ! OLND+OLND=1.21*ALD+...
      LOSS( ALD     ) =           RXRAT(  13 )      ! ALD=HO2+ETHP+CO
     &                 +          RXRAT(  91 )      ! ALD+HO=RCO3
     &                 +          RXRAT( 150 )      ! ALD+NO3=RCO3+HNO3

      PROD( ETHP    ) =           RXRAT(  13 )      ! ALD=ETHP+HO2+CO
     &                 +  0.500 * RXRAT(  16 )      ! MEK=0.5*ETHP+0.5*MO2+ACO3
     &                 +          RXRAT(  17 )      ! KET=ETHP+ACO3
     &                 +          RXRAT(  72 )      ! ETH+HO=ETHP
     &                 +  0.060 * RXRAT( 127 )      ! OLT+O3=0.06*ETHP+0.32*HO2+...
     &                 +  0.070 * RXRAT( 128 )      ! OLI+O3=0.07*ETHP+0.07*HO2+...
     &                 +  0.200 * RXRAT( 131 )      ! API+O3=0.2*ETHP+0.1*HO2+...
     &                 +  0.160 * RXRAT( 132 )      ! LIM+O3=0.16*ETHP+0.1*HO2+...
     &                 +  0.048 * RXRAT( 174 )      ! HC3P+NO=0.048*ETHP+...
     &                 +  0.231 * RXRAT( 175 )      ! HC5P+NO=0.231*ETHP+...
     &                 +  0.133 * RXRAT( 176 )      ! HC8P+NO=0.133*ETHP+...
     &                 +          RXRAT( 192 )      ! RCO3+NO=ETHP+NO2
     &                 +  0.440 * RXRAT( 232 )      ! RCO3+HO2=0.44*ETHP+0.44*HO+...
     &                 +  0.026 * RXRAT( 250 )      ! HC3P+MO2=0.026*ETHP+...
     &                 +  0.140 * RXRAT( 251 )      ! HC5P+MO2=0.14*ETHP+...
     &                 +  0.090 * RXRAT( 252 )      ! HC8P+MO2=0.09*ETHP+...
     &                 +  0.026 * RXRAT( 286 )      ! HC3P+ACO3=0.026*ETHP+...
     &                 +  0.140 * RXRAT( 287 )      ! HC5P+ACO3=0.14*ETHP+...
     &                 +  0.067 * RXRAT( 288 )      ! HC8P+ACO3=0.067*ETHP+...
     &                 +          RXRAT( 304 )      ! RCO3+ACO3=ETHP+MO2
     &                 +  2.000 * RXRAT( 321 )      ! RCO3+RCO3=2*ETHP
     &                 +  0.503 * RXRAT( 324 )      ! HC3P+NO3=0.503*ETHP+...
     &                 +  0.280 * RXRAT( 325 )      ! HC5P+NO3=0.28*ETHP+...
     &                 +  0.180 * RXRAT( 326 )      ! HC8P+NO3=0.18*ETHP+...
     &                 +          RXRAT( 342 )      ! RCO3+NO3=ETHP+NO2
     &                 +          RXRAT( 362 )      ! XO2+RCO3=ETHP
      LOSS( ETHP    ) =           RXRAT( 173 )      ! ETHP+NO=HO2+NO2+ACD
     &                 +          RXRAT( 213 )      ! ETHP+HO2=OP2
     &                 +          RXRAT( 249 )      ! ETHP+MO2=HO2+0.75*HCHO+...
     &                 +          RXRAT( 285 )      ! ETHP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 323 )      ! ETHP+NO3=HO2+NO2+ACD

      PROD( ACT     ) =   0.030 * RXRAT( 127 )      ! OLT+O3=0.03*ACT+0.32*HO2+...
     &                 +  0.110 * RXRAT( 128 )      ! OLI+O3=0.11*ACT+0.07*HO2+...
     &                 +  0.165 * RXRAT( 174 )      ! HC3P+NO=0.165*ACT+...
     &                 +  0.217 * RXRAT( 175 )      ! HC5P+NO=0.217*ACT+...
     &                 +  0.060 * RXRAT( 178 )      ! OLTP+NO=0.06*ACT+0.97*NO2+...
     &                 +  0.200 * RXRAT( 179 )      ! OLIP+NO=0.2*ACT+0.95*NO2+...
     &                 +  0.110 * RXRAT( 189 )      ! APIP+NO=0.11*ACT+0.82*NO2+...
     &                 +  0.095 * RXRAT( 324 )      ! HC3P+NO3=0.095*ACT+...
     &                 +  0.247 * RXRAT( 325 )      ! HC5P+NO3=0.247*ACT+...
     &                 +  0.090 * RXRAT( 328 )      ! OLTP+NO3=0.09*ACT+...
     &                 +  0.200 * RXRAT( 329 )      ! OLIP+NO3=0.2*ACT+0.72*ALD+...
      LOSS( ACT     ) =           RXRAT(  14 )      ! ACT=MO2+ACO3
     &                 +          RXRAT(  92 )      ! ACT+HO=ACTP

      PROD( UALD    ) =   0.700 * RXRAT(  20 )      ! MVK=0.7*UALD+0.3*MACP+...
     &                 +  0.680 * RXRAT( 190 )      ! LIMP+NO=0.68*UALD+HO2+...
      LOSS( UALD    ) =           RXRAT(  15 )      ! UALD=1.22*HO2+0.784*ACO3+...
     &                 +          RXRAT(  98 )      ! UALD+HO=0.313*ACO3+0.687*UALP
     &                 +          RXRAT( 135 )      ! UALD+O3=0.1*HO+0.072*HO2+...
     &                 +          RXRAT( 152 )      ! UALD+NO3=HO2+XO2+0.668*CO+...

      PROD( KET     ) =   0.216 * RXRAT(  15 )      ! UALD=0.216*KET+0.784*ACO3+...
     &                 +  0.800 * RXRAT(  31 )      ! ONIT=0.8*KET+NO2+0.2*ALD+HO2
     &                 +  0.780 * RXRAT( 101 )      ! DCB1+HO=0.78*KET+0.33*CO+...
     &                 +  0.410 * RXRAT( 114 )      ! OP2+HO=0.41*KET+0.44*HC3P+...
     &                 +  0.017 * RXRAT( 128 )      ! OLI+O3=0.017*KET+0.07*HO2+...
     &                 +  0.530 * RXRAT( 131 )      ! API+O3=0.53*KET+0.1*HO2+...
     &                 +  0.028 * RXRAT( 135 )      ! UALD+O3=0.028*KET+...
     &                 +  0.033 * RXRAT( 175 )      ! HC5P+NO=0.033*KET+...
     &                 +  0.642 * RXRAT( 176 )      ! HC8P+NO=0.642*KET+...
     &                 +  0.090 * RXRAT( 179 )      ! OLIP+NO=0.09*KET+0.95*NO2+...
     &                 +  0.440 * RXRAT( 189 )      ! APIP+NO=0.44*KET+0.82*NO2+...
     &                 +  0.700 * RXRAT( 199 )      ! UALP+NO=0.7*KET+NO2+...
     &                 +  0.464 * RXRAT( 206 )      ! OLND+NO=0.464*KET+...
     &                 +  0.497 * RXRAT( 250 )      ! HC3P+MO2=0.497*KET+...
     &                 +  0.618 * RXRAT( 251 )      ! HC5P+MO2=0.618*KET+...
     &                 +  0.652 * RXRAT( 252 )      ! HC8P+MO2=0.652*KET+...
     &                 +  0.045 * RXRAT( 254 )      ! OLTP+MO2=0.045*KET+...
     &                 +  0.218 * RXRAT( 255 )      ! OLIP+MO2=0.218*KET+...
     &                 +  0.750 * RXRAT( 265 )      ! APIP+MO2=0.75*KET+...
     &                 +  0.525 * RXRAT( 275 )      ! UALP+MO2=0.525*KET+...
     &                 +  0.348 * RXRAT( 282 )      ! OLND+MO2=0.348*KET+0.5*NO2+...
     &                 +  0.662 * RXRAT( 286 )      ! HC3P+ACO3=0.662*KET+...
     &                 +  0.824 * RXRAT( 287 )      ! HC5P+ACO3=0.824*KET+...
     &                 +  0.642 * RXRAT( 288 )      ! HC8P+ACO3=0.642*KET+...
     &                 +  0.060 * RXRAT( 290 )      ! OLTP+ACO3=0.06*KET+0.5*MO2+...
     &                 +  0.290 * RXRAT( 291 )      ! OLIP+ACO3=0.29*KET+0.5*MO2+...
     &                 +          RXRAT( 301 )      ! APIP+ACO3=KET+0.5*MO2+ALD+...
     &                 +  0.700 * RXRAT( 311 )      ! UALP+ACO3=0.7*KET+0.5*MO2+...
     &                 +  0.464 * RXRAT( 318 )      ! OLND+ACO3=0.464*KET+NO2+...
     &                 +  0.060 * RXRAT( 325 )      ! HC5P+NO3=0.06*KET+...
     &                 +  0.869 * RXRAT( 326 )      ! HC8P+NO3=0.869*KET+...
     &                 +  0.110 * RXRAT( 329 )      ! OLIP+NO3=0.11*KET+0.72*ALD+...
     &                 +          RXRAT( 339 )      ! APIP+NO3=KET+NO2+ALD+HO2
     &                 +  0.700 * RXRAT( 349 )      ! UALP+NO3=0.7*KET+NO2+...
     &                 +  0.464 * RXRAT( 356 )      ! OLND+NO3=0.464*KET+...
     &                 +  0.149 * RXRAT( 359 )      ! OLNN+OLND=0.149*KET+...
     &                 +  0.285 * RXRAT( 360 )      ! OLND+OLND=0.285*KET+...
      LOSS( KET     ) =           RXRAT(  17 )      ! KET=ETHP+ACO3
     &                 +          RXRAT(  94 )      ! KET+HO=KETP

      PROD( MEK     ) =   0.130 * RXRAT( 102 )      ! DCB2+HO=0.13*MEK+0.33*CO+...
     &                 +  0.060 * RXRAT( 127 )      ! OLT+O3=0.06*MEK+0.32*HO2+...
     &                 +  0.042 * RXRAT( 174 )      ! HC3P+NO=0.042*MEK+...
     &                 +  0.033 * RXRAT( 175 )      ! HC5P+NO=0.033*MEK+...
     &                 +  0.130 * RXRAT( 178 )      ! OLTP+NO=0.13*MEK+0.97*NO2+...
     &                 +  0.075 * RXRAT( 324 )      ! HC3P+NO3=0.075*MEK+...
     &                 +  0.063 * RXRAT( 325 )      ! HC5P+NO3=0.063*MEK+...
     &                 +  0.180 * RXRAT( 328 )      ! OLTP+NO3=0.18*MEK+...
      LOSS( MEK     ) =           RXRAT(  16 )      ! MEK=0.5*MO2+0.5*ETHP+ACO3
     &                 +          RXRAT(  93 )      ! MEK+HO=MEKP

      PROD( HKET    ) =   0.024 * RXRAT(  75 )      ! HC8+HO=0.024*HKET+...
     &                 +          RXRAT( 122 )      ! MPAN+HO=HKET+NO2
     &                 +          RXRAT( 124 )      ! NALD+HO=HKET+XO2+NO2
     &                 +  0.070 * RXRAT( 125 )      ! ISON+HO=0.07*HKET+NALD+...
     &                 +  0.010 * RXRAT( 127 )      ! OLT+O3=0.01*HKET+0.32*HO2+...
     &                 +  0.044 * RXRAT( 128 )      ! OLI+O3=0.044*HKET+0.07*HO2+...
     &                 +  0.272 * RXRAT( 175 )      ! HC5P+NO=0.272*HKET+...
     &                 +  0.020 * RXRAT( 179 )      ! OLIP+NO=0.02*HKET+0.95*NO2+...
     &                 +  0.029 * RXRAT( 188 )      ! ISOP+NO=0.029*HKET+...
     &                 +          RXRAT( 197 )      ! MCP+NO=HKET+0.5*HO2+...
     &                 +  0.320 * RXRAT( 202 )      ! ADDC+NO=0.32*HKET+NO2+HO2+...
     &                 +  0.016 * RXRAT( 264 )      ! ISOP+MO2=0.016*HKET+...
     &                 +  0.500 * RXRAT( 273 )      ! MCP+MO2=0.5*HKET+HO2+...
     &                 +  0.320 * RXRAT( 278 )      ! ADDC+MO2=0.32*HKET+HCHO+...
     &                 +  0.500 * RXRAT( 309 )      ! MCP+ACO3=0.5*HKET+0.5*HO2+...
     &                 +  0.320 * RXRAT( 314 )      ! ADDC+ACO3=0.32*HKET+MO2+...
     &                 +  0.275 * RXRAT( 325 )      ! HC5P+NO3=0.275*HKET+...
     &                 +  0.040 * RXRAT( 329 )      ! OLIP+NO3=0.04*HKET+...
     &                 +  0.033 * RXRAT( 338 )      ! ISOP+NO3=0.033*HKET+NO2+...
     &                 +          RXRAT( 347 )      ! MCP+NO3=HKET+HO2+HCHO+NO2
     &                 +  0.320 * RXRAT( 352 )      ! ADDC+NO3=0.32*HKET+NO2+HO2+...
      LOSS( HKET    ) =           RXRAT(  18 )      ! HKET=HO2+ACO3+HCHO
     &                 +          RXRAT(  95 )      ! HKET+HO=HO2+MGLY

      PROD( MACR    ) =           RXRAT( 115 )      ! ISHP+HO=MACR+HO
     &                 +  0.390 * RXRAT( 129 )      ! DIEN+O3=0.39*MACR+0.28*HO+...
     &                 +  0.461 * RXRAT( 130 )      ! ISO+O3=0.461*MACR+0.25*HO2+...
     &                 +  0.790 * RXRAT( 132 )      ! LIM+O3=0.79*MACR+0.1*HO2+...
     &                 +  0.900 * RXRAT( 144 )      ! DIEN+NO3=0.9*MACR+0.1*OLND+...
     &                 +  0.280 * RXRAT( 188 )      ! ISOP+NO=0.28*MACR+0.88*NO2+...
     &                 +  0.159 * RXRAT( 264 )      ! ISOP+MO2=0.159*MACR+...
     &                 +  0.308 * RXRAT( 266 )      ! LIMP+MO2=0.308*MACR+...
     &                 +  0.219 * RXRAT( 300 )      ! ISOP+ACO3=0.219*MACR+...
     &                 +  0.308 * RXRAT( 302 )      ! LIMP+ACO3=0.308*MACR+...
     &                 +  0.318 * RXRAT( 338 )      ! ISOP+NO3=0.318*MACR+NO2+...
     &                 +  0.615 * RXRAT( 340 )      ! LIMP+NO3=0.615*MACR+NO2+...
      LOSS( MACR    ) =           RXRAT(  19 )      ! MACR=0.34*HO+0.66*HO2+...
     &                 +          RXRAT(  96 )      ! MACR+HO=0.57*MACP+0.43*MCP
     &                 +          RXRAT( 133 )      ! MACR+O3=0.19*HO+0.14*HO2+...
     &                 +          RXRAT( 151 )      ! MACR+NO3=0.68*HCHO+...

      PROD( MACP    ) =   0.330 * RXRAT(  19 )      ! MACR=0.33*MACP+0.66*HO2+...
     &                 +  0.300 * RXRAT(  20 )      ! MVK=0.3*MACP+0.3*MO2+...
     &                 +  0.570 * RXRAT(  96 )      ! MACR+HO=0.57*MACP+0.43*MCP
     &                 +  0.210 * RXRAT( 103 )      ! DCB3+HO=0.21*MACP+0.56*HO2+...
     &                 +          RXRAT( 116 )      ! MAHP+HO=MACP
     &                 +  0.100 * RXRAT( 130 )      ! ISO+O3=0.1*MACP+0.25*HO2+...
     &                 +  0.320 * RXRAT( 151 )      ! MACR+NO3=0.32*MACP+...
     &                 +          RXRAT( 159 )      ! MPAN+NO3=MACP+NO2
     &                 +          RXRAT( 171 )      ! MPAN=MACP+NO2
      LOSS( MACP    ) =           RXRAT( 170 )      ! MACP+NO2=MPAN
     &                 +          RXRAT( 196 )      ! MACP+NO=0.65*MO2+0.35*ACO3+...
     &                 +          RXRAT( 236 )      ! MACP+HO2=MAHP
     &                 +          RXRAT( 272 )      ! MACP+MO2=0.5*HO2+...
     &                 +          RXRAT( 308 )      ! MACP+ACO3=0.635*ORA2+...
     &                 +          RXRAT( 346 )      ! MACP+NO3=HCHO+0.538*ACO3+...

      PROD( XO2     ) =   0.340 * RXRAT(  19 )      ! MACR=0.34*XO2+0.66*HO2+...
     &                 +  0.200 * RXRAT(  25 )      ! DCB1=0.2*XO2+0.25*ACO3+...
     &                 +  0.200 * RXRAT(  26 )      ! DCB2=0.2*XO2+0.25*ACO3+...
     &                 +          RXRAT( 107 )      ! EPX+HO=XO2+HO2+CO+ALD
     &                 +  0.070 * RXRAT( 114 )      ! OP2+HO=0.07*XO2+0.44*HC3P+...
     &                 +  0.350 * RXRAT( 119 )      ! PAA+HO=0.35*XO2+0.65*ACO3+...
     &                 +          RXRAT( 120 )      ! PAN+HO=XO2+NO3+HCHO
     &                 +          RXRAT( 121 )      ! PPN+HO=XO2+NO3+HCHO
     &                 +          RXRAT( 124 )      ! NALD+HO=XO2+NO2+HKET
     &                 +  0.130 * RXRAT( 129 )      ! DIEN+O3=0.13*XO2+0.28*HO+...
     &                 +  0.010 * RXRAT( 134 )      ! MVK+O3=0.01*XO2+0.11*HO2+...
     &                 +  0.100 * RXRAT( 135 )      ! UALD+O3=0.1*XO2+0.072*HO2+...
     &                 +  0.600 * RXRAT( 136 )      ! DCB1+O3=0.6*XO2+HO2+...
     &                 +  0.600 * RXRAT( 137 )      ! DCB2+O3=0.6*XO2+HO2+...
     &                 +  0.680 * RXRAT( 151 )      ! MACR+NO3=0.68*XO2+...
     &                 +          RXRAT( 152 )      ! UALD+NO3=XO2+HO2+0.668*CO+...
     &                 +  0.089 * RXRAT( 174 )      ! HC3P+NO=0.089*XO2+...
     &                 +  0.235 * RXRAT( 175 )      ! HC5P+NO=0.235*XO2+...
     &                 +  0.416 * RXRAT( 176 )      ! HC8P+NO=0.416*XO2+...
     &                 +  0.160 * RXRAT( 195 )      ! KETP+NO=0.16*XO2+0.23*ACO3+...
     &                 +  0.700 * RXRAT( 198 )      ! MVKP+NO=0.7*XO2+0.7*ACO3+...
     &                 +  0.026 * RXRAT( 250 )      ! HC3P+MO2=0.026*XO2+...
     &                 +  0.191 * RXRAT( 251 )      ! HC5P+MO2=0.191*XO2+...
     &                 +  0.281 * RXRAT( 252 )      ! HC8P+MO2=0.281*XO2+...
     &                 +  1.160 * RXRAT( 274 )      ! MVKP+MO2=1.16*XO2+...
     &                 +  0.026 * RXRAT( 286 )      ! HC3P+ACO3=0.026*XO2+...
     &                 +  0.191 * RXRAT( 287 )      ! HC5P+ACO3=0.191*XO2+...
     &                 +  0.208 * RXRAT( 288 )      ! HC8P+ACO3=0.208*XO2+...
     &                 +  1.160 * RXRAT( 310 )      ! MVKP+ACO3=1.16*XO2+0.5*MO2+...
     &                 +  0.092 * RXRAT( 324 )      ! HC3P+NO3=0.092*XO2+...
     &                 +  0.485 * RXRAT( 325 )      ! HC5P+NO3=0.485*XO2+...
     &                 +  0.563 * RXRAT( 326 )      ! HC8P+NO3=0.563*XO2+...
     &                 +  0.700 * RXRAT( 348 )      ! MVKP+NO3=0.7*XO2+0.7*ACO3+...
      LOSS( XO2     ) =           RXRAT( 208 )      ! XO2+NO=NO2
     &                 +          RXRAT( 247 )      ! XO2+HO2=OP2
     &                 +          RXRAT( 284 )      ! XO2+MO2=HO2+HCHO
     &                 +          RXRAT( 320 )      ! XO2+ACO3=MO2
     &                 +          RXRAT( 361 )      ! XO2+NO3=NO2
     &                 +          RXRAT( 362 )      ! XO2+RCO3=ETHP
     &                 +  2.000 * RXRAT( 363 )      ! XO2+XO2=

      PROD( MVK     ) =   0.189 * RXRAT( 130 )      ! ISO+O3=0.189*MVK+0.25*HO2+...
     &                 +  0.440 * RXRAT( 188 )      ! ISOP+NO=0.44*MVK+0.88*NO2+...
     &                 +  0.250 * RXRAT( 264 )      ! ISOP+MO2=0.25*MVK+...
     &                 +  0.305 * RXRAT( 300 )      ! ISOP+ACO3=0.305*MVK+...
     &                 +  0.500 * RXRAT( 338 )      ! ISOP+NO3=0.5*MVK+NO2+...
      LOSS( MVK     ) =           RXRAT(  20 )      ! MVK=0.3*MO2+0.3*MACP+...
     &                 +          RXRAT(  97 )      ! MVK+HO=MVKP
     &                 +          RXRAT( 134 )      ! MVK+O3=0.16*HO+0.11*HO2+...

      PROD( GLY     ) =   0.500 * RXRAT(  25 )      ! DCB1=0.5*GLY+0.25*ACO3+...
     &                 +  0.500 * RXRAT(  26 )      ! DCB2=0.5*GLY+0.25*ACO3+...
     &                 +  0.650 * RXRAT(  80 )      ! ACE+HO=0.65*GLY+0.35*HO2+...
     &                 +  0.100 * RXRAT( 101 )      ! DCB1+HO=0.1*GLY+0.33*CO+...
     &                 +  0.100 * RXRAT( 102 )      ! DCB2+HO=0.1*GLY+0.33*CO+...
     &                 +  0.270 * RXRAT( 103 )      ! DCB3+HO=0.27*GLY+0.21*MACP+...
     &                 +  0.491 * RXRAT( 135 )      ! UALD+O3=0.491*GLY+...
     &                 +  0.050 * RXRAT( 136 )      ! DCB1+O3=0.05*GLY+HO2+...
     &                 +  0.050 * RXRAT( 137 )      ! DCB2+O3=0.05*GLY+HO2+...
     &                 +  0.480 * RXRAT( 138 )      ! DCB3+O3=0.48*GLY+HO2+...
     &                 +          RXRAT( 139 )      ! EPX+O3=GLY+1.5*HO2+1.5*CO+...
     &                 +          RXRAT( 157 )      ! EPX+NO3=GLY+1.5*HO2+1.5*CO+...
     &                 +  0.918 * RXRAT( 180 )      ! BENP+NO=0.918*GLY+...
     &                 +  0.350 * RXRAT( 187 )      ! XYOP+NO=0.35*GLY+0.95*NO2+...
     &                 +  0.021 * RXRAT( 188 )      ! ISOP+NO=0.021*GLY+0.88*NO2+...
     &                 +  0.180 * RXRAT( 199 )      ! UALP+NO=0.18*GLY+NO2+...
     &                 +  0.680 * RXRAT( 202 )      ! ADDC+NO=0.68*GLY+NO2+...
     &                 +          RXRAT( 204 )      ! ORAP+NO=GLY+NO2+HO2
     &                 +          RXRAT( 207 )      ! ADCN+NO=GLY+2*NO2+OP2
     &                 +  0.050 * RXRAT( 250 )      ! HC3P+MO2=0.05*GLY+0.08*MO2+...
     &                 +  0.600 * RXRAT( 256 )      ! BENP+MO2=0.6*GLY+...
     &                 +  0.271 * RXRAT( 258 )      ! TOLP+MO2=0.271*GLY+HCHO+...
     &                 +  0.368 * RXRAT( 263 )      ! XYOP+MO2=0.368*GLY+HCHO+...
     &                 +  0.018 * RXRAT( 264 )      ! ISOP+MO2=0.018*GLY+...
     &                 +  0.135 * RXRAT( 275 )      ! UALP+MO2=0.135*GLY+...
     &                 +  0.680 * RXRAT( 278 )      ! ADDC+MO2=0.68*GLY+HCHO+...
     &                 +          RXRAT( 280 )      ! ORAP+MO2=GLY+HO2+HCHO
     &                 +  0.700 * RXRAT( 283 )      ! ADCN+MO2=0.7*GLY+0.7*NO2+...
     &                 +  0.067 * RXRAT( 286 )      ! HC3P+ACO3=0.067*GLY+...
     &                 +  0.600 * RXRAT( 292 )      ! BENP+ACO3=0.6*GLY+MO2+...
     &                 +  0.368 * RXRAT( 299 )      ! XYOP+ACO3=0.368*GLY+MO2+...
     &                 +  0.180 * RXRAT( 311 )      ! UALP+ACO3=0.18*GLY+0.5*MO2+...
     &                 +  0.680 * RXRAT( 314 )      ! ADDC+ACO3=0.68*GLY+MO2+...
     &                 +          RXRAT( 316 )      ! ORAP+ACO3=GLY+MO2
     &                 +  0.700 * RXRAT( 319 )      ! ADCN+ACO3=0.7*GLY+MO2+...
     &                 +          RXRAT( 330 )      ! BENP+NO3=GLY+NO2+0.5*DCB2+...
     &                 +  0.368 * RXRAT( 337 )      ! XYOP+NO3=0.368*GLY+NO2+HO2+...
     &                 +  0.024 * RXRAT( 338 )      ! ISOP+NO3=0.024*GLY+NO2+...
     &                 +  0.180 * RXRAT( 349 )      ! UALP+NO3=0.18*GLY+NO2+...
     &                 +  0.680 * RXRAT( 352 )      ! ADDC+NO3=0.68*GLY+NO2+...
     &                 +          RXRAT( 354 )      ! ORAP+NO3=GLY+NO2+HO2
     &                 +          RXRAT( 357 )      ! ADCN+NO3=GLY+2*NO2+OP2
      LOSS( GLY     ) =           RXRAT(  21 )      ! GLY=2*CO
     &                 +          RXRAT(  22 )      ! GLY=HCHO+CO
     &                 +          RXRAT(  23 )      ! GLY=2*HO2+2*CO
     &                 +          RXRAT(  99 )      ! GLY+HO=HO2+2*CO
     &                 +          RXRAT( 153 )      ! GLY+NO3=HO2+2*CO+HNO3

      PROD( MGLY    ) =   0.500 * RXRAT(  25 )      ! DCB1=0.5*MGLY+0.25*ACO3+...
     &                 +  0.500 * RXRAT(  26 )      ! DCB2=0.5*MGLY+0.25*ACO3+...
     &                 +          RXRAT(  95 )      ! HKET+HO=MGLY+HO2
     &                 +  0.010 * RXRAT( 101 )      ! DCB1+HO=0.01*MGLY+0.33*CO+...
     &                 +  0.010 * RXRAT( 102 )      ! DCB2+HO=0.01*MGLY+0.33*CO+...
     &                 +  0.010 * RXRAT( 103 )      ! DCB3+HO=0.01*MGLY+...
     &                 +  0.500 * RXRAT( 133 )      ! MACR+O3=0.5*MGLY+0.14*HO2+...
     &                 +  0.540 * RXRAT( 134 )      ! MVK+O3=0.54*MGLY+0.11*HO2+...
     &                 +  0.003 * RXRAT( 135 )      ! UALD+O3=0.003*MGLY+...
     &                 +  0.080 * RXRAT( 136 )      ! DCB1+O3=0.08*MGLY+HO2+...
     &                 +  0.080 * RXRAT( 137 )      ! DCB2+O3=0.08*MGLY+HO2+...
     &                 +  0.680 * RXRAT( 151 )      ! MACR+NO3=0.68*MGLY+...
     &                 +  0.500 * RXRAT( 183 )      ! PER1+NO=0.5*MGLY+0.95*NO2+...
     &                 +  0.950 * RXRAT( 186 )      ! PER2+NO=0.95*MGLY+0.95*NO2+...
     &                 +  0.600 * RXRAT( 187 )      ! XYOP+NO=0.6*MGLY+0.95*NO2+...
     &                 +  0.540 * RXRAT( 195 )      ! KETP+NO=0.54*MGLY+...
     &                 +  0.300 * RXRAT( 198 )      ! MVKP+NO=0.3*MGLY+0.7*ACO3+...
     &                 +  0.210 * RXRAT( 199 )      ! UALP+NO=0.21*MGLY+NO2+...
     &                 +          RXRAT( 259 )      ! PER1+MO2=MGLY+HCHO+2*HO2+DCB1
     &                 +          RXRAT( 262 )      ! PER2+MO2=MGLY+HCHO+2*HO2+...
     &                 +  0.632 * RXRAT( 263 )      ! XYOP+MO2=0.632*MGLY+HCHO+...
     &                 +  0.500 * RXRAT( 274 )      ! MVKP+MO2=0.5*MGLY+...
     &                 +  0.105 * RXRAT( 275 )      ! UALP+MO2=0.105*MGLY+...
     &                 +          RXRAT( 295 )      ! PER1+ACO3=MGLY+MO2+HO2+DCB1
     &                 +          RXRAT( 298 )      ! PER2+ACO3=MGLY+MO2+HO2+...
     &                 +  0.632 * RXRAT( 299 )      ! XYOP+ACO3=0.632*MGLY+MO2+...
     &                 +  0.500 * RXRAT( 310 )      ! MVKP+ACO3=0.5*MGLY+0.5*MO2+...
     &                 +  0.105 * RXRAT( 311 )      ! UALP+ACO3=0.105*MGLY+...
     &                 +  0.500 * RXRAT( 333 )      ! PER1+NO3=0.5*MGLY+NO2+...
     &                 +          RXRAT( 336 )      ! PER2+NO3=MGLY+NO2+HO2+DCB1+...
     &                 +  0.632 * RXRAT( 337 )      ! XYOP+NO3=0.632*MGLY+NO2+...
     &                 +  0.300 * RXRAT( 348 )      ! MVKP+NO3=0.3*MGLY+0.7*ACO3+...
     &                 +  0.210 * RXRAT( 349 )      ! UALP+NO3=0.21*MGLY+NO2+...
      LOSS( MGLY    ) =           RXRAT(  24 )      ! MGLY=HO2+ACO3+CO
     &                 +          RXRAT( 100 )      ! MGLY+HO=ACO3+CO
     &                 +          RXRAT( 154 )      ! MGLY+NO3=ACO3+CO+HNO3

      PROD( DCB1    ) =   0.700 * RXRAT( 137 )      ! DCB2+O3=0.7*DCB1+HO2+...
     &                 +  0.700 * RXRAT( 138 )      ! DCB3+O3=0.7*DCB1+HO2+...
     &                 +  0.500 * RXRAT( 183 )      ! PER1+NO=0.5*DCB1+0.95*NO2+...
     &                 +  0.950 * RXRAT( 186 )      ! PER2+NO=0.95*DCB1+0.95*NO2+...
     &                 +  0.700 * RXRAT( 187 )      ! XYOP+NO=0.7*DCB1+0.95*NO2+...
     &                 +  0.670 * RXRAT( 194 )      ! MEKP+NO=0.67*DCB1+NO2+...
     &                 +          RXRAT( 259 )      ! PER1+MO2=DCB1+HCHO+MGLY+2*HO2
     &                 +          RXRAT( 262 )      ! PER2+MO2=DCB1+HCHO+MGLY+...
     &                 +  0.737 * RXRAT( 263 )      ! XYOP+MO2=0.737*DCB1+HCHO+...
     &                 +  0.334 * RXRAT( 270 )      ! MEKP+MO2=0.334*DCB1+HCHO+...
     &                 +  0.500 * RXRAT( 271 )      ! KETP+MO2=0.5*DCB1+...
     &                 +          RXRAT( 295 )      ! PER1+ACO3=DCB1+MO2+MGLY+HO2
     &                 +          RXRAT( 298 )      ! PER2+ACO3=DCB1+MO2+MGLY+...
     &                 +  0.737 * RXRAT( 299 )      ! XYOP+ACO3=0.737*DCB1+MO2+...
     &                 +  0.334 * RXRAT( 306 )      ! MEKP+ACO3=0.334*DCB1+...
     &                 +  0.500 * RXRAT( 307 )      ! KETP+ACO3=0.5*DCB1+0.5*MO2+...
     &                 +  0.500 * RXRAT( 333 )      ! PER1+NO3=0.5*DCB1+NO2+...
     &                 +          RXRAT( 336 )      ! PER2+NO3=DCB1+NO2+MGLY+HO2+...
     &                 +  0.737 * RXRAT( 337 )      ! XYOP+NO3=0.737*DCB1+NO2+...
     &                 +  0.670 * RXRAT( 344 )      ! MEKP+NO3=0.67*DCB1+NO2+...
     &                 +          RXRAT( 345 )      ! KETP+NO3=DCB1+NO2+HO2
      LOSS( DCB1    ) =           RXRAT(  25 )      ! DCB1=1.5*HO2+0.25*ACO3+...
     &                 +          RXRAT( 101 )      ! DCB1+HO=0.52*HO2+0.33*CO+...
     &                 +          RXRAT( 136 )      ! DCB1+O3=0.05*HO+HO2+...

      PROD( DCB2    ) =   0.280 * RXRAT( 160 )      ! TR2=0.28*DCB2+0.29*HO2+...
     &                 +  0.490 * RXRAT( 161 )      ! TOLP=0.49*DCB2+0.01*HO2+...
     &                 +  0.224 * RXRAT( 162 )      ! XY2=0.224*DCB2+0.308*HO2+...
     &                 +  0.490 * RXRAT( 163 )      ! XYLP=0.49*DCB2+0.01*HO2+...
     &                 +  0.224 * RXRAT( 164 )      ! XYO2=0.224*DCB2+0.308*HO2+...
     &                 +  0.490 * RXRAT( 165 )      ! XYOP=0.49*DCB2+0.01*HO2+...
     &                 +  0.459 * RXRAT( 180 )      ! BENP+NO=0.459*DCB2+...
     &                 +  0.950 * RXRAT( 182 )      ! TOLP+NO=0.95*DCB2+0.95*NO2+...
     &                 +  0.073 * RXRAT( 187 )      ! XYOP+NO=0.073*DCB2+...
     &                 +  0.459 * RXRAT( 256 )      ! BENP+MO2=0.459*DCB2+...
     &                 +          RXRAT( 258 )      ! TOLP+MO2=DCB2+HCHO+...
     &                 +          RXRAT( 261 )      ! XYLP+MO2=DCB2+HCHO+2*HO2
     &                 +  0.077 * RXRAT( 263 )      ! XYOP+MO2=0.077*DCB2+HCHO+...
     &                 +  0.459 * RXRAT( 292 )      ! BENP+ACO3=0.459*DCB2+MO2+...
     &                 +          RXRAT( 294 )      ! TOLP+ACO3=DCB2+MO2+HO2
     &                 +          RXRAT( 297 )      ! XYLP+ACO3=DCB2+MO2+HO2
     &                 +  0.077 * RXRAT( 299 )      ! XYOP+ACO3=0.077*DCB2+MO2+...
     &                 +  0.500 * RXRAT( 330 )      ! BENP+NO3=0.5*DCB2+NO2+HO2+...
     &                 +          RXRAT( 332 )      ! TOLP+NO3=DCB2+NO2+HO2
     &                 +  0.077 * RXRAT( 337 )      ! XYOP+NO3=0.077*DCB2+NO2+...
      LOSS( DCB2    ) =           RXRAT(  26 )      ! DCB2=1.5*HO2+0.25*ACO3+...
     &                 +          RXRAT( 102 )      ! DCB2+HO=0.52*HO2+0.33*CO+...
     &                 +          RXRAT( 137 )      ! DCB2+O3=0.05*HO+HO2+...

      PROD( BALD    ) =   0.020 * RXRAT( 127 )      ! OLT+O3=0.02*BALD+0.32*HO2+...
     &                 +  0.850 * RXRAT( 139 )      ! EPX+O3=0.85*BALD+1.5*HO2+...
     &                 +          RXRAT( 181 )      ! TLP1+NO=BALD+NO2
     &                 +  0.500 * RXRAT( 183 )      ! PER1+NO=0.5*BALD+0.95*NO2+...
     &                 +          RXRAT( 184 )      ! XYL1+NO=BALD+NO2
     &                 +          RXRAT( 257 )      ! TLP1+MO2=BALD+HCHO+HO2
     &                 +          RXRAT( 260 )      ! XYL1+MO2=BALD+HCHO+HO2
     &                 +          RXRAT( 293 )      ! TLP1+ACO3=BALD+MO2
     &                 +          RXRAT( 296 )      ! XYL1+ACO3=BALD+MO2
     &                 +          RXRAT( 331 )      ! TLP1+NO3=BALD+NO2
     &                 +  0.500 * RXRAT( 333 )      ! PER1+NO3=0.5*BALD+NO2+...
     &                 +          RXRAT( 334 )      ! XYL1+NO3=BALD+NO2
      LOSS( BALD    ) =           RXRAT(  27 )      ! BALD=CHO+HO2+CO
     &                 +          RXRAT( 104 )      ! BALD+HO=BALP

      PROD( CHO     ) =           RXRAT(  27 )      ! BALD=CHO+HO2+CO
     &                 +  0.070 * RXRAT( 105 )      ! PHEN+HO=0.07*CHO+0.2*ADDC+...
     &                 +  0.070 * RXRAT( 106 )      ! CSL+HO=0.07*CHO+0.2*ADDC+...
     &                 +  0.400 * RXRAT( 155 )      ! PHEN+NO3=0.4*CHO+0.1*ADDC+...
     &                 +  0.400 * RXRAT( 156 )      ! CSL+NO3=0.4*CHO+0.1*ADDC+...
      LOSS( CHO     ) =           RXRAT( 210 )      ! CHO+NO2=ONIT
     &                 +          RXRAT( 241 )      ! CHO+HO2=CSL

      PROD( OP1     ) =           RXRAT( 212 )      ! MO2+HO2=OP1
      LOSS( OP1     ) =           RXRAT(  28 )      ! OP1=HO+HO2+HCHO
     &                 +          RXRAT( 113 )      ! OP1+HO=0.35*HO+0.65*MO2+...

      PROD( OP2     ) =   0.780 * RXRAT( 102 )      ! DCB2+HO=0.78*OP2+0.33*CO+...
     &                 +  0.790 * RXRAT( 103 )      ! DCB3+HO=0.79*OP2+0.21*MACP+...
     &                 +  0.650 * RXRAT( 136 )      ! DCB1+O3=0.65*OP2+HO2+...
     &                 +  0.650 * RXRAT( 137 )      ! DCB2+O3=0.65*OP2+HO2+...
     &                 +  0.680 * RXRAT( 202 )      ! ADDC+NO=0.68*OP2+NO2+...
     &                 +          RXRAT( 207 )      ! ADCN+NO=OP2+GLY+2*NO2
     &                 +          RXRAT( 213 )      ! ETHP+HO2=OP2
     &                 +          RXRAT( 214 )      ! HC3P+HO2=OP2
     &                 +          RXRAT( 215 )      ! HC5P+HO2=OP2
     &                 +          RXRAT( 216 )      ! HC8P+HO2=OP2
     &                 +          RXRAT( 217 )      ! ETEP+HO2=OP2
     &                 +          RXRAT( 218 )      ! OLTP+HO2=OP2
     &                 +          RXRAT( 219 )      ! OLIP+HO2=OP2
     &                 +          RXRAT( 220 )      ! BENP+HO2=OP2
     &                 +          RXRAT( 221 )      ! TLP1+HO2=OP2
     &                 +          RXRAT( 222 )      ! TOLP+HO2=OP2
     &                 +          RXRAT( 223 )      ! PER1+HO2=OP2
     &                 +          RXRAT( 224 )      ! XYL1+HO2=OP2
     &                 +          RXRAT( 225 )      ! XYLP+HO2=OP2
     &                 +          RXRAT( 226 )      ! PER2+HO2=OP2
     &                 +          RXRAT( 227 )      ! XYOP+HO2=OP2
     &                 +          RXRAT( 229 )      ! APIP+HO2=OP2
     &                 +          RXRAT( 230 )      ! LIMP+HO2=OP2
     &                 +  0.850 * RXRAT( 233 )      ! ACTP+HO2=0.85*OP2+...
     &                 +          RXRAT( 234 )      ! MEKP+HO2=OP2
     &                 +          RXRAT( 235 )      ! KETP+HO2=OP2
     &                 +          RXRAT( 238 )      ! MVKP+HO2=OP2
     &                 +          RXRAT( 239 )      ! UALP+HO2=OP2
     &                 +          RXRAT( 240 )      ! ADDC+HO2=OP2
     &                 +          RXRAT( 242 )      ! MCTP+HO2=OP2
     &                 +          RXRAT( 243 )      ! ORAP+HO2=OP2
     &                 +          RXRAT( 246 )      ! ADCN+HO2=OP2
     &                 +          RXRAT( 247 )      ! XO2+HO2=OP2
     &                 +  0.680 * RXRAT( 278 )      ! ADDC+MO2=0.68*OP2+HCHO+...
     &                 +  0.700 * RXRAT( 283 )      ! ADCN+MO2=0.7*OP2+0.7*NO2+...
     &                 +  0.680 * RXRAT( 314 )      ! ADDC+ACO3=0.68*OP2+MO2+...
     &                 +  0.700 * RXRAT( 319 )      ! ADCN+ACO3=0.7*OP2+MO2+...
     &                 +  0.680 * RXRAT( 352 )      ! ADDC+NO3=0.68*OP2+NO2+...
     &                 +          RXRAT( 357 )      ! ADCN+NO3=OP2+GLY+2*NO2
      LOSS( OP2     ) =           RXRAT(  29 )      ! OP2=HO+HO2+ALD
     &                 +          RXRAT( 114 )      ! OP2+HO=0.01*HO+0.44*HC3P+...

      PROD( PAA     ) =   0.110 * RXRAT( 138 )      ! DCB3+O3=0.11*PAA+HO2+...
     &                 +  0.410 * RXRAT( 231 )      ! ACO3+HO2=0.41*PAA+0.44*MO2+...
     &                 +  0.410 * RXRAT( 232 )      ! RCO3+HO2=0.41*PAA+...
      LOSS( PAA     ) =           RXRAT(  30 )      ! PAA=HO+MO2
     &                 +          RXRAT( 119 )      ! PAA+HO=0.35*HO+0.65*ACO3+...

      PROD( ONIT    ) =           RXRAT( 152 )      ! UALD+NO3=ONIT+XO2+0.668*CO+...
     &                 +  0.065 * RXRAT( 174 )      ! HC3P+NO=0.065*ONIT+...
     &                 +  0.136 * RXRAT( 175 )      ! HC5P+NO=0.136*ONIT+...
     &                 +  0.261 * RXRAT( 176 )      ! HC8P+NO=0.261*ONIT+...
     &                 +  0.030 * RXRAT( 178 )      ! OLTP+NO=0.03*ONIT+0.97*NO2+...
     &                 +  0.050 * RXRAT( 179 )      ! OLIP+NO=0.05*ONIT+0.95*NO2+...
     &                 +  0.082 * RXRAT( 180 )      ! BENP+NO=0.082*ONIT+...
     &                 +  0.050 * RXRAT( 182 )      ! TOLP+NO=0.05*ONIT+0.95*NO2+...
     &                 +  0.050 * RXRAT( 183 )      ! PER1+NO=0.05*ONIT+0.95*NO2+...
     &                 +  0.050 * RXRAT( 185 )      ! XYLP+NO=0.05*ONIT+0.95*NO2+...
     &                 +  0.050 * RXRAT( 186 )      ! PER2+NO=0.05*ONIT+0.95*NO2+...
     &                 +  0.050 * RXRAT( 187 )      ! XYOP+NO=0.05*ONIT+0.95*NO2+...
     &                 +  0.180 * RXRAT( 189 )      ! APIP+NO=0.18*ONIT+0.82*NO2+...
     &                 +          RXRAT( 205 )      ! OLNN+NO=ONIT+HO2+NO2
     &                 +          RXRAT( 209 )      ! BAL2+NO2=ONIT
     &                 +          RXRAT( 210 )      ! CHO+NO2=ONIT
     &                 +          RXRAT( 211 )      ! MCTO+NO2=ONIT
     &                 +          RXRAT( 244 )      ! OLNN+HO2=ONIT
     &                 +          RXRAT( 245 )      ! OLND+HO2=ONIT
     &                 +          RXRAT( 281 )      ! OLNN+MO2=ONIT+HCHO+2*HO2
     &                 +  0.500 * RXRAT( 282 )      ! OLND+MO2=0.5*ONIT+0.5*NO2+...
     &                 +  0.300 * RXRAT( 283 )      ! ADCN+MO2=0.3*ONIT+0.7*NO2+...
     &                 +          RXRAT( 317 )      ! OLNN+ACO3=ONIT+MO2+HO2
     &                 +  0.300 * RXRAT( 319 )      ! ADCN+ACO3=0.3*ONIT+MO2+...
     &                 +          RXRAT( 355 )      ! OLNN+NO3=ONIT+NO2+HO2
     &                 +  2.000 * RXRAT( 358 )      ! OLNN+OLNN=2*ONIT+HO2
     &                 +  1.500 * RXRAT( 359 )      ! OLNN+OLND=1.5*ONIT+0.5*NO2+...
     &                 +          RXRAT( 360 )      ! OLND+OLND=ONIT+0.504*HCHO+...
      LOSS( ONIT    ) =           RXRAT(  31 )      ! ONIT=HO2+NO2+0.2*ALD+0.8*KET
     &                 +          RXRAT( 123 )      ! ONIT+HO=HC3P+NO2

      PROD( SO2     ) =   0.0
      LOSS( SO2     ) =           RXRAT(  69 )      ! SO2+HO=HO2+SULF+SULRXN

      PROD( SULF    ) =           RXRAT(  69 )      ! SO2+HO=SULF+HO2+SULRXN
      LOSS( SULF    ) =   0.0

      PROD( SULRXN  ) =           RXRAT(  69 )      ! SO2+HO=SULRXN+SULF+HO2
      LOSS( SULRXN  ) =   0.0

      PROD( ETH     ) =   0.020 * RXRAT( 127 )      ! OLT+O3=0.02*ETH+0.32*HO2+...
     &                 +  0.010 * RXRAT( 128 )      ! OLI+O3=0.01*ETH+0.07*HO2+...
      LOSS( ETH     ) =           RXRAT(  72 )      ! ETH+HO=ETHP

      PROD( HC3     ) =   0.015 * RXRAT( 127 )      ! OLT+O3=0.015*HC3+0.32*HO2+...
     &                 +  0.010 * RXRAT( 128 )      ! OLI+O3=0.01*HC3+0.07*HO2+...
      LOSS( HC3     ) =           RXRAT(  73 )      ! HC3+HO=HC3P

      PROD( HC3P    ) =           RXRAT(  73 )      ! HC3+HO=HC3P
     &                 +  0.440 * RXRAT( 114 )      ! OP2+HO=0.44*HC3P+0.01*HO+...
     &                 +          RXRAT( 123 )      ! ONIT+HO=HC3P+NO2
     &                 +  0.040 * RXRAT( 127 )      ! OLT+O3=0.04*HC3P+0.32*HO2+...
     &                 +  0.040 * RXRAT( 128 )      ! OLI+O3=0.04*HC3P+0.07*HO2+...
      LOSS( HC3P    ) =           RXRAT( 174 )      ! HC3P+NO=0.66*HO2+0.131*MO2+...
     &                 +          RXRAT( 214 )      ! HC3P+HO2=OP2
     &                 +          RXRAT( 250 )      ! HC3P+MO2=0.894*HO2+...
     &                 +          RXRAT( 286 )      ! HC3P+ACO3=0.394*HO2+...
     &                 +          RXRAT( 324 )      ! HC3P+NO3=0.254*HO2+...

      PROD( HC5     ) =   0.006 * RXRAT( 127 )      ! OLT+O3=0.006*HC5+0.32*HO2+...
      LOSS( HC5     ) =           RXRAT(  74 )      ! HC5+HO=HC5P

      PROD( HC5P    ) =           RXRAT(  74 )      ! HC5+HO=HC5P
     &                 +  0.020 * RXRAT( 127 )      ! OLT+O3=0.02*HC5P+0.32*HO2+...
      LOSS( HC5P    ) =           RXRAT( 175 )      ! HC5P+NO=0.2*HO2+0.051*MO2+...
     &                 +          RXRAT( 215 )      ! HC5P+HO2=OP2
     &                 +          RXRAT( 251 )      ! HC5P+MO2=0.842*HO2+...
     &                 +          RXRAT( 287 )      ! HC5P+ACO3=0.342*HO2+...
     &                 +          RXRAT( 325 )      ! HC5P+NO3=0.488*HO2+...

      PROD( HC8     ) =   0.0
      LOSS( HC8     ) =           RXRAT(  75 )      ! HC8+HO=0.049*HO2+...

      PROD( HC8P    ) =   0.951 * RXRAT(  75 )      ! HC8+HO=0.951*HC8P+...
      LOSS( HC8P    ) =           RXRAT( 176 )      ! HC8P+NO=0.606*HO2+...
     &                 +          RXRAT( 216 )      ! HC8P+HO2=OP2
     &                 +          RXRAT( 252 )      ! HC8P+MO2=0.91*HO2+...
     &                 +          RXRAT( 288 )      ! HC8P+ACO3=0.303*HO2+...
     &                 +          RXRAT( 326 )      ! HC8P+NO3=0.82*HO2+...

      PROD( ALK5RXN ) =           RXRAT(  75 )      ! HC8+HO=ALK5RXN+0.951*HC8P+...
      LOSS( ALK5RXN ) =   0.0

      PROD( ETE     ) =   0.0
      LOSS( ETE     ) =           RXRAT(  76 )      ! ETE+HO=ETEP
     &                 +          RXRAT( 126 )      ! ETE+O3=0.08*HO+0.15*HO2+...
     &                 +          RXRAT( 141 )      ! ETE+NO3=0.8*OLNN+0.2*OLND

      PROD( ETEP    ) =           RXRAT(  76 )      ! ETE+HO=ETEP
      LOSS( ETEP    ) =           RXRAT( 177 )      ! ETEP+NO=HO2+NO2+1.6*HCHO+...
     &                 +          RXRAT( 217 )      ! ETEP+HO2=OP2
     &                 +          RXRAT( 253 )      ! ETEP+MO2=HO2+1.95*HCHO+...
     &                 +          RXRAT( 289 )      ! ETEP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 327 )      ! ETEP+NO3=HO2+NO2+1.6*HCHO+...

      PROD( OLT     ) =   0.350 * RXRAT( 129 )      ! DIEN+O3=0.35*OLT+0.28*HO+...
     &                 +  0.153 * RXRAT( 130 )      ! ISO+O3=0.153*OLT+0.25*HO2+...
     &                 +  0.460 * RXRAT( 132 )      ! LIM+O3=0.46*OLT+0.1*HO2+...
      LOSS( OLT     ) =           RXRAT(  77 )      ! OLT+HO=OLTP
     &                 +          RXRAT( 127 )      ! OLT+O3=0.22*HO+0.32*HO2+...
     &                 +          RXRAT( 142 )      ! OLT+NO3=0.43*OLNN+0.57*OLND

      PROD( OLTP    ) =           RXRAT(  77 )      ! OLT+HO=OLTP
      LOSS( OLTP    ) =           RXRAT( 178 )      ! OLTP+NO=0.78*HO2+0.97*NO2+...
     &                 +          RXRAT( 218 )      ! OLTP+HO2=OP2
     &                 +          RXRAT( 254 )      ! OLTP+MO2=HO2+1.5*HCHO+...
     &                 +          RXRAT( 290 )      ! OLTP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 328 )      ! OLTP+NO3=0.47*ALD+...

      PROD( OLI     ) =   0.050 * RXRAT( 190 )      ! LIMP+NO=0.05*OLI+0.68*UALD+...
     &                 +  0.192 * RXRAT( 266 )      ! LIMP+MO2=0.192*OLI+...
     &                 +  0.192 * RXRAT( 302 )      ! LIMP+ACO3=0.192*OLI+...
     &                 +  0.385 * RXRAT( 340 )      ! LIMP+NO3=0.385*OLI+NO2+HO2+...
      LOSS( OLI     ) =           RXRAT(  78 )      ! OLI+HO=OLIP
     &                 +          RXRAT( 128 )      ! OLI+O3=0.46*HO+0.07*HO2+...
     &                 +          RXRAT( 143 )      ! OLI+NO3=0.11*OLNN+0.89*OLND

      PROD( OLIP    ) =           RXRAT(  78 )      ! OLI+HO=OLIP
     &                 +          RXRAT(  79 )      ! DIEN+HO=OLIP
      LOSS( OLIP    ) =           RXRAT( 179 )      ! OLIP+NO=0.83*HO2+0.95*NO2+...
     &                 +          RXRAT( 219 )      ! OLIP+HO2=OP2
     &                 +          RXRAT( 255 )      ! OLIP+MO2=HO2+0.75*HCHO+...
     &                 +          RXRAT( 291 )      ! OLIP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 329 )      ! OLIP+NO3=0.86*HO2+0.72*ALD+...

      PROD( DIEN    ) =   0.0
      LOSS( DIEN    ) =           RXRAT(  79 )      ! DIEN+HO=OLIP
     &                 +          RXRAT( 129 )      ! DIEN+O3=0.09*O3P+0.28*HO+...
     &                 +          RXRAT( 144 )      ! DIEN+NO3=0.9*OLNN+0.1*OLND+...

      PROD( ACE     ) =   0.0
      LOSS( ACE     ) =           RXRAT(  80 )      ! ACE+HO=0.65*HO+0.35*HO2+...

      PROD( ORA1    ) =   0.350 * RXRAT(  80 )      ! ACE+HO=0.35*ORA1+0.35*HO2+...
     &                 +  0.370 * RXRAT( 126 )      ! ETE+O3=0.37*ORA1+0.15*HO2+...
     &                 +  0.030 * RXRAT( 127 )      ! OLT+O3=0.03*ORA1+0.32*HO2+...
     &                 +  0.150 * RXRAT( 129 )      ! DIEN+O3=0.15*ORA1+0.28*HO+...
     &                 +  0.280 * RXRAT( 130 )      ! ISO+O3=0.28*ORA1+0.25*HO2+...
     &                 +  0.010 * RXRAT( 132 )      ! LIM+O3=0.01*ORA1+0.1*HO2+...
     &                 +  0.450 * RXRAT( 133 )      ! MACR+O3=0.45*ORA1+0.14*HO2+...
     &                 +  0.070 * RXRAT( 134 )      ! MVK+O3=0.07*ORA1+0.11*HO2+...
     &                 +  0.044 * RXRAT( 135 )      ! UALD+O3=0.044*ORA1+...
     &                 +  0.250 * RXRAT( 138 )      ! DCB3+O3=0.25*ORA1+HO2+...
     &                 +  0.070 * RXRAT( 189 )      ! APIP+NO=0.07*ORA1+0.82*NO2+...
     &                 +  0.070 * RXRAT( 190 )      ! LIMP+NO=0.07*ORA1+...
      LOSS( ORA1    ) =           RXRAT( 117 )      ! ORA1+HO=HO2

      PROD( BENZENE ) =   0.032 * RXRAT( 127 )      ! OLT+O3=0.032*BENZENE+...
      LOSS( BENZENE ) =           RXRAT(  81 )      ! BENZENE+HO=0.648*HO2+...

      PROD( BENP    ) =   0.352 * RXRAT(  81 )      ! BENZENE+HO=0.352*BENP+...
      LOSS( BENP    ) =           RXRAT( 180 )      ! BENP+NO=0.918*HO2+...
     &                 +          RXRAT( 220 )      ! BENP+HO2=OP2
     &                 +          RXRAT( 256 )      ! BENP+MO2=1.6*HO2+...
     &                 +          RXRAT( 292 )      ! BENP+ACO3=0.6*HO2+MO2+...
     &                 +          RXRAT( 330 )      ! BENP+NO3=HO2+NO2+0.5*DCB2+...

      PROD( EPX     ) =   0.118 * RXRAT(  81 )      ! BENZENE+HO=0.118*EPX+...
     &                 +  0.280 * RXRAT( 160 )      ! TR2=0.28*EPX+0.29*HO2+...
     &                 +  0.840 * RXRAT( 162 )      ! XY2=0.84*EPX+0.308*HO2+...
     &                 +  0.840 * RXRAT( 164 )      ! XYO2=0.84*EPX+0.308*HO2+...
      LOSS( EPX     ) =           RXRAT( 107 )      ! EPX+HO=HO2+XO2+CO+ALD
     &                 +          RXRAT( 139 )      ! EPX+O3=0.05*HO+1.5*HO2+...
     &                 +          RXRAT( 157 )      ! EPX+NO3=0.5*HO+1.5*HO2+...

      PROD( PHEN    ) =   0.530 * RXRAT(  81 )      ! BENZENE+HO=0.53*PHEN+...
      LOSS( PHEN    ) =           RXRAT( 105 )      ! PHEN+HO=0.73*HO2+0.2*ADDC+...
     &                 +          RXRAT( 155 )      ! PHEN+NO3=0.4*CHO+0.1*ADDC+...

      PROD( BENZRO2 ) =           RXRAT(  81 )      ! BENZENE+HO=BENZRO2+...
      LOSS( BENZRO2 ) =           RXRAT( 368 )      ! BENZRO2+NO=NO+BNZNRXN
     &                 +          RXRAT( 369 )      ! BENZRO2+HO2=HO2+BNZHRXN

      PROD( TOL     ) =   0.0
      LOSS( TOL     ) =           RXRAT(  82 )      ! TOL+HO=0.177*HO2+0.763*TR2+...

      PROD( TR2     ) =   0.763 * RXRAT(  82 )      ! TOL+HO=0.763*TR2+0.177*HO2+...
      LOSS( TR2     ) =           RXRAT( 160 )      ! TR2=0.28*HO+0.29*HO2+...

      PROD( TLP1    ) =   0.060 * RXRAT(  82 )      ! TOL+HO=0.06*TLP1+0.763*TR2+...
      LOSS( TLP1    ) =           RXRAT( 181 )      ! TLP1+NO=NO2+BALD
     &                 +          RXRAT( 221 )      ! TLP1+HO2=OP2
     &                 +          RXRAT( 257 )      ! TLP1+MO2=HO2+HCHO+BALD
     &                 +          RXRAT( 293 )      ! TLP1+ACO3=MO2+BALD
     &                 +          RXRAT( 331 )      ! TLP1+NO3=NO2+BALD

      PROD( CSL     ) =   0.177 * RXRAT(  82 )      ! TOL+HO=0.177*CSL+0.763*TR2+...
     &                 +  0.177 * RXRAT(  83 )      ! XYM+HO=0.177*CSL+0.763*XY2+...
     &                 +  0.177 * RXRAT(  84 )      ! XYP+HO=0.177*CSL+0.763*XY2+...
     &                 +  0.177 * RXRAT(  85 )      ! XYO+HO=0.177*CSL+...
     &                 +  0.010 * RXRAT( 160 )      ! TR2=0.01*CSL+0.29*HO2+...
     &                 +  0.010 * RXRAT( 161 )      ! TOLP=0.01*CSL+0.01*HO2+...
     &                 +  0.010 * RXRAT( 162 )      ! XY2=0.01*CSL+0.308*HO2+...
     &                 +  0.010 * RXRAT( 163 )      ! XYLP=0.01*CSL+0.01*HO2+...
     &                 +  0.010 * RXRAT( 164 )      ! XYO2=0.01*CSL+0.308*HO2+...
     &                 +  0.010 * RXRAT( 165 )      ! XYOP=0.01*CSL+0.01*HO2+...
     &                 +          RXRAT( 241 )      ! CHO+HO2=CSL
      LOSS( CSL     ) =           RXRAT( 106 )      ! CSL+HO=0.73*HO2+0.2*ADDC+...
     &                 +          RXRAT( 156 )      ! CSL+NO3=0.4*CHO+0.1*ADDC+...

      PROD( TOLRO2  ) =           RXRAT(  82 )      ! TOL+HO=TOLRO2+0.763*TR2+...
      LOSS( TOLRO2  ) =           RXRAT( 364 )      ! TOLRO2+NO=NO+TOLNRXN
     &                 +          RXRAT( 365 )      ! TOLRO2+HO2=HO2+TOLHRXN

      PROD( XYM     ) =   0.0
      LOSS( XYM     ) =           RXRAT(  83 )      ! XYM+HO=0.177*HO2+0.763*XY2+...

      PROD( XY2     ) =   0.763 * RXRAT(  83 )      ! XYM+HO=0.763*XY2+0.177*HO2+...
     &                 +  0.763 * RXRAT(  84 )      ! XYP+HO=0.763*XY2+0.177*HO2+...
      LOSS( XY2     ) =           RXRAT( 162 )      ! XY2=0.158*HO+0.308*HO2+...

      PROD( XYL1    ) =   0.060 * RXRAT(  83 )      ! XYM+HO=0.06*XYL1+0.763*XY2+...
     &                 +  0.060 * RXRAT(  84 )      ! XYP+HO=0.06*XYL1+0.763*XY2+...
     &                 +  0.060 * RXRAT(  85 )      ! XYO+HO=0.06*XYL1+...
      LOSS( XYL1    ) =           RXRAT( 184 )      ! XYL1+NO=NO2+BALD
     &                 +          RXRAT( 224 )      ! XYL1+HO2=OP2
     &                 +          RXRAT( 260 )      ! XYL1+MO2=HO2+HCHO+BALD
     &                 +          RXRAT( 296 )      ! XYL1+ACO3=MO2+BALD
     &                 +          RXRAT( 334 )      ! XYL1+NO3=NO2+BALD

      PROD( XYLRO2  ) =           RXRAT(  83 )      ! XYM+HO=XYLRO2+0.763*XY2+...
     &                 +          RXRAT(  84 )      ! XYP+HO=XYLRO2+0.763*XY2+...
     &                 +          RXRAT(  85 )      ! XYO+HO=XYLRO2+0.763*XYO2+...
      LOSS( XYLRO2  ) =           RXRAT( 366 )      ! XYLRO2+NO=NO+XYLNRXN
     &                 +          RXRAT( 367 )      ! XYLRO2+HO2=HO2+XYLHRXN

      PROD( XYP     ) =   0.0
      LOSS( XYP     ) =           RXRAT(  84 )      ! XYP+HO=0.177*HO2+0.763*XY2+...

      PROD( XYO     ) =   0.0
      LOSS( XYO     ) =           RXRAT(  85 )      ! XYO+HO=0.177*HO2+...

      PROD( XYO2    ) =   0.763 * RXRAT(  85 )      ! XYO+HO=0.763*XYO2+...
      LOSS( XYO2    ) =           RXRAT( 164 )      ! XYO2=0.158*HO+0.308*HO2+...

      PROD( ISO     ) =   0.0
      LOSS( ISO     ) =           RXRAT(  86 )      ! ISO+HO=ISOP+ISOPRXN
     &                 +          RXRAT( 130 )      ! ISO+O3=0.25*HO+0.25*HO2+...
     &                 +          RXRAT( 145 )      ! ISO+NO3=ISON

      PROD( ISOP    ) =           RXRAT(  86 )      ! ISO+HO=ISOP+ISOPRXN
      LOSS( ISOP    ) =           RXRAT( 188 )      ! ISOP+NO=0.88*HO2+0.88*NO2+...
     &                 +          RXRAT( 228 )      ! ISOP+HO2=ISHP
     &                 +          RXRAT( 264 )      ! ISOP+MO2=HO2+1.31*HCHO+...
     &                 +          RXRAT( 300 )      ! ISOP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 338 )      ! ISOP+NO3=HO2+NO2+0.75*HCHO+...

      PROD( ISOPRXN ) =           RXRAT(  86 )      ! ISO+HO=ISOPRXN+ISOP
      LOSS( ISOPRXN ) =   0.0

      PROD( API     ) =   0.0
      LOSS( API     ) =           RXRAT(  87 )      ! API+HO=APIP+TRPRXN
     &                 +          RXRAT( 131 )      ! API+O3=0.85*HO+0.1*HO2+...
     &                 +          RXRAT( 146 )      ! API+NO3=0.1*OLNN+0.9*OLND+...

      PROD( APIP    ) =           RXRAT(  87 )      ! API+HO=APIP+TRPRXN
      LOSS( APIP    ) =           RXRAT( 189 )      ! APIP+NO=0.82*HO2+0.82*NO2+...
     &                 +          RXRAT( 229 )      ! APIP+HO2=OP2
     &                 +          RXRAT( 265 )      ! APIP+MO2=HO2+0.75*HCHO+...
     &                 +          RXRAT( 301 )      ! APIP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 339 )      ! APIP+NO3=HO2+NO2+ALD+KET

      PROD( TRPRXN  ) =           RXRAT(  87 )      ! API+HO=TRPRXN+APIP
     &                 +          RXRAT(  88 )      ! LIM+HO=TRPRXN+LIMP
     &                 +          RXRAT( 131 )      ! API+O3=TRPRXN+0.1*HO2+...
     &                 +          RXRAT( 132 )      ! LIM+O3=TRPRXN+0.1*HO2+...
     &                 +          RXRAT( 146 )      ! API+NO3=TRPRXN+0.9*OLND+...
     &                 +          RXRAT( 147 )      ! LIM+NO3=TRPRXN+0.29*OLND+...
      LOSS( TRPRXN  ) =   0.0

      PROD( LIM     ) =   0.0
      LOSS( LIM     ) =           RXRAT(  88 )      ! LIM+HO=LIMP+TRPRXN
     &                 +          RXRAT( 132 )      ! LIM+O3=0.85*HO+0.1*HO2+...
     &                 +          RXRAT( 147 )      ! LIM+NO3=0.71*OLNN+...

      PROD( LIMP    ) =           RXRAT(  88 )      ! LIM+HO=LIMP+TRPRXN
      LOSS( LIMP    ) =           RXRAT( 190 )      ! LIMP+NO=HO2+0.68*UALD+...
     &                 +          RXRAT( 230 )      ! LIMP+HO2=OP2
     &                 +          RXRAT( 266 )      ! LIMP+MO2=HO2+1.04*HCHO+...
     &                 +          RXRAT( 302 )      ! LIMP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 340 )      ! LIMP+NO3=HO2+NO2+0.385*OLI+...

      PROD( RCO3    ) =           RXRAT(  91 )      ! ALD+HO=RCO3
     &                 +  0.600 * RXRAT( 136 )      ! DCB1+O3=0.6*RCO3+HO2+...
     &                 +  0.600 * RXRAT( 137 )      ! DCB2+O3=0.6*RCO3+HO2+...
     &                 +          RXRAT( 150 )      ! ALD+NO3=RCO3+HNO3
     &                 +  0.250 * RXRAT( 162 )      ! XY2=0.25*RCO3+0.308*HO2+...
     &                 +  0.250 * RXRAT( 164 )      ! XYO2=0.25*RCO3+0.308*HO2+...
     &                 +          RXRAT( 169 )      ! PPN=RCO3+NO2
      LOSS( RCO3    ) =           RXRAT( 168 )      ! RCO3+NO2=PPN
     &                 +          RXRAT( 192 )      ! RCO3+NO=ETHP+NO2
     &                 +          RXRAT( 232 )      ! RCO3+HO2=0.44*HO+0.44*ETHP+...
     &                 +          RXRAT( 268 )      ! RCO3+MO2=0.9*HO2+0.9*MO2+...
     &                 +          RXRAT( 304 )      ! RCO3+ACO3=MO2+ETHP
     &                 +  2.000 * RXRAT( 321 )      ! RCO3+RCO3=2*ETHP
     &                 +          RXRAT( 342 )      ! RCO3+NO3=ETHP+NO2
     &                 +          RXRAT( 362 )      ! RCO3+XO2=ETHP

      PROD( ACTP    ) =           RXRAT(  92 )      ! ACT+HO=ACTP
      LOSS( ACTP    ) =           RXRAT( 193 )      ! ACTP+NO=ACO3+NO2+HCHO
     &                 +          RXRAT( 233 )      ! ACTP+HO2=0.15*HO+0.15*ACO3+...
     &                 +          RXRAT( 269 )      ! ACTP+MO2=0.5*HO2+0.5*ACO3+...
     &                 +          RXRAT( 305 )      ! ACTP+ACO3=0.5*MO2+0.5*ACO3+...
     &                 +          RXRAT( 343 )      ! ACTP+NO3=ACO3+NO2+HCHO

      PROD( MEKP    ) =           RXRAT(  93 )      ! MEK+HO=MEKP
      LOSS( MEKP    ) =           RXRAT( 194 )      ! MEKP+NO=0.67*HO2+NO2+...
     &                 +          RXRAT( 234 )      ! MEKP+HO2=OP2
     &                 +          RXRAT( 270 )      ! MEKP+MO2=0.834*HO2+HCHO+...
     &                 +          RXRAT( 306 )      ! MEKP+ACO3=0.33*HO2+0.5*MO2+...
     &                 +          RXRAT( 344 )      ! MEKP+NO3=0.67*HO2+NO2+...

      PROD( KETP    ) =           RXRAT(  94 )      ! KET+HO=KETP
     &                 +  0.020 * RXRAT( 129 )      ! DIEN+O3=0.02*KETP+0.28*HO+...
     &                 +  0.420 * RXRAT( 131 )      ! API+O3=0.42*KETP+0.1*HO2+...
     &                 +  0.420 * RXRAT( 132 )      ! LIM+O3=0.42*KETP+0.1*HO2+...
      LOSS( KETP    ) =           RXRAT( 195 )      ! KETP+NO=0.77*HO2+0.23*ACO3+...
     &                 +          RXRAT( 235 )      ! KETP+HO2=OP2
     &                 +          RXRAT( 271 )      ! KETP+MO2=HO2+0.75*HCHO+...
     &                 +          RXRAT( 307 )      ! KETP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 345 )      ! KETP+NO3=HO2+NO2+DCB1

      PROD( MCP     ) =   0.430 * RXRAT(  96 )      ! MACR+HO=0.43*MCP+0.57*MACP
      LOSS( MCP     ) =           RXRAT( 197 )      ! MCP+NO=NO2+0.5*HO2+...
     &                 +          RXRAT( 237 )      ! MCP+HO2=MAHP
     &                 +          RXRAT( 273 )      ! MCP+MO2=NO2+HO2+1.5*HCHO+...
     &                 +          RXRAT( 309 )      ! MCP+ACO3=NO2+0.5*HO2+HCHO+...
     &                 +          RXRAT( 347 )      ! MCP+NO3=NO2+HO2+HCHO+HKET

      PROD( MVKP    ) =           RXRAT(  97 )      ! MVK+HO=MVKP
      LOSS( MVKP    ) =           RXRAT( 198 )      ! MVKP+NO=0.3*HO2+0.7*ACO3+...
     &                 +          RXRAT( 238 )      ! MVKP+HO2=OP2
     &                 +          RXRAT( 274 )      ! MVKP+MO2=HO2+1.16*ACO3+...
     &                 +          RXRAT( 310 )      ! MVKP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 348 )      ! MVKP+NO3=0.3*HO2+0.7*ACO3+...

      PROD( UALP    ) =   0.687 * RXRAT(  98 )      ! UALD+HO=0.687*UALP+0.313*ACO3
      LOSS( UALP    ) =           RXRAT( 199 )      ! UALP+NO=HO2+NO2+0.61*CO+...
     &                 +          RXRAT( 239 )      ! UALP+HO2=OP2
     &                 +          RXRAT( 275 )      ! UALP+MO2=HO2+0.305*CO+...
     &                 +          RXRAT( 311 )      ! UALP+ACO3=0.5*HO2+0.5*MO2+...
     &                 +          RXRAT( 349 )      ! UALP+NO3=HO2+NO2+0.61*CO+...

      PROD( DCB3    ) =   0.459 * RXRAT( 180 )      ! BENP+NO=0.459*DCB3+...
     &                 +  0.950 * RXRAT( 185 )      ! XYLP+NO=0.95*DCB3+0.95*NO2+...
     &                 +  1.050 * RXRAT( 186 )      ! PER2+NO=1.05*DCB3+0.95*NO2+...
     &                 +  0.177 * RXRAT( 187 )      ! XYOP+NO=0.177*DCB3+...
     &                 +  0.459 * RXRAT( 256 )      ! BENP+MO2=0.459*DCB3+...
     &                 +  1.050 * RXRAT( 262 )      ! PER2+MO2=1.05*DCB3+HCHO+...
     &                 +  0.186 * RXRAT( 263 )      ! XYOP+MO2=0.186*DCB3+HCHO+...
     &                 +  0.458 * RXRAT( 292 )      ! BENP+ACO3=0.458*DCB3+MO2+...
     &                 +  1.050 * RXRAT( 298 )      ! PER2+ACO3=1.05*DCB3+MO2+...
     &                 +  0.186 * RXRAT( 299 )      ! XYOP+ACO3=0.186*DCB3+MO2+...
     &                 +  0.500 * RXRAT( 330 )      ! BENP+NO3=0.5*DCB3+NO2+...
     &                 +          RXRAT( 335 )      ! XYLP+NO3=DCB3+NO2+HO2
     &                 +  1.050 * RXRAT( 336 )      ! PER2+NO3=1.05*DCB3+NO2+...
     &                 +  0.186 * RXRAT( 337 )      ! XYOP+NO3=0.186*DCB3+NO2+...
      LOSS( DCB3    ) =           RXRAT( 103 )      ! DCB3+HO=0.56*HO2+0.21*MACP+...
     &                 +          RXRAT( 138 )      ! DCB3+O3=0.05*HO+HO2+1.5*CO+...

      PROD( BALP    ) =           RXRAT( 104 )      ! BALD+HO=BALP
      LOSS( BALP    ) =           RXRAT( 200 )      ! BALP+NO=BAL1+NO2
     &                 +          RXRAT( 276 )      ! BALP+MO2=HO2+BAL1+HCHO
     &                 +          RXRAT( 312 )      ! BALP+ACO3=MO2+BAL1
     &                 +          RXRAT( 350 )      ! BALP+NO3=BAL1+NO2

      PROD( ADDC    ) =   0.200 * RXRAT( 105 )      ! PHEN+HO=0.2*ADDC+0.73*HO2+...
     &                 +  0.200 * RXRAT( 106 )      ! CSL+HO=0.2*ADDC+0.73*HO2+...
     &                 +  0.100 * RXRAT( 155 )      ! PHEN+NO3=0.1*ADDC+0.4*CHO+...
     &                 +  0.100 * RXRAT( 156 )      ! CSL+NO3=0.1*ADDC+0.4*CHO+...
      LOSS( ADDC    ) =           RXRAT( 202 )      ! ADDC+NO=HO2+NO2+0.32*HKET+...
     &                 +          RXRAT( 240 )      ! ADDC+HO2=OP2
     &                 +          RXRAT( 278 )      ! ADDC+MO2=2*HO2+HCHO+...
     &                 +          RXRAT( 314 )      ! ADDC+ACO3=2*HO2+MO2+...
     &                 +          RXRAT( 352 )      ! ADDC+NO3=HO2+NO2+0.32*HKET+...

      PROD( MCT     ) =   0.730 * RXRAT( 105 )      ! PHEN+HO=0.73*MCT+0.2*ADDC+...
     &                 +  0.730 * RXRAT( 106 )      ! CSL+HO=0.73*MCT+0.2*ADDC+...
      LOSS( MCT     ) =           RXRAT( 108 )      ! MCT+HO=MCTO
     &                 +          RXRAT( 158 )      ! MCT+NO3=MCTO+HNO3

      PROD( MCTO    ) =           RXRAT( 108 )      ! MCT+HO=MCTO
     &                 +          RXRAT( 158 )      ! MCT+NO3=MCTO+HNO3
     &                 +          RXRAT( 203 )      ! MCTP+NO=MCTO+NO2
     &                 +          RXRAT( 279 )      ! MCTP+MO2=MCTO+HO2+HCHO
     &                 +          RXRAT( 315 )      ! MCTP+ACO3=MCTO+MO2+HO2
     &                 +          RXRAT( 353 )      ! MCTP+NO3=MCTO+NO2
      LOSS( MCTO    ) =           RXRAT( 140 )      ! MCTO+O3=MCTP
     &                 +          RXRAT( 211 )      ! MCTO+NO2=ONIT

      PROD( MOH     ) =   0.630 * RXRAT( 248 )      ! MO2+MO2=0.63*MOH+1.37*HCHO+...
     &                 +  0.250 * RXRAT( 249 )      ! ETHP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 250 )      ! HC3P+MO2=0.25*MOH+0.08*MO2+...
     &                 +  0.250 * RXRAT( 251 )      ! HC5P+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 252 )      ! HC8P+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 253 )      ! ETEP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 254 )      ! OLTP+MO2=0.25*MOH+1.5*HCHO+...
     &                 +  0.250 * RXRAT( 255 )      ! OLIP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 264 )      ! ISOP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 265 )      ! APIP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 266 )      ! LIMP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 269 )      ! ACTP+MO2=0.25*MOH+0.5*ACO3+...
     &                 +  0.250 * RXRAT( 270 )      ! MEKP+MO2=0.25*MOH+HCHO+...
     &                 +  0.250 * RXRAT( 271 )      ! KETP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 272 )      ! MACP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 273 )      ! MCP+MO2=0.25*MOH+HO2+...
     &                 +  0.250 * RXRAT( 274 )      ! MVKP+MO2=0.25*MOH+...
     &                 +  0.250 * RXRAT( 275 )      ! UALP+MO2=0.25*MOH+0.305*CO+...
     &                 +  0.250 * RXRAT( 282 )      ! OLND+MO2=0.25*MOH+0.5*NO2+...
      LOSS( MOH     ) =           RXRAT( 109 )      ! MOH+HO=HO2+HCHO

      PROD( EOH     ) =   0.250 * RXRAT( 249 )      ! ETHP+MO2=0.25*EOH+...
      LOSS( EOH     ) =           RXRAT( 110 )      ! EOH+HO=HO2+ACD

      PROD( ROH     ) =   0.250 * RXRAT( 250 )      ! HC3P+MO2=0.25*ROH+0.08*MO2+...
     &                 +  0.250 * RXRAT( 251 )      ! HC5P+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 252 )      ! HC8P+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 254 )      ! OLTP+MO2=0.25*ROH+1.5*HCHO+...
     &                 +  0.250 * RXRAT( 255 )      ! OLIP+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 264 )      ! ISOP+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 265 )      ! APIP+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 266 )      ! LIMP+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 269 )      ! ACTP+MO2=0.25*ROH+0.5*ACO3+...
     &                 +  0.250 * RXRAT( 270 )      ! MEKP+MO2=0.25*ROH+HCHO+...
     &                 +  0.250 * RXRAT( 271 )      ! KETP+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 272 )      ! MACP+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 273 )      ! MCP+MO2=0.25*ROH+HO2+...
     &                 +  0.250 * RXRAT( 274 )      ! MVKP+MO2=0.25*ROH+...
     &                 +  0.250 * RXRAT( 275 )      ! UALP+MO2=0.25*ROH+0.305*CO+...
     &                 +  0.250 * RXRAT( 282 )      ! OLND+MO2=0.25*ROH+0.5*NO2+...
      LOSS( ROH     ) =           RXRAT( 111 )      ! ROH+HO=HO2+0.719*ALD+0.184*ACD

      PROD( ETEG    ) =   0.250 * RXRAT( 253 )      ! ETEP+MO2=0.25*ETEG+...
      LOSS( ETEG    ) =           RXRAT( 112 )      ! ETEG+HO=HO2+ALD

      PROD( ISHP    ) =           RXRAT( 228 )      ! ISOP+HO2=ISHP
      LOSS( ISHP    ) =           RXRAT( 115 )      ! ISHP+HO=HO+MACR

      PROD( MAHP    ) =           RXRAT( 236 )      ! MACP+HO2=MAHP
     &                 +          RXRAT( 237 )      ! MCP+HO2=MAHP
      LOSS( MAHP    ) =           RXRAT( 116 )      ! MAHP+HO=MACP

      PROD( ORA2    ) =   0.060 * RXRAT( 127 )      ! OLT+O3=0.06*ORA2+0.32*HO2+...
     &                 +  0.017 * RXRAT( 128 )      ! OLI+O3=0.017*ORA2+0.07*HO2+...
     &                 +  0.070 * RXRAT( 132 )      ! LIM+O3=0.07*ORA2+0.1*HO2+...
     &                 +  0.070 * RXRAT( 134 )      ! MVK+O3=0.07*ORA2+0.11*HO2+...
     &                 +  0.250 * RXRAT( 138 )      ! DCB3+O3=0.25*ORA2+HO2+...
     &                 +  0.150 * RXRAT( 231 )      ! ACO3+HO2=0.15*ORA2+...
     &                 +  0.150 * RXRAT( 232 )      ! RCO3+HO2=0.15*ORA2+...
     &                 +  0.100 * RXRAT( 267 )      ! ACO3+MO2=0.1*ORA2+0.9*MO2+...
     &                 +  0.100 * RXRAT( 268 )      ! RCO3+MO2=0.1*ORA2+0.9*MO2+...
     &                 +  0.125 * RXRAT( 269 )      ! ACTP+MO2=0.125*ORA2+...
     &                 +  0.067 * RXRAT( 272 )      ! MACP+MO2=0.067*ORA2+...
     &                 +  0.292 * RXRAT( 274 )      ! MVKP+MO2=0.292*ORA2+...
     &                 +  0.500 * RXRAT( 285 )      ! ETHP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +  0.500 * RXRAT( 286 )      ! HC3P+ACO3=0.5*ORA2+...
     &                 +  0.500 * RXRAT( 287 )      ! HC5P+ACO3=0.5*ORA2+...
     &                 +  0.495 * RXRAT( 288 )      ! HC8P+ACO3=0.495*ORA2+...
     &                 +  0.500 * RXRAT( 289 )      ! ETEP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +  0.500 * RXRAT( 290 )      ! OLTP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +  0.500 * RXRAT( 291 )      ! OLIP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +  0.500 * RXRAT( 300 )      ! ISOP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +          RXRAT( 301 )      ! APIP+ACO3=ORA2+0.5*MO2+ALD+...
     &                 +  0.500 * RXRAT( 302 )      ! LIMP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +  0.750 * RXRAT( 305 )      ! ACTP+ACO3=0.75*ORA2+...
     &                 +  0.500 * RXRAT( 306 )      ! MEKP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +  0.500 * RXRAT( 307 )      ! KETP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +  0.635 * RXRAT( 308 )      ! MACP+ACO3=0.635*ORA2+...
     &                 +  0.500 * RXRAT( 309 )      ! MCP+ACO3=0.5*ORA2+0.5*HO2+...
     &                 +  1.083 * RXRAT( 310 )      ! MVKP+ACO3=1.083*ORA2+...
     &                 +  0.500 * RXRAT( 311 )      ! UALP+ACO3=0.5*ORA2+0.5*MO2+...
     &                 +  0.500 * RXRAT( 318 )      ! OLND+ACO3=0.5*ORA2+NO2+...
      LOSS( ORA2    ) =           RXRAT( 118 )      ! ORA2+HO=0.64*MO2+0.36*ORAP

      PROD( ORAP    ) =   0.360 * RXRAT( 118 )      ! ORA2+HO=0.36*ORAP+0.64*MO2
      LOSS( ORAP    ) =           RXRAT( 204 )      ! ORAP+NO=NO2+GLY+HO2
     &                 +          RXRAT( 243 )      ! ORAP+HO2=OP2
     &                 +          RXRAT( 280 )      ! ORAP+MO2=HCHO+HO2+GLY
     &                 +          RXRAT( 316 )      ! ORAP+ACO3=MO2+GLY
     &                 +          RXRAT( 354 )      ! ORAP+NO3=NO2+GLY+HO2

      PROD( PPN     ) =           RXRAT( 168 )      ! RCO3+NO2=PPN
      LOSS( PPN     ) =           RXRAT( 121 )      ! PPN+HO=XO2+NO3+HCHO
     &                 +          RXRAT( 169 )      ! PPN=RCO3+NO2

      PROD( MPAN    ) =           RXRAT( 170 )      ! MACP+NO2=MPAN
      LOSS( MPAN    ) =           RXRAT( 122 )      ! MPAN+HO=NO2+HKET
     &                 +          RXRAT( 159 )      ! MPAN+NO3=MACP+NO2
     &                 +          RXRAT( 171 )      ! MPAN=MACP+NO2

      PROD( NALD    ) =           RXRAT( 125 )      ! ISON+HO=NALD+0.07*HKET+...
      LOSS( NALD    ) =           RXRAT( 124 )      ! NALD+HO=NO2+XO2+HKET

      PROD( ISON    ) =           RXRAT( 145 )      ! ISO+NO3=ISON
     &                 +  0.120 * RXRAT( 188 )      ! ISOP+NO=0.12*ISON+0.88*NO2+...
      LOSS( ISON    ) =           RXRAT( 125 )      ! ISON+HO=NALD+0.07*HKET+...

      PROD( MCTP    ) =           RXRAT( 140 )      ! MCTO+O3=MCTP
      LOSS( MCTP    ) =           RXRAT( 203 )      ! MCTP+NO=MCTO+NO2
     &                 +          RXRAT( 242 )      ! MCTP+HO2=OP2
     &                 +          RXRAT( 279 )      ! MCTP+MO2=HO2+MCTO+HCHO
     &                 +          RXRAT( 315 )      ! MCTP+ACO3=HO2+MO2+MCTO
     &                 +          RXRAT( 353 )      ! MCTP+NO3=NO2+MCTO

      PROD( OLNN    ) =   0.800 * RXRAT( 141 )      ! ETE+NO3=0.8*OLNN+0.2*OLND
     &                 +  0.430 * RXRAT( 142 )      ! OLT+NO3=0.43*OLNN+0.57*OLND
     &                 +  0.110 * RXRAT( 143 )      ! OLI+NO3=0.11*OLNN+0.89*OLND
     &                 +  0.900 * RXRAT( 144 )      ! DIEN+NO3=0.9*OLNN+0.1*OLND+...
     &                 +  0.100 * RXRAT( 146 )      ! API+NO3=0.1*OLNN+0.9*OLND+...
     &                 +  0.710 * RXRAT( 147 )      ! LIM+NO3=0.71*OLNN+...
      LOSS( OLNN    ) =           RXRAT( 205 )      ! OLNN+NO=NO2+HO2+ONIT
     &                 +          RXRAT( 244 )      ! OLNN+HO2=ONIT
     &                 +          RXRAT( 281 )      ! OLNN+MO2=2*HO2+HCHO+ONIT
     &                 +          RXRAT( 317 )      ! OLNN+ACO3=HO2+MO2+ONIT
     &                 +          RXRAT( 355 )      ! OLNN+NO3=HO2+NO2+ONIT
     &                 +  2.000 * RXRAT( 358 )      ! OLNN+OLNN=HO2+2*ONIT
     &                 +          RXRAT( 359 )      ! OLNN+OLND=0.5*HO2+0.5*NO2+...

      PROD( OLND    ) =   0.200 * RXRAT( 141 )      ! ETE+NO3=0.2*OLND+0.8*OLNN
     &                 +  0.570 * RXRAT( 142 )      ! OLT+NO3=0.57*OLND+0.43*OLNN
     &                 +  0.890 * RXRAT( 143 )      ! OLI+NO3=0.89*OLND+0.11*OLNN
     &                 +  0.100 * RXRAT( 144 )      ! DIEN+NO3=0.1*OLND+0.9*OLNN+...
     &                 +  0.900 * RXRAT( 146 )      ! API+NO3=0.9*OLND+0.1*OLNN+...
     &                 +  0.290 * RXRAT( 147 )      ! LIM+NO3=0.29*OLND+...
      LOSS( OLND    ) =           RXRAT( 206 )      ! OLND+NO=2*NO2+0.287*HCHO+...
     &                 +          RXRAT( 245 )      ! OLND+HO2=ONIT
     &                 +          RXRAT( 282 )      ! OLND+MO2=0.5*HO2+0.5*NO2+...
     &                 +          RXRAT( 318 )      ! OLND+ACO3=0.5*MO2+NO2+...
     &                 +          RXRAT( 356 )      ! OLND+NO3=2*NO2+0.287*HCHO+...
     &                 +          RXRAT( 359 )      ! OLND+OLNN=0.5*HO2+0.5*NO2+...
     &                 +  2.000 * RXRAT( 360 )      ! OLND+OLND=NO2+0.504*HCHO+...

      PROD( ADCN    ) =   0.500 * RXRAT( 155 )      ! PHEN+NO3=0.5*ADCN+0.1*ADDC+...
     &                 +  0.500 * RXRAT( 156 )      ! CSL+NO3=0.5*ADCN+0.1*ADDC+...
      LOSS( ADCN    ) =           RXRAT( 207 )      ! ADCN+NO=2*NO2+GLY+OP2
     &                 +          RXRAT( 246 )      ! ADCN+HO2=OP2
     &                 +          RXRAT( 283 )      ! ADCN+MO2=HO2+0.7*NO2+HCHO+...
     &                 +          RXRAT( 319 )      ! ADCN+ACO3=HO2+MO2+0.7*NO2+...
     &                 +          RXRAT( 357 )      ! ADCN+NO3=2*NO2+GLY+OP2

      PROD( TOLP    ) =   0.280 * RXRAT( 160 )      ! TR2=0.28*TOLP+0.29*HO2+...
      LOSS( TOLP    ) =           RXRAT( 161 )      ! TOLP=0.49*HO+0.01*HO2+...
     &                 +          RXRAT( 182 )      ! TOLP+NO=0.95*HO2+0.95*NO2+...
     &                 +          RXRAT( 222 )      ! TOLP+HO2=OP2
     &                 +          RXRAT( 258 )      ! TOLP+MO2=2*HO2+HCHO+...
     &                 +          RXRAT( 294 )      ! TOLP+ACO3=HO2+MO2+DCB2
     &                 +          RXRAT( 332 )      ! TOLP+NO3=HO2+NO2+DCB2

      PROD( PER1    ) =   0.150 * RXRAT( 160 )      ! TR2=0.15*PER1+0.29*HO2+...
     &                 +  0.500 * RXRAT( 161 )      ! TOLP=0.5*PER1+0.01*HO2+...
      LOSS( PER1    ) =           RXRAT( 183 )      ! PER1+NO=0.5*HO2+0.95*NO2+...
     &                 +          RXRAT( 223 )      ! PER1+HO2=OP2
     &                 +          RXRAT( 259 )      ! PER1+MO2=2*HO2+HCHO+MGLY+DCB1
     &                 +          RXRAT( 295 )      ! PER1+ACO3=HO2+MO2+MGLY+DCB1
     &                 +          RXRAT( 333 )      ! PER1+NO3=0.5*HO2+NO2+...

      PROD( XYLP    ) =   0.308 * RXRAT( 162 )      ! XY2=0.308*XYLP+0.308*HO2+...
      LOSS( XYLP    ) =           RXRAT( 163 )      ! XYLP=0.39*HO+0.01*HO2+...
     &                 +          RXRAT( 185 )      ! XYLP+NO=0.95*HO2+0.95*NO2+...
     &                 +          RXRAT( 225 )      ! XYLP+HO2=OP2
     &                 +          RXRAT( 261 )      ! XYLP+MO2=2*HO2+HCHO+DCB2
     &                 +          RXRAT( 297 )      ! XYLP+ACO3=HO2+MO2+DCB2
     &                 +          RXRAT( 335 )      ! XYLP+NO3=HO2+NO2+DCB3

      PROD( PER2    ) =   0.150 * RXRAT( 162 )      ! XY2=0.15*PER2+0.308*HO2+...
     &                 +  0.300 * RXRAT( 163 )      ! XYLP=0.3*PER2+0.01*HO2+...
     &                 +  0.150 * RXRAT( 164 )      ! XYO2=0.15*PER2+0.308*HO2+...
     &                 +  0.500 * RXRAT( 165 )      ! XYOP=0.5*PER2+0.01*HO2+...
      LOSS( PER2    ) =           RXRAT( 186 )      ! PER2+NO=0.95*HO2+0.95*NO2+...
     &                 +          RXRAT( 226 )      ! PER2+HO2=OP2
     &                 +          RXRAT( 262 )      ! PER2+MO2=2*HO2+HCHO+MGLY+...
     &                 +          RXRAT( 298 )      ! PER2+ACO3=HO2+MO2+MGLY+...
     &                 +          RXRAT( 336 )      ! PER2+NO3=HO2+NO2+MGLY+DCB1+...

      PROD( XYOP    ) =   0.308 * RXRAT( 164 )      ! XYO2=0.308*XYOP+0.308*HO2+...
      LOSS( XYOP    ) =           RXRAT( 165 )      ! XYOP=0.39*HO+0.01*HO2+...
     &                 +          RXRAT( 187 )      ! XYOP+NO=0.95*HO2+0.95*NO2+...
     &                 +          RXRAT( 227 )      ! XYOP+HO2=OP2
     &                 +          RXRAT( 263 )      ! XYOP+MO2=2*HO2+HCHO+...
     &                 +          RXRAT( 299 )      ! XYOP+ACO3=HO2+MO2+...
     &                 +          RXRAT( 337 )      ! XYOP+NO3=HO2+NO2+0.368*GLY+...

      PROD( BAL1    ) =           RXRAT( 200 )      ! BALP+NO=BAL1+NO2
     &                 +          RXRAT( 276 )      ! BALP+MO2=BAL1+HO2+HCHO
     &                 +          RXRAT( 312 )      ! BALP+ACO3=BAL1+MO2
     &                 +          RXRAT( 350 )      ! BALP+NO3=BAL1+NO2
      LOSS( BAL1    ) =           RXRAT( 201 )      ! BAL1+NO=BAL2+NO2
     &                 +          RXRAT( 277 )      ! BAL1+MO2=HO2+BAL2+HCHO
     &                 +          RXRAT( 313 )      ! BAL1+ACO3=MO2+BAL2
     &                 +          RXRAT( 351 )      ! BAL1+NO3=BAL2+NO2

      PROD( BAL2    ) =           RXRAT( 201 )      ! BAL1+NO=BAL2+NO2
     &                 +          RXRAT( 277 )      ! BAL1+MO2=BAL2+HO2+HCHO
     &                 +          RXRAT( 313 )      ! BAL1+ACO3=BAL2+MO2
     &                 +          RXRAT( 351 )      ! BAL1+NO3=BAL2+NO2
      LOSS( BAL2    ) =           RXRAT( 209 )      ! BAL2+NO2=ONIT

      PROD( TOLNRXN ) =           RXRAT( 364 )      ! TOLRO2+NO=TOLNRXN+NO
      LOSS( TOLNRXN ) =   0.0

      PROD( TOLHRXN ) =           RXRAT( 365 )      ! TOLRO2+HO2=TOLHRXN+HO2
      LOSS( TOLHRXN ) =   0.0

      PROD( XYLNRXN ) =           RXRAT( 366 )      ! XYLRO2+NO=XYLNRXN+NO
      LOSS( XYLNRXN ) =   0.0

      PROD( XYLHRXN ) =           RXRAT( 367 )      ! XYLRO2+HO2=XYLHRXN+HO2
      LOSS( XYLHRXN ) =   0.0

      PROD( BNZNRXN ) =           RXRAT( 368 )      ! BENZRO2+NO=BNZNRXN+NO
      LOSS( BNZNRXN ) =   0.0

      PROD( BNZHRXN ) =           RXRAT( 369 )      ! BENZRO2+HO2=BNZHRXN+HO2
      LOSS( BNZHRXN ) =   0.0

      PROD( SESQ    ) =   0.0
      LOSS( SESQ    ) =           RXRAT( 370 )      ! SESQ+O3=O3+SESQRXN
     &                 +          RXRAT( 371 )      ! SESQ+HO=HO+SESQRXN
     &                 +          RXRAT( 372 )      ! SESQ+NO3=NO3+SESQRXN

      PROD( SESQRXN ) =           RXRAT( 370 )      ! SESQ+O3=SESQRXN+O3
     &                 +          RXRAT( 371 )      ! SESQ+HO=SESQRXN+HO
     &                 +          RXRAT( 372 )      ! SESQ+NO3=SESQRXN+NO3
      LOSS( SESQRXN ) =   0.0

      RETURN

      END
