
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header$

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

      SUBROUTINE HRRATES

C***********************************************************************
C
C  FUNCTION: To calculate rates of reactions
C
C  PRECONDITIONS: For the SAPRC07TC family of mechanisms
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, June 1, 2012
C
C***********************************************************************
      USE HRDATA

      IMPLICIT NONE

C.....INCLUDES: NONE


C.....ARGUMENTS: NONE


C.....PARAMETERS: NONE


C.....EXTERNAL FUNCTIONS: NONE


C.....SAVED VARIABLES: NONE
      CHARACTER( 16 ), SAVE  ::  PNAME = 'HRRATES'   ! Program name


C.....LOCAL VARIABLES: None

C***********************************************************************
C..NOTE: RXRAT commented out are not referenced by any routine
!     RXRAT(   1 ) = RKI(   1 ) * YC( NO2 )
!     RXRAT(   2 ) = RKI(   2 ) * YC( O3P )
!     RXRAT(   3 ) = RKI(   3 ) * YC( O3P ) * YC( O3 )
!     RXRAT(   4 ) = RKI(   4 ) * YC( O3P ) * YC( NO )
!     RXRAT(   5 ) = RKI(   5 ) * YC( O3P ) * YC( NO2 )
      RXRAT(   6 ) = RKI(   6 ) * YC( O3P ) * YC( NO2 )
!     RXRAT(   7 ) = RKI(   7 ) * YC( O3 ) * YC( NO )
      RXRAT(   8 ) = RKI(   8 ) * YC( O3 ) * YC( NO2 )
      RXRAT(   9 ) = RKI(   9 ) * YC( NO ) * YC( NO3 )
!     RXRAT(  10 ) = RKI(  10 ) * YC( NO ) * YC( NO )
!     RXRAT(  11 ) = RKI(  11 ) * YC( NO2 ) * YC( NO3 )
      RXRAT(  12 ) = RKI(  12 ) * YC( N2O5 )
      RXRAT(  13 ) = RKI(  13 ) * YC( N2O5 )
      RXRAT(  14 ) = RKI(  14 ) * YC( N2O5 )
      RXRAT(  15 ) = RKI(  15 ) * YC( NO2 ) * YC( NO3 )
      RXRAT(  16 ) = RKI(  16 ) * YC( NO3 )
      RXRAT(  17 ) = RKI(  17 ) * YC( NO3 )
      RXRAT(  18 ) = RKI(  18 ) * YC( O3 )
      RXRAT(  19 ) = RKI(  19 ) * YC( O3 )
!     RXRAT(  20 ) = RKI(  20 ) * YC( O1D )
!     RXRAT(  21 ) = RKI(  21 ) * YC( O1D )
!     RXRAT(  22 ) = RKI(  22 ) * YC( OH ) * YC( NO )
      RXRAT(  23 ) = RKI(  23 ) * YC( HONO )
      RXRAT(  24 ) = RKI(  24 ) * YC( OH ) * YC( HONO )
      RXRAT(  25 ) = RKI(  25 ) * YC( OH ) * YC( NO2 )
      RXRAT(  26 ) = RKI(  26 ) * YC( OH ) * YC( NO3 )
      RXRAT(  27 ) = RKI(  27 ) * YC( OH ) * YC( HNO3 )
      RXRAT(  28 ) = RKI(  28 ) * YC( HNO3 )
      RXRAT(  29 ) = RKI(  29 ) * YC( OH ) * YC( CO )
!     RXRAT(  30 ) = RKI(  30 ) * YC( OH ) * YC( O3 )
!     RXRAT(  31 ) = RKI(  31 ) * YC( HO2 ) * YC( NO )
!     RXRAT(  32 ) = RKI(  32 ) * YC( HO2 ) * YC( NO2 )
      RXRAT(  33 ) = RKI(  33 ) * YC( HNO4 )
      RXRAT(  34 ) = RKI(  34 ) * YC( HNO4 )
      RXRAT(  35 ) = RKI(  35 ) * YC( HNO4 ) * YC( OH )
!     RXRAT(  36 ) = RKI(  36 ) * YC( HO2 ) * YC( O3 )
      RXRAT(  37 ) = RKI(  37 ) * YC( HO2 ) * YC( HO2 )
      RXRAT(  38 ) = RKI(  38 ) * YC( HO2 ) * YC( HO2 )
      RXRAT(  39 ) = RKI(  39 ) * YC( NO3 ) * YC( HO2 )
      RXRAT(  40 ) = RKI(  40 ) * YC( NO3 ) * YC( NO3 )
      RXRAT(  41 ) = RKI(  41 ) * YC( HO2H )
      RXRAT(  42 ) = RKI(  42 ) * YC( HO2H ) * YC( OH )
!     RXRAT(  43 ) = RKI(  43 ) * YC( OH ) * YC( HO2 )
      RXRAT(  44 ) = RKI(  44 ) * YC( OH ) * YC( SO2 )
!     RXRAT(  45 ) = RKI(  45 ) * YC( OH )
      RXRAT(  46 ) = RKI(  46 ) * YC( NO2 )
      RXRAT(  47 ) = RKI(  47 ) * YC( NO2EX )
      RXRAT(  48 ) = RKI(  48 ) * YC( NO2EX )
      RXRAT(  49 ) = RKI(  49 ) * YC( NO2EX )
      RXRAT(  50 ) = RKI(  50 ) * YC( MEO2 ) * YC( NO )
      RXRAT(  51 ) = RKI(  51 ) * YC( MEO2 ) * YC( HO2 )
      RXRAT(  52 ) = RKI(  52 ) * YC( MEO2 ) * YC( HO2 )
      RXRAT(  53 ) = RKI(  53 ) * YC( MEO2 ) * YC( NO3 )
      RXRAT(  54 ) = RKI(  54 ) * YC( MEO2 ) * YC( MEO2 )
      RXRAT(  55 ) = RKI(  55 ) * YC( MEO2 ) * YC( MEO2 )
      RXRAT(  56 ) = RKI(  56 ) * YC( RO2C ) * YC( NO )
      RXRAT(  57 ) = RKI(  57 ) * YC( RO2C ) * YC( HO2 )
      RXRAT(  58 ) = RKI(  58 ) * YC( RO2C ) * YC( NO3 )
      RXRAT(  59 ) = RKI(  59 ) * YC( RO2C ) * YC( MEO2 )
      RXRAT(  60 ) = RKI(  60 ) * YC( RO2C ) * YC( RO2C )
      RXRAT(  61 ) = RKI(  61 ) * YC( RO2XC ) * YC( NO )
      RXRAT(  62 ) = RKI(  62 ) * YC( RO2XC ) * YC( HO2 )
      RXRAT(  63 ) = RKI(  63 ) * YC( RO2XC ) * YC( NO3 )
      RXRAT(  64 ) = RKI(  64 ) * YC( RO2XC ) * YC( MEO2 )
      RXRAT(  65 ) = RKI(  65 ) * YC( RO2XC ) * YC( RO2C )
      RXRAT(  66 ) = RKI(  66 ) * YC( RO2XC ) * YC( RO2XC )
!     RXRAT(  67 ) = RKI(  67 ) * YC( MECO3 ) * YC( NO2 )
      RXRAT(  68 ) = RKI(  68 ) * YC( PAN )
      RXRAT(  69 ) = RKI(  69 ) * YC( PAN )
      RXRAT(  70 ) = RKI(  70 ) * YC( MECO3 ) * YC( NO )
      RXRAT(  71 ) = RKI(  71 ) * YC( MECO3 ) * YC( HO2 )
      RXRAT(  72 ) = RKI(  72 ) * YC( MECO3 ) * YC( NO3 )
      RXRAT(  73 ) = RKI(  73 ) * YC( MECO3 ) * YC( MEO2 )
      RXRAT(  74 ) = RKI(  74 ) * YC( MECO3 ) * YC( RO2C )
      RXRAT(  75 ) = RKI(  75 ) * YC( MECO3 ) * YC( RO2XC )
      RXRAT(  76 ) = RKI(  76 ) * YC( MECO3 ) * YC( MECO3 )
      RXRAT(  77 ) = RKI(  77 ) * YC( RCO3 ) * YC( NO2 )
      RXRAT(  78 ) = RKI(  78 ) * YC( PAN2 )
      RXRAT(  79 ) = RKI(  79 ) * YC( PAN2 )
      RXRAT(  80 ) = RKI(  80 ) * YC( RCO3 ) * YC( NO )
      RXRAT(  81 ) = RKI(  81 ) * YC( RCO3 ) * YC( HO2 )
      RXRAT(  82 ) = RKI(  82 ) * YC( RCO3 ) * YC( NO3 )
      RXRAT(  83 ) = RKI(  83 ) * YC( RCO3 ) * YC( MEO2 )
      RXRAT(  84 ) = RKI(  84 ) * YC( RCO3 ) * YC( RO2C )
      RXRAT(  85 ) = RKI(  85 ) * YC( RCO3 ) * YC( RO2XC )
      RXRAT(  86 ) = RKI(  86 ) * YC( RCO3 ) * YC( MECO3 )
      RXRAT(  87 ) = RKI(  87 ) * YC( RCO3 ) * YC( RCO3 )
      RXRAT(  88 ) = RKI(  88 ) * YC( BZCO3 ) * YC( NO2 )
      RXRAT(  89 ) = RKI(  89 ) * YC( PBZN )
      RXRAT(  90 ) = RKI(  90 ) * YC( PBZN )
      RXRAT(  91 ) = RKI(  91 ) * YC( BZCO3 ) * YC( NO )
      RXRAT(  92 ) = RKI(  92 ) * YC( BZCO3 ) * YC( HO2 )
      RXRAT(  93 ) = RKI(  93 ) * YC( BZCO3 ) * YC( NO3 )
      RXRAT(  94 ) = RKI(  94 ) * YC( BZCO3 ) * YC( MEO2 )
      RXRAT(  95 ) = RKI(  95 ) * YC( BZCO3 ) * YC( RO2C )
      RXRAT(  96 ) = RKI(  96 ) * YC( BZCO3 ) * YC( RO2XC )
      RXRAT(  97 ) = RKI(  97 ) * YC( BZCO3 ) * YC( MECO3 )
      RXRAT(  98 ) = RKI(  98 ) * YC( BZCO3 ) * YC( RCO3 )
      RXRAT(  99 ) = RKI(  99 ) * YC( BZCO3 ) * YC( BZCO3 )
      RXRAT( 100 ) = RKI( 100 ) * YC( MACO3 ) * YC( NO2 )
      RXRAT( 101 ) = RKI( 101 ) * YC( MAPAN )
      RXRAT( 102 ) = RKI( 102 ) * YC( MAPAN )
      RXRAT( 103 ) = RKI( 103 ) * YC( MACO3 ) * YC( NO )
      RXRAT( 104 ) = RKI( 104 ) * YC( MACO3 ) * YC( HO2 )
      RXRAT( 105 ) = RKI( 105 ) * YC( MACO3 ) * YC( NO3 )
      RXRAT( 106 ) = RKI( 106 ) * YC( MACO3 ) * YC( MEO2 )
      RXRAT( 107 ) = RKI( 107 ) * YC( MACO3 ) * YC( RO2C )
      RXRAT( 108 ) = RKI( 108 ) * YC( MACO3 ) * YC( RO2XC )
      RXRAT( 109 ) = RKI( 109 ) * YC( MACO3 ) * YC( MECO3 )
      RXRAT( 110 ) = RKI( 110 ) * YC( MACO3 ) * YC( RCO3 )
      RXRAT( 111 ) = RKI( 111 ) * YC( MACO3 ) * YC( BZCO3 )
      RXRAT( 112 ) = RKI( 112 ) * YC( MACO3 ) * YC( MACO3 )
      RXRAT( 113 ) = RKI( 113 ) * YC( TBUO ) * YC( NO2 )
      RXRAT( 114 ) = RKI( 114 ) * YC( TBUO )
      RXRAT( 115 ) = RKI( 115 ) * YC( BZO ) * YC( NO2 )
      RXRAT( 116 ) = RKI( 116 ) * YC( BZO ) * YC( HO2 )
      RXRAT( 117 ) = RKI( 117 ) * YC( BZO )
      RXRAT( 118 ) = RKI( 118 ) * YC( xHO2 ) * YC( NO )
      RXRAT( 119 ) = RKI( 119 ) * YC( xHO2 ) * YC( HO2 )
      RXRAT( 120 ) = RKI( 120 ) * YC( xHO2 ) * YC( NO3 )
      RXRAT( 121 ) = RKI( 121 ) * YC( xHO2 ) * YC( MEO2 )
      RXRAT( 122 ) = RKI( 122 ) * YC( xHO2 ) * YC( RO2C )
      RXRAT( 123 ) = RKI( 123 ) * YC( xHO2 ) * YC( RO2XC )
      RXRAT( 124 ) = RKI( 124 ) * YC( xHO2 ) * YC( MECO3 )
      RXRAT( 125 ) = RKI( 125 ) * YC( xHO2 ) * YC( RCO3 )
      RXRAT( 126 ) = RKI( 126 ) * YC( xHO2 ) * YC( BZCO3 )
      RXRAT( 127 ) = RKI( 127 ) * YC( xHO2 ) * YC( MACO3 )
      RXRAT( 128 ) = RKI( 128 ) * YC( xOH ) * YC( NO )
      RXRAT( 129 ) = RKI( 129 ) * YC( xOH ) * YC( HO2 )
      RXRAT( 130 ) = RKI( 130 ) * YC( xOH ) * YC( NO3 )
      RXRAT( 131 ) = RKI( 131 ) * YC( xOH ) * YC( MEO2 )
      RXRAT( 132 ) = RKI( 132 ) * YC( xOH ) * YC( RO2C )
      RXRAT( 133 ) = RKI( 133 ) * YC( xOH ) * YC( RO2XC )
      RXRAT( 134 ) = RKI( 134 ) * YC( xOH ) * YC( MECO3 )
      RXRAT( 135 ) = RKI( 135 ) * YC( xOH ) * YC( RCO3 )
      RXRAT( 136 ) = RKI( 136 ) * YC( xOH ) * YC( BZCO3 )
      RXRAT( 137 ) = RKI( 137 ) * YC( xOH ) * YC( MACO3 )
      RXRAT( 138 ) = RKI( 138 ) * YC( xNO2 ) * YC( NO )
      RXRAT( 139 ) = RKI( 139 ) * YC( xNO2 ) * YC( HO2 )
      RXRAT( 140 ) = RKI( 140 ) * YC( xNO2 ) * YC( NO3 )
      RXRAT( 141 ) = RKI( 141 ) * YC( xNO2 ) * YC( MEO2 )
      RXRAT( 142 ) = RKI( 142 ) * YC( xNO2 ) * YC( RO2C )
      RXRAT( 143 ) = RKI( 143 ) * YC( xNO2 ) * YC( RO2XC )
      RXRAT( 144 ) = RKI( 144 ) * YC( xNO2 ) * YC( MECO3 )
      RXRAT( 145 ) = RKI( 145 ) * YC( xNO2 ) * YC( RCO3 )
      RXRAT( 146 ) = RKI( 146 ) * YC( xNO2 ) * YC( BZCO3 )
      RXRAT( 147 ) = RKI( 147 ) * YC( xNO2 ) * YC( MACO3 )
      RXRAT( 148 ) = RKI( 148 ) * YC( xMEO2 ) * YC( NO )
      RXRAT( 149 ) = RKI( 149 ) * YC( xMEO2 ) * YC( HO2 )
      RXRAT( 150 ) = RKI( 150 ) * YC( xMEO2 ) * YC( NO3 )
      RXRAT( 151 ) = RKI( 151 ) * YC( xMEO2 ) * YC( MEO2 )
      RXRAT( 152 ) = RKI( 152 ) * YC( xMEO2 ) * YC( RO2C )
      RXRAT( 153 ) = RKI( 153 ) * YC( xMEO2 ) * YC( RO2XC )
      RXRAT( 154 ) = RKI( 154 ) * YC( xMEO2 ) * YC( MECO3 )
      RXRAT( 155 ) = RKI( 155 ) * YC( xMEO2 ) * YC( RCO3 )
      RXRAT( 156 ) = RKI( 156 ) * YC( xMEO2 ) * YC( BZCO3 )
      RXRAT( 157 ) = RKI( 157 ) * YC( xMEO2 ) * YC( MACO3 )
      RXRAT( 158 ) = RKI( 158 ) * YC( xMECO3 ) * YC( NO )
      RXRAT( 159 ) = RKI( 159 ) * YC( xMECO3 ) * YC( HO2 )
      RXRAT( 160 ) = RKI( 160 ) * YC( xMECO3 ) * YC( NO3 )
      RXRAT( 161 ) = RKI( 161 ) * YC( xMECO3 ) * YC( MEO2 )
      RXRAT( 162 ) = RKI( 162 ) * YC( xMECO3 ) * YC( RO2C )
      RXRAT( 163 ) = RKI( 163 ) * YC( xMECO3 ) * YC( RO2XC )
      RXRAT( 164 ) = RKI( 164 ) * YC( xMECO3 ) * YC( MECO3 )
      RXRAT( 165 ) = RKI( 165 ) * YC( xMECO3 ) * YC( RCO3 )
      RXRAT( 166 ) = RKI( 166 ) * YC( xMECO3 ) * YC( BZCO3 )
      RXRAT( 167 ) = RKI( 167 ) * YC( xMECO3 ) * YC( MACO3 )
      RXRAT( 168 ) = RKI( 168 ) * YC( xRCO3 ) * YC( NO )
      RXRAT( 169 ) = RKI( 169 ) * YC( xRCO3 ) * YC( HO2 )
      RXRAT( 170 ) = RKI( 170 ) * YC( xRCO3 ) * YC( NO3 )
      RXRAT( 171 ) = RKI( 171 ) * YC( xRCO3 ) * YC( MEO2 )
      RXRAT( 172 ) = RKI( 172 ) * YC( xRCO3 ) * YC( RO2C )
      RXRAT( 173 ) = RKI( 173 ) * YC( xRCO3 ) * YC( RO2XC )
      RXRAT( 174 ) = RKI( 174 ) * YC( xRCO3 ) * YC( MECO3 )
      RXRAT( 175 ) = RKI( 175 ) * YC( xRCO3 ) * YC( RCO3 )
      RXRAT( 176 ) = RKI( 176 ) * YC( xRCO3 ) * YC( BZCO3 )
      RXRAT( 177 ) = RKI( 177 ) * YC( xRCO3 ) * YC( MACO3 )
      RXRAT( 178 ) = RKI( 178 ) * YC( xMACO3 ) * YC( NO )
      RXRAT( 179 ) = RKI( 179 ) * YC( xMACO3 ) * YC( HO2 )
      RXRAT( 180 ) = RKI( 180 ) * YC( xMACO3 ) * YC( NO3 )
      RXRAT( 181 ) = RKI( 181 ) * YC( xMACO3 ) * YC( MEO2 )
      RXRAT( 182 ) = RKI( 182 ) * YC( xMACO3 ) * YC( RO2C )
      RXRAT( 183 ) = RKI( 183 ) * YC( xMACO3 ) * YC( RO2XC )
      RXRAT( 184 ) = RKI( 184 ) * YC( xMACO3 ) * YC( MECO3 )
      RXRAT( 185 ) = RKI( 185 ) * YC( xMACO3 ) * YC( RCO3 )
      RXRAT( 186 ) = RKI( 186 ) * YC( xMACO3 ) * YC( BZCO3 )
      RXRAT( 187 ) = RKI( 187 ) * YC( xMACO3 ) * YC( MACO3 )
      RXRAT( 188 ) = RKI( 188 ) * YC( xTBUO ) * YC( NO )
      RXRAT( 189 ) = RKI( 189 ) * YC( xTBUO ) * YC( HO2 )
      RXRAT( 190 ) = RKI( 190 ) * YC( xTBUO ) * YC( NO3 )
      RXRAT( 191 ) = RKI( 191 ) * YC( xTBUO ) * YC( MEO2 )
      RXRAT( 192 ) = RKI( 192 ) * YC( xTBUO ) * YC( RO2C )
      RXRAT( 193 ) = RKI( 193 ) * YC( xTBUO ) * YC( RO2XC )
      RXRAT( 194 ) = RKI( 194 ) * YC( xTBUO ) * YC( MECO3 )
      RXRAT( 195 ) = RKI( 195 ) * YC( xTBUO ) * YC( RCO3 )
      RXRAT( 196 ) = RKI( 196 ) * YC( xTBUO ) * YC( BZCO3 )
      RXRAT( 197 ) = RKI( 197 ) * YC( xTBUO ) * YC( MACO3 )
      RXRAT( 198 ) = RKI( 198 ) * YC( xCO ) * YC( NO )
      RXRAT( 199 ) = RKI( 199 ) * YC( xCO ) * YC( HO2 )
      RXRAT( 200 ) = RKI( 200 ) * YC( xCO ) * YC( NO3 )
      RXRAT( 201 ) = RKI( 201 ) * YC( xCO ) * YC( MEO2 )
      RXRAT( 202 ) = RKI( 202 ) * YC( xCO ) * YC( RO2C )
      RXRAT( 203 ) = RKI( 203 ) * YC( xCO ) * YC( RO2XC )
      RXRAT( 204 ) = RKI( 204 ) * YC( xCO ) * YC( MECO3 )
      RXRAT( 205 ) = RKI( 205 ) * YC( xCO ) * YC( RCO3 )
      RXRAT( 206 ) = RKI( 206 ) * YC( xCO ) * YC( BZCO3 )
      RXRAT( 207 ) = RKI( 207 ) * YC( xCO ) * YC( MACO3 )
      RXRAT( 208 ) = RKI( 208 ) * YC( HCHO )
      RXRAT( 209 ) = RKI( 209 ) * YC( HCHO )
      RXRAT( 210 ) = RKI( 210 ) * YC( HCHO ) * YC( OH )
      RXRAT( 211 ) = RKI( 211 ) * YC( HCHO ) * YC( NO3 )
      RXRAT( 212 ) = RKI( 212 ) * YC( CCHO ) * YC( OH )
      RXRAT( 213 ) = RKI( 213 ) * YC( CCHO )
      RXRAT( 214 ) = RKI( 214 ) * YC( CCHO ) * YC( NO3 )
      RXRAT( 215 ) = RKI( 215 ) * YC( RCHO ) * YC( OH )
      RXRAT( 216 ) = RKI( 216 ) * YC( RCHO )
      RXRAT( 217 ) = RKI( 217 ) * YC( RCHO ) * YC( NO3 )
      RXRAT( 218 ) = RKI( 218 ) * YC( ACETONE ) * YC( OH )
      RXRAT( 219 ) = RKI( 219 ) * YC( ACETONE )
      RXRAT( 220 ) = RKI( 220 ) * YC( MEK ) * YC( OH )
      RXRAT( 221 ) = RKI( 221 ) * YC( MEK )
      RXRAT( 222 ) = RKI( 222 ) * YC( MEOH ) * YC( OH )
      RXRAT( 223 ) = RKI( 223 ) * YC( HCOOH ) * YC( OH )
      RXRAT( 224 ) = RKI( 224 ) * YC( CCOOH ) * YC( OH )
      RXRAT( 225 ) = RKI( 225 ) * YC( RCOOH ) * YC( OH )
      RXRAT( 226 ) = RKI( 226 ) * YC( COOH ) * YC( OH )
      RXRAT( 227 ) = RKI( 227 ) * YC( COOH )
      RXRAT( 228 ) = RKI( 228 ) * YC( ROOH ) * YC( OH )
      RXRAT( 229 ) = RKI( 229 ) * YC( ROOH )
      RXRAT( 230 ) = RKI( 230 ) * YC( R6OOH ) * YC( OH )
      RXRAT( 231 ) = RKI( 231 ) * YC( R6OOH )
      RXRAT( 232 ) = RKI( 232 ) * YC( RAOOH ) * YC( OH )
      RXRAT( 233 ) = RKI( 233 ) * YC( RAOOH )
      RXRAT( 234 ) = RKI( 234 ) * YC( GLY )
      RXRAT( 235 ) = RKI( 235 ) * YC( GLY )
      RXRAT( 236 ) = RKI( 236 ) * YC( GLY ) * YC( OH )
      RXRAT( 237 ) = RKI( 237 ) * YC( GLY ) * YC( NO3 )
      RXRAT( 238 ) = RKI( 238 ) * YC( MGLY )
      RXRAT( 239 ) = RKI( 239 ) * YC( MGLY ) * YC( OH )
      RXRAT( 240 ) = RKI( 240 ) * YC( MGLY ) * YC( NO3 )
      RXRAT( 241 ) = RKI( 241 ) * YC( BACL )
      RXRAT( 242 ) = RKI( 242 ) * YC( CRES ) * YC( OH )
      RXRAT( 243 ) = RKI( 243 ) * YC( CRES ) * YC( NO3 )
      RXRAT( 244 ) = RKI( 244 ) * YC( NPHE ) * YC( OH )
      RXRAT( 245 ) = RKI( 245 ) * YC( NPHE )
      RXRAT( 246 ) = RKI( 246 ) * YC( NPHE )
      RXRAT( 247 ) = RKI( 247 ) * YC( BALD ) * YC( OH )
      RXRAT( 248 ) = RKI( 248 ) * YC( BALD )
      RXRAT( 249 ) = RKI( 249 ) * YC( BALD ) * YC( NO3 )
      RXRAT( 250 ) = RKI( 250 ) * YC( AFG1 ) * YC( OH )
      RXRAT( 251 ) = RKI( 251 ) * YC( AFG1 ) * YC( O3 )
      RXRAT( 252 ) = RKI( 252 ) * YC( AFG1 )
      RXRAT( 253 ) = RKI( 253 ) * YC( AFG2 ) * YC( OH )
      RXRAT( 254 ) = RKI( 254 ) * YC( AFG2 ) * YC( O3 )
      RXRAT( 255 ) = RKI( 255 ) * YC( AFG2 )
      RXRAT( 256 ) = RKI( 256 ) * YC( AFG3 ) * YC( OH )
      RXRAT( 257 ) = RKI( 257 ) * YC( AFG3 ) * YC( O3 )
      RXRAT( 258 ) = RKI( 258 ) * YC( MACR ) * YC( OH )
      RXRAT( 259 ) = RKI( 259 ) * YC( MACR ) * YC( O3 )
      RXRAT( 260 ) = RKI( 260 ) * YC( MACR ) * YC( NO3 )
      RXRAT( 261 ) = RKI( 261 ) * YC( MACR ) * YC( O3P )
      RXRAT( 262 ) = RKI( 262 ) * YC( MACR )
      RXRAT( 263 ) = RKI( 263 ) * YC( MVK ) * YC( OH )
      RXRAT( 264 ) = RKI( 264 ) * YC( MVK ) * YC( O3 )
      RXRAT( 265 ) = RKI( 265 ) * YC( MVK ) * YC( O3P )
      RXRAT( 266 ) = RKI( 266 ) * YC( MVK )
      RXRAT( 267 ) = RKI( 267 ) * YC( IPRD ) * YC( OH )
      RXRAT( 268 ) = RKI( 268 ) * YC( IPRD ) * YC( O3 )
      RXRAT( 269 ) = RKI( 269 ) * YC( IPRD ) * YC( NO3 )
      RXRAT( 270 ) = RKI( 270 ) * YC( IPRD )
      RXRAT( 271 ) = RKI( 271 ) * YC( PRD2 ) * YC( OH )
      RXRAT( 272 ) = RKI( 272 ) * YC( PRD2 )
      RXRAT( 273 ) = RKI( 273 ) * YC( RNO3 ) * YC( OH )
      RXRAT( 274 ) = RKI( 274 ) * YC( RNO3 )
      RXRAT( 275 ) = RKI( 275 ) * YC( HOCCHO ) * YC( OH )
      RXRAT( 276 ) = RKI( 276 ) * YC( HOCCHO )
      RXRAT( 277 ) = RKI( 277 ) * YC( HOCCHO ) * YC( NO3 )
      RXRAT( 278 ) = RKI( 278 ) * YC( ACROLEIN ) * YC( OH )
      RXRAT( 279 ) = RKI( 279 ) * YC( ACROLEIN ) * YC( O3 )
      RXRAT( 280 ) = RKI( 280 ) * YC( ACROLEIN ) * YC( NO3 )
      RXRAT( 281 ) = RKI( 281 ) * YC( ACROLEIN ) * YC( O3P )
      RXRAT( 282 ) = RKI( 282 ) * YC( ACROLEIN )
      RXRAT( 283 ) = RKI( 283 ) * YC( CCOOOH ) * YC( OH )
      RXRAT( 284 ) = RKI( 284 ) * YC( CCOOOH )
      RXRAT( 285 ) = RKI( 285 ) * YC( RCOOOH ) * YC( OH )
      RXRAT( 286 ) = RKI( 286 ) * YC( RCOOOH )
      RXRAT( 287 ) = RKI( 287 ) * YC( xHCHO ) * YC( NO )
      RXRAT( 288 ) = RKI( 288 ) * YC( xHCHO ) * YC( HO2 )
      RXRAT( 289 ) = RKI( 289 ) * YC( xHCHO ) * YC( NO3 )
      RXRAT( 290 ) = RKI( 290 ) * YC( xHCHO ) * YC( MEO2 )
      RXRAT( 291 ) = RKI( 291 ) * YC( xHCHO ) * YC( RO2C )
      RXRAT( 292 ) = RKI( 292 ) * YC( xHCHO ) * YC( RO2XC )
      RXRAT( 293 ) = RKI( 293 ) * YC( xHCHO ) * YC( MECO3 )
      RXRAT( 294 ) = RKI( 294 ) * YC( xHCHO ) * YC( RCO3 )
      RXRAT( 295 ) = RKI( 295 ) * YC( xHCHO ) * YC( BZCO3 )
      RXRAT( 296 ) = RKI( 296 ) * YC( xHCHO ) * YC( MACO3 )
      RXRAT( 297 ) = RKI( 297 ) * YC( xCCHO ) * YC( NO )
      RXRAT( 298 ) = RKI( 298 ) * YC( xCCHO ) * YC( HO2 )
      RXRAT( 299 ) = RKI( 299 ) * YC( xCCHO ) * YC( NO3 )
      RXRAT( 300 ) = RKI( 300 ) * YC( xCCHO ) * YC( MEO2 )
      RXRAT( 301 ) = RKI( 301 ) * YC( xCCHO ) * YC( RO2C )
      RXRAT( 302 ) = RKI( 302 ) * YC( xCCHO ) * YC( RO2XC )
      RXRAT( 303 ) = RKI( 303 ) * YC( xCCHO ) * YC( MECO3 )
      RXRAT( 304 ) = RKI( 304 ) * YC( xCCHO ) * YC( RCO3 )
      RXRAT( 305 ) = RKI( 305 ) * YC( xCCHO ) * YC( BZCO3 )
      RXRAT( 306 ) = RKI( 306 ) * YC( xCCHO ) * YC( MACO3 )
      RXRAT( 307 ) = RKI( 307 ) * YC( xRCHO ) * YC( NO )
      RXRAT( 308 ) = RKI( 308 ) * YC( xRCHO ) * YC( HO2 )
      RXRAT( 309 ) = RKI( 309 ) * YC( xRCHO ) * YC( NO3 )
      RXRAT( 310 ) = RKI( 310 ) * YC( xRCHO ) * YC( MEO2 )
      RXRAT( 311 ) = RKI( 311 ) * YC( xRCHO ) * YC( RO2C )
      RXRAT( 312 ) = RKI( 312 ) * YC( xRCHO ) * YC( RO2XC )
      RXRAT( 313 ) = RKI( 313 ) * YC( xRCHO ) * YC( MECO3 )
      RXRAT( 314 ) = RKI( 314 ) * YC( xRCHO ) * YC( RCO3 )
      RXRAT( 315 ) = RKI( 315 ) * YC( xRCHO ) * YC( BZCO3 )
      RXRAT( 316 ) = RKI( 316 ) * YC( xRCHO ) * YC( MACO3 )
      RXRAT( 317 ) = RKI( 317 ) * YC( xACETONE ) * YC( NO )
      RXRAT( 318 ) = RKI( 318 ) * YC( xACETONE ) * YC( HO2 )
      RXRAT( 319 ) = RKI( 319 ) * YC( xACETONE ) * YC( NO3 )
      RXRAT( 320 ) = RKI( 320 ) * YC( xACETONE ) * YC( MEO2 )
      RXRAT( 321 ) = RKI( 321 ) * YC( xACETONE ) * YC( RO2C )
      RXRAT( 322 ) = RKI( 322 ) * YC( xACETONE ) * YC( RO2XC )
      RXRAT( 323 ) = RKI( 323 ) * YC( xACETONE ) * YC( MECO3 )
      RXRAT( 324 ) = RKI( 324 ) * YC( xACETONE ) * YC( RCO3 )
      RXRAT( 325 ) = RKI( 325 ) * YC( xACETONE ) * YC( BZCO3 )
      RXRAT( 326 ) = RKI( 326 ) * YC( xACETONE ) * YC( MACO3 )
      RXRAT( 327 ) = RKI( 327 ) * YC( xMEK ) * YC( NO )
      RXRAT( 328 ) = RKI( 328 ) * YC( xMEK ) * YC( HO2 )
      RXRAT( 329 ) = RKI( 329 ) * YC( xMEK ) * YC( NO3 )
      RXRAT( 330 ) = RKI( 330 ) * YC( xMEK ) * YC( MEO2 )
      RXRAT( 331 ) = RKI( 331 ) * YC( xMEK ) * YC( RO2C )
      RXRAT( 332 ) = RKI( 332 ) * YC( xMEK ) * YC( RO2XC )
      RXRAT( 333 ) = RKI( 333 ) * YC( xMEK ) * YC( MECO3 )
      RXRAT( 334 ) = RKI( 334 ) * YC( xMEK ) * YC( RCO3 )
      RXRAT( 335 ) = RKI( 335 ) * YC( xMEK ) * YC( BZCO3 )
      RXRAT( 336 ) = RKI( 336 ) * YC( xMEK ) * YC( MACO3 )
      RXRAT( 337 ) = RKI( 337 ) * YC( xPROD2 ) * YC( NO )
      RXRAT( 338 ) = RKI( 338 ) * YC( xPROD2 ) * YC( HO2 )
      RXRAT( 339 ) = RKI( 339 ) * YC( xPROD2 ) * YC( NO3 )
      RXRAT( 340 ) = RKI( 340 ) * YC( xPROD2 ) * YC( MEO2 )
      RXRAT( 341 ) = RKI( 341 ) * YC( xPROD2 ) * YC( RO2C )
      RXRAT( 342 ) = RKI( 342 ) * YC( xPROD2 ) * YC( RO2XC )
      RXRAT( 343 ) = RKI( 343 ) * YC( xPROD2 ) * YC( MECO3 )
      RXRAT( 344 ) = RKI( 344 ) * YC( xPROD2 ) * YC( RCO3 )
      RXRAT( 345 ) = RKI( 345 ) * YC( xPROD2 ) * YC( BZCO3 )
      RXRAT( 346 ) = RKI( 346 ) * YC( xPROD2 ) * YC( MACO3 )
      RXRAT( 347 ) = RKI( 347 ) * YC( xGLY ) * YC( NO )
      RXRAT( 348 ) = RKI( 348 ) * YC( xGLY ) * YC( HO2 )
      RXRAT( 349 ) = RKI( 349 ) * YC( xGLY ) * YC( NO3 )
      RXRAT( 350 ) = RKI( 350 ) * YC( xGLY ) * YC( MEO2 )
      RXRAT( 351 ) = RKI( 351 ) * YC( xGLY ) * YC( RO2C )
      RXRAT( 352 ) = RKI( 352 ) * YC( xGLY ) * YC( RO2XC )
      RXRAT( 353 ) = RKI( 353 ) * YC( xGLY ) * YC( MECO3 )
      RXRAT( 354 ) = RKI( 354 ) * YC( xGLY ) * YC( RCO3 )
      RXRAT( 355 ) = RKI( 355 ) * YC( xGLY ) * YC( BZCO3 )
      RXRAT( 356 ) = RKI( 356 ) * YC( xGLY ) * YC( MACO3 )
      RXRAT( 357 ) = RKI( 357 ) * YC( xMGLY ) * YC( NO )
      RXRAT( 358 ) = RKI( 358 ) * YC( xMGLY ) * YC( HO2 )
      RXRAT( 359 ) = RKI( 359 ) * YC( xMGLY ) * YC( NO3 )
      RXRAT( 360 ) = RKI( 360 ) * YC( xMGLY ) * YC( MEO2 )
      RXRAT( 361 ) = RKI( 361 ) * YC( xMGLY ) * YC( RO2C )
      RXRAT( 362 ) = RKI( 362 ) * YC( xMGLY ) * YC( RO2XC )
      RXRAT( 363 ) = RKI( 363 ) * YC( xMGLY ) * YC( MECO3 )
      RXRAT( 364 ) = RKI( 364 ) * YC( xMGLY ) * YC( RCO3 )
      RXRAT( 365 ) = RKI( 365 ) * YC( xMGLY ) * YC( BZCO3 )
      RXRAT( 366 ) = RKI( 366 ) * YC( xMGLY ) * YC( MACO3 )
      RXRAT( 367 ) = RKI( 367 ) * YC( xBACL ) * YC( NO )
      RXRAT( 368 ) = RKI( 368 ) * YC( xBACL ) * YC( HO2 )
      RXRAT( 369 ) = RKI( 369 ) * YC( xBACL ) * YC( NO3 )
      RXRAT( 370 ) = RKI( 370 ) * YC( xBACL ) * YC( MEO2 )
      RXRAT( 371 ) = RKI( 371 ) * YC( xBACL ) * YC( RO2C )
      RXRAT( 372 ) = RKI( 372 ) * YC( xBACL ) * YC( RO2XC )
      RXRAT( 373 ) = RKI( 373 ) * YC( xBACL ) * YC( MECO3 )
      RXRAT( 374 ) = RKI( 374 ) * YC( xBACL ) * YC( RCO3 )
      RXRAT( 375 ) = RKI( 375 ) * YC( xBACL ) * YC( BZCO3 )
      RXRAT( 376 ) = RKI( 376 ) * YC( xBACL ) * YC( MACO3 )
      RXRAT( 377 ) = RKI( 377 ) * YC( xBALD ) * YC( NO )
      RXRAT( 378 ) = RKI( 378 ) * YC( xBALD ) * YC( HO2 )
      RXRAT( 379 ) = RKI( 379 ) * YC( xBALD ) * YC( NO3 )
      RXRAT( 380 ) = RKI( 380 ) * YC( xBALD ) * YC( MEO2 )
      RXRAT( 381 ) = RKI( 381 ) * YC( xBALD ) * YC( RO2C )
      RXRAT( 382 ) = RKI( 382 ) * YC( xBALD ) * YC( RO2XC )
      RXRAT( 383 ) = RKI( 383 ) * YC( xBALD ) * YC( MECO3 )
      RXRAT( 384 ) = RKI( 384 ) * YC( xBALD ) * YC( RCO3 )
      RXRAT( 385 ) = RKI( 385 ) * YC( xBALD ) * YC( BZCO3 )
      RXRAT( 386 ) = RKI( 386 ) * YC( xBALD ) * YC( MACO3 )
      RXRAT( 387 ) = RKI( 387 ) * YC( xAFG1 ) * YC( NO )
      RXRAT( 388 ) = RKI( 388 ) * YC( xAFG1 ) * YC( HO2 )
      RXRAT( 389 ) = RKI( 389 ) * YC( xAFG1 ) * YC( NO3 )
      RXRAT( 390 ) = RKI( 390 ) * YC( xAFG1 ) * YC( MEO2 )
      RXRAT( 391 ) = RKI( 391 ) * YC( xAFG1 ) * YC( RO2C )
      RXRAT( 392 ) = RKI( 392 ) * YC( xAFG1 ) * YC( RO2XC )
      RXRAT( 393 ) = RKI( 393 ) * YC( xAFG1 ) * YC( MECO3 )
      RXRAT( 394 ) = RKI( 394 ) * YC( xAFG1 ) * YC( RCO3 )
      RXRAT( 395 ) = RKI( 395 ) * YC( xAFG1 ) * YC( BZCO3 )
      RXRAT( 396 ) = RKI( 396 ) * YC( xAFG1 ) * YC( MACO3 )
      RXRAT( 397 ) = RKI( 397 ) * YC( xAFG2 ) * YC( NO )
      RXRAT( 398 ) = RKI( 398 ) * YC( xAFG2 ) * YC( HO2 )
      RXRAT( 399 ) = RKI( 399 ) * YC( xAFG2 ) * YC( NO3 )
      RXRAT( 400 ) = RKI( 400 ) * YC( xAFG2 ) * YC( MEO2 )
      RXRAT( 401 ) = RKI( 401 ) * YC( xAFG2 ) * YC( RO2C )
      RXRAT( 402 ) = RKI( 402 ) * YC( xAFG2 ) * YC( RO2XC )
      RXRAT( 403 ) = RKI( 403 ) * YC( xAFG2 ) * YC( MECO3 )
      RXRAT( 404 ) = RKI( 404 ) * YC( xAFG2 ) * YC( RCO3 )
      RXRAT( 405 ) = RKI( 405 ) * YC( xAFG2 ) * YC( BZCO3 )
      RXRAT( 406 ) = RKI( 406 ) * YC( xAFG2 ) * YC( MACO3 )
      RXRAT( 407 ) = RKI( 407 ) * YC( xAFG3 ) * YC( NO )
      RXRAT( 408 ) = RKI( 408 ) * YC( xAFG3 ) * YC( HO2 )
      RXRAT( 409 ) = RKI( 409 ) * YC( xAFG3 ) * YC( NO3 )
      RXRAT( 410 ) = RKI( 410 ) * YC( xAFG3 ) * YC( MEO2 )
      RXRAT( 411 ) = RKI( 411 ) * YC( xAFG3 ) * YC( RO2C )
      RXRAT( 412 ) = RKI( 412 ) * YC( xAFG3 ) * YC( RO2XC )
      RXRAT( 413 ) = RKI( 413 ) * YC( xAFG3 ) * YC( MECO3 )
      RXRAT( 414 ) = RKI( 414 ) * YC( xAFG3 ) * YC( RCO3 )
      RXRAT( 415 ) = RKI( 415 ) * YC( xAFG3 ) * YC( BZCO3 )
      RXRAT( 416 ) = RKI( 416 ) * YC( xAFG3 ) * YC( MACO3 )
      RXRAT( 417 ) = RKI( 417 ) * YC( xMACR ) * YC( NO )
      RXRAT( 418 ) = RKI( 418 ) * YC( xMACR ) * YC( HO2 )
      RXRAT( 419 ) = RKI( 419 ) * YC( xMACR ) * YC( NO3 )
      RXRAT( 420 ) = RKI( 420 ) * YC( xMACR ) * YC( MEO2 )
      RXRAT( 421 ) = RKI( 421 ) * YC( xMACR ) * YC( RO2C )
      RXRAT( 422 ) = RKI( 422 ) * YC( xMACR ) * YC( RO2XC )
      RXRAT( 423 ) = RKI( 423 ) * YC( xMACR ) * YC( MECO3 )
      RXRAT( 424 ) = RKI( 424 ) * YC( xMACR ) * YC( RCO3 )
      RXRAT( 425 ) = RKI( 425 ) * YC( xMACR ) * YC( BZCO3 )
      RXRAT( 426 ) = RKI( 426 ) * YC( xMACR ) * YC( MACO3 )
      RXRAT( 427 ) = RKI( 427 ) * YC( xMVK ) * YC( NO )
      RXRAT( 428 ) = RKI( 428 ) * YC( xMVK ) * YC( HO2 )
      RXRAT( 429 ) = RKI( 429 ) * YC( xMVK ) * YC( NO3 )
      RXRAT( 430 ) = RKI( 430 ) * YC( xMVK ) * YC( MEO2 )
      RXRAT( 431 ) = RKI( 431 ) * YC( xMVK ) * YC( RO2C )
      RXRAT( 432 ) = RKI( 432 ) * YC( xMVK ) * YC( RO2XC )
      RXRAT( 433 ) = RKI( 433 ) * YC( xMVK ) * YC( MECO3 )
      RXRAT( 434 ) = RKI( 434 ) * YC( xMVK ) * YC( RCO3 )
      RXRAT( 435 ) = RKI( 435 ) * YC( xMVK ) * YC( BZCO3 )
      RXRAT( 436 ) = RKI( 436 ) * YC( xMVK ) * YC( MACO3 )
      RXRAT( 437 ) = RKI( 437 ) * YC( xIPRD ) * YC( NO )
      RXRAT( 438 ) = RKI( 438 ) * YC( xIPRD ) * YC( HO2 )
      RXRAT( 439 ) = RKI( 439 ) * YC( xIPRD ) * YC( NO3 )
      RXRAT( 440 ) = RKI( 440 ) * YC( xIPRD ) * YC( MEO2 )
      RXRAT( 441 ) = RKI( 441 ) * YC( xIPRD ) * YC( RO2C )
      RXRAT( 442 ) = RKI( 442 ) * YC( xIPRD ) * YC( RO2XC )
      RXRAT( 443 ) = RKI( 443 ) * YC( xIPRD ) * YC( MECO3 )
      RXRAT( 444 ) = RKI( 444 ) * YC( xIPRD ) * YC( RCO3 )
      RXRAT( 445 ) = RKI( 445 ) * YC( xIPRD ) * YC( BZCO3 )
      RXRAT( 446 ) = RKI( 446 ) * YC( xIPRD ) * YC( MACO3 )
      RXRAT( 447 ) = RKI( 447 ) * YC( xRNO3 ) * YC( NO )
      RXRAT( 448 ) = RKI( 448 ) * YC( xRNO3 ) * YC( HO2 )
      RXRAT( 449 ) = RKI( 449 ) * YC( xRNO3 ) * YC( NO3 )
      RXRAT( 450 ) = RKI( 450 ) * YC( xRNO3 ) * YC( MEO2 )
      RXRAT( 451 ) = RKI( 451 ) * YC( xRNO3 ) * YC( RO2C )
      RXRAT( 452 ) = RKI( 452 ) * YC( xRNO3 ) * YC( RO2XC )
      RXRAT( 453 ) = RKI( 453 ) * YC( xRNO3 ) * YC( MECO3 )
      RXRAT( 454 ) = RKI( 454 ) * YC( xRNO3 ) * YC( RCO3 )
      RXRAT( 455 ) = RKI( 455 ) * YC( xRNO3 ) * YC( BZCO3 )
      RXRAT( 456 ) = RKI( 456 ) * YC( xRNO3 ) * YC( MACO3 )
      RXRAT( 457 ) = RKI( 457 ) * YC( yROOH ) * YC( NO )
      RXRAT( 458 ) = RKI( 458 ) * YC( yROOH ) * YC( HO2 )
      RXRAT( 459 ) = RKI( 459 ) * YC( yROOH ) * YC( NO3 )
      RXRAT( 460 ) = RKI( 460 ) * YC( yROOH ) * YC( MEO2 )
      RXRAT( 461 ) = RKI( 461 ) * YC( yROOH ) * YC( RO2C )
      RXRAT( 462 ) = RKI( 462 ) * YC( yROOH ) * YC( RO2XC )
      RXRAT( 463 ) = RKI( 463 ) * YC( yROOH ) * YC( MECO3 )
      RXRAT( 464 ) = RKI( 464 ) * YC( yROOH ) * YC( RCO3 )
      RXRAT( 465 ) = RKI( 465 ) * YC( yROOH ) * YC( BZCO3 )
      RXRAT( 466 ) = RKI( 466 ) * YC( yROOH ) * YC( MACO3 )
      RXRAT( 467 ) = RKI( 467 ) * YC( yR6OOH ) * YC( NO )
      RXRAT( 468 ) = RKI( 468 ) * YC( yR6OOH ) * YC( HO2 )
      RXRAT( 469 ) = RKI( 469 ) * YC( yR6OOH ) * YC( NO3 )
      RXRAT( 470 ) = RKI( 470 ) * YC( yR6OOH ) * YC( MEO2 )
      RXRAT( 471 ) = RKI( 471 ) * YC( yR6OOH ) * YC( RO2C )
      RXRAT( 472 ) = RKI( 472 ) * YC( yR6OOH ) * YC( RO2XC )
      RXRAT( 473 ) = RKI( 473 ) * YC( yR6OOH ) * YC( MECO3 )
      RXRAT( 474 ) = RKI( 474 ) * YC( yR6OOH ) * YC( RCO3 )
      RXRAT( 475 ) = RKI( 475 ) * YC( yR6OOH ) * YC( BZCO3 )
      RXRAT( 476 ) = RKI( 476 ) * YC( yR6OOH ) * YC( MACO3 )
      RXRAT( 477 ) = RKI( 477 ) * YC( yRAOOH ) * YC( NO )
      RXRAT( 478 ) = RKI( 478 ) * YC( yRAOOH ) * YC( HO2 )
      RXRAT( 479 ) = RKI( 479 ) * YC( yRAOOH ) * YC( NO3 )
      RXRAT( 480 ) = RKI( 480 ) * YC( yRAOOH ) * YC( MEO2 )
      RXRAT( 481 ) = RKI( 481 ) * YC( yRAOOH ) * YC( RO2C )
      RXRAT( 482 ) = RKI( 482 ) * YC( yRAOOH ) * YC( RO2XC )
      RXRAT( 483 ) = RKI( 483 ) * YC( yRAOOH ) * YC( MECO3 )
      RXRAT( 484 ) = RKI( 484 ) * YC( yRAOOH ) * YC( RCO3 )
      RXRAT( 485 ) = RKI( 485 ) * YC( yRAOOH ) * YC( BZCO3 )
      RXRAT( 486 ) = RKI( 486 ) * YC( yRAOOH ) * YC( MACO3 )
      RXRAT( 487 ) = RKI( 487 ) * YC( zRNO3 ) * YC( NO )
      RXRAT( 488 ) = RKI( 488 ) * YC( zRNO3 ) * YC( HO2 )
      RXRAT( 489 ) = RKI( 489 ) * YC( zRNO3 ) * YC( NO3 )
      RXRAT( 490 ) = RKI( 490 ) * YC( zRNO3 ) * YC( MEO2 )
      RXRAT( 491 ) = RKI( 491 ) * YC( zRNO3 ) * YC( RO2C )
      RXRAT( 492 ) = RKI( 492 ) * YC( zRNO3 ) * YC( RO2XC )
      RXRAT( 493 ) = RKI( 493 ) * YC( zRNO3 ) * YC( MECO3 )
      RXRAT( 494 ) = RKI( 494 ) * YC( zRNO3 ) * YC( RCO3 )
      RXRAT( 495 ) = RKI( 495 ) * YC( zRNO3 ) * YC( BZCO3 )
      RXRAT( 496 ) = RKI( 496 ) * YC( zRNO3 ) * YC( MACO3 )
      RXRAT( 497 ) = RKI( 497 ) * YC( xHOCCHO ) * YC( NO )
      RXRAT( 498 ) = RKI( 498 ) * YC( xHOCCHO ) * YC( HO2 )
      RXRAT( 499 ) = RKI( 499 ) * YC( xHOCCHO ) * YC( NO3 )
      RXRAT( 500 ) = RKI( 500 ) * YC( xHOCCHO ) * YC( MEO2 )
      RXRAT( 501 ) = RKI( 501 ) * YC( xHOCCHO ) * YC( RO2C )
      RXRAT( 502 ) = RKI( 502 ) * YC( xHOCCHO ) * YC( RO2XC )
      RXRAT( 503 ) = RKI( 503 ) * YC( xHOCCHO ) * YC( MECO3 )
      RXRAT( 504 ) = RKI( 504 ) * YC( xHOCCHO ) * YC( RCO3 )
      RXRAT( 505 ) = RKI( 505 ) * YC( xHOCCHO ) * YC( BZCO3 )
      RXRAT( 506 ) = RKI( 506 ) * YC( xHOCCHO ) * YC( MACO3 )
      RXRAT( 507 ) = RKI( 507 ) * YC( xACROLEIN ) * YC( NO )
      RXRAT( 508 ) = RKI( 508 ) * YC( xACROLEIN ) * YC( HO2 )
      RXRAT( 509 ) = RKI( 509 ) * YC( xACROLEIN ) * YC( NO3 )
      RXRAT( 510 ) = RKI( 510 ) * YC( xACROLEIN ) * YC( MEO2 )
      RXRAT( 511 ) = RKI( 511 ) * YC( xACROLEIN ) * YC( RO2C )
      RXRAT( 512 ) = RKI( 512 ) * YC( xACROLEIN ) * YC( RO2XC )
      RXRAT( 513 ) = RKI( 513 ) * YC( xACROLEIN ) * YC( MECO3 )
      RXRAT( 514 ) = RKI( 514 ) * YC( xACROLEIN ) * YC( RCO3 )
      RXRAT( 515 ) = RKI( 515 ) * YC( xACROLEIN ) * YC( BZCO3 )
      RXRAT( 516 ) = RKI( 516 ) * YC( xACROLEIN ) * YC( MACO3 )
      RXRAT( 517 ) = RKI( 517 ) * YC( OH )
      RXRAT( 518 ) = RKI( 518 ) * YC( ETHENE ) * YC( OH )
      RXRAT( 519 ) = RKI( 519 ) * YC( ETHENE ) * YC( O3 )
      RXRAT( 520 ) = RKI( 520 ) * YC( ETHENE ) * YC( NO3 )
      RXRAT( 521 ) = RKI( 521 ) * YC( ETHENE ) * YC( O3P )
      RXRAT( 522 ) = RKI( 522 ) * YC( PROPENE ) * YC( OH )
      RXRAT( 523 ) = RKI( 523 ) * YC( PROPENE ) * YC( O3 )
      RXRAT( 524 ) = RKI( 524 ) * YC( PROPENE ) * YC( NO3 )
      RXRAT( 525 ) = RKI( 525 ) * YC( PROPENE ) * YC( O3P )
      RXRAT( 526 ) = RKI( 526 ) * YC( BUTADIENE13 ) * YC( OH )
      RXRAT( 527 ) = RKI( 527 ) * YC( BUTADIENE13 ) * YC( O3 )
      RXRAT( 528 ) = RKI( 528 ) * YC( BUTADIENE13 ) * YC( NO3 )
      RXRAT( 529 ) = RKI( 529 ) * YC( BUTADIENE13 ) * YC( O3P )
      RXRAT( 530 ) = RKI( 530 ) * YC( ISOPRENE ) * YC( OH )
      RXRAT( 531 ) = RKI( 531 ) * YC( ISOPRENE ) * YC( O3 )
      RXRAT( 532 ) = RKI( 532 ) * YC( ISOPRENE ) * YC( NO3 )
      RXRAT( 533 ) = RKI( 533 ) * YC( ISOPRENE ) * YC( O3P )
      RXRAT( 534 ) = RKI( 534 ) * YC( APIN ) * YC( OH )
      RXRAT( 535 ) = RKI( 535 ) * YC( APIN ) * YC( O3 )
      RXRAT( 536 ) = RKI( 536 ) * YC( APIN ) * YC( NO3 )
      RXRAT( 537 ) = RKI( 537 ) * YC( APIN ) * YC( O3P )
      RXRAT( 538 ) = RKI( 538 ) * YC( ACETYLENE ) * YC( OH )
      RXRAT( 539 ) = RKI( 539 ) * YC( ACETYLENE ) * YC( O3 )
      RXRAT( 540 ) = RKI( 540 ) * YC( BENZENE ) * YC( OH )
      RXRAT( 541 ) = RKI( 541 ) * YC( TOLUENE ) * YC( OH )
      RXRAT( 542 ) = RKI( 542 ) * YC( MXYL ) * YC( OH )
      RXRAT( 543 ) = RKI( 543 ) * YC( OXYL ) * YC( OH )
      RXRAT( 544 ) = RKI( 544 ) * YC( PXYL ) * YC( OH )
      RXRAT( 545 ) = RKI( 545 ) * YC( TRIMETH_BENZ124 ) * YC( OH )
      RXRAT( 546 ) = RKI( 546 ) * YC( ETOH ) * YC( OH )
      RXRAT( 547 ) = RKI( 547 ) * YC( ALK1 ) * YC( OH )
      RXRAT( 548 ) = RKI( 548 ) * YC( ALK2 ) * YC( OH )
      RXRAT( 549 ) = RKI( 549 ) * YC( ALK3 ) * YC( OH )
      RXRAT( 550 ) = RKI( 550 ) * YC( ALK4 ) * YC( OH )
      RXRAT( 551 ) = RKI( 551 ) * YC( ALK5 ) * YC( OH )
      RXRAT( 552 ) = RKI( 552 ) * YC( OLE1 ) * YC( OH )
      RXRAT( 553 ) = RKI( 553 ) * YC( OLE1 ) * YC( O3 )
      RXRAT( 554 ) = RKI( 554 ) * YC( OLE1 ) * YC( NO3 )
      RXRAT( 555 ) = RKI( 555 ) * YC( OLE1 ) * YC( O3P )
      RXRAT( 556 ) = RKI( 556 ) * YC( OLE2 ) * YC( OH )
      RXRAT( 557 ) = RKI( 557 ) * YC( OLE2 ) * YC( O3 )
      RXRAT( 558 ) = RKI( 558 ) * YC( OLE2 ) * YC( NO3 )
      RXRAT( 559 ) = RKI( 559 ) * YC( OLE2 ) * YC( O3P )
      RXRAT( 560 ) = RKI( 560 ) * YC( ARO1 ) * YC( OH )
      RXRAT( 561 ) = RKI( 561 ) * YC( ARO2 ) * YC( OH )
      RXRAT( 562 ) = RKI( 562 ) * YC( TERP ) * YC( OH )
      RXRAT( 563 ) = RKI( 563 ) * YC( TERP ) * YC( O3 )
      RXRAT( 564 ) = RKI( 564 ) * YC( TERP ) * YC( NO3 )
      RXRAT( 565 ) = RKI( 565 ) * YC( TERP ) * YC( O3P )
      RXRAT( 566 ) = RKI( 566 ) * YC( SESQ ) * YC( OH )
      RXRAT( 567 ) = RKI( 567 ) * YC( SESQ ) * YC( O3 )
      RXRAT( 568 ) = RKI( 568 ) * YC( SESQ ) * YC( NO3 )
      RXRAT( 569 ) = RKI( 569 ) * YC( SESQ ) * YC( O3P )
      RXRAT( 570 ) = RKI( 570 ) * YC( CL2 )
      RXRAT( 571 ) = RKI( 571 ) * YC( CL ) * YC( NO )
      RXRAT( 572 ) = RKI( 572 ) * YC( CLNO )
      RXRAT( 573 ) = RKI( 573 ) * YC( CL ) * YC( NO2 )
      RXRAT( 574 ) = RKI( 574 ) * YC( CL ) * YC( NO2 )
      RXRAT( 575 ) = RKI( 575 ) * YC( CLONO )
      RXRAT( 576 ) = RKI( 576 ) * YC( CLNO2 )
      RXRAT( 577 ) = RKI( 577 ) * YC( CL ) * YC( HO2 )
      RXRAT( 578 ) = RKI( 578 ) * YC( CL ) * YC( HO2 )
      RXRAT( 579 ) = RKI( 579 ) * YC( CL ) * YC( O3 )
      RXRAT( 580 ) = RKI( 580 ) * YC( CL ) * YC( NO3 )
      RXRAT( 581 ) = RKI( 581 ) * YC( CLO ) * YC( NO )
      RXRAT( 582 ) = RKI( 582 ) * YC( CLO ) * YC( NO2 )
      RXRAT( 583 ) = RKI( 583 ) * YC( CLONO2 )
      RXRAT( 584 ) = RKI( 584 ) * YC( CLONO2 )
      RXRAT( 585 ) = RKI( 585 ) * YC( CLONO2 )
      RXRAT( 586 ) = RKI( 586 ) * YC( CL ) * YC( CLONO2 )
      RXRAT( 587 ) = RKI( 587 ) * YC( CLO ) * YC( HO2 )
      RXRAT( 588 ) = RKI( 588 ) * YC( HOCL )
      RXRAT( 589 ) = RKI( 589 ) * YC( CLO ) * YC( CLO )
      RXRAT( 590 ) = RKI( 590 ) * YC( OH ) * YC( HCL )
      RXRAT( 591 ) = RKI( 591 ) * YC( CL )
      RXRAT( 592 ) = RKI( 592 ) * YC( HCHO ) * YC( CL )
      RXRAT( 593 ) = RKI( 593 ) * YC( CCHO ) * YC( CL )
      RXRAT( 594 ) = RKI( 594 ) * YC( MEOH ) * YC( CL )
      RXRAT( 595 ) = RKI( 595 ) * YC( RCHO ) * YC( CL )
      RXRAT( 596 ) = RKI( 596 ) * YC( ACETONE ) * YC( CL )
      RXRAT( 597 ) = RKI( 597 ) * YC( MEK ) * YC( CL )
      RXRAT( 598 ) = RKI( 598 ) * YC( RNO3 ) * YC( CL )
      RXRAT( 599 ) = RKI( 599 ) * YC( PRD2 ) * YC( CL )
      RXRAT( 600 ) = RKI( 600 ) * YC( GLY ) * YC( CL )
      RXRAT( 601 ) = RKI( 601 ) * YC( MGLY ) * YC( CL )
      RXRAT( 602 ) = RKI( 602 ) * YC( CRES ) * YC( CL )
      RXRAT( 603 ) = RKI( 603 ) * YC( BALD ) * YC( CL )
      RXRAT( 604 ) = RKI( 604 ) * YC( ROOH ) * YC( CL )
      RXRAT( 605 ) = RKI( 605 ) * YC( R6OOH ) * YC( CL )
      RXRAT( 606 ) = RKI( 606 ) * YC( RAOOH ) * YC( CL )
      RXRAT( 607 ) = RKI( 607 ) * YC( ACROLEIN ) * YC( CL )
      RXRAT( 608 ) = RKI( 608 ) * YC( MACR ) * YC( CL )
      RXRAT( 609 ) = RKI( 609 ) * YC( MVK ) * YC( CL )
      RXRAT( 610 ) = RKI( 610 ) * YC( IPRD ) * YC( CL )
      RXRAT( 611 ) = RKI( 611 ) * YC( CLCCHO )
      RXRAT( 612 ) = RKI( 612 ) * YC( CLCCHO ) * YC( OH )
      RXRAT( 613 ) = RKI( 613 ) * YC( CLCCHO ) * YC( CL )
      RXRAT( 614 ) = RKI( 614 ) * YC( CLACET )
      RXRAT( 615 ) = RKI( 615 ) * YC( xCL ) * YC( NO )
      RXRAT( 616 ) = RKI( 616 ) * YC( xCL ) * YC( HO2 )
      RXRAT( 617 ) = RKI( 617 ) * YC( xCL ) * YC( NO3 )
      RXRAT( 618 ) = RKI( 618 ) * YC( xCL ) * YC( MEO2 )
      RXRAT( 619 ) = RKI( 619 ) * YC( xCL ) * YC( RO2C )
      RXRAT( 620 ) = RKI( 620 ) * YC( xCL ) * YC( RO2XC )
      RXRAT( 621 ) = RKI( 621 ) * YC( xCL ) * YC( MECO3 )
      RXRAT( 622 ) = RKI( 622 ) * YC( xCL ) * YC( RCO3 )
      RXRAT( 623 ) = RKI( 623 ) * YC( xCL ) * YC( BZCO3 )
      RXRAT( 624 ) = RKI( 624 ) * YC( xCL ) * YC( MACO3 )
      RXRAT( 625 ) = RKI( 625 ) * YC( xCLCCHO ) * YC( NO )
      RXRAT( 626 ) = RKI( 626 ) * YC( xCLCCHO ) * YC( HO2 )
      RXRAT( 627 ) = RKI( 627 ) * YC( xCLCCHO ) * YC( NO3 )
      RXRAT( 628 ) = RKI( 628 ) * YC( xCLCCHO ) * YC( MEO2 )
      RXRAT( 629 ) = RKI( 629 ) * YC( xCLCCHO ) * YC( RO2C )
      RXRAT( 630 ) = RKI( 630 ) * YC( xCLCCHO ) * YC( RO2XC )
      RXRAT( 631 ) = RKI( 631 ) * YC( xCLCCHO ) * YC( MECO3 )
      RXRAT( 632 ) = RKI( 632 ) * YC( xCLCCHO ) * YC( RCO3 )
      RXRAT( 633 ) = RKI( 633 ) * YC( xCLCCHO ) * YC( BZCO3 )
      RXRAT( 634 ) = RKI( 634 ) * YC( xCLCCHO ) * YC( MACO3 )
      RXRAT( 635 ) = RKI( 635 ) * YC( xCLACET ) * YC( NO )
      RXRAT( 636 ) = RKI( 636 ) * YC( xCLACET ) * YC( HO2 )
      RXRAT( 637 ) = RKI( 637 ) * YC( xCLACET ) * YC( NO3 )
      RXRAT( 638 ) = RKI( 638 ) * YC( xCLACET ) * YC( MEO2 )
      RXRAT( 639 ) = RKI( 639 ) * YC( xCLACET ) * YC( RO2C )
      RXRAT( 640 ) = RKI( 640 ) * YC( xCLACET ) * YC( RO2XC )
      RXRAT( 641 ) = RKI( 641 ) * YC( xCLACET ) * YC( MECO3 )
      RXRAT( 642 ) = RKI( 642 ) * YC( xCLACET ) * YC( RCO3 )
      RXRAT( 643 ) = RKI( 643 ) * YC( xCLACET ) * YC( BZCO3 )
      RXRAT( 644 ) = RKI( 644 ) * YC( xCLACET ) * YC( MACO3 )
      RXRAT( 645 ) = RKI( 645 ) * YC( CL )
      RXRAT( 646 ) = RKI( 646 ) * YC( ETHENE ) * YC( CL )
      RXRAT( 647 ) = RKI( 647 ) * YC( PROPENE ) * YC( CL )
      RXRAT( 648 ) = RKI( 648 ) * YC( BUTADIENE13 ) * YC( CL )
      RXRAT( 649 ) = RKI( 649 ) * YC( ISOPRENE ) * YC( CL )
      RXRAT( 650 ) = RKI( 650 ) * YC( APIN ) * YC( CL )
      RXRAT( 651 ) = RKI( 651 ) * YC( ACETYLENE ) * YC( CL )
      RXRAT( 652 ) = RKI( 652 ) * YC( TOLUENE ) * YC( CL )
      RXRAT( 653 ) = RKI( 653 ) * YC( MXYL ) * YC( CL )
      RXRAT( 654 ) = RKI( 654 ) * YC( OXYL ) * YC( CL )
      RXRAT( 655 ) = RKI( 655 ) * YC( PXYL ) * YC( CL )
      RXRAT( 656 ) = RKI( 656 ) * YC( TRIMETH_BENZ124 ) * YC( CL )
      RXRAT( 657 ) = RKI( 657 ) * YC( ETOH ) * YC( CL )
      RXRAT( 658 ) = RKI( 658 ) * YC( ALK1 ) * YC( CL )
      RXRAT( 659 ) = RKI( 659 ) * YC( ALK2 ) * YC( CL )
      RXRAT( 660 ) = RKI( 660 ) * YC( ALK3 ) * YC( CL )
      RXRAT( 661 ) = RKI( 661 ) * YC( ALK4 ) * YC( CL )
      RXRAT( 662 ) = RKI( 662 ) * YC( ALK5 ) * YC( CL )
      RXRAT( 663 ) = RKI( 663 ) * YC( OLE1 ) * YC( CL )
      RXRAT( 664 ) = RKI( 664 ) * YC( OLE2 ) * YC( CL )
      RXRAT( 665 ) = RKI( 665 ) * YC( ARO1 ) * YC( CL )
      RXRAT( 666 ) = RKI( 666 ) * YC( ARO2 ) * YC( CL )
      RXRAT( 667 ) = RKI( 667 ) * YC( TERP ) * YC( CL )
      RXRAT( 668 ) = RKI( 668 ) * YC( SESQ ) * YC( CL )
      RXRAT( 669 ) = RKI( 669 ) * YC( BENZRO2 ) * YC( NO )
      RXRAT( 670 ) = RKI( 670 ) * YC( BENZRO2 ) * YC( HO2 )
      RXRAT( 671 ) = RKI( 671 ) * YC( XYLRO2 ) * YC( NO )
      RXRAT( 672 ) = RKI( 672 ) * YC( XYLRO2 ) * YC( HO2 )
      RXRAT( 673 ) = RKI( 673 ) * YC( TOLRO2 ) * YC( NO )
      RXRAT( 674 ) = RKI( 674 ) * YC( TOLRO2 ) * YC( HO2 )
      RXRAT( 675 ) = RKI( 675 ) * YC( HCHO_PRIMARY )
      RXRAT( 676 ) = RKI( 676 ) * YC( HCHO_PRIMARY )
      RXRAT( 677 ) = RKI( 677 ) * YC( HCHO_PRIMARY ) * YC( OH )
      RXRAT( 678 ) = RKI( 678 ) * YC( HCHO_PRIMARY ) * YC( NO3 )
      RXRAT( 679 ) = RKI( 679 ) * YC( HCHO_PRIMARY ) * YC( CL )
      RXRAT( 680 ) = RKI( 680 ) * YC( CCHO_PRIMARY ) * YC( OH )
      RXRAT( 681 ) = RKI( 681 ) * YC( CCHO_PRIMARY )
      RXRAT( 682 ) = RKI( 682 ) * YC( CCHO_PRIMARY ) * YC( NO3 )
      RXRAT( 683 ) = RKI( 683 ) * YC( CCHO_PRIMARY ) * YC( CL )
      RXRAT( 684 ) = RKI( 684 ) * YC( ACROLEIN_PRIMARY ) * YC( OH )
      RXRAT( 685 ) = RKI( 685 ) * YC( ACROLEIN_PRIMARY ) * YC( O3 )
      RXRAT( 686 ) = RKI( 686 ) * YC( ACROLEIN_PRIMARY ) * YC( NO3 )
      RXRAT( 687 ) = RKI( 687 ) * YC( ACROLEIN_PRIMARY ) * YC( O3P )
      RXRAT( 688 ) = RKI( 688 ) * YC( ACROLEIN_PRIMARY )
      RXRAT( 689 ) = RKI( 689 ) * YC( ACROLEIN_PRIMARY ) * YC( CL )


      RETURN

      END
