
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header$

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

      SUBROUTINE HRG2( DTC )
C**********************************************************************
C
C  FUNCTION: To solve for the concentration of HO, HO2, HONO, and
C            HNO4 alebraically.
C
C  PRECONDITIONS: For the SAPRC07TIC family of mechanisms
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, Oct. 18, 2012
C
C**********************************************************************
      USE HRDATA

      IMPLICIT NONE


C..INCLUDES: None


C..ARGUMENTS:
      REAL DTC                      ! Time step

C..PARAMETERS: None


C..EXTERNAL FUNCTIONS: NONE


C..SAVED LOCAL VARIABLES:
      CHARACTER( 16 ), SAVE :: PNAME = 'HRG2'             ! Program name


C..SCRATCH LOCAL VARIABLES:
      REAL    O1D_S                 ! sum of O1D loss frequencies
      REAL    OH_S                  ! stoich coeff for OH from O1D+H2O or H2
      REAL    HO2_S                 ! stoich coeff for HO2 from O1D+H2 rxn
      REAL    EXN_S                 ! sum of NO2EX loss frequencies
      REAL    XOH_S                 ! stoich coeff for OH & HONO from NO2EX loss rxn
      REAL    R4_19                 ! production of OH from HONO
      REAL    R19_4                 ! production of HONO from OH
      REAL    R4_5                  ! production of OH from HO2
      REAL    R5_4                  ! production of HO2 from OH
      REAL    R5_21                 ! production of HO2 from HNO4
      REAL    R21_5                 ! production of HNO4 from HO2
      REAL    P4, P5, P19, P21      ! species production form other paths
      REAL    L4, L5, L19, L21      ! species loss terms

      REAL    A, B, C               ! coeffs of quadratic eq. for HO2
      REAL    Q                     ! intermediate term

      REAL    T1, T2, T3            ! intermediate terms

      REAL    L21_INV               ! reciprocal of HNO4 loss term

C**********************************************************************


c..stoichiometric coefficient for production of HOX from O3 via O1D
      O1D_S =           RKI(  20 )                            ! O1D=2*OH
     &       +          RKI(  21 )                            ! O1D=O3P

      OH_S =   2.000 * RKI(  20 )                            ! O1D=2*OH

      OH_S  = OH_S / O1D_S


c..Production of HO from HO2 (r4,5 terms )
      R4_5 =           RKI(  31 ) * YC( NO               )   ! HO2+NO=OH+NO2
     &      +          RKI(  36 ) * YC( O3               )   ! HO2+O3=OH
     &      +  0.800 * RKI(  39 ) * YC( NO3              )   ! HO2+NO3=0.8*OH+...
     &      +          RKI( 563 ) * YC( CL               )   ! HO2+CL=OH+CLO
     &      +  0.120 * RKI( 677 ) * YC( ISOPO2           )   ! HO2+ISOPO2=...
     &      +  0.175 * RKI( 781 ) * YC( IEPOXOO          )   ! HO2+IEPOXOO=...

      R4_5  = R4_5 * DTC

c..Production of HO from HONO (r4,19 terms )
      R4_19 =           RKI(  23 )                            ! HONO=OH+NO

      R4_19 = R4_19 * DTC

c..Remaining HO production
      P4 =   OH_S  * RXRAT(  18 )      ! O1D=2*OH
     &    +          RXRAT(  28 )      ! HNO3=OH+NO2
     &    +  0.390 * RXRAT(  34 )      ! HNO4=0.39*OH+0.61*HO2+...
     &    +  2.000 * RXRAT(  41 )      ! HO2H=2*OH
     &    +          RXRAT(  49 )      ! NO2EX=OH+HONO
     &    +          RXRAT( 118 )      ! xOH+NO=OH+NO
     &    +          RXRAT( 120 )      ! xOH+NO3=OH+NO3
     &    +  0.500 * RXRAT( 121 )      ! xOH+MEO2=0.5*OH+MEO2
     &    +  0.500 * RXRAT( 122 )      ! xOH+RO2C=0.5*OH+RO2C
     &    +  0.500 * RXRAT( 123 )      ! xOH+RO2XC=0.5*OH+RO2XC
     &    +          RXRAT( 124 )      ! xOH+MECO3=OH+MECO3
     &    +          RXRAT( 125 )      ! xOH+RCO3=OH+RCO3
     &    +          RXRAT( 126 )      ! xOH+BZCO3=OH+BZCO3
     &    +          RXRAT( 127 )      ! xOH+MACO3=OH+MACO3
     &    +          RXRAT( 217 )      ! COOH=OH+HO2+HCHO
     &    +          RXRAT( 219 )      ! ROOH=OH+HO2+RCHO
     &    +          RXRAT( 221 )      ! R6OOH=OH+0.142*HO2+...
     &    +          RXRAT( 223 )      ! RAOOH=OH+HO2+0.5*GLY+...
     &    +  0.826 * RXRAT( 241 )      ! AFG1+O3=0.826*OH+0.522*HO2+...
     &    +  0.826 * RXRAT( 244 )      ! AFG2+O3=0.826*OH+0.522*HO2+...
     &    +  0.471 * RXRAT( 247 )      ! AFG3+O3=0.471*OH+0.554*HO2+...
     &    +  0.208 * RXRAT( 248 )      ! MACR+O3=0.208*OH+0.108*HO2+...
     &    +  0.330 * RXRAT( 251 )      ! MACR=0.33*OH+0.67*HO2+...
     &    +  0.164 * RXRAT( 252 )      ! MVK+O3=0.164*OH+0.064*HO2+...
     &    +  0.285 * RXRAT( 256 )      ! IPRD+O3=0.285*OH+0.4*HO2+...
     &    +  0.330 * RXRAT( 266 )      ! ACROLEIN+O3=0.33*OH+...
     &    +  0.178 * RXRAT( 269 )      ! ACROLEIN=0.178*OH+...
     &    +          RXRAT( 271 )      ! CCOOOH=OH+CO2+MEO2
     &    +          RXRAT( 273 )      ! RCOOOH=OH+xCCHO+yROOH+CO2+xHO2
     &    +  0.160 * RXRAT( 506 )      ! ETHENE+O3=0.16*OH+0.16*HO2+...
     &    +  0.350 * RXRAT( 510 )      ! PROPENE+O3=0.35*OH+...
     &    +  0.080 * RXRAT( 514 )      ! BUTADIENE13+O3=0.08*OH+...
     &    +  0.266 * RXRAT( 517 )      ! ISOPRENE+O3=0.266*OH+...
     &    +  0.728 * RXRAT( 520 )      ! APIN+O3=0.728*OH+0.009*HO2+...
     &    +  0.500 * RXRAT( 524 )      ! ACETYLENE+O3=0.5*OH+...
     &    +  0.128 * RXRAT( 538 )      ! OLE1+O3=0.128*OH+0.095*HO2+...
     &    +  0.443 * RXRAT( 542 )      ! OLE2+O3=0.443*OH+0.094*HO2+...
     &    +  0.499 * RXRAT( 548 )      ! TERP+O3=0.499*OH+0.078*HO2+...
     &    +  0.499 * RXRAT( 552 )      ! SESQ+O3=0.499*OH+0.078*HO2+...
     &    +          RXRAT( 573 )      ! HOCL=OH+CL
     &    +  0.414 * RXRAT( 589 )      ! ROOH+CL=0.414*OH+HCL+...
     &    +  0.145 * RXRAT( 590 )      ! R6OOH+CL=0.145*OH+HCL+...
     &    +  0.139 * RXRAT( 591 )      ! RAOOH+CL=0.139*OH+...
     &    +          RXRAT( 683 )      ! HPALD=OH+HO2+0.5*HACET+...
     &    +  0.790 * RXRAT( 699 )      ! HC5+O3=0.79*OH+0.6*HO2+...
     &    +  0.730 * RXRAT( 706 )      ! ISOPND+O3=0.73*OH+0.39*HO2+...
     &    +  0.340 * RXRAT( 713 )      ! ISOPNB+O3=0.34*OH+0.08*HO2+...
     &    +  0.850 * RXRAT( 728 )      ! NIT1+O3=0.15*OH+0.45*HO2+...
     &    +  0.950 * RXRAT( 781 )      ! HO2+IEPOXOO=1.125*OH+...
     &    +  0.125 * RXRAT( 782 )      ! IEPOXOO+NO=0.125*OH+...
     &    +  0.063 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.063*OH+...
     &    +  0.063 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.063*OH+...
     &    +  0.125 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.125*OH+...
     &    +          RXRAT( 786 )      ! ISOPOOH=OH+0.91*HO2+...

      P4 = YC0( OH ) + P4 * DTC

c..Production of HO2 from OH ( r5,4 terms )
      R5_4 =           RKI(  26 ) * YC( NO3              )   ! OH+NO3=HO2+NO2
     &      +          RKI(  29 ) * YC( CO               )   ! OH+CO=HO2+CO2
     &      +          RKI(  30 ) * YC( O3               )   ! OH+O3=HO2
     &      +          RKI(  42 ) * YC( HO2H             )   ! OH+HO2H=HO2
     &      +          RKI(  44 ) * YC( SO2              )   ! OH+SO2=HO2+SULF+...
     &      +          RKI(  45 )                            ! OH=HO2
     &      +          RKI( 200 ) * YC( HCHO             )   ! OH+HCHO=HO2+CO
     &      +          RKI( 212 ) * YC( MEOH             )   ! OH+MEOH=HO2+HCHO
     &      +          RKI( 213 ) * YC( HCOOH            )   ! OH+HCOOH=HO2+CO2
     &      +  0.148 * RKI( 222 ) * YC( RAOOH            )   ! OH+RAOOH=...
     &      +  0.630 * RKI( 226 ) * YC( GLY              )   ! OH+GLY=0.63*HO2+...
     &      +  0.472 * RKI( 259 ) * YC( PRD2             )   ! OH+PRD2=...
     &      +  0.189 * RKI( 261 ) * YC( RNO3             )   ! OH+RNO3=...
     &      +  0.300 * RKI( 523 ) * YC( ACETYLENE        )   ! OH+ACETYLENE=...
     &      +  0.570 * RKI( 525 ) * YC( BENZENE          )   ! OH+BENZENE=...
     &      +  0.181 * RKI( 526 ) * YC( TOLUENE          )   ! OH+TOLUENE=...
     &      +  0.159 * RKI( 527 ) * YC( MXYL             )   ! OH+MXYL=...
     &      +  0.161 * RKI( 528 ) * YC( OXYL             )   ! OH+OXYL=...
     &      +  0.159 * RKI( 529 ) * YC( PXYL             )   ! OH+PXYL=...
     &      +  0.022 * RKI( 530 ) * YC( TRIMETH_BENZ124  )   ! OH+...
     &      +  0.950 * RKI( 531 ) * YC( ETOH             )   ! OH+ETOH=0.95*HO2+...
     &      +  0.123 * RKI( 545 ) * YC( ARO1             )   ! OH+ARO1=...
     &      +  0.077 * RKI( 546 ) * YC( ARO2             )   ! OH+ARO2=...
     &      +  0.750 * RKI( 763 ) * YC( HOCCHO           )   ! OH+HOCCHO=...
     &      +  0.825 * RKI( 764 ) * YC( HACET            )   ! OH+HACET=...
     &      +  0.500 * RKI( 776 ) * YC( DHMOB            )   ! OH+DHMOB=0.5*HO2+...
     &      +          RKI( 787 ) * YC( RNO3I            )   ! OH+RNO3I=HO2+NO2+...

      R5_4  = R5_4 * DTC

c..Production of HO2 from HNO4 (r5,21 term )
      R5_21 =           RKI(  33 )                            ! HNO4=HO2+NO2
     &       +  0.610 * RKI(  34 )                            ! HNO4=0.61*HO2+...

      R5_21 = R5_21 * DTC

c..Remaining HO2 production terms
      P5 =           RXRAT(  50 )      ! MEO2+NO=HO2+HCHO+NO2
     &    +          RXRAT(  53 )      ! MEO2+NO3=HO2+HCHO+NO2
     &    +  2.000 * RXRAT(  55 )      ! MEO2+MEO2=2*HO2+2*HCHO
     &    +  0.500 * RXRAT(  59 )      ! RO2C+MEO2=0.5*HO2+...
     &    +  0.500 * RXRAT(  64 )      ! RO2XC+MEO2=0.5*HO2+...
     &    +  0.900 * RXRAT(  73 )      ! MECO3+MEO2=0.9*HO2+...
     &    +          RXRAT(  83 )      ! RCO3+MEO2=HO2+HCHO+RO2C+...
     &    +          RXRAT(  94 )      ! BZCO3+MEO2=HO2+HCHO+RO2C+...
     &    +          RXRAT( 108 )      ! xHO2+NO=HO2+NO
     &    +          RXRAT( 110 )      ! xHO2+NO3=HO2+NO3
     &    +  0.500 * RXRAT( 111 )      ! xHO2+MEO2=0.5*HO2+MEO2
     &    +  0.500 * RXRAT( 112 )      ! xHO2+RO2C=0.5*HO2+RO2C
     &    +  0.500 * RXRAT( 113 )      ! xHO2+RO2XC=0.5*HO2+RO2XC
     &    +          RXRAT( 114 )      ! xHO2+MECO3=HO2+MECO3
     &    +          RXRAT( 115 )      ! xHO2+RCO3=HO2+RCO3
     &    +          RXRAT( 116 )      ! xHO2+BZCO3=HO2+BZCO3
     &    +          RXRAT( 117 )      ! xHO2+MACO3=HO2+MACO3
     &    +  2.000 * RXRAT( 198 )      ! HCHO=2*HO2+CO
     &    +          RXRAT( 201 )      ! HCHO+NO3=HO2+HNO3+CO
     &    +          RXRAT( 203 )      ! CCHO=HO2+CO+MEO2
     &    +          RXRAT( 206 )      ! RCHO=HO2+xHO2+yROOH+xCCHO+...
     &    +          RXRAT( 217 )      ! COOH=HO2+OH+HCHO
     &    +          RXRAT( 219 )      ! ROOH=HO2+OH+RCHO
     &    +  0.142 * RXRAT( 221 )      ! R6OOH=0.142*HO2+OH+...
     &    +          RXRAT( 223 )      ! RAOOH=HO2+OH+0.5*GLY+...
     &    +  2.000 * RXRAT( 224 )      ! GLY=2*HO2+2*CO
     &    +  0.630 * RXRAT( 227 )      ! GLY+NO3=0.63*HO2+HNO3+...
     &    +          RXRAT( 228 )      ! MGLY=HO2+CO+MECO3
     &    +  0.522 * RXRAT( 241 )      ! AFG1+O3=0.522*HO2+0.826*OH+...
     &    +  1.023 * RXRAT( 242 )      ! AFG1=1.023*HO2+0.173*MEO2+...
     &    +  0.522 * RXRAT( 244 )      ! AFG2+O3=0.522*HO2+0.826*OH+...
     &    +  0.554 * RXRAT( 247 )      ! AFG3+O3=0.554*HO2+0.471*OH+...
     &    +  0.108 * RXRAT( 248 )      ! MACR+O3=0.108*HO2+0.208*OH+...
     &    +  0.670 * RXRAT( 251 )      ! MACR=0.67*HO2+0.33*OH+...
     &    +  0.064 * RXRAT( 252 )      ! MVK+O3=0.064*HO2+0.164*OH+...
     &    +  0.400 * RXRAT( 256 )      ! IPRD+O3=0.4*HO2+0.285*OH+...
     &    +  1.233 * RXRAT( 258 )      ! IPRD=1.233*HO2+0.467*MECO3+...
     &    +  0.344 * RXRAT( 262 )      ! RNO3=0.344*HO2+0.554*xHO2+...
     &    +  2.000 * RXRAT( 263 )      ! HOCCHO=2*HO2+CO+HCHO
     &    +  0.830 * RXRAT( 266 )      ! ACROLEIN+O3=0.83*HO2+...
     &    +  1.066 * RXRAT( 269 )      ! ACROLEIN=1.066*HO2+...
     &    +          RXRAT( 476 )      ! zRNO3+NO3=HO2+PRD2+NO3
     &    +  0.500 * RXRAT( 477 )      ! zRNO3+MEO2=0.5*HO2+...
     &    +  0.500 * RXRAT( 478 )      ! zRNO3+RO2C=0.5*HO2+...
     &    +  0.500 * RXRAT( 479 )      ! zRNO3+RO2XC=0.5*HO2+...
     &    +          RXRAT( 480 )      ! zRNO3+MECO3=HO2+PRD2+MECO3
     &    +          RXRAT( 481 )      ! zRNO3+RCO3=HO2+PRD2+RCO3
     &    +          RXRAT( 482 )      ! zRNO3+BZCO3=HO2+PRD2+BZCO3
     &    +          RXRAT( 483 )      ! zRNO3+MACO3=HO2+PRD2+MACO3
     &    +  0.160 * RXRAT( 506 )      ! ETHENE+O3=0.16*HO2+0.16*OH+...
     &    +  0.800 * RXRAT( 508 )      ! ETHENE+O3P=0.8*HO2+...
     &    +  0.165 * RXRAT( 510 )      ! PROPENE+O3=0.165*HO2+...
     &    +  0.080 * RXRAT( 514 )      ! BUTADIENE13+O3=0.08*HO2+...
     &    +  0.250 * RXRAT( 516 )      ! BUTADIENE13+O3P=0.25*HO2+...
     &    +  0.066 * RXRAT( 517 )      ! ISOPRENE+O3=0.066*HO2+...
     &    +  0.009 * RXRAT( 520 )      ! APIN+O3=0.009*HO2+0.728*OH+...
     &    +  1.500 * RXRAT( 524 )      ! ACETYLENE+O3=1.5*HO2+...
     &    +  0.095 * RXRAT( 538 )      ! OLE1+O3=0.095*HO2+0.128*OH+...
     &    +  0.094 * RXRAT( 542 )      ! OLE2+O3=0.094*HO2+0.443*OH+...
     &    +  0.078 * RXRAT( 548 )      ! TERP+O3=0.078*HO2+0.499*OH+...
     &    +  0.078 * RXRAT( 552 )      ! SESQ+O3=0.078*HO2+0.499*OH+...
     &    +          RXRAT( 576 )      ! CL=HO2+HCL
     &    +          RXRAT( 577 )      ! HCHO+CL=HO2+HCL+CO
     &    +          RXRAT( 579 )      ! MEOH+CL=HO2+HCHO+HCL
     &    +  0.055 * RXRAT( 583 )      ! RNO3+CL=0.055*HO2+...
     &    +  0.314 * RXRAT( 584 )      ! PRD2+CL=0.314*HO2+HCL+...
     &    +  0.630 * RXRAT( 585 )      ! GLY+CL=0.63*HO2+HCL+...
     &    +  0.148 * RXRAT( 591 )      ! RAOOH+CL=0.148*HO2+...
     &    +  0.084 * RXRAT( 595 )      ! IPRD+CL=0.084*HO2+...
     &    +          RXRAT( 596 )      ! CLCCHO=HO2+CO+RO2C+xCL+...
     &    +          RXRAT( 636 )      ! ACETYLENE+CL=HO2+CO
     &    +  0.688 * RXRAT( 642 )      ! ETOH+CL=0.688*HO2+HCL+...
     &    +  0.803 * RXRAT( 676 )      ! ISOPO2+NO=0.803*HO2+...
     &    +  0.950 * RXRAT( 678 )      ! ISOPO2+MEO2=0.5*HO2+...
     &    +  0.450 * RXRAT( 679 )      ! ISOPO2+RO2C=0.45*HO2+...
     &    +  0.910 * RXRAT( 680 )      ! ISOPO2+ISOPO2=0.91*HO2+...
     &    +  0.910 * RXRAT( 681 )      ! ISOPO2+MECO3=0.91*HO2+CO2+...
     &    +          RXRAT( 682 )      ! ISOPO2=HO2+HPALD
     &    +          RXRAT( 683 )      ! HPALD=HO2+OH+0.5*HACET+...
     &    +  0.800 * RXRAT( 686 )      ! NISOPO2+NO3=0.8*HO2+...
     &    +  0.800 * RXRAT( 687 )      ! NISOPO2+NO=0.8*HO2+...
     &    +  0.900 * RXRAT( 689 )      ! NISOPO2+MEO2=0.4*HO2+...
     &    +  0.400 * RXRAT( 690 )      ! NISOPO2+RO2C=0.4*HO2+...
     &    +  0.800 * RXRAT( 691 )      ! NISOPO2+NISOPO2=0.8*HO2+...
     &    +  0.800 * RXRAT( 692 )      ! NISOPO2+MECO3=0.8*HO2+CO2+...
     &    +          RXRAT( 694 )      ! HC5OO+NO=HO2+0.234*HOCCHO+...
     &    +          RXRAT( 696 )      ! HC5OO+MEO2=0.5*HO2+...
     &    +  0.500 * RXRAT( 697 )      ! HC5OO+RO2C=0.5*HO2+...
     &    +          RXRAT( 698 )      ! HC5OO+MECO3=HO2+CO2+...
     &    +  0.600 * RXRAT( 699 )      ! HC5+O3=0.6*HO2+0.79*OH+...
     &    +  0.350 * RXRAT( 701 )      ! ISOPNOOD+NO=0.35*HO2+...
     &    +  0.675 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.175*HO2+...
     &    +  0.175 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.175*HO2+...
     &    +  0.350 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.35*HO2+...
     &    +  0.390 * RXRAT( 706 )      ! ISOPND+O3=0.39*HO2+0.73*OH+...
     &    +  0.400 * RXRAT( 708 )      ! ISOPNOOB+NO=0.4*HO2+...
     &    +  0.700 * RXRAT( 710 )      ! ISOPNOOB+MEO2=0.2*HO2+...
     &    +  0.200 * RXRAT( 711 )      ! ISOPNOOB+RO2C=0.2*HO2+...
     &    +  0.400 * RXRAT( 712 )      ! ISOPNOOB+MECO3=0.4*HO2+CO2+...
     &    +  0.080 * RXRAT( 713 )      ! ISOPNB+O3=0.08*HO2+0.34*OH+...
     &    +          RXRAT( 715 )      ! NIT1NO3OOA+NO3=HO2+PROPNNB+...
     &    +          RXRAT( 716 )      ! NIT1NO3OOA+NO=HO2+PROPNNB+...
     &    +          RXRAT( 719 )      ! NIT1NO3OOA+RO2C=HO2+CO+CO2+...
     &    +  2.000 * RXRAT( 720 )      ! NIT1NO3OOA+MEO2=HO2+CO+CO2+...
     &    +          RXRAT( 721 )      ! NIT1NO3OOA+MECO3=HO2+CO2+...
     &    +  0.500 * RXRAT( 726 )      ! NIT1NO3OOB+MEO2=0.5*HO2+...
     &    +  0.450 * RXRAT( 728 )      ! NIT1+O3=0.45*HO2+0.15*OH+...
     &    +  0.934 * RXRAT( 730 )      ! NIT1OHOO+NO=0.934*HO2+...
     &    +  0.700 * RXRAT( 732 )      ! NIT1OHOO+RO2C=0.7*HO2+...
     &    +  1.200 * RXRAT( 733 )      ! NIT1OHOO+MEO2=0.7*HO2+...
     &    +          RXRAT( 734 )      ! NIT1OHOO+MECO3=HO2+CO2+...
     &    +          RXRAT( 735 )      ! DIBOO+NO=HO2+NO2+...
     &    +          RXRAT( 737 )      ! DIBOO+MEO2=0.5*HO2+...
     &    +  0.500 * RXRAT( 738 )      ! DIBOO+RO2C=0.5*HO2+...
     &    +          RXRAT( 739 )      ! DIBOO+MECO3=HO2+...
     &    +  0.265 * RXRAT( 741 )      ! MVKOO+NO=0.265*HO2+...
     &    +  0.650 * RXRAT( 743 )      ! MVKOO+MEO2=0.15*HO2+...
     &    +  0.150 * RXRAT( 744 )      ! MVKOO+RO2C=0.15*HO2+...
     &    +  0.300 * RXRAT( 745 )      ! MVKOO+MECO3=0.3*HO2+CO2+...
     &    +  0.850 * RXRAT( 747 )      ! MACROO+NO=0.85*HO2+...
     &    +          RXRAT( 749 )      ! MACROO+MEO2=0.5*HO2+...
     &    +  0.500 * RXRAT( 750 )      ! MACROO+RO2C=0.5*HO2+...
     &    +          RXRAT( 751 )      ! MACROO+MECO3=HO2+CO2+MEO2+...
     &    +          RXRAT( 755 )      ! MACO3+MEO2=HO2+HCHO+CO+CO2+...
     &    +          RXRAT( 765 )      ! HACET=HO2+MECO3+HCHO
     &    +          RXRAT( 767 )      ! ETHLN=HO2+HCHO+NO2+CO
     &    +          RXRAT( 775 )      ! MACRN=HO2+NO2+CO+HACET
     &    +  0.825 * RXRAT( 782 )      ! IEPOXOO+NO=0.825*HO2+...
     &    +  0.913 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.413*HO2+...
     &    +  0.413 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.413*HO2+...
     &    +  0.825 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.825*HO2+...
     &    +  0.910 * RXRAT( 786 )      ! ISOPOOH=0.91*HO2+OH+...

      P5 = YC0( HO2 ) + P5 * DTC

c..Production of HONO from OH (r19,4 terms )

      R19_4 =   RKI(  22 ) * YC( NO               ) * DTC      ! OH+NO=HONO

c..Remaining HONO production terms
      P19 =           RXRAT(  49 )      ! NO2EX=HONO+OH
     &     +          RXRAT( 235 )      ! NPHE=HONO

      P19 = YC0( HONO ) + P19 * DTC

c..Production of HNO4 from HO2 (r21,5 term )

      R21_5 =   RKI(  32 ) * YC( NO2              ) * DTC      ! HO2+NO2=HNO4

c..Remaining HNO4 production terms
      P21   =   YC0( HNO4 )

c..OH loss terms not in R5_4 & R19_4
      L4 =           RKI(  24 ) * YC( HONO             )   ! OH+HONO=NO2
     &    +          RKI(  25 ) * YC( NO2              )   ! OH+NO2=HNO3
     &    +          RKI(  27 ) * YC( HNO3             )   ! OH+HNO3=NO3
     &    +          RKI(  35 ) * YC( HNO4             )   ! OH+HNO4=NO2
     &    +          RKI(  43 ) * YC( HO2              )   ! OH+HO2=
     &    +          RKI( 202 ) * YC( CCHO             )   ! OH+CCHO=MECO3
     &    +          RKI( 205 ) * YC( RCHO             )   ! OH+RCHO=...
     &    +          RKI( 208 ) * YC( ACETONE          )   ! OH+ACETONE=RO2C+...
     &    +          RKI( 210 ) * YC( MEK              )   ! OH+MEK=...
     &    +          RKI( 214 ) * YC( CCOOH            )   ! OH+CCOOH=...
     &    +          RKI( 215 ) * YC( RCOOH            )   ! OH+RCOOH=RO2C+...
     &    +  0.700 * RKI( 216 ) * YC( COOH             )   ! OH+COOH=0.3*HCHO+...
     &    +  0.256 * RKI( 218 ) * YC( ROOH             )   ! OH+ROOH=0.744*OH+...
     &    +  0.160 * RKI( 220 ) * YC( R6OOH            )   ! OH+R6OOH=0.84*OH+...
     &    +  0.713 * RKI( 222 ) * YC( RAOOH            )   ! OH+RAOOH=...
     &    +  0.370 * RKI( 226 ) * YC( GLY              )   ! OH+GLY=0.63*HO2+...
     &    +          RKI( 229 ) * YC( MGLY             )   ! OH+MGLY=CO+MECO3
     &    +          RKI( 232 ) * YC( CRES             )   ! OH+CRES=0.2*BZO+...
     &    +          RKI( 234 ) * YC( NPHE             )   ! OH+NPHE=BZO
     &    +          RKI( 237 ) * YC( BALD             )   ! OH+BALD=BZCO3
     &    +          RKI( 240 ) * YC( AFG1             )   ! OH+AFG1=...
     &    +          RKI( 243 ) * YC( AFG2             )   ! OH+AFG2=...
     &    +          RKI( 246 ) * YC( AFG3             )   ! OH+AFG3=...
     &    +          RKI( 255 ) * YC( IPRD             )   ! OH+IPRD=...
     &    +  0.528 * RKI( 259 ) * YC( PRD2             )   ! OH+PRD2=...
     &    +  0.811 * RKI( 261 ) * YC( RNO3             )   ! OH+RNO3=...
     &    +          RKI( 265 ) * YC( ACROLEIN         )   ! OH+ACROLEIN=...
     &    +          RKI( 270 ) * YC( CCOOOH           )   ! OH+CCOOOH=...
     &    +          RKI( 272 ) * YC( RCOOOH           )   ! OH+RCOOOH=...
     &    +          RKI( 504 )                            ! OH=MEO2
     &    +          RKI( 505 ) * YC( ETHENE           )   ! OH+ETHENE=xHO2+...
     &    +          RKI( 509 ) * YC( PROPENE          )   ! OH+PROPENE=...
     &    +          RKI( 513 ) * YC( BUTADIENE13      )   ! OH+BUTADIENE13=...
     &    +          RKI( 519 ) * YC( APIN             )   ! OH+APIN=...
     &    +  0.314 * RKI( 525 ) * YC( BENZENE          )   ! OH+BENZENE=...
     &    +  0.507 * RKI( 526 ) * YC( TOLUENE          )   ! OH+TOLUENE=...
     &    +  0.602 * RKI( 527 ) * YC( MXYL             )   ! OH+MXYL=...
     &    +  0.641 * RKI( 528 ) * YC( OXYL             )   ! OH+OXYL=...
     &    +  0.563 * RKI( 529 ) * YC( PXYL             )   ! OH+PXYL=...
     &    +  0.748 * RKI( 530 ) * YC( TRIMETH_BENZ124  )   ! OH+...
     &    +  0.050 * RKI( 531 ) * YC( ETOH             )   ! OH+ETOH=0.95*HO2+...
     &    +          RKI( 532 ) * YC( ALK1             )   ! OH+ALK1=xHO2+...
     &    +          RKI( 533 ) * YC( ALK2             )   ! OH+ALK2=...
     &    +          RKI( 534 ) * YC( ALK3             )   ! OH+ALK3=...
     &    +          RKI( 535 ) * YC( ALK4             )   ! OH+ALK4=...
     &    +          RKI( 536 ) * YC( ALK5             )   ! OH+ALK5=...
     &    +          RKI( 537 ) * YC( OLE1             )   ! OH+OLE1=...
     &    +          RKI( 541 ) * YC( OLE2             )   ! OH+OLE2=...
     &    +  0.675 * RKI( 545 ) * YC( ARO1             )   ! OH+ARO1=...
     &    +  0.745 * RKI( 546 ) * YC( ARO2             )   ! OH+ARO2=...
     &    +          RKI( 547 ) * YC( TERP             )   ! OH+TERP=...
     &    +          RKI( 551 ) * YC( SESQ             )   ! OH+SESQ=...
     &    +          RKI( 575 ) * YC( HCL              )   ! OH+HCL=CL
     &    +          RKI( 597 ) * YC( CLCCHO           )   ! OH+CLCCHO=RCO3
     &    +          RKI( 675 ) * YC( ISOPRENE         )   ! OH+ISOPRENE=...
     &    +          RKI( 693 ) * YC( HC5              )   ! OH+HC5=HC5OO
     &    +          RKI( 700 ) * YC( ISOPND           )   ! OH+ISOPND=ISOPNOOD
     &    +          RKI( 707 ) * YC( ISOPNB           )   ! OH+ISOPNB=ISOPNOOB
     &    +          RKI( 729 ) * YC( NIT1             )   ! OH+NIT1=...
     &    +          RKI( 740 ) * YC( MVK              )   ! OH+MVK=MVKOO
     &    +          RKI( 746 ) * YC( MACR             )   ! OH+MACR=...
     &    +          RKI( 762 ) * YC( MAPAN            )   ! OH+MAPAN=HACET+...
     &    +  0.075 * RKI( 764 ) * YC( HACET            )   ! OH+HACET=...
     &    +          RKI( 766 ) * YC( ETHLN            )   ! OH+ETHLN=HCHO+...
     &    +          RKI( 768 ) * YC( PROPNN           )   ! OH+PROPNN=MGLY+NO2
     &    +          RKI( 769 ) * YC( PROPNNB          )   ! OH+PROPNNB=MGLY+NO2
     &    +          RKI( 772 ) * YC( MVKN             )   ! OH+MVKN=...
     &    +          RKI( 774 ) * YC( MACRN            )   ! OH+MACRN=...
     &    +  0.500 * RKI( 776 ) * YC( DHMOB            )   ! OH+DHMOB=0.5*HO2+...
     &    +  0.387 * RKI( 779 ) * YC( ISOPOOH          )   ! OH+ISOPOOH=...
     &    +          RKI( 780 ) * YC( IEPOX            )   ! OH+IEPOX=IEPOXOO
     &    +  0.300 * RKI( 789 ) * YC( NISOPOOH         )   ! OH+NISOPOOH=...

      L4    = 1.0 + L4 * DTC + R5_4 + R19_4

c..HO2 loss terms not included in R4_5 & R21_5 (except for HO2+HO2 )
      L5 =   0.200 * RKI(  39 ) * YC( NO3              )   ! HO2+NO3=0.8*OH+...
     &    +          RKI(  43 ) * YC( OH               )   ! HO2+OH=
     &    +          RKI(  51 ) * YC( MEO2             )   ! HO2+MEO2=COOH
     &    +          RKI(  52 ) * YC( MEO2             )   ! HO2+MEO2=HCHO
     &    +          RKI(  57 ) * YC( RO2C             )   ! HO2+RO2C=
     &    +          RKI(  62 ) * YC( RO2XC            )   ! HO2+RO2XC=
     &    +          RKI(  71 ) * YC( MECO3            )   ! HO2+MECO3=...
     &    +          RKI(  81 ) * YC( RCO3             )   ! HO2+RCO3=...
     &    +          RKI(  92 ) * YC( BZCO3            )   ! HO2+BZCO3=...
     &    +          RKI( 106 ) * YC( BZO              )   ! HO2+BZO=CRES
     &    +          RKI( 562 ) * YC( CL               )   ! HO2+CL=HCL
     &    +          RKI( 572 ) * YC( CLO              )   ! HO2+CLO=HOCL
     &    +  0.760 * RKI( 677 ) * YC( ISOPO2           )   ! HO2+ISOPO2=...
     &    +          RKI( 688 ) * YC( NISOPO2          )   ! HO2+NISOPO2=NISOPOOH
     &    +          RKI( 695 ) * YC( HC5OO            )   ! HO2+HC5OO=R6OOH
     &    +          RKI( 702 ) * YC( ISOPNOOD         )   ! HO2+ISOPNOOD=RNO3I
     &    +          RKI( 709 ) * YC( ISOPNOOB         )   ! HO2+ISOPNOOB=RNO3I
     &    +          RKI( 718 ) * YC( NIT1NO3OOA       )   ! HO2+NIT1NO3OOA=...
     &    +          RKI( 724 ) * YC( NIT1NO3OOB       )   ! HO2+NIT1NO3OOB=RNO3I
     &    +          RKI( 731 ) * YC( NIT1OHOO         )   ! HO2+NIT1OHOO=R6OOH
     &    +          RKI( 736 ) * YC( DIBOO            )   ! HO2+DIBOO=R6OOH
     &    +          RKI( 742 ) * YC( MVKOO            )   ! HO2+MVKOO=ROOH
     &    +          RKI( 748 ) * YC( MACROO           )   ! HO2+MACROO=ROOH
     &    +          RKI( 753 ) * YC( MACO3            )   ! HO2+MACO3=...

      L5    = 1.0 + L5 * DTC + R4_5 + R21_5

c..HONO loss terms not included in R4_19
      L19 =           RKI(  24 ) * YC( OH               )   ! HONO+OH=NO2

      L19   = 1.0 + L19 * DTC + R4_19

c..HNO4 loss terms not inluded in R5_21
      L21 =   0.390 * RKI(  34 )                            ! HNO4=0.61*HO2+...
     &     +          RKI(  35 ) * YC( OH               )   ! HNO4+OH=NO2

      L21   = 1.0 + L21 * DTC + R5_21

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Solution section
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c..compute terms used to calculate a,b & c
      L21_INV = 1.0 / L21
      T1 = 1.0 / ( L4 * L19 - R4_19 * R19_4 )
      T2 = R5_4 * T1
      T3 = R5_21 * L21_INV

c..solve quadratic equation for HO2
      A = 2.0 * ( RKI(  37 ) + RKI(  38 ) ) * DTC

      B = L5 - T3 * R21_5 - T2 * R4_5 * L19

      C = P5 + T3 * P21 + T2 * ( P4 * L19 + P19 * R4_19 )

      Q = -0.5 * ( B + SIGN( 1.0, B ) * SQRT( B * B + 4.0 * A * C ) )

      YCP( HO2 ) = MAX( Q / A , -C / Q  )

c..compute remaining species concentrations
      YCP( OH ) = ( ( P4 + R4_5 * YCP( HO2 ) ) * L19 + R4_19 * P19 ) * T1

      YCP( HNO4 ) = ( P21 + R21_5 * YCP( HO2 ) ) * L21_INV

      YCP( HONO ) = ( P19 + R19_4 * YCP( OH ) ) / L19

      RETURN

      END


