
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header$

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

      SUBROUTINE HRPRODLOSS

C***********************************************************************
C
C  FUNCTION: To compute the production and loss rates for EBI species
C
C  PRECONDITIONS: For the CB05TUMP family of mechanisms
C
C  RETURN VALUES: None
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, July 16, 2012
C
C***********************************************************************
      USE HRDATA

      IMPLICIT NONE


C.....INCLUDES: NONE


C.....ARGUMENTS: NONE


C.....PARAMETERS: NONE


C.....EXTERNAL FUNCTIONS: NONE


C.....SAVED VARIABLES: NONE
      CHARACTER( 16 ), SAVE  ::  PNAME = 'HRPRODLOSS'   ! Program name


C.....LOCAL VARIABLES:

      INTEGER ESP

C***********************************************************************


      PROD( HNO3             ) =   2.000 * RXRAT(  19 )      ! N2O5=2*HNO3
     &                          +  2.000 * RXRAT(  20 )      ! N2O5=2*HNO3
     &                          +          RXRAT(  28 )      ! NO2+OH=HNO3
     &                          +          RXRAT(  48 )      ! NO3+HO2=HNO3
     &                          +          RXRAT(  61 )      ! NTR+OH=HNO3+HO2+0.33*FORM+...
     &                          +          RXRAT(  77 )      ! FORM+NO3=HNO3+HO2+CO
     &                          +          RXRAT(  85 )      ! ALD2+NO3=HNO3+C2O3
     &                          +          RXRAT( 100 )      ! ALDX+NO3=HNO3+CXO3
     &                          +          RXRAT( 132 )      ! CRES+NO3=HNO3+0.3*CRO+...
     &                          +          RXRAT( 136 )      ! CRON+NO3=HNO3+CRNO
     &                          +          RXRAT( 146 )      ! OPEN+NO3=HNO3+OPO3
     &                          +          RXRAT( 148 )      ! CAT1+NO3=HNO3+CRO
     &                          +  0.150 * RXRAT( 163 )      ! NO3+ISPD=0.15*HNO3+...
      LOSS( HNO3             ) =           RXRAT(  29 )      ! HNO3+OH=NO3
     &                          +          RXRAT(  52 )      ! HNO3=OH+NO2

      PROD( H2O2             ) =           RXRAT(  34 )      ! HO2+HO2=H2O2
     &                          +          RXRAT(  35 )      ! HO2+HO2=H2O2
     &                          +          RXRAT(  42 )      ! OH+OH=H2O2
      LOSS( H2O2             ) =           RXRAT(  36 )      ! H2O2=2*OH
     &                          +          RXRAT(  37 )      ! H2O2+OH=HO2
     &                          +          RXRAT(  45 )      ! H2O2+O=OH+HO2
     &                          +          RXRAT( 251 )      ! H2O2+HG=HGIIGAS

      PROD( XO2              ) =           RXRAT(  63 )      ! ROOH+OH=XO2+0.5*ALD2+0.5*ALDX
     &                          +  0.300 * RXRAT(  70 )      ! MEPX+OH=0.3*XO2+0.7*MEO2+...
     &                          +          RXRAT( 102 )      ! CXO3+NO=XO2+NO2+HO2+ALD2
     &                          +  0.900 * RXRAT( 108 )      ! CXO3+MEO2=0.9*XO2+0.9*ALD2+...
     &                          +  2.000 * RXRAT( 110 )      ! CXO3+CXO3=2*XO2+2*ALD2+2*HO2
     &                          +          RXRAT( 111 )      ! CXO3+C2O3=XO2+MEO2+HO2+ALD2
     &                          +  0.870 * RXRAT( 112 )      ! PAR+OH=0.87*XO2+0.13*XO2N+...
     &                          +  0.960 * RXRAT( 113 )      ! ROR=0.96*XO2+0.6*ALD2+...
     &                          +  0.200 * RXRAT( 116 )      ! O+OLE=0.2*XO2+0.3*ALDX+...
     &                          +  0.800 * RXRAT( 117 )      ! OH+OLE=0.8*XO2+0.33*ALD2+...
     &                          +  0.220 * RXRAT( 118 )      ! O3+OLE=0.22*XO2+0.74*FORM+...
     &                          +  0.910 * RXRAT( 119 )      ! NO3+OLE=0.91*XO2+FORM+NO2+...
     &                          +  0.700 * RXRAT( 120 )      ! O+ETH=0.7*XO2+1.7*HO2+CO+...
     &                          +          RXRAT( 121 )      ! OH+ETH=XO2+1.56*FORM+...
     &                          +          RXRAT( 123 )      ! NO3+ETH=XO2+NO2+2*FORM
     &                          +  0.100 * RXRAT( 124 )      ! IOLE+O=0.1*XO2+0.66*ALDX+...
     &                          +          RXRAT( 125 )      ! IOLE+OH=XO2+0.7*ALDX+HO2+...
     &                          +  0.100 * RXRAT( 128 )      ! TOL+OH=0.1*XO2+0.28*HO2+...
     &                          +  0.120 * RXRAT( 131 )      ! OH+CRES=0.12*XO2+0.06*CRO+...
     &                          +  0.600 * RXRAT( 132 )      ! CRES+NO3=0.6*XO2+HNO3+...
     &                          +  0.030 * RXRAT( 145 )      ! OPEN+O3=0.03*XO2+0.62*OPO3+...
     &                          +          RXRAT( 151 )      ! OPO3+NO=XO2+NO2+HO2+ALDX
     &                          +  0.500 * RXRAT( 154 )      ! OH+XYL=0.5*XO2+0.7*HO2+...
     &                          +          RXRAT( 155 )      ! OH+MGLY=XO2+C2O3
     &                          +  0.250 * RXRAT( 157 )      ! O+ISOP=0.25*XO2+0.5*FORM+...
     &                          +  0.991 * RXRAT( 158 )      ! OH+ISOP=0.991*XO2+...
     &                          +  0.200 * RXRAT( 159 )      ! O3+ISOP=0.2*XO2+0.6*FORM+...
     &                          +          RXRAT( 160 )      ! NO3+ISOP=XO2+0.8*NTR+...
     &                          +  0.713 * RXRAT( 161 )      ! OH+ISPD=0.713*XO2+...
     &                          +  0.064 * RXRAT( 162 )      ! O3+ISPD=0.064*XO2+...
     &                          +  0.075 * RXRAT( 163 )      ! NO3+ISPD=0.075*XO2+...
     &                          +  0.700 * RXRAT( 164 )      ! ISPD=0.7*XO2+0.067*ALD2+...
     &                          +  1.250 * RXRAT( 166 )      ! TERP+OH=1.25*XO2+0.75*HO2+...
     &                          +  0.760 * RXRAT( 167 )      ! TERP+O3=0.76*XO2+0.07*HO2+...
     &                          +  1.030 * RXRAT( 168 )      ! TERP+NO3=1.03*XO2+0.28*HO2+...
     &                          +  0.100 * RXRAT( 170 )      ! OH+ETOH=0.1*XO2+0.9*ALD2+...
     &                          +  0.991 * RXRAT( 171 )      ! OH+ETHA=0.991*XO2+...
     &                          +          RXRAT( 172 )      ! NO2+ISOP=XO2+0.8*NTR+...
     &                          +  0.870 * RXRAT( 182 )      ! CL+PAR=0.87*XO2+HCL+...
     &                          +  0.991 * RXRAT( 183 )      ! CL+ETHA=0.991*XO2+...
     &                          +  2.000 * RXRAT( 184 )      ! CL+ETH=2*XO2+FMCL+HO2+FORM
     &                          +  2.000 * RXRAT( 185 )      ! CL+OLE=2*XO2+0.33*ALD2+...
     &                          +  1.700 * RXRAT( 186 )      ! CL+IOLE=1.7*XO2+0.7*FMCL+...
     &                          +          RXRAT( 187 )      ! CL+ISOP=XO2+0.15*HCL+HO2+...
     &                          +  0.880 * RXRAT( 194 )      ! CL+TOL=0.88*XO2+HCL+...
     &                          +  0.840 * RXRAT( 195 )      ! CL+XYL=0.84*XO2+HCL+...
      LOSS( XO2              ) =           RXRAT(  54 )      ! XO2+NO=NO2
     &                          +          RXRAT(  56 )      ! XO2+HO2=ROOH
     &                          +  2.000 * RXRAT(  58 )      ! XO2+XO2=
     &                          +          RXRAT(  60 )      ! XO2+XO2N=
     &                          +          RXRAT(  93 )      ! XO2+C2O3=0.9*MEO2+0.1*AACD
     &                          +          RXRAT( 109 )      ! XO2+CXO3=0.9*ALD2+0.1*AACD

      PROD( XO2N             ) =   0.130 * RXRAT( 112 )      ! PAR+OH=0.13*XO2N+0.87*XO2+...
     &                          +  0.040 * RXRAT( 113 )      ! ROR=0.04*XO2N+0.6*ALD2+...
     &                          +  0.010 * RXRAT( 116 )      ! O+OLE=0.01*XO2N+0.3*ALDX+...
     &                          +  0.090 * RXRAT( 119 )      ! NO3+OLE=0.09*XO2N+FORM+...
     &                          +  0.060 * RXRAT( 131 )      ! OH+CRES=0.06*XO2N+0.12*XO2+...
     &                          +  0.100 * RXRAT( 132 )      ! CRES+NO3=0.1*XO2N+HNO3+...
     &                          +  0.088 * RXRAT( 158 )      ! OH+ISOP=0.088*XO2N+...
     &                          +  0.250 * RXRAT( 166 )      ! TERP+OH=0.25*XO2N+1.25*XO2+...
     &                          +  0.180 * RXRAT( 167 )      ! TERP+O3=0.18*XO2N+0.07*HO2+...
     &                          +  0.250 * RXRAT( 168 )      ! TERP+NO3=0.25*XO2N+...
     &                          +  0.009 * RXRAT( 171 )      ! OH+ETHA=0.009*XO2N+...
     &                          +  0.130 * RXRAT( 182 )      ! CL+PAR=0.13*XO2N+0.87*XO2+...
     &                          +  0.009 * RXRAT( 183 )      ! CL+ETHA=0.009*XO2N+...
     &                          +  0.120 * RXRAT( 194 )      ! CL+TOL=0.12*XO2N+0.88*XO2+...
     &                          +  0.160 * RXRAT( 195 )      ! CL+XYL=0.16*XO2N+0.84*XO2+...
      LOSS( XO2N             ) =           RXRAT(  55 )      ! XO2N+NO=NTR
     &                          +          RXRAT(  57 )      ! XO2N+HO2=ROOH
     &                          +  2.000 * RXRAT(  59 )      ! XO2N+XO2N=
     &                          +          RXRAT(  60 )      ! XO2N+XO2=

      PROD( NTR              ) =           RXRAT(  55 )      ! XO2N+NO=NTR
     &                          +          RXRAT( 115 )      ! ROR+NO2=NTR
     &                          +  0.140 * RXRAT( 129 )      ! TO2+NO=0.14*NTR+1.2*HO2+...
     &                          +  2.000 * RXRAT( 137 )      ! CRNO+NO2=2*NTR
     &                          +  0.140 * RXRAT( 149 )      ! CAO2+NO=0.14*NTR+1.2*HO2+...
     &                          +  0.800 * RXRAT( 160 )      ! NO3+ISOP=0.8*NTR+0.2*ISPD+...
     &                          +  0.850 * RXRAT( 163 )      ! NO3+ISPD=0.85*NTR+...
     &                          +  0.530 * RXRAT( 168 )      ! TERP+NO3=0.53*NTR+0.28*HO2+...
     &                          +  0.800 * RXRAT( 172 )      ! NO2+ISOP=0.8*NTR+0.2*ISPD+...
      LOSS( NTR              ) =           RXRAT(  61 )      ! NTR+OH=HNO3+HO2+0.33*FORM+...
     &                          +          RXRAT(  62 )      ! NTR=NO2+HO2+0.33*FORM+...

      PROD( ROOH             ) =           RXRAT(  56 )      ! XO2+HO2=ROOH
     &                          +          RXRAT(  57 )      ! XO2N+HO2=ROOH
      LOSS( ROOH             ) =           RXRAT(  63 )      ! ROOH+OH=XO2+0.5*ALD2+0.5*ALDX
     &                          +          RXRAT(  64 )      ! ROOH=OH+HO2+0.5*ALD2+0.5*ALDX

      PROD( FORM             ) =   0.330 * RXRAT(  61 )      ! NTR+OH=0.33*FORM+HO2+HNO3+...
     &                          +  0.330 * RXRAT(  62 )      ! NTR=0.33*FORM+HO2+NO2+...
     &                          +          RXRAT(  67 )      ! MEO2+NO=FORM+HO2+NO2
     &                          +  1.370 * RXRAT(  69 )      ! MEO2+MEO2=1.37*FORM+...
     &                          +          RXRAT(  71 )      ! MEPX=FORM+HO2+OH
     &                          +          RXRAT(  72 )      ! MEOH+OH=FORM+HO2
     &                          +          RXRAT(  79 )      ! HCO3=FORM+HO2
     &                          +          RXRAT(  92 )      ! C2O3+MEO2=FORM+0.9*HO2+...
     &                          +  0.100 * RXRAT( 108 )      ! CXO3+MEO2=0.1*FORM+0.9*XO2+...
     &                          +  0.200 * RXRAT( 116 )      ! O+OLE=0.2*FORM+0.3*ALDX+...
     &                          +  0.800 * RXRAT( 117 )      ! OH+OLE=0.8*FORM+0.33*ALD2+...
     &                          +  0.740 * RXRAT( 118 )      ! O3+OLE=0.74*FORM+0.18*ALD2+...
     &                          +          RXRAT( 119 )      ! NO3+OLE=FORM+NO2+0.91*XO2+...
     &                          +          RXRAT( 120 )      ! O+ETH=FORM+1.7*HO2+CO+...
     &                          +  1.560 * RXRAT( 121 )      ! OH+ETH=1.56*FORM+XO2+...
     &                          +          RXRAT( 122 )      ! O3+ETH=FORM+0.63*CO+...
     &                          +  2.000 * RXRAT( 123 )      ! NO3+ETH=2*FORM+XO2+NO2
     &                          +  0.250 * RXRAT( 126 )      ! IOLE+O3=0.25*FORM+...
     &                          +  0.336 * RXRAT( 129 )      ! TO2+NO=0.336*FORM+1.2*HO2+...
     &                          +  0.060 * RXRAT( 131 )      ! OH+CRES=0.06*FORM+0.12*XO2+...
     &                          +  0.240 * RXRAT( 132 )      ! CRES+NO3=0.24*FORM+HNO3+...
     &                          +  0.700 * RXRAT( 145 )      ! OPEN+O3=0.7*FORM+0.62*OPO3+...
     &                          +  0.344 * RXRAT( 149 )      ! CAO2+NO=0.344*FORM+1.2*HO2+...
     &                          +  0.500 * RXRAT( 157 )      ! O+ISOP=0.5*FORM+0.75*ISPD+...
     &                          +  0.629 * RXRAT( 158 )      ! OH+ISOP=0.629*FORM+...
     &                          +  0.600 * RXRAT( 159 )      ! O3+ISOP=0.6*FORM+0.65*ISPD+...
     &                          +  0.167 * RXRAT( 161 )      ! OH+ISPD=0.167*FORM+...
     &                          +  0.150 * RXRAT( 162 )      ! O3+ISPD=0.15*FORM+...
     &                          +  0.282 * RXRAT( 163 )      ! NO3+ISPD=0.282*FORM+...
     &                          +  0.900 * RXRAT( 164 )      ! ISPD=0.9*FORM+0.067*ALD2+...
     &                          +  0.280 * RXRAT( 166 )      ! TERP+OH=0.28*FORM+1.25*XO2+...
     &                          +  0.240 * RXRAT( 167 )      ! TERP+O3=0.24*FORM+0.07*HO2+...
     &                          +  0.100 * RXRAT( 170 )      ! OH+ETOH=0.1*FORM+0.9*ALD2+...
     &                          +          RXRAT( 184 )      ! CL+ETH=FORM+2*XO2+HO2+FMCL
     &                          +          RXRAT( 191 )      ! CL+MEOH=FORM+HO2+HCL
      LOSS( FORM             ) =           RXRAT(  73 )      ! FORM+OH=HO2+CO
     &                          +          RXRAT(  74 )      ! FORM=2*HO2+CO
     &                          +          RXRAT(  75 )      ! FORM=CO
     &                          +          RXRAT(  76 )      ! FORM+O=OH+HO2+CO
     &                          +          RXRAT(  77 )      ! FORM+NO3=HNO3+HO2+CO
     &                          +          RXRAT(  78 )      ! FORM+HO2=HCO3
     &                          +          RXRAT( 188 )      ! FORM+CL=HCL+HO2+CO

      PROD( ALD2             ) =   0.330 * RXRAT(  61 )      ! NTR+OH=0.33*ALD2+HO2+...
     &                          +  0.330 * RXRAT(  62 )      ! NTR=0.33*ALD2+HO2+...
     &                          +  0.500 * RXRAT(  63 )      ! ROOH+OH=0.5*ALD2+XO2+0.5*ALDX
     &                          +  0.500 * RXRAT(  64 )      ! ROOH=0.5*ALD2+HO2+OH+0.5*ALDX
     &                          +          RXRAT( 102 )      ! CXO3+NO=ALD2+NO2+HO2+XO2
     &                          +          RXRAT( 106 )      ! PANX+OH=ALD2+NO2
     &                          +  0.900 * RXRAT( 108 )      ! CXO3+MEO2=0.9*ALD2+0.9*XO2+...
     &                          +  0.900 * RXRAT( 109 )      ! CXO3+XO2=0.9*ALD2+0.1*AACD
     &                          +  2.000 * RXRAT( 110 )      ! CXO3+CXO3=2*ALD2+2*XO2+2*HO2
     &                          +          RXRAT( 111 )      ! CXO3+C2O3=ALD2+XO2+HO2+MEO2
     &                          +  0.060 * RXRAT( 112 )      ! PAR+OH=0.06*ALD2+0.13*XO2N+...
     &                          +  0.600 * RXRAT( 113 )      ! ROR=0.6*ALD2+0.96*XO2+...
     &                          +  0.200 * RXRAT( 116 )      ! O+OLE=0.2*ALD2+0.3*ALDX+...
     &                          +  0.330 * RXRAT( 117 )      ! OH+OLE=0.33*ALD2+0.8*FORM+...
     &                          +  0.180 * RXRAT( 118 )      ! O3+OLE=0.18*ALD2+0.74*FORM+...
     &                          +  0.350 * RXRAT( 119 )      ! NO3+OLE=0.35*ALD2+FORM+...
     &                          +  1.240 * RXRAT( 124 )      ! IOLE+O=1.24*ALD2+0.66*ALDX+...
     &                          +  1.300 * RXRAT( 125 )      ! IOLE+OH=1.3*ALD2+0.7*ALDX+...
     &                          +  0.650 * RXRAT( 126 )      ! IOLE+O3=0.65*ALD2+...
     &                          +  1.180 * RXRAT( 127 )      ! IOLE+NO3=1.18*ALD2+...
     &                          +  0.252 * RXRAT( 161 )      ! OH+ISPD=0.252*ALD2+...
     &                          +  0.020 * RXRAT( 162 )      ! O3+ISPD=0.02*ALD2+...
     &                          +  0.067 * RXRAT( 164 )      ! ISPD=0.067*ALD2+0.333*CO+...
     &                          +  0.900 * RXRAT( 170 )      ! OH+ETOH=0.9*ALD2+HO2+...
     &                          +  0.991 * RXRAT( 171 )      ! OH+ETHA=0.991*ALD2+...
     &                          +  0.060 * RXRAT( 182 )      ! CL+PAR=0.06*ALD2+0.87*XO2+...
     &                          +  0.991 * RXRAT( 183 )      ! CL+ETHA=0.991*ALD2+HCL+...
     &                          +  0.330 * RXRAT( 185 )      ! CL+OLE=0.33*ALD2+FMCL+...
     &                          +  0.450 * RXRAT( 186 )      ! CL+IOLE=0.45*ALD2+0.7*FMCL+...
     &                          +          RXRAT( 192 )      ! CL+ETOH=ALD2+HO2+HCL
      LOSS( ALD2             ) =           RXRAT(  83 )      ! ALD2+O=C2O3+OH
     &                          +          RXRAT(  84 )      ! ALD2+OH=C2O3
     &                          +          RXRAT(  85 )      ! ALD2+NO3=C2O3+HNO3
     &                          +          RXRAT(  86 )      ! ALD2=MEO2+CO+HO2
     &                          +          RXRAT( 189 )      ! ALD2+CL=HCL+C2O3

      PROD( ALDX             ) =   0.330 * RXRAT(  61 )      ! NTR+OH=0.33*ALDX+HO2+...
     &                          +  0.330 * RXRAT(  62 )      ! NTR=0.33*ALDX+HO2+...
     &                          +  0.500 * RXRAT(  63 )      ! ROOH+OH=0.5*ALDX+0.5*ALD2+XO2
     &                          +  0.500 * RXRAT(  64 )      ! ROOH=0.5*ALDX+HO2+0.5*ALD2+OH
     &                          +  0.050 * RXRAT( 112 )      ! PAR+OH=0.05*ALDX+0.13*XO2N+...
     &                          +  0.500 * RXRAT( 113 )      ! ROR=0.5*ALDX+0.6*ALD2+...
     &                          +  0.300 * RXRAT( 116 )      ! O+OLE=0.3*ALDX+0.2*ALD2+...
     &                          +  0.620 * RXRAT( 117 )      ! OH+OLE=0.62*ALDX+0.33*ALD2+...
     &                          +  0.320 * RXRAT( 118 )      ! O3+OLE=0.32*ALDX+0.74*FORM+...
     &                          +  0.560 * RXRAT( 119 )      ! NO3+OLE=0.56*ALDX+FORM+...
     &                          +  0.220 * RXRAT( 121 )      ! OH+ETH=0.22*ALDX+1.56*FORM+...
     &                          +  0.660 * RXRAT( 124 )      ! IOLE+O=0.66*ALDX+1.24*ALD2+...
     &                          +  0.700 * RXRAT( 125 )      ! IOLE+OH=0.7*ALDX+1.3*ALD2+...
     &                          +  0.350 * RXRAT( 126 )      ! IOLE+O3=0.35*ALDX+...
     &                          +  0.640 * RXRAT( 127 )      ! IOLE+NO3=0.64*ALDX+...
     &                          +  0.480 * RXRAT( 132 )      ! CRES+NO3=0.48*ALDX+HNO3+...
     &                          +  0.030 * RXRAT( 145 )      ! OPEN+O3=0.03*ALDX+...
     &                          +          RXRAT( 151 )      ! OPO3+NO=ALDX+XO2+HO2+NO2
     &                          +  0.150 * RXRAT( 159 )      ! O3+ISOP=0.15*ALDX+0.6*FORM+...
     &                          +  0.800 * RXRAT( 160 )      ! NO3+ISOP=0.8*ALDX+0.8*NTR+...
     &                          +  0.120 * RXRAT( 161 )      ! OH+ISPD=0.12*ALDX+...
     &                          +  0.357 * RXRAT( 163 )      ! NO3+ISPD=0.357*ALDX+...
     &                          +  0.150 * RXRAT( 165 )      ! TERP+O=0.15*ALDX+5.12*PAR+...
     &                          +  0.470 * RXRAT( 166 )      ! TERP+OH=0.47*ALDX+1.25*XO2+...
     &                          +  0.210 * RXRAT( 167 )      ! TERP+O3=0.21*ALDX+0.07*HO2+...
     &                          +  0.470 * RXRAT( 168 )      ! TERP+NO3=0.47*ALDX+...
     &                          +  0.050 * RXRAT( 170 )      ! OH+ETOH=0.05*ALDX+0.9*ALD2+...
     &                          +  0.800 * RXRAT( 172 )      ! NO2+ISOP=0.8*ALDX+0.8*NTR+...
     &                          +  0.050 * RXRAT( 182 )      ! CL+PAR=0.05*ALDX+0.87*XO2+...
     &                          +  0.670 * RXRAT( 185 )      ! CL+OLE=0.67*ALDX+0.33*ALD2+...
     &                          +  0.550 * RXRAT( 186 )      ! CL+IOLE=0.55*ALDX+0.7*FMCL+...
      LOSS( ALDX             ) =           RXRAT(  98 )      ! ALDX+O=CXO3+OH
     &                          +          RXRAT(  99 )      ! ALDX+OH=CXO3
     &                          +          RXRAT( 100 )      ! ALDX+NO3=CXO3+HNO3
     &                          +          RXRAT( 101 )      ! ALDX=MEO2+CO+HO2
     &                          +          RXRAT( 190 )      ! ALDX+CL=HCL+CXO3

      PROD( PAR              ) =   0.200 * RXRAT( 116 )      ! O+OLE=0.2*PAR+0.3*ALDX+...
     &                          +  0.100 * RXRAT( 124 )      ! IOLE+O=0.1*PAR+0.66*ALDX+...
     &                          +  1.100 * RXRAT( 154 )      ! OH+XYL=1.1*PAR+0.5*XO2+...
     &                          +  0.250 * RXRAT( 157 )      ! O+ISOP=0.25*PAR+0.5*FORM+...
     &                          +  0.350 * RXRAT( 159 )      ! O3+ISOP=0.35*PAR+0.6*FORM+...
     &                          +  2.400 * RXRAT( 160 )      ! NO3+ISOP=2.4*PAR+0.8*NTR+...
     &                          +  1.565 * RXRAT( 161 )      ! OH+ISPD=1.565*PAR+...
     &                          +  0.360 * RXRAT( 162 )      ! O3+ISPD=0.36*PAR+0.15*FORM+...
     &                          +  1.282 * RXRAT( 163 )      ! NO3+ISPD=1.282*PAR+...
     &                          +  0.832 * RXRAT( 164 )      ! ISPD=0.832*PAR+0.067*ALD2+...
     &                          +  5.120 * RXRAT( 165 )      ! TERP+O=5.12*PAR+0.15*ALDX+...
     &                          +  1.660 * RXRAT( 166 )      ! TERP+OH=1.66*PAR+1.25*XO2+...
     &                          +  7.000 * RXRAT( 167 )      ! TERP+O3=7*PAR+0.07*HO2+...
     &                          +  2.400 * RXRAT( 172 )      ! NO2+ISOP=2.4*PAR+0.8*NTR+...
     &                          +  0.300 * RXRAT( 186 )      ! CL+IOLE=0.3*PAR+0.7*FMCL+...
      LOSS( PAR              ) =   0.660 * RXRAT(  61 )      ! NTR+OH=-0.66*PAR+HO2+...
     &                          +  0.660 * RXRAT(  62 )      ! NTR=-0.66*PAR+HO2+...
     &                          +  1.110 * RXRAT( 112 )      ! PAR+OH=-0.11*PAR+0.13*XO2N+...
     &                          +  2.100 * RXRAT( 113 )      ! ROR=-2.1*PAR+0.6*ALD2+...
     &                          +  0.700 * RXRAT( 117 )      ! OH+OLE=-0.7*PAR+0.33*ALD2+...
     &                          +          RXRAT( 118 )      ! O3+OLE=-PAR+0.74*FORM+...
     &                          +          RXRAT( 119 )      ! NO3+OLE=-PAR+FORM+0.91*XO2+...
     &                          +  1.110 * RXRAT( 182 )      ! PAR+CL=-0.11*PAR+0.87*XO2+...
     &                          +          RXRAT( 185 )      ! CL+OLE=-PAR+0.33*ALD2+...

      PROD( CO               ) =           RXRAT(  73 )      ! FORM+OH=CO+HO2
     &                          +          RXRAT(  74 )      ! FORM=CO+2*HO2
     &                          +          RXRAT(  75 )      ! FORM=CO
     &                          +          RXRAT(  76 )      ! FORM+O=CO+HO2+OH
     &                          +          RXRAT(  77 )      ! FORM+NO3=CO+HO2+HNO3
     &                          +          RXRAT(  86 )      ! ALD2=CO+MEO2+HO2
     &                          +          RXRAT( 101 )      ! ALDX=CO+MEO2+HO2
     &                          +  0.200 * RXRAT( 116 )      ! O+OLE=0.2*CO+0.3*ALDX+...
     &                          +  0.330 * RXRAT( 118 )      ! O3+OLE=0.33*CO+0.74*FORM+...
     &                          +          RXRAT( 120 )      ! O+ETH=CO+1.7*HO2+FORM+...
     &                          +  0.630 * RXRAT( 122 )      ! O3+ETH=0.63*CO+FORM+...
     &                          +  0.100 * RXRAT( 124 )      ! IOLE+O=0.1*CO+0.66*ALDX+...
     &                          +  0.250 * RXRAT( 126 )      ! IOLE+O3=0.25*CO+0.35*ALDX+...
     &                          +  0.336 * RXRAT( 129 )      ! TO2+NO=0.336*CO+1.2*HO2+...
     &                          +  0.060 * RXRAT( 131 )      ! OH+CRES=0.06*CO+0.12*XO2+...
     &                          +  0.240 * RXRAT( 132 )      ! CRES+NO3=0.24*CO+HNO3+...
     &                          +          RXRAT( 143 )      ! OPEN=CO+HO2+OPO3
     &                          +  0.690 * RXRAT( 145 )      ! OPEN+O3=0.69*CO+0.62*OPO3+...
     &                          +  0.344 * RXRAT( 149 )      ! CAO2+NO=0.344*CO+1.2*HO2+...
     &                          +          RXRAT( 156 )      ! MGLY=CO+HO2+C2O3
     &                          +  0.066 * RXRAT( 159 )      ! O3+ISOP=0.066*CO+0.6*FORM+...
     &                          +  0.334 * RXRAT( 161 )      ! OH+ISPD=0.334*CO+...
     &                          +  0.225 * RXRAT( 162 )      ! O3+ISPD=0.225*CO+0.15*FORM+...
     &                          +  0.643 * RXRAT( 163 )      ! NO3+ISPD=0.643*CO+...
     &                          +  0.333 * RXRAT( 164 )      ! ISPD=0.333*CO+0.067*ALD2+...
     &                          +  0.001 * RXRAT( 167 )      ! TERP+O3=0.001*CO+0.07*HO2+...
     &                          +          RXRAT( 179 )      ! OH+FMCL=CO+CL
     &                          +          RXRAT( 180 )      ! FMCL=CO+CL+HO2
     &                          +          RXRAT( 188 )      ! CL+FORM=CO+HO2+HCL
      LOSS( CO               ) =           RXRAT(  65 )      ! CO+OH=HO2

      PROD( MEO2             ) =           RXRAT(  66 )      ! OH=MEO2
     &                          +  0.700 * RXRAT(  70 )      ! MEPX+OH=0.7*MEO2+0.3*XO2+...
     &                          +          RXRAT(  86 )      ! ALD2=MEO2+CO+HO2
     &                          +          RXRAT(  87 )      ! C2O3+NO=MEO2+NO2
     &                          +  0.900 * RXRAT(  93 )      ! C2O3+XO2=0.9*MEO2+0.1*AACD
     &                          +  2.000 * RXRAT(  94 )      ! C2O3+C2O3=2*MEO2
     &                          +          RXRAT(  96 )      ! PACD=MEO2+OH
     &                          +          RXRAT(  97 )      ! AACD+OH=MEO2
     &                          +          RXRAT( 101 )      ! ALDX=MEO2+CO+HO2
     &                          +          RXRAT( 111 )      ! CXO3+C2O3=MEO2+XO2+HO2+ALD2
     &                          +          RXRAT( 181 )      ! CL=MEO2+HCL
      LOSS( MEO2             ) =           RXRAT(  67 )      ! MEO2+NO=FORM+HO2+NO2
     &                          +          RXRAT(  68 )      ! MEO2+HO2=MEPX
     &                          +  2.000 * RXRAT(  69 )      ! MEO2+MEO2=1.37*FORM+...
     &                          +  0.100 * RXRAT(  92 )      ! MEO2+C2O3=0.9*MEO2+0.9*HO2+...
     &                          +          RXRAT( 108 )      ! MEO2+CXO3=0.9*ALD2+0.9*XO2+...

      PROD( MEPX             ) =           RXRAT(  68 )      ! MEO2+HO2=MEPX
     &                          +          RXRAT(  81 )      ! HCO3+HO2=MEPX
      LOSS( MEPX             ) =           RXRAT(  70 )      ! MEPX+OH=0.7*MEO2+0.3*XO2+...
     &                          +          RXRAT(  71 )      ! MEPX=FORM+HO2+OH

      PROD( MEOH             ) =   0.630 * RXRAT(  69 )      ! MEO2+MEO2=0.63*MEOH+...
      LOSS( MEOH             ) =           RXRAT(  72 )      ! MEOH+OH=FORM+HO2
     &                          +          RXRAT( 191 )      ! MEOH+CL=HCL+HO2+FORM

      PROD( HCO3             ) =           RXRAT(  78 )      ! FORM+HO2=HCO3
      LOSS( HCO3             ) =           RXRAT(  79 )      ! HCO3=FORM+HO2
     &                          +          RXRAT(  80 )      ! HCO3+NO=FACD+NO2+HO2
     &                          +          RXRAT(  81 )      ! HCO3+HO2=MEPX

      PROD( FACD             ) =           RXRAT(  80 )      ! HCO3+NO=FACD+NO2+HO2
     &                          +  0.370 * RXRAT( 122 )      ! O3+ETH=0.37*FACD+0.63*CO+...
      LOSS( FACD             ) =           RXRAT(  82 )      ! FACD+OH=HO2

      PROD( PACD             ) =   0.800 * RXRAT(  91 )      ! C2O3+HO2=0.8*PACD+0.2*AACD+...
     &                          +  0.800 * RXRAT( 107 )      ! CXO3+HO2=0.8*PACD+0.2*AACD+...
      LOSS( PACD             ) =           RXRAT(  95 )      ! PACD+OH=C2O3
     &                          +          RXRAT(  96 )      ! PACD=MEO2+OH

      PROD( AACD             ) =   0.200 * RXRAT(  91 )      ! C2O3+HO2=0.2*AACD+0.8*PACD+...
     &                          +  0.100 * RXRAT(  92 )      ! C2O3+MEO2=0.1*AACD+0.9*HO2+...
     &                          +  0.100 * RXRAT(  93 )      ! C2O3+XO2=0.1*AACD+0.9*MEO2
     &                          +  0.200 * RXRAT( 107 )      ! CXO3+HO2=0.2*AACD+0.8*PACD+...
     &                          +  0.100 * RXRAT( 108 )      ! CXO3+MEO2=0.1*AACD+0.9*XO2+...
     &                          +  0.100 * RXRAT( 109 )      ! CXO3+XO2=0.1*AACD+0.9*ALD2
      LOSS( AACD             ) =           RXRAT(  97 )      ! AACD+OH=MEO2

      PROD( CXO3             ) =           RXRAT(  98 )      ! ALDX+O=CXO3+OH
     &                          +          RXRAT(  99 )      ! ALDX+OH=CXO3
     &                          +          RXRAT( 100 )      ! ALDX+NO3=CXO3+HNO3
     &                          +          RXRAT( 104 )      ! PANX=CXO3+NO2
     &                          +          RXRAT( 105 )      ! PANX=CXO3+NO2
     &                          +  0.250 * RXRAT( 157 )      ! O+ISOP=0.25*CXO3+0.5*FORM+...
     &                          +  0.200 * RXRAT( 159 )      ! O3+ISOP=0.2*CXO3+0.6*FORM+...
     &                          +  0.250 * RXRAT( 161 )      ! OH+ISPD=0.25*CXO3+...
     &                          +  0.075 * RXRAT( 163 )      ! NO3+ISPD=0.075*CXO3+...
     &                          +  0.390 * RXRAT( 167 )      ! TERP+O3=0.39*CXO3+0.07*HO2+...
     &                          +          RXRAT( 190 )      ! CL+ALDX=CXO3+HCL
      LOSS( CXO3             ) =           RXRAT( 102 )      ! CXO3+NO=ALD2+NO2+HO2+XO2
     &                          +          RXRAT( 103 )      ! CXO3+NO2=PANX
     &                          +          RXRAT( 107 )      ! CXO3+HO2=0.8*PACD+0.2*AACD+...
     &                          +          RXRAT( 108 )      ! CXO3+MEO2=0.9*ALD2+0.9*XO2+...
     &                          +          RXRAT( 109 )      ! CXO3+XO2=0.9*ALD2+0.1*AACD
     &                          +  2.000 * RXRAT( 110 )      ! CXO3+CXO3=2*ALD2+2*XO2+2*HO2
     &                          +          RXRAT( 111 )      ! CXO3+C2O3=MEO2+XO2+HO2+ALD2

      PROD( PANX             ) =           RXRAT( 103 )      ! CXO3+NO2=PANX
      LOSS( PANX             ) =           RXRAT( 104 )      ! PANX=CXO3+NO2
     &                          +          RXRAT( 105 )      ! PANX=CXO3+NO2
     &                          +          RXRAT( 106 )      ! PANX+OH=ALD2+NO2

      PROD( ROR              ) =   0.760 * RXRAT( 112 )      ! PAR+OH=0.76*ROR+0.13*XO2N+...
     &                          +  0.760 * RXRAT( 182 )      ! CL+PAR=0.76*ROR+0.87*XO2+...
      LOSS( ROR              ) =   0.980 * RXRAT( 113 )      ! ROR=0.02*ROR+0.6*ALD2+...
     &                          +          RXRAT( 114 )      ! ROR=HO2
     &                          +          RXRAT( 115 )      ! ROR+NO2=NTR

      PROD( OLE              ) =   0.300 * RXRAT( 186 )      ! CL+IOLE=0.3*OLE+0.7*FMCL+...
      LOSS( OLE              ) =           RXRAT( 116 )      ! OLE+O=0.2*ALD2+0.3*ALDX+...
     &                          +          RXRAT( 117 )      ! OLE+OH=0.8*FORM+0.33*ALD2+...
     &                          +          RXRAT( 118 )      ! OLE+O3=0.18*ALD2+0.74*FORM+...
     &                          +          RXRAT( 119 )      ! OLE+NO3=NO2+FORM+0.91*XO2+...
     &                          +          RXRAT( 185 )      ! OLE+CL=FMCL+0.33*ALD2+...

      PROD( ETH              ) =   0.0
      LOSS( ETH              ) =           RXRAT( 120 )      ! ETH+O=FORM+1.7*HO2+CO+...
     &                          +          RXRAT( 121 )      ! ETH+OH=XO2+1.56*FORM+...
     &                          +          RXRAT( 122 )      ! ETH+O3=FORM+0.63*CO+...
     &                          +          RXRAT( 123 )      ! ETH+NO3=NO2+XO2+2*FORM
     &                          +          RXRAT( 184 )      ! ETH+CL=FMCL+2*XO2+HO2+FORM

      PROD( IOLE             ) =   0.0
      LOSS( IOLE             ) =           RXRAT( 124 )      ! IOLE+O=1.24*ALD2+0.66*ALDX+...
     &                          +          RXRAT( 125 )      ! IOLE+OH=1.3*ALD2+0.7*ALDX+...
     &                          +          RXRAT( 126 )      ! IOLE+O3=0.65*ALD2+...
     &                          +          RXRAT( 127 )      ! IOLE+NO3=1.18*ALD2+...
     &                          +          RXRAT( 186 )      ! IOLE+CL=0.3*HCL+0.7*FMCL+...

      PROD( TOL              ) =   0.0
      LOSS( TOL              ) =           RXRAT( 128 )      ! TOL+OH=0.28*HO2+0.1*XO2+...
     &                          +          RXRAT( 194 )      ! TOL+CL=HCL+0.88*XO2+...

      PROD( CRES             ) =   0.180 * RXRAT( 128 )      ! TOL+OH=0.18*CRES+0.1*XO2+...
     &                          +          RXRAT( 134 )      ! CRO+HO2=CRES
     &                          +  0.200 * RXRAT( 154 )      ! OH+XYL=0.2*CRES+0.5*XO2+...
      LOSS( CRES             ) =           RXRAT( 131 )      ! CRES+OH=0.06*CRO+0.12*XO2+...
     &                          +          RXRAT( 132 )      ! CRES+NO3=0.3*CRO+HNO3+...

      PROD( TO2              ) =   0.650 * RXRAT( 128 )      ! TOL+OH=0.65*TO2+0.1*XO2+...
     &                          +  0.300 * RXRAT( 154 )      ! OH+XYL=0.3*TO2+0.5*XO2+...
      LOSS( TO2              ) =           RXRAT( 129 )      ! TO2+NO=0.86*NO2+1.2*HO2+...
     &                          +          RXRAT( 130 )      ! TO2+HO2=

      PROD( TOLRO2           ) =           RXRAT( 128 )      ! TOL+OH=TOLRO2+0.1*XO2+...
      LOSS( TOLRO2           ) =           RXRAT( 196 )      ! TOLRO2+NO=NO+TOLNRXN
     &                          +          RXRAT( 197 )      ! TOLRO2+HO2=HO2+TOLHRXN

      PROD( OPEN             ) =   0.860 * RXRAT( 129 )      ! TO2+NO=0.86*OPEN+1.2*HO2+...
     &                          +  0.130 * RXRAT( 131 )      ! OH+CRES=0.13*OPEN+0.12*XO2+...
     &                          +  0.120 * RXRAT( 132 )      ! CRES+NO3=0.12*OPEN+HNO3+...
      LOSS( OPEN             ) =           RXRAT( 143 )      ! OPEN=OPO3+HO2+CO
     &                          +          RXRAT( 144 )      ! OPEN+OH=0.6*OPO3+0.4*CAO2
     &                          +          RXRAT( 145 )      ! OPEN+O3=0.03*ALDX+...
     &                          +          RXRAT( 146 )      ! OPEN+NO3=OPO3+HNO3

      PROD( MGLY             ) =   0.520 * RXRAT( 129 )      ! TO2+NO=0.52*MGLY+1.2*HO2+...
     &                          +  0.240 * RXRAT( 132 )      ! CRES+NO3=0.24*MGLY+HNO3+...
     &                          +  0.200 * RXRAT( 145 )      ! OPEN+O3=0.2*MGLY+0.62*OPO3+...
     &                          +  0.800 * RXRAT( 154 )      ! OH+XYL=0.8*MGLY+0.5*XO2+...
     &                          +  0.168 * RXRAT( 161 )      ! OH+ISPD=0.168*MGLY+...
     &                          +  0.850 * RXRAT( 162 )      ! O3+ISPD=0.85*MGLY+...
      LOSS( MGLY             ) =           RXRAT( 155 )      ! MGLY+OH=XO2+C2O3
     &                          +          RXRAT( 156 )      ! MGLY=C2O3+HO2+CO

      PROD( CRO              ) =   0.060 * RXRAT( 131 )      ! OH+CRES=0.06*CRO+0.12*XO2+...
     &                          +  0.300 * RXRAT( 132 )      ! CRES+NO3=0.3*CRO+HNO3+...
     &                          +          RXRAT( 148 )      ! CAT1+NO3=CRO+HNO3
      LOSS( CRO              ) =           RXRAT( 133 )      ! CRO+NO2=CRON
     &                          +          RXRAT( 134 )      ! CRO+HO2=CRES

      PROD( CAT1             ) =   0.732 * RXRAT( 131 )      ! OH+CRES=0.732*CAT1+...
      LOSS( CAT1             ) =           RXRAT( 147 )      ! CAT1+OH=CAO2
     &                          +          RXRAT( 148 )      ! CAT1+NO3=CRO+HNO3

      PROD( CRON             ) =           RXRAT( 133 )      ! CRO+NO2=CRON
      LOSS( CRON             ) =           RXRAT( 135 )      ! CRON+OH=CRNO
     &                          +          RXRAT( 136 )      ! CRON+NO3=CRNO+HNO3

      PROD( CRNO             ) =           RXRAT( 135 )      ! CRON+OH=CRNO
     &                          +          RXRAT( 136 )      ! CRON+NO3=CRNO+HNO3
     &                          +          RXRAT( 139 )      ! CRN2+NO=CRNO+NO2
     &                          +          RXRAT( 141 )      ! CRPX=CRNO+OH
      LOSS( CRNO             ) =           RXRAT( 137 )      ! CRNO+NO2=2*NTR
     &                          +          RXRAT( 138 )      ! CRNO+O3=CRN2

      PROD( CRN2             ) =           RXRAT( 138 )      ! CRNO+O3=CRN2
     &                          +          RXRAT( 142 )      ! CRPX+OH=CRN2
      LOSS( CRN2             ) =           RXRAT( 139 )      ! CRN2+NO=CRNO+NO2
     &                          +          RXRAT( 140 )      ! CRN2+HO2=CRPX

      PROD( CRPX             ) =           RXRAT( 140 )      ! CRN2+HO2=CRPX
      LOSS( CRPX             ) =           RXRAT( 141 )      ! CRPX=CRNO+OH
     &                          +          RXRAT( 142 )      ! CRPX+OH=CRN2

      PROD( OPO3             ) =           RXRAT( 143 )      ! OPEN=OPO3+HO2+CO
     &                          +  0.600 * RXRAT( 144 )      ! OPEN+OH=0.6*OPO3+0.4*CAO2
     &                          +  0.620 * RXRAT( 145 )      ! OPEN+O3=0.62*OPO3+...
     &                          +          RXRAT( 146 )      ! OPEN+NO3=OPO3+HNO3
     &                          +          RXRAT( 153 )      ! OPAN=OPO3+NO2
      LOSS( OPO3             ) =           RXRAT( 151 )      ! OPO3+NO=NO2+XO2+HO2+ALDX
     &                          +          RXRAT( 152 )      ! OPO3+NO2=OPAN

      PROD( CAO2             ) =   0.400 * RXRAT( 144 )      ! OPEN+OH=0.4*CAO2+0.6*OPO3
     &                          +          RXRAT( 147 )      ! CAT1+OH=CAO2
      LOSS( CAO2             ) =           RXRAT( 149 )      ! CAO2+NO=0.86*NO2+1.2*HO2+...
     &                          +          RXRAT( 150 )      ! CAO2+HO2=

      PROD( OPAN             ) =           RXRAT( 152 )      ! OPO3+NO2=OPAN
      LOSS( OPAN             ) =           RXRAT( 153 )      ! OPAN=OPO3+NO2

      PROD( XYL              ) =   0.0
      LOSS( XYL              ) =           RXRAT( 154 )      ! XYL+OH=0.7*HO2+0.5*XO2+...
     &                          +          RXRAT( 195 )      ! XYL+CL=HCL+0.84*XO2+...

      PROD( XYLRO2           ) =           RXRAT( 154 )      ! OH+XYL=XYLRO2+0.5*XO2+...
      LOSS( XYLRO2           ) =           RXRAT( 198 )      ! XYLRO2+NO=NO+XYLNRXN
     &                          +          RXRAT( 199 )      ! XYLRO2+HO2=HO2+XYLHRXN

      PROD( ISOP             ) =   0.0
      LOSS( ISOP             ) =           RXRAT( 157 )      ! ISOP+O=0.75*ISPD+0.5*FORM+...
     &                          +          RXRAT( 158 )      ! ISOP+OH=0.912*ISPD+...
     &                          +          RXRAT( 159 )      ! ISOP+O3=0.65*ISPD+0.6*FORM+...
     &                          +          RXRAT( 160 )      ! ISOP+NO3=0.2*ISPD+0.8*NTR+...
     &                          +          RXRAT( 172 )      ! ISOP+NO2=0.2*ISPD+0.8*NTR+...
     &                          +          RXRAT( 187 )      ! ISOP+CL=0.15*HCL+XO2+HO2+...

      PROD( ISPD             ) =   0.750 * RXRAT( 157 )      ! O+ISOP=0.75*ISPD+0.5*FORM+...
     &                          +  0.912 * RXRAT( 158 )      ! OH+ISOP=0.912*ISPD+...
     &                          +  0.650 * RXRAT( 159 )      ! O3+ISOP=0.65*ISPD+0.6*FORM+...
     &                          +  0.200 * RXRAT( 160 )      ! NO3+ISOP=0.2*ISPD+0.8*NTR+...
     &                          +  0.200 * RXRAT( 172 )      ! NO2+ISOP=0.2*ISPD+0.8*NTR+...
     &                          +          RXRAT( 187 )      ! CL+ISOP=ISPD+XO2+HO2+...
      LOSS( ISPD             ) =           RXRAT( 161 )      ! ISPD+OH=1.565*PAR+...
     &                          +          RXRAT( 162 )      ! ISPD+O3=0.114*C2O3+...
     &                          +          RXRAT( 163 )      ! ISPD+NO3=0.357*ALDX+...
     &                          +          RXRAT( 164 )      ! ISPD=0.333*CO+0.067*ALD2+...

      PROD( ISOPRXN          ) =           RXRAT( 158 )      ! OH+ISOP=ISOPRXN+0.629*FORM+...
      LOSS( ISOPRXN          ) =   0.0

      PROD( TERP             ) =   0.0
      LOSS( TERP             ) =           RXRAT( 165 )      ! TERP+O=0.15*ALDX+5.12*PAR+...
     &                          +          RXRAT( 166 )      ! TERP+OH=0.75*HO2+1.25*XO2+...
     &                          +          RXRAT( 167 )      ! TERP+O3=0.57*OH+0.07*HO2+...
     &                          +          RXRAT( 168 )      ! TERP+NO3=0.47*NO2+0.28*HO2+...

      PROD( TRPRXN           ) =           RXRAT( 165 )      ! TERP+O=TRPRXN+5.12*PAR+...
     &                          +          RXRAT( 166 )      ! TERP+OH=TRPRXN+1.25*XO2+...
     &                          +          RXRAT( 167 )      ! TERP+O3=TRPRXN+0.07*HO2+...
     &                          +          RXRAT( 168 )      ! TERP+NO3=TRPRXN+0.28*HO2+...
      LOSS( TRPRXN           ) =   0.0

      PROD( SO2              ) =   0.0
      LOSS( SO2              ) =           RXRAT( 169 )      ! SO2+OH=SULF+HO2+SULRXN

      PROD( SULF             ) =           RXRAT( 169 )      ! SO2+OH=SULF+HO2+SULRXN
      LOSS( SULF             ) =   0.0

      PROD( SULRXN           ) =           RXRAT( 169 )      ! SO2+OH=SULRXN+HO2+SULF
      LOSS( SULRXN           ) =   0.0

      PROD( ETOH             ) =   0.0
      LOSS( ETOH             ) =           RXRAT( 170 )      ! ETOH+OH=HO2+0.9*ALD2+...
     &                          +          RXRAT( 192 )      ! ETOH+CL=HCL+HO2+ALD2

      PROD( ETHA             ) =   0.0
      LOSS( ETHA             ) =           RXRAT( 171 )      ! ETHA+OH=0.991*ALD2+...
     &                          +          RXRAT( 183 )      ! ETHA+CL=HCL+0.991*ALD2+...

      PROD( CL2              ) =   0.300 * RXRAT( 176 )      ! CLO+CLO=0.3*CL2+1.4*CL
      LOSS( CL2              ) =           RXRAT( 173 )      ! CL2=2*CL
     &                          +          RXRAT( 250 )      ! CL2+HG=HGIIGAS

      PROD( CL               ) =   2.000 * RXRAT( 173 )      ! CL2=2*CL
     &                          +          RXRAT( 174 )      ! HOCL=CL+OH
     &                          +  1.400 * RXRAT( 176 )      ! CLO+CLO=1.4*CL+0.3*CL2
     &                          +          RXRAT( 177 )      ! CLO+NO=CL+NO2
     &                          +          RXRAT( 179 )      ! OH+FMCL=CL+CO
     &                          +          RXRAT( 180 )      ! FMCL=CL+CO+HO2
     &                          +          RXRAT( 193 )      ! HCL+OH=CL
      LOSS( CL               ) =           RXRAT( 175 )      ! CL+O3=CLO
     &                          +          RXRAT( 181 )      ! CL=HCL+MEO2
     &                          +          RXRAT( 182 )      ! CL+PAR=HCL+0.87*XO2+...
     &                          +          RXRAT( 183 )      ! CL+ETHA=HCL+0.991*ALD2+...
     &                          +          RXRAT( 184 )      ! CL+ETH=FMCL+2*XO2+HO2+FORM
     &                          +          RXRAT( 185 )      ! CL+OLE=FMCL+0.33*ALD2+...
     &                          +          RXRAT( 186 )      ! CL+IOLE=0.3*HCL+0.7*FMCL+...
     &                          +          RXRAT( 187 )      ! CL+ISOP=0.15*HCL+XO2+HO2+...
     &                          +          RXRAT( 188 )      ! CL+FORM=HCL+HO2+CO
     &                          +          RXRAT( 189 )      ! CL+ALD2=HCL+C2O3
     &                          +          RXRAT( 190 )      ! CL+ALDX=HCL+CXO3
     &                          +          RXRAT( 191 )      ! CL+MEOH=HCL+HO2+FORM
     &                          +          RXRAT( 192 )      ! CL+ETOH=HCL+HO2+ALD2
     &                          +          RXRAT( 194 )      ! CL+TOL=HCL+0.88*XO2+...
     &                          +          RXRAT( 195 )      ! CL+XYL=HCL+0.84*XO2+...
     &                          +          RXRAT( 253 )      ! CL+HG=0.5*HG+0.5*HGIIGAS

      PROD( HOCL             ) =           RXRAT( 178 )      ! CLO+HO2=HOCL
      LOSS( HOCL             ) =           RXRAT( 174 )      ! HOCL=OH+CL

      PROD( CLO              ) =           RXRAT( 175 )      ! CL+O3=CLO
      LOSS( CLO              ) =   2.000 * RXRAT( 176 )      ! CLO+CLO=0.3*CL2+1.4*CL
     &                          +          RXRAT( 177 )      ! CLO+NO=CL+NO2
     &                          +          RXRAT( 178 )      ! CLO+HO2=HOCL

      PROD( FMCL             ) =           RXRAT( 184 )      ! CL+ETH=FMCL+2*XO2+HO2+FORM
     &                          +          RXRAT( 185 )      ! CL+OLE=FMCL+0.33*ALD2+...
     &                          +  0.700 * RXRAT( 186 )      ! CL+IOLE=0.7*FMCL+0.3*HCL+...
     &                          +  0.850 * RXRAT( 187 )      ! CL+ISOP=0.85*FMCL+XO2+HO2+...
      LOSS( FMCL             ) =           RXRAT( 179 )      ! FMCL+OH=CL+CO
     &                          +          RXRAT( 180 )      ! FMCL=CL+CO+HO2

      PROD( HCL              ) =           RXRAT( 181 )      ! CL=HCL+MEO2
     &                          +          RXRAT( 182 )      ! CL+PAR=HCL+0.87*XO2+...
     &                          +          RXRAT( 183 )      ! CL+ETHA=HCL+0.991*ALD2+...
     &                          +  0.300 * RXRAT( 186 )      ! CL+IOLE=0.3*HCL+0.7*FMCL+...
     &                          +  0.150 * RXRAT( 187 )      ! CL+ISOP=0.15*HCL+XO2+HO2+...
     &                          +          RXRAT( 188 )      ! CL+FORM=HCL+HO2+CO
     &                          +          RXRAT( 189 )      ! CL+ALD2=HCL+C2O3
     &                          +          RXRAT( 190 )      ! CL+ALDX=HCL+CXO3
     &                          +          RXRAT( 191 )      ! CL+MEOH=HCL+HO2+FORM
     &                          +          RXRAT( 192 )      ! CL+ETOH=HCL+HO2+ALD2
     &                          +          RXRAT( 194 )      ! CL+TOL=HCL+0.88*XO2+...
     &                          +          RXRAT( 195 )      ! CL+XYL=HCL+0.84*XO2+...
      LOSS( HCL              ) =           RXRAT( 193 )      ! HCL+OH=CL

      PROD( TOLNRXN          ) =           RXRAT( 196 )      ! TOLRO2+NO=TOLNRXN+NO
      LOSS( TOLNRXN          ) =   0.0

      PROD( TOLHRXN          ) =           RXRAT( 197 )      ! TOLRO2+HO2=TOLHRXN+HO2
      LOSS( TOLHRXN          ) =   0.0

      PROD( XYLNRXN          ) =           RXRAT( 198 )      ! XYLRO2+NO=XYLNRXN+NO
      LOSS( XYLNRXN          ) =   0.0

      PROD( XYLHRXN          ) =           RXRAT( 199 )      ! XYLRO2+HO2=XYLHRXN+HO2
      LOSS( XYLHRXN          ) =   0.0

      PROD( BENZENE          ) =   0.0
      LOSS( BENZENE          ) =           RXRAT( 200 )      ! BENZENE+OH=OH+BENZRO2

      PROD( BENZRO2          ) =           RXRAT( 200 )      ! BENZENE+OH=BENZRO2+OH
      LOSS( BENZRO2          ) =           RXRAT( 201 )      ! BENZRO2+NO=NO+BNZNRXN
     &                          +          RXRAT( 202 )      ! BENZRO2+HO2=HO2+BNZHRXN

      PROD( BNZNRXN          ) =           RXRAT( 201 )      ! BENZRO2+NO=BNZNRXN+NO
      LOSS( BNZNRXN          ) =   0.0

      PROD( BNZHRXN          ) =           RXRAT( 202 )      ! BENZRO2+HO2=BNZHRXN+HO2
      LOSS( BNZHRXN          ) =   0.0

      PROD( SESQ             ) =   0.0
      LOSS( SESQ             ) =           RXRAT( 203 )      ! SESQ+O3=O3+SESQRXN
     &                          +          RXRAT( 204 )      ! SESQ+OH=OH+SESQRXN
     &                          +          RXRAT( 205 )      ! SESQ+NO3=NO3+SESQRXN

      PROD( SESQRXN          ) =           RXRAT( 203 )      ! SESQ+O3=SESQRXN+O3
     &                          +          RXRAT( 204 )      ! SESQ+OH=SESQRXN+OH
     &                          +          RXRAT( 205 )      ! SESQ+NO3=SESQRXN+NO3
      LOSS( SESQRXN          ) =   0.0

      PROD( FORM_PRIMARY     ) =   0.0
      LOSS( FORM_PRIMARY     ) =           RXRAT( 206 )      ! FORM_PRIMARY+OH=OH
     &                          +          RXRAT( 207 )      ! FORM_PRIMARY+NO3=NO3
     &                          +          RXRAT( 208 )      ! FORM_PRIMARY+O=O
     &                          +          RXRAT( 209 )      ! FORM_PRIMARY=
     &                          +          RXRAT( 210 )      ! FORM_PRIMARY=
     &                          +          RXRAT( 211 )      ! FORM_PRIMARY+CL=CL

      PROD( ALD2_PRIMARY     ) =   0.0
      LOSS( ALD2_PRIMARY     ) =           RXRAT( 212 )      ! ALD2_PRIMARY+OH=OH
     &                          +          RXRAT( 213 )      ! ALD2_PRIMARY+NO3=NO3
     &                          +          RXRAT( 214 )      ! ALD2_PRIMARY+O=O
     &                          +          RXRAT( 215 )      ! ALD2_PRIMARY=
     &                          +          RXRAT( 216 )      ! ALD2_PRIMARY+CL=CL

      PROD( BUTADIENE13      ) =   0.0
      LOSS( BUTADIENE13      ) =           RXRAT( 217 )      ! BUTADIENE13+OH=OH+...
     &                          +          RXRAT( 218 )      ! BUTADIENE13+O3=O3+...
     &                          +          RXRAT( 219 )      ! BUTADIENE13+NO3=NO3+...
     &                          +          RXRAT( 220 )      ! BUTADIENE13+CL=CL+...

      PROD( ACROLEIN         ) =   0.580 * RXRAT( 217 )      ! BUTADIENE13+OH=...
     &                          +  0.520 * RXRAT( 218 )      ! BUTADIENE13+O3=...
     &                          +  0.045 * RXRAT( 219 )      ! BUTADIENE13+NO3=...
     &                          +  0.580 * RXRAT( 220 )      ! BUTADIENE13+CL=...
      LOSS( ACROLEIN         ) =           RXRAT( 226 )      ! ACROLEIN+OH=OH
     &                          +          RXRAT( 227 )      ! ACROLEIN+O3=O3
     &                          +          RXRAT( 228 )      ! ACROLEIN+NO3=NO3
     &                          +          RXRAT( 229 )      ! ACROLEIN=
     &                          +          RXRAT( 230 )      ! ACROLEIN+CL=CL

      PROD( ACROLEIN_PRIMARY ) =   0.0
      LOSS( ACROLEIN_PRIMARY ) =           RXRAT( 221 )      ! ACROLEIN_PRIMARY+OH=OH
     &                          +          RXRAT( 222 )      ! ACROLEIN_PRIMARY+O3=O3
     &                          +          RXRAT( 223 )      ! ACROLEIN_PRIMARY+NO3=NO3
     &                          +          RXRAT( 224 )      ! ACROLEIN_PRIMARY=
     &                          +          RXRAT( 225 )      ! ACROLEIN_PRIMARY+CL=CL

      PROD( TOLU             ) =   0.0
      LOSS( TOLU             ) =           RXRAT( 231 )      ! TOLU+OH=OH
     &                          +          RXRAT( 232 )      ! TOLU+CL=CL

      PROD( MXYL             ) =   0.0
      LOSS( MXYL             ) =           RXRAT( 233 )      ! MXYL+OH=OH
     &                          +          RXRAT( 234 )      ! MXYL+CL=CL

      PROD( OXYL             ) =   0.0
      LOSS( OXYL             ) =           RXRAT( 235 )      ! OXYL+OH=OH
     &                          +          RXRAT( 236 )      ! OXYL+CL=CL

      PROD( PXYL             ) =   0.0
      LOSS( PXYL             ) =           RXRAT( 237 )      ! PXYL+OH=OH
     &                          +          RXRAT( 238 )      ! PXYL+CL=CL

      PROD( APIN             ) =   0.0
      LOSS( APIN             ) =           RXRAT( 239 )      ! APIN+O=O
     &                          +          RXRAT( 240 )      ! APIN+OH=OH
     &                          +          RXRAT( 241 )      ! APIN+O3=O3
     &                          +          RXRAT( 242 )      ! APIN+NO3=NO3
     &                          +          RXRAT( 243 )      ! APIN+CL=CL

      PROD( BPIN             ) =   0.0
      LOSS( BPIN             ) =           RXRAT( 244 )      ! BPIN+O=O
     &                          +          RXRAT( 245 )      ! BPIN+OH=OH
     &                          +          RXRAT( 246 )      ! BPIN+O3=O3
     &                          +          RXRAT( 247 )      ! BPIN+NO3=NO3
     &                          +          RXRAT( 248 )      ! BPIN+CL=CL

      PROD( HG               ) =   0.0
      LOSS( HG               ) =           RXRAT( 249 )      ! HG+O3=0.5*HGIIAER+0.5*HGIIGAS
     &                          +          RXRAT( 250 )      ! HG+CL2=HGIIGAS
     &                          +          RXRAT( 251 )      ! HG+H2O2=HGIIGAS
     &                          +          RXRAT( 252 )      ! HG+OH=0.5*HGIIAER+0.5*HGIIGAS
     &                          +  0.500 * RXRAT( 253 )      ! HG+CL=0.5*HG+0.5*HGIIGAS

      PROD( HGIIAER          ) =   0.500 * RXRAT( 249 )      ! HG+O3=0.5*HGIIAER+0.5*HGIIGAS
     &                          +  0.500 * RXRAT( 252 )      ! HG+OH=0.5*HGIIAER+0.5*HGIIGAS
      LOSS( HGIIAER          ) =   0.0

      PROD( HGIIGAS          ) =   0.500 * RXRAT( 249 )      ! HG+O3=0.5*HGIIGAS+0.5*HGIIAER
     &                          +          RXRAT( 250 )      ! HG+CL2=HGIIGAS
     &                          +          RXRAT( 251 )      ! HG+H2O2=HGIIGAS
     &                          +  0.500 * RXRAT( 252 )      ! HG+OH=0.5*HGIIGAS+0.5*HGIIAER
     &                          +  0.500 * RXRAT( 253 )      ! HG+CL=0.5*HGIIGAS+0.5*HG
      LOSS( HGIIGAS          ) =   0.0

      PNEG( PAR ) =   0.660 * RXRAT(  61 )      ! NTR+OH=-0.66*PAR+HO2+...
     &             +  0.660 * RXRAT(  62 )      ! NTR=-0.66*PAR+HO2+...
     &             +  2.100 * RXRAT( 113 )      ! ROR=-2.1*PAR+0.6*ALD2+...
     &             +  0.700 * RXRAT( 117 )      ! OH+OLE=-0.7*PAR+0.33*ALD2+...
     &             +          RXRAT( 118 )      ! O3+OLE=-PAR+0.74*FORM+...
     &             +          RXRAT( 119 )      ! NO3+OLE=-PAR+FORM+0.91*XO2+...
     &             +          RXRAT( 185 )      ! CL+OLE=-PAR+0.33*ALD2+...

      RETURN

      END
