
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header$

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

      SUBROUTINE HRG2( DTC )
C**********************************************************************
C
C  FUNCTION: To solve for the concentration of HO, HO2, HONO, and
C            HNO4 alebraically.
C
C  PRECONDITIONS: For the SAPRC07TC family of mechanisms
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, June 1, 2012
C
C**********************************************************************
      USE HRDATA

      IMPLICIT NONE


C..INCLUDES: None


C..ARGUMENTS:
      REAL DTC                      ! Time step

C..PARAMETERS: None


C..EXTERNAL FUNCTIONS: NONE


C..SAVED LOCAL VARIABLES:
      CHARACTER( 16 ), SAVE :: PNAME = 'HRG2'             ! Program name


C..SCRATCH LOCAL VARIABLES:
      REAL    O1D_S                 ! sum of O1D loss frequencies
      REAL    OH_S                  ! stoich coeff for OH from O1D+H2O or H2
      REAL    HO2_S                 ! stoich coeff for HO2 from O1D+H2 rxn
      REAL    EXN_S                 ! sum of NO2EX loss frequencies
      REAL    XOH_S                 ! stoich coeff for OH & HONO from NO2EX loss rxn
      REAL    R4_19                 ! production of OH from HONO
      REAL    R19_4                 ! production of HONO from OH
      REAL    R4_5                  ! production of OH from HO2
      REAL    R5_4                  ! production of HO2 from OH
      REAL    R5_21                 ! production of HO2 from HNO4
      REAL    R21_5                 ! production of HNO4 from HO2
      REAL    P4, P5, P19, P21      ! species production form other paths
      REAL    L4, L5, L19, L21      ! species loss terms

      REAL    A, B, C               ! coeffs of quadratic eq. for HO2
      REAL    Q                     ! intermediate term

      REAL    T1, T2, T3            ! intermediate terms

      REAL    L21_INV               ! reciprocal of HNO4 loss term

C**********************************************************************


c..stoichiometric coefficient for production of HOX from O3 via O1D
      O1D_S =           RKI(  20 )                            ! O1D=2*OH
     &       +          RKI(  21 )                            ! O1D=O3P

      OH_S =   2.000 * RKI(  20 )                            ! O1D=2*OH

      OH_S  = OH_S / O1D_S


c..Production of HO from HO2 (r4,5 terms )
      R4_5 =           RKI(  31 ) * YC( NO               )   ! HO2+NO=OH+NO2
     &      +          RKI(  36 ) * YC( O3               )   ! HO2+O3=OH
     &      +  0.800 * RKI(  39 ) * YC( NO3              )   ! HO2+NO3=0.8*OH+...
     &      +          RKI( 578 ) * YC( CL               )   ! HO2+CL=OH+CLO

      R4_5  = R4_5 * DTC

c..Production of HO from HONO (r4,19 terms )
      R4_19 =           RKI(  23 )                            ! HONO=OH+NO

      R4_19 = R4_19 * DTC

c..Remaining HO production
      P4 =   OH_S  * RXRAT(  18 )      ! O1D=2*OH
     &    +          RXRAT(  28 )      ! HNO3=OH+NO2
     &    +  0.390 * RXRAT(  34 )      ! HNO4=0.39*OH+0.61*HO2+...
     &    +  2.000 * RXRAT(  41 )      ! HO2H=2*OH
     &    +          RXRAT(  49 )      ! NO2EX=OH+HONO
     &    +          RXRAT( 128 )      ! xOH+NO=OH+NO
     &    +          RXRAT( 130 )      ! xOH+NO3=OH+NO3
     &    +  0.500 * RXRAT( 131 )      ! xOH+MEO2=0.5*OH+MEO2
     &    +  0.500 * RXRAT( 132 )      ! xOH+RO2C=0.5*OH+RO2C
     &    +  0.500 * RXRAT( 133 )      ! xOH+RO2XC=0.5*OH+RO2XC
     &    +          RXRAT( 134 )      ! xOH+MECO3=OH+MECO3
     &    +          RXRAT( 135 )      ! xOH+RCO3=OH+RCO3
     &    +          RXRAT( 136 )      ! xOH+BZCO3=OH+BZCO3
     &    +          RXRAT( 137 )      ! xOH+MACO3=OH+MACO3
     &    +          RXRAT( 227 )      ! COOH=OH+HO2+HCHO
     &    +          RXRAT( 229 )      ! ROOH=OH+HO2+RCHO
     &    +          RXRAT( 231 )      ! R6OOH=OH+0.142*HO2+...
     &    +          RXRAT( 233 )      ! RAOOH=OH+HO2+0.5*GLY+...
     &    +  0.826 * RXRAT( 251 )      ! AFG1+O3=0.826*OH+0.522*HO2+...
     &    +  0.826 * RXRAT( 254 )      ! AFG2+O3=0.826*OH+0.522*HO2+...
     &    +  0.471 * RXRAT( 257 )      ! AFG3+O3=0.471*OH+0.554*HO2+...
     &    +  0.208 * RXRAT( 259 )      ! MACR+O3=0.208*OH+0.108*HO2+...
     &    +  0.330 * RXRAT( 262 )      ! MACR=0.33*OH+0.67*HO2+...
     &    +  0.164 * RXRAT( 264 )      ! MVK+O3=0.164*OH+0.064*HO2+...
     &    +  0.285 * RXRAT( 268 )      ! IPRD+O3=0.285*OH+0.4*HO2+...
     &    +  0.330 * RXRAT( 279 )      ! ACROLEIN+O3=0.33*OH+...
     &    +  0.178 * RXRAT( 282 )      ! ACROLEIN=0.178*OH+...
     &    +          RXRAT( 284 )      ! CCOOOH=OH+CO2+MEO2
     &    +          RXRAT( 286 )      ! RCOOOH=OH+xCCHO+yROOH+CO2+xHO2
     &    +  0.160 * RXRAT( 519 )      ! ETHENE+O3=0.16*OH+0.16*HO2+...
     &    +  0.350 * RXRAT( 523 )      ! PROPENE+O3=0.35*OH+...
     &    +  0.080 * RXRAT( 527 )      ! BUTADIENE13+O3=0.08*OH+...
     &    +  0.266 * RXRAT( 531 )      ! ISOPRENE+O3=0.266*OH+...
     &    +  0.728 * RXRAT( 535 )      ! APIN+O3=0.728*OH+0.009*HO2+...
     &    +  0.500 * RXRAT( 539 )      ! ACETYLENE+O3=0.5*OH+...
     &    +  0.128 * RXRAT( 553 )      ! OLE1+O3=0.128*OH+0.095*HO2+...
     &    +  0.443 * RXRAT( 557 )      ! OLE2+O3=0.443*OH+0.094*HO2+...
     &    +  0.499 * RXRAT( 563 )      ! TERP+O3=0.499*OH+0.078*HO2+...
     &    +  0.499 * RXRAT( 567 )      ! SESQ+O3=0.499*OH+0.078*HO2+...
     &    +          RXRAT( 588 )      ! HOCL=OH+CL
     &    +  0.414 * RXRAT( 604 )      ! ROOH+CL=0.414*OH+HCL+...
     &    +  0.145 * RXRAT( 605 )      ! R6OOH+CL=0.145*OH+HCL+...
     &    +  0.139 * RXRAT( 606 )      ! RAOOH+CL=0.139*OH+...

      P4 = YC0( OH ) + P4 * DTC

c..Production of HO2 from OH ( r5,4 terms )
      R5_4 =           RKI(  26 ) * YC( NO3              )   ! OH+NO3=HO2+NO2
     &      +          RKI(  29 ) * YC( CO               )   ! OH+CO=HO2+CO2
     &      +          RKI(  30 ) * YC( O3               )   ! OH+O3=HO2
     &      +          RKI(  42 ) * YC( HO2H             )   ! OH+HO2H=HO2
     &      +          RKI(  44 ) * YC( SO2              )   ! OH+SO2=HO2+SULF+...
     &      +          RKI(  45 )                            ! OH=HO2
     &      +          RKI( 210 ) * YC( HCHO             )   ! OH+HCHO=HO2+CO
     &      +          RKI( 222 ) * YC( MEOH             )   ! OH+MEOH=HO2+HCHO
     &      +          RKI( 223 ) * YC( HCOOH            )   ! OH+HCOOH=HO2+CO2
     &      +  0.148 * RKI( 232 ) * YC( RAOOH            )   ! OH+RAOOH=...
     &      +  0.630 * RKI( 236 ) * YC( GLY              )   ! OH+GLY=0.63*HO2+...
     &      +  0.472 * RKI( 271 ) * YC( PRD2             )   ! OH+PRD2=...
     &      +  0.189 * RKI( 273 ) * YC( RNO3             )   ! OH+RNO3=...
     &      +  0.300 * RKI( 538 ) * YC( ACETYLENE        )   ! OH+ACETYLENE=...
     &      +  0.570 * RKI( 540 ) * YC( BENZENE          )   ! OH+BENZENE=...
     &      +  0.181 * RKI( 541 ) * YC( TOLUENE          )   ! OH+TOLUENE=...
     &      +  0.159 * RKI( 542 ) * YC( MXYL             )   ! OH+MXYL=...
     &      +  0.161 * RKI( 543 ) * YC( OXYL             )   ! OH+OXYL=...
     &      +  0.159 * RKI( 544 ) * YC( PXYL             )   ! OH+PXYL=...
     &      +  0.022 * RKI( 545 ) * YC( TRIMETH_BENZ124  )   ! OH+...
     &      +  0.950 * RKI( 546 ) * YC( ETOH             )   ! OH+ETOH=0.95*HO2+...
     &      +  0.123 * RKI( 560 ) * YC( ARO1             )   ! OH+ARO1=...
     &      +  0.077 * RKI( 561 ) * YC( ARO2             )   ! OH+ARO2=...

      R5_4  = R5_4 * DTC

c..Production of HO2 from HNO4 (r5,21 term )
      R5_21 =           RKI(  33 )                            ! HNO4=HO2+NO2
     &       +  0.610 * RKI(  34 )                            ! HNO4=0.61*HO2+...

      R5_21 = R5_21 * DTC

c..Remaining HO2 production terms
      P5 =           RXRAT(  50 )      ! MEO2+NO=HO2+HCHO+NO2
     &    +          RXRAT(  53 )      ! MEO2+NO3=HO2+HCHO+NO2
     &    +  2.000 * RXRAT(  55 )      ! MEO2+MEO2=2*HO2+2*HCHO
     &    +  0.500 * RXRAT(  59 )      ! RO2C+MEO2=0.5*HO2+...
     &    +  0.500 * RXRAT(  64 )      ! RO2XC+MEO2=0.5*HO2+...
     &    +  0.900 * RXRAT(  73 )      ! MECO3+MEO2=0.9*HO2+...
     &    +          RXRAT(  83 )      ! RCO3+MEO2=HO2+HCHO+RO2C+...
     &    +          RXRAT(  94 )      ! BZCO3+MEO2=HO2+HCHO+RO2C+...
     &    +          RXRAT( 106 )      ! MACO3+MEO2=HO2+2*HCHO+CO2+...
     &    +          RXRAT( 118 )      ! xHO2+NO=HO2+NO
     &    +          RXRAT( 120 )      ! xHO2+NO3=HO2+NO3
     &    +  0.500 * RXRAT( 121 )      ! xHO2+MEO2=0.5*HO2+MEO2
     &    +  0.500 * RXRAT( 122 )      ! xHO2+RO2C=0.5*HO2+RO2C
     &    +  0.500 * RXRAT( 123 )      ! xHO2+RO2XC=0.5*HO2+RO2XC
     &    +          RXRAT( 124 )      ! xHO2+MECO3=HO2+MECO3
     &    +          RXRAT( 125 )      ! xHO2+RCO3=HO2+RCO3
     &    +          RXRAT( 126 )      ! xHO2+BZCO3=HO2+BZCO3
     &    +          RXRAT( 127 )      ! xHO2+MACO3=HO2+MACO3
     &    +  2.000 * RXRAT( 208 )      ! HCHO=2*HO2+CO
     &    +          RXRAT( 211 )      ! HCHO+NO3=HO2+HNO3+CO
     &    +          RXRAT( 213 )      ! CCHO=HO2+CO+MEO2
     &    +          RXRAT( 216 )      ! RCHO=HO2+xHO2+yROOH+xCCHO+...
     &    +          RXRAT( 227 )      ! COOH=HO2+OH+HCHO
     &    +          RXRAT( 229 )      ! ROOH=HO2+OH+RCHO
     &    +  0.142 * RXRAT( 231 )      ! R6OOH=0.142*HO2+OH+...
     &    +          RXRAT( 233 )      ! RAOOH=HO2+OH+0.5*GLY+...
     &    +  2.000 * RXRAT( 234 )      ! GLY=2*HO2+2*CO
     &    +  0.630 * RXRAT( 237 )      ! GLY+NO3=0.63*HO2+HNO3+...
     &    +          RXRAT( 238 )      ! MGLY=HO2+CO+MECO3
     &    +  0.522 * RXRAT( 251 )      ! AFG1+O3=0.522*HO2+0.826*OH+...
     &    +  1.023 * RXRAT( 252 )      ! AFG1=1.023*HO2+0.173*MEO2+...
     &    +  0.522 * RXRAT( 254 )      ! AFG2+O3=0.522*HO2+0.826*OH+...
     &    +  0.554 * RXRAT( 257 )      ! AFG3+O3=0.554*HO2+0.471*OH+...
     &    +  0.108 * RXRAT( 259 )      ! MACR+O3=0.108*HO2+0.208*OH+...
     &    +  0.670 * RXRAT( 262 )      ! MACR=0.67*HO2+0.33*OH+...
     &    +  0.064 * RXRAT( 264 )      ! MVK+O3=0.064*HO2+0.164*OH+...
     &    +  0.400 * RXRAT( 268 )      ! IPRD+O3=0.4*HO2+0.285*OH+...
     &    +  1.233 * RXRAT( 270 )      ! IPRD=1.233*HO2+0.467*MECO3+...
     &    +  0.344 * RXRAT( 274 )      ! RNO3=0.344*HO2+0.554*xHO2+...
     &    +  2.000 * RXRAT( 276 )      ! HOCCHO=2*HO2+CO+HCHO
     &    +  0.830 * RXRAT( 279 )      ! ACROLEIN+O3=0.83*HO2+...
     &    +  1.066 * RXRAT( 282 )      ! ACROLEIN=1.066*HO2+...
     &    +          RXRAT( 489 )      ! zRNO3+NO3=HO2+PRD2+NO3
     &    +  0.500 * RXRAT( 490 )      ! zRNO3+MEO2=0.5*HO2+...
     &    +  0.500 * RXRAT( 491 )      ! zRNO3+RO2C=0.5*HO2+...
     &    +  0.500 * RXRAT( 492 )      ! zRNO3+RO2XC=0.5*HO2+...
     &    +          RXRAT( 493 )      ! zRNO3+MECO3=HO2+PRD2+MECO3
     &    +          RXRAT( 494 )      ! zRNO3+RCO3=HO2+PRD2+RCO3
     &    +          RXRAT( 495 )      ! zRNO3+BZCO3=HO2+PRD2+BZCO3
     &    +          RXRAT( 496 )      ! zRNO3+MACO3=HO2+PRD2+MACO3
     &    +  0.160 * RXRAT( 519 )      ! ETHENE+O3=0.16*HO2+0.16*OH+...
     &    +  0.800 * RXRAT( 521 )      ! ETHENE+O3P=0.8*HO2+...
     &    +  0.165 * RXRAT( 523 )      ! PROPENE+O3=0.165*HO2+...
     &    +  0.080 * RXRAT( 527 )      ! BUTADIENE13+O3=0.08*HO2+...
     &    +  0.250 * RXRAT( 529 )      ! BUTADIENE13+O3P=0.25*HO2+...
     &    +  0.066 * RXRAT( 531 )      ! ISOPRENE+O3=0.066*HO2+...
     &    +  0.009 * RXRAT( 535 )      ! APIN+O3=0.009*HO2+0.728*OH+...
     &    +  1.500 * RXRAT( 539 )      ! ACETYLENE+O3=1.5*HO2+...
     &    +  0.095 * RXRAT( 553 )      ! OLE1+O3=0.095*HO2+0.128*OH+...
     &    +  0.094 * RXRAT( 557 )      ! OLE2+O3=0.094*HO2+0.443*OH+...
     &    +  0.078 * RXRAT( 563 )      ! TERP+O3=0.078*HO2+0.499*OH+...
     &    +  0.078 * RXRAT( 567 )      ! SESQ+O3=0.078*HO2+0.499*OH+...
     &    +          RXRAT( 591 )      ! CL=HO2+HCL
     &    +          RXRAT( 592 )      ! HCHO+CL=HO2+HCL+CO
     &    +          RXRAT( 594 )      ! MEOH+CL=HO2+HCHO+HCL
     &    +  0.055 * RXRAT( 598 )      ! RNO3+CL=0.055*HO2+...
     &    +  0.314 * RXRAT( 599 )      ! PRD2+CL=0.314*HO2+HCL+...
     &    +  0.630 * RXRAT( 600 )      ! GLY+CL=0.63*HO2+HCL+...
     &    +  0.148 * RXRAT( 606 )      ! RAOOH+CL=0.148*HO2+...
     &    +  0.084 * RXRAT( 610 )      ! IPRD+CL=0.084*HO2+...
     &    +          RXRAT( 611 )      ! CLCCHO=HO2+CO+RO2C+xCL+...
     &    +          RXRAT( 651 )      ! ACETYLENE+CL=HO2+CO
     &    +  0.688 * RXRAT( 657 )      ! ETOH+CL=0.688*HO2+HCL+...

      P5 = YC0( HO2 ) + P5 * DTC

c..Production of HONO from OH (r19,4 terms )

      R19_4 =   RKI(  22 ) * YC( NO               ) * DTC      ! OH+NO=HONO

c..Remaining HONO production terms
      P19 =           RXRAT(  49 )      ! NO2EX=HONO+OH
     &     +          RXRAT( 245 )      ! NPHE=HONO

      P19 = YC0( HONO ) + P19 * DTC

c..Production of HNO4 from HO2 (r21,5 term )

      R21_5 =   RKI(  32 ) * YC( NO2              ) * DTC      ! HO2+NO2=HNO4

c..Remaining HNO4 production terms
      P21   =   YC0( HNO4 )

c..OH loss terms not in R5_4 & R19_4
      L4 =           RKI(  24 ) * YC( HONO             )   ! OH+HONO=NO2
     &    +          RKI(  25 ) * YC( NO2              )   ! OH+NO2=HNO3
     &    +          RKI(  27 ) * YC( HNO3             )   ! OH+HNO3=NO3
     &    +          RKI(  35 ) * YC( HNO4             )   ! OH+HNO4=NO2
     &    +          RKI(  43 ) * YC( HO2              )   ! OH+HO2=
     &    +          RKI( 212 ) * YC( CCHO             )   ! OH+CCHO=MECO3
     &    +          RKI( 215 ) * YC( RCHO             )   ! OH+RCHO=...
     &    +          RKI( 218 ) * YC( ACETONE          )   ! OH+ACETONE=RO2C+...
     &    +          RKI( 220 ) * YC( MEK              )   ! OH+MEK=...
     &    +          RKI( 224 ) * YC( CCOOH            )   ! OH+CCOOH=...
     &    +          RKI( 225 ) * YC( RCOOH            )   ! OH+RCOOH=RO2C+...
     &    +  0.700 * RKI( 226 ) * YC( COOH             )   ! OH+COOH=0.3*HCHO+...
     &    +  0.256 * RKI( 228 ) * YC( ROOH             )   ! OH+ROOH=0.744*OH+...
     &    +  0.160 * RKI( 230 ) * YC( R6OOH            )   ! OH+R6OOH=0.84*OH+...
     &    +  0.713 * RKI( 232 ) * YC( RAOOH            )   ! OH+RAOOH=...
     &    +  0.370 * RKI( 236 ) * YC( GLY              )   ! OH+GLY=0.63*HO2+...
     &    +          RKI( 239 ) * YC( MGLY             )   ! OH+MGLY=CO+MECO3
     &    +          RKI( 242 ) * YC( CRES             )   ! OH+CRES=0.2*BZO+...
     &    +          RKI( 244 ) * YC( NPHE             )   ! OH+NPHE=BZO
     &    +          RKI( 247 ) * YC( BALD             )   ! OH+BALD=BZCO3
     &    +          RKI( 250 ) * YC( AFG1             )   ! OH+AFG1=...
     &    +          RKI( 253 ) * YC( AFG2             )   ! OH+AFG2=...
     &    +          RKI( 256 ) * YC( AFG3             )   ! OH+AFG3=...
     &    +          RKI( 258 ) * YC( MACR             )   ! OH+MACR=...
     &    +          RKI( 263 ) * YC( MVK              )   ! OH+MVK=...
     &    +          RKI( 267 ) * YC( IPRD             )   ! OH+IPRD=...
     &    +  0.528 * RKI( 271 ) * YC( PRD2             )   ! OH+PRD2=...
     &    +  0.811 * RKI( 273 ) * YC( RNO3             )   ! OH+RNO3=...
     &    +          RKI( 275 ) * YC( HOCCHO           )   ! OH+HOCCHO=MECO3
     &    +          RKI( 278 ) * YC( ACROLEIN         )   ! OH+ACROLEIN=...
     &    +          RKI( 283 ) * YC( CCOOOH           )   ! OH+CCOOOH=...
     &    +          RKI( 285 ) * YC( RCOOOH           )   ! OH+RCOOOH=...
     &    +          RKI( 517 )                            ! OH=MEO2
     &    +          RKI( 518 ) * YC( ETHENE           )   ! OH+ETHENE=xHO2+...
     &    +          RKI( 522 ) * YC( PROPENE          )   ! OH+PROPENE=...
     &    +          RKI( 526 ) * YC( BUTADIENE13      )   ! OH+BUTADIENE13=...
     &    +          RKI( 530 ) * YC( ISOPRENE         )   ! OH+ISOPRENE=...
     &    +          RKI( 534 ) * YC( APIN             )   ! OH+APIN=...
     &    +  0.314 * RKI( 540 ) * YC( BENZENE          )   ! OH+BENZENE=...
     &    +  0.507 * RKI( 541 ) * YC( TOLUENE          )   ! OH+TOLUENE=...
     &    +  0.602 * RKI( 542 ) * YC( MXYL             )   ! OH+MXYL=...
     &    +  0.641 * RKI( 543 ) * YC( OXYL             )   ! OH+OXYL=...
     &    +  0.563 * RKI( 544 ) * YC( PXYL             )   ! OH+PXYL=...
     &    +  0.748 * RKI( 545 ) * YC( TRIMETH_BENZ124  )   ! OH+...
     &    +  0.050 * RKI( 546 ) * YC( ETOH             )   ! OH+ETOH=0.95*HO2+...
     &    +          RKI( 547 ) * YC( ALK1             )   ! OH+ALK1=xHO2+...
     &    +          RKI( 548 ) * YC( ALK2             )   ! OH+ALK2=...
     &    +          RKI( 549 ) * YC( ALK3             )   ! OH+ALK3=...
     &    +          RKI( 550 ) * YC( ALK4             )   ! OH+ALK4=...
     &    +          RKI( 551 ) * YC( ALK5             )   ! OH+ALK5=...
     &    +          RKI( 552 ) * YC( OLE1             )   ! OH+OLE1=...
     &    +          RKI( 556 ) * YC( OLE2             )   ! OH+OLE2=...
     &    +  0.675 * RKI( 560 ) * YC( ARO1             )   ! OH+ARO1=...
     &    +  0.745 * RKI( 561 ) * YC( ARO2             )   ! OH+ARO2=...
     &    +          RKI( 562 ) * YC( TERP             )   ! OH+TERP=...
     &    +          RKI( 566 ) * YC( SESQ             )   ! OH+SESQ=...
     &    +          RKI( 590 ) * YC( HCL              )   ! OH+HCL=CL
     &    +          RKI( 612 ) * YC( CLCCHO           )   ! OH+CLCCHO=RCO3

      L4    = 1.0 + L4 * DTC + R5_4 + R19_4

c..HO2 loss terms not included in R4_5 & R21_5 (except for HO2+HO2 )
      L5 =   0.200 * RKI(  39 ) * YC( NO3              )   ! HO2+NO3=0.8*OH+...
     &    +          RKI(  43 ) * YC( OH               )   ! HO2+OH=
     &    +          RKI(  51 ) * YC( MEO2             )   ! HO2+MEO2=COOH
     &    +          RKI(  52 ) * YC( MEO2             )   ! HO2+MEO2=HCHO
     &    +          RKI(  57 ) * YC( RO2C             )   ! HO2+RO2C=
     &    +          RKI(  62 ) * YC( RO2XC            )   ! HO2+RO2XC=
     &    +          RKI(  71 ) * YC( MECO3            )   ! HO2+MECO3=...
     &    +          RKI(  81 ) * YC( RCO3             )   ! HO2+RCO3=...
     &    +          RKI(  92 ) * YC( BZCO3            )   ! HO2+BZCO3=...
     &    +          RKI( 104 ) * YC( MACO3            )   ! HO2+MACO3=...
     &    +          RKI( 116 ) * YC( BZO              )   ! HO2+BZO=CRES
     &    +          RKI( 577 ) * YC( CL               )   ! HO2+CL=HCL
     &    +          RKI( 587 ) * YC( CLO              )   ! HO2+CLO=HOCL

      L5    = 1.0 + L5 * DTC + R4_5 + R21_5

c..HONO loss terms not included in R4_19
      L19 =           RKI(  24 ) * YC( OH               )   ! HONO+OH=NO2

      L19   = 1.0 + L19 * DTC + R4_19

c..HNO4 loss terms not inluded in R5_21
      L21 =   0.390 * RKI(  34 )                            ! HNO4=0.61*HO2+...
     &     +          RKI(  35 ) * YC( OH               )   ! HNO4+OH=NO2

      L21   = 1.0 + L21 * DTC + R5_21

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Solution section
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c..compute terms used to calculate a,b & c
      L21_INV = 1.0 / L21
      T1 = 1.0 / ( L4 * L19 - R4_19 * R19_4 )
      T2 = R5_4 * T1
      T3 = R5_21 * L21_INV

c..solve quadratic equation for HO2
      A = 2.0 * ( RKI(  37 ) + RKI(  38 ) ) * DTC

      B = L5 - T3 * R21_5 - T2 * R4_5 * L19

      C = P5 + T3 * P21 + T2 * ( P4 * L19 + P19 * R4_19 )

      Q = -0.5 * ( B + SIGN( 1.0, B ) * SQRT( B * B + 4.0 * A * C ) )

      YCP( HO2 ) = MAX( Q / A , -C / Q  )

c..compute remaining species concentrations
      YCP( OH ) = ( ( P4 + R4_5 * YCP( HO2 ) ) * L19 + R4_19 * P19 ) * T1

      YCP( HNO4 ) = ( P21 + R21_5 * YCP( HO2 ) ) * L21_INV

      YCP( HONO ) = ( P19 + R19_4 * YCP( OH ) ) / L19

      RETURN

      END


