
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!


C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header: /project/yoj/arc/CCTM/src/gas/ebi_saprc99/hrprodloss.F,v 1.3 2011/10/21 16:11:06 yoj Exp $

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

      SUBROUTINE HRPRODLOSS

C***********************************************************************
C
C  FUNCTION: To compute the production and loss rates for EBI species
C
C  PRECONDITIONS: For the SAPRC99 family of mechanisms
C
C  RETURN VALUES: None
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, Feb. 11, 2011
C
C***********************************************************************
      USE HRDATA

      IMPLICIT NONE


C.....INCLUDES: NONE


C.....ARGUMENTS: NONE


C.....PARAMETERS: NONE


C.....EXTERNAL FUNCTIONS: NONE


C.....SAVED VARIABLES: NONE
      CHARACTER( 16 ), SAVE  ::  PNAME = 'HRPRODLOSS'   ! Program name


C.....LOCAL VARIABLES:

      INTEGER ESP

C***********************************************************************


      PROD( HNO3     ) =   2.000 * RXRAT(  13 )      ! N2O5=2*HNO3
     &                  +          RXRAT(  25 )      ! HO+NO2=HNO3
     &                  +  0.200 * RXRAT(  39 )      ! NO3+HO2=0.2*HNO3+0.8*NO2+...
     &                  +          RXRAT( 129 )      ! HCHO+NO3=HNO3+HO2+CO
     &                  +          RXRAT( 132 )      ! CCHO+NO3=HNO3+CCO_O2
     &                  +          RXRAT( 135 )      ! RCHO+NO3=HNO3+RCO_O2
     &                  +          RXRAT( 148 )      ! GLY+NO3=HNO3+0.63*HO2+...
     &                  +          RXRAT( 151 )      ! MGLY+NO3=HNO3+CO+CCO_O2
     &                  +          RXRAT( 154 )      ! PHEN+NO3=HNO3+BZ_O
     &                  +          RXRAT( 156 )      ! CRES+NO3=HNO3+BZ_O
     &                  +          RXRAT( 157 )      ! NPHE+NO3=HNO3+BZNO2_O
     &                  +          RXRAT( 160 )      ! BALD+NO3=HNO3+BZCO_O2
     &                  +  0.500 * RXRAT( 163 )      ! METHACRO+NO3=0.5*HNO3+...
     &                  +  0.150 * RXRAT( 172 )      ! ISOPROD+NO3=0.15*HNO3+...
      LOSS( HNO3     ) =           RXRAT(  27 )      ! HNO3+HO=NO3
     &                  +          RXRAT(  28 )      ! HNO3=HO+NO2

      PROD( CO       ) =           RXRAT( 123 )      ! HCHO=CO+2*HO2
     &                  +          RXRAT( 124 )      ! HCHO=CO
     &                  +          RXRAT( 125 )      ! HCHO+HO=CO+HO2
     &                  +          RXRAT( 129 )      ! HCHO+NO3=CO+HO2+HNO3
     &                  +          RXRAT( 131 )      ! CCHO=CO+HO2+C_O2
     &                  +  0.034 * RXRAT( 133 )      ! RCHO+HO=0.034*CO+...
     &                  +          RXRAT( 134 )      ! RCHO=CO+RO2_R+CCHO+HO2
     &                  +  2.000 * RXRAT( 145 )      ! GLY=2*CO+2*HO2
     &                  +          RXRAT( 146 )      ! GLY=CO+HCHO
     &                  +  1.260 * RXRAT( 147 )      ! GLY+HO=1.26*CO+0.63*HO2+...
     &                  +  1.260 * RXRAT( 148 )      ! GLY+NO3=1.26*CO+0.63*HO2+...
     &                  +          RXRAT( 149 )      ! MGLY=CO+HO2+CCO_O2
     &                  +          RXRAT( 150 )      ! MGLY+HO=CO+CCO_O2
     &                  +          RXRAT( 151 )      ! MGLY+NO3=CO+HNO3+CCO_O2
     &                  +  0.416 * RXRAT( 161 )      ! METHACRO+HO=0.416*CO+...
     &                  +  0.450 * RXRAT( 162 )      ! METHACRO+O3=0.45*CO+...
     &                  +  0.500 * RXRAT( 163 )      ! METHACRO+NO3=0.5*CO+...
     &                  +  0.670 * RXRAT( 165 )      ! METHACRO=0.67*CO+...
     &                  +  0.475 * RXRAT( 167 )      ! MVK+O3=0.475*CO+0.05*RO2_R+...
     &                  +  0.700 * RXRAT( 169 )      ! MVK=0.7*CO+0.3*C_O2+...
     &                  +  0.336 * RXRAT( 170 )      ! ISOPROD+HO=0.336*CO+...
     &                  +  0.498 * RXRAT( 171 )      ! ISOPROD+O3=0.498*CO+...
     &                  +  0.572 * RXRAT( 172 )      ! ISOPROD+NO3=0.572*CO+...
     &                  +  1.233 * RXRAT( 173 )      ! ISOPROD=1.233*CO+...
     &                  +          RXRAT( 178 )      ! DCB1+HO=CO+RO2_R+RCHO
     &                  +  1.500 * RXRAT( 179 )      ! DCB1+O3=1.5*CO+0.5*HO+...
     &                  +          RXRAT( 181 )      ! DCB2=CO+0.5*CCO_O2+0.5*HO2+...
     &                  +          RXRAT( 183 )      ! DCB3=CO+0.5*CCO_O2+0.5*HO2+...
     &                  +  0.500 * RXRAT( 186 )      ! ETHENE+O3=0.5*CO+0.12*HO2+...
     &                  +  0.491 * RXRAT( 188 )      ! ETHENE+O3P=0.491*CO+...
     &                  +  0.275 * RXRAT( 190 )      ! ISOPRENE+O3=0.275*CO+...
     &                  +  0.157 * RXRAT( 194 )      ! TRP1+O3=0.157*CO+0.033*HO2+...
     &                  +  0.160 * RXRAT( 198 )      ! ALK2+HO=0.16*CO+0.121*HO2+...
     &                  +  0.002 * RXRAT( 200 )      ! ALK4+HO=0.002*CO+...
     &                  +  0.345 * RXRAT( 212 )      ! OLE1+O3=0.345*CO+0.056*HO2+...
     &                  +  0.265 * RXRAT( 216 )      ! OLE2+O3=0.265*CO+0.003*HO2+...
     &                  +  0.012 * RXRAT( 218 )      ! OLE2+O3P=0.012*CO+...
      LOSS( CO       ) =           RXRAT(  29 )      ! CO+HO=HO2

      PROD( HO2H     ) =           RXRAT(  37 )      ! HO2+HO2=HO2H
     &                  +          RXRAT(  38 )      ! HO2+HO2=HO2H
      LOSS( HO2H     ) =           RXRAT(  41 )      ! HO2H=2*HO
     &                  +          RXRAT(  42 )      ! HO2H+HO=HO2

      PROD( SO2      ) =   0.0
      LOSS( SO2      ) =           RXRAT(  44 )      ! SO2+HO=HO2+SULF+SULRXN

      PROD( SULF     ) =           RXRAT(  44 )      ! HO+SO2=SULF+HO2+SULRXN
      LOSS( SULF     ) =   0.0

      PROD( SULRXN   ) =           RXRAT(  44 )      ! HO+SO2=SULRXN+SULF+HO2
      LOSS( SULRXN   ) =   0.0

      PROD( C_O2     ) =           RXRAT(  71 )      ! CCO_O2+NO=C_O2+NO2
     &                  +          RXRAT(  73 )      ! CCO_O2+NO3=C_O2+NO2
     &                  +  2.000 * RXRAT(  78 )      ! CCO_O2+CCO_O2=2*C_O2
     &                  +          RXRAT(  88 )      ! RCO_O2+CCO_O2=C_O2+CCHO+RO2_R
     &                  +          RXRAT(  99 )      ! BZCO_O2+CCO_O2=C_O2+BZ_O+R2O2
     &                  +          RXRAT( 111 )      ! MA_RCO3+CCO_O2=C_O2+HCHO+...
     &                  +          RXRAT( 116 )      ! TBU_O=C_O2+ACET
     &                  +          RXRAT( 131 )      ! CCHO=C_O2+HO2+CO
     &                  +          RXRAT( 137 )      ! ACET=C_O2+CCO_O2
     &                  +  0.650 * RXRAT( 141 )      ! COOH+HO=0.65*C_O2+0.35*HO+...
     &                  +  0.300 * RXRAT( 169 )      ! MVK=0.3*C_O2+0.7*CO+...
     &                  +          RXRAT( 184 )      ! HO=C_O2
     &                  +  0.300 * RXRAT( 188 )      ! ETHENE+O3P=0.3*C_O2+...
     &                  +  0.250 * RXRAT( 192 )      ! ISOPRENE+O3P=0.25*C_O2+...
     &                  +  0.011 * RXRAT( 200 )      ! ALK4+HO=0.011*C_O2+...
     &                  +  0.076 * RXRAT( 212 )      ! OLE1+O3=0.076*C_O2+...
     &                  +  0.197 * RXRAT( 216 )      ! OLE2+O3=0.197*C_O2+...
     &                  +  0.030 * RXRAT( 217 )      ! OLE2+NO3=0.03*C_O2+...
     &                  +  0.870 * RXRAT( 220 )      ! CCO_OH+HO=0.87*C_O2+...
      LOSS( C_O2     ) =           RXRAT(  46 )      ! C_O2+NO=NO2+HCHO+HO2
     &                  +          RXRAT(  47 )      ! C_O2+HO2=COOH
     &                  +          RXRAT(  48 )      ! C_O2+NO3=HCHO+HO2+NO2
     &                  +  2.000 * RXRAT(  49 )      ! C_O2+C_O2=MEOH+HCHO
     &                  +  2.000 * RXRAT(  50 )      ! C_O2+C_O2=2*HCHO+2*HO2
     &                  +          RXRAT(  54 )      ! C_O2+RO2_R=HO2+0.75*HCHO+...
     &                  +          RXRAT(  64 )      ! C_O2+RO2_N=HO2+0.25*MEOH+...
     &                  +          RXRAT(  74 )      ! C_O2+CCO_O2=CCO_OH+HCHO
     &                  +          RXRAT(  84 )      ! C_O2+RCO_O2=RCO_OH+HCHO
     &                  +          RXRAT(  95 )      ! C_O2+BZCO_O2=RCO_OH+HCHO
     &                  +          RXRAT( 107 )      ! C_O2+MA_RCO3=RCO_OH+HCHO

      PROD( HCHO     ) =           RXRAT(  46 )      ! C_O2+NO=HCHO+NO2+HO2
     &                  +          RXRAT(  48 )      ! C_O2+NO3=HCHO+HO2+NO2
     &                  +          RXRAT(  49 )      ! C_O2+C_O2=HCHO+MEOH
     &                  +  2.000 * RXRAT(  50 )      ! C_O2+C_O2=2*HCHO+2*HO2
     &                  +  0.750 * RXRAT(  54 )      ! RO2_R+C_O2=0.75*HCHO+HO2+...
     &                  +  0.750 * RXRAT(  64 )      ! RO2_N+C_O2=0.75*HCHO+...
     &                  +          RXRAT(  74 )      ! CCO_O2+C_O2=HCHO+CCO_OH
     &                  +          RXRAT(  84 )      ! RCO_O2+C_O2=HCHO+RCO_OH
     &                  +          RXRAT(  95 )      ! BZCO_O2+C_O2=HCHO+RCO_OH
     &                  +          RXRAT( 104 )      ! MA_RCO3+NO=HCHO+NO2+CCO_O2
     &                  +          RXRAT( 106 )      ! MA_RCO3+NO3=HCHO+NO2+CCO_O2
     &                  +          RXRAT( 107 )      ! MA_RCO3+C_O2=HCHO+RCO_OH
     &                  +          RXRAT( 111 )      ! MA_RCO3+CCO_O2=HCHO+C_O2+...
     &                  +          RXRAT( 112 )      ! MA_RCO3+RCO_O2=HCHO+CCO_O2+...
     &                  +          RXRAT( 113 )      ! MA_RCO3+BZCO_O2=HCHO+...
     &                  +  2.000 * RXRAT( 114 )      ! MA_RCO3+MA_RCO3=2*HCHO+...
     &                  +          RXRAT( 127 )      ! HOCOO=HCHO+HO2
     &                  +          RXRAT( 136 )      ! ACET+HO=HCHO+CCO_O2+R2O2
     &                  +  0.115 * RXRAT( 138 )      ! MEK+HO=0.115*HCHO+...
     &                  +          RXRAT( 140 )      ! MEOH+HO=HCHO+HO2
     &                  +  0.350 * RXRAT( 141 )      ! COOH+HO=0.35*HCHO+0.35*HO+...
     &                  +          RXRAT( 142 )      ! COOH=HCHO+HO2+HO
     &                  +          RXRAT( 146 )      ! GLY=HCHO+CO
     &                  +  0.084 * RXRAT( 161 )      ! METHACRO+HO=0.084*HCHO+...
     &                  +  0.200 * RXRAT( 162 )      ! METHACRO+O3=0.2*HCHO+...
     &                  +  0.670 * RXRAT( 165 )      ! METHACRO=0.67*HCHO+...
     &                  +  0.300 * RXRAT( 166 )      ! MVK+HO=0.3*HCHO+...
     &                  +  0.100 * RXRAT( 167 )      ! MVK+O3=0.1*HCHO+0.05*RO2_R+...
     &                  +  0.055 * RXRAT( 170 )      ! ISOPROD+HO=0.055*HCHO+...
     &                  +  0.125 * RXRAT( 171 )      ! ISOPROD+O3=0.125*HCHO+...
     &                  +  0.227 * RXRAT( 172 )      ! ISOPROD+NO3=0.227*HCHO+...
     &                  +  0.300 * RXRAT( 173 )      ! ISOPROD=0.3*HCHO+...
     &                  +  0.213 * RXRAT( 174 )      ! PROD2+HO=0.213*HCHO+...
     &                  +  0.506 * RXRAT( 175 )      ! PROD2=0.506*HCHO+...
     &                  +  0.010 * RXRAT( 176 )      ! RNO3+HO=0.01*HCHO+...
     &                  +  0.134 * RXRAT( 177 )      ! RNO3=0.134*HCHO+0.341*HO2+...
     &                  +  1.610 * RXRAT( 185 )      ! ETHENE+HO=1.61*HCHO+RO2_R+...
     &                  +          RXRAT( 186 )      ! ETHENE+O3=HCHO+0.12*HO2+...
     &                  +  0.191 * RXRAT( 188 )      ! ETHENE+O3P=0.191*HCHO+...
     &                  +  0.624 * RXRAT( 189 )      ! ISOPRENE+HO=0.624*HCHO+...
     &                  +  0.592 * RXRAT( 190 )      ! ISOPRENE+O3=0.592*HCHO+...
     &                  +  0.240 * RXRAT( 192 )      ! ISOPRENE+O3P=0.24*HCHO+...
     &                  +  0.276 * RXRAT( 193 )      ! TRP1+HO=0.276*HCHO+...
     &                  +  0.235 * RXRAT( 194 )      ! TRP1+O3=0.235*HCHO+...
     &                  +  0.039 * RXRAT( 198 )      ! ALK2+HO=0.039*HCHO+...
     &                  +  0.026 * RXRAT( 199 )      ! ALK3+HO=0.026*HCHO+...
     &                  +  0.024 * RXRAT( 200 )      ! ALK4+HO=0.024*HCHO+...
     &                  +  0.026 * RXRAT( 201 )      ! ALK5+HO=0.026*HCHO+...
     &                  +  0.732 * RXRAT( 211 )      ! OLE1+HO=0.732*HCHO+...
     &                  +  0.500 * RXRAT( 212 )      ! OLE1+O3=0.5*HCHO+0.056*HO2+...
     &                  +  0.244 * RXRAT( 215 )      ! OLE2+HO=0.244*HCHO+...
     &                  +  0.269 * RXRAT( 216 )      ! OLE2+O3=0.269*HCHO+...
     &                  +  0.079 * RXRAT( 217 )      ! OLE2+NO3=0.079*HCHO+...
      LOSS( HCHO     ) =           RXRAT( 123 )      ! HCHO=2*HO2+CO
     &                  +          RXRAT( 124 )      ! HCHO=CO
     &                  +          RXRAT( 125 )      ! HCHO+HO=HO2+CO
     &                  +          RXRAT( 126 )      ! HCHO+HO2=HOCOO
     &                  +          RXRAT( 129 )      ! HCHO+NO3=HNO3+HO2+CO

      PROD( COOH     ) =           RXRAT(  47 )      ! C_O2+HO2=COOH
      LOSS( COOH     ) =           RXRAT( 141 )      ! COOH+HO=0.35*HCHO+0.35*HO+...
     &                  +          RXRAT( 142 )      ! COOH=HCHO+HO2+HO

      PROD( MEOH     ) =           RXRAT(  49 )      ! C_O2+C_O2=MEOH+HCHO
     &                  +  0.250 * RXRAT(  54 )      ! RO2_R+C_O2=0.25*MEOH+...
     &                  +  0.250 * RXRAT(  64 )      ! RO2_N+C_O2=0.25*MEOH+HO2+...
      LOSS( MEOH     ) =           RXRAT( 140 )      ! MEOH+HO=HCHO+HO2

      PROD( RO2_R    ) =           RXRAT(  81 )      ! RCO_O2+NO=RO2_R+CCHO+NO2
     &                  +          RXRAT(  83 )      ! RCO_O2+NO3=RO2_R+CCHO+NO2
     &                  +          RXRAT(  88 )      ! RCO_O2+CCO_O2=RO2_R+CCHO+C_O2
     &                  +  2.000 * RXRAT(  89 )      ! RCO_O2+RCO_O2=2*RO2_R+2*CCHO
     &                  +          RXRAT( 100 )      ! BZCO_O2+RCO_O2=RO2_R+CCHO+...
     &                  +          RXRAT( 112 )      ! MA_RCO3+RCO_O2=RO2_R+...
     &                  +  0.034 * RXRAT( 133 )      ! RCHO+HO=0.034*RO2_R+...
     &                  +          RXRAT( 134 )      ! RCHO=RO2_R+CCHO+CO+HO2
     &                  +  0.370 * RXRAT( 138 )      ! MEK+HO=0.37*RO2_R+...
     &                  +          RXRAT( 139 )      ! MEK=RO2_R+CCHO+CCO_O2
     &                  +  0.340 * RXRAT( 143 )      ! ROOH+HO=0.34*RO2_R+RCHO+...
     &                  +  0.760 * RXRAT( 153 )      ! PHEN+HO=0.76*RO2_R+...
     &                  +  0.760 * RXRAT( 155 )      ! CRES+HO=0.76*RO2_R+...
     &                  +  0.500 * RXRAT( 161 )      ! METHACRO+HO=0.5*RO2_R+...
     &                  +  0.100 * RXRAT( 162 )      ! METHACRO+O3=0.1*RO2_R+...
     &                  +  0.500 * RXRAT( 163 )      ! METHACRO+NO3=0.5*RO2_R+...
     &                  +  0.330 * RXRAT( 165 )      ! METHACRO=0.33*RO2_R+...
     &                  +  0.300 * RXRAT( 166 )      ! MVK+HO=0.3*RO2_R+...
     &                  +  0.050 * RXRAT( 167 )      ! MVK+O3=0.05*RO2_R+...
     &                  +  0.670 * RXRAT( 170 )      ! ISOPROD+HO=0.67*RO2_R+...
     &                  +  0.048 * RXRAT( 171 )      ! ISOPROD+O3=0.048*RO2_R+...
     &                  +  0.799 * RXRAT( 172 )      ! ISOPROD+NO3=0.799*RO2_R+...
     &                  +  0.473 * RXRAT( 174 )      ! PROD2+HO=0.473*RO2_R+...
     &                  +  0.960 * RXRAT( 175 )      ! PROD2=0.96*RO2_R+...
     &                  +  0.376 * RXRAT( 176 )      ! RNO3+HO=0.376*RO2_R+...
     &                  +  0.564 * RXRAT( 177 )      ! RNO3=0.564*RO2_R+0.341*HO2+...
     &                  +          RXRAT( 178 )      ! DCB1+HO=RO2_R+RCHO+CO
     &                  +          RXRAT( 181 )      ! DCB2=RO2_R+0.5*CCO_O2+...
     &                  +          RXRAT( 183 )      ! DCB3=RO2_R+0.5*CCO_O2+...
     &                  +          RXRAT( 185 )      ! ETHENE+HO=RO2_R+1.61*HCHO+...
     &                  +          RXRAT( 187 )      ! ETHENE+NO3=RO2_R+RCHO
     &                  +  0.200 * RXRAT( 188 )      ! ETHENE+O3P=0.2*RO2_R+...
     &                  +  0.907 * RXRAT( 189 )      ! ISOPRENE+HO=0.907*RO2_R+...
     &                  +  0.066 * RXRAT( 190 )      ! ISOPRENE+O3=0.066*RO2_R+...
     &                  +  0.749 * RXRAT( 191 )      ! ISOPRENE+NO3=0.749*RO2_R+...
     &                  +  0.750 * RXRAT( 193 )      ! TRP1+HO=0.75*RO2_R+...
     &                  +  0.031 * RXRAT( 194 )      ! TRP1+O3=0.031*RO2_R+...
     &                  +  0.276 * RXRAT( 195 )      ! TRP1+NO3=0.276*RO2_R+...
     &                  +          RXRAT( 197 )      ! ALK1+HO=RO2_R+CCHO
     &                  +  0.612 * RXRAT( 198 )      ! ALK2+HO=0.612*RO2_R+...
     &                  +  0.695 * RXRAT( 199 )      ! ALK3+HO=0.695*RO2_R+...
     &                  +  0.835 * RXRAT( 200 )      ! ALK4+HO=0.835*RO2_R+...
     &                  +  0.653 * RXRAT( 201 )      ! ALK5+HO=0.653*RO2_R+...
     &                  +  0.765 * RXRAT( 202 )      ! ARO1+HO=0.765*RO2_R+...
     &                  +  0.804 * RXRAT( 205 )      ! ARO2+HO=0.804*RO2_R+...
     &                  +  0.764 * RXRAT( 208 )      ! BENZENE+HO=0.764*RO2_R+...
     &                  +  0.910 * RXRAT( 211 )      ! OLE1+HO=0.91*RO2_R+...
     &                  +  0.022 * RXRAT( 212 )      ! OLE1+O3=0.022*RO2_R+...
     &                  +  0.824 * RXRAT( 213 )      ! OLE1+NO3=0.824*RO2_R+...
     &                  +  0.918 * RXRAT( 215 )      ! OLE2+HO=0.918*RO2_R+...
     &                  +  0.033 * RXRAT( 216 )      ! OLE2+O3=0.033*RO2_R+...
     &                  +  0.442 * RXRAT( 217 )      ! OLE2+NO3=0.442*RO2_R+...
     &                  +  0.012 * RXRAT( 218 )      ! OLE2+O3P=0.012*RO2_R+...
     &                  +  0.130 * RXRAT( 220 )      ! CCO_OH+HO=0.13*RO2_R+...
     &                  +          RXRAT( 221 )      ! RCO_OH+HO=RO2_R+0.605*CCHO+...
      LOSS( RO2_R    ) =           RXRAT(  51 )      ! RO2_R+NO=NO2+HO2
     &                  +          RXRAT(  52 )      ! RO2_R+HO2=ROOH
     &                  +          RXRAT(  53 )      ! RO2_R+NO3=NO2+HO2
     &                  +          RXRAT(  54 )      ! RO2_R+C_O2=HO2+0.75*HCHO+...
     &                  +  2.000 * RXRAT(  55 )      ! RO2_R+RO2_R=HO2
     &                  +          RXRAT(  66 )      ! RO2_R+RO2_N=HO2+0.5*MEK+...
     &                  +          RXRAT(  75 )      ! RO2_R+CCO_O2=CCO_OH
     &                  +          RXRAT(  85 )      ! RO2_R+RCO_O2=RCO_OH
     &                  +          RXRAT(  96 )      ! RO2_R+BZCO_O2=RCO_OH
     &                  +          RXRAT( 108 )      ! RO2_R+MA_RCO3=RCO_OH

      PROD( ROOH     ) =           RXRAT(  52 )      ! RO2_R+HO2=ROOH
     &                  +          RXRAT(  63 )      ! RO2_N+HO2=ROOH
      LOSS( ROOH     ) =           RXRAT( 143 )      ! ROOH+HO=RCHO+0.34*RO2_R+...
     &                  +          RXRAT( 144 )      ! ROOH=RCHO+HO2+HO

      PROD( R2O2     ) =           RXRAT(  92 )      ! BZCO_O2+NO=R2O2+BZ_O+NO2
     &                  +          RXRAT(  94 )      ! BZCO_O2+NO3=R2O2+BZ_O+NO2
     &                  +          RXRAT(  99 )      ! BZCO_O2+CCO_O2=R2O2+BZ_O+C_O2
     &                  +          RXRAT( 100 )      ! BZCO_O2+RCO_O2=R2O2+RO2_R+...
     &                  +  2.000 * RXRAT( 101 )      ! BZCO_O2+BZCO_O2=2*R2O2+2*BZ_O
     &                  +          RXRAT( 113 )      ! MA_RCO3+BZCO_O2=R2O2+...
     &                  +          RXRAT( 136 )      ! ACET+HO=R2O2+CCO_O2+HCHO
     &                  +  0.616 * RXRAT( 138 )      ! MEK+HO=0.616*R2O2+...
     &                  +  0.675 * RXRAT( 166 )      ! MVK+HO=0.675*R2O2+...
     &                  +  0.515 * RXRAT( 175 )      ! PROD2=0.515*R2O2+...
     &                  +  0.596 * RXRAT( 176 )      ! RNO3+HO=0.596*R2O2+...
     &                  +  0.152 * RXRAT( 177 )      ! RNO3=0.152*R2O2+0.341*HO2+...
     &                  +          RXRAT( 180 )      ! DCB2+HO=R2O2+RCHO+CCO_O2
     &                  +          RXRAT( 181 )      ! DCB2=R2O2+0.5*CCO_O2+...
     &                  +          RXRAT( 182 )      ! DCB3+HO=R2O2+RCHO+CCO_O2
     &                  +          RXRAT( 183 )      ! DCB3=R2O2+0.5*CCO_O2+...
     &                  +  0.079 * RXRAT( 189 )      ! ISOPRENE+HO=0.079*R2O2+...
     &                  +  0.126 * RXRAT( 190 )      ! ISOPRENE+O3=0.126*R2O2+...
     &                  +  0.187 * RXRAT( 191 )      ! ISOPRENE+NO3=0.187*R2O2+...
     &                  +  0.240 * RXRAT( 192 )      ! ISOPRENE+O3P=0.24*R2O2+...
     &                  +  0.500 * RXRAT( 193 )      ! TRP1+HO=0.5*R2O2+...
     &                  +  0.729 * RXRAT( 194 )      ! TRP1+O3=0.729*R2O2+...
     &                  +  0.750 * RXRAT( 195 )      ! TRP1+NO3=0.75*R2O2+...
     &                  +  0.559 * RXRAT( 199 )      ! ALK3+HO=0.559*R2O2+...
     &                  +  0.936 * RXRAT( 200 )      ! ALK4+HO=0.936*R2O2+...
     &                  +  0.948 * RXRAT( 201 )      ! ALK5+HO=0.948*R2O2+...
     &                  +  0.205 * RXRAT( 211 )      ! OLE1+HO=0.205*R2O2+...
     &                  +  0.488 * RXRAT( 213 )      ! OLE1+NO3=0.488*R2O2+...
     &                  +  0.001 * RXRAT( 215 )      ! OLE2+HO=0.001*R2O2+...
     &                  +  0.137 * RXRAT( 216 )      ! OLE2+O3=0.137*R2O2+...
     &                  +  0.711 * RXRAT( 217 )      ! OLE2+NO3=0.711*R2O2+...
      LOSS( R2O2     ) =           RXRAT(  56 )      ! R2O2+NO=NO2
     &                  +          RXRAT(  57 )      ! R2O2+HO2=HO2
     &                  +          RXRAT(  58 )      ! R2O2+NO3=NO2
     &                  +          RXRAT(  59 )      ! R2O2+C_O2=C_O2
     &                  +          RXRAT(  60 )      ! R2O2+RO2_R=RO2_R
     &                  +  2.000 * RXRAT(  61 )      ! R2O2+R2O2=
     &                  +          RXRAT(  67 )      ! R2O2+RO2_N=RO2_N
     &                  +          RXRAT(  76 )      ! R2O2+CCO_O2=CCO_O2
     &                  +          RXRAT(  86 )      ! R2O2+RCO_O2=RCO_O2
     &                  +          RXRAT(  97 )      ! R2O2+BZCO_O2=BZCO_O2
     &                  +          RXRAT( 109 )      ! R2O2+MA_RCO3=MA_RCO3

      PROD( RO2_N    ) =   0.001 * RXRAT( 133 )      ! RCHO+HO=0.001*RO2_N+...
     &                  +  0.042 * RXRAT( 138 )      ! MEK+HO=0.042*RO2_N+...
     &                  +  0.025 * RXRAT( 166 )      ! MVK+HO=0.025*RO2_N+...
     &                  +  0.041 * RXRAT( 170 )      ! ISOPROD+HO=0.041*RO2_N+...
     &                  +  0.051 * RXRAT( 172 )      ! ISOPROD+NO3=0.051*RO2_N+...
     &                  +  0.070 * RXRAT( 174 )      ! PROD2+HO=0.07*RO2_N+...
     &                  +  0.040 * RXRAT( 175 )      ! PROD2=0.04*RO2_N+...
     &                  +  0.173 * RXRAT( 176 )      ! RNO3+HO=0.173*RO2_N+...
     &                  +  0.095 * RXRAT( 177 )      ! RNO3=0.095*RO2_N+0.341*HO2+...
     &                  +  0.093 * RXRAT( 189 )      ! ISOPRENE+HO=0.093*RO2_N+...
     &                  +  0.008 * RXRAT( 190 )      ! ISOPRENE+O3=0.008*RO2_N+...
     &                  +  0.064 * RXRAT( 191 )      ! ISOPRENE+NO3=0.064*RO2_N+...
     &                  +  0.010 * RXRAT( 192 )      ! ISOPRENE+O3P=0.01*RO2_N+...
     &                  +  0.250 * RXRAT( 193 )      ! TRP1+HO=0.25*RO2_N+...
     &                  +  0.180 * RXRAT( 194 )      ! TRP1+O3=0.18*RO2_N+...
     &                  +  0.250 * RXRAT( 195 )      ! TRP1+NO3=0.25*RO2_N+...
     &                  +  0.021 * RXRAT( 198 )      ! ALK2+HO=0.021*RO2_N+...
     &                  +  0.070 * RXRAT( 199 )      ! ALK3+HO=0.07*RO2_N+...
     &                  +  0.143 * RXRAT( 200 )      ! ALK4+HO=0.143*RO2_N+...
     &                  +  0.347 * RXRAT( 201 )      ! ALK5+HO=0.347*RO2_N+...
     &                  +  0.011 * RXRAT( 202 )      ! ARO1+HO=0.011*RO2_N+...
     &                  +  0.009 * RXRAT( 205 )      ! ARO2+HO=0.009*RO2_N+...
     &                  +  0.090 * RXRAT( 211 )      ! OLE1+HO=0.09*RO2_N+...
     &                  +  0.001 * RXRAT( 212 )      ! OLE1+O3=0.001*RO2_N+...
     &                  +  0.176 * RXRAT( 213 )      ! OLE1+NO3=0.176*RO2_N+...
     &                  +  0.082 * RXRAT( 215 )      ! OLE2+HO=0.082*RO2_N+...
     &                  +  0.002 * RXRAT( 216 )      ! OLE2+O3=0.002*RO2_N+...
     &                  +  0.136 * RXRAT( 217 )      ! OLE2+NO3=0.136*RO2_N+...
     &                  +  0.001 * RXRAT( 218 )      ! OLE2+O3P=0.001*RO2_N+...
      LOSS( RO2_N    ) =           RXRAT(  62 )      ! RO2_N+NO=RNO3
     &                  +          RXRAT(  63 )      ! RO2_N+HO2=ROOH
     &                  +          RXRAT(  64 )      ! RO2_N+C_O2=HO2+0.25*MEOH+...
     &                  +          RXRAT(  65 )      ! RO2_N+NO3=NO2+HO2+MEK
     &                  +          RXRAT(  66 )      ! RO2_N+RO2_R=HO2+0.5*MEK+...
     &                  +  2.000 * RXRAT(  68 )      ! RO2_N+RO2_N=MEK+HO2+PROD2
     &                  +          RXRAT(  77 )      ! RO2_N+CCO_O2=CCO_OH+PROD2
     &                  +          RXRAT(  87 )      ! RO2_N+RCO_O2=RCO_OH+PROD2
     &                  +          RXRAT(  98 )      ! RO2_N+BZCO_O2=RCO_OH+PROD2
     &                  +          RXRAT( 110 )      ! RO2_N+MA_RCO3=2*RCO_OH

      PROD( RNO3     ) =           RXRAT(  62 )      ! RO2_N+NO=RNO3
     &                  +          RXRAT( 115 )      ! TBU_O+NO2=RNO3
     &                  +  0.572 * RXRAT( 172 )      ! ISOPROD+NO3=0.572*RNO3+...
     &                  +  0.276 * RXRAT( 195 )      ! TRP1+NO3=0.276*RNO3+...
     &                  +  0.511 * RXRAT( 213 )      ! OLE1+NO3=0.511*RNO3+...
     &                  +  0.321 * RXRAT( 217 )      ! OLE2+NO3=0.321*RNO3+...
      LOSS( RNO3     ) =   0.690 * RXRAT( 176 )      ! RNO3+HO=0.31*RNO3+...
     &                  +          RXRAT( 177 )      ! RNO3=NO2+0.341*HO2+...

      PROD( MEK      ) =   0.500 * RXRAT(  64 )      ! RO2_N+C_O2=0.5*MEK+...
     &                  +          RXRAT(  65 )      ! RO2_N+NO3=MEK+HO2+NO2
     &                  +  0.500 * RXRAT(  66 )      ! RO2_N+RO2_R=0.5*MEK+HO2+...
     &                  +          RXRAT(  68 )      ! RO2_N+RO2_N=MEK+HO2+PROD2
     &                  +  0.416 * RXRAT( 161 )      ! METHACRO+HO=0.416*MEK+...
     &                  +  0.550 * RXRAT( 168 )      ! MVK+O3P=0.55*MEK+0.45*RCHO
     &                  +  0.150 * RXRAT( 170 )      ! ISOPROD+HO=0.15*MEK+...
     &                  +  0.210 * RXRAT( 171 )      ! ISOPROD+O3=0.21*MEK+...
     &                  +  0.233 * RXRAT( 173 )      ! ISOPROD=0.233*MEK+...
     &                  +  0.115 * RXRAT( 174 )      ! PROD2+HO=0.115*MEK+...
     &                  +  0.177 * RXRAT( 176 )      ! RNO3+HO=0.177*MEK+...
     &                  +  0.243 * RXRAT( 177 )      ! RNO3=0.243*MEK+0.341*HO2+...
     &                  +  0.332 * RXRAT( 199 )      ! ALK3+HO=0.332*MEK+...
     &                  +  0.110 * RXRAT( 200 )      ! ALK4+HO=0.11*MEK+...
     &                  +  0.089 * RXRAT( 201 )      ! ALK5+HO=0.089*MEK+...
     &                  +  0.437 * RXRAT( 214 )      ! OLE1+O3P=0.437*MEK+...
     &                  +  0.072 * RXRAT( 215 )      ! OLE2+HO=0.072*MEK+...
     &                  +  0.026 * RXRAT( 216 )      ! OLE2+O3=0.026*MEK+...
     &                  +  0.001 * RXRAT( 217 )      ! OLE2+NO3=0.001*MEK+...
     &                  +  0.659 * RXRAT( 218 )      ! OLE2+O3P=0.659*MEK+...
      LOSS( MEK      ) =           RXRAT( 138 )      ! MEK+HO=0.37*RO2_R+...
     &                  +          RXRAT( 139 )      ! MEK=CCO_O2+CCHO+RO2_R

      PROD( PROD2    ) =   0.500 * RXRAT(  64 )      ! RO2_N+C_O2=0.5*PROD2+...
     &                  +  0.500 * RXRAT(  66 )      ! RO2_N+RO2_R=0.5*PROD2+...
     &                  +          RXRAT(  68 )      ! RO2_N+RO2_N=PROD2+HO2+MEK
     &                  +          RXRAT(  77 )      ! CCO_O2+RO2_N=PROD2+CCO_OH
     &                  +          RXRAT(  87 )      ! RCO_O2+RO2_N=PROD2+RCO_OH
     &                  +          RXRAT(  98 )      ! BZCO_O2+RO2_N=PROD2+RCO_OH
     &                  +  0.700 * RXRAT( 169 )      ! MVK=0.7*PROD2+0.7*CO+...
     &                  +  0.332 * RXRAT( 170 )      ! ISOPROD+HO=0.332*PROD2+...
     &                  +  0.048 * RXRAT( 176 )      ! RNO3+HO=0.048*PROD2+...
     &                  +  0.435 * RXRAT( 177 )      ! RNO3=0.435*PROD2+0.341*HO2+...
     &                  +  0.100 * RXRAT( 190 )      ! ISOPRENE+O3=0.1*PROD2+...
     &                  +  0.750 * RXRAT( 192 )      ! ISOPRENE+O3P=0.75*PROD2+...
     &                  +  0.276 * RXRAT( 193 )      ! TRP1+HO=0.276*PROD2+...
     &                  +  0.276 * RXRAT( 194 )      ! TRP1+O3=0.276*PROD2+...
     &                  +  0.853 * RXRAT( 196 )      ! TRP1+O3P=0.853*PROD2+...
     &                  +  0.125 * RXRAT( 200 )      ! ALK4+HO=0.125*PROD2+...
     &                  +  0.417 * RXRAT( 201 )      ! ALK5+HO=0.417*PROD2+...
     &                  +  0.055 * RXRAT( 202 )      ! ARO1+HO=0.055*PROD2+...
     &                  +  0.119 * RXRAT( 211 )      ! OLE1+HO=0.119*PROD2+...
     &                  +  0.215 * RXRAT( 212 )      ! OLE1+O3=0.215*PROD2+...
     &                  +  0.113 * RXRAT( 214 )      ! OLE1+O3P=0.113*PROD2+...
     &                  +  0.006 * RXRAT( 216 )      ! OLE2+O3=0.006*PROD2+...
     &                  +  0.259 * RXRAT( 218 )      ! OLE2+O3P=0.259*PROD2+...
      LOSS( PROD2    ) =   0.671 * RXRAT( 174 )      ! PROD2+HO=0.329*PROD2+...
     &                  +          RXRAT( 175 )      ! PROD2=0.96*RO2_R+...

      PROD( CCO_OOH  ) =   0.750 * RXRAT(  72 )      ! CCO_O2+HO2=0.75*CCO_OOH+...
      LOSS( CCO_OOH  ) =   0.0

      PROD( CCO_OH   ) =   0.250 * RXRAT(  72 )      ! CCO_O2+HO2=0.25*CCO_OH+...
     &                  +          RXRAT(  74 )      ! CCO_O2+C_O2=CCO_OH+HCHO
     &                  +          RXRAT(  75 )      ! CCO_O2+RO2_R=CCO_OH
     &                  +          RXRAT(  77 )      ! CCO_O2+RO2_N=CCO_OH+PROD2
     &                  +  0.050 * RXRAT( 212 )      ! OLE1+O3=0.05*CCO_OH+...
     &                  +  0.129 * RXRAT( 216 )      ! OLE2+O3=0.129*CCO_OH+...
      LOSS( CCO_OH   ) =           RXRAT( 220 )      ! CCO_OH+HO=0.13*RO2_R+...

      PROD( RCO_O2   ) =           RXRAT(  80 )      ! PAN2=RCO_O2+NO2
     &                  +  0.965 * RXRAT( 133 )      ! RCHO+HO=0.965*RCO_O2+...
     &                  +          RXRAT( 135 )      ! RCHO+NO3=RCO_O2+HNO3
     &                  +  0.096 * RXRAT( 138 )      ! MEK+HO=0.096*RCO_O2+...
     &                  +  0.370 * RXRAT( 147 )      ! GLY+HO=0.37*RCO_O2+1.26*CO+...
     &                  +  0.370 * RXRAT( 148 )      ! GLY+NO3=0.37*RCO_O2+...
     &                  +  0.100 * RXRAT( 162 )      ! METHACRO+O3=0.1*RCO_O2+...
     &                  +  0.050 * RXRAT( 167 )      ! MVK+O3=0.05*RCO_O2+...
     &                  +  0.048 * RXRAT( 171 )      ! ISOPROD+O3=0.048*RCO_O2+...
     &                  +  0.300 * RXRAT( 173 )      ! ISOPROD=0.3*RCO_O2+...
     &                  +  0.049 * RXRAT( 174 )      ! PROD2+HO=0.049*RCO_O2+...
     &                  +  0.333 * RXRAT( 175 )      ! PROD2=0.333*RCO_O2+...
     &                  +  0.201 * RXRAT( 194 )      ! TRP1+O3=0.201*RCO_O2+...
     &                  +  0.006 * RXRAT( 216 )      ! OLE2+O3=0.006*RCO_O2+...
      LOSS( RCO_O2   ) =           RXRAT(  79 )      ! RCO_O2+NO2=PAN2
     &                  +          RXRAT(  81 )      ! RCO_O2+NO=NO2+CCHO+RO2_R
     &                  +          RXRAT(  82 )      ! RCO_O2+HO2=0.75*RCO_OOH+...
     &                  +          RXRAT(  83 )      ! RCO_O2+NO3=NO2+CCHO+RO2_R
     &                  +          RXRAT(  84 )      ! RCO_O2+C_O2=RCO_OH+HCHO
     &                  +          RXRAT(  85 )      ! RCO_O2+RO2_R=RCO_OH
     &                  +          RXRAT(  87 )      ! RCO_O2+RO2_N=RCO_OH+PROD2
     &                  +          RXRAT(  88 )      ! RCO_O2+CCO_O2=C_O2+CCHO+RO2_R
     &                  +  2.000 * RXRAT(  89 )      ! RCO_O2+RCO_O2=2*CCHO+2*RO2_R
     &                  +          RXRAT( 100 )      ! RCO_O2+BZCO_O2=CCHO+RO2_R+...
     &                  +          RXRAT( 112 )      ! RCO_O2+MA_RCO3=HCHO+CCO_O2+...

      PROD( PAN2     ) =           RXRAT(  79 )      ! RCO_O2+NO2=PAN2
      LOSS( PAN2     ) =           RXRAT(  80 )      ! PAN2=RCO_O2+NO2

      PROD( CCHO     ) =           RXRAT(  81 )      ! RCO_O2+NO=CCHO+NO2+RO2_R
     &                  +          RXRAT(  83 )      ! RCO_O2+NO3=CCHO+NO2+RO2_R
     &                  +          RXRAT(  88 )      ! RCO_O2+CCO_O2=CCHO+C_O2+RO2_R
     &                  +  2.000 * RXRAT(  89 )      ! RCO_O2+RCO_O2=2*CCHO+2*RO2_R
     &                  +          RXRAT( 100 )      ! BZCO_O2+RCO_O2=CCHO+RO2_R+...
     &                  +          RXRAT( 112 )      ! MA_RCO3+RCO_O2=CCHO+CCO_O2+...
     &                  +  0.034 * RXRAT( 133 )      ! RCHO+HO=0.034*CCHO+...
     &                  +          RXRAT( 134 )      ! RCHO=CCHO+RO2_R+CO+HO2
     &                  +  0.482 * RXRAT( 138 )      ! MEK+HO=0.482*CCHO+...
     &                  +          RXRAT( 139 )      ! MEK=CCHO+CCO_O2+RO2_R
     &                  +  0.129 * RXRAT( 170 )      ! ISOPROD+HO=0.129*CCHO+...
     &                  +  0.047 * RXRAT( 171 )      ! ISOPROD+O3=0.047*CCHO+...
     &                  +  0.467 * RXRAT( 173 )      ! ISOPROD=0.467*CCHO+...
     &                  +  0.084 * RXRAT( 174 )      ! PROD2+HO=0.084*CCHO+...
     &                  +  0.246 * RXRAT( 175 )      ! PROD2=0.246*CCHO+...
     &                  +  0.439 * RXRAT( 176 )      ! RNO3+HO=0.439*CCHO+...
     &                  +  0.431 * RXRAT( 177 )      ! RNO3=0.431*CCHO+0.341*HO2+...
     &                  +  0.195 * RXRAT( 185 )      ! ETHENE+HO=0.195*CCHO+...
     &                  +  0.250 * RXRAT( 188 )      ! ETHENE+O3P=0.25*CCHO+...
     &                  +          RXRAT( 197 )      ! ALK1+HO=CCHO+RO2_R
     &                  +  0.445 * RXRAT( 199 )      ! ALK3+HO=0.445*CCHO+...
     &                  +  0.455 * RXRAT( 200 )      ! ALK4+HO=0.455*CCHO+...
     &                  +  0.099 * RXRAT( 201 )      ! ALK5+HO=0.099*CCHO+...
     &                  +  0.294 * RXRAT( 211 )      ! OLE1+HO=0.294*CCHO+...
     &                  +  0.154 * RXRAT( 212 )      ! OLE1+O3=0.154*CCHO+...
     &                  +  0.009 * RXRAT( 213 )      ! OLE1+NO3=0.009*CCHO+...
     &                  +  0.732 * RXRAT( 215 )      ! OLE2+HO=0.732*CCHO+...
     &                  +  0.456 * RXRAT( 216 )      ! OLE2+O3=0.456*CCHO+...
     &                  +  0.507 * RXRAT( 217 )      ! OLE2+NO3=0.507*CCHO+...
     &                  +  0.605 * RXRAT( 221 )      ! RCO_OH+HO=0.605*CCHO+RO2_R+...
      LOSS( CCHO     ) =           RXRAT( 130 )      ! CCHO+HO=CCO_O2
     &                  +          RXRAT( 131 )      ! CCHO=CO+HO2+C_O2
     &                  +          RXRAT( 132 )      ! CCHO+NO3=HNO3+CCO_O2

      PROD( RCO_OOH  ) =   0.750 * RXRAT(  82 )      ! RCO_O2+HO2=0.75*RCO_OOH+...
     &                  +  0.750 * RXRAT(  93 )      ! BZCO_O2+HO2=0.75*RCO_OOH+...
     &                  +  0.750 * RXRAT( 105 )      ! MA_RCO3+HO2=0.75*RCO_OOH+...
      LOSS( RCO_OOH  ) =   0.0

      PROD( RCO_OH   ) =   0.250 * RXRAT(  82 )      ! RCO_O2+HO2=0.25*RCO_OH+...
     &                  +          RXRAT(  84 )      ! RCO_O2+C_O2=RCO_OH+HCHO
     &                  +          RXRAT(  85 )      ! RCO_O2+RO2_R=RCO_OH
     &                  +          RXRAT(  87 )      ! RCO_O2+RO2_N=RCO_OH+PROD2
     &                  +  0.250 * RXRAT(  93 )      ! BZCO_O2+HO2=0.25*RCO_OH+...
     &                  +          RXRAT(  95 )      ! BZCO_O2+C_O2=RCO_OH+HCHO
     &                  +          RXRAT(  96 )      ! BZCO_O2+RO2_R=RCO_OH
     &                  +          RXRAT(  98 )      ! BZCO_O2+RO2_N=RCO_OH+PROD2
     &                  +  0.250 * RXRAT( 105 )      ! MA_RCO3+HO2=0.25*RCO_OH+...
     &                  +          RXRAT( 107 )      ! MA_RCO3+C_O2=RCO_OH+HCHO
     &                  +          RXRAT( 108 )      ! MA_RCO3+RO2_R=RCO_OH
     &                  +  2.000 * RXRAT( 110 )      ! MA_RCO3+RO2_N=2*RCO_OH
     &                  +  0.372 * RXRAT( 171 )      ! ISOPROD+O3=0.372*RCO_OH+...
     &                  +  0.150 * RXRAT( 190 )      ! ISOPRENE+O3=0.15*RCO_OH+...
     &                  +  0.189 * RXRAT( 194 )      ! TRP1+O3=0.189*RCO_OH+...
     &                  +  0.119 * RXRAT( 212 )      ! OLE1+O3=0.119*RCO_OH+...
     &                  +  0.303 * RXRAT( 216 )      ! OLE2+O3=0.303*RCO_OH+...
      LOSS( RCO_OH   ) =           RXRAT( 221 )      ! RCO_OH+HO=RO2_R+0.605*CCHO+...

      PROD( BZCO_O2  ) =           RXRAT(  91 )      ! PBZN=BZCO_O2+NO2
     &                  +          RXRAT( 158 )      ! BALD+HO=BZCO_O2
     &                  +          RXRAT( 160 )      ! BALD+NO3=BZCO_O2+HNO3
      LOSS( BZCO_O2  ) =           RXRAT(  90 )      ! BZCO_O2+NO2=PBZN
     &                  +          RXRAT(  92 )      ! BZCO_O2+NO=NO2+BZ_O+R2O2
     &                  +          RXRAT(  93 )      ! BZCO_O2+HO2=0.75*RCO_OOH+...
     &                  +          RXRAT(  94 )      ! BZCO_O2+NO3=NO2+BZ_O+R2O2
     &                  +          RXRAT(  95 )      ! BZCO_O2+C_O2=RCO_OH+HCHO
     &                  +          RXRAT(  96 )      ! BZCO_O2+RO2_R=RCO_OH
     &                  +          RXRAT(  98 )      ! BZCO_O2+RO2_N=RCO_OH+PROD2
     &                  +          RXRAT(  99 )      ! BZCO_O2+CCO_O2=C_O2+BZ_O+R2O2
     &                  +          RXRAT( 100 )      ! BZCO_O2+RCO_O2=CCHO+RO2_R+...
     &                  +  2.000 * RXRAT( 101 )      ! BZCO_O2+BZCO_O2=2*BZ_O+2*R2O2
     &                  +          RXRAT( 113 )      ! BZCO_O2+MA_RCO3=HCHO+...

      PROD( PBZN     ) =           RXRAT(  90 )      ! BZCO_O2+NO2=PBZN
      LOSS( PBZN     ) =           RXRAT(  91 )      ! PBZN=BZCO_O2+NO2

      PROD( BZ_O     ) =           RXRAT(  92 )      ! BZCO_O2+NO=BZ_O+NO2+R2O2
     &                  +          RXRAT(  94 )      ! BZCO_O2+NO3=BZ_O+NO2+R2O2
     &                  +          RXRAT(  99 )      ! BZCO_O2+CCO_O2=BZ_O+C_O2+R2O2
     &                  +          RXRAT( 100 )      ! BZCO_O2+RCO_O2=BZ_O+RO2_R+...
     &                  +  2.000 * RXRAT( 101 )      ! BZCO_O2+BZCO_O2=2*BZ_O+2*R2O2
     &                  +          RXRAT( 113 )      ! MA_RCO3+BZCO_O2=BZ_O+...
     &                  +  0.240 * RXRAT( 153 )      ! PHEN+HO=0.24*BZ_O+...
     &                  +          RXRAT( 154 )      ! PHEN+NO3=BZ_O+HNO3
     &                  +  0.240 * RXRAT( 155 )      ! CRES+HO=0.24*BZ_O+...
     &                  +          RXRAT( 156 )      ! CRES+NO3=BZ_O+HNO3
      LOSS( BZ_O     ) =           RXRAT( 117 )      ! BZ_O+NO2=NPHE
     &                  +          RXRAT( 118 )      ! BZ_O+HO2=PHEN
     &                  +          RXRAT( 119 )      ! BZ_O=PHEN

      PROD( MA_RCO3  ) =           RXRAT( 103 )      ! MA_PAN=MA_RCO3+NO2
     &                  +  0.500 * RXRAT( 161 )      ! METHACRO+HO=0.5*MA_RCO3+...
     &                  +  0.500 * RXRAT( 163 )      ! METHACRO+NO3=0.5*MA_RCO3+...
     &                  +  0.330 * RXRAT( 165 )      ! METHACRO=0.33*MA_RCO3+...
     &                  +  0.300 * RXRAT( 169 )      ! MVK=0.3*MA_RCO3+0.7*CO+...
     &                  +  0.289 * RXRAT( 170 )      ! ISOPROD+HO=0.289*MA_RCO3+...
     &                  +  0.150 * RXRAT( 172 )      ! ISOPROD+NO3=0.15*MA_RCO3+...
     &                  +  0.192 * RXRAT( 190 )      ! ISOPRENE+O3=0.192*MA_RCO3+...
     &                  +  0.240 * RXRAT( 192 )      ! ISOPRENE+O3P=0.24*MA_RCO3+...
      LOSS( MA_RCO3  ) =           RXRAT( 102 )      ! MA_RCO3+NO2=MA_PAN
     &                  +          RXRAT( 104 )      ! MA_RCO3+NO=NO2+HCHO+CCO_O2
     &                  +          RXRAT( 105 )      ! MA_RCO3+HO2=0.75*RCO_OOH+...
     &                  +          RXRAT( 106 )      ! MA_RCO3+NO3=NO2+HCHO+CCO_O2
     &                  +          RXRAT( 107 )      ! MA_RCO3+C_O2=RCO_OH+HCHO
     &                  +          RXRAT( 108 )      ! MA_RCO3+RO2_R=RCO_OH
     &                  +          RXRAT( 110 )      ! MA_RCO3+RO2_N=2*RCO_OH
     &                  +          RXRAT( 111 )      ! MA_RCO3+CCO_O2=C_O2+HCHO+...
     &                  +          RXRAT( 112 )      ! MA_RCO3+RCO_O2=HCHO+CCO_O2+...
     &                  +          RXRAT( 113 )      ! MA_RCO3+BZCO_O2=HCHO+...
     &                  +  2.000 * RXRAT( 114 )      ! MA_RCO3+MA_RCO3=2*HCHO+...

      PROD( MA_PAN   ) =           RXRAT( 102 )      ! MA_RCO3+NO2=MA_PAN
      LOSS( MA_PAN   ) =           RXRAT( 103 )      ! MA_PAN=MA_RCO3+NO2

      PROD( TBU_O    ) =   0.236 * RXRAT( 199 )      ! ALK3+HO=0.236*TBU_O+...
      LOSS( TBU_O    ) =           RXRAT( 115 )      ! TBU_O+NO2=RNO3
     &                  +          RXRAT( 116 )      ! TBU_O=ACET+C_O2

      PROD( ACET     ) =           RXRAT( 116 )      ! TBU_O=ACET+C_O2
     &                  +  0.006 * RXRAT( 176 )      ! RNO3+HO=0.006*ACET+...
     &                  +  0.020 * RXRAT( 177 )      ! RNO3=0.02*ACET+0.341*HO2+...
     &                  +  0.130 * RXRAT( 194 )      ! TRP1+O3=0.13*ACET+...
     &                  +  0.417 * RXRAT( 198 )      ! ALK2+HO=0.417*ACET+...
     &                  +  0.024 * RXRAT( 199 )      ! ALK3+HO=0.024*ACET+...
     &                  +  0.452 * RXRAT( 200 )      ! ALK4+HO=0.452*ACET+...
     &                  +  0.072 * RXRAT( 201 )      ! ALK5+HO=0.072*ACET+...
     &                  +  0.005 * RXRAT( 211 )      ! OLE1+HO=0.005*ACET+...
     &                  +  0.001 * RXRAT( 212 )      ! OLE1+O3=0.001*ACET+...
     &                  +  0.024 * RXRAT( 213 )      ! OLE1+NO3=0.024*ACET+...
     &                  +  0.127 * RXRAT( 215 )      ! OLE2+HO=0.127*ACET+...
     &                  +  0.045 * RXRAT( 216 )      ! OLE2+O3=0.045*ACET+...
     &                  +  0.102 * RXRAT( 217 )      ! OLE2+NO3=0.102*ACET+...
      LOSS( ACET     ) =           RXRAT( 136 )      ! ACET+HO=HCHO+CCO_O2+R2O2
     &                  +          RXRAT( 137 )      ! ACET=CCO_O2+C_O2

      PROD( NPHE     ) =           RXRAT( 117 )      ! BZ_O+NO2=NPHE
     &                  +          RXRAT( 121 )      ! BZNO2_O+HO2=NPHE
     &                  +          RXRAT( 122 )      ! BZNO2_O=NPHE
      LOSS( NPHE     ) =           RXRAT( 157 )      ! NPHE+NO3=HNO3+BZNO2_O

      PROD( PHEN     ) =           RXRAT( 118 )      ! BZ_O+HO2=PHEN
     &                  +          RXRAT( 119 )      ! BZ_O=PHEN
     &                  +  0.017 * RXRAT( 202 )      ! ARO1+HO=0.017*PHEN+...
     &                  +  0.236 * RXRAT( 208 )      ! BENZENE+HO=0.236*PHEN+...
      LOSS( PHEN     ) =           RXRAT( 153 )      ! PHEN+HO=0.24*BZ_O+...
     &                  +          RXRAT( 154 )      ! PHEN+NO3=HNO3+BZ_O

      PROD( BZNO2_O  ) =           RXRAT( 157 )      ! NPHE+NO3=BZNO2_O+HNO3
      LOSS( BZNO2_O  ) =           RXRAT( 120 )      ! BZNO2_O+NO2=
     &                  +          RXRAT( 121 )      ! BZNO2_O+HO2=NPHE
     &                  +          RXRAT( 122 )      ! BZNO2_O=NPHE

      PROD( HOCOO    ) =           RXRAT( 126 )      ! HCHO+HO2=HOCOO
      LOSS( HOCOO    ) =           RXRAT( 127 )      ! HOCOO=HO2+HCHO
     &                  +          RXRAT( 128 )      ! HOCOO+NO=HCOOH+NO2+HO2

      PROD( HCOOH    ) =           RXRAT( 128 )      ! HOCOO+NO=HCOOH+NO2+HO2
     &                  +  0.333 * RXRAT( 162 )      ! METHACRO+O3=0.333*HCOOH+...
     &                  +  0.351 * RXRAT( 167 )      ! MVK+O3=0.351*HCOOH+...
     &                  +  0.100 * RXRAT( 171 )      ! ISOPROD+O3=0.1*HCOOH+...
     &                  +  0.370 * RXRAT( 186 )      ! ETHENE+O3=0.37*HCOOH+...
     &                  +  0.204 * RXRAT( 190 )      ! ISOPRENE+O3=0.204*HCOOH+...
     &                  +  0.103 * RXRAT( 194 )      ! TRP1+O3=0.103*HCOOH+...
     &                  +  0.121 * RXRAT( 198 )      ! ALK2+HO=0.121*HCOOH+...
     &                  +  0.185 * RXRAT( 212 )      ! OLE1+O3=0.185*HCOOH+...
     &                  +  0.073 * RXRAT( 216 )      ! OLE2+O3=0.073*HCOOH+...
      LOSS( HCOOH    ) =           RXRAT( 219 )      ! HCOOH+HO=HO2

      PROD( RCHO     ) =   0.370 * RXRAT( 138 )      ! MEK+HO=0.37*RCHO+...
     &                  +          RXRAT( 143 )      ! ROOH+HO=RCHO+0.34*RO2_R+...
     &                  +          RXRAT( 144 )      ! ROOH=RCHO+HO2+HO
     &                  +          RXRAT( 164 )      ! METHACRO+O3P=RCHO
     &                  +  0.675 * RXRAT( 166 )      ! MVK+HO=0.675*RCHO+...
     &                  +  0.450 * RXRAT( 168 )      ! MVK+O3P=0.45*RCHO+0.55*MEK
     &                  +  0.013 * RXRAT( 170 )      ! ISOPROD+HO=0.013*RCHO+...
     &                  +  0.218 * RXRAT( 172 )      ! ISOPROD+NO3=0.218*RCHO+...
     &                  +  0.558 * RXRAT( 174 )      ! PROD2+HO=0.558*RCHO+...
     &                  +  0.710 * RXRAT( 175 )      ! PROD2=0.71*RCHO+0.04*RO2_N+...
     &                  +  0.213 * RXRAT( 176 )      ! RNO3+HO=0.213*RCHO+...
     &                  +  0.147 * RXRAT( 177 )      ! RNO3=0.147*RCHO+0.341*HO2+...
     &                  +          RXRAT( 178 )      ! DCB1+HO=RCHO+RO2_R+CO
     &                  +          RXRAT( 180 )      ! DCB2+HO=RCHO+R2O2+CCO_O2
     &                  +          RXRAT( 182 )      ! DCB3+HO=RCHO+R2O2+CCO_O2
     &                  +          RXRAT( 187 )      ! ETHENE+NO3=RCHO+RO2_R
     &                  +  0.474 * RXRAT( 193 )      ! TRP1+HO=0.474*RCHO+...
     &                  +  0.205 * RXRAT( 194 )      ! TRP1+O3=0.205*RCHO+...
     &                  +  0.474 * RXRAT( 195 )      ! TRP1+NO3=0.474*RCHO+...
     &                  +  0.147 * RXRAT( 196 )      ! TRP1+O3P=0.147*RCHO+...
     &                  +  0.155 * RXRAT( 198 )      ! ALK2+HO=0.155*RCHO+...
     &                  +  0.122 * RXRAT( 199 )      ! ALK3+HO=0.122*RCHO+...
     &                  +  0.244 * RXRAT( 200 )      ! ALK4+HO=0.244*RCHO+...
     &                  +  0.204 * RXRAT( 201 )      ! ALK5+HO=0.204*RCHO+...
     &                  +  0.497 * RXRAT( 211 )      ! OLE1+HO=0.497*RCHO+...
     &                  +  0.363 * RXRAT( 212 )      ! OLE1+O3=0.363*RCHO+...
     &                  +  0.037 * RXRAT( 213 )      ! OLE1+NO3=0.037*RCHO+...
     &                  +  0.450 * RXRAT( 214 )      ! OLE1+O3P=0.45*RCHO+...
     &                  +  0.511 * RXRAT( 215 )      ! OLE2+HO=0.511*RCHO+...
     &                  +  0.305 * RXRAT( 216 )      ! OLE2+O3=0.305*RCHO+...
     &                  +  0.151 * RXRAT( 217 )      ! OLE2+NO3=0.151*RCHO+...
     &                  +  0.069 * RXRAT( 218 )      ! OLE2+O3P=0.069*RCHO+...
     &                  +  0.210 * RXRAT( 221 )      ! RCO_OH+HO=0.21*RCHO+...
      LOSS( RCHO     ) =           RXRAT( 133 )      ! RCHO+HO=0.034*RO2_R+...
     &                  +          RXRAT( 134 )      ! RCHO=CCHO+RO2_R+CO+HO2
     &                  +          RXRAT( 135 )      ! RCHO+NO3=HNO3+RCO_O2

      PROD( GLY      ) =   0.230 * RXRAT( 153 )      ! PHEN+HO=0.23*GLY+...
     &                  +  0.150 * RXRAT( 170 )      ! ISOPROD+HO=0.15*GLY+...
     &                  +  0.023 * RXRAT( 171 )      ! ISOPROD+O3=0.023*GLY+...
     &                  +          RXRAT( 179 )      ! DCB1+O3=GLY+0.5*HO+1.5*CO+...
     &                  +  0.500 * RXRAT( 181 )      ! DCB2=0.5*GLY+0.5*CCO_O2+...
     &                  +  0.500 * RXRAT( 183 )      ! DCB3=0.5*GLY+0.5*CCO_O2+...
     &                  +  0.009 * RXRAT( 188 )      ! ETHENE+O3P=0.009*GLY+...
     &                  +  0.001 * RXRAT( 194 )      ! TRP1+O3=0.001*GLY+...
     &                  +  0.248 * RXRAT( 198 )      ! ALK2+HO=0.248*GLY+...
     &                  +  0.118 * RXRAT( 202 )      ! ARO1+HO=0.118*GLY+...
     &                  +  0.097 * RXRAT( 205 )      ! ARO2+HO=0.097*GLY+...
     &                  +  0.207 * RXRAT( 208 )      ! BENZENE+HO=0.207*GLY+...
      LOSS( GLY      ) =           RXRAT( 145 )      ! GLY=2*CO+2*HO2
     &                  +          RXRAT( 146 )      ! GLY=HCHO+CO
     &                  +          RXRAT( 147 )      ! GLY+HO=0.63*HO2+1.26*CO+...
     &                  +          RXRAT( 148 )      ! GLY+NO3=HNO3+0.63*HO2+...

      PROD( MGLY     ) =   0.230 * RXRAT( 155 )      ! CRES+HO=0.23*MGLY+...
     &                  +  0.084 * RXRAT( 161 )      ! METHACRO+HO=0.084*MGLY+...
     &                  +  0.900 * RXRAT( 162 )      ! METHACRO+O3=0.9*MGLY+...
     &                  +  0.300 * RXRAT( 166 )      ! MVK+HO=0.3*MGLY+...
     &                  +  0.950 * RXRAT( 167 )      ! MVK+O3=0.95*MGLY+...
     &                  +  0.174 * RXRAT( 170 )      ! ISOPROD+HO=0.174*MGLY+...
     &                  +  0.742 * RXRAT( 171 )      ! ISOPROD+O3=0.742*MGLY+...
     &                  +  0.008 * RXRAT( 172 )      ! ISOPROD+NO3=0.008*MGLY+...
     &                  +  0.500 * RXRAT( 181 )      ! DCB2=0.5*MGLY+0.5*CCO_O2+...
     &                  +  0.500 * RXRAT( 183 )      ! DCB3=0.5*MGLY+0.5*CCO_O2+...
     &                  +  0.119 * RXRAT( 202 )      ! ARO1+HO=0.119*MGLY+...
     &                  +  0.287 * RXRAT( 205 )      ! ARO2+HO=0.287*MGLY+...
     &                  +  0.130 * RXRAT( 220 )      ! CCO_OH+HO=0.13*MGLY+...
      LOSS( MGLY     ) =           RXRAT( 149 )      ! MGLY=HO2+CO+CCO_O2
     &                  +          RXRAT( 150 )      ! MGLY+HO=CO+CCO_O2
     &                  +          RXRAT( 151 )      ! MGLY+NO3=HNO3+CO+CCO_O2

      PROD( BACL     ) =   0.031 * RXRAT( 194 )      ! TRP1+O3=0.031*BACL+...
     &                  +  0.087 * RXRAT( 205 )      ! ARO2+HO=0.087*BACL+...
     &                  +  0.185 * RXRAT( 221 )      ! RCO_OH+HO=0.185*BACL+...
      LOSS( BACL     ) =           RXRAT( 152 )      ! BACL=2*CCO_O2

      PROD( CRES     ) =   0.207 * RXRAT( 202 )      ! ARO1+HO=0.207*CRES+...
     &                  +  0.187 * RXRAT( 205 )      ! ARO2+HO=0.187*CRES+...
      LOSS( CRES     ) =           RXRAT( 155 )      ! CRES+HO=0.24*BZ_O+...
     &                  +          RXRAT( 156 )      ! CRES+NO3=HNO3+BZ_O

      PROD( BALD     ) =   0.059 * RXRAT( 202 )      ! ARO1+HO=0.059*BALD+...
     &                  +  0.050 * RXRAT( 205 )      ! ARO2+HO=0.05*BALD+...
     &                  +  0.061 * RXRAT( 215 )      ! OLE2+HO=0.061*BALD+...
     &                  +  0.042 * RXRAT( 216 )      ! OLE2+O3=0.042*BALD+...
     &                  +  0.015 * RXRAT( 217 )      ! OLE2+NO3=0.015*BALD+...
      LOSS( BALD     ) =           RXRAT( 158 )      ! BALD+HO=BZCO_O2
     &                  +          RXRAT( 159 )      ! BALD=
     &                  +          RXRAT( 160 )      ! BALD+NO3=HNO3+BZCO_O2

      PROD( METHACRO ) =   0.230 * RXRAT( 189 )      ! ISOPRENE+HO=0.23*METHACRO+...
     &                  +  0.390 * RXRAT( 190 )      ! ISOPRENE+O3=0.39*METHACRO+...
     &                  +  0.025 * RXRAT( 215 )      ! OLE2+HO=0.025*METHACRO+...
     &                  +  0.026 * RXRAT( 216 )      ! OLE2+O3=0.026*METHACRO+...
     &                  +  0.012 * RXRAT( 218 )      ! OLE2+O3P=0.012*METHACRO+...
      LOSS( METHACRO ) =           RXRAT( 161 )      ! METHACRO+HO=0.5*RO2_R+...
     &                  +          RXRAT( 162 )      ! METHACRO+O3=0.008*HO2+...
     &                  +          RXRAT( 163 )      ! METHACRO+NO3=0.5*HNO3+...
     &                  +          RXRAT( 164 )      ! METHACRO+O3P=RCHO
     &                  +          RXRAT( 165 )      ! METHACRO=0.34*HO2+...

      PROD( MVK      ) =   0.320 * RXRAT( 189 )      ! ISOPRENE+HO=0.32*MVK+...
     &                  +  0.160 * RXRAT( 190 )      ! ISOPRENE+O3=0.16*MVK+...
     &                  +  0.048 * RXRAT( 217 )      ! OLE2+NO3=0.048*MVK+...
      LOSS( MVK      ) =           RXRAT( 166 )      ! MVK+HO=0.3*RO2_R+...
     &                  +          RXRAT( 167 )      ! MVK+O3=0.064*HO2+...
     &                  +          RXRAT( 168 )      ! MVK+O3P=0.45*RCHO+0.55*MEK
     &                  +          RXRAT( 169 )      ! MVK=0.3*C_O2+0.7*CO+...

      PROD( ISOPROD  ) =   0.357 * RXRAT( 189 )      ! ISOPRENE+HO=0.357*ISOPROD+...
     &                  +  0.936 * RXRAT( 191 )      ! ISOPRENE+NO3=0.936*ISOPROD+...
     &                  +  0.025 * RXRAT( 215 )      ! OLE2+HO=0.025*ISOPROD+...
      LOSS( ISOPROD  ) =           RXRAT( 170 )      ! ISOPROD+HO=0.67*RO2_R+...
     &                  +          RXRAT( 171 )      ! ISOPROD+O3=0.4*HO2+...
     &                  +          RXRAT( 172 )      ! ISOPROD+NO3=0.799*RO2_R+...
     &                  +          RXRAT( 173 )      ! ISOPROD=1.233*HO2+...

      PROD( DCB1     ) =   0.491 * RXRAT( 202 )      ! ARO1+HO=0.491*DCB1+...
     &                  +  0.561 * RXRAT( 205 )      ! ARO2+HO=0.561*DCB1+...
     &                  +  0.764 * RXRAT( 208 )      ! BENZENE+HO=0.764*DCB1+...
      LOSS( DCB1     ) =           RXRAT( 178 )      ! DCB1+HO=RCHO+RO2_R+CO
     &                  +          RXRAT( 179 )      ! DCB1+O3=1.5*HO2+0.5*HO+...

      PROD( DCB2     ) =   0.108 * RXRAT( 202 )      ! ARO1+HO=0.108*DCB2+...
     &                  +  0.099 * RXRAT( 205 )      ! ARO2+HO=0.099*DCB2+...
      LOSS( DCB2     ) =           RXRAT( 180 )      ! DCB2+HO=R2O2+RCHO+CCO_O2
     &                  +          RXRAT( 181 )      ! DCB2=RO2_R+0.5*CCO_O2+...

      PROD( DCB3     ) =   0.051 * RXRAT( 202 )      ! ARO1+HO=0.051*DCB3+...
     &                  +  0.093 * RXRAT( 205 )      ! ARO2+HO=0.093*DCB3+...
      LOSS( DCB3     ) =           RXRAT( 182 )      ! DCB3+HO=R2O2+RCHO+CCO_O2
     &                  +          RXRAT( 183 )      ! DCB3=RO2_R+0.5*CCO_O2+...

      PROD( ETHENE   ) =   0.0
      LOSS( ETHENE   ) =           RXRAT( 185 )      ! ETHENE+HO=RO2_R+1.61*HCHO+...
     &                  +          RXRAT( 186 )      ! ETHENE+O3=0.12*HO+0.12*HO2+...
     &                  +          RXRAT( 187 )      ! ETHENE+NO3=RO2_R+RCHO
     &                  +          RXRAT( 188 )      ! ETHENE+O3P=0.5*HO2+...

      PROD( ISOPRENE ) =   0.0
      LOSS( ISOPRENE ) =           RXRAT( 189 )      ! ISOPRENE+HO=0.907*RO2_R+...
     &                  +          RXRAT( 190 )      ! ISOPRENE+O3=0.266*HO+...
     &                  +          RXRAT( 191 )      ! ISOPRENE+NO3=0.187*NO2+...
     &                  +          RXRAT( 192 )      ! ISOPRENE+O3P=0.01*RO2_N+...

      PROD( ISOPRXN  ) =           RXRAT( 189 )      ! ISOPRENE+HO=ISOPRXN+...
      LOSS( ISOPRXN  ) =   0.0

      PROD( TRP1     ) =   0.0
      LOSS( TRP1     ) =           RXRAT( 193 )      ! TRP1+HO=0.75*RO2_R+...
     &                  +          RXRAT( 194 )      ! TRP1+O3=0.567*HO+0.033*HO2+...
     &                  +          RXRAT( 195 )      ! TRP1+NO3=0.474*NO2+...
     &                  +          RXRAT( 196 )      ! TRP1+O3P=0.147*RCHO+...

      PROD( TRPRXN   ) =           RXRAT( 193 )      ! TRP1+HO=TRPRXN+0.25*RO2_N+...
     &                  +          RXRAT( 194 )      ! TRP1+O3=TRPRXN+0.033*HO2+...
     &                  +          RXRAT( 195 )      ! TRP1+NO3=TRPRXN+...
     &                  +          RXRAT( 196 )      ! TRP1+O3P=TRPRXN+...
      LOSS( TRPRXN   ) =   0.0

      PROD( ALK1     ) =   0.0
      LOSS( ALK1     ) =           RXRAT( 197 )      ! ALK1+HO=RO2_R+CCHO

      PROD( ALK2     ) =   0.0
      LOSS( ALK2     ) =           RXRAT( 198 )      ! ALK2+HO=0.246*HO+0.121*HO2+...

      PROD( ALK3     ) =   0.0
      LOSS( ALK3     ) =           RXRAT( 199 )      ! ALK3+HO=0.695*RO2_R+...

      PROD( ALK4     ) =   0.0
      LOSS( ALK4     ) =           RXRAT( 200 )      ! ALK4+HO=0.835*RO2_R+...

      PROD( ALK5     ) =   0.0
      LOSS( ALK5     ) =           RXRAT( 201 )      ! ALK5+HO=0.653*RO2_R+...

      PROD( ALK5RXN  ) =           RXRAT( 201 )      ! ALK5+HO=ALK5RXN+...
      LOSS( ALK5RXN  ) =   0.0

      PROD( ARO1     ) =   0.0
      LOSS( ARO1     ) =           RXRAT( 202 )      ! ARO1+HO=0.224*HO2+...

      PROD( ARO1RO2  ) =           RXRAT( 202 )      ! ARO1+HO=ARO1RO2+...
      LOSS( ARO1RO2  ) =           RXRAT( 203 )      ! ARO1RO2+NO=NO+TOLNRXN
     &                  +          RXRAT( 204 )      ! ARO1RO2+HO2=HO2+TOLHRXN

      PROD( TOLNRXN  ) =           RXRAT( 203 )      ! ARO1RO2+NO=TOLNRXN+NO
      LOSS( TOLNRXN  ) =   0.0

      PROD( TOLHRXN  ) =           RXRAT( 204 )      ! ARO1RO2+HO2=TOLHRXN+HO2
      LOSS( TOLHRXN  ) =   0.0

      PROD( ARO2     ) =   0.0
      LOSS( ARO2     ) =           RXRAT( 205 )      ! ARO2+HO=0.187*HO2+...

      PROD( ARO2RO2  ) =           RXRAT( 205 )      ! ARO2+HO=ARO2RO2+...
      LOSS( ARO2RO2  ) =           RXRAT( 206 )      ! ARO2RO2+NO=NO+XYLNRXN
     &                  +          RXRAT( 207 )      ! ARO2RO2+HO2=HO2+XYLHRXN

      PROD( XYLNRXN  ) =           RXRAT( 206 )      ! ARO2RO2+NO=XYLNRXN+NO
      LOSS( XYLNRXN  ) =   0.0

      PROD( XYLHRXN  ) =           RXRAT( 207 )      ! ARO2RO2+HO2=XYLHRXN+HO2
      LOSS( XYLHRXN  ) =   0.0

      PROD( BENZENE  ) =   0.0
      LOSS( BENZENE  ) =           RXRAT( 208 )      ! BENZENE+HO=0.236*HO2+...

      PROD( BENZRO2  ) =           RXRAT( 208 )      ! BENZENE+HO=BENZRO2+...
      LOSS( BENZRO2  ) =           RXRAT( 209 )      ! BENZRO2+NO=NO+BNZNRXN
     &                  +          RXRAT( 210 )      ! BENZRO2+HO2=HO2+BNZHRXN

      PROD( BNZNRXN  ) =           RXRAT( 209 )      ! BENZRO2+NO=BNZNRXN+NO
      LOSS( BNZNRXN  ) =   0.0

      PROD( BNZHRXN  ) =           RXRAT( 210 )      ! BENZRO2+HO2=BNZHRXN+HO2
      LOSS( BNZHRXN  ) =   0.0

      PROD( OLE1     ) =   0.0
      LOSS( OLE1     ) =           RXRAT( 211 )      ! OLE1+HO=0.91*RO2_R+...
     &                  +          RXRAT( 212 )      ! OLE1+O3=0.155*HO+0.056*HO2+...
     &                  +          RXRAT( 213 )      ! OLE1+NO3=0.824*RO2_R+...
     &                  +          RXRAT( 214 )      ! OLE1+O3P=0.45*RCHO+...

      PROD( OLE2     ) =   0.0
      LOSS( OLE2     ) =           RXRAT( 215 )      ! OLE2+HO=0.918*RO2_R+...
     &                  +          RXRAT( 216 )      ! OLE2+O3=0.378*HO+0.003*HO2+...
     &                  +          RXRAT( 217 )      ! OLE2+NO3=0.391*NO2+...
     &                  +          RXRAT( 218 )      ! OLE2+O3P=0.013*HO2+...

      PROD( SESQ     ) =   0.0
      LOSS( SESQ     ) =           RXRAT( 222 )      ! SESQ+O3=O3+SESQRXN
     &                  +          RXRAT( 223 )      ! SESQ+HO=HO+SESQRXN
     &                  +          RXRAT( 224 )      ! SESQ+NO3=NO3+SESQRXN

      PROD( SESQRXN  ) =           RXRAT( 222 )      ! SESQ+O3=SESQRXN+O3
     &                  +          RXRAT( 223 )      ! SESQ+HO=SESQRXN+HO
     &                  +          RXRAT( 224 )      ! SESQ+NO3=SESQRXN+NO3
      LOSS( SESQRXN  ) =   0.0

      RETURN

      END
