
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!


C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header: /project/yoj/arc/PROCAN/src/driver/pa/pa_driver.F,v 1.6 2011/11/02 14:33:56 yoj Exp $

      PROGRAM PA_MAIN

C***********************************************************************
C Function: Read the process analysis input commands and generate
C           include files, listings, etc. for use in Models3
C           process analysis  
 
C Preconditions: The program requires include files generated by
C                the Models3 Program Control Processor that 
C                contain information about the chemical mechanism,
C                its species, aerosols, non-reactives, and tracers
  
C Key Subroutines/Functions Called: PA_GLBLDAT
C                                   PA_INIT
C                                   PA_READ
C                                   PA_ERRCHECK
C                                   PA_SETUP_IRR
C                                   PA_SETUP_IPR
C                                   PA_COMPMECH
C                                   PA_REPORT
C                                   PA_WRTCTLEXT
C                                   PA_WRTCMNEXT
C                                   PA_WRTDATEXT

C  Revision History:
C  Prototype created by Jerry Gipson, August, 1996
C  Modified May, 1997 by Jerry Gipson to be consistent with beta CTM
C  Modified Sept, 1997 by Jerry Gipson to be consistent with targeted CTM
C  Updated May 2002 by Jerry Gipson for dynamic allocation version of CCTM
C  Aug 2011 Jeff Young: Replaced I/O API include files with IOAPI's M3UTILIO
                       
C***********************************************************************
      USE M3UTILIO      ! IOAPI module
      IMPLICIT NONE
      
C Includes:
      INCLUDE 'PA_GLOBAL.EXT'
      INCLUDE 'PA_PARSE.EXT'    ! for LOGUNIT
      
C Arguments: None
                                        
C Parameters: None

C External Functions: None

C Local Variables: None

C***********************************************************************
      LOGUNIT = INIT3( )

C Load COMMON blocks in PA_GLOBAL.EXT
C       COMMON /SPCCHR/ RXLABEL,
C      &                N_SPC, <- set in PA_ERRCHECK
C      &                GC_SPC,
C      &                AE_SPC,
C      &                NR_SPC,
C      &                TR_SPC
C       COMMON /SPCINT/ SPC_NAME,
C      &                N_GC_SPC,
C      &                N_AE_SPC,
C      &                N_NR_SPC,
C      &                N_TR_SPC,
C      &                NRXNS,
C      &                IRXBITS,
C      &                NREACT,
C      &                NPRDCT,
C      &                IRR
C       COMMON /SPCREAL/ SC
C       MXSPEC, MXRXNS, MXPRD are parameters in PA_GLOBAL.EXT

      CALL PA_GLBLDAT( MXSPEC, MXRXNS, MXPRD,
     &                 N_GC_SPC, N_AE_SPC, N_NR_SPC, N_TR_SPC,
     &                 GC_SPC, AE_SPC, NR_SPC, TR_SPC,
     &                 NRXNS, RXLABEL, NREACT, NPRDCT, IRXBITS, IRR, SC )
      CALL PA_INIT
      CALL PA_READ
      CALL PA_ERRCHECK
      CALL PA_SETUP_IRR
      CALL PA_SETUP_IPR
      CALL PA_COMPMECH
      CALL PA_REPORT
      CALL PA_WRTCTLEXT
      CALL PA_WRTCMNEXT
      CALL PA_WRTDATEXT
      CALL M3MESG( 'Normal End of Process Analysis Control Program' )

      IF ( .NOT. SHUT3( ) ) THEN
         CALL M3MESG( 'Error shutting down I/O API -- stopping' )
      END IF

      STOP
      END
