
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header$

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

      SUBROUTINE HRPRODLOSS

C***********************************************************************
C
C  FUNCTION: To compute the production and loss rates for EBI species
C
C  PRECONDITIONS: For the SAPRC07TIC family of mechanisms
C
C  RETURN VALUES: None
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, Oct. 18, 2012
C
C***********************************************************************
      USE HRDATA

      IMPLICIT NONE


C.....INCLUDES: NONE


C.....ARGUMENTS: NONE


C.....PARAMETERS: NONE


C.....EXTERNAL FUNCTIONS: NONE


C.....SAVED VARIABLES: NONE
      CHARACTER( 16 ), SAVE  ::  PNAME = 'HRPRODLOSS'   ! Program name


C.....LOCAL VARIABLES:

      INTEGER ESP

C***********************************************************************


      PROD( HNO3             ) =   2.000 * RXRAT(  13 )      ! N2O5=2*HNO3
     &                          +  2.000 * RXRAT(  14 )      ! N2O5=2*HNO3
     &                          +          RXRAT(  25 )      ! OH+NO2=HNO3
     &                          +  0.200 * RXRAT(  39 )      ! NO3+HO2=0.2*HNO3+0.8*NO2+...
     &                          +          RXRAT( 201 )      ! HCHO+NO3=HNO3+HO2+CO
     &                          +          RXRAT( 204 )      ! CCHO+NO3=HNO3+MECO3
     &                          +          RXRAT( 207 )      ! RCHO+NO3=HNO3+RCO3
     &                          +          RXRAT( 227 )      ! GLY+NO3=HNO3+0.63*HO2+...
     &                          +          RXRAT( 230 )      ! MGLY+NO3=HNO3+CO+MECO3
     &                          +          RXRAT( 233 )      ! CRES+NO3=HNO3+BZO
     &                          +          RXRAT( 239 )      ! BALD+NO3=HNO3+BZCO3
     &                          +  0.500 * RXRAT( 249 )      ! MACR+NO3=0.5*HNO3+0.5*RO2C+...
     &                          +  0.150 * RXRAT( 257 )      ! IPRD+NO3=0.15*HNO3+...
     &                          +          RXRAT( 264 )      ! HOCCHO+NO3=HNO3+MECO3
     &                          +  0.967 * RXRAT( 267 )      ! ACROLEIN+NO3=0.967*HNO3+...
     &                          +  0.600 * RXRAT( 714 )      ! NIT1+NO3=0.6*HNO3+...
      LOSS( HNO3             ) =           RXRAT(  27 )      ! HNO3+OH=NO3
     &                          +          RXRAT(  28 )      ! HNO3=OH+NO2

      PROD( CO               ) =           RXRAT( 188 )      ! xCO+NO=CO+NO
     &                          +          RXRAT( 190 )      ! xCO+NO3=CO+NO3
     &                          +  0.500 * RXRAT( 191 )      ! xCO+MEO2=0.5*CO+MEO2
     &                          +  0.500 * RXRAT( 192 )      ! xCO+RO2C=0.5*CO+RO2C
     &                          +  0.500 * RXRAT( 193 )      ! xCO+RO2XC=0.5*CO+RO2XC
     &                          +          RXRAT( 194 )      ! xCO+MECO3=CO+MECO3
     &                          +          RXRAT( 195 )      ! xCO+RCO3=CO+RCO3
     &                          +          RXRAT( 196 )      ! xCO+BZCO3=CO+BZCO3
     &                          +          RXRAT( 197 )      ! xCO+MACO3=CO+MACO3
     &                          +          RXRAT( 198 )      ! HCHO=CO+2*HO2
     &                          +          RXRAT( 199 )      ! HCHO=CO
     &                          +          RXRAT( 200 )      ! HCHO+OH=CO+HO2
     &                          +          RXRAT( 201 )      ! HCHO+NO3=CO+HO2+HNO3
     &                          +          RXRAT( 203 )      ! CCHO=CO+HO2+MEO2
     &                          +          RXRAT( 206 )      ! RCHO=CO+xHO2+yROOH+xCCHO+...
     &                          +  0.380 * RXRAT( 209 )      ! ACETONE=0.38*CO+1.38*MEO2+...
     &                          +  2.000 * RXRAT( 224 )      ! GLY=2*CO+2*HO2
     &                          +          RXRAT( 225 )      ! GLY=CO+HCHO
     &                          +  1.260 * RXRAT( 226 )      ! GLY+OH=1.26*CO+0.63*HO2+...
     &                          +  1.260 * RXRAT( 227 )      ! GLY+NO3=1.26*CO+0.63*HO2+...
     &                          +          RXRAT( 228 )      ! MGLY=CO+HO2+MECO3
     &                          +          RXRAT( 229 )      ! MGLY+OH=CO+MECO3
     &                          +          RXRAT( 230 )      ! MGLY+NO3=CO+HNO3+MECO3
     &                          +  0.522 * RXRAT( 241 )      ! AFG1+O3=0.522*CO+0.522*HO2+...
     &                          +  0.695 * RXRAT( 242 )      ! AFG1=0.695*CO+0.173*MEO2+...
     &                          +  0.522 * RXRAT( 244 )      ! AFG2+O3=0.522*CO+0.522*HO2+...
     &                          +  0.580 * RXRAT( 247 )      ! AFG3+O3=0.58*CO+0.554*HO2+...
     &                          +  0.450 * RXRAT( 248 )      ! MACR+O3=0.45*CO+0.108*HO2+...
     &                          +  0.670 * RXRAT( 251 )      ! MACR=0.67*CO+0.67*HO2+...
     &                          +  0.475 * RXRAT( 252 )      ! MVK+O3=0.475*CO+0.064*HO2+...
     &                          +  0.600 * RXRAT( 254 )      ! MVK=0.6*CO+0.4*MEO2+...
     &                          +  0.498 * RXRAT( 256 )      ! IPRD+O3=0.498*CO+0.4*HO2+...
     &                          +  1.233 * RXRAT( 258 )      ! IPRD=1.233*CO+0.467*MECO3+...
     &                          +          RXRAT( 263 )      ! HOCCHO=CO+2*HO2+HCHO
     &                          +  1.005 * RXRAT( 266 )      ! ACROLEIN+O3=1.005*CO+...
     &                          +  1.188 * RXRAT( 269 )      ! ACROLEIN=1.188*CO+0.178*OH+...
     &                          +  0.510 * RXRAT( 506 )      ! ETHENE+O3=0.51*CO+0.16*OH+...
     &                          +  0.510 * RXRAT( 508 )      ! ETHENE+O3P=0.51*CO+...
     &                          +  0.525 * RXRAT( 510 )      ! PROPENE+O3=0.525*CO+...
     &                          +  0.255 * RXRAT( 514 )      ! BUTADIENE13+O3=0.255*CO+...
     &                          +  0.275 * RXRAT( 517 )      ! ISOPRENE+O3=0.275*CO+...
     &                          +  0.029 * RXRAT( 520 )      ! APIN+O3=0.029*CO+...
     &                          +  0.300 * RXRAT( 523 )      ! ACETYLENE+OH=0.3*CO+0.7*OH+...
     &                          +  1.500 * RXRAT( 524 )      ! ACETYLENE+O3=1.5*CO+0.5*OH+...
     &                          +  0.303 * RXRAT( 538 )      ! OLE1+O3=0.303*CO+...
     &                          +  0.299 * RXRAT( 542 )      ! OLE2+O3=0.299*CO+...
     &                          +  0.249 * RXRAT( 548 )      ! TERP+O3=0.249*CO+...
     &                          +  0.249 * RXRAT( 552 )      ! SESQ+O3=0.249*CO+...
     &                          +          RXRAT( 577 )      ! HCHO+CL=CO+HO2+HCL
     &                          +  1.260 * RXRAT( 585 )      ! GLY+CL=1.26*CO+0.63*HO2+...
     &                          +          RXRAT( 586 )      ! MGLY+CL=CO+HCL+MECO3
     &                          +          RXRAT( 596 )      ! CLCCHO=CO+HO2+RO2C+xCL+...
     &                          +          RXRAT( 636 )      ! ACETYLENE+CL=CO+HO2
     &                          +  0.090 * RXRAT( 694 )      ! HC5OO+NO=0.09*CO+...
     &                          +  0.045 * RXRAT( 696 )      ! HC5OO+MEO2=0.045*CO+...
     &                          +  0.045 * RXRAT( 697 )      ! HC5OO+RO2C=0.045*CO+...
     &                          +  0.090 * RXRAT( 698 )      ! HC5OO+MECO3=0.09*CO+CO2+...
     &                          +  0.590 * RXRAT( 699 )      ! HC5+O3=0.59*CO+0.35*GLY+...
     &                          +  0.029 * RXRAT( 706 )      ! ISOPND+O3=0.029*CO+...
     &                          +  0.260 * RXRAT( 713 )      ! ISOPNB+O3=0.26*CO+...
     &                          +          RXRAT( 715 )      ! NIT1NO3OOA+NO3=CO+PROPNNB+...
     &                          +          RXRAT( 716 )      ! NIT1NO3OOA+NO=CO+PROPNNB+...
     &                          +          RXRAT( 719 )      ! NIT1NO3OOA+RO2C=CO+PROPNNB+...
     &                          +          RXRAT( 720 )      ! NIT1NO3OOA+MEO2=CO+PROPNNB+...
     &                          +          RXRAT( 721 )      ! NIT1NO3OOA+MECO3=CO+CO2+...
     &                          +  0.450 * RXRAT( 728 )      ! NIT1+O3=0.45*CO+...
     &                          +  0.015 * RXRAT( 730 )      ! NIT1OHOO+NO=0.015*CO+...
     &                          +  0.011 * RXRAT( 732 )      ! NIT1OHOO+RO2C=0.011*CO+...
     &                          +  0.011 * RXRAT( 733 )      ! NIT1OHOO+MEO2=0.011*CO+...
     &                          +  0.016 * RXRAT( 734 )      ! NIT1OHOO+MECO3=0.016*CO+...
     &                          +  0.720 * RXRAT( 747 )      ! MACROO+NO=0.72*CO+0.85*HO2+...
     &                          +  0.424 * RXRAT( 749 )      ! MACROO+MEO2=0.424*CO+...
     &                          +  0.424 * RXRAT( 750 )      ! MACROO+RO2C=0.424*CO+...
     &                          +  0.850 * RXRAT( 751 )      ! MACROO+MECO3=0.85*CO+CO2+...
     &                          +          RXRAT( 752 )      ! MACO3+NO=CO+NO2+CO2+HCHO+MEO2
     &                          +          RXRAT( 754 )      ! MACO3+NO3=CO+NO2+CO2+HCHO+MEO2
     &                          +          RXRAT( 755 )      ! MACO3+MEO2=CO+HO2+HCHO+CO2+...
     &                          +          RXRAT( 756 )      ! MACO3+RO2C=CO+CO2+HCHO+MEO2
     &                          +          RXRAT( 757 )      ! MACO3+RO2XC=CO+CO2+HCHO+MEO2
     &                          +          RXRAT( 758 )      ! MACO3+MECO3=CO+MEO2+CO2+...
     &                          +          RXRAT( 759 )      ! MACO3+RCO3=CO+CO2+HCHO+...
     &                          +          RXRAT( 760 )      ! MACO3+BZCO3=CO+CO2+HCHO+...
     &                          +  2.000 * RXRAT( 761 )      ! MACO3+MACO3=2*CO+2*CO2+...
     &                          +          RXRAT( 762 )      ! MAPAN+OH=CO+HACET+NO2
     &                          +  0.520 * RXRAT( 763 )      ! HOCCHO+OH=0.52*CO+0.25*OH+...
     &                          +  0.050 * RXRAT( 764 )      ! HACET+OH=0.05*CO+0.825*HO2+...
     &                          +          RXRAT( 767 )      ! ETHLN=CO+HCHO+HO2+NO2
     &                          +          RXRAT( 775 )      ! MACRN=CO+NO2+HACET+HO2
     &                          +  1.500 * RXRAT( 776 )      ! DHMOB+OH=1.5*CO+0.5*HO2+...
     &                          +  0.251 * RXRAT( 781 )      ! IEPOXOO+HO2=0.251*CO+...
     &                          +  0.251 * RXRAT( 782 )      ! IEPOXOO+NO=0.251*CO+...
     &                          +  0.126 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.126*CO+...
     &                          +  0.126 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.126*CO+...
     &                          +  0.251 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.251*CO+...
      LOSS( CO               ) =           RXRAT(  29 )      ! CO+OH=HO2+CO2

      PROD( CO2              ) =           RXRAT(  29 )      ! OH+CO=CO2+HO2
     &                          +  0.400 * RXRAT(  69 )      ! PAN=0.4*CO2+0.6*NO2+...
     &                          +          RXRAT(  70 )      ! MECO3+NO=CO2+MEO2+NO2
     &                          +          RXRAT(  72 )      ! MECO3+NO3=CO2+MEO2+NO2
     &                          +  0.900 * RXRAT(  73 )      ! MECO3+MEO2=0.9*CO2+...
     &                          +          RXRAT(  74 )      ! MECO3+RO2C=CO2+MEO2
     &                          +          RXRAT(  75 )      ! MECO3+RO2XC=CO2+MEO2
     &                          +  2.000 * RXRAT(  76 )      ! MECO3+MECO3=2*CO2+2*MEO2
     &                          +  0.400 * RXRAT(  79 )      ! PAN2=0.4*CO2+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=CO2+RO2C+xHO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=CO2+RO2C+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=CO2+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=CO2+xHO2+xCCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=CO2+xHO2+xCCHO+...
     &                          +  2.000 * RXRAT(  86 )      ! RCO3+MECO3=2*CO2+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*CO2+2*xHO2+...
     &                          +  0.400 * RXRAT(  90 )      ! PBZN=0.4*CO2+0.6*NO2+...
     &                          +          RXRAT(  91 )      ! BZCO3+NO=CO2+NO2+BZO+RO2C
     &                          +          RXRAT(  93 )      ! BZCO3+NO3=CO2+NO2+BZO+RO2C
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=CO2+HO2+RO2C+...
     &                          +          RXRAT(  95 )      ! BZCO3+RO2C=CO2+BZO+RO2C
     &                          +          RXRAT(  96 )      ! BZCO3+RO2XC=CO2+BZO+RO2C
     &                          +  2.000 * RXRAT(  97 )      ! BZCO3+MECO3=2*CO2+MEO2+BZO+...
     &                          +  2.000 * RXRAT(  98 )      ! BZCO3+RCO3=2*CO2+RO2C+xHO2+...
     &                          +  2.000 * RXRAT(  99 )      ! BZCO3+BZCO3=2*CO2+2*RO2C+2*BZO
     &                          +  0.400 * RXRAT( 102 )      ! MAPAN=0.4*CO2+0.6*NO2+...
     &                          +          RXRAT( 213 )      ! HCOOH+OH=CO2+HO2
     &                          +  0.509 * RXRAT( 214 )      ! CCOOH+OH=0.509*CO2+...
     &                          +  0.143 * RXRAT( 215 )      ! RCOOH+OH=0.143*CO2+xHO2+...
     &                          +  0.174 * RXRAT( 241 )      ! AFG1+O3=0.174*CO2+...
     &                          +  0.174 * RXRAT( 244 )      ! AFG2+O3=0.174*CO2+...
     &                          +  0.190 * RXRAT( 247 )      ! AFG3+O3=0.19*CO2+0.554*HO2+...
     &                          +  0.117 * RXRAT( 248 )      ! MACR+O3=0.117*CO2+...
     &                          +  0.124 * RXRAT( 252 )      ! MVK+O3=0.124*CO2+0.064*HO2+...
     &                          +  0.140 * RXRAT( 256 )      ! IPRD+O3=0.14*CO2+0.4*HO2+...
     &                          +  0.310 * RXRAT( 266 )      ! ACROLEIN+O3=0.31*CO2+...
     &                          +  0.102 * RXRAT( 269 )      ! ACROLEIN=0.102*CO2+...
     &                          +  0.020 * RXRAT( 270 )      ! CCOOOH+OH=0.02*CO2+...
     &                          +          RXRAT( 271 )      ! CCOOOH=CO2+MEO2+OH
     &                          +  0.110 * RXRAT( 272 )      ! RCOOOH+OH=0.11*CO2+...
     &                          +          RXRAT( 273 )      ! RCOOOH=CO2+xCCHO+yROOH+xHO2+OH
     &                          +  0.120 * RXRAT( 506 )      ! ETHENE+O3=0.12*CO2+0.16*OH+...
     &                          +  0.215 * RXRAT( 510 )      ! PROPENE+O3=0.215*CO2+...
     &                          +  0.185 * RXRAT( 514 )      ! BUTADIENE13+O3=0.185*CO2+...
     &                          +  0.122 * RXRAT( 517 )      ! ISOPRENE+O3=0.122*CO2+...
     &                          +  0.017 * RXRAT( 520 )      ! APIN+O3=0.017*CO2+...
     &                          +  0.500 * RXRAT( 524 )      ! ACETYLENE+O3=0.5*CO2+...
     &                          +  0.088 * RXRAT( 538 )      ! OLE1+O3=0.088*CO2+...
     &                          +  0.161 * RXRAT( 542 )      ! OLE2+O3=0.161*CO2+...
     &                          +  0.063 * RXRAT( 548 )      ! TERP+O3=0.063*CO2+...
     &                          +  0.063 * RXRAT( 552 )      ! SESQ+O3=0.063*CO2+...
     &                          +          RXRAT( 681 )      ! ISOPO2+MECO3=CO2+MEO2+...
     &                          +          RXRAT( 692 )      ! NISOPO2+MECO3=CO2+MEO2+...
     &                          +          RXRAT( 698 )      ! HC5OO+MECO3=CO2+MEO2+...
     &                          +  0.080 * RXRAT( 699 )      ! HC5+O3=0.08*CO2+0.35*GLY+...
     &                          +          RXRAT( 705 )      ! ISOPNOOD+MECO3=CO2+MEO2+...
     &                          +  0.017 * RXRAT( 706 )      ! ISOPND+O3=0.017*CO2+...
     &                          +          RXRAT( 712 )      ! ISOPNOOB+MECO3=CO2+MEO2+...
     &                          +  0.070 * RXRAT( 713 )      ! ISOPNB+O3=0.07*CO2+...
     &                          +          RXRAT( 715 )      ! NIT1NO3OOA+NO3=CO2+PROPNNB+...
     &                          +          RXRAT( 716 )      ! NIT1NO3OOA+NO=CO2+PROPNNB+...
     &                          +          RXRAT( 719 )      ! NIT1NO3OOA+RO2C=CO2+CO+...
     &                          +          RXRAT( 720 )      ! NIT1NO3OOA+MEO2=CO2+CO+...
     &                          +  2.000 * RXRAT( 721 )      ! NIT1NO3OOA+MECO3=CO2+MEO2+...
     &                          +          RXRAT( 727 )      ! NIT1NO3OOB+MECO3=CO2+MEO2+...
     &                          +  0.150 * RXRAT( 728 )      ! NIT1+O3=0.15*CO2+0.45*CO+...
     &                          +          RXRAT( 734 )      ! NIT1OHOO+MECO3=CO2+MEO2+...
     &                          +          RXRAT( 739 )      ! DIBOO+MECO3=CO2+...
     &                          +          RXRAT( 745 )      ! MVKOO+MECO3=CO2+MEO2+...
     &                          +          RXRAT( 751 )      ! MACROO+MECO3=CO2+MEO2+HO2+...
     &                          +          RXRAT( 752 )      ! MACO3+NO=CO2+CO+NO2+HCHO+MEO2
     &                          +          RXRAT( 754 )      ! MACO3+NO3=CO2+CO+NO2+HCHO+MEO2
     &                          +          RXRAT( 755 )      ! MACO3+MEO2=CO2+HO2+CO+HCHO+...
     &                          +          RXRAT( 756 )      ! MACO3+RO2C=CO2+CO+HCHO+MEO2
     &                          +          RXRAT( 757 )      ! MACO3+RO2XC=CO2+CO+HCHO+MEO2
     &                          +  2.000 * RXRAT( 758 )      ! MACO3+MECO3=CO2+MEO2+CO+...
     &                          +  2.000 * RXRAT( 759 )      ! MACO3+RCO3=CO2+CO+HCHO+...
     &                          +  2.000 * RXRAT( 760 )      ! MACO3+BZCO3=CO2+CO+HCHO+...
     &                          +  2.000 * RXRAT( 761 )      ! MACO3+MACO3=2*CO2+2*CO+...
     &                          +  0.350 * RXRAT( 763 )      ! HOCCHO+OH=0.35*CO2+0.25*OH+...
     &                          +  0.200 * RXRAT( 764 )      ! HACET+OH=0.2*CO2+0.825*HO2+...
     &                          +          RXRAT( 766 )      ! ETHLN+OH=CO2+HCHO+NO2
     &                          +  0.930 * RXRAT( 774 )      ! MACRN+OH=0.93*CO2+...
     &                          +          RXRAT( 777 )      ! PYRUACD=CO2+CCHO
     &                          +  0.200 * RXRAT( 781 )      ! IEPOXOO+HO2=0.2*CO2+...
     &                          +  0.200 * RXRAT( 782 )      ! IEPOXOO+NO=0.2*CO2+...
     &                          +  0.100 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.1*CO2+...
     &                          +  0.100 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.1*CO2+...
     &                          +  1.200 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.2*CO2+...
      LOSS( CO2              ) =   0.0

      PROD( HO2H             ) =           RXRAT(  37 )      ! HO2+HO2=HO2H
     &                          +          RXRAT(  38 )      ! HO2+HO2=HO2H
      LOSS( HO2H             ) =           RXRAT(  41 )      ! HO2H=2*OH
     &                          +          RXRAT(  42 )      ! HO2H+OH=HO2

      PROD( SO2              ) =   0.0
      LOSS( SO2              ) =           RXRAT(  44 )      ! SO2+OH=HO2+SULF+SULRXN

      PROD( SULF             ) =           RXRAT(  44 )      ! OH+SO2=SULF+HO2+SULRXN
      LOSS( SULF             ) =   0.0

      PROD( SULRXN           ) =           RXRAT(  44 )      ! OH+SO2=SULRXN+SULF+HO2
      LOSS( SULRXN           ) =   0.0

      PROD( NO2EX            ) =           RXRAT(  46 )      ! NO2=NO2EX
      LOSS( NO2EX            ) =           RXRAT(  47 )      ! NO2EX=NO2
     &                          +          RXRAT(  48 )      ! NO2EX=NO2
     &                          +          RXRAT(  49 )      ! NO2EX=OH+HONO

      PROD( MEO2             ) =   0.400 * RXRAT(  69 )      ! PAN=0.4*MEO2+0.6*NO2+...
     &                          +          RXRAT(  70 )      ! MECO3+NO=MEO2+CO2+NO2
     &                          +          RXRAT(  72 )      ! MECO3+NO3=MEO2+CO2+NO2
     &                          +          RXRAT(  74 )      ! MECO3+RO2C=MEO2+CO2
     &                          +          RXRAT(  75 )      ! MECO3+RO2XC=MEO2+CO2
     &                          +  2.000 * RXRAT(  76 )      ! MECO3+MECO3=2*MEO2+2*CO2
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=MEO2+2*CO2+RO2C+...
     &                          +          RXRAT(  97 )      ! BZCO3+MECO3=MEO2+2*CO2+BZO+...
     &                          +          RXRAT( 104 )      ! TBUO=MEO2+ACETONE
     &                          +          RXRAT( 138 )      ! xMEO2+NO=MEO2+NO
     &                          +          RXRAT( 140 )      ! xMEO2+NO3=MEO2+NO3
     &                          +  0.500 * RXRAT( 141 )      ! xMEO2+MEO2=0.5*MEO2+MEO2
     &                          +  0.500 * RXRAT( 142 )      ! xMEO2+RO2C=0.5*MEO2+RO2C
     &                          +  0.500 * RXRAT( 143 )      ! xMEO2+RO2XC=0.5*MEO2+RO2XC
     &                          +          RXRAT( 144 )      ! xMEO2+MECO3=MEO2+MECO3
     &                          +          RXRAT( 145 )      ! xMEO2+RCO3=MEO2+RCO3
     &                          +          RXRAT( 146 )      ! xMEO2+BZCO3=MEO2+BZCO3
     &                          +          RXRAT( 147 )      ! xMEO2+MACO3=MEO2+MACO3
     &                          +          RXRAT( 203 )      ! CCHO=MEO2+HO2+CO
     &                          +  1.380 * RXRAT( 209 )      ! ACETONE=1.38*MEO2+...
     &                          +  0.509 * RXRAT( 214 )      ! CCOOH+OH=0.509*MEO2+...
     &                          +  0.700 * RXRAT( 216 )      ! COOH+OH=0.7*MEO2+0.3*OH+...
     &                          +  0.173 * RXRAT( 242 )      ! AFG1=0.173*MEO2+1.023*HO2+...
     &                          +  0.400 * RXRAT( 254 )      ! MVK=0.4*MEO2+0.6*CO+...
     &                          +  0.234 * RXRAT( 269 )      ! ACROLEIN=0.234*MEO2+...
     &                          +          RXRAT( 271 )      ! CCOOOH=MEO2+CO2+OH
     &                          +          RXRAT( 504 )      ! OH=MEO2
     &                          +  0.510 * RXRAT( 508 )      ! ETHENE+O3P=0.51*MEO2+...
     &                          +  0.355 * RXRAT( 510 )      ! PROPENE+O3=0.355*MEO2+...
     &                          +  0.250 * RXRAT( 518 )      ! ISOPRENE+O3P=0.25*MEO2+...
     &                          +  0.307 * RXRAT( 542 )      ! OLE2+O3=0.307*MEO2+...
     &                          +          RXRAT( 630 )      ! CL=MEO2+HCL
     &                          +          RXRAT( 681 )      ! ISOPO2+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 692 )      ! NISOPO2+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 698 )      ! HC5OO+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 705 )      ! ISOPNOOD+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 712 )      ! ISOPNOOB+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 721 )      ! NIT1NO3OOA+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 727 )      ! NIT1NO3OOB+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 734 )      ! NIT1OHOO+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 739 )      ! DIBOO+MECO3=MEO2+...
     &                          +          RXRAT( 745 )      ! MVKOO+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 751 )      ! MACROO+MECO3=MEO2+CO2+HO2+...
     &                          +          RXRAT( 752 )      ! MACO3+NO=MEO2+CO+CO2+HCHO+NO2
     &                          +          RXRAT( 754 )      ! MACO3+NO3=MEO2+CO+CO2+HCHO+NO2
     &                          +          RXRAT( 756 )      ! MACO3+RO2C=MEO2+CO2+HCHO+CO
     &                          +          RXRAT( 757 )      ! MACO3+RO2XC=MEO2+CO2+HCHO+CO
     &                          +  2.000 * RXRAT( 758 )      ! MACO3+MECO3=MEO2+CO2+CO+...
     &                          +          RXRAT( 759 )      ! MACO3+RCO3=MEO2+CO2+HCHO+...
     &                          +          RXRAT( 760 )      ! MACO3+BZCO3=MEO2+CO2+HCHO+...
     &                          +  2.000 * RXRAT( 761 )      ! MACO3+MACO3=2*MEO2+2*CO2+...
     &                          +  0.125 * RXRAT( 764 )      ! HACET+OH=0.125*MEO2+...
     &                          +          RXRAT( 785 )      ! IEPOXOO+MECO3=MEO2+...
      LOSS( MEO2             ) =           RXRAT(  50 )      ! MEO2+NO=NO2+HCHO+HO2
     &                          +          RXRAT(  51 )      ! MEO2+HO2=COOH
     &                          +          RXRAT(  52 )      ! MEO2+HO2=HCHO
     &                          +          RXRAT(  53 )      ! MEO2+NO3=HCHO+HO2+NO2
     &                          +  2.000 * RXRAT(  54 )      ! MEO2+MEO2=MEOH+HCHO
     &                          +  2.000 * RXRAT(  55 )      ! MEO2+MEO2=2*HCHO+2*HO2
     &                          +          RXRAT(  59 )      ! MEO2+RO2C=0.5*HO2+...
     &                          +          RXRAT(  64 )      ! MEO2+RO2XC=0.5*HO2+...
     &                          +  0.100 * RXRAT(  73 )      ! MEO2+MECO3=0.9*MEO2+...
     &                          +          RXRAT(  83 )      ! MEO2+RCO3=HCHO+HO2+RO2C+...
     &                          +          RXRAT(  94 )      ! MEO2+BZCO3=HCHO+HO2+RO2C+...
     &                          +          RXRAT( 678 )      ! MEO2+ISOPO2=0.45*HO2+...
     &                          +          RXRAT( 689 )      ! MEO2+NISOPO2=0.35*NIT1+...
     &                          +          RXRAT( 696 )      ! MEO2+HC5OO=0.117*HOCCHO+...
     &                          +          RXRAT( 703 )      ! MEO2+ISOPNOOD=0.17*PRD2+...
     &                          +          RXRAT( 710 )      ! MEO2+ISOPNOOB=0.3*HOCCHO+...
     &                          +          RXRAT( 720 )      ! MEO2+NIT1NO3OOA=PROPNNB+CO+...
     &                          +          RXRAT( 726 )      ! MEO2+NIT1NO3OOB=...
     &                          +          RXRAT( 733 )      ! MEO2+NIT1OHOO=...
     &                          +          RXRAT( 737 )      ! MEO2+DIBOO=0.5*HO2+...
     &                          +          RXRAT( 743 )      ! MEO2+MVKOO=0.35*HOCCHO+...
     &                          +          RXRAT( 749 )      ! MEO2+MACROO=0.5*HO2+...
     &                          +          RXRAT( 783 )      ! MEO2+IEPOXOO=0.363*HACET+...

      PROD( HCHO             ) =           RXRAT(  50 )      ! MEO2+NO=HCHO+NO2+HO2
     &                          +          RXRAT(  52 )      ! MEO2+HO2=HCHO
     &                          +          RXRAT(  53 )      ! MEO2+NO3=HCHO+HO2+NO2
     &                          +          RXRAT(  54 )      ! MEO2+MEO2=HCHO+MEOH
     &                          +  2.000 * RXRAT(  55 )      ! MEO2+MEO2=2*HCHO+2*HO2
     &                          +  0.750 * RXRAT(  59 )      ! RO2C+MEO2=0.75*HCHO+...
     &                          +  0.750 * RXRAT(  64 )      ! RO2XC+MEO2=0.75*HCHO+...
     &                          +          RXRAT(  73 )      ! MECO3+MEO2=0.1*HCHO+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=HCHO+HO2+RO2C+...
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=HCHO+HO2+RO2C+...
     &                          +  0.400 * RXRAT( 102 )      ! MAPAN=0.4*HCHO+0.6*NO2+...
     &                          +          RXRAT( 212 )      ! MEOH+OH=HCHO+HO2
     &                          +  0.300 * RXRAT( 216 )      ! COOH+OH=0.3*HCHO+0.3*OH+...
     &                          +          RXRAT( 217 )      ! COOH=HCHO+HO2+OH
     &                          +          RXRAT( 225 )      ! GLY=HCHO+CO
     &                          +  0.100 * RXRAT( 248 )      ! MACR+O3=0.1*HCHO+0.108*HO2+...
     &                          +  0.340 * RXRAT( 251 )      ! MACR=0.34*HCHO+0.67*HO2+...
     &                          +  0.050 * RXRAT( 252 )      ! MVK+O3=0.05*HCHO+0.064*HO2+...
     &                          +  0.124 * RXRAT( 256 )      ! IPRD+O3=0.124*HCHO+0.4*HO2+...
     &                          +  0.300 * RXRAT( 258 )      ! IPRD=0.3*HCHO+0.467*MECO3+...
     &                          +  0.002 * RXRAT( 259 )      ! PRD2+OH=0.002*HCHO+...
     &                          +  0.074 * RXRAT( 262 )      ! RNO3=0.074*HCHO+0.554*xHO2+...
     &                          +          RXRAT( 263 )      ! HOCCHO=HCHO+2*HO2+CO
     &                          +  0.500 * RXRAT( 266 )      ! ACROLEIN+O3=0.5*HCHO+...
     &                          +  0.340 * RXRAT( 269 )      ! ACROLEIN=0.34*HCHO+...
     &                          +          RXRAT( 274 )      ! xHCHO+NO=HCHO+NO
     &                          +          RXRAT( 276 )      ! xHCHO+NO3=HCHO+NO3
     &                          +  0.500 * RXRAT( 277 )      ! xHCHO+MEO2=0.5*HCHO+MEO2
     &                          +  0.500 * RXRAT( 278 )      ! xHCHO+RO2C=0.5*HCHO+RO2C
     &                          +  0.500 * RXRAT( 279 )      ! xHCHO+RO2XC=0.5*HCHO+RO2XC
     &                          +          RXRAT( 280 )      ! xHCHO+MECO3=HCHO+MECO3
     &                          +          RXRAT( 281 )      ! xHCHO+RCO3=HCHO+RCO3
     &                          +          RXRAT( 282 )      ! xHCHO+BZCO3=HCHO+BZCO3
     &                          +          RXRAT( 283 )      ! xHCHO+MACO3=HCHO+MACO3
     &                          +          RXRAT( 506 )      ! ETHENE+O3=HCHO+0.16*OH+...
     &                          +  0.500 * RXRAT( 510 )      ! PROPENE+O3=0.5*HCHO+...
     &                          +  0.500 * RXRAT( 514 )      ! BUTADIENE13+O3=0.5*HCHO+...
     &                          +  0.400 * RXRAT( 517 )      ! ISOPRENE+O3=0.4*HCHO+...
     &                          +  0.500 * RXRAT( 538 )      ! OLE1+O3=0.5*HCHO+...
     &                          +  0.131 * RXRAT( 542 )      ! OLE2+O3=0.131*HCHO+...
     &                          +  0.127 * RXRAT( 548 )      ! TERP+O3=0.127*HCHO+...
     &                          +  0.127 * RXRAT( 552 )      ! SESQ+O3=0.127*HCHO+...
     &                          +          RXRAT( 579 )      ! MEOH+CL=HCHO+HCL+HO2
     &                          +  0.660 * RXRAT( 676 )      ! ISOPO2+NO=0.66*HCHO+...
     &                          +  0.120 * RXRAT( 677 )      ! ISOPO2+HO2=0.12*HCHO+...
     &                          +  1.120 * RXRAT( 678 )      ! ISOPO2+MEO2=0.37*HCHO+...
     &                          +  0.370 * RXRAT( 679 )      ! ISOPO2+RO2C=0.37*HCHO+...
     &                          +  0.750 * RXRAT( 680 )      ! ISOPO2+ISOPO2=0.75*HCHO+...
     &                          +  0.750 * RXRAT( 681 )      ! ISOPO2+MECO3=0.75*HCHO+CO2+...
     &                          +          RXRAT( 683 )      ! HPALD=HCHO+HO2+0.5*HACET+...
     &                          +  0.070 * RXRAT( 686 )      ! NISOPO2+NO3=0.07*HCHO+...
     &                          +  0.070 * RXRAT( 687 )      ! NISOPO2+NO=0.07*HCHO+...
     &                          +  0.785 * RXRAT( 689 )      ! NISOPO2+MEO2=0.035*HCHO+...
     &                          +  0.035 * RXRAT( 690 )      ! NISOPO2+RO2C=0.035*HCHO+...
     &                          +  0.070 * RXRAT( 691 )      ! NISOPO2+NISOPO2=0.07*HCHO+...
     &                          +  0.070 * RXRAT( 692 )      ! NISOPO2+MECO3=0.07*HCHO+...
     &                          +  0.750 * RXRAT( 696 )      ! HC5OO+MEO2=0.25*HCHO+...
     &                          +  0.020 * RXRAT( 699 )      ! HC5+O3=0.02*HCHO+0.35*GLY+...
     &                          +  0.720 * RXRAT( 701 )      ! ISOPNOOD+NO=0.72*HCHO+...
     &                          +  1.110 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.36*HCHO+...
     &                          +  0.360 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.36*HCHO+...
     &                          +  0.720 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.72*HCHO+...
     &                          +  0.038 * RXRAT( 706 )      ! ISOPND+O3=0.038*HCHO+...
     &                          +  0.400 * RXRAT( 708 )      ! ISOPNOOB+NO=0.4*HCHO+...
     &                          +  0.950 * RXRAT( 710 )      ! ISOPNOOB+MEO2=0.2*HCHO+...
     &                          +  0.200 * RXRAT( 711 )      ! ISOPNOOB+RO2C=0.2*HCHO+...
     &                          +  0.400 * RXRAT( 712 )      ! ISOPNOOB+MECO3=0.4*HCHO+...
     &                          +  0.560 * RXRAT( 713 )      ! ISOPNB+O3=0.56*HCHO+...
     &                          +          RXRAT( 720 )      ! NIT1NO3OOA+MEO2=HCHO+CO+...
     &                          +  0.750 * RXRAT( 726 )      ! NIT1NO3OOB+MEO2=0.25*HCHO+...
     &                          +  0.750 * RXRAT( 733 )      ! NIT1OHOO+MEO2=0.25*HCHO+...
     &                          +  0.750 * RXRAT( 737 )      ! DIBOO+MEO2=0.25*HCHO+...
     &                          +  0.265 * RXRAT( 741 )      ! MVKOO+NO=0.265*HCHO+...
     &                          +  0.900 * RXRAT( 743 )      ! MVKOO+MEO2=0.15*HCHO+...
     &                          +  0.150 * RXRAT( 744 )      ! MVKOO+RO2C=0.15*HCHO+...
     &                          +  0.300 * RXRAT( 745 )      ! MVKOO+MECO3=0.3*HCHO+CO2+...
     &                          +  0.130 * RXRAT( 747 )      ! MACROO+NO=0.13*HCHO+...
     &                          +  0.826 * RXRAT( 749 )      ! MACROO+MEO2=0.076*HCHO+...
     &                          +  0.076 * RXRAT( 750 )      ! MACROO+RO2C=0.076*HCHO+...
     &                          +  0.150 * RXRAT( 751 )      ! MACROO+MECO3=0.15*HCHO+CO2+...
     &                          +          RXRAT( 752 )      ! MACO3+NO=HCHO+CO+CO2+NO2+MEO2
     &                          +          RXRAT( 754 )      ! MACO3+NO3=HCHO+CO+CO2+NO2+MEO2
     &                          +  2.000 * RXRAT( 755 )      ! MACO3+MEO2=HCHO+HO2+CO+CO2+...
     &                          +          RXRAT( 756 )      ! MACO3+RO2C=HCHO+CO2+CO+MEO2
     &                          +          RXRAT( 757 )      ! MACO3+RO2XC=HCHO+CO2+CO+MEO2
     &                          +          RXRAT( 758 )      ! MACO3+MECO3=HCHO+MEO2+CO+...
     &                          +          RXRAT( 759 )      ! MACO3+RCO3=HCHO+CO2+CO+...
     &                          +          RXRAT( 760 )      ! MACO3+BZCO3=HCHO+CO2+CO+...
     &                          +  2.000 * RXRAT( 761 )      ! MACO3+MACO3=2*HCHO+2*CO2+...
     &                          +  0.710 * RXRAT( 763 )      ! HOCCHO+OH=0.71*HCHO+...
     &                          +          RXRAT( 765 )      ! HACET=HCHO+MECO3+HO2
     &                          +          RXRAT( 766 )      ! ETHLN+OH=HCHO+CO2+NO2
     &                          +          RXRAT( 767 )      ! ETHLN=HCHO+NO2+HO2+CO
     &                          +          RXRAT( 770 )      ! PROPNN=HCHO+MECO3+NO2
     &                          +          RXRAT( 771 )      ! PROPNNB=HCHO+MECO3+NO2
     &                          +  0.350 * RXRAT( 772 )      ! MVKN+OH=0.35*HCHO+...
     &                          +  0.080 * RXRAT( 774 )      ! MACRN+OH=0.08*HCHO+...
     &                          +  0.375 * RXRAT( 781 )      ! IEPOXOO+HO2=0.375*HCHO+...
     &                          +  0.375 * RXRAT( 782 )      ! IEPOXOO+NO=0.375*HCHO+...
     &                          +  0.938 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.188*HCHO+...
     &                          +  0.188 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.188*HCHO+...
     &                          +  0.375 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.375*HCHO+...
     &                          +  0.750 * RXRAT( 786 )      ! ISOPOOH=0.75*HCHO+0.91*HO2+...
      LOSS( HCHO             ) =           RXRAT( 198 )      ! HCHO=2*HO2+CO
     &                          +          RXRAT( 199 )      ! HCHO=CO
     &                          +          RXRAT( 200 )      ! HCHO+OH=HO2+CO
     &                          +          RXRAT( 201 )      ! HCHO+NO3=HNO3+HO2+CO
     &                          +          RXRAT( 577 )      ! HCHO+CL=HCL+HO2+CO

      PROD( COOH             ) =           RXRAT(  51 )      ! MEO2+HO2=COOH
      LOSS( COOH             ) =           RXRAT( 216 )      ! COOH+OH=0.3*HCHO+0.3*OH+...
     &                          +          RXRAT( 217 )      ! COOH=HCHO+HO2+OH

      PROD( MEOH             ) =           RXRAT(  54 )      ! MEO2+MEO2=MEOH+HCHO
     &                          +  0.250 * RXRAT(  59 )      ! RO2C+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT(  64 )      ! RO2XC+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 678 )      ! ISOPO2+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 689 )      ! NISOPO2+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 696 )      ! HC5OO+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 710 )      ! ISOPNOOB+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 726 )      ! NIT1NO3OOB+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 733 )      ! NIT1OHOO+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 737 )      ! DIBOO+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 743 )      ! MVKOO+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 749 )      ! MACROO+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.25*MEOH+...
      LOSS( MEOH             ) =           RXRAT( 212 )      ! MEOH+OH=HCHO+HO2
     &                          +          RXRAT( 579 )      ! MEOH+CL=HCL+HCHO+HO2

      PROD( RO2C             ) =   0.400 * RXRAT(  79 )      ! PAN2=0.4*RO2C+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=RO2C+NO2+xHO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=RO2C+NO2+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=RO2C+HO2+HCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=RO2C+xHO2+xCCHO+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=RO2C+MEO2+2*CO2+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*RO2C+2*xHO2+...
     &                          +  0.400 * RXRAT(  90 )      ! PBZN=0.4*RO2C+0.6*NO2+...
     &                          +          RXRAT(  91 )      ! BZCO3+NO=RO2C+CO2+BZO+NO2
     &                          +          RXRAT(  93 )      ! BZCO3+NO3=RO2C+CO2+BZO+NO2
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=RO2C+HO2+HCHO+...
     &                          +          RXRAT(  96 )      ! BZCO3+RO2XC=RO2C+BZO+CO2
     &                          +          RXRAT(  97 )      ! BZCO3+MECO3=RO2C+MEO2+BZO+...
     &                          +  2.000 * RXRAT(  98 )      ! BZCO3+RCO3=RO2C+2*CO2+xHO2+...
     &                          +  2.000 * RXRAT(  99 )      ! BZCO3+BZCO3=2*RO2C+2*BZO+2*CO2
     &                          +          RXRAT( 107 )      ! BZO=RO2C+CRES+xHO2
     &                          +  0.035 * RXRAT( 205 )      ! RCHO+OH=0.035*RO2C+...
     &                          +          RXRAT( 206 )      ! RCHO=RO2C+xHO2+yROOH+xCCHO+...
     &                          +          RXRAT( 208 )      ! ACETONE+OH=RO2C+xMECO3+...
     &                          +  0.967 * RXRAT( 210 )      ! MEK+OH=0.967*RO2C+...
     &                          +          RXRAT( 211 )      ! MEK=RO2C+MECO3+xHO2+xCCHO+...
     &                          +  0.491 * RXRAT( 214 )      ! CCOOH+OH=0.491*RO2C+...
     &                          +          RXRAT( 215 )      ! RCOOH+OH=RO2C+xHO2+...
     &                          +  0.251 * RXRAT( 218 )      ! ROOH+OH=0.251*RO2C+...
     &                          +  0.222 * RXRAT( 220 )      ! R6OOH+OH=0.222*RO2C+...
     &                          +  0.782 * RXRAT( 221 )      ! R6OOH=0.782*RO2C+0.142*HO2+...
     &                          +  0.589 * RXRAT( 222 )      ! RAOOH+OH=0.589*RO2C+...
     &                          +  0.800 * RXRAT( 232 )      ! CRES+OH=0.8*RO2C+0.2*BZO+...
     &                          +  0.723 * RXRAT( 240 )      ! AFG1+OH=0.723*RO2C+...
     &                          +  0.652 * RXRAT( 241 )      ! AFG1+O3=0.652*RO2C+...
     &                          +  0.723 * RXRAT( 243 )      ! AFG2+OH=0.723*RO2C+...
     &                          +  0.652 * RXRAT( 244 )      ! AFG2+O3=0.652*RO2C+...
     &                          +  0.733 * RXRAT( 246 )      ! AFG3+OH=0.733*RO2C+...
     &                          +  0.258 * RXRAT( 247 )      ! AFG3+O3=0.258*RO2C+...
     &                          +  0.100 * RXRAT( 248 )      ! MACR+O3=0.1*RO2C+0.108*HO2+...
     &                          +  0.500 * RXRAT( 249 )      ! MACR+NO3=0.5*RO2C+...
     &                          +  0.330 * RXRAT( 251 )      ! MACR=0.33*RO2C+0.67*HO2+...
     &                          +  0.050 * RXRAT( 252 )      ! MVK+O3=0.05*RO2C+0.064*HO2+...
     &                          +  0.670 * RXRAT( 255 )      ! IPRD+OH=0.67*RO2C+...
     &                          +  0.048 * RXRAT( 256 )      ! IPRD+O3=0.048*RO2C+0.4*HO2+...
     &                          +  0.799 * RXRAT( 257 )      ! IPRD+NO3=0.799*RO2C+...
     &                          +  0.473 * RXRAT( 259 )      ! PRD2+OH=0.473*RO2C+...
     &                          +  1.590 * RXRAT( 260 )      ! PRD2=1.59*RO2C+0.4*MECO3+...
     &                          +  0.976 * RXRAT( 261 )      ! RNO3+OH=0.976*RO2C+...
     &                          +  0.721 * RXRAT( 262 )      ! RNO3=0.721*RO2C+0.554*xHO2+...
     &                          +  0.250 * RXRAT( 265 )      ! ACROLEIN+OH=0.25*RO2C+...
     &                          +  0.031 * RXRAT( 267 )      ! ACROLEIN+NO3=0.031*RO2C+...
     &                          +  0.020 * RXRAT( 270 )      ! CCOOOH+OH=0.02*RO2C+...
     &                          +  0.194 * RXRAT( 272 )      ! RCOOOH+OH=0.194*RO2C+...
     &                          +          RXRAT( 505 )      ! ETHENE+OH=RO2C+xHO2+...
     &                          +          RXRAT( 507 )      ! ETHENE+NO3=RO2C+xHO2+xRCHO+...
     &                          +  0.290 * RXRAT( 508 )      ! ETHENE+O3P=0.29*RO2C+...
     &                          +  0.984 * RXRAT( 509 )      ! PROPENE+OH=0.984*RO2C+...
     &                          +  0.949 * RXRAT( 511 )      ! PROPENE+NO3=0.949*RO2C+...
     &                          +  1.189 * RXRAT( 513 )      ! BUTADIENE13+OH=1.189*RO2C+...
     &                          +  1.055 * RXRAT( 515 )      ! BUTADIENE13+NO3=1.055*RO2C+...
     &                          +  0.235 * RXRAT( 516 )      ! BUTADIENE13+O3P=0.235*RO2C+...
     &                          +  0.192 * RXRAT( 517 )      ! ISOPRENE+O3=0.192*RO2C+...
     &                          +  0.240 * RXRAT( 518 )      ! ISOPRENE+O3P=0.24*RO2C+...
     &                          +  1.042 * RXRAT( 519 )      ! APIN+OH=1.042*RO2C+...
     &                          +  1.511 * RXRAT( 520 )      ! APIN+O3=1.511*RO2C+...
     &                          +  1.050 * RXRAT( 521 )      ! APIN+NO3=1.05*RO2C+...
     &                          +  0.290 * RXRAT( 525 )      ! BENZENE+OH=0.29*RO2C+...
     &                          +  0.454 * RXRAT( 526 )      ! TOLUENE+OH=0.454*RO2C+...
     &                          +  0.520 * RXRAT( 527 )      ! MXYL+OH=0.52*RO2C+...
     &                          +  0.554 * RXRAT( 528 )      ! OXYL+OH=0.554*RO2C+...
     &                          +  0.487 * RXRAT( 529 )      ! PXYL+OH=0.487*RO2C+...
     &                          +  0.627 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.050 * RXRAT( 531 )      ! ETOH+OH=0.05*RO2C+...
     &                          +          RXRAT( 532 )      ! ALK1+OH=RO2C+xHO2+xCCHO+yROOH
     &                          +  0.965 * RXRAT( 533 )      ! ALK2+OH=0.965*RO2C+...
     &                          +  1.253 * RXRAT( 534 )      ! ALK3+OH=1.253*RO2C+...
     &                          +  1.763 * RXRAT( 535 )      ! ALK4+OH=1.763*RO2C+...
     &                          +  1.605 * RXRAT( 536 )      ! ALK5+OH=1.605*RO2C+...
     &                          +  1.202 * RXRAT( 537 )      ! OLE1+OH=1.202*RO2C+...
     &                          +  0.090 * RXRAT( 538 )      ! OLE1+O3=0.09*RO2C+...
     &                          +  1.463 * RXRAT( 539 )      ! OLE1+NO3=1.463*RO2C+...
     &                          +  0.953 * RXRAT( 541 )      ! OLE2+OH=0.953*RO2C+...
     &                          +  0.212 * RXRAT( 542 )      ! OLE2+O3=0.212*RO2C+...
     &                          +  1.193 * RXRAT( 543 )      ! OLE2+NO3=1.193*RO2C+...
     &                          +  0.566 * RXRAT( 545 )      ! ARO1+OH=0.566*RO2C+...
     &                          +  0.617 * RXRAT( 546 )      ! ARO2+OH=0.617*RO2C+...
     &                          +  1.211 * RXRAT( 547 )      ! TERP+OH=1.211*RO2C+...
     &                          +  0.490 * RXRAT( 548 )      ! TERP+O3=0.49*RO2C+...
     &                          +  1.786 * RXRAT( 549 )      ! TERP+NO3=1.786*RO2C+...
     &                          +  1.211 * RXRAT( 551 )      ! SESQ+OH=1.211*RO2C+...
     &                          +  0.490 * RXRAT( 552 )      ! SESQ+O3=0.49*RO2C+...
     &                          +  1.786 * RXRAT( 553 )      ! SESQ+NO3=1.786*RO2C+...
     &                          +  0.100 * RXRAT( 580 )      ! RCHO+CL=0.1*RO2C+0.9*RCO3+...
     &                          +          RXRAT( 581 )      ! ACETONE+CL=RO2C+HCL+xHCHO+...
     &                          +  0.975 * RXRAT( 582 )      ! MEK+CL=0.975*RO2C+HCL+...
     &                          +  1.282 * RXRAT( 583 )      ! RNO3+CL=1.282*RO2C+...
     &                          +  0.680 * RXRAT( 584 )      ! PRD2+CL=0.68*RO2C+...
     &                          +  0.588 * RXRAT( 589 )      ! ROOH+CL=0.588*RO2C+...
     &                          +  1.078 * RXRAT( 590 )      ! R6OOH+CL=1.078*RO2C+...
     &                          +  0.589 * RXRAT( 591 )      ! RAOOH+CL=0.589*RO2C+...
     &                          +  1.032 * RXRAT( 592 )      ! ACROLEIN+CL=1.032*RO2C+...
     &                          +  0.802 * RXRAT( 593 )      ! MACR+CL=0.802*RO2C+...
     &                          +  1.283 * RXRAT( 594 )      ! MVK+CL=1.283*RO2C+...
     &                          +  0.730 * RXRAT( 595 )      ! IPRD+CL=0.73*RO2C+...
     &                          +          RXRAT( 596 )      ! CLCCHO=RO2C+CO+HO2+xCL+...
     &                          +          RXRAT( 599 )      ! CLACET=RO2C+MECO3+xCL+...
     &                          +  2.000 * RXRAT( 631 )      ! ETHENE+CL=2*RO2C+xHO2+...
     &                          +  0.971 * RXRAT( 632 )      ! PROPENE+CL=0.971*RO2C+...
     &                          +  1.884 * RXRAT( 633 )      ! BUTADIENE13+CL=1.884*RO2C+...
     &                          +  1.168 * RXRAT( 634 )      ! ISOPRENE+CL=1.168*RO2C+...
     &                          +  2.258 * RXRAT( 635 )      ! APIN+CL=2.258*RO2C+...
     &                          +  0.894 * RXRAT( 637 )      ! TOLUENE+CL=0.894*RO2C+...
     &                          +  0.864 * RXRAT( 638 )      ! MXYL+CL=0.864*RO2C+...
     &                          +  0.864 * RXRAT( 639 )      ! OXYL+CL=0.864*RO2C+...
     &                          +  0.864 * RXRAT( 640 )      ! PXYL+CL=0.864*RO2C+...
     &                          +  0.838 * RXRAT( 641 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.312 * RXRAT( 642 )      ! ETOH+CL=0.312*RO2C+...
     &                          +          RXRAT( 643 )      ! ALK1+CL=RO2C+xHO2+HCL+...
     &                          +  0.970 * RXRAT( 644 )      ! ALK2+CL=0.97*RO2C+...
     &                          +  1.361 * RXRAT( 645 )      ! ALK3+CL=1.361*RO2C+...
     &                          +  1.737 * RXRAT( 646 )      ! ALK4+CL=1.737*RO2C+...
     &                          +  1.541 * RXRAT( 647 )      ! ALK5+CL=1.541*RO2C+...
     &                          +  1.608 * RXRAT( 648 )      ! OLE1+CL=1.608*RO2C+...
     &                          +  1.492 * RXRAT( 649 )      ! OLE2+CL=1.492*RO2C+...
     &                          +  0.840 * RXRAT( 650 )      ! ARO1+CL=0.84*RO2C+...
     &                          +  0.828 * RXRAT( 651 )      ! ARO2+CL=0.828*RO2C+...
     &                          +  2.258 * RXRAT( 652 )      ! TERP+CL=2.258*RO2C+...
     &                          +  2.258 * RXRAT( 653 )      ! SESQ+CL=2.258*RO2C+...
     &                          +          RXRAT( 759 )      ! MACO3+RCO3=RO2C+CO2+HCHO+...
     &                          +          RXRAT( 760 )      ! MACO3+BZCO3=RO2C+CO2+HCHO+...
      LOSS( RO2C             ) =           RXRAT(  56 )      ! RO2C+NO=NO2
     &                          +          RXRAT(  57 )      ! RO2C+HO2=
     &                          +          RXRAT(  58 )      ! RO2C+NO3=NO2
     &                          +          RXRAT(  59 )      ! RO2C+MEO2=0.5*HO2+...
     &                          +  2.000 * RXRAT(  60 )      ! RO2C+RO2C=
     &                          +          RXRAT(  65 )      ! RO2C+RO2XC=
     &                          +          RXRAT(  74 )      ! RO2C+MECO3=MEO2+CO2
     &                          +          RXRAT( 679 )      ! RO2C+ISOPO2=0.45*HO2+...
     &                          +          RXRAT( 690 )      ! RO2C+NISOPO2=0.35*NIT1+...
     &                          +          RXRAT( 697 )      ! RO2C+HC5OO=0.117*HOCCHO+...
     &                          +          RXRAT( 704 )      ! RO2C+ISOPNOOD=0.17*PRD2+...
     &                          +          RXRAT( 711 )      ! RO2C+ISOPNOOB=0.3*HOCCHO+...
     &                          +          RXRAT( 719 )      ! RO2C+NIT1NO3OOA=PROPNNB+CO+...
     &                          +          RXRAT( 725 )      ! RO2C+NIT1NO3OOB=...
     &                          +          RXRAT( 732 )      ! RO2C+NIT1OHOO=...
     &                          +          RXRAT( 738 )      ! RO2C+DIBOO=0.5*HO2+...
     &                          +          RXRAT( 744 )      ! RO2C+MVKOO=0.35*HOCCHO+...
     &                          +          RXRAT( 750 )      ! RO2C+MACROO=0.5*HO2+...
     &                          +          RXRAT( 756 )      ! RO2C+MACO3=CO+CO2+HCHO+MEO2
     &                          +          RXRAT( 784 )      ! RO2C+IEPOXOO=0.363*HACET+...

      PROD( RO2XC            ) =   0.039 * RXRAT( 210 )      ! MEK+OH=0.039*RO2XC+...
     &                          +  0.004 * RXRAT( 218 )      ! ROOH+OH=0.004*RO2XC+...
     &                          +  0.029 * RXRAT( 220 )      ! R6OOH+OH=0.029*RO2XC+...
     &                          +  0.077 * RXRAT( 221 )      ! R6OOH=0.077*RO2XC+...
     &                          +  0.124 * RXRAT( 222 )      ! RAOOH+OH=0.124*RO2XC+...
     &                          +  0.060 * RXRAT( 240 )      ! AFG1+OH=0.06*RO2XC+...
     &                          +  0.060 * RXRAT( 243 )      ! AFG2+OH=0.06*RO2XC+...
     &                          +  0.117 * RXRAT( 246 )      ! AFG3+OH=0.117*RO2XC+...
     &                          +  0.007 * RXRAT( 247 )      ! AFG3+O3=0.007*RO2XC+...
     &                          +  0.041 * RXRAT( 255 )      ! IPRD+OH=0.041*RO2XC+...
     &                          +  0.051 * RXRAT( 257 )      ! IPRD+NO3=0.051*RO2XC+...
     &                          +  0.071 * RXRAT( 259 )      ! PRD2+OH=0.071*RO2XC+...
     &                          +  0.087 * RXRAT( 260 )      ! PRD2=0.087*RO2XC+0.4*MECO3+...
     &                          +  0.175 * RXRAT( 261 )      ! RNO3+OH=0.175*RO2XC+...
     &                          +  0.102 * RXRAT( 262 )      ! RNO3=0.102*RO2XC+...
     &                          +  0.002 * RXRAT( 267 )      ! ACROLEIN+NO3=0.002*RO2XC+...
     &                          +  0.016 * RXRAT( 509 )      ! PROPENE+OH=0.016*RO2XC+...
     &                          +  0.051 * RXRAT( 511 )      ! PROPENE+NO3=0.051*RO2XC+...
     &                          +  0.049 * RXRAT( 513 )      ! BUTADIENE13+OH=0.049*RO2XC+...
     &                          +  0.065 * RXRAT( 515 )      ! BUTADIENE13+NO3=...
     &                          +  0.015 * RXRAT( 516 )      ! BUTADIENE13+O3P=...
     &                          +  0.008 * RXRAT( 517 )      ! ISOPRENE+O3=0.008*RO2XC+...
     &                          +  0.010 * RXRAT( 518 )      ! ISOPRENE+O3P=0.01*RO2XC+...
     &                          +  0.197 * RXRAT( 519 )      ! APIN+OH=0.197*RO2XC+...
     &                          +  0.337 * RXRAT( 520 )      ! APIN+O3=0.337*RO2XC+...
     &                          +  0.293 * RXRAT( 521 )      ! APIN+NO3=0.293*RO2XC+...
     &                          +  0.024 * RXRAT( 525 )      ! BENZENE+OH=0.024*RO2XC+...
     &                          +  0.054 * RXRAT( 526 )      ! TOLUENE+OH=0.054*RO2XC+...
     &                          +  0.082 * RXRAT( 527 )      ! MXYL+OH=0.082*RO2XC+...
     &                          +  0.087 * RXRAT( 528 )      ! OXYL+OH=0.087*RO2XC+...
     &                          +  0.076 * RXRAT( 529 )      ! PXYL+OH=0.076*RO2XC+...
     &                          +  0.121 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.035 * RXRAT( 533 )      ! ALK2+OH=0.035*RO2XC+...
     &                          +  0.070 * RXRAT( 534 )      ! ALK3+OH=0.07*RO2XC+...
     &                          +  0.149 * RXRAT( 535 )      ! ALK4+OH=0.149*RO2XC+...
     &                          +  0.353 * RXRAT( 536 )      ! ALK5+OH=0.353*RO2XC+...
     &                          +  0.128 * RXRAT( 537 )      ! OLE1+OH=0.128*RO2XC+...
     &                          +  0.005 * RXRAT( 538 )      ! OLE1+O3=0.005*RO2XC+...
     &                          +  0.228 * RXRAT( 539 )      ! OLE1+NO3=0.228*RO2XC+...
     &                          +  0.088 * RXRAT( 541 )      ! OLE2+OH=0.088*RO2XC+...
     &                          +  0.003 * RXRAT( 542 )      ! OLE2+O3=0.003*RO2XC+...
     &                          +  0.140 * RXRAT( 543 )      ! OLE2+NO3=0.14*RO2XC+...
     &                          +  0.110 * RXRAT( 545 )      ! ARO1+OH=0.11*RO2XC+...
     &                          +  0.128 * RXRAT( 546 )      ! ARO2+OH=0.128*RO2XC+...
     &                          +  0.201 * RXRAT( 547 )      ! TERP+OH=0.201*RO2XC+...
     &                          +  0.121 * RXRAT( 548 )      ! TERP+O3=0.121*RO2XC+...
     &                          +  0.460 * RXRAT( 549 )      ! TERP+NO3=0.46*RO2XC+...
     &                          +  0.201 * RXRAT( 551 )      ! SESQ+OH=0.201*RO2XC+...
     &                          +  0.121 * RXRAT( 552 )      ! SESQ+O3=0.121*RO2XC+...
     &                          +  0.460 * RXRAT( 553 )      ! SESQ+NO3=0.46*RO2XC+...
     &                          +  0.039 * RXRAT( 582 )      ! MEK+CL=0.039*RO2XC+...
     &                          +  0.202 * RXRAT( 583 )      ! RNO3+CL=0.202*RO2XC+...
     &                          +  0.116 * RXRAT( 584 )      ! PRD2+CL=0.116*RO2XC+...
     &                          +  0.117 * RXRAT( 590 )      ! R6OOH+CL=0.117*RO2XC+...
     &                          +  0.124 * RXRAT( 591 )      ! RAOOH+CL=0.124*RO2XC+...
     &                          +  0.026 * RXRAT( 592 )      ! ACROLEIN+CL=0.026*RO2XC+...
     &                          +  0.033 * RXRAT( 593 )      ! MACR+CL=0.033*RO2XC+...
     &                          +  0.053 * RXRAT( 594 )      ! MVK+CL=0.053*RO2XC+...
     &                          +  0.051 * RXRAT( 595 )      ! IPRD+CL=0.051*RO2XC+...
     &                          +  0.029 * RXRAT( 632 )      ! PROPENE+CL=0.029*RO2XC+...
     &                          +  0.069 * RXRAT( 633 )      ! BUTADIENE13+CL=0.069*RO2XC+...
     &                          +  0.085 * RXRAT( 634 )      ! ISOPRENE+CL=0.085*RO2XC+...
     &                          +  0.582 * RXRAT( 635 )      ! APIN+CL=0.582*RO2XC+...
     &                          +  0.106 * RXRAT( 637 )      ! TOLUENE+CL=0.106*RO2XC+...
     &                          +  0.136 * RXRAT( 638 )      ! MXYL+CL=0.136*RO2XC+...
     &                          +  0.136 * RXRAT( 639 )      ! OXYL+CL=0.136*RO2XC+...
     &                          +  0.136 * RXRAT( 640 )      ! PXYL+CL=0.136*RO2XC+...
     &                          +  0.162 * RXRAT( 641 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.030 * RXRAT( 644 )      ! ALK2+CL=0.03*RO2XC+...
     &                          +  0.070 * RXRAT( 645 )      ! ALK3+CL=0.07*RO2XC+...
     &                          +  0.165 * RXRAT( 646 )      ! ALK4+CL=0.165*RO2XC+...
     &                          +  0.352 * RXRAT( 647 )      ! ALK5+CL=0.352*RO2XC+...
     &                          +  0.127 * RXRAT( 648 )      ! OLE1+CL=0.127*RO2XC+...
     &                          +  0.106 * RXRAT( 649 )      ! OLE2+CL=0.106*RO2XC+...
     &                          +  0.160 * RXRAT( 650 )      ! ARO1+CL=0.16*RO2XC+...
     &                          +  0.172 * RXRAT( 651 )      ! ARO2+CL=0.172*RO2XC+...
     &                          +  0.582 * RXRAT( 652 )      ! TERP+CL=0.582*RO2XC+...
     &                          +  0.582 * RXRAT( 653 )      ! SESQ+CL=0.582*RO2XC+...
      LOSS( RO2XC            ) =           RXRAT(  61 )      ! RO2XC+NO=
     &                          +          RXRAT(  62 )      ! RO2XC+HO2=
     &                          +          RXRAT(  63 )      ! RO2XC+NO3=NO2
     &                          +          RXRAT(  64 )      ! RO2XC+MEO2=0.5*HO2+...
     &                          +          RXRAT(  65 )      ! RO2XC+RO2C=
     &                          +  2.000 * RXRAT(  66 )      ! RO2XC+RO2XC=
     &                          +          RXRAT(  75 )      ! RO2XC+MECO3=MEO2+CO2
     &                          +          RXRAT(  85 )      ! RO2XC+RCO3=RO2C+xHO2+xCCHO+...
     &                          +          RXRAT(  96 )      ! RO2XC+BZCO3=RO2C+BZO+CO2
     &                          +          RXRAT( 757 )      ! RO2XC+MACO3=CO+CO2+HCHO+MEO2

      PROD( CCOOOH           ) =   0.700 * RXRAT(  71 )      ! MECO3+HO2=0.7*CCOOOH+...
      LOSS( CCOOOH           ) =           RXRAT( 270 )      ! CCOOOH+OH=0.98*MECO3+...
     &                          +          RXRAT( 271 )      ! CCOOOH=MEO2+CO2+OH

      PROD( CCOOH            ) =   0.300 * RXRAT(  71 )      ! MECO3+HO2=0.3*CCOOH+...
     &                          +  0.100 * RXRAT(  73 )      ! MECO3+MEO2=0.1*CCOOH+...
     &                          +  0.050 * RXRAT( 269 )      ! ACROLEIN=0.05*CCOOH+...
     &                          +  0.075 * RXRAT( 510 )      ! PROPENE+O3=0.075*CCOOH+...
     &                          +  0.065 * RXRAT( 542 )      ! OLE2+O3=0.065*CCOOH+...
     &                          +  0.125 * RXRAT( 764 )      ! HACET+OH=0.125*CCOOH+...
     &                          +  0.080 * RXRAT( 774 )      ! MACRN+OH=0.08*CCOOH+...
      LOSS( CCOOH            ) =           RXRAT( 214 )      ! CCOOH+OH=0.509*MEO2+...

      PROD( RCO3             ) =           RXRAT(  78 )      ! PAN2=RCO3+NO2
     &                          +  0.600 * RXRAT(  79 )      ! PAN2=0.6*RCO3+0.6*NO2+...
     &                          +          RXRAT( 158 )      ! xRCO3+NO=RCO3+NO
     &                          +          RXRAT( 160 )      ! xRCO3+NO3=RCO3+NO3
     &                          +  0.500 * RXRAT( 161 )      ! xRCO3+MEO2=0.5*RCO3+MEO2
     &                          +  0.500 * RXRAT( 162 )      ! xRCO3+RO2C=0.5*RCO3+RO2C
     &                          +  0.500 * RXRAT( 163 )      ! xRCO3+RO2XC=0.5*RCO3+RO2XC
     &                          +          RXRAT( 164 )      ! xRCO3+MECO3=RCO3+MECO3
     &                          +          RXRAT( 165 )      ! xRCO3+RCO3=RCO3+RCO3
     &                          +          RXRAT( 166 )      ! xRCO3+BZCO3=RCO3+BZCO3
     &                          +          RXRAT( 167 )      ! xRCO3+MACO3=RCO3+MACO3
     &                          +  0.965 * RXRAT( 205 )      ! RCHO+OH=0.965*RCO3+...
     &                          +          RXRAT( 207 )      ! RCHO+NO3=RCO3+HNO3
     &                          +  0.370 * RXRAT( 226 )      ! GLY+OH=0.37*RCO3+1.26*CO+...
     &                          +  0.370 * RXRAT( 227 )      ! GLY+NO3=0.37*RCO3+0.63*HO2+...
     &                          +  0.300 * RXRAT( 258 )      ! IPRD=0.3*RCO3+0.467*MECO3+...
     &                          +  0.600 * RXRAT( 260 )      ! PRD2=0.6*RCO3+0.4*MECO3+...
     &                          +  0.806 * RXRAT( 272 )      ! RCOOOH+OH=0.806*RCO3+...
     &                          +  0.900 * RXRAT( 580 )      ! RCHO+CL=0.9*RCO3+HCL+...
     &                          +  0.370 * RXRAT( 585 )      ! GLY+CL=0.37*RCO3+0.63*HO2+...
     &                          +          RXRAT( 597 )      ! CLCCHO+OH=RCO3
     &                          +          RXRAT( 598 )      ! CLCCHO+CL=RCO3+HCL
      LOSS( RCO3             ) =           RXRAT(  77 )      ! RCO3+NO2=PAN2
     &                          +          RXRAT(  80 )      ! RCO3+NO=NO2+RO2C+xHO2+...
     &                          +          RXRAT(  81 )      ! RCO3+HO2=0.75*RCOOOH+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=NO2+RO2C+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=HCHO+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=RO2C+xHO2+xCCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=RO2C+xHO2+xCCHO+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=2*CO2+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*RO2C+2*xHO2+...
     &                          +          RXRAT(  98 )      ! RCO3+BZCO3=2*CO2+RO2C+xHO2+...
     &                          +          RXRAT( 759 )      ! RCO3+MACO3=CO+CO2+HCHO+...

      PROD( PAN2             ) =           RXRAT(  77 )      ! RCO3+NO2=PAN2
      LOSS( PAN2             ) =           RXRAT(  78 )      ! PAN2=RCO3+NO2
     &                          +          RXRAT(  79 )      ! PAN2=0.6*RCO3+0.6*NO2+...

      PROD( xHO2             ) =   0.400 * RXRAT(  79 )      ! PAN2=0.4*xHO2+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=xHO2+RO2C+NO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=xHO2+RO2C+NO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=xHO2+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=xHO2+RO2C+xCCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=xHO2+RO2C+xCCHO+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=xHO2+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*xHO2+2*RO2C+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=xHO2+RO2C+2*CO2+...
     &                          +          RXRAT( 107 )      ! BZO=xHO2+RO2C+CRES
     &                          +  0.035 * RXRAT( 205 )      ! RCHO+OH=0.035*xHO2+...
     &                          +          RXRAT( 206 )      ! RCHO=xHO2+RO2C+yROOH+xCCHO+...
     &                          +  0.376 * RXRAT( 210 )      ! MEK+OH=0.376*xHO2+...
     &                          +          RXRAT( 211 )      ! MEK=xHO2+RO2C+MECO3+xCCHO+...
     &                          +  0.491 * RXRAT( 214 )      ! CCOOH+OH=0.491*xHO2+...
     &                          +          RXRAT( 215 )      ! RCOOH+OH=xHO2+RO2C+...
     &                          +  0.239 * RXRAT( 218 )      ! ROOH+OH=0.239*xHO2+...
     &                          +  0.090 * RXRAT( 220 )      ! R6OOH+OH=0.09*xHO2+...
     &                          +  0.782 * RXRAT( 221 )      ! R6OOH=0.782*xHO2+0.142*HO2+...
     &                          +  0.565 * RXRAT( 222 )      ! RAOOH+OH=0.565*xHO2+...
     &                          +  0.800 * RXRAT( 232 )      ! CRES+OH=0.8*xHO2+0.8*RO2C+...
     &                          +  0.521 * RXRAT( 240 )      ! AFG1+OH=0.521*xHO2+...
     &                          +  0.521 * RXRAT( 243 )      ! AFG2+OH=0.521*xHO2+...
     &                          +  0.561 * RXRAT( 246 )      ! AFG3+OH=0.561*xHO2+...
     &                          +  0.095 * RXRAT( 247 )      ! AFG3+O3=0.095*xHO2+...
     &                          +  0.500 * RXRAT( 249 )      ! MACR+NO3=0.5*xHO2+0.5*RO2C+...
     &                          +  0.050 * RXRAT( 252 )      ! MVK+O3=0.05*xHO2+0.064*HO2+...
     &                          +  0.670 * RXRAT( 255 )      ! IPRD+OH=0.67*xHO2+...
     &                          +  0.799 * RXRAT( 257 )      ! IPRD+NO3=0.799*xHO2+...
     &                          +  0.379 * RXRAT( 259 )      ! PRD2+OH=0.379*xHO2+...
     &                          +  0.913 * RXRAT( 260 )      ! PRD2=0.913*xHO2+0.4*MECO3+...
     &                          +  0.305 * RXRAT( 261 )      ! RNO3+OH=0.305*xHO2+...
     &                          +  0.554 * RXRAT( 262 )      ! RNO3=0.554*xHO2+0.344*HO2+...
     &                          +  0.250 * RXRAT( 265 )      ! ACROLEIN+OH=0.25*xHO2+...
     &                          +  0.031 * RXRAT( 267 )      ! ACROLEIN+NO3=0.031*xHO2+...
     &                          +  0.084 * RXRAT( 272 )      ! RCOOOH+OH=0.084*xHO2+...
     &                          +          RXRAT( 273 )      ! RCOOOH=xHO2+xCCHO+yROOH+CO2+OH
     &                          +          RXRAT( 505 )      ! ETHENE+OH=xHO2+RO2C+...
     &                          +          RXRAT( 507 )      ! ETHENE+NO3=xHO2+RO2C+xRCHO+...
     &                          +  0.290 * RXRAT( 508 )      ! ETHENE+O3P=0.29*xHO2+...
     &                          +  0.984 * RXRAT( 509 )      ! PROPENE+OH=0.984*xHO2+...
     &                          +  0.949 * RXRAT( 511 )      ! PROPENE+NO3=0.949*xHO2+...
     &                          +  0.951 * RXRAT( 513 )      ! BUTADIENE13+OH=0.951*xHO2+...
     &                          +  0.815 * RXRAT( 515 )      ! BUTADIENE13+NO3=0.815*xHO2+...
     &                          +  0.117 * RXRAT( 516 )      ! BUTADIENE13+O3P=0.117*xHO2+...
     &                          +  0.799 * RXRAT( 519 )      ! APIN+OH=0.799*xHO2+...
     &                          +  0.102 * RXRAT( 520 )      ! APIN+O3=0.102*xHO2+...
     &                          +  0.056 * RXRAT( 521 )      ! APIN+NO3=0.056*xHO2+...
     &                          +  0.290 * RXRAT( 525 )      ! BENZENE+OH=0.29*xHO2+...
     &                          +  0.454 * RXRAT( 526 )      ! TOLUENE+OH=0.454*xHO2+...
     &                          +  0.520 * RXRAT( 527 )      ! MXYL+OH=0.52*xHO2+...
     &                          +  0.554 * RXRAT( 528 )      ! OXYL+OH=0.554*xHO2+...
     &                          +  0.487 * RXRAT( 529 )      ! PXYL+OH=0.487*xHO2+...
     &                          +  0.627 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.050 * RXRAT( 531 )      ! ETOH+OH=0.05*xHO2+0.95*HO2+...
     &                          +          RXRAT( 532 )      ! ALK1+OH=xHO2+RO2C+xCCHO+yROOH
     &                          +  0.965 * RXRAT( 533 )      ! ALK2+OH=0.965*xHO2+...
     &                          +  0.695 * RXRAT( 534 )      ! ALK3+OH=0.695*xHO2+...
     &                          +  0.830 * RXRAT( 535 )      ! ALK4+OH=0.83*xHO2+...
     &                          +  0.647 * RXRAT( 536 )      ! ALK5+OH=0.647*xHO2+...
     &                          +  0.871 * RXRAT( 537 )      ! OLE1+OH=0.871*xHO2+...
     &                          +  0.057 * RXRAT( 538 )      ! OLE1+O3=0.057*xHO2+...
     &                          +  0.772 * RXRAT( 539 )      ! OLE1+NO3=0.772*xHO2+...
     &                          +  0.912 * RXRAT( 541 )      ! OLE2+OH=0.912*xHO2+...
     &                          +  0.041 * RXRAT( 542 )      ! OLE2+O3=0.041*xHO2+...
     &                          +  0.400 * RXRAT( 543 )      ! OLE2+NO3=0.4*xHO2+...
     &                          +  0.566 * RXRAT( 545 )      ! ARO1+OH=0.566*xHO2+...
     &                          +  0.617 * RXRAT( 546 )      ! ARO2+OH=0.617*xHO2+...
     &                          +  0.734 * RXRAT( 547 )      ! TERP+OH=0.734*xHO2+...
     &                          +  0.046 * RXRAT( 548 )      ! TERP+O3=0.046*xHO2+...
     &                          +  0.227 * RXRAT( 549 )      ! TERP+NO3=0.227*xHO2+...
     &                          +  0.734 * RXRAT( 551 )      ! SESQ+OH=0.734*xHO2+...
     &                          +  0.046 * RXRAT( 552 )      ! SESQ+O3=0.046*xHO2+...
     &                          +  0.227 * RXRAT( 553 )      ! SESQ+NO3=0.227*xHO2+...
     &                          +  0.100 * RXRAT( 580 )      ! RCHO+CL=0.1*xHO2+0.9*RCO3+...
     &                          +  0.840 * RXRAT( 582 )      ! MEK+CL=0.84*xHO2+...
     &                          +  0.547 * RXRAT( 583 )      ! RNO3+CL=0.547*xHO2+...
     &                          +  0.541 * RXRAT( 584 )      ! PRD2+CL=0.541*xHO2+...
     &                          +          RXRAT( 587 )      ! CRES+CL=xHO2+HCL+xBALD+yR6OOH
     &                          +  0.482 * RXRAT( 589 )      ! ROOH+CL=0.482*xHO2+...
     &                          +  0.237 * RXRAT( 590 )      ! R6OOH+CL=0.237*xHO2+...
     &                          +  0.565 * RXRAT( 591 )      ! RAOOH+CL=0.565*xHO2+...
     &                          +  0.484 * RXRAT( 592 )      ! ACROLEIN+CL=0.484*xHO2+...
     &                          +  0.802 * RXRAT( 593 )      ! MACR+CL=0.802*xHO2+...
     &                          +  0.322 * RXRAT( 594 )      ! MVK+CL=0.322*xHO2+...
     &                          +  0.712 * RXRAT( 595 )      ! IPRD+CL=0.712*xHO2+...
     &                          +          RXRAT( 631 )      ! ETHENE+CL=xHO2+2*RO2C+...
     &                          +  0.971 * RXRAT( 632 )      ! PROPENE+CL=0.971*xHO2+...
     &                          +  0.390 * RXRAT( 633 )      ! BUTADIENE13+CL=0.39*xHO2+...
     &                          +  0.738 * RXRAT( 634 )      ! ISOPRENE+CL=0.738*xHO2+...
     &                          +  0.252 * RXRAT( 635 )      ! APIN+CL=0.252*xHO2+...
     &                          +  0.894 * RXRAT( 637 )      ! TOLUENE+CL=0.894*xHO2+...
     &                          +  0.864 * RXRAT( 638 )      ! MXYL+CL=0.864*xHO2+...
     &                          +  0.864 * RXRAT( 639 )      ! OXYL+CL=0.864*xHO2+...
     &                          +  0.864 * RXRAT( 640 )      ! PXYL+CL=0.864*xHO2+...
     &                          +  0.838 * RXRAT( 641 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.312 * RXRAT( 642 )      ! ETOH+CL=0.312*xHO2+...
     &                          +          RXRAT( 643 )      ! ALK1+CL=xHO2+HCL+RO2C+...
     &                          +  0.970 * RXRAT( 644 )      ! ALK2+CL=0.97*xHO2+HCL+...
     &                          +  0.835 * RXRAT( 645 )      ! ALK3+CL=0.835*xHO2+HCL+...
     &                          +  0.827 * RXRAT( 646 )      ! ALK4+CL=0.827*xHO2+HCL+...
     &                          +  0.647 * RXRAT( 647 )      ! ALK5+CL=0.647*xHO2+HCL+...
     &                          +  0.873 * RXRAT( 648 )      ! OLE1+CL=0.873*xHO2+...
     &                          +  0.450 * RXRAT( 649 )      ! OLE2+CL=0.45*xHO2+...
     &                          +  0.840 * RXRAT( 650 )      ! ARO1+CL=0.84*xHO2+...
     &                          +  0.828 * RXRAT( 651 )      ! ARO2+CL=0.828*xHO2+...
     &                          +  0.252 * RXRAT( 652 )      ! TERP+CL=0.252*xHO2+...
     &                          +  0.252 * RXRAT( 653 )      ! SESQ+CL=0.252*xHO2+...
     &                          +          RXRAT( 759 )      ! MACO3+RCO3=xHO2+CO2+HCHO+...
      LOSS( xHO2             ) =           RXRAT( 108 )      ! xHO2+NO=NO+HO2
     &                          +          RXRAT( 109 )      ! xHO2+HO2=HO2
     &                          +          RXRAT( 110 )      ! xHO2+NO3=NO3+HO2
     &                          +          RXRAT( 111 )      ! xHO2+MEO2=MEO2+0.5*HO2
     &                          +          RXRAT( 112 )      ! xHO2+RO2C=RO2C+0.5*HO2
     &                          +          RXRAT( 113 )      ! xHO2+RO2XC=RO2XC+0.5*HO2
     &                          +          RXRAT( 114 )      ! xHO2+MECO3=MECO3+HO2
     &                          +          RXRAT( 115 )      ! xHO2+RCO3=RCO3+HO2
     &                          +          RXRAT( 116 )      ! xHO2+BZCO3=BZCO3+HO2
     &                          +          RXRAT( 117 )      ! xHO2+MACO3=MACO3+HO2

      PROD( yROOH            ) =   0.400 * RXRAT(  79 )      ! PAN2=0.4*yROOH+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=yROOH+RO2C+xHO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=yROOH+RO2C+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=yROOH+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=yROOH+xHO2+xCCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=yROOH+xHO2+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=yROOH+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*yROOH+2*xHO2+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=yROOH+RO2C+xHO2+...
     &                          +  0.035 * RXRAT( 205 )      ! RCHO+OH=0.035*yROOH+...
     &                          +          RXRAT( 206 )      ! RCHO=yROOH+xHO2+RO2C+xCCHO+...
     &                          +          RXRAT( 208 )      ! ACETONE+OH=yROOH+xMECO3+...
     &                          +          RXRAT( 210 )      ! MEK+OH=yROOH+0.039*RO2XC+...
     &                          +          RXRAT( 211 )      ! MEK=yROOH+RO2C+xHO2+xCCHO+...
     &                          +  0.491 * RXRAT( 214 )      ! CCOOH+OH=0.491*yROOH+...
     &                          +          RXRAT( 215 )      ! RCOOH+OH=yROOH+xHO2+...
     &                          +  0.256 * RXRAT( 218 )      ! ROOH+OH=0.256*yROOH+...
     &                          +  0.160 * RXRAT( 220 )      ! R6OOH+OH=0.16*yROOH+...
     &                          +  0.100 * RXRAT( 248 )      ! MACR+O3=0.1*yROOH+...
     &                          +  0.500 * RXRAT( 249 )      ! MACR+NO3=0.5*yROOH+...
     &                          +  0.330 * RXRAT( 251 )      ! MACR=0.33*yROOH+0.67*HO2+...
     &                          +  0.050 * RXRAT( 252 )      ! MVK+O3=0.05*yROOH+...
     &                          +  0.157 * RXRAT( 261 )      ! RNO3+OH=0.157*yROOH+...
     &                          +  0.066 * RXRAT( 262 )      ! RNO3=0.066*yROOH+...
     &                          +  0.250 * RXRAT( 265 )      ! ACROLEIN+OH=0.25*yROOH+...
     &                          +  0.033 * RXRAT( 267 )      ! ACROLEIN+NO3=0.033*yROOH+...
     &                          +  0.020 * RXRAT( 270 )      ! CCOOOH+OH=0.02*yROOH+...
     &                          +  0.194 * RXRAT( 272 )      ! RCOOOH+OH=0.194*yROOH+...
     &                          +          RXRAT( 273 )      ! RCOOOH=yROOH+xCCHO+xHO2+CO2+OH
     &                          +          RXRAT( 505 )      ! ETHENE+OH=yROOH+RO2C+...
     &                          +          RXRAT( 507 )      ! ETHENE+NO3=yROOH+RO2C+...
     &                          +  0.290 * RXRAT( 508 )      ! ETHENE+O3P=0.29*yROOH+...
     &                          +          RXRAT( 509 )      ! PROPENE+OH=yROOH+...
     &                          +          RXRAT( 511 )      ! PROPENE+NO3=yROOH+...
     &                          +          RXRAT( 513 )      ! BUTADIENE13+OH=yROOH+...
     &                          +          RXRAT( 515 )      ! BUTADIENE13+NO3=yROOH+...
     &                          +  0.250 * RXRAT( 516 )      ! BUTADIENE13+O3P=0.25*yROOH+...
     &                          +  0.050 * RXRAT( 531 )      ! ETOH+OH=0.05*yROOH+...
     &                          +          RXRAT( 532 )      ! ALK1+OH=yROOH+RO2C+xCCHO+xHO2
     &                          +          RXRAT( 533 )      ! ALK2+OH=yROOH+0.965*RO2C+...
     &                          +  0.983 * RXRAT( 534 )      ! ALK3+OH=0.983*yROOH+...
     &                          +  0.169 * RXRAT( 537 )      ! OLE1+OH=0.169*yROOH+...
     &                          +  0.011 * RXRAT( 538 )      ! OLE1+O3=0.011*yROOH+...
     &                          +  0.169 * RXRAT( 539 )      ! OLE1+NO3=0.169*yROOH+...
     &                          +  0.319 * RXRAT( 541 )      ! OLE2+OH=0.319*yROOH+...
     &                          +  0.073 * RXRAT( 542 )      ! OLE2+O3=0.073*yROOH+...
     &                          +  0.319 * RXRAT( 543 )      ! OLE2+NO3=0.319*yROOH+...
     &                          +  0.100 * RXRAT( 580 )      ! RCHO+CL=0.1*yROOH+0.9*RCO3+...
     &                          +          RXRAT( 581 )      ! ACETONE+CL=yROOH+RO2C+...
     &                          +          RXRAT( 582 )      ! MEK+CL=yROOH+0.975*RO2C+...
     &                          +  0.586 * RXRAT( 589 )      ! ROOH+CL=0.586*yROOH+...
     &                          +  0.784 * RXRAT( 592 )      ! ACROLEIN+CL=0.784*yROOH+...
     &                          +  0.835 * RXRAT( 593 )      ! MACR+CL=0.835*yROOH+...
     &                          +          RXRAT( 594 )      ! MVK+CL=yROOH+0.053*RO2XC+...
     &                          +          RXRAT( 596 )      ! CLCCHO=yROOH+CO+RO2C+xCL+...
     &                          +          RXRAT( 599 )      ! CLACET=yROOH+RO2C+xCL+...
     &                          +          RXRAT( 632 )      ! PROPENE+CL=yROOH+...
     &                          +          RXRAT( 633 )      ! BUTADIENE13+CL=yROOH+...
     &                          +  0.312 * RXRAT( 642 )      ! ETOH+CL=0.312*yROOH+...
     &                          +          RXRAT( 643 )      ! ALK1+CL=yROOH+xHO2+RO2C+...
     &                          +          RXRAT( 644 )      ! ALK2+CL=yROOH+0.97*xHO2+...
     &                          +  0.983 * RXRAT( 645 )      ! ALK3+CL=0.983*yROOH+...
     &                          +  0.169 * RXRAT( 648 )      ! OLE1+CL=0.169*yROOH+...
     &                          +  0.319 * RXRAT( 649 )      ! OLE2+CL=0.319*yROOH+...
     &                          +          RXRAT( 759 )      ! MACO3+RCO3=yROOH+CO2+HCHO+...
      LOSS( yROOH            ) =           RXRAT( 444 )      ! yROOH+NO=NO
     &                          +          RXRAT( 445 )      ! yROOH+HO2=HO2+ROOH
     &                          +          RXRAT( 446 )      ! yROOH+NO3=NO3
     &                          +          RXRAT( 447 )      ! yROOH+MEO2=MEO2+0.5*MEK
     &                          +          RXRAT( 448 )      ! yROOH+RO2C=RO2C+0.5*MEK
     &                          +          RXRAT( 449 )      ! yROOH+RO2XC=RO2XC+0.5*MEK
     &                          +          RXRAT( 450 )      ! yROOH+MECO3=MECO3
     &                          +          RXRAT( 451 )      ! yROOH+RCO3=RCO3
     &                          +          RXRAT( 452 )      ! yROOH+BZCO3=BZCO3
     &                          +          RXRAT( 453 )      ! yROOH+MACO3=MACO3

      PROD( xCCHO            ) =   0.400 * RXRAT(  79 )      ! PAN2=0.4*xCCHO+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=xCCHO+RO2C+xHO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=xCCHO+RO2C+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=xCCHO+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=xCCHO+xHO2+RO2C+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=xCCHO+xHO2+RO2C+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=xCCHO+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*xCCHO+2*xHO2+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=xCCHO+RO2C+xHO2+...
     &                          +  0.035 * RXRAT( 205 )      ! RCHO+OH=0.035*xCCHO+...
     &                          +          RXRAT( 206 )      ! RCHO=xCCHO+xHO2+yROOH+RO2C+...
     &                          +  0.504 * RXRAT( 210 )      ! MEK+OH=0.504*xCCHO+...
     &                          +          RXRAT( 211 )      ! MEK=xCCHO+RO2C+xHO2+MECO3+...
     &                          +  0.142 * RXRAT( 215 )      ! RCOOH+OH=0.142*xCCHO+xHO2+...
     &                          +  0.012 * RXRAT( 218 )      ! ROOH+OH=0.012*xCCHO+...
     &                          +  0.020 * RXRAT( 220 )      ! R6OOH+OH=0.02*xCCHO+...
     &                          +  0.026 * RXRAT( 221 )      ! R6OOH=0.026*xCCHO+...
     &                          +  0.083 * RXRAT( 259 )      ! PRD2+OH=0.083*xCCHO+...
     &                          +  0.163 * RXRAT( 260 )      ! PRD2=0.163*xCCHO+0.4*MECO3+...
     &                          +  0.429 * RXRAT( 261 )      ! RNO3+OH=0.429*xCCHO+...
     &                          +  0.230 * RXRAT( 262 )      ! RNO3=0.23*xCCHO+0.554*xHO2+...
     &                          +  0.167 * RXRAT( 265 )      ! ACROLEIN+OH=0.167*xCCHO+...
     &                          +  0.110 * RXRAT( 272 )      ! RCOOOH+OH=0.11*xCCHO+...
     &                          +          RXRAT( 273 )      ! RCOOOH=xCCHO+xHO2+yROOH+CO2+OH
     &                          +  0.984 * RXRAT( 509 )      ! PROPENE+OH=0.984*xCCHO+...
     &                          +          RXRAT( 532 )      ! ALK1+OH=xCCHO+RO2C+xHO2+yROOH
     &                          +  0.445 * RXRAT( 534 )      ! ALK3+OH=0.445*xCCHO+...
     &                          +  0.438 * RXRAT( 535 )      ! ALK4+OH=0.438*xCCHO+...
     &                          +  0.106 * RXRAT( 536 )      ! ALK5+OH=0.106*xCCHO+...
     &                          +  0.010 * RXRAT( 537 )      ! OLE1+OH=0.01*xCCHO+...
     &                          +  0.011 * RXRAT( 538 )      ! OLE1+O3=0.011*xCCHO+...
     &                          +  0.013 * RXRAT( 539 )      ! OLE1+NO3=0.013*xCCHO+...
     &                          +  0.835 * RXRAT( 541 )      ! OLE2+OH=0.835*xCCHO+...
     &                          +  0.071 * RXRAT( 542 )      ! OLE2+O3=0.071*xCCHO+...
     &                          +  0.579 * RXRAT( 543 )      ! OLE2+NO3=0.579*xCCHO+...
     &                          +  0.002 * RXRAT( 553 )      ! SESQ+NO3=0.002*xCCHO+...
     &                          +  0.100 * RXRAT( 580 )      ! RCHO+CL=0.1*xCCHO+0.9*RCO3+...
     &                          +  0.070 * RXRAT( 582 )      ! MEK+CL=0.07*xCCHO+...
     &                          +  0.300 * RXRAT( 583 )      ! RNO3+CL=0.3*xCCHO+...
     &                          +  0.109 * RXRAT( 584 )      ! PRD2+CL=0.109*xCCHO+...
     &                          +  0.104 * RXRAT( 589 )      ! ROOH+CL=0.104*xCCHO+...
     &                          +  0.186 * RXRAT( 590 )      ! R6OOH+CL=0.186*xCCHO+...
     &                          +          RXRAT( 643 )      ! ALK1+CL=xCCHO+xHO2+RO2C+...
     &                          +  0.340 * RXRAT( 645 )      ! ALK3+CL=0.34*xCCHO+...
     &                          +  0.287 * RXRAT( 646 )      ! ALK4+CL=0.287*xCCHO+...
     &                          +  0.080 * RXRAT( 647 )      ! ALK5+CL=0.08*xCCHO+...
     &                          +  0.206 * RXRAT( 648 )      ! OLE1+CL=0.206*xCCHO+...
     &                          +  0.383 * RXRAT( 649 )      ! OLE2+CL=0.383*xCCHO+...
     &                          +          RXRAT( 759 )      ! MACO3+RCO3=xCCHO+CO2+HCHO+...
      LOSS( xCCHO            ) =           RXRAT( 284 )      ! xCCHO+NO=NO+CCHO
     &                          +          RXRAT( 285 )      ! xCCHO+HO2=HO2
     &                          +          RXRAT( 286 )      ! xCCHO+NO3=NO3+CCHO
     &                          +          RXRAT( 287 )      ! xCCHO+MEO2=MEO2+0.5*CCHO
     &                          +          RXRAT( 288 )      ! xCCHO+RO2C=RO2C+0.5*CCHO
     &                          +          RXRAT( 289 )      ! xCCHO+RO2XC=RO2XC+0.5*CCHO
     &                          +          RXRAT( 290 )      ! xCCHO+MECO3=MECO3+CCHO
     &                          +          RXRAT( 291 )      ! xCCHO+RCO3=RCO3+CCHO
     &                          +          RXRAT( 292 )      ! xCCHO+BZCO3=BZCO3+CCHO
     &                          +          RXRAT( 293 )      ! xCCHO+MACO3=MACO3+CCHO

      PROD( RCOOOH           ) =   0.750 * RXRAT(  81 )      ! RCO3+HO2=0.75*RCOOOH+...
     &                          +  0.750 * RXRAT(  92 )      ! BZCO3+HO2=0.75*RCOOOH+...
     &                          +  0.750 * RXRAT( 718 )      ! NIT1NO3OOA+HO2=0.75*RCOOOH+...
     &                          +  0.750 * RXRAT( 753 )      ! MACO3+HO2=0.75*RCOOOH+...
      LOSS( RCOOOH           ) =           RXRAT( 272 )      ! RCOOOH+OH=0.806*RCO3+...
     &                          +          RXRAT( 273 )      ! RCOOOH=xHO2+xCCHO+yROOH+CO2+OH

      PROD( RCOOH            ) =   0.250 * RXRAT(  81 )      ! RCO3+HO2=0.25*RCOOH+...
     &                          +  0.250 * RXRAT(  92 )      ! BZCO3+HO2=0.25*RCOOH+...
     &                          +  0.372 * RXRAT( 256 )      ! IPRD+O3=0.372*RCOOH+...
     &                          +  0.159 * RXRAT( 538 )      ! OLE1+O3=0.159*RCOOH+...
     &                          +  0.235 * RXRAT( 542 )      ! OLE2+O3=0.235*RCOOH+...
     &                          +  0.068 * RXRAT( 548 )      ! TERP+O3=0.068*RCOOH+...
     &                          +  0.068 * RXRAT( 552 )      ! SESQ+O3=0.068*RCOOH+...
     &                          +  0.170 * RXRAT( 694 )      ! HC5OO+NO=0.17*RCOOH+...
     &                          +  0.085 * RXRAT( 696 )      ! HC5OO+MEO2=0.085*RCOOH+...
     &                          +  0.085 * RXRAT( 697 )      ! HC5OO+RO2C=0.085*RCOOH+...
     &                          +  0.170 * RXRAT( 698 )      ! HC5OO+MECO3=0.17*RCOOH+CO2+...
     &                          +  0.130 * RXRAT( 699 )      ! HC5+O3=0.13*RCOOH+0.35*GLY+...
     &                          +  0.120 * RXRAT( 706 )      ! ISOPND+O3=0.12*RCOOH+...
     &                          +  0.250 * RXRAT( 718 )      ! NIT1NO3OOA+HO2=0.25*RCOOH+...
     &                          +  0.250 * RXRAT( 753 )      ! MACO3+HO2=0.25*RCOOH+...
      LOSS( RCOOH            ) =           RXRAT( 215 )      ! RCOOH+OH=RO2C+xHO2+...

      PROD( BZCO3            ) =           RXRAT(  89 )      ! PBZN=BZCO3+NO2
     &                          +  0.600 * RXRAT(  90 )      ! PBZN=0.6*BZCO3+0.6*NO2+...
     &                          +          RXRAT( 237 )      ! BALD+OH=BZCO3
     &                          +          RXRAT( 239 )      ! BALD+NO3=BZCO3+HNO3
     &                          +          RXRAT( 588 )      ! BALD+CL=BZCO3+HCL
      LOSS( BZCO3            ) =           RXRAT(  88 )      ! BZCO3+NO2=PBZN
     &                          +          RXRAT(  91 )      ! BZCO3+NO=NO2+CO2+BZO+RO2C
     &                          +          RXRAT(  92 )      ! BZCO3+HO2=0.75*RCOOOH+...
     &                          +          RXRAT(  93 )      ! BZCO3+NO3=NO2+CO2+BZO+RO2C
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=HCHO+HO2+RO2C+...
     &                          +          RXRAT(  95 )      ! BZCO3+RO2C=RO2C+BZO+CO2
     &                          +          RXRAT(  96 )      ! BZCO3+RO2XC=RO2C+BZO+CO2
     &                          +          RXRAT(  97 )      ! BZCO3+MECO3=2*CO2+MEO2+BZO+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=2*CO2+RO2C+xHO2+...
     &                          +  2.000 * RXRAT(  99 )      ! BZCO3+BZCO3=2*BZO+2*RO2C+2*CO2
     &                          +          RXRAT( 760 )      ! BZCO3+MACO3=CO+CO2+HCHO+...

      PROD( PBZN             ) =           RXRAT(  88 )      ! BZCO3+NO2=PBZN
      LOSS( PBZN             ) =           RXRAT(  89 )      ! PBZN=BZCO3+NO2
     &                          +          RXRAT(  90 )      ! PBZN=0.6*BZCO3+0.6*NO2+...

      PROD( BZO              ) =   0.400 * RXRAT(  90 )      ! PBZN=0.4*BZO+0.6*NO2+...
     &                          +          RXRAT(  91 )      ! BZCO3+NO=BZO+CO2+NO2+RO2C
     &                          +          RXRAT(  93 )      ! BZCO3+NO3=BZO+CO2+NO2+RO2C
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=BZO+HO2+RO2C+...
     &                          +          RXRAT(  95 )      ! BZCO3+RO2C=BZO+RO2C+CO2
     &                          +          RXRAT(  96 )      ! BZCO3+RO2XC=BZO+RO2C+CO2
     &                          +          RXRAT(  97 )      ! BZCO3+MECO3=BZO+MEO2+2*CO2+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=BZO+RO2C+xHO2+...
     &                          +  2.000 * RXRAT(  99 )      ! BZCO3+BZCO3=2*BZO+2*RO2C+2*CO2
     &                          +  0.200 * RXRAT( 232 )      ! CRES+OH=0.2*BZO+0.8*RO2C+...
     &                          +          RXRAT( 233 )      ! CRES+NO3=BZO+HNO3
     &                          +          RXRAT( 234 )      ! NPHE+OH=BZO
     &                          +          RXRAT( 760 )      ! MACO3+BZCO3=BZO+CO2+HCHO+...
      LOSS( BZO              ) =           RXRAT( 105 )      ! BZO+NO2=NPHE
     &                          +          RXRAT( 106 )      ! BZO+HO2=CRES
     &                          +          RXRAT( 107 )      ! BZO=CRES+RO2C+xHO2

      PROD( MACO3            ) =           RXRAT( 101 )      ! MAPAN=MACO3+NO2
     &                          +  0.600 * RXRAT( 102 )      ! MAPAN=0.6*MACO3+0.6*NO2+...
     &                          +          RXRAT( 168 )      ! xMACO3+NO=MACO3+NO
     &                          +          RXRAT( 170 )      ! xMACO3+NO3=MACO3+NO3
     &                          +  0.500 * RXRAT( 171 )      ! xMACO3+MEO2=0.5*MACO3+MEO2
     &                          +  0.500 * RXRAT( 172 )      ! xMACO3+RO2C=0.5*MACO3+RO2C
     &                          +  0.500 * RXRAT( 173 )      ! xMACO3+RO2XC=0.5*MACO3+RO2XC
     &                          +          RXRAT( 174 )      ! xMACO3+MECO3=MACO3+MECO3
     &                          +          RXRAT( 175 )      ! xMACO3+RCO3=MACO3+RCO3
     &                          +          RXRAT( 176 )      ! xMACO3+BZCO3=MACO3+BZCO3
     &                          +          RXRAT( 177 )      ! xMACO3+MACO3=MACO3+MACO3
     &                          +  0.217 * RXRAT( 240 )      ! AFG1+OH=0.217*MACO3+...
     &                          +  0.500 * RXRAT( 242 )      ! AFG1=0.5*MACO3+0.173*MEO2+...
     &                          +  0.217 * RXRAT( 243 )      ! AFG2+OH=0.217*MACO3+...
     &                          +  0.206 * RXRAT( 246 )      ! AFG3+OH=0.206*MACO3+...
     &                          +  0.500 * RXRAT( 249 )      ! MACR+NO3=0.5*MACO3+...
     &                          +  0.330 * RXRAT( 251 )      ! MACR=0.33*MACO3+0.67*HO2+...
     &                          +  0.400 * RXRAT( 254 )      ! MVK=0.4*MACO3+0.6*CO+...
     &                          +  0.289 * RXRAT( 255 )      ! IPRD+OH=0.289*MACO3+...
     &                          +  0.150 * RXRAT( 257 )      ! IPRD+NO3=0.15*MACO3+...
     &                          +  0.750 * RXRAT( 265 )      ! ACROLEIN+OH=0.75*MACO3+...
     &                          +  0.967 * RXRAT( 267 )      ! ACROLEIN+NO3=0.967*MACO3+...
     &                          +  0.330 * RXRAT( 269 )      ! ACROLEIN=0.33*MACO3+...
     &                          +  0.216 * RXRAT( 592 )      ! ACROLEIN+CL=0.216*MACO3+...
     &                          +  0.165 * RXRAT( 593 )      ! MACR+CL=0.165*MACO3+...
     &                          +  0.154 * RXRAT( 595 )      ! IPRD+CL=0.154*MACO3+...
     &                          +  0.530 * RXRAT( 746 )      ! MACR+OH=0.53*MACO3+0.47*MACROO
      LOSS( MACO3            ) =           RXRAT( 100 )      ! MACO3+NO2=MAPAN
     &                          +          RXRAT( 752 )      ! MACO3+NO=NO2+CO+CO2+HCHO+MEO2
     &                          +          RXRAT( 753 )      ! MACO3+HO2=0.75*RCOOOH+...
     &                          +          RXRAT( 754 )      ! MACO3+NO3=NO2+CO+CO2+HCHO+MEO2
     &                          +          RXRAT( 755 )      ! MACO3+MEO2=HCHO+HO2+CO+CO2+...
     &                          +          RXRAT( 756 )      ! MACO3+RO2C=CO+CO2+HCHO+MEO2
     &                          +          RXRAT( 757 )      ! MACO3+RO2XC=CO+CO2+HCHO+MEO2
     &                          +          RXRAT( 758 )      ! MACO3+MECO3=CO2+MEO2+CO+...
     &                          +          RXRAT( 759 )      ! MACO3+RCO3=CO+CO2+HCHO+...
     &                          +          RXRAT( 760 )      ! MACO3+BZCO3=CO+CO2+HCHO+...
     &                          +  2.000 * RXRAT( 761 )      ! MACO3+MACO3=2*CO+2*CO2+...

      PROD( MAPAN            ) =           RXRAT( 100 )      ! MACO3+NO2=MAPAN
     &                          +          RXRAT( 717 )      ! NIT1NO3OOA+NO2=MAPAN
      LOSS( MAPAN            ) =           RXRAT( 101 )      ! MAPAN=MACO3+NO2
     &                          +          RXRAT( 102 )      ! MAPAN=0.6*MACO3+0.6*NO2+...
     &                          +          RXRAT( 762 )      ! MAPAN+OH=HACET+CO+NO2

      PROD( TBUO             ) =           RXRAT( 178 )      ! xTBUO+NO=TBUO+NO
     &                          +          RXRAT( 180 )      ! xTBUO+NO3=TBUO+NO3
     &                          +  0.500 * RXRAT( 181 )      ! xTBUO+MEO2=0.5*TBUO+MEO2
     &                          +  0.500 * RXRAT( 182 )      ! xTBUO+RO2C=0.5*TBUO+RO2C
     &                          +  0.500 * RXRAT( 183 )      ! xTBUO+RO2XC=0.5*TBUO+RO2XC
     &                          +          RXRAT( 184 )      ! xTBUO+MECO3=TBUO+MECO3
     &                          +          RXRAT( 185 )      ! xTBUO+RCO3=TBUO+RCO3
     &                          +          RXRAT( 186 )      ! xTBUO+BZCO3=TBUO+BZCO3
     &                          +          RXRAT( 187 )      ! xTBUO+MACO3=TBUO+MACO3
      LOSS( TBUO             ) =           RXRAT( 103 )      ! TBUO+NO2=RNO3
     &                          +          RXRAT( 104 )      ! TBUO=ACETONE+MEO2

      PROD( RNO3             ) =           RXRAT( 103 )      ! TBUO+NO2=RNO3
     &                          +          RXRAT( 434 )      ! xRNO3+NO=RNO3+NO
     &                          +          RXRAT( 436 )      ! xRNO3+NO3=RNO3+NO3
     &                          +  0.500 * RXRAT( 437 )      ! xRNO3+MEO2=0.5*RNO3+MEO2
     &                          +  0.500 * RXRAT( 438 )      ! xRNO3+RO2C=0.5*RNO3+RO2C
     &                          +  0.500 * RXRAT( 439 )      ! xRNO3+RO2XC=0.5*RNO3+RO2XC
     &                          +          RXRAT( 440 )      ! xRNO3+MECO3=RNO3+MECO3
     &                          +          RXRAT( 441 )      ! xRNO3+RCO3=RNO3+RCO3
     &                          +          RXRAT( 442 )      ! xRNO3+BZCO3=RNO3+BZCO3
     &                          +          RXRAT( 443 )      ! xRNO3+MACO3=RNO3+MACO3
     &                          +          RXRAT( 474 )      ! zRNO3+NO=RNO3+NO
      LOSS( RNO3             ) =   0.811 * RXRAT( 261 )      ! RNO3+OH=0.189*RNO3+...
     &                          +          RXRAT( 262 )      ! RNO3=0.344*HO2+0.554*xHO2+...
     &                          +  0.945 * RXRAT( 583 )      ! RNO3+CL=0.055*RNO3+...

      PROD( ACETONE          ) =           RXRAT( 104 )      ! TBUO=ACETONE+MEO2
     &                          +          RXRAT( 304 )      ! xACETONE+NO=ACETONE+NO
     &                          +          RXRAT( 306 )      ! xACETONE+NO3=ACETONE+NO3
     &                          +  0.500 * RXRAT( 307 )      ! xACETONE+MEO2=0.5*ACETONE+MEO2
     &                          +  0.500 * RXRAT( 308 )      ! xACETONE+RO2C=0.5*ACETONE+RO2C
     &                          +  0.500 * RXRAT( 309 )      ! xACETONE+RO2XC=0.5*ACETONE+...
     &                          +          RXRAT( 310 )      ! xACETONE+MECO3=ACETONE+MECO3
     &                          +          RXRAT( 311 )      ! xACETONE+RCO3=ACETONE+RCO3
     &                          +          RXRAT( 312 )      ! xACETONE+BZCO3=ACETONE+BZCO3
     &                          +          RXRAT( 313 )      ! xACETONE+MACO3=ACETONE+MACO3
     &                          +  0.051 * RXRAT( 542 )      ! OLE2+O3=0.051*ACETONE+...
      LOSS( ACETONE          ) =           RXRAT( 208 )      ! ACETONE+OH=RO2C+xMECO3+...
     &                          +          RXRAT( 209 )      ! ACETONE=0.62*MECO3+...
     &                          +          RXRAT( 581 )      ! ACETONE+CL=HCL+RO2C+xHCHO+...

      PROD( NPHE             ) =           RXRAT( 105 )      ! BZO+NO2=NPHE
      LOSS( NPHE             ) =           RXRAT( 234 )      ! NPHE+OH=BZO
     &                          +          RXRAT( 235 )      ! NPHE=HONO
     &                          +          RXRAT( 236 )      ! NPHE=

      PROD( CRES             ) =           RXRAT( 106 )      ! BZO+HO2=CRES
     &                          +          RXRAT( 107 )      ! BZO=CRES+RO2C+xHO2
     &                          +  0.570 * RXRAT( 525 )      ! BENZENE+OH=0.57*CRES+...
     &                          +  0.181 * RXRAT( 526 )      ! TOLUENE+OH=0.181*CRES+...
     &                          +  0.159 * RXRAT( 527 )      ! MXYL+OH=0.159*CRES+...
     &                          +  0.161 * RXRAT( 528 )      ! OXYL+OH=0.161*CRES+...
     &                          +  0.159 * RXRAT( 529 )      ! PXYL+OH=0.159*CRES+...
     &                          +  0.022 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.123 * RXRAT( 545 )      ! ARO1+OH=0.123*CRES+...
     &                          +  0.077 * RXRAT( 546 )      ! ARO2+OH=0.077*CRES+...
      LOSS( CRES             ) =           RXRAT( 232 )      ! CRES+OH=0.2*BZO+0.8*RO2C+...
     &                          +          RXRAT( 233 )      ! CRES+NO3=HNO3+BZO
     &                          +          RXRAT( 587 )      ! CRES+CL=HCL+xHO2+xBALD+yR6OOH

      PROD( xOH              ) =   0.012 * RXRAT( 218 )      ! ROOH+OH=0.012*xOH+...
     &                          +  0.041 * RXRAT( 220 )      ! R6OOH+OH=0.041*xOH+...
     &                          +  0.024 * RXRAT( 222 )      ! RAOOH+OH=0.024*xOH+...
     &                          +  0.020 * RXRAT( 270 )      ! CCOOOH+OH=0.02*xOH+...
     &                          +  0.110 * RXRAT( 272 )      ! RCOOOH+OH=0.11*xOH+...
     &                          +  0.104 * RXRAT( 589 )      ! ROOH+CL=0.104*xOH+0.414*OH+...
     &                          +  0.502 * RXRAT( 590 )      ! R6OOH+CL=0.502*xOH+...
     &                          +  0.024 * RXRAT( 591 )      ! RAOOH+CL=0.024*xOH+...
      LOSS( xOH              ) =           RXRAT( 118 )      ! xOH+NO=NO+OH
     &                          +          RXRAT( 119 )      ! xOH+HO2=HO2
     &                          +          RXRAT( 120 )      ! xOH+NO3=NO3+OH
     &                          +          RXRAT( 121 )      ! xOH+MEO2=MEO2+0.5*OH
     &                          +          RXRAT( 122 )      ! xOH+RO2C=RO2C+0.5*OH
     &                          +          RXRAT( 123 )      ! xOH+RO2XC=RO2XC+0.5*OH
     &                          +          RXRAT( 124 )      ! xOH+MECO3=MECO3+OH
     &                          +          RXRAT( 125 )      ! xOH+RCO3=RCO3+OH
     &                          +          RXRAT( 126 )      ! xOH+BZCO3=BZCO3+OH
     &                          +          RXRAT( 127 )      ! xOH+MACO3=MACO3+OH

      PROD( xNO2             ) =   0.313 * RXRAT( 261 )      ! RNO3+OH=0.313*xNO2+...
     &                          +  0.120 * RXRAT( 515 )      ! BUTADIENE13+NO3=0.12*xNO2+...
     &                          +  0.643 * RXRAT( 521 )      ! APIN+NO3=0.643*xNO2+...
     &                          +  0.426 * RXRAT( 543 )      ! OLE2+NO3=0.426*xNO2+...
     &                          +  0.287 * RXRAT( 549 )      ! TERP+NO3=0.287*xNO2+...
     &                          +  0.287 * RXRAT( 553 )      ! SESQ+NO3=0.287*xNO2+...
     &                          +  0.159 * RXRAT( 583 )      ! RNO3+CL=0.159*xNO2+...
      LOSS( xNO2             ) =           RXRAT( 128 )      ! xNO2+NO=NO+NO2
     &                          +          RXRAT( 129 )      ! xNO2+HO2=HO2
     &                          +          RXRAT( 130 )      ! xNO2+NO3=NO3+NO2
     &                          +          RXRAT( 131 )      ! xNO2+MEO2=MEO2+0.5*NO2
     &                          +          RXRAT( 132 )      ! xNO2+RO2C=RO2C+0.5*NO2
     &                          +          RXRAT( 133 )      ! xNO2+RO2XC=RO2XC+0.5*NO2
     &                          +          RXRAT( 134 )      ! xNO2+MECO3=MECO3+NO2
     &                          +          RXRAT( 135 )      ! xNO2+RCO3=RCO3+NO2
     &                          +          RXRAT( 136 )      ! xNO2+BZCO3=BZCO3+NO2
     &                          +          RXRAT( 137 )      ! xNO2+MACO3=MACO3+NO2

      PROD( xMEO2            ) =   0.010 * RXRAT( 535 )      ! ALK4+OH=0.01*xMEO2+...
     &                          +  0.001 * RXRAT( 537 )      ! OLE1+OH=0.001*xMEO2+...
     &                          +  0.035 * RXRAT( 543 )      ! OLE2+NO3=0.035*xMEO2+...
     &                          +  0.003 * RXRAT( 646 )      ! ALK4+CL=0.003*xMEO2+...
     &                          +  0.001 * RXRAT( 649 )      ! OLE2+CL=0.001*xMEO2+...
      LOSS( xMEO2            ) =           RXRAT( 138 )      ! xMEO2+NO=NO+MEO2
     &                          +          RXRAT( 139 )      ! xMEO2+HO2=HO2
     &                          +          RXRAT( 140 )      ! xMEO2+NO3=NO3+MEO2
     &                          +          RXRAT( 141 )      ! xMEO2+MEO2=MEO2+0.5*MEO2
     &                          +          RXRAT( 142 )      ! xMEO2+RO2C=RO2C+0.5*MEO2
     &                          +          RXRAT( 143 )      ! xMEO2+RO2XC=RO2XC+0.5*MEO2
     &                          +          RXRAT( 144 )      ! xMEO2+MECO3=MECO3+MEO2
     &                          +          RXRAT( 145 )      ! xMEO2+RCO3=RCO3+MEO2
     &                          +          RXRAT( 146 )      ! xMEO2+BZCO3=BZCO3+MEO2
     &                          +          RXRAT( 147 )      ! xMEO2+MACO3=MACO3+MEO2

      PROD( xMECO3           ) =           RXRAT( 208 )      ! ACETONE+OH=xMECO3+RO2C+...
     &                          +  0.510 * RXRAT( 210 )      ! MEK+OH=0.51*xMECO3+...
     &                          +  0.201 * RXRAT( 240 )      ! AFG1+OH=0.201*xMECO3+...
     &                          +  0.201 * RXRAT( 243 )      ! AFG2+OH=0.201*xMECO3+...
     &                          +  0.117 * RXRAT( 246 )      ! AFG3+OH=0.117*xMECO3+...
     &                          +  0.330 * RXRAT( 251 )      ! MACR=0.33*xMECO3+0.67*HO2+...
     &                          +  0.029 * RXRAT( 259 )      ! PRD2+OH=0.029*xMECO3+...
     &                          +  0.001 * RXRAT( 520 )      ! APIN+O3=0.001*xMECO3+...
     &                          +  0.011 * RXRAT( 535 )      ! ALK4+OH=0.011*xMECO3+...
     &                          +  0.156 * RXRAT( 542 )      ! OLE2+O3=0.156*xMECO3+...
     &                          +  0.202 * RXRAT( 548 )      ! TERP+O3=0.202*xMECO3+...
     &                          +  0.202 * RXRAT( 552 )      ! SESQ+O3=0.202*xMECO3+...
     &                          +          RXRAT( 581 )      ! ACETONE+CL=xMECO3+RO2C+...
     &                          +  0.085 * RXRAT( 582 )      ! MEK+CL=0.085*xMECO3+...
     &                          +  0.007 * RXRAT( 584 )      ! PRD2+CL=0.007*xMECO3+...
     &                          +  0.625 * RXRAT( 594 )      ! MVK+CL=0.625*xMECO3+...
     &                          +  0.034 * RXRAT( 635 )      ! APIN+CL=0.034*xMECO3+...
     &                          +  0.004 * RXRAT( 646 )      ! ALK4+CL=0.004*xMECO3+...
     &                          +  0.034 * RXRAT( 652 )      ! TERP+CL=0.034*xMECO3+...
     &                          +  0.034 * RXRAT( 653 )      ! SESQ+CL=0.034*xMECO3+...
      LOSS( xMECO3           ) =           RXRAT( 148 )      ! xMECO3+NO=NO+MECO3
     &                          +          RXRAT( 149 )      ! xMECO3+HO2=HO2
     &                          +          RXRAT( 150 )      ! xMECO3+NO3=NO3+MECO3
     &                          +          RXRAT( 151 )      ! xMECO3+MEO2=MEO2+0.5*MECO3
     &                          +          RXRAT( 152 )      ! xMECO3+RO2C=RO2C+0.5*MECO3
     &                          +          RXRAT( 153 )      ! xMECO3+RO2XC=RO2XC+0.5*MECO3
     &                          +          RXRAT( 154 )      ! xMECO3+MECO3=MECO3+MECO3
     &                          +          RXRAT( 155 )      ! xMECO3+RCO3=RCO3+MECO3
     &                          +          RXRAT( 156 )      ! xMECO3+BZCO3=BZCO3+MECO3
     &                          +          RXRAT( 157 )      ! xMECO3+MACO3=MACO3+MECO3

      PROD( xRCO3            ) =   0.074 * RXRAT( 210 )      ! MEK+OH=0.074*xRCO3+...
     &                          +  0.652 * RXRAT( 241 )      ! AFG1+O3=0.652*xRCO3+...
     &                          +  0.652 * RXRAT( 244 )      ! AFG2+O3=0.652*xRCO3+...
     &                          +  0.163 * RXRAT( 247 )      ! AFG3+O3=0.163*xRCO3+...
     &                          +  0.100 * RXRAT( 248 )      ! MACR+O3=0.1*xRCO3+...
     &                          +  0.050 * RXRAT( 252 )      ! MVK+O3=0.05*xRCO3+...
     &                          +  0.048 * RXRAT( 256 )      ! IPRD+O3=0.048*xRCO3+...
     &                          +  0.049 * RXRAT( 259 )      ! PRD2+OH=0.049*xRCO3+...
     &                          +  0.004 * RXRAT( 519 )      ! APIN+OH=0.004*xRCO3+...
     &                          +  0.297 * RXRAT( 520 )      ! APIN+O3=0.297*xRCO3+...
     &                          +  0.007 * RXRAT( 521 )      ! APIN+NO3=0.007*xRCO3+...
     &                          +  0.008 * RXRAT( 542 )      ! OLE2+O3=0.008*xRCO3+...
     &                          +  0.064 * RXRAT( 547 )      ! TERP+OH=0.064*xRCO3+...
     &                          +  0.059 * RXRAT( 548 )      ! TERP+O3=0.059*xRCO3+...
     &                          +  0.026 * RXRAT( 549 )      ! TERP+NO3=0.026*xRCO3+...
     &                          +  0.064 * RXRAT( 551 )      ! SESQ+OH=0.064*xRCO3+...
     &                          +  0.059 * RXRAT( 552 )      ! SESQ+O3=0.059*xRCO3+...
     &                          +  0.026 * RXRAT( 553 )      ! SESQ+NO3=0.026*xRCO3+...
     &                          +  0.036 * RXRAT( 582 )      ! MEK+CL=0.036*xRCO3+...
     &                          +  0.022 * RXRAT( 584 )      ! PRD2+CL=0.022*xRCO3+...
     &                          +  0.050 * RXRAT( 635 )      ! APIN+CL=0.05*xRCO3+...
     &                          +  0.050 * RXRAT( 652 )      ! TERP+CL=0.05*xRCO3+...
     &                          +  0.050 * RXRAT( 653 )      ! SESQ+CL=0.05*xRCO3+...
      LOSS( xRCO3            ) =           RXRAT( 158 )      ! xRCO3+NO=NO+RCO3
     &                          +          RXRAT( 159 )      ! xRCO3+HO2=HO2
     &                          +          RXRAT( 160 )      ! xRCO3+NO3=NO3+RCO3
     &                          +          RXRAT( 161 )      ! xRCO3+MEO2=MEO2+0.5*RCO3
     &                          +          RXRAT( 162 )      ! xRCO3+RO2C=RO2C+0.5*RCO3
     &                          +          RXRAT( 163 )      ! xRCO3+RO2XC=RO2XC+0.5*RCO3
     &                          +          RXRAT( 164 )      ! xRCO3+MECO3=MECO3+RCO3
     &                          +          RXRAT( 165 )      ! xRCO3+RCO3=RCO3+RCO3
     &                          +          RXRAT( 166 )      ! xRCO3+BZCO3=BZCO3+RCO3
     &                          +          RXRAT( 167 )      ! xRCO3+MACO3=MACO3+RCO3

      PROD( xMACO3           ) =   0.118 * RXRAT( 516 )      ! BUTADIENE13+O3P=...
     &                          +  0.192 * RXRAT( 517 )      ! ISOPRENE+O3=0.192*xMACO3+...
     &                          +  0.240 * RXRAT( 518 )      ! ISOPRENE+O3P=0.24*xMACO3+...
     &                          +  0.016 * RXRAT( 635 )      ! APIN+CL=0.016*xMACO3+...
     &                          +  0.016 * RXRAT( 652 )      ! TERP+CL=0.016*xMACO3+...
     &                          +  0.016 * RXRAT( 653 )      ! SESQ+CL=0.016*xMACO3+...
      LOSS( xMACO3           ) =           RXRAT( 168 )      ! xMACO3+NO=NO+MACO3
     &                          +          RXRAT( 169 )      ! xMACO3+HO2=HO2
     &                          +          RXRAT( 170 )      ! xMACO3+NO3=NO3+MACO3
     &                          +          RXRAT( 171 )      ! xMACO3+MEO2=MEO2+0.5*MACO3
     &                          +          RXRAT( 172 )      ! xMACO3+RO2C=RO2C+0.5*MACO3
     &                          +          RXRAT( 173 )      ! xMACO3+RO2XC=RO2XC+0.5*MACO3
     &                          +          RXRAT( 174 )      ! xMACO3+MECO3=MECO3+MACO3
     &                          +          RXRAT( 175 )      ! xMACO3+RCO3=RCO3+MACO3
     &                          +          RXRAT( 176 )      ! xMACO3+BZCO3=BZCO3+MACO3
     &                          +          RXRAT( 177 )      ! xMACO3+MACO3=MACO3+MACO3

      PROD( xTBUO            ) =   0.236 * RXRAT( 534 )      ! ALK3+OH=0.236*xTBUO+...
     &                          +  0.094 * RXRAT( 645 )      ! ALK3+CL=0.094*xTBUO+...
      LOSS( xTBUO            ) =           RXRAT( 178 )      ! xTBUO+NO=NO+TBUO
     &                          +          RXRAT( 179 )      ! xTBUO+HO2=HO2
     &                          +          RXRAT( 180 )      ! xTBUO+NO3=NO3+TBUO
     &                          +          RXRAT( 181 )      ! xTBUO+MEO2=MEO2+0.5*TBUO
     &                          +          RXRAT( 182 )      ! xTBUO+RO2C=RO2C+0.5*TBUO
     &                          +          RXRAT( 183 )      ! xTBUO+RO2XC=RO2XC+0.5*TBUO
     &                          +          RXRAT( 184 )      ! xTBUO+MECO3=MECO3+TBUO
     &                          +          RXRAT( 185 )      ! xTBUO+RCO3=RCO3+TBUO
     &                          +          RXRAT( 186 )      ! xTBUO+BZCO3=BZCO3+TBUO
     &                          +          RXRAT( 187 )      ! xTBUO+MACO3=MACO3+TBUO

      PROD( xCO              ) =   0.035 * RXRAT( 205 )      ! RCHO+OH=0.035*xCO+...
     &                          +  0.334 * RXRAT( 240 )      ! AFG1+OH=0.334*xCO+...
     &                          +  0.334 * RXRAT( 243 )      ! AFG2+OH=0.334*xCO+...
     &                          +  0.114 * RXRAT( 246 )      ! AFG3+OH=0.114*xCO+...
     &                          +  0.500 * RXRAT( 249 )      ! MACR+NO3=0.5*xCO+0.5*RO2C+...
     &                          +  0.336 * RXRAT( 255 )      ! IPRD+OH=0.336*xCO+...
     &                          +  0.572 * RXRAT( 257 )      ! IPRD+NO3=0.572*xCO+...
     &                          +  0.167 * RXRAT( 265 )      ! ACROLEIN+OH=0.167*xCO+...
     &                          +  0.031 * RXRAT( 267 )      ! ACROLEIN+NO3=0.031*xCO+...
     &                          +  0.278 * RXRAT( 508 )      ! ETHENE+O3P=0.278*xCO+...
     &                          +  0.115 * RXRAT( 516 )      ! BUTADIENE13+O3P=0.115*xCO+...
     &                          +  0.002 * RXRAT( 519 )      ! APIN+OH=0.002*xCO+...
     &                          +  0.051 * RXRAT( 520 )      ! APIN+O3=0.051*xCO+...
     &                          +  0.005 * RXRAT( 521 )      ! APIN+NO3=0.005*xCO+...
     &                          +  0.002 * RXRAT( 535 )      ! ALK4+OH=0.002*xCO+...
     &                          +  0.001 * RXRAT( 547 )      ! TERP+OH=0.001*xCO+...
     &                          +  0.012 * RXRAT( 549 )      ! TERP+NO3=0.012*xCO+...
     &                          +  0.001 * RXRAT( 551 )      ! SESQ+OH=0.001*xCO+...
     &                          +  0.012 * RXRAT( 553 )      ! SESQ+NO3=0.012*xCO+...
     &                          +  0.100 * RXRAT( 580 )      ! RCHO+CL=0.1*xCO+0.9*RCO3+...
     &                          +  0.484 * RXRAT( 592 )      ! ACROLEIN+CL=0.484*xCO+...
     &                          +  0.541 * RXRAT( 593 )      ! MACR+CL=0.541*xCO+...
     &                          +  0.498 * RXRAT( 595 )      ! IPRD+CL=0.498*xCO+...
     &                          +  0.035 * RXRAT( 635 )      ! APIN+CL=0.035*xCO+...
     &                          +  0.003 * RXRAT( 646 )      ! ALK4+CL=0.003*xCO+...
     &                          +  0.035 * RXRAT( 652 )      ! TERP+CL=0.035*xCO+...
     &                          +  0.035 * RXRAT( 653 )      ! SESQ+CL=0.035*xCO+...
      LOSS( xCO              ) =           RXRAT( 188 )      ! xCO+NO=NO+CO
     &                          +          RXRAT( 189 )      ! xCO+HO2=HO2
     &                          +          RXRAT( 190 )      ! xCO+NO3=NO3+CO
     &                          +          RXRAT( 191 )      ! xCO+MEO2=MEO2+0.5*CO
     &                          +          RXRAT( 192 )      ! xCO+RO2C=RO2C+0.5*CO
     &                          +          RXRAT( 193 )      ! xCO+RO2XC=RO2XC+0.5*CO
     &                          +          RXRAT( 194 )      ! xCO+MECO3=MECO3+CO
     &                          +          RXRAT( 195 )      ! xCO+RCO3=RCO3+CO
     &                          +          RXRAT( 196 )      ! xCO+BZCO3=BZCO3+CO
     &                          +          RXRAT( 197 )      ! xCO+MACO3=MACO3+CO

      PROD( CCHO             ) =   0.001 * RXRAT( 259 )      ! PRD2+OH=0.001*CCHO+...
     &                          +  0.214 * RXRAT( 262 )      ! RNO3=0.214*CCHO+0.554*xHO2+...
     &                          +          RXRAT( 284 )      ! xCCHO+NO=CCHO+NO
     &                          +          RXRAT( 286 )      ! xCCHO+NO3=CCHO+NO3
     &                          +  0.500 * RXRAT( 287 )      ! xCCHO+MEO2=0.5*CCHO+MEO2
     &                          +  0.500 * RXRAT( 288 )      ! xCCHO+RO2C=0.5*CCHO+RO2C
     &                          +  0.500 * RXRAT( 289 )      ! xCCHO+RO2XC=0.5*CCHO+RO2XC
     &                          +          RXRAT( 290 )      ! xCCHO+MECO3=CCHO+MECO3
     &                          +          RXRAT( 291 )      ! xCCHO+RCO3=CCHO+RCO3
     &                          +          RXRAT( 292 )      ! xCCHO+BZCO3=CCHO+BZCO3
     &                          +          RXRAT( 293 )      ! xCCHO+MACO3=CCHO+MACO3
     &                          +  0.100 * RXRAT( 508 )      ! ETHENE+O3P=0.1*CCHO+...
     &                          +  0.500 * RXRAT( 510 )      ! PROPENE+O3=0.5*CCHO+...
     &                          +  0.950 * RXRAT( 531 )      ! ETOH+OH=0.95*CCHO+...
     &                          +  0.453 * RXRAT( 542 )      ! OLE2+O3=0.453*CCHO+...
     &                          +  0.688 * RXRAT( 642 )      ! ETOH+CL=0.688*CCHO+...
     &                          +          RXRAT( 777 )      ! PYRUACD=CCHO+CO2
      LOSS( CCHO             ) =           RXRAT( 202 )      ! CCHO+OH=MECO3
     &                          +          RXRAT( 203 )      ! CCHO=CO+HO2+MEO2
     &                          +          RXRAT( 204 )      ! CCHO+NO3=HNO3+MECO3
     &                          +          RXRAT( 578 )      ! CCHO+CL=HCL+MECO3

      PROD( RCHO             ) =   0.744 * RXRAT( 218 )      ! ROOH+OH=0.744*RCHO+...
     &                          +          RXRAT( 219 )      ! ROOH=RCHO+HO2+OH
     &                          +  0.085 * RXRAT( 221 )      ! R6OOH=0.085*RCHO+0.142*HO2+...
     &                          +          RXRAT( 250 )      ! MACR+O3P=RCHO
     &                          +  0.450 * RXRAT( 253 )      ! MVK+O3P=0.45*RCHO+0.55*MEK
     &                          +  0.143 * RXRAT( 259 )      ! PRD2+OH=0.143*RCHO+...
     &                          +  0.001 * RXRAT( 261 )      ! RNO3+OH=0.001*RCHO+...
     &                          +  0.074 * RXRAT( 262 )      ! RNO3=0.074*RCHO+0.554*xHO2+...
     &                          +          RXRAT( 268 )      ! ACROLEIN+O3P=RCHO
     &                          +          RXRAT( 294 )      ! xRCHO+NO=RCHO+NO
     &                          +          RXRAT( 296 )      ! xRCHO+NO3=RCHO+NO3
     &                          +  0.500 * RXRAT( 297 )      ! xRCHO+MEO2=0.5*RCHO+MEO2
     &                          +  0.500 * RXRAT( 298 )      ! xRCHO+RO2C=0.5*RCHO+RO2C
     &                          +  0.500 * RXRAT( 299 )      ! xRCHO+RO2XC=0.5*RCHO+RO2XC
     &                          +          RXRAT( 300 )      ! xRCHO+MECO3=RCHO+MECO3
     &                          +          RXRAT( 301 )      ! xRCHO+RCO3=RCHO+RCO3
     &                          +          RXRAT( 302 )      ! xRCHO+BZCO3=RCHO+BZCO3
     &                          +          RXRAT( 303 )      ! xRCHO+MACO3=RCHO+MACO3
     &                          +  0.450 * RXRAT( 512 )      ! PROPENE+O3P=0.45*RCHO+0.55*MEK
     &                          +  0.500 * RXRAT( 538 )      ! OLE1+O3=0.5*RCHO+...
     &                          +  0.450 * RXRAT( 540 )      ! OLE1+O3P=0.45*RCHO+...
     &                          +  0.333 * RXRAT( 542 )      ! OLE2+O3=0.333*RCHO+...
     &                          +  0.079 * RXRAT( 544 )      ! OLE2+O3P=0.079*RCHO+...
     &                          +  0.237 * RXRAT( 550 )      ! TERP+O3P=0.237*RCHO+...
     &                          +  0.237 * RXRAT( 554 )      ! SESQ+O3P=0.237*RCHO+...
     &                          +  0.009 * RXRAT( 583 )      ! RNO3+CL=0.009*RCHO+...
     &                          +  0.198 * RXRAT( 584 )      ! PRD2+CL=0.198*RCHO+...
     &                          +  0.414 * RXRAT( 589 )      ! ROOH+CL=0.414*RCHO+...
      LOSS( RCHO             ) =           RXRAT( 205 )      ! RCHO+OH=0.965*RCO3+...
     &                          +          RXRAT( 206 )      ! RCHO=RO2C+xHO2+yROOH+xCCHO+...
     &                          +          RXRAT( 207 )      ! RCHO+NO3=HNO3+RCO3
     &                          +          RXRAT( 580 )      ! RCHO+CL=HCL+0.9*RCO3+...

      PROD( xHCHO            ) =           RXRAT( 208 )      ! ACETONE+OH=xHCHO+xMECO3+...
     &                          +  0.088 * RXRAT( 210 )      ! MEK+OH=0.088*xHCHO+...
     &                          +  0.012 * RXRAT( 218 )      ! ROOH+OH=0.012*xHCHO+...
     &                          +  0.652 * RXRAT( 241 )      ! AFG1+O3=0.652*xHCHO+...
     &                          +  0.652 * RXRAT( 244 )      ! AFG2+O3=0.652*xHCHO+...
     &                          +  0.163 * RXRAT( 247 )      ! AFG3+O3=0.163*xHCHO+...
     &                          +  0.100 * RXRAT( 248 )      ! MACR+O3=0.1*xHCHO+...
     &                          +  0.330 * RXRAT( 251 )      ! MACR=0.33*xHCHO+0.67*HO2+...
     &                          +  0.050 * RXRAT( 252 )      ! MVK+O3=0.05*xHCHO+...
     &                          +  0.055 * RXRAT( 255 )      ! IPRD+OH=0.055*xHCHO+...
     &                          +  0.001 * RXRAT( 256 )      ! IPRD+O3=0.001*xHCHO+...
     &                          +  0.227 * RXRAT( 257 )      ! IPRD+NO3=0.227*xHCHO+...
     &                          +  0.211 * RXRAT( 259 )      ! PRD2+OH=0.211*xHCHO+...
     &                          +  0.303 * RXRAT( 260 )      ! PRD2=0.303*xHCHO+0.4*MECO3+...
     &                          +  0.011 * RXRAT( 261 )      ! RNO3+OH=0.011*xHCHO+...
     &                          +  0.061 * RXRAT( 262 )      ! RNO3=0.061*xHCHO+...
     &                          +  0.083 * RXRAT( 265 )      ! ACROLEIN+OH=0.083*xHCHO+...
     &                          +  0.020 * RXRAT( 270 )      ! CCOOOH+OH=0.02*xHCHO+...
     &                          +  1.610 * RXRAT( 505 )      ! ETHENE+OH=1.61*xHCHO+RO2C+...
     &                          +  0.278 * RXRAT( 508 )      ! ETHENE+O3P=0.278*xHCHO+...
     &                          +  0.984 * RXRAT( 509 )      ! PROPENE+OH=0.984*xHCHO+...
     &                          +  0.708 * RXRAT( 513 )      ! BUTADIENE13+OH=0.708*xHCHO+...
     &                          +  0.115 * RXRAT( 515 )      ! BUTADIENE13+NO3=...
     &                          +  0.192 * RXRAT( 517 )      ! ISOPRENE+O3=0.192*xHCHO+...
     &                          +  0.240 * RXRAT( 518 )      ! ISOPRENE+O3P=0.24*xHCHO+...
     &                          +  0.022 * RXRAT( 519 )      ! APIN+OH=0.022*xHCHO+...
     &                          +  0.344 * RXRAT( 520 )      ! APIN+O3=0.344*xHCHO+...
     &                          +  0.007 * RXRAT( 521 )      ! APIN+NO3=0.007*xHCHO+...
     &                          +  0.081 * RXRAT( 531 )      ! ETOH+OH=0.081*xHCHO+...
     &                          +  0.026 * RXRAT( 534 )      ! ALK3+OH=0.026*xHCHO+...
     &                          +  0.029 * RXRAT( 535 )      ! ALK4+OH=0.029*xHCHO+...
     &                          +  0.040 * RXRAT( 536 )      ! ALK5+OH=0.04*xHCHO+...
     &                          +  0.582 * RXRAT( 537 )      ! OLE1+OH=0.582*xHCHO+...
     &                          +  0.179 * RXRAT( 541 )      ! OLE2+OH=0.179*xHCHO+...
     &                          +  0.114 * RXRAT( 542 )      ! OLE2+O3=0.114*xHCHO+...
     &                          +  0.072 * RXRAT( 543 )      ! OLE2+NO3=0.072*xHCHO+...
     &                          +  0.411 * RXRAT( 547 )      ! TERP+OH=0.411*xHCHO+...
     &                          +  0.033 * RXRAT( 548 )      ! TERP+O3=0.033*xHCHO+...
     &                          +  0.023 * RXRAT( 549 )      ! TERP+NO3=0.023*xHCHO+...
     &                          +  0.411 * RXRAT( 551 )      ! SESQ+OH=0.411*xHCHO+...
     &                          +  0.033 * RXRAT( 552 )      ! SESQ+O3=0.033*xHCHO+...
     &                          +  0.023 * RXRAT( 553 )      ! SESQ+NO3=0.023*xHCHO+...
     &                          +          RXRAT( 581 )      ! ACETONE+CL=xHCHO+RO2C+HCL+...
     &                          +  0.065 * RXRAT( 582 )      ! MEK+CL=0.065*xHCHO+...
     &                          +  0.045 * RXRAT( 583 )      ! RNO3+CL=0.045*xHCHO+...
     &                          +  0.237 * RXRAT( 584 )      ! PRD2+CL=0.237*xHCHO+...
     &                          +  0.106 * RXRAT( 589 )      ! ROOH+CL=0.106*xHCHO+...
     &                          +  0.274 * RXRAT( 592 )      ! ACROLEIN+CL=0.274*xHCHO+...
     &                          +  0.195 * RXRAT( 595 )      ! IPRD+CL=0.195*xHCHO+...
     &                          +          RXRAT( 596 )      ! CLCCHO=xHCHO+CO+RO2C+xCL+...
     &                          +          RXRAT( 599 )      ! CLACET=xHCHO+RO2C+xCL+...
     &                          +          RXRAT( 631 )      ! ETHENE+CL=xHCHO+2*RO2C+...
     &                          +  0.863 * RXRAT( 633 )      ! BUTADIENE13+CL=0.863*xHCHO+...
     &                          +  0.275 * RXRAT( 634 )      ! ISOPRENE+CL=0.275*xHCHO+...
     &                          +  0.158 * RXRAT( 635 )      ! APIN+CL=0.158*xHCHO+...
     &                          +  0.503 * RXRAT( 642 )      ! ETOH+CL=0.503*xHCHO+...
     &                          +  0.078 * RXRAT( 645 )      ! ALK3+CL=0.078*xHCHO+...
     &                          +  0.034 * RXRAT( 646 )      ! ALK4+CL=0.034*xHCHO+...
     &                          +  0.022 * RXRAT( 647 )      ! ALK5+CL=0.022*xHCHO+...
     &                          +  0.036 * RXRAT( 648 )      ! OLE1+CL=0.036*xHCHO+...
     &                          +  0.190 * RXRAT( 649 )      ! OLE2+CL=0.19*xHCHO+...
     &                          +  0.158 * RXRAT( 652 )      ! TERP+CL=0.158*xHCHO+...
     &                          +  0.158 * RXRAT( 653 )      ! SESQ+CL=0.158*xHCHO+...
      LOSS( xHCHO            ) =           RXRAT( 274 )      ! xHCHO+NO=NO+HCHO
     &                          +          RXRAT( 275 )      ! xHCHO+HO2=HO2
     &                          +          RXRAT( 276 )      ! xHCHO+NO3=NO3+HCHO
     &                          +          RXRAT( 277 )      ! xHCHO+MEO2=MEO2+0.5*HCHO
     &                          +          RXRAT( 278 )      ! xHCHO+RO2C=RO2C+0.5*HCHO
     &                          +          RXRAT( 279 )      ! xHCHO+RO2XC=RO2XC+0.5*HCHO
     &                          +          RXRAT( 280 )      ! xHCHO+MECO3=MECO3+HCHO
     &                          +          RXRAT( 281 )      ! xHCHO+RCO3=RCO3+HCHO
     &                          +          RXRAT( 282 )      ! xHCHO+BZCO3=BZCO3+HCHO
     &                          +          RXRAT( 283 )      ! xHCHO+MACO3=MACO3+HCHO

      PROD( MEK              ) =   0.550 * RXRAT( 253 )      ! MVK+O3P=0.55*MEK+0.45*RCHO
     &                          +  0.210 * RXRAT( 256 )      ! IPRD+O3=0.21*MEK+0.4*HO2+...
     &                          +  0.233 * RXRAT( 258 )      ! IPRD=0.233*MEK+0.467*MECO3+...
     &                          +  0.010 * RXRAT( 261 )      ! RNO3+OH=0.01*MEK+...
     &                          +  0.124 * RXRAT( 262 )      ! RNO3=0.124*MEK+0.554*xHO2+...
     &                          +          RXRAT( 314 )      ! xMEK+NO=MEK+NO
     &                          +          RXRAT( 316 )      ! xMEK+NO3=MEK+NO3
     &                          +  0.500 * RXRAT( 317 )      ! xMEK+MEO2=0.5*MEK+MEO2
     &                          +  0.500 * RXRAT( 318 )      ! xMEK+RO2C=0.5*MEK+RO2C
     &                          +  0.500 * RXRAT( 319 )      ! xMEK+RO2XC=0.5*MEK+RO2XC
     &                          +          RXRAT( 320 )      ! xMEK+MECO3=MEK+MECO3
     &                          +          RXRAT( 321 )      ! xMEK+RCO3=MEK+RCO3
     &                          +          RXRAT( 322 )      ! xMEK+BZCO3=MEK+BZCO3
     &                          +          RXRAT( 323 )      ! xMEK+MACO3=MEK+MACO3
     &                          +  0.500 * RXRAT( 447 )      ! yROOH+MEO2=0.5*MEK+MEO2
     &                          +  0.500 * RXRAT( 448 )      ! yROOH+RO2C=0.5*MEK+RO2C
     &                          +  0.500 * RXRAT( 449 )      ! yROOH+RO2XC=0.5*MEK+RO2XC
     &                          +  0.550 * RXRAT( 512 )      ! PROPENE+O3P=0.55*MEK+0.45*RCHO
     &                          +  0.008 * RXRAT( 520 )      ! APIN+O3=0.008*MEK+...
     &                          +  0.009 * RXRAT( 538 )      ! OLE1+O3=0.009*MEK+...
     &                          +  0.390 * RXRAT( 540 )      ! OLE1+O3P=0.39*MEK+...
     &                          +  0.033 * RXRAT( 542 )      ! OLE2+O3=0.033*MEK+...
     &                          +  0.751 * RXRAT( 544 )      ! OLE2+O3P=0.751*MEK+...
     &                          +  0.002 * RXRAT( 548 )      ! TERP+O3=0.002*MEK+...
     &                          +  0.002 * RXRAT( 552 )      ! SESQ+O3=0.002*MEK+...
     &                          +  0.018 * RXRAT( 583 )      ! RNO3+CL=0.018*MEK+...
     &                          +  0.500 * RXRAT( 743 )      ! MVKOO+MEO2=0.5*MEK+...
     &                          +  0.500 * RXRAT( 744 )      ! MVKOO+RO2C=0.5*MEK+...
      LOSS( MEK              ) =           RXRAT( 210 )      ! MEK+OH=0.967*RO2C+...
     &                          +          RXRAT( 211 )      ! MEK=MECO3+RO2C+xHO2+xCCHO+...
     &                          +          RXRAT( 582 )      ! MEK+CL=HCL+0.975*RO2C+...

      PROD( zRNO3            ) =   0.039 * RXRAT( 210 )      ! MEK+OH=0.039*zRNO3+...
     &                          +  0.004 * RXRAT( 218 )      ! ROOH+OH=0.004*zRNO3+...
     &                          +  0.029 * RXRAT( 220 )      ! R6OOH+OH=0.029*zRNO3+...
     &                          +  0.077 * RXRAT( 221 )      ! R6OOH=0.077*zRNO3+...
     &                          +  0.124 * RXRAT( 222 )      ! RAOOH+OH=0.124*zRNO3+...
     &                          +  0.060 * RXRAT( 240 )      ! AFG1+OH=0.06*zRNO3+...
     &                          +  0.060 * RXRAT( 243 )      ! AFG2+OH=0.06*zRNO3+...
     &                          +  0.117 * RXRAT( 246 )      ! AFG3+OH=0.117*zRNO3+...
     &                          +  0.007 * RXRAT( 247 )      ! AFG3+O3=0.007*zRNO3+...
     &                          +  0.041 * RXRAT( 255 )      ! IPRD+OH=0.041*zRNO3+...
     &                          +  0.051 * RXRAT( 257 )      ! IPRD+NO3=0.051*zRNO3+...
     &                          +  0.071 * RXRAT( 259 )      ! PRD2+OH=0.071*zRNO3+...
     &                          +  0.087 * RXRAT( 260 )      ! PRD2=0.087*zRNO3+0.4*MECO3+...
     &                          +  0.175 * RXRAT( 261 )      ! RNO3+OH=0.175*zRNO3+...
     &                          +  0.102 * RXRAT( 262 )      ! RNO3=0.102*zRNO3+...
     &                          +  0.002 * RXRAT( 267 )      ! ACROLEIN+NO3=0.002*zRNO3+...
     &                          +  0.016 * RXRAT( 509 )      ! PROPENE+OH=0.016*zRNO3+...
     &                          +  0.051 * RXRAT( 511 )      ! PROPENE+NO3=0.051*zRNO3+...
     &                          +  0.049 * RXRAT( 513 )      ! BUTADIENE13+OH=0.049*zRNO3+...
     &                          +  0.065 * RXRAT( 515 )      ! BUTADIENE13+NO3=...
     &                          +  0.015 * RXRAT( 516 )      ! BUTADIENE13+O3P=...
     &                          +  0.008 * RXRAT( 517 )      ! ISOPRENE+O3=0.008*zRNO3+...
     &                          +  0.010 * RXRAT( 518 )      ! ISOPRENE+O3P=0.01*zRNO3+...
     &                          +  0.197 * RXRAT( 519 )      ! APIN+OH=0.197*zRNO3+...
     &                          +  0.337 * RXRAT( 520 )      ! APIN+O3=0.337*zRNO3+...
     &                          +  0.293 * RXRAT( 521 )      ! APIN+NO3=0.293*zRNO3+...
     &                          +  0.024 * RXRAT( 525 )      ! BENZENE+OH=0.024*zRNO3+...
     &                          +  0.054 * RXRAT( 526 )      ! TOLUENE+OH=0.054*zRNO3+...
     &                          +  0.082 * RXRAT( 527 )      ! MXYL+OH=0.082*zRNO3+...
     &                          +  0.087 * RXRAT( 528 )      ! OXYL+OH=0.087*zRNO3+...
     &                          +  0.076 * RXRAT( 529 )      ! PXYL+OH=0.076*zRNO3+...
     &                          +  0.121 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.035 * RXRAT( 533 )      ! ALK2+OH=0.035*zRNO3+...
     &                          +  0.070 * RXRAT( 534 )      ! ALK3+OH=0.07*zRNO3+...
     &                          +  0.149 * RXRAT( 535 )      ! ALK4+OH=0.149*zRNO3+...
     &                          +  0.353 * RXRAT( 536 )      ! ALK5+OH=0.353*zRNO3+...
     &                          +  0.128 * RXRAT( 537 )      ! OLE1+OH=0.128*zRNO3+...
     &                          +  0.005 * RXRAT( 538 )      ! OLE1+O3=0.005*zRNO3+...
     &                          +  0.228 * RXRAT( 539 )      ! OLE1+NO3=0.228*zRNO3+...
     &                          +  0.088 * RXRAT( 541 )      ! OLE2+OH=0.088*zRNO3+...
     &                          +  0.003 * RXRAT( 542 )      ! OLE2+O3=0.003*zRNO3+...
     &                          +  0.140 * RXRAT( 543 )      ! OLE2+NO3=0.14*zRNO3+...
     &                          +  0.110 * RXRAT( 545 )      ! ARO1+OH=0.11*zRNO3+...
     &                          +  0.128 * RXRAT( 546 )      ! ARO2+OH=0.128*zRNO3+...
     &                          +  0.201 * RXRAT( 547 )      ! TERP+OH=0.201*zRNO3+...
     &                          +  0.121 * RXRAT( 548 )      ! TERP+O3=0.121*zRNO3+...
     &                          +  0.460 * RXRAT( 549 )      ! TERP+NO3=0.46*zRNO3+...
     &                          +  0.201 * RXRAT( 551 )      ! SESQ+OH=0.201*zRNO3+...
     &                          +  0.121 * RXRAT( 552 )      ! SESQ+O3=0.121*zRNO3+...
     &                          +  0.460 * RXRAT( 553 )      ! SESQ+NO3=0.46*zRNO3+...
     &                          +  0.039 * RXRAT( 582 )      ! MEK+CL=0.039*zRNO3+...
     &                          +  0.202 * RXRAT( 583 )      ! RNO3+CL=0.202*zRNO3+...
     &                          +  0.116 * RXRAT( 584 )      ! PRD2+CL=0.116*zRNO3+...
     &                          +  0.117 * RXRAT( 590 )      ! R6OOH+CL=0.117*zRNO3+...
     &                          +  0.124 * RXRAT( 591 )      ! RAOOH+CL=0.124*zRNO3+...
     &                          +  0.026 * RXRAT( 592 )      ! ACROLEIN+CL=0.026*zRNO3+...
     &                          +  0.033 * RXRAT( 593 )      ! MACR+CL=0.033*zRNO3+...
     &                          +  0.053 * RXRAT( 594 )      ! MVK+CL=0.053*zRNO3+...
     &                          +  0.051 * RXRAT( 595 )      ! IPRD+CL=0.051*zRNO3+...
     &                          +  0.029 * RXRAT( 632 )      ! PROPENE+CL=0.029*zRNO3+...
     &                          +  0.069 * RXRAT( 633 )      ! BUTADIENE13+CL=0.069*zRNO3+...
     &                          +  0.085 * RXRAT( 634 )      ! ISOPRENE+CL=0.085*zRNO3+...
     &                          +  0.582 * RXRAT( 635 )      ! APIN+CL=0.582*zRNO3+...
     &                          +  0.106 * RXRAT( 637 )      ! TOLUENE+CL=0.106*zRNO3+...
     &                          +  0.136 * RXRAT( 638 )      ! MXYL+CL=0.136*zRNO3+...
     &                          +  0.136 * RXRAT( 639 )      ! OXYL+CL=0.136*zRNO3+...
     &                          +  0.136 * RXRAT( 640 )      ! PXYL+CL=0.136*zRNO3+...
     &                          +  0.162 * RXRAT( 641 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.030 * RXRAT( 644 )      ! ALK2+CL=0.03*zRNO3+...
     &                          +  0.070 * RXRAT( 645 )      ! ALK3+CL=0.07*zRNO3+...
     &                          +  0.165 * RXRAT( 646 )      ! ALK4+CL=0.165*zRNO3+...
     &                          +  0.352 * RXRAT( 647 )      ! ALK5+CL=0.352*zRNO3+...
     &                          +  0.127 * RXRAT( 648 )      ! OLE1+CL=0.127*zRNO3+...
     &                          +  0.106 * RXRAT( 649 )      ! OLE2+CL=0.106*zRNO3+...
     &                          +  0.160 * RXRAT( 650 )      ! ARO1+CL=0.16*zRNO3+...
     &                          +  0.172 * RXRAT( 651 )      ! ARO2+CL=0.172*zRNO3+...
     &                          +  0.582 * RXRAT( 652 )      ! TERP+CL=0.582*zRNO3+...
     &                          +  0.582 * RXRAT( 653 )      ! SESQ+CL=0.582*zRNO3+...
      LOSS( zRNO3            ) =           RXRAT( 474 )      ! zRNO3+NO=NO+RNO3
     &                          +          RXRAT( 475 )      ! zRNO3+HO2=HO2
     &                          +          RXRAT( 476 )      ! zRNO3+NO3=NO3+PRD2+HO2
     &                          +          RXRAT( 477 )      ! zRNO3+MEO2=MEO2+0.5*PRD2+...
     &                          +          RXRAT( 478 )      ! zRNO3+RO2C=RO2C+0.5*PRD2+...
     &                          +          RXRAT( 479 )      ! zRNO3+RO2XC=RO2XC+0.5*PRD2+...
     &                          +          RXRAT( 480 )      ! zRNO3+MECO3=MECO3+PRD2+HO2
     &                          +          RXRAT( 481 )      ! zRNO3+RCO3=RCO3+PRD2+HO2
     &                          +          RXRAT( 482 )      ! zRNO3+BZCO3=BZCO3+PRD2+HO2
     &                          +          RXRAT( 483 )      ! zRNO3+MACO3=MACO3+PRD2+HO2

      PROD( xRCHO            ) =   0.376 * RXRAT( 210 )      ! MEK+OH=0.376*xRCHO+...
     &                          +  0.400 * RXRAT( 215 )      ! RCOOH+OH=0.4*xRCHO+xHO2+...
     &                          +  0.205 * RXRAT( 218 )      ! ROOH+OH=0.205*xRCHO+...
     &                          +  0.075 * RXRAT( 220 )      ! R6OOH+OH=0.075*xRCHO+...
     &                          +  0.058 * RXRAT( 221 )      ! R6OOH=0.058*xRCHO+...
     &                          +  0.448 * RXRAT( 222 )      ! RAOOH+OH=0.448*xRCHO+...
     &                          +  0.407 * RXRAT( 240 )      ! AFG1+OH=0.407*xRCHO+...
     &                          +  0.407 * RXRAT( 243 )      ! AFG2+OH=0.407*xRCHO+...
     &                          +  0.013 * RXRAT( 255 )      ! IPRD+OH=0.013*xRCHO+...
     &                          +  0.218 * RXRAT( 257 )      ! IPRD+NO3=0.218*xRCHO+...
     &                          +  0.402 * RXRAT( 259 )      ! PRD2+OH=0.402*xRCHO+...
     &                          +  0.780 * RXRAT( 260 )      ! PRD2=0.78*xRCHO+0.4*MECO3+...
     &                          +  0.036 * RXRAT( 261 )      ! RNO3+OH=0.036*xRCHO+...
     &                          +  0.063 * RXRAT( 262 )      ! RNO3=0.063*xRCHO+...
     &                          +  0.084 * RXRAT( 272 )      ! RCOOOH+OH=0.084*xRCHO+...
     &                          +          RXRAT( 507 )      ! ETHENE+NO3=xRCHO+RO2C+xHO2+...
     &                          +  0.776 * RXRAT( 519 )      ! APIN+OH=0.776*xRCHO+...
     &                          +  0.240 * RXRAT( 520 )      ! APIN+O3=0.24*xRCHO+...
     &                          +  0.684 * RXRAT( 521 )      ! APIN+NO3=0.684*xRCHO+...
     &                          +  0.261 * RXRAT( 533 )      ! ALK2+OH=0.261*xRCHO+...
     &                          +  0.122 * RXRAT( 534 )      ! ALK3+OH=0.122*xRCHO+...
     &                          +  0.236 * RXRAT( 535 )      ! ALK4+OH=0.236*xRCHO+...
     &                          +  0.209 * RXRAT( 536 )      ! ALK5+OH=0.209*xRCHO+...
     &                          +  0.666 * RXRAT( 537 )      ! OLE1+OH=0.666*xRCHO+...
     &                          +  0.044 * RXRAT( 538 )      ! OLE1+O3=0.044*xRCHO+...
     &                          +  0.003 * RXRAT( 539 )      ! OLE1+NO3=0.003*xRCHO+...
     &                          +  0.510 * RXRAT( 541 )      ! OLE2+OH=0.51*xRCHO+...
     &                          +  0.019 * RXRAT( 542 )      ! OLE2+O3=0.019*xRCHO+...
     &                          +  0.163 * RXRAT( 543 )      ! OLE2+NO3=0.163*xRCHO+...
     &                          +  0.385 * RXRAT( 547 )      ! TERP+OH=0.385*xRCHO+...
     &                          +  0.208 * RXRAT( 548 )      ! TERP+O3=0.208*xRCHO+...
     &                          +  0.403 * RXRAT( 549 )      ! TERP+NO3=0.403*xRCHO+...
     &                          +  0.385 * RXRAT( 551 )      ! SESQ+OH=0.385*xRCHO+...
     &                          +  0.208 * RXRAT( 552 )      ! SESQ+O3=0.208*xRCHO+...
     &                          +  0.403 * RXRAT( 553 )      ! SESQ+NO3=0.403*xRCHO+...
     &                          +  0.840 * RXRAT( 582 )      ! MEK+CL=0.84*xRCHO+...
     &                          +  0.020 * RXRAT( 583 )      ! RNO3+CL=0.02*xRCHO+...
     &                          +  0.591 * RXRAT( 584 )      ! PRD2+CL=0.591*xRCHO+...
     &                          +  0.197 * RXRAT( 589 )      ! ROOH+CL=0.197*xRCHO+...
     &                          +  0.676 * RXRAT( 590 )      ! R6OOH+CL=0.676*xRCHO+...
     &                          +  0.448 * RXRAT( 591 )      ! RAOOH+CL=0.448*xRCHO+...
     &                          +  0.185 * RXRAT( 635 )      ! APIN+CL=0.185*xRCHO+...
     &                          +  0.482 * RXRAT( 644 )      ! ALK2+CL=0.482*xRCHO+...
     &                          +  0.343 * RXRAT( 645 )      ! ALK3+CL=0.343*xRCHO+...
     &                          +  0.412 * RXRAT( 646 )      ! ALK4+CL=0.412*xRCHO+...
     &                          +  0.258 * RXRAT( 647 )      ! ALK5+CL=0.258*xRCHO+...
     &                          +  0.072 * RXRAT( 648 )      ! OLE1+CL=0.072*xRCHO+...
     &                          +  0.317 * RXRAT( 649 )      ! OLE2+CL=0.317*xRCHO+...
     &                          +  0.185 * RXRAT( 652 )      ! TERP+CL=0.185*xRCHO+...
     &                          +  0.185 * RXRAT( 653 )      ! SESQ+CL=0.185*xRCHO+...
      LOSS( xRCHO            ) =           RXRAT( 294 )      ! xRCHO+NO=NO+RCHO
     &                          +          RXRAT( 295 )      ! xRCHO+HO2=HO2
     &                          +          RXRAT( 296 )      ! xRCHO+NO3=NO3+RCHO
     &                          +          RXRAT( 297 )      ! xRCHO+MEO2=MEO2+0.5*RCHO
     &                          +          RXRAT( 298 )      ! xRCHO+RO2C=RO2C+0.5*RCHO
     &                          +          RXRAT( 299 )      ! xRCHO+RO2XC=RO2XC+0.5*RCHO
     &                          +          RXRAT( 300 )      ! xRCHO+MECO3=MECO3+RCHO
     &                          +          RXRAT( 301 )      ! xRCHO+RCO3=RCO3+RCHO
     &                          +          RXRAT( 302 )      ! xRCHO+BZCO3=BZCO3+RCHO
     &                          +          RXRAT( 303 )      ! xRCHO+MACO3=MACO3+RCHO

      PROD( HCOOH            ) =   0.333 * RXRAT( 248 )      ! MACR+O3=0.333*HCOOH+...
     &                          +  0.351 * RXRAT( 252 )      ! MVK+O3=0.351*HCOOH+...
     &                          +  0.100 * RXRAT( 256 )      ! IPRD+O3=0.1*HCOOH+0.4*HO2+...
     &                          +  0.185 * RXRAT( 266 )      ! ACROLEIN+O3=0.185*HCOOH+...
     &                          +  0.370 * RXRAT( 506 )      ! ETHENE+O3=0.37*HCOOH+...
     &                          +  0.185 * RXRAT( 510 )      ! PROPENE+O3=0.185*HCOOH+...
     &                          +  0.185 * RXRAT( 514 )      ! BUTADIENE13+O3=0.185*HCOOH+...
     &                          +  0.204 * RXRAT( 517 )      ! ISOPRENE+O3=0.204*HCOOH+...
     &                          +  0.300 * RXRAT( 523 )      ! ACETYLENE+OH=0.3*HCOOH+...
     &                          +  0.185 * RXRAT( 538 )      ! OLE1+O3=0.185*HCOOH+...
     &                          +  0.024 * RXRAT( 542 )      ! OLE2+O3=0.024*HCOOH+...
     &                          +  0.172 * RXRAT( 548 )      ! TERP+O3=0.172*HCOOH+...
     &                          +  0.172 * RXRAT( 552 )      ! SESQ+O3=0.172*HCOOH+...
     &                          +  0.310 * RXRAT( 701 )      ! ISOPNOOD+NO=0.31*HCOOH+...
     &                          +  0.155 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.155*HCOOH+...
     &                          +  0.155 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.155*HCOOH+...
     &                          +  0.310 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.31*HCOOH+...
     &                          +  0.160 * RXRAT( 713 )      ! ISOPNB+O3=0.16*HCOOH+...
     &                          +  0.160 * RXRAT( 763 )      ! HOCCHO+OH=0.16*HCOOH+...
     &                          +  0.125 * RXRAT( 764 )      ! HACET+OH=0.125*HCOOH+...
     &                          +  0.650 * RXRAT( 772 )      ! MVKN+OH=0.65*HCOOH+...
     &                          +  0.070 * RXRAT( 774 )      ! MACRN+OH=0.07*HCOOH+...
     &                          +  0.074 * RXRAT( 781 )      ! IEPOXOO+HO2=0.074*HCOOH+...
     &                          +  0.074 * RXRAT( 782 )      ! IEPOXOO+NO=0.074*HCOOH+...
     &                          +  0.037 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.037*HCOOH+...
     &                          +  0.037 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.037*HCOOH+...
     &                          +  0.074 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.074*HCOOH+...
      LOSS( HCOOH            ) =           RXRAT( 213 )      ! HCOOH+OH=HO2+CO2

      PROD( xMGLY            ) =   0.491 * RXRAT( 214 )      ! CCOOH+OH=0.491*xMGLY+...
     &                          +  0.252 * RXRAT( 222 )      ! RAOOH+OH=0.252*xMGLY+...
     &                          +  0.250 * RXRAT( 232 )      ! CRES+OH=0.25*xMGLY+...
     &                          +  0.267 * RXRAT( 240 )      ! AFG1+OH=0.267*xMGLY+...
     &                          +  0.267 * RXRAT( 243 )      ! AFG2+OH=0.267*xMGLY+...
     &                          +  0.153 * RXRAT( 246 )      ! AFG3+OH=0.153*xMGLY+...
     &                          +  0.095 * RXRAT( 247 )      ! AFG3+O3=0.095*xMGLY+...
     &                          +  0.174 * RXRAT( 255 )      ! IPRD+OH=0.174*xMGLY+...
     &                          +  0.008 * RXRAT( 257 )      ! IPRD+NO3=0.008*xMGLY+...
     &                          +  0.020 * RXRAT( 519 )      ! APIN+OH=0.02*xMGLY+...
     &                          +  0.002 * RXRAT( 521 )      ! APIN+NO3=0.002*xMGLY+...
     &                          +  0.151 * RXRAT( 526 )      ! TOLUENE+OH=0.151*xMGLY+...
     &                          +  0.380 * RXRAT( 527 )      ! MXYL+OH=0.38*xMGLY+...
     &                          +  0.238 * RXRAT( 528 )      ! OXYL+OH=0.238*xMGLY+...
     &                          +  0.112 * RXRAT( 529 )      ! PXYL+OH=0.112*xMGLY+...
     &                          +  0.405 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.100 * RXRAT( 545 )      ! ARO1+OH=0.1*xMGLY+...
     &                          +  0.312 * RXRAT( 546 )      ! ARO2+OH=0.312*xMGLY+...
     &                          +  0.003 * RXRAT( 547 )      ! TERP+OH=0.003*xMGLY+...
     &                          +  0.003 * RXRAT( 548 )      ! TERP+O3=0.003*xMGLY+...
     &                          +  0.003 * RXRAT( 551 )      ! SESQ+OH=0.003*xMGLY+...
     &                          +  0.003 * RXRAT( 552 )      ! SESQ+O3=0.003*xMGLY+...
     &                          +  0.252 * RXRAT( 591 )      ! RAOOH+CL=0.252*xMGLY+...
     &                          +  0.017 * RXRAT( 595 )      ! IPRD+CL=0.017*xMGLY+...
      LOSS( xMGLY            ) =           RXRAT( 344 )      ! xMGLY+NO=NO+MGLY
     &                          +          RXRAT( 345 )      ! xMGLY+HO2=HO2
     &                          +          RXRAT( 346 )      ! xMGLY+NO3=NO3+MGLY
     &                          +          RXRAT( 347 )      ! xMGLY+MEO2=MEO2+0.5*MGLY
     &                          +          RXRAT( 348 )      ! xMGLY+RO2C=RO2C+0.5*MGLY
     &                          +          RXRAT( 349 )      ! xMGLY+RO2XC=RO2XC+0.5*MGLY
     &                          +          RXRAT( 350 )      ! xMGLY+MECO3=MECO3+MGLY
     &                          +          RXRAT( 351 )      ! xMGLY+RCO3=RCO3+MGLY
     &                          +          RXRAT( 352 )      ! xMGLY+BZCO3=BZCO3+MGLY
     &                          +          RXRAT( 353 )      ! xMGLY+MACO3=MACO3+MGLY

      PROD( xBACL            ) =   0.457 * RXRAT( 215 )      ! RCOOH+OH=0.457*xBACL+xHO2+...
     &                          +  0.019 * RXRAT( 246 )      ! AFG3+OH=0.019*xBACL+...
     &                          +  0.023 * RXRAT( 519 )      ! APIN+OH=0.023*xBACL+...
     &                          +  0.081 * RXRAT( 520 )      ! APIN+O3=0.081*xBACL+...
     &                          +  0.185 * RXRAT( 528 )      ! OXYL+OH=0.185*xBACL+...
     &                          +  0.112 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.134 * RXRAT( 546 )      ! ARO2+OH=0.134*xBACL+...
     &                          +  0.009 * RXRAT( 547 )      ! TERP+OH=0.009*xBACL+...
     &                          +  0.039 * RXRAT( 548 )      ! TERP+O3=0.039*xBACL+...
     &                          +  0.009 * RXRAT( 551 )      ! SESQ+OH=0.009*xBACL+...
     &                          +  0.039 * RXRAT( 552 )      ! SESQ+O3=0.039*xBACL+...
     &                          +  0.003 * RXRAT( 635 )      ! APIN+CL=0.003*xBACL+...
     &                          +  0.003 * RXRAT( 652 )      ! TERP+CL=0.003*xBACL+...
     &                          +  0.003 * RXRAT( 653 )      ! SESQ+CL=0.003*xBACL+...
      LOSS( xBACL            ) =           RXRAT( 354 )      ! xBACL+NO=NO+BACL
     &                          +          RXRAT( 355 )      ! xBACL+HO2=HO2
     &                          +          RXRAT( 356 )      ! xBACL+NO3=NO3+BACL
     &                          +          RXRAT( 357 )      ! xBACL+MEO2=MEO2+0.5*BACL
     &                          +          RXRAT( 358 )      ! xBACL+RO2C=RO2C+0.5*BACL
     &                          +          RXRAT( 359 )      ! xBACL+RO2XC=RO2XC+0.5*BACL
     &                          +          RXRAT( 360 )      ! xBACL+MECO3=MECO3+BACL
     &                          +          RXRAT( 361 )      ! xBACL+RCO3=RCO3+BACL
     &                          +          RXRAT( 362 )      ! xBACL+BZCO3=BZCO3+BACL
     &                          +          RXRAT( 363 )      ! xBACL+MACO3=MACO3+BACL

      PROD( ROOH             ) =           RXRAT( 445 )      ! yROOH+HO2=ROOH+HO2
     &                          +          RXRAT( 742 )      ! MVKOO+HO2=ROOH
     &                          +          RXRAT( 748 )      ! MACROO+HO2=ROOH
      LOSS( ROOH             ) =           RXRAT( 218 )      ! ROOH+OH=0.744*OH+...
     &                          +          RXRAT( 219 )      ! ROOH=RCHO+HO2+OH
     &                          +          RXRAT( 589 )      ! ROOH+CL=HCL+0.414*OH+...

      PROD( xPROD2           ) =   0.034 * RXRAT( 218 )      ! ROOH+OH=0.034*xPROD2+...
     &                          +  0.084 * RXRAT( 220 )      ! R6OOH+OH=0.084*xPROD2+...
     &                          +  0.698 * RXRAT( 221 )      ! R6OOH=0.698*xPROD2+...
     &                          +  0.332 * RXRAT( 255 )      ! IPRD+OH=0.332*xPROD2+...
     &                          +  0.007 * RXRAT( 259 )      ! PRD2+OH=0.007*xPROD2+...
     &                          +  0.031 * RXRAT( 261 )      ! RNO3+OH=0.031*xPROD2+...
     &                          +  0.261 * RXRAT( 262 )      ! RNO3=0.261*xPROD2+...
     &                          +  0.146 * RXRAT( 535 )      ! ALK4+OH=0.146*xPROD2+...
     &                          +  0.407 * RXRAT( 536 )      ! ALK5+OH=0.407*xPROD2+...
     &                          +  0.168 * RXRAT( 537 )      ! OLE1+OH=0.168*xPROD2+...
     &                          +  0.023 * RXRAT( 541 )      ! OLE2+OH=0.023*xPROD2+...
     &                          +  0.309 * RXRAT( 545 )      ! ARO1+OH=0.309*xPROD2+...
     &                          +  0.057 * RXRAT( 546 )      ! ARO2+OH=0.057*xPROD2+...
     &                          +  0.409 * RXRAT( 547 )      ! TERP+OH=0.409*xPROD2+...
     &                          +  0.409 * RXRAT( 551 )      ! SESQ+OH=0.409*xPROD2+...
     &                          +  0.046 * RXRAT( 583 )      ! RNO3+CL=0.046*xPROD2+...
     &                          +  0.040 * RXRAT( 584 )      ! PRD2+CL=0.04*xPROD2+...
     &                          +  0.280 * RXRAT( 590 )      ! R6OOH+CL=0.28*xPROD2+...
     &                          +  0.130 * RXRAT( 646 )      ! ALK4+CL=0.13*xPROD2+...
     &                          +  0.378 * RXRAT( 647 )      ! ALK5+CL=0.378*xPROD2+...
     &                          +  0.840 * RXRAT( 650 )      ! ARO1+CL=0.84*xPROD2+...
     &                          +  0.359 * RXRAT( 651 )      ! ARO2+CL=0.359*xPROD2+...
      LOSS( xPROD2           ) =           RXRAT( 324 )      ! xPROD2+NO=NO+PRD2
     &                          +          RXRAT( 325 )      ! xPROD2+HO2=HO2
     &                          +          RXRAT( 326 )      ! xPROD2+NO3=NO3+PRD2
     &                          +          RXRAT( 327 )      ! xPROD2+MEO2=MEO2+0.5*PRD2
     &                          +          RXRAT( 328 )      ! xPROD2+RO2C=RO2C+0.5*PRD2
     &                          +          RXRAT( 329 )      ! xPROD2+RO2XC=RO2XC+0.5*PRD2
     &                          +          RXRAT( 330 )      ! xPROD2+MECO3=MECO3+PRD2
     &                          +          RXRAT( 331 )      ! xPROD2+RCO3=RCO3+PRD2
     &                          +          RXRAT( 332 )      ! xPROD2+BZCO3=BZCO3+PRD2
     &                          +          RXRAT( 333 )      ! xPROD2+MACO3=MACO3+PRD2

      PROD( R6OOH            ) =           RXRAT( 455 )      ! yR6OOH+HO2=R6OOH+HO2
     &                          +          RXRAT( 695 )      ! HC5OO+HO2=R6OOH
     &                          +          RXRAT( 731 )      ! NIT1OHOO+HO2=R6OOH
     &                          +          RXRAT( 736 )      ! DIBOO+HO2=R6OOH
      LOSS( R6OOH            ) =           RXRAT( 220 )      ! R6OOH+OH=0.84*OH+...
     &                          +          RXRAT( 221 )      ! R6OOH=OH+0.142*HO2+...
     &                          +          RXRAT( 590 )      ! R6OOH+CL=HCL+0.145*OH+...

      PROD( PRD2             ) =   0.840 * RXRAT( 220 )      ! R6OOH+OH=0.84*PRD2+...
     &                          +  0.142 * RXRAT( 221 )      ! R6OOH=0.142*PRD2+0.142*HO2+...
     &                          +  0.074 * RXRAT( 222 )      ! RAOOH+OH=0.074*PRD2+...
     &                          +          RXRAT( 245 )      ! AFG2=PRD2
     &                          +  0.600 * RXRAT( 254 )      ! MVK=0.6*PRD2+0.6*CO+...
     &                          +  0.008 * RXRAT( 261 )      ! RNO3+OH=0.008*PRD2+...
     &                          +  0.190 * RXRAT( 262 )      ! RNO3=0.19*PRD2+0.554*xHO2+...
     &                          +          RXRAT( 324 )      ! xPROD2+NO=PRD2+NO
     &                          +          RXRAT( 326 )      ! xPROD2+NO3=PRD2+NO3
     &                          +  0.500 * RXRAT( 327 )      ! xPROD2+MEO2=0.5*PRD2+MEO2
     &                          +  0.500 * RXRAT( 328 )      ! xPROD2+RO2C=0.5*PRD2+RO2C
     &                          +  0.500 * RXRAT( 329 )      ! xPROD2+RO2XC=0.5*PRD2+RO2XC
     &                          +          RXRAT( 330 )      ! xPROD2+MECO3=PRD2+MECO3
     &                          +          RXRAT( 331 )      ! xPROD2+RCO3=PRD2+RCO3
     &                          +          RXRAT( 332 )      ! xPROD2+BZCO3=PRD2+BZCO3
     &                          +          RXRAT( 333 )      ! xPROD2+MACO3=PRD2+MACO3
     &                          +  0.500 * RXRAT( 457 )      ! yR6OOH+MEO2=0.5*PRD2+MEO2
     &                          +  0.500 * RXRAT( 458 )      ! yR6OOH+RO2C=0.5*PRD2+RO2C
     &                          +  0.500 * RXRAT( 459 )      ! yR6OOH+RO2XC=0.5*PRD2+RO2XC
     &                          +  0.500 * RXRAT( 467 )      ! yRAOOH+MEO2=0.5*PRD2+MEO2
     &                          +  0.500 * RXRAT( 468 )      ! yRAOOH+RO2C=0.5*PRD2+RO2C
     &                          +  0.500 * RXRAT( 469 )      ! yRAOOH+RO2XC=0.5*PRD2+RO2XC
     &                          +          RXRAT( 476 )      ! zRNO3+NO3=PRD2+NO3+HO2
     &                          +  0.500 * RXRAT( 477 )      ! zRNO3+MEO2=0.5*PRD2+MEO2+...
     &                          +  0.500 * RXRAT( 478 )      ! zRNO3+RO2C=0.5*PRD2+RO2C+...
     &                          +  0.500 * RXRAT( 479 )      ! zRNO3+RO2XC=0.5*PRD2+RO2XC+...
     &                          +          RXRAT( 480 )      ! zRNO3+MECO3=PRD2+MECO3+HO2
     &                          +          RXRAT( 481 )      ! zRNO3+RCO3=PRD2+RCO3+HO2
     &                          +          RXRAT( 482 )      ! zRNO3+BZCO3=PRD2+BZCO3+HO2
     &                          +          RXRAT( 483 )      ! zRNO3+MACO3=PRD2+MACO3+HO2
     &                          +  0.125 * RXRAT( 514 )      ! BUTADIENE13+O3=0.125*PRD2+...
     &                          +  0.750 * RXRAT( 516 )      ! BUTADIENE13+O3P=0.75*PRD2+...
     &                          +  0.100 * RXRAT( 517 )      ! ISOPRENE+O3=0.1*PRD2+...
     &                          +  0.750 * RXRAT( 518 )      ! ISOPRENE+O3P=0.75*PRD2+...
     &                          +  0.255 * RXRAT( 520 )      ! APIN+O3=0.255*PRD2+...
     &                          +          RXRAT( 522 )      ! APIN+O3P=PRD2+TRPRXN
     &                          +  0.268 * RXRAT( 538 )      ! OLE1+O3=0.268*PRD2+...
     &                          +  0.160 * RXRAT( 540 )      ! OLE1+O3P=0.16*PRD2+...
     &                          +  0.037 * RXRAT( 542 )      ! OLE2+O3=0.037*PRD2+...
     &                          +  0.170 * RXRAT( 544 )      ! OLE2+O3P=0.17*PRD2+...
     &                          +  0.502 * RXRAT( 548 )      ! TERP+O3=0.502*PRD2+...
     &                          +  0.763 * RXRAT( 550 )      ! TERP+O3P=0.763*PRD2+...
     &                          +  0.502 * RXRAT( 552 )      ! SESQ+O3=0.502*PRD2+...
     &                          +  0.763 * RXRAT( 554 )      ! SESQ+O3P=0.763*PRD2+...
     &                          +  0.012 * RXRAT( 583 )      ! RNO3+CL=0.012*PRD2+...
     &                          +  0.145 * RXRAT( 590 )      ! R6OOH+CL=0.145*PRD2+...
     &                          +  0.074 * RXRAT( 591 )      ! RAOOH+CL=0.074*PRD2+...
     &                          +  0.500 * RXRAT( 678 )      ! ISOPO2+MEO2=0.5*PRD2+...
     &                          +  0.500 * RXRAT( 679 )      ! ISOPO2+RO2C=0.5*PRD2+...
     &                          +          RXRAT( 680 )      ! ISOPO2+ISOPO2=PRD2+...
     &                          +          RXRAT( 684 )      ! HPALD+OH=PRD2+OH
     &                          +  0.090 * RXRAT( 694 )      ! HC5OO+NO=0.09*PRD2+...
     &                          +  0.545 * RXRAT( 696 )      ! HC5OO+MEO2=0.045*PRD2+...
     &                          +  0.545 * RXRAT( 697 )      ! HC5OO+RO2C=0.045*PRD2+...
     &                          +  0.090 * RXRAT( 698 )      ! HC5OO+MECO3=0.09*PRD2+CO2+...
     &                          +  0.340 * RXRAT( 701 )      ! ISOPNOOD+NO=0.34*PRD2+...
     &                          +  0.170 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.17*PRD2+...
     &                          +  0.170 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.17*PRD2+...
     &                          +  0.340 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.34*PRD2+...
     &                          +  0.500 * RXRAT( 737 )      ! DIBOO+MEO2=0.5*PRD2+...
     &                          +  0.500 * RXRAT( 738 )      ! DIBOO+RO2C=0.5*PRD2+...
     &                          +  0.500 * RXRAT( 749 )      ! MACROO+MEO2=0.5*PRD2+...
     &                          +  0.500 * RXRAT( 750 )      ! MACROO+RO2C=0.5*PRD2+...
     &                          +  0.500 * RXRAT( 776 )      ! DHMOB+OH=0.5*PRD2+0.5*HO2+...
     &                          +  0.500 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.5*PRD2+...
     &                          +  0.500 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.5*PRD2+...
     &                          +          RXRAT( 787 )      ! RNO3I+OH=PRD2+HO2+NO2
      LOSS( PRD2             ) =   0.671 * RXRAT( 259 )      ! PRD2+OH=0.329*PRD2+...
     &                          +          RXRAT( 260 )      ! PRD2=0.913*xHO2+0.4*MECO3+...
     &                          +  0.884 * RXRAT( 584 )      ! PRD2+CL=0.116*PRD2+...

      PROD( yR6OOH           ) =   0.858 * RXRAT( 221 )      ! R6OOH=0.858*yR6OOH+...
     &                          +  0.713 * RXRAT( 222 )      ! RAOOH+OH=0.713*yR6OOH+...
     &                          +  0.800 * RXRAT( 232 )      ! CRES+OH=0.8*yR6OOH+...
     &                          +  0.783 * RXRAT( 240 )      ! AFG1+OH=0.783*yR6OOH+...
     &                          +  0.652 * RXRAT( 241 )      ! AFG1+O3=0.652*yR6OOH+...
     &                          +  0.783 * RXRAT( 243 )      ! AFG2+OH=0.783*yR6OOH+...
     &                          +  0.652 * RXRAT( 244 )      ! AFG2+O3=0.652*yR6OOH+...
     &                          +  0.794 * RXRAT( 246 )      ! AFG3+OH=0.794*yR6OOH+...
     &                          +  0.264 * RXRAT( 247 )      ! AFG3+O3=0.264*yR6OOH+...
     &                          +  0.711 * RXRAT( 255 )      ! IPRD+OH=0.711*yR6OOH+...
     &                          +  0.048 * RXRAT( 256 )      ! IPRD+O3=0.048*yR6OOH+...
     &                          +  0.850 * RXRAT( 257 )      ! IPRD+NO3=0.85*yR6OOH+...
     &                          +  0.528 * RXRAT( 259 )      ! PRD2+OH=0.528*yR6OOH+...
     &                          +          RXRAT( 260 )      ! PRD2=yR6OOH+0.4*MECO3+...
     &                          +  0.636 * RXRAT( 261 )      ! RNO3+OH=0.636*yR6OOH+...
     &                          +  0.591 * RXRAT( 262 )      ! RNO3=0.591*yR6OOH+...
     &                          +  0.200 * RXRAT( 517 )      ! ISOPRENE+O3=0.2*yR6OOH+...
     &                          +  0.250 * RXRAT( 518 )      ! ISOPRENE+O3P=0.25*yR6OOH+...
     &                          +          RXRAT( 519 )      ! APIN+OH=yR6OOH+0.004*xRCO3+...
     &                          +  0.737 * RXRAT( 520 )      ! APIN+O3=0.737*yR6OOH+...
     &                          +          RXRAT( 521 )      ! APIN+NO3=yR6OOH+0.643*xNO2+...
     &                          +  0.073 * RXRAT( 526 )      ! TOLUENE+OH=0.073*yR6OOH+...
     &                          +  0.047 * RXRAT( 527 )      ! MXYL+OH=0.047*yR6OOH+...
     &                          +  0.055 * RXRAT( 528 )      ! OXYL+OH=0.055*yR6OOH+...
     &                          +  0.102 * RXRAT( 529 )      ! PXYL+OH=0.102*yR6OOH+...
     &                          +  0.043 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.017 * RXRAT( 534 )      ! ALK3+OH=0.017*yR6OOH+...
     &                          +          RXRAT( 535 )      ! ALK4+OH=yR6OOH+0.01*xMEO2+...
     &                          +          RXRAT( 536 )      ! ALK5+OH=yR6OOH+1.605*RO2C+...
     &                          +  0.831 * RXRAT( 537 )      ! OLE1+OH=0.831*yR6OOH+...
     &                          +  0.052 * RXRAT( 538 )      ! OLE1+O3=0.052*yR6OOH+...
     &                          +  0.831 * RXRAT( 539 )      ! OLE1+NO3=0.831*yR6OOH+...
     &                          +  0.681 * RXRAT( 541 )      ! OLE2+OH=0.681*yR6OOH+...
     &                          +  0.136 * RXRAT( 542 )      ! OLE2+O3=0.136*yR6OOH+...
     &                          +  0.681 * RXRAT( 543 )      ! OLE2+NO3=0.681*yR6OOH+...
     &                          +  0.369 * RXRAT( 545 )      ! ARO1+OH=0.369*yR6OOH+...
     &                          +  0.101 * RXRAT( 546 )      ! ARO2+OH=0.101*yR6OOH+...
     &                          +          RXRAT( 547 )      ! TERP+OH=yR6OOH+0.064*xRCO3+...
     &                          +  0.428 * RXRAT( 548 )      ! TERP+O3=0.428*yR6OOH+...
     &                          +          RXRAT( 549 )      ! TERP+NO3=yR6OOH+0.287*xNO2+...
     &                          +          RXRAT( 551 )      ! SESQ+OH=yR6OOH+0.064*xRCO3+...
     &                          +  0.428 * RXRAT( 552 )      ! SESQ+O3=0.428*yR6OOH+...
     &                          +          RXRAT( 553 )      ! SESQ+NO3=yR6OOH+0.287*xNO2+...
     &                          +  0.908 * RXRAT( 583 )      ! RNO3+CL=0.908*yR6OOH+...
     &                          +  0.686 * RXRAT( 584 )      ! PRD2+CL=0.686*yR6OOH+...
     &                          +          RXRAT( 587 )      ! CRES+CL=yR6OOH+xHO2+xBALD+HCL
     &                          +  0.855 * RXRAT( 590 )      ! R6OOH+CL=0.855*yR6OOH+...
     &                          +  0.713 * RXRAT( 591 )      ! RAOOH+CL=0.713*yR6OOH+...
     &                          +  0.762 * RXRAT( 595 )      ! IPRD+CL=0.762*yR6OOH+...
     &                          +          RXRAT( 634 )      ! ISOPRENE+CL=yR6OOH+...
     &                          +          RXRAT( 635 )      ! APIN+CL=yR6OOH+0.252*xHO2+...
     &                          +  0.017 * RXRAT( 645 )      ! ALK3+CL=0.017*yR6OOH+...
     &                          +          RXRAT( 646 )      ! ALK4+CL=yR6OOH+0.827*xHO2+...
     &                          +          RXRAT( 647 )      ! ALK5+CL=yR6OOH+0.647*xHO2+...
     &                          +  0.831 * RXRAT( 648 )      ! OLE1+CL=0.831*yR6OOH+...
     &                          +  0.681 * RXRAT( 649 )      ! OLE2+CL=0.681*yR6OOH+...
     &                          +          RXRAT( 652 )      ! TERP+CL=yR6OOH+0.252*xHO2+...
     &                          +          RXRAT( 653 )      ! SESQ+CL=yR6OOH+0.068*xCL+...
      LOSS( yR6OOH           ) =           RXRAT( 454 )      ! yR6OOH+NO=NO
     &                          +          RXRAT( 455 )      ! yR6OOH+HO2=HO2+R6OOH
     &                          +          RXRAT( 456 )      ! yR6OOH+NO3=NO3
     &                          +          RXRAT( 457 )      ! yR6OOH+MEO2=MEO2+0.5*PRD2
     &                          +          RXRAT( 458 )      ! yR6OOH+RO2C=RO2C+0.5*PRD2
     &                          +          RXRAT( 459 )      ! yR6OOH+RO2XC=RO2XC+0.5*PRD2
     &                          +          RXRAT( 460 )      ! yR6OOH+MECO3=MECO3
     &                          +          RXRAT( 461 )      ! yR6OOH+RCO3=RCO3
     &                          +          RXRAT( 462 )      ! yR6OOH+BZCO3=BZCO3
     &                          +          RXRAT( 463 )      ! yR6OOH+MACO3=MACO3

      PROD( RAOOH            ) =           RXRAT( 465 )      ! yRAOOH+HO2=RAOOH+HO2
      LOSS( RAOOH            ) =           RXRAT( 222 )      ! RAOOH+OH=0.139*OH+...
     &                          +          RXRAT( 223 )      ! RAOOH=OH+HO2+0.5*GLY+...
     &                          +          RXRAT( 591 )      ! RAOOH+CL=0.404*HCL+...

      PROD( MGLY             ) =   0.147 * RXRAT( 222 )      ! RAOOH+OH=0.147*MGLY+...
     &                          +  0.500 * RXRAT( 223 )      ! RAOOH=0.5*MGLY+HO2+0.5*GLY+...
     &                          +  0.568 * RXRAT( 241 )      ! AFG1+O3=0.568*MGLY+...
     &                          +  0.305 * RXRAT( 242 )      ! AFG1=0.305*MGLY+0.173*MEO2+...
     &                          +  0.568 * RXRAT( 244 )      ! AFG2+O3=0.568*MGLY+...
     &                          +  0.184 * RXRAT( 247 )      ! AFG3+O3=0.184*MGLY+...
     &                          +  0.900 * RXRAT( 248 )      ! MACR+O3=0.9*MGLY+0.108*HO2+...
     &                          +  0.950 * RXRAT( 252 )      ! MVK+O3=0.95*MGLY+0.064*HO2+...
     &                          +  0.742 * RXRAT( 256 )      ! IPRD+O3=0.742*MGLY+0.4*HO2+...
     &                          +          RXRAT( 344 )      ! xMGLY+NO=MGLY+NO
     &                          +          RXRAT( 346 )      ! xMGLY+NO3=MGLY+NO3
     &                          +  0.500 * RXRAT( 347 )      ! xMGLY+MEO2=0.5*MGLY+MEO2
     &                          +  0.500 * RXRAT( 348 )      ! xMGLY+RO2C=0.5*MGLY+RO2C
     &                          +  0.500 * RXRAT( 349 )      ! xMGLY+RO2XC=0.5*MGLY+RO2XC
     &                          +          RXRAT( 350 )      ! xMGLY+MECO3=MGLY+MECO3
     &                          +          RXRAT( 351 )      ! xMGLY+RCO3=MGLY+RCO3
     &                          +          RXRAT( 352 )      ! xMGLY+BZCO3=MGLY+BZCO3
     &                          +          RXRAT( 353 )      ! xMGLY+MACO3=MGLY+MACO3
     &                          +  0.147 * RXRAT( 591 )      ! RAOOH+CL=0.147*MGLY+...
     &                          +  0.500 * RXRAT( 683 )      ! HPALD=0.5*MGLY+HO2+...
     &                          +  0.234 * RXRAT( 694 )      ! HC5OO+NO=0.234*MGLY+...
     &                          +  0.117 * RXRAT( 696 )      ! HC5OO+MEO2=0.117*MGLY+...
     &                          +  0.117 * RXRAT( 697 )      ! HC5OO+RO2C=0.117*MGLY+...
     &                          +  0.234 * RXRAT( 698 )      ! HC5OO+MECO3=0.234*MGLY+CO2+...
     &                          +  0.500 * RXRAT( 699 )      ! HC5+O3=0.5*MGLY+0.35*GLY+...
     &                          +  0.700 * RXRAT( 706 )      ! ISOPND+O3=0.7*MGLY+...
     &                          +  0.700 * RXRAT( 728 )      ! NIT1+O3=0.7*MGLY+0.45*CO+...
     &                          +  0.520 * RXRAT( 735 )      ! DIBOO+NO=0.52*MGLY+HO2+...
     &                          +  0.260 * RXRAT( 737 )      ! DIBOO+MEO2=0.26*MGLY+...
     &                          +  0.260 * RXRAT( 738 )      ! DIBOO+RO2C=0.26*MGLY+...
     &                          +  0.520 * RXRAT( 739 )      ! DIBOO+MECO3=0.52*MGLY+...
     &                          +  0.265 * RXRAT( 741 )      ! MVKOO+NO=0.265*MGLY+...
     &                          +  0.150 * RXRAT( 743 )      ! MVKOO+MEO2=0.15*MGLY+...
     &                          +  0.150 * RXRAT( 744 )      ! MVKOO+RO2C=0.15*MGLY+...
     &                          +  0.300 * RXRAT( 745 )      ! MVKOO+MECO3=0.3*MGLY+CO2+...
     &                          +  0.130 * RXRAT( 747 )      ! MACROO+NO=0.13*MGLY+...
     &                          +  0.076 * RXRAT( 749 )      ! MACROO+MEO2=0.076*MGLY+...
     &                          +  0.076 * RXRAT( 750 )      ! MACROO+RO2C=0.076*MGLY+...
     &                          +  0.150 * RXRAT( 751 )      ! MACROO+MECO3=0.15*MGLY+CO2+...
     &                          +  0.750 * RXRAT( 764 )      ! HACET+OH=0.75*MGLY+...
     &                          +          RXRAT( 768 )      ! PROPNN+OH=MGLY+NO2
     &                          +          RXRAT( 769 )      ! PROPNNB+OH=MGLY+NO2
     &                          +  0.650 * RXRAT( 772 )      ! MVKN+OH=0.65*MGLY+...
     &                          +  0.070 * RXRAT( 774 )      ! MACRN+OH=0.07*MGLY+...
     &                          +  0.275 * RXRAT( 781 )      ! IEPOXOO+HO2=0.275*MGLY+...
     &                          +  0.275 * RXRAT( 782 )      ! IEPOXOO+NO=0.275*MGLY+...
     &                          +  0.138 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.138*MGLY+...
     &                          +  0.138 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.138*MGLY+...
     &                          +  0.275 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.275*MGLY+...
      LOSS( MGLY             ) =           RXRAT( 228 )      ! MGLY=HO2+CO+MECO3
     &                          +          RXRAT( 229 )      ! MGLY+OH=CO+MECO3
     &                          +          RXRAT( 230 )      ! MGLY+NO3=HNO3+CO+MECO3
     &                          +          RXRAT( 586 )      ! MGLY+CL=HCL+CO+MECO3

      PROD( IPRD             ) =   0.139 * RXRAT( 222 )      ! RAOOH+OH=0.139*IPRD+...
     &                          +  0.003 * RXRAT( 247 )      ! AFG3+O3=0.003*IPRD+...
     &                          +          RXRAT( 424 )      ! xIPRD+NO=IPRD+NO
     &                          +          RXRAT( 426 )      ! xIPRD+NO3=IPRD+NO3
     &                          +  0.500 * RXRAT( 427 )      ! xIPRD+MEO2=0.5*IPRD+MEO2
     &                          +  0.500 * RXRAT( 428 )      ! xIPRD+RO2C=0.5*IPRD+RO2C
     &                          +  0.500 * RXRAT( 429 )      ! xIPRD+RO2XC=0.5*IPRD+RO2XC
     &                          +          RXRAT( 430 )      ! xIPRD+MECO3=IPRD+MECO3
     &                          +          RXRAT( 431 )      ! xIPRD+RCO3=IPRD+RCO3
     &                          +          RXRAT( 432 )      ! xIPRD+BZCO3=IPRD+BZCO3
     &                          +          RXRAT( 433 )      ! xIPRD+MACO3=IPRD+MACO3
     &                          +  0.150 * RXRAT( 517 )      ! ISOPRENE+O3=0.15*IPRD+...
     &                          +  0.139 * RXRAT( 591 )      ! RAOOH+CL=0.139*IPRD+...
      LOSS( IPRD             ) =           RXRAT( 255 )      ! IPRD+OH=0.289*MACO3+...
     &                          +          RXRAT( 256 )      ! IPRD+O3=0.285*OH+0.4*HO2+...
     &                          +          RXRAT( 257 )      ! IPRD+NO3=0.15*MACO3+...
     &                          +          RXRAT( 258 )      ! IPRD=1.233*HO2+0.467*MECO3+...
     &                          +          RXRAT( 595 )      ! IPRD+CL=0.401*HCL+...

      PROD( xGLY             ) =   0.026 * RXRAT( 222 )      ! RAOOH+OH=0.026*xGLY+...
     &                          +  0.107 * RXRAT( 240 )      ! AFG1+OH=0.107*xGLY+...
     &                          +  0.107 * RXRAT( 243 )      ! AFG2+OH=0.107*xGLY+...
     &                          +  0.274 * RXRAT( 246 )      ! AFG3+OH=0.274*xGLY+...
     &                          +  0.150 * RXRAT( 255 )      ! IPRD+OH=0.15*xGLY+...
     &                          +  0.083 * RXRAT( 265 )      ! ACROLEIN+OH=0.083*xGLY+...
     &                          +  0.012 * RXRAT( 508 )      ! ETHENE+O3P=0.012*xGLY+...
     &                          +  0.002 * RXRAT( 520 )      ! APIN+O3=0.002*xGLY+...
     &                          +  0.290 * RXRAT( 525 )      ! BENZENE+OH=0.29*xGLY+...
     &                          +  0.238 * RXRAT( 526 )      ! TOLUENE+OH=0.238*xGLY+...
     &                          +  0.100 * RXRAT( 527 )      ! MXYL+OH=0.1*xGLY+0.52*xHO2+...
     &                          +  0.084 * RXRAT( 528 )      ! OXYL+OH=0.084*xGLY+...
     &                          +  0.286 * RXRAT( 529 )      ! PXYL+OH=0.286*xGLY+...
     &                          +  0.074 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.158 * RXRAT( 545 )      ! ARO1+OH=0.158*xGLY+...
     &                          +  0.088 * RXRAT( 546 )      ! ARO2+OH=0.088*xGLY+...
     &                          +  0.026 * RXRAT( 591 )      ! RAOOH+CL=0.026*xGLY+...
     &                          +  0.274 * RXRAT( 592 )      ! ACROLEIN+CL=0.274*xGLY+...
     &                          +  0.007 * RXRAT( 635 )      ! APIN+CL=0.007*xGLY+...
     &                          +  0.007 * RXRAT( 652 )      ! TERP+CL=0.007*xGLY+...
     &                          +  0.007 * RXRAT( 653 )      ! SESQ+CL=0.007*xGLY+...
      LOSS( xGLY             ) =           RXRAT( 334 )      ! xGLY+NO=NO+GLY
     &                          +          RXRAT( 335 )      ! xGLY+HO2=HO2
     &                          +          RXRAT( 336 )      ! xGLY+NO3=NO3+GLY
     &                          +          RXRAT( 337 )      ! xGLY+MEO2=MEO2+0.5*GLY
     &                          +          RXRAT( 338 )      ! xGLY+RO2C=RO2C+0.5*GLY
     &                          +          RXRAT( 339 )      ! xGLY+RO2XC=RO2XC+0.5*GLY
     &                          +          RXRAT( 340 )      ! xGLY+MECO3=MECO3+GLY
     &                          +          RXRAT( 341 )      ! xGLY+RCO3=RCO3+GLY
     &                          +          RXRAT( 342 )      ! xGLY+BZCO3=BZCO3+GLY
     &                          +          RXRAT( 343 )      ! xGLY+MACO3=MACO3+GLY

      PROD( xMEK             ) =   0.030 * RXRAT( 222 )      ! RAOOH+OH=0.03*xMEK+...
     &                          +  0.129 * RXRAT( 240 )      ! AFG1+OH=0.129*xMEK+...
     &                          +  0.129 * RXRAT( 243 )      ! AFG2+OH=0.129*xMEK+...
     &                          +  0.150 * RXRAT( 255 )      ! IPRD+OH=0.15*xMEK+...
     &                          +  0.115 * RXRAT( 259 )      ! PRD2+OH=0.115*xMEK+...
     &                          +  0.170 * RXRAT( 261 )      ! RNO3+OH=0.17*xMEK+...
     &                          +  0.083 * RXRAT( 262 )      ! RNO3=0.083*xMEK+0.554*xHO2+...
     &                          +  0.332 * RXRAT( 534 )      ! ALK3+OH=0.332*xMEK+...
     &                          +  0.106 * RXRAT( 535 )      ! ALK4+OH=0.106*xMEK+...
     &                          +  0.086 * RXRAT( 536 )      ! ALK5+OH=0.086*xMEK+...
     &                          +  0.080 * RXRAT( 541 )      ! OLE2+OH=0.08*xMEK+...
     &                          +  0.001 * RXRAT( 542 )      ! OLE2+O3=0.001*xMEK+...
     &                          +  0.002 * RXRAT( 543 )      ! OLE2+NO3=0.002*xMEK+...
     &                          +  0.007 * RXRAT( 547 )      ! TERP+OH=0.007*xMEK+...
     &                          +  0.007 * RXRAT( 551 )      ! SESQ+OH=0.007*xMEK+...
     &                          +  0.041 * RXRAT( 583 )      ! RNO3+CL=0.041*xMEK+...
     &                          +  0.051 * RXRAT( 584 )      ! PRD2+CL=0.051*xMEK+...
     &                          +  0.285 * RXRAT( 589 )      ! ROOH+CL=0.285*xMEK+...
     &                          +  0.030 * RXRAT( 591 )      ! RAOOH+CL=0.03*xMEK+...
     &                          +  0.253 * RXRAT( 645 )      ! ALK3+CL=0.253*xMEK+...
     &                          +  0.076 * RXRAT( 646 )      ! ALK4+CL=0.076*xMEK+...
     &                          +  0.041 * RXRAT( 647 )      ! ALK5+CL=0.041*xMEK+...
     &                          +  0.042 * RXRAT( 649 )      ! OLE2+CL=0.042*xMEK+...
      LOSS( xMEK             ) =           RXRAT( 314 )      ! xMEK+NO=NO+MEK
     &                          +          RXRAT( 315 )      ! xMEK+HO2=HO2
     &                          +          RXRAT( 316 )      ! xMEK+NO3=NO3+MEK
     &                          +          RXRAT( 317 )      ! xMEK+MEO2=MEO2+0.5*MEK
     &                          +          RXRAT( 318 )      ! xMEK+RO2C=RO2C+0.5*MEK
     &                          +          RXRAT( 319 )      ! xMEK+RO2XC=RO2XC+0.5*MEK
     &                          +          RXRAT( 320 )      ! xMEK+MECO3=MECO3+MEK
     &                          +          RXRAT( 321 )      ! xMEK+RCO3=RCO3+MEK
     &                          +          RXRAT( 322 )      ! xMEK+BZCO3=BZCO3+MEK
     &                          +          RXRAT( 323 )      ! xMEK+MACO3=MACO3+MEK

      PROD( xAFG1            ) =   0.073 * RXRAT( 222 )      ! RAOOH+OH=0.073*xAFG1+...
     &                          +  0.195 * RXRAT( 246 )      ! AFG3+OH=0.195*xAFG1+...
     &                          +  0.001 * RXRAT( 516 )      ! BUTADIENE13+O3P=...
     &                          +  0.029 * RXRAT( 525 )      ! BENZENE+OH=0.029*xAFG1+...
     &                          +  0.195 * RXRAT( 526 )      ! TOLUENE+OH=0.195*xAFG1+...
     &                          +  0.336 * RXRAT( 527 )      ! MXYL+OH=0.336*xAFG1+...
     &                          +  0.253 * RXRAT( 528 )      ! OXYL+OH=0.253*xAFG1+...
     &                          +  0.045 * RXRAT( 529 )      ! PXYL+OH=0.045*xAFG1+...
     &                          +  0.088 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.072 * RXRAT( 545 )      ! ARO1+OH=0.072*xAFG1+...
     &                          +  0.221 * RXRAT( 546 )      ! ARO2+OH=0.221*xAFG1+...
     &                          +  0.073 * RXRAT( 591 )      ! RAOOH+CL=0.073*xAFG1+...
     &                          +  0.009 * RXRAT( 595 )      ! IPRD+CL=0.009*xAFG1+...
     &                          +  0.006 * RXRAT( 635 )      ! APIN+CL=0.006*xAFG1+...
     &                          +  0.006 * RXRAT( 652 )      ! TERP+CL=0.006*xAFG1+...
     &                          +  0.006 * RXRAT( 653 )      ! SESQ+CL=0.006*xAFG1+...
      LOSS( xAFG1            ) =           RXRAT( 374 )      ! xAFG1+NO=NO+AFG1
     &                          +          RXRAT( 375 )      ! xAFG1+HO2=HO2
     &                          +          RXRAT( 376 )      ! xAFG1+NO3=NO3+AFG1
     &                          +          RXRAT( 377 )      ! xAFG1+MEO2=MEO2+0.5*AFG1
     &                          +          RXRAT( 378 )      ! xAFG1+RO2C=RO2C+0.5*AFG1
     &                          +          RXRAT( 379 )      ! xAFG1+RO2XC=RO2XC+0.5*AFG1
     &                          +          RXRAT( 380 )      ! xAFG1+MECO3=MECO3+AFG1
     &                          +          RXRAT( 381 )      ! xAFG1+RCO3=RCO3+AFG1
     &                          +          RXRAT( 382 )      ! xAFG1+BZCO3=BZCO3+AFG1
     &                          +          RXRAT( 383 )      ! xAFG1+MACO3=MACO3+AFG1

      PROD( xAFG2            ) =   0.073 * RXRAT( 222 )      ! RAOOH+OH=0.073*xAFG2+...
     &                          +  0.195 * RXRAT( 246 )      ! AFG3+OH=0.195*xAFG2+...
     &                          +  0.001 * RXRAT( 516 )      ! BUTADIENE13+O3P=...
     &                          +  0.261 * RXRAT( 525 )      ! BENZENE+OH=0.261*xAFG2+...
     &                          +  0.195 * RXRAT( 526 )      ! TOLUENE+OH=0.195*xAFG2+...
     &                          +  0.144 * RXRAT( 527 )      ! MXYL+OH=0.144*xAFG2+...
     &                          +  0.253 * RXRAT( 528 )      ! OXYL+OH=0.253*xAFG2+...
     &                          +  0.067 * RXRAT( 529 )      ! PXYL+OH=0.067*xAFG2+...
     &                          +  0.352 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.185 * RXRAT( 545 )      ! ARO1+OH=0.185*xAFG2+...
     &                          +  0.247 * RXRAT( 546 )      ! ARO2+OH=0.247*xAFG2+...
     &                          +  0.073 * RXRAT( 591 )      ! RAOOH+CL=0.073*xAFG2+...
     &                          +  0.009 * RXRAT( 595 )      ! IPRD+CL=0.009*xAFG2+...
     &                          +  0.006 * RXRAT( 635 )      ! APIN+CL=0.006*xAFG2+...
     &                          +  0.006 * RXRAT( 652 )      ! TERP+CL=0.006*xAFG2+...
     &                          +  0.006 * RXRAT( 653 )      ! SESQ+CL=0.006*xAFG2+...
      LOSS( xAFG2            ) =           RXRAT( 384 )      ! xAFG2+NO=NO+AFG2
     &                          +          RXRAT( 385 )      ! xAFG2+HO2=HO2
     &                          +          RXRAT( 386 )      ! xAFG2+NO3=NO3+AFG2
     &                          +          RXRAT( 387 )      ! xAFG2+MEO2=MEO2+0.5*AFG2
     &                          +          RXRAT( 388 )      ! xAFG2+RO2C=RO2C+0.5*AFG2
     &                          +          RXRAT( 389 )      ! xAFG2+RO2XC=RO2XC+0.5*AFG2
     &                          +          RXRAT( 390 )      ! xAFG2+MECO3=MECO3+AFG2
     &                          +          RXRAT( 391 )      ! xAFG2+RCO3=RCO3+AFG2
     &                          +          RXRAT( 392 )      ! xAFG2+BZCO3=BZCO3+AFG2
     &                          +          RXRAT( 393 )      ! xAFG2+MACO3=MACO3+AFG2

      PROD( GLY              ) =   0.500 * RXRAT( 223 )      ! RAOOH=0.5*GLY+HO2+OH+...
     &                          +  0.432 * RXRAT( 241 )      ! AFG1+O3=0.432*GLY+...
     &                          +  0.195 * RXRAT( 242 )      ! AFG1=0.195*GLY+0.173*MEO2+...
     &                          +  0.432 * RXRAT( 244 )      ! AFG2+O3=0.432*GLY+...
     &                          +  0.366 * RXRAT( 247 )      ! AFG3+O3=0.366*GLY+...
     &                          +  0.023 * RXRAT( 256 )      ! IPRD+O3=0.023*GLY+0.4*HO2+...
     &                          +  0.500 * RXRAT( 266 )      ! ACROLEIN+O3=0.5*GLY+...
     &                          +          RXRAT( 334 )      ! xGLY+NO=GLY+NO
     &                          +          RXRAT( 336 )      ! xGLY+NO3=GLY+NO3
     &                          +  0.500 * RXRAT( 337 )      ! xGLY+MEO2=0.5*GLY+MEO2
     &                          +  0.500 * RXRAT( 338 )      ! xGLY+RO2C=0.5*GLY+RO2C
     &                          +  0.500 * RXRAT( 339 )      ! xGLY+RO2XC=0.5*GLY+RO2XC
     &                          +          RXRAT( 340 )      ! xGLY+MECO3=GLY+MECO3
     &                          +          RXRAT( 341 )      ! xGLY+RCO3=GLY+RCO3
     &                          +          RXRAT( 342 )      ! xGLY+BZCO3=GLY+BZCO3
     &                          +          RXRAT( 343 )      ! xGLY+MACO3=GLY+MACO3
     &                          +  0.700 * RXRAT( 523 )      ! ACETYLENE+OH=0.7*GLY+...
     &                          +  0.250 * RXRAT( 683 )      ! HPALD=0.25*GLY+HO2+...
     &                          +  0.216 * RXRAT( 694 )      ! HC5OO+NO=0.216*GLY+...
     &                          +  0.108 * RXRAT( 696 )      ! HC5OO+MEO2=0.108*GLY+...
     &                          +  0.108 * RXRAT( 697 )      ! HC5OO+RO2C=0.108*GLY+...
     &                          +  0.216 * RXRAT( 698 )      ! HC5OO+MECO3=0.216*GLY+CO2+...
     &                          +  0.350 * RXRAT( 699 )      ! HC5+O3=0.35*GLY+0.5*MGLY+...
     &                          +          RXRAT( 722 )      ! NIT1NO3OOB+NO3=GLY+PROPNNB+...
     &                          +  0.940 * RXRAT( 723 )      ! NIT1NO3OOB+NO=0.94*GLY+...
     &                          +  0.700 * RXRAT( 725 )      ! NIT1NO3OOB+RO2C=0.7*GLY+...
     &                          +  0.700 * RXRAT( 726 )      ! NIT1NO3OOB+MEO2=0.7*GLY+...
     &                          +          RXRAT( 727 )      ! NIT1NO3OOB+MECO3=GLY+CO2+...
     &                          +  0.700 * RXRAT( 728 )      ! NIT1+O3=0.7*GLY+0.45*CO+...
     &                          +  0.919 * RXRAT( 730 )      ! NIT1OHOO+NO=0.919*GLY+...
     &                          +  0.689 * RXRAT( 732 )      ! NIT1OHOO+RO2C=0.689*GLY+...
     &                          +  0.689 * RXRAT( 733 )      ! NIT1OHOO+MEO2=0.689*GLY+...
     &                          +  0.984 * RXRAT( 734 )      ! NIT1OHOO+MECO3=0.984*GLY+...
     &                          +  0.480 * RXRAT( 735 )      ! DIBOO+NO=0.48*GLY+HO2+...
     &                          +  0.240 * RXRAT( 737 )      ! DIBOO+MEO2=0.24*GLY+...
     &                          +  0.240 * RXRAT( 738 )      ! DIBOO+RO2C=0.24*GLY+...
     &                          +  0.480 * RXRAT( 739 )      ! DIBOO+MECO3=0.48*GLY+...
     &                          +  0.130 * RXRAT( 763 )      ! HOCCHO+OH=0.13*GLY+0.25*OH+...
     &                          +  0.275 * RXRAT( 781 )      ! IEPOXOO+HO2=0.275*GLY+...
     &                          +  0.275 * RXRAT( 782 )      ! IEPOXOO+NO=0.275*GLY+...
     &                          +  0.138 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.138*GLY+...
     &                          +  0.138 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.138*GLY+...
     &                          +  0.275 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.275*GLY+...
      LOSS( GLY              ) =           RXRAT( 224 )      ! GLY=2*CO+2*HO2
     &                          +          RXRAT( 225 )      ! GLY=HCHO+CO
     &                          +          RXRAT( 226 )      ! GLY+OH=0.63*HO2+1.26*CO+...
     &                          +          RXRAT( 227 )      ! GLY+NO3=HNO3+0.63*HO2+...
     &                          +          RXRAT( 585 )      ! GLY+CL=HCL+0.63*HO2+...

      PROD( AFG1             ) =   0.500 * RXRAT( 223 )      ! RAOOH=0.5*AFG1+HO2+0.5*GLY+...
     &                          +  0.350 * RXRAT( 247 )      ! AFG3+O3=0.35*AFG1+...
     &                          +          RXRAT( 374 )      ! xAFG1+NO=AFG1+NO
     &                          +          RXRAT( 376 )      ! xAFG1+NO3=AFG1+NO3
     &                          +  0.500 * RXRAT( 377 )      ! xAFG1+MEO2=0.5*AFG1+MEO2
     &                          +  0.500 * RXRAT( 378 )      ! xAFG1+RO2C=0.5*AFG1+RO2C
     &                          +  0.500 * RXRAT( 379 )      ! xAFG1+RO2XC=0.5*AFG1+RO2XC
     &                          +          RXRAT( 380 )      ! xAFG1+MECO3=AFG1+MECO3
     &                          +          RXRAT( 381 )      ! xAFG1+RCO3=AFG1+RCO3
     &                          +          RXRAT( 382 )      ! xAFG1+BZCO3=AFG1+BZCO3
     &                          +          RXRAT( 383 )      ! xAFG1+MACO3=AFG1+MACO3
     &                          +  0.042 * RXRAT( 595 )      ! IPRD+CL=0.042*AFG1+...
      LOSS( AFG1             ) =           RXRAT( 240 )      ! AFG1+OH=0.217*MACO3+...
     &                          +          RXRAT( 241 )      ! AFG1+O3=0.826*OH+0.522*HO2+...
     &                          +          RXRAT( 242 )      ! AFG1=1.023*HO2+0.173*MEO2+...

      PROD( AFG2             ) =   0.500 * RXRAT( 223 )      ! RAOOH=0.5*AFG2+HO2+0.5*GLY+...
     &                          +  0.350 * RXRAT( 247 )      ! AFG3+O3=0.35*AFG2+...
     &                          +          RXRAT( 384 )      ! xAFG2+NO=AFG2+NO
     &                          +          RXRAT( 386 )      ! xAFG2+NO3=AFG2+NO3
     &                          +  0.500 * RXRAT( 387 )      ! xAFG2+MEO2=0.5*AFG2+MEO2
     &                          +  0.500 * RXRAT( 388 )      ! xAFG2+RO2C=0.5*AFG2+RO2C
     &                          +  0.500 * RXRAT( 389 )      ! xAFG2+RO2XC=0.5*AFG2+RO2XC
     &                          +          RXRAT( 390 )      ! xAFG2+MECO3=AFG2+MECO3
     &                          +          RXRAT( 391 )      ! xAFG2+RCO3=AFG2+RCO3
     &                          +          RXRAT( 392 )      ! xAFG2+BZCO3=AFG2+BZCO3
     &                          +          RXRAT( 393 )      ! xAFG2+MACO3=AFG2+MACO3
     &                          +  0.042 * RXRAT( 595 )      ! IPRD+CL=0.042*AFG2+...
      LOSS( AFG2             ) =           RXRAT( 243 )      ! AFG2+OH=0.217*MACO3+...
     &                          +          RXRAT( 244 )      ! AFG2+O3=0.826*OH+0.522*HO2+...
     &                          +          RXRAT( 245 )      ! AFG2=PRD2

      PROD( BACL             ) =           RXRAT( 354 )      ! xBACL+NO=BACL+NO
     &                          +          RXRAT( 356 )      ! xBACL+NO3=BACL+NO3
     &                          +  0.500 * RXRAT( 357 )      ! xBACL+MEO2=0.5*BACL+MEO2
     &                          +  0.500 * RXRAT( 358 )      ! xBACL+RO2C=0.5*BACL+RO2C
     &                          +  0.500 * RXRAT( 359 )      ! xBACL+RO2XC=0.5*BACL+RO2XC
     &                          +          RXRAT( 360 )      ! xBACL+MECO3=BACL+MECO3
     &                          +          RXRAT( 361 )      ! xBACL+RCO3=BACL+RCO3
     &                          +          RXRAT( 362 )      ! xBACL+BZCO3=BACL+BZCO3
     &                          +          RXRAT( 363 )      ! xBACL+MACO3=BACL+MACO3
     &                          +  0.240 * RXRAT( 713 )      ! ISOPNB+O3=0.24*BACL+...
      LOSS( BACL             ) =           RXRAT( 231 )      ! BACL=2*MECO3

      PROD( BALD             ) =           RXRAT( 364 )      ! xBALD+NO=BALD+NO
     &                          +          RXRAT( 366 )      ! xBALD+NO3=BALD+NO3
     &                          +  0.500 * RXRAT( 367 )      ! xBALD+MEO2=0.5*BALD+MEO2
     &                          +  0.500 * RXRAT( 368 )      ! xBALD+RO2C=0.5*BALD+RO2C
     &                          +  0.500 * RXRAT( 369 )      ! xBALD+RO2XC=0.5*BALD+RO2XC
     &                          +          RXRAT( 370 )      ! xBALD+MECO3=BALD+MECO3
     &                          +          RXRAT( 371 )      ! xBALD+RCO3=BALD+RCO3
     &                          +          RXRAT( 372 )      ! xBALD+BZCO3=BALD+BZCO3
     &                          +          RXRAT( 373 )      ! xBALD+MACO3=BALD+MACO3
      LOSS( BALD             ) =           RXRAT( 237 )      ! BALD+OH=BZCO3
     &                          +          RXRAT( 238 )      ! BALD=
     &                          +          RXRAT( 239 )      ! BALD+NO3=HNO3+BZCO3
     &                          +          RXRAT( 588 )      ! BALD+CL=HCL+BZCO3

      PROD( AFG3             ) =           RXRAT( 394 )      ! xAFG3+NO=AFG3+NO
     &                          +          RXRAT( 396 )      ! xAFG3+NO3=AFG3+NO3
     &                          +  0.500 * RXRAT( 397 )      ! xAFG3+MEO2=0.5*AFG3+MEO2
     &                          +  0.500 * RXRAT( 398 )      ! xAFG3+RO2C=0.5*AFG3+RO2C
     &                          +  0.500 * RXRAT( 399 )      ! xAFG3+RO2XC=0.5*AFG3+RO2XC
     &                          +          RXRAT( 400 )      ! xAFG3+MECO3=AFG3+MECO3
     &                          +          RXRAT( 401 )      ! xAFG3+RCO3=AFG3+RCO3
     &                          +          RXRAT( 402 )      ! xAFG3+BZCO3=AFG3+BZCO3
     &                          +          RXRAT( 403 )      ! xAFG3+MACO3=AFG3+MACO3
     &                          +  0.116 * RXRAT( 525 )      ! BENZENE+OH=0.116*AFG3+...
     &                          +  0.312 * RXRAT( 526 )      ! TOLUENE+OH=0.312*AFG3+...
     &                          +  0.239 * RXRAT( 527 )      ! MXYL+OH=0.239*AFG3+...
     &                          +  0.198 * RXRAT( 528 )      ! OXYL+OH=0.198*AFG3+...
     &                          +  0.278 * RXRAT( 529 )      ! PXYL+OH=0.278*AFG3+...
     &                          +  0.230 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.202 * RXRAT( 545 )      ! ARO1+OH=0.202*AFG3+...
     &                          +  0.178 * RXRAT( 546 )      ! ARO2+OH=0.178*AFG3+...
      LOSS( AFG3             ) =           RXRAT( 246 )      ! AFG3+OH=0.206*MACO3+...
     &                          +  0.861 * RXRAT( 247 )      ! AFG3+O3=0.139*AFG3+...

      PROD( xIPRD            ) =   0.231 * RXRAT( 246 )      ! AFG3+OH=0.231*xIPRD+...
     &                          +  0.471 * RXRAT( 513 )      ! BUTADIENE13+OH=0.471*xIPRD+...
     &                          +  0.120 * RXRAT( 515 )      ! BUTADIENE13+NO3=0.12*xIPRD+...
     &                          +  0.009 * RXRAT( 537 )      ! OLE1+OH=0.009*xIPRD+...
     &                          +  0.012 * RXRAT( 541 )      ! OLE2+OH=0.012*xIPRD+...
     &                          +  0.002 * RXRAT( 547 )      ! TERP+OH=0.002*xIPRD+...
     &                          +  0.001 * RXRAT( 548 )      ! TERP+O3=0.001*xIPRD+...
     &                          +  0.004 * RXRAT( 549 )      ! TERP+NO3=0.004*xIPRD+...
     &                          +  0.002 * RXRAT( 551 )      ! SESQ+OH=0.002*xIPRD+...
     &                          +  0.001 * RXRAT( 552 )      ! SESQ+O3=0.001*xIPRD+...
     &                          +  0.004 * RXRAT( 553 )      ! SESQ+NO3=0.004*xIPRD+...
     &                          +  0.082 * RXRAT( 593 )      ! MACR+CL=0.082*xIPRD+...
     &                          +  0.115 * RXRAT( 595 )      ! IPRD+CL=0.115*xIPRD+...
     &                          +  0.473 * RXRAT( 633 )      ! BUTADIENE13+CL=0.473*xIPRD+...
     &                          +  0.671 * RXRAT( 634 )      ! ISOPRENE+CL=0.671*xIPRD+...
     &                          +  0.158 * RXRAT( 635 )      ! APIN+CL=0.158*xIPRD+...
     &                          +  0.038 * RXRAT( 648 )      ! OLE1+CL=0.038*xIPRD+...
     &                          +  0.161 * RXRAT( 649 )      ! OLE2+CL=0.161*xIPRD+...
     &                          +  0.158 * RXRAT( 652 )      ! TERP+CL=0.158*xIPRD+...
     &                          +  0.158 * RXRAT( 653 )      ! SESQ+CL=0.158*xIPRD+...
      LOSS( xIPRD            ) =           RXRAT( 424 )      ! xIPRD+NO=NO+IPRD
     &                          +          RXRAT( 425 )      ! xIPRD+HO2=HO2
     &                          +          RXRAT( 426 )      ! xIPRD+NO3=NO3+IPRD
     &                          +          RXRAT( 427 )      ! xIPRD+MEO2=MEO2+0.5*IPRD
     &                          +          RXRAT( 428 )      ! xIPRD+RO2C=RO2C+0.5*IPRD
     &                          +          RXRAT( 429 )      ! xIPRD+RO2XC=RO2XC+0.5*IPRD
     &                          +          RXRAT( 430 )      ! xIPRD+MECO3=MECO3+IPRD
     &                          +          RXRAT( 431 )      ! xIPRD+RCO3=RCO3+IPRD
     &                          +          RXRAT( 432 )      ! xIPRD+BZCO3=BZCO3+IPRD
     &                          +          RXRAT( 433 )      ! xIPRD+MACO3=MACO3+IPRD

      PROD( MACR             ) =   0.003 * RXRAT( 247 )      ! AFG3+O3=0.003*MACR+...
     &                          +          RXRAT( 404 )      ! xMACR+NO=MACR+NO
     &                          +          RXRAT( 406 )      ! xMACR+NO3=MACR+NO3
     &                          +  0.500 * RXRAT( 407 )      ! xMACR+MEO2=0.5*MACR+MEO2
     &                          +  0.500 * RXRAT( 408 )      ! xMACR+RO2C=0.5*MACR+RO2C
     &                          +  0.500 * RXRAT( 409 )      ! xMACR+RO2XC=0.5*MACR+RO2XC
     &                          +          RXRAT( 410 )      ! xMACR+MECO3=MACR+MECO3
     &                          +          RXRAT( 411 )      ! xMACR+RCO3=MACR+RCO3
     &                          +          RXRAT( 412 )      ! xMACR+BZCO3=MACR+BZCO3
     &                          +          RXRAT( 413 )      ! xMACR+MACO3=MACR+MACO3
     &                          +  0.390 * RXRAT( 517 )      ! ISOPRENE+O3=0.39*MACR+...
     &                          +  0.260 * RXRAT( 676 )      ! ISOPO2+NO=0.26*MACR+...
     &                          +  0.047 * RXRAT( 677 )      ! ISOPO2+HO2=0.047*MACR+...
     &                          +  0.150 * RXRAT( 678 )      ! ISOPO2+MEO2=0.15*MACR+...
     &                          +  0.150 * RXRAT( 679 )      ! ISOPO2+RO2C=0.15*MACR+...
     &                          +  0.290 * RXRAT( 680 )      ! ISOPO2+ISOPO2=0.29*MACR+...
     &                          +  0.290 * RXRAT( 681 )      ! ISOPO2+MECO3=0.29*MACR+CO2+...
     &                          +  0.035 * RXRAT( 686 )      ! NISOPO2+NO3=0.035*MACR+...
     &                          +  0.035 * RXRAT( 687 )      ! NISOPO2+NO=0.035*MACR+...
     &                          +  0.018 * RXRAT( 689 )      ! NISOPO2+MEO2=0.018*MACR+...
     &                          +  0.018 * RXRAT( 690 )      ! NISOPO2+RO2C=0.018*MACR+...
     &                          +  0.035 * RXRAT( 691 )      ! NISOPO2+NISOPO2=0.035*MACR+...
     &                          +  0.035 * RXRAT( 692 )      ! NISOPO2+MECO3=0.035*MACR+...
     &                          +  0.290 * RXRAT( 786 )      ! ISOPOOH=0.29*MACR+0.91*HO2+...
      LOSS( MACR             ) =           RXRAT( 248 )      ! MACR+O3=0.208*OH+0.108*HO2+...
     &                          +          RXRAT( 249 )      ! MACR+NO3=0.5*MACO3+...
     &                          +          RXRAT( 250 )      ! MACR+O3P=RCHO
     &                          +          RXRAT( 251 )      ! MACR=0.33*OH+0.67*HO2+...
     &                          +          RXRAT( 593 )      ! MACR+CL=0.25*HCL+...
     &                          +          RXRAT( 746 )      ! MACR+OH=0.47*MACROO+0.53*MACO3

      PROD( MVK              ) =   0.004 * RXRAT( 247 )      ! AFG3+O3=0.004*MVK+...
     &                          +          RXRAT( 414 )      ! xMVK+NO=MVK+NO
     &                          +          RXRAT( 416 )      ! xMVK+NO3=MVK+NO3
     &                          +  0.500 * RXRAT( 417 )      ! xMVK+MEO2=0.5*MVK+MEO2
     &                          +  0.500 * RXRAT( 418 )      ! xMVK+RO2C=0.5*MVK+RO2C
     &                          +  0.500 * RXRAT( 419 )      ! xMVK+RO2XC=0.5*MVK+RO2XC
     &                          +          RXRAT( 420 )      ! xMVK+MECO3=MVK+MECO3
     &                          +          RXRAT( 421 )      ! xMVK+RCO3=MVK+RCO3
     &                          +          RXRAT( 422 )      ! xMVK+BZCO3=MVK+BZCO3
     &                          +          RXRAT( 423 )      ! xMVK+MACO3=MVK+MACO3
     &                          +  0.375 * RXRAT( 514 )      ! BUTADIENE13+O3=0.375*MVK+...
     &                          +  0.160 * RXRAT( 517 )      ! ISOPRENE+O3=0.16*MVK+...
     &                          +  0.400 * RXRAT( 676 )      ! ISOPO2+NO=0.4*MVK+...
     &                          +  0.073 * RXRAT( 677 )      ! ISOPO2+HO2=0.073*MVK+...
     &                          +  0.230 * RXRAT( 678 )      ! ISOPO2+MEO2=0.23*MVK+...
     &                          +  0.230 * RXRAT( 679 )      ! ISOPO2+RO2C=0.23*MVK+...
     &                          +  0.450 * RXRAT( 680 )      ! ISOPO2+ISOPO2=0.45*MVK+...
     &                          +  0.450 * RXRAT( 681 )      ! ISOPO2+MECO3=0.45*MVK+CO2+...
     &                          +  0.035 * RXRAT( 686 )      ! NISOPO2+NO3=0.035*MVK+...
     &                          +  0.035 * RXRAT( 687 )      ! NISOPO2+NO=0.035*MVK+...
     &                          +  0.018 * RXRAT( 689 )      ! NISOPO2+MEO2=0.018*MVK+...
     &                          +  0.018 * RXRAT( 690 )      ! NISOPO2+RO2C=0.018*MVK+...
     &                          +  0.035 * RXRAT( 691 )      ! NISOPO2+NISOPO2=0.035*MVK+...
     &                          +  0.035 * RXRAT( 692 )      ! NISOPO2+MECO3=0.035*MVK+...
     &                          +  0.450 * RXRAT( 786 )      ! ISOPOOH=0.45*MVK+0.91*HO2+...
      LOSS( MVK              ) =           RXRAT( 252 )      ! MVK+O3=0.164*OH+0.064*HO2+...
     &                          +          RXRAT( 253 )      ! MVK+O3P=0.45*RCHO+0.55*MEK
     &                          +          RXRAT( 254 )      ! MVK=0.4*MEO2+0.6*CO+...
     &                          +          RXRAT( 594 )      ! MVK+CL=1.283*RO2C+...
     &                          +          RXRAT( 740 )      ! MVK+OH=MVKOO

      PROD( xHOCCHO          ) =   0.129 * RXRAT( 255 )      ! IPRD+OH=0.129*xHOCCHO+...
     &                          +  0.047 * RXRAT( 256 )      ! IPRD+O3=0.047*xHOCCHO+...
     &                          +  0.195 * RXRAT( 505 )      ! ETHENE+OH=0.195*xHOCCHO+...
     &                          +  0.010 * RXRAT( 531 )      ! ETOH+OH=0.01*xHOCCHO+...
     &                          +  0.007 * RXRAT( 537 )      ! OLE1+OH=0.007*xHOCCHO+...
     &                          +  0.002 * RXRAT( 549 )      ! TERP+NO3=0.002*xHOCCHO+...
     &                          +  0.061 * RXRAT( 642 )      ! ETOH+CL=0.061*xHOCCHO+...
      LOSS( xHOCCHO          ) =           RXRAT( 484 )      ! xHOCCHO+NO=NO+HOCCHO
     &                          +          RXRAT( 485 )      ! xHOCCHO+HO2=HO2
     &                          +          RXRAT( 486 )      ! xHOCCHO+NO3=NO3+HOCCHO
     &                          +          RXRAT( 487 )      ! xHOCCHO+MEO2=MEO2+0.5*HOCCHO
     &                          +          RXRAT( 488 )      ! xHOCCHO+RO2C=RO2C+0.5*HOCCHO
     &                          +          RXRAT( 489 )      ! xHOCCHO+RO2XC=RO2XC+0.5*HOCCHO
     &                          +          RXRAT( 490 )      ! xHOCCHO+MECO3=MECO3+HOCCHO
     &                          +          RXRAT( 491 )      ! xHOCCHO+RCO3=RCO3+HOCCHO
     &                          +          RXRAT( 492 )      ! xHOCCHO+BZCO3=BZCO3+HOCCHO
     &                          +          RXRAT( 493 )      ! xHOCCHO+MACO3=MACO3+HOCCHO

      PROD( xRNO3            ) =   0.572 * RXRAT( 257 )      ! IPRD+NO3=0.572*xRNO3+...
     &                          +  0.305 * RXRAT( 261 )      ! RNO3+OH=0.305*xRNO3+...
     &                          +  0.031 * RXRAT( 267 )      ! ACROLEIN+NO3=0.031*xRNO3+...
     &                          +  0.355 * RXRAT( 515 )      ! BUTADIENE13+NO3=...
     &                          +  0.056 * RXRAT( 521 )      ! APIN+NO3=0.056*xRNO3+...
     &                          +  0.774 * RXRAT( 539 )      ! OLE1+NO3=0.774*xRNO3+...
     &                          +  0.320 * RXRAT( 543 )      ! OLE2+NO3=0.32*xRNO3+...
     &                          +  0.228 * RXRAT( 549 )      ! TERP+NO3=0.228*xRNO3+...
     &                          +  0.228 * RXRAT( 553 )      ! SESQ+NO3=0.228*xRNO3+...
     &                          +  0.547 * RXRAT( 583 )      ! RNO3+CL=0.547*xRNO3+...
      LOSS( xRNO3            ) =           RXRAT( 434 )      ! xRNO3+NO=NO+RNO3
     &                          +          RXRAT( 435 )      ! xRNO3+HO2=HO2
     &                          +          RXRAT( 436 )      ! xRNO3+NO3=NO3+RNO3
     &                          +          RXRAT( 437 )      ! xRNO3+MEO2=MEO2+0.5*RNO3
     &                          +          RXRAT( 438 )      ! xRNO3+RO2C=RO2C+0.5*RNO3
     &                          +          RXRAT( 439 )      ! xRNO3+RO2XC=RO2XC+0.5*RNO3
     &                          +          RXRAT( 440 )      ! xRNO3+MECO3=MECO3+RNO3
     &                          +          RXRAT( 441 )      ! xRNO3+RCO3=RCO3+RNO3
     &                          +          RXRAT( 442 )      ! xRNO3+BZCO3=BZCO3+RNO3
     &                          +          RXRAT( 443 )      ! xRNO3+MACO3=MACO3+RNO3

      PROD( HOCCHO           ) =   0.467 * RXRAT( 258 )      ! IPRD=0.467*HOCCHO+...
     &                          +          RXRAT( 484 )      ! xHOCCHO+NO=HOCCHO+NO
     &                          +          RXRAT( 486 )      ! xHOCCHO+NO3=HOCCHO+NO3
     &                          +  0.500 * RXRAT( 487 )      ! xHOCCHO+MEO2=0.5*HOCCHO+MEO2
     &                          +  0.500 * RXRAT( 488 )      ! xHOCCHO+RO2C=0.5*HOCCHO+RO2C
     &                          +  0.500 * RXRAT( 489 )      ! xHOCCHO+RO2XC=0.5*HOCCHO+RO2XC
     &                          +          RXRAT( 490 )      ! xHOCCHO+MECO3=HOCCHO+MECO3
     &                          +          RXRAT( 491 )      ! xHOCCHO+RCO3=HOCCHO+RCO3
     &                          +          RXRAT( 492 )      ! xHOCCHO+BZCO3=HOCCHO+BZCO3
     &                          +          RXRAT( 493 )      ! xHOCCHO+MACO3=HOCCHO+MACO3
     &                          +  0.250 * RXRAT( 683 )      ! HPALD=0.25*HOCCHO+HO2+...
     &                          +  0.234 * RXRAT( 694 )      ! HC5OO+NO=0.234*HOCCHO+NO2+...
     &                          +  0.117 * RXRAT( 696 )      ! HC5OO+MEO2=0.117*HOCCHO+...
     &                          +  0.117 * RXRAT( 697 )      ! HC5OO+RO2C=0.117*HOCCHO+...
     &                          +  0.234 * RXRAT( 698 )      ! HC5OO+MECO3=0.234*HOCCHO+...
     &                          +  0.350 * RXRAT( 699 )      ! HC5+O3=0.35*HOCCHO+...
     &                          +  0.150 * RXRAT( 701 )      ! ISOPNOOD+NO=0.15*HOCCHO+...
     &                          +  0.075 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.075*HOCCHO+...
     &                          +  0.075 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.075*HOCCHO+...
     &                          +  0.150 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.15*HOCCHO+...
     &                          +  0.340 * RXRAT( 706 )      ! ISOPND+O3=0.34*HOCCHO+...
     &                          +  0.600 * RXRAT( 708 )      ! ISOPNOOB+NO=0.6*HOCCHO+...
     &                          +  0.300 * RXRAT( 710 )      ! ISOPNOOB+MEO2=0.3*HOCCHO+...
     &                          +  0.300 * RXRAT( 711 )      ! ISOPNOOB+RO2C=0.3*HOCCHO+...
     &                          +  0.600 * RXRAT( 712 )      ! ISOPNOOB+MECO3=0.6*HOCCHO+...
     &                          +  0.520 * RXRAT( 735 )      ! DIBOO+NO=0.52*HOCCHO+HO2+...
     &                          +  0.260 * RXRAT( 737 )      ! DIBOO+MEO2=0.26*HOCCHO+...
     &                          +  0.260 * RXRAT( 738 )      ! DIBOO+RO2C=0.26*HOCCHO+...
     &                          +  0.520 * RXRAT( 739 )      ! DIBOO+MECO3=0.52*HOCCHO+...
     &                          +  0.625 * RXRAT( 741 )      ! MVKOO+NO=0.625*HOCCHO+...
     &                          +  0.350 * RXRAT( 743 )      ! MVKOO+MEO2=0.35*HOCCHO+...
     &                          +  0.350 * RXRAT( 744 )      ! MVKOO+RO2C=0.35*HOCCHO+...
     &                          +  0.700 * RXRAT( 745 )      ! MVKOO+MECO3=0.7*HOCCHO+CO2+...
     &                          +          RXRAT( 773 )      ! MVKN=HOCCHO+NO2+MECO3
     &                          +  0.275 * RXRAT( 781 )      ! IEPOXOO+HO2=0.275*HOCCHO+...
     &                          +  0.275 * RXRAT( 782 )      ! IEPOXOO+NO=0.275*HOCCHO+...
     &                          +  0.138 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.138*HOCCHO+...
     &                          +  0.138 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.138*HOCCHO+...
     &                          +  0.275 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.275*HOCCHO+...
      LOSS( HOCCHO           ) =           RXRAT( 263 )      ! HOCCHO=CO+2*HO2+HCHO
     &                          +          RXRAT( 264 )      ! HOCCHO+NO3=HNO3+MECO3
     &                          +          RXRAT( 763 )      ! HOCCHO+OH=0.75*HO2+0.25*OH+...

      PROD( xACETONE         ) =   0.004 * RXRAT( 261 )      ! RNO3+OH=0.004*xACETONE+...
     &                          +  0.008 * RXRAT( 262 )      ! RNO3=0.008*xACETONE+...
     &                          +  0.034 * RXRAT( 519 )      ! APIN+OH=0.034*xACETONE+...
     &                          +  0.345 * RXRAT( 520 )      ! APIN+O3=0.345*xACETONE+...
     &                          +  0.069 * RXRAT( 521 )      ! APIN+NO3=0.069*xACETONE+...
     &                          +  0.704 * RXRAT( 533 )      ! ALK2+OH=0.704*xACETONE+...
     &                          +  0.024 * RXRAT( 534 )      ! ALK3+OH=0.024*xACETONE+...
     &                          +  0.426 * RXRAT( 535 )      ! ALK4+OH=0.426*xACETONE+...
     &                          +  0.071 * RXRAT( 536 )      ! ALK5+OH=0.071*xACETONE+...
     &                          +  0.007 * RXRAT( 537 )      ! OLE1+OH=0.007*xACETONE+...
     &                          +  0.003 * RXRAT( 538 )      ! OLE1+O3=0.003*xACETONE+...
     &                          +  0.034 * RXRAT( 539 )      ! OLE1+NO3=0.034*xACETONE+...
     &                          +  0.144 * RXRAT( 541 )      ! OLE2+OH=0.144*xACETONE+...
     &                          +  0.116 * RXRAT( 543 )      ! OLE2+NO3=0.116*xACETONE+...
     &                          +  0.037 * RXRAT( 547 )      ! TERP+OH=0.037*xACETONE+...
     &                          +  0.057 * RXRAT( 548 )      ! TERP+O3=0.057*xACETONE+...
     &                          +  0.239 * RXRAT( 549 )      ! TERP+NO3=0.239*xACETONE+...
     &                          +  0.037 * RXRAT( 551 )      ! SESQ+OH=0.037*xACETONE+...
     &                          +  0.057 * RXRAT( 552 )      ! SESQ+O3=0.057*xACETONE+...
     &                          +  0.239 * RXRAT( 553 )      ! SESQ+NO3=0.239*xACETONE+...
     &                          +  0.003 * RXRAT( 583 )      ! RNO3+CL=0.003*xACETONE+...
     &                          +  0.274 * RXRAT( 635 )      ! APIN+CL=0.274*xACETONE+...
     &                          +  0.488 * RXRAT( 644 )      ! ALK2+CL=0.488*xACETONE+...
     &                          +  0.075 * RXRAT( 645 )      ! ALK3+CL=0.075*xACETONE+...
     &                          +  0.247 * RXRAT( 646 )      ! ALK4+CL=0.247*xACETONE+...
     &                          +  0.044 * RXRAT( 647 )      ! ALK5+CL=0.044*xACETONE+...
     &                          +  0.086 * RXRAT( 649 )      ! OLE2+CL=0.086*xACETONE+...
     &                          +  0.274 * RXRAT( 652 )      ! TERP+CL=0.274*xACETONE+...
     &                          +  0.274 * RXRAT( 653 )      ! SESQ+CL=0.274*xACETONE+...
      LOSS( xACETONE         ) =           RXRAT( 304 )      ! xACETONE+NO=NO+ACETONE
     &                          +          RXRAT( 305 )      ! xACETONE+HO2=HO2
     &                          +          RXRAT( 306 )      ! xACETONE+NO3=NO3+ACETONE
     &                          +          RXRAT( 307 )      ! xACETONE+MEO2=MEO2+0.5*ACETONE
     &                          +          RXRAT( 308 )      ! xACETONE+RO2C=RO2C+0.5*ACETONE
     &                          +          RXRAT( 309 )      ! xACETONE+RO2XC=RO2XC+...
     &                          +          RXRAT( 310 )      ! xACETONE+MECO3=MECO3+ACETONE
     &                          +          RXRAT( 311 )      ! xACETONE+RCO3=RCO3+ACETONE
     &                          +          RXRAT( 312 )      ! xACETONE+BZCO3=BZCO3+ACETONE
     &                          +          RXRAT( 313 )      ! xACETONE+MACO3=MACO3+ACETONE

      PROD( ACROLEIN         ) =           RXRAT( 494 )      ! xACROLEIN+NO=ACROLEIN+NO
     &                          +          RXRAT( 496 )      ! xACROLEIN+NO3=ACROLEIN+NO3
     &                          +  0.500 * RXRAT( 497 )      ! xACROLEIN+MEO2=...
     &                          +  0.500 * RXRAT( 498 )      ! xACROLEIN+RO2C=...
     &                          +  0.500 * RXRAT( 499 )      ! xACROLEIN+RO2XC=...
     &                          +          RXRAT( 500 )      ! xACROLEIN+MECO3=ACROLEIN+MECO3
     &                          +          RXRAT( 501 )      ! xACROLEIN+RCO3=ACROLEIN+RCO3
     &                          +          RXRAT( 502 )      ! xACROLEIN+BZCO3=ACROLEIN+BZCO3
     &                          +          RXRAT( 503 )      ! xACROLEIN+MACO3=ACROLEIN+MACO3
     &                          +  0.500 * RXRAT( 514 )      ! BUTADIENE13+O3=...
      LOSS( ACROLEIN         ) =           RXRAT( 265 )      ! ACROLEIN+OH=0.25*xHO2+...
     &                          +          RXRAT( 266 )      ! ACROLEIN+O3=0.83*HO2+...
     &                          +          RXRAT( 267 )      ! ACROLEIN+NO3=0.031*xHO2+...
     &                          +          RXRAT( 268 )      ! ACROLEIN+O3P=RCHO
     &                          +          RXRAT( 269 )      ! ACROLEIN=1.066*HO2+...
     &                          +          RXRAT( 592 )      ! ACROLEIN+CL=0.484*xHO2+...

      PROD( xBALD            ) =   0.065 * RXRAT( 526 )      ! TOLUENE+OH=0.065*xBALD+...
     &                          +  0.041 * RXRAT( 527 )      ! MXYL+OH=0.041*xBALD+...
     &                          +  0.047 * RXRAT( 528 )      ! OXYL+OH=0.047*xBALD+...
     &                          +  0.088 * RXRAT( 529 )      ! PXYL+OH=0.088*xBALD+...
     &                          +  0.036 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.026 * RXRAT( 546 )      ! ARO2+OH=0.026*xBALD+...
     &                          +          RXRAT( 587 )      ! CRES+CL=xBALD+xHO2+HCL+yR6OOH
     &                          +  0.894 * RXRAT( 637 )      ! TOLUENE+CL=0.894*xBALD+...
     &                          +  0.864 * RXRAT( 638 )      ! MXYL+CL=0.864*xBALD+...
     &                          +  0.864 * RXRAT( 639 )      ! OXYL+CL=0.864*xBALD+...
     &                          +  0.864 * RXRAT( 640 )      ! PXYL+CL=0.864*xBALD+...
     &                          +  0.838 * RXRAT( 641 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.469 * RXRAT( 651 )      ! ARO2+CL=0.469*xBALD+...
      LOSS( xBALD            ) =           RXRAT( 364 )      ! xBALD+NO=NO+BALD
     &                          +          RXRAT( 365 )      ! xBALD+HO2=HO2
     &                          +          RXRAT( 366 )      ! xBALD+NO3=NO3+BALD
     &                          +          RXRAT( 367 )      ! xBALD+MEO2=MEO2+0.5*BALD
     &                          +          RXRAT( 368 )      ! xBALD+RO2C=RO2C+0.5*BALD
     &                          +          RXRAT( 369 )      ! xBALD+RO2XC=RO2XC+0.5*BALD
     &                          +          RXRAT( 370 )      ! xBALD+MECO3=MECO3+BALD
     &                          +          RXRAT( 371 )      ! xBALD+RCO3=RCO3+BALD
     &                          +          RXRAT( 372 )      ! xBALD+BZCO3=BZCO3+BALD
     &                          +          RXRAT( 373 )      ! xBALD+MACO3=MACO3+BALD

      PROD( xAFG3            ) =   0.286 * RXRAT( 529 )      ! PXYL+OH=0.286*xAFG3+...
     &                          +  0.151 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.068 * RXRAT( 546 )      ! ARO2+OH=0.068*xAFG3+...
     &                          +  0.001 * RXRAT( 635 )      ! APIN+CL=0.001*xAFG3+...
     &                          +  0.001 * RXRAT( 652 )      ! TERP+CL=0.001*xAFG3+...
     &                          +  0.001 * RXRAT( 653 )      ! SESQ+CL=0.001*xAFG3+...
      LOSS( xAFG3            ) =           RXRAT( 394 )      ! xAFG3+NO=NO+AFG3
     &                          +          RXRAT( 395 )      ! xAFG3+HO2=HO2
     &                          +          RXRAT( 396 )      ! xAFG3+NO3=NO3+AFG3
     &                          +          RXRAT( 397 )      ! xAFG3+MEO2=MEO2+0.5*AFG3
     &                          +          RXRAT( 398 )      ! xAFG3+RO2C=RO2C+0.5*AFG3
     &                          +          RXRAT( 399 )      ! xAFG3+RO2XC=RO2XC+0.5*AFG3
     &                          +          RXRAT( 400 )      ! xAFG3+MECO3=MECO3+AFG3
     &                          +          RXRAT( 401 )      ! xAFG3+RCO3=RCO3+AFG3
     &                          +          RXRAT( 402 )      ! xAFG3+BZCO3=BZCO3+AFG3
     &                          +          RXRAT( 403 )      ! xAFG3+MACO3=MACO3+AFG3

      PROD( xMACR            ) =   0.001 * RXRAT( 537 )      ! OLE1+OH=0.001*xMACR+...
     &                          +  0.002 * RXRAT( 548 )      ! TERP+O3=0.002*xMACR+...
     &                          +  0.005 * RXRAT( 549 )      ! TERP+NO3=0.005*xMACR+...
     &                          +  0.002 * RXRAT( 552 )      ! SESQ+O3=0.002*xMACR+...
     &                          +  0.005 * RXRAT( 553 )      ! SESQ+NO3=0.005*xMACR+...
     &                          +  0.025 * RXRAT( 649 )      ! OLE2+CL=0.025*xMACR+...
      LOSS( xMACR            ) =           RXRAT( 404 )      ! xMACR+NO=NO+MACR
     &                          +          RXRAT( 405 )      ! xMACR+HO2=HO2
     &                          +          RXRAT( 406 )      ! xMACR+NO3=NO3+MACR
     &                          +          RXRAT( 407 )      ! xMACR+MEO2=MEO2+0.5*MACR
     &                          +          RXRAT( 408 )      ! xMACR+RO2C=RO2C+0.5*MACR
     &                          +          RXRAT( 409 )      ! xMACR+RO2XC=RO2XC+0.5*MACR
     &                          +          RXRAT( 410 )      ! xMACR+MECO3=MECO3+MACR
     &                          +          RXRAT( 411 )      ! xMACR+RCO3=RCO3+MACR
     &                          +          RXRAT( 412 )      ! xMACR+BZCO3=BZCO3+MACR
     &                          +          RXRAT( 413 )      ! xMACR+MACO3=MACO3+MACR

      PROD( xMVK             ) =   0.460 * RXRAT( 515 )      ! BUTADIENE13+NO3=0.46*xMVK+...
     &                          +  0.012 * RXRAT( 537 )      ! OLE1+OH=0.012*xMVK+...
     &                          +  0.002 * RXRAT( 541 )      ! OLE2+OH=0.002*xMVK+...
     &                          +  0.003 * RXRAT( 547 )      ! TERP+OH=0.003*xMVK+...
     &                          +  0.001 * RXRAT( 549 )      ! TERP+NO3=0.001*xMVK+...
     &                          +  0.003 * RXRAT( 551 )      ! SESQ+OH=0.003*xMVK+...
     &                          +  0.001 * RXRAT( 553 )      ! SESQ+NO3=0.001*xMVK+...
     &                          +  0.177 * RXRAT( 634 )      ! ISOPRENE+CL=0.177*xMVK+...
     &                          +  0.003 * RXRAT( 635 )      ! APIN+CL=0.003*xMVK+...
     &                          +  0.019 * RXRAT( 648 )      ! OLE1+CL=0.019*xMVK+...
     &                          +  0.058 * RXRAT( 649 )      ! OLE2+CL=0.058*xMVK+...
     &                          +  0.003 * RXRAT( 652 )      ! TERP+CL=0.003*xMVK+...
     &                          +  0.003 * RXRAT( 653 )      ! SESQ+CL=0.003*xMVK+...
      LOSS( xMVK             ) =           RXRAT( 414 )      ! xMVK+NO=NO+MVK
     &                          +          RXRAT( 415 )      ! xMVK+HO2=HO2
     &                          +          RXRAT( 416 )      ! xMVK+NO3=NO3+MVK
     &                          +          RXRAT( 417 )      ! xMVK+MEO2=MEO2+0.5*MVK
     &                          +          RXRAT( 418 )      ! xMVK+RO2C=RO2C+0.5*MVK
     &                          +          RXRAT( 419 )      ! xMVK+RO2XC=RO2XC+0.5*MVK
     &                          +          RXRAT( 420 )      ! xMVK+MECO3=MECO3+MVK
     &                          +          RXRAT( 421 )      ! xMVK+RCO3=RCO3+MVK
     &                          +          RXRAT( 422 )      ! xMVK+BZCO3=BZCO3+MVK
     &                          +          RXRAT( 423 )      ! xMVK+MACO3=MACO3+MVK

      PROD( yRAOOH           ) =   0.314 * RXRAT( 525 )      ! BENZENE+OH=0.314*yRAOOH+...
     &                          +  0.435 * RXRAT( 526 )      ! TOLUENE+OH=0.435*yRAOOH+...
     &                          +  0.555 * RXRAT( 527 )      ! MXYL+OH=0.555*yRAOOH+...
     &                          +  0.586 * RXRAT( 528 )      ! OXYL+OH=0.586*yRAOOH+...
     &                          +  0.461 * RXRAT( 529 )      ! PXYL+OH=0.461*yRAOOH+...
     &                          +  0.705 * RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
      LOSS( yRAOOH           ) =           RXRAT( 464 )      ! yRAOOH+NO=NO
     &                          +          RXRAT( 465 )      ! yRAOOH+HO2=HO2+RAOOH
     &                          +          RXRAT( 466 )      ! yRAOOH+NO3=NO3
     &                          +          RXRAT( 467 )      ! yRAOOH+MEO2=MEO2+0.5*PRD2
     &                          +          RXRAT( 468 )      ! yRAOOH+RO2C=RO2C+0.5*PRD2
     &                          +          RXRAT( 469 )      ! yRAOOH+RO2XC=RO2XC+0.5*PRD2
     &                          +          RXRAT( 470 )      ! yRAOOH+MECO3=MECO3
     &                          +          RXRAT( 471 )      ! yRAOOH+RCO3=RCO3
     &                          +          RXRAT( 472 )      ! yRAOOH+BZCO3=BZCO3
     &                          +          RXRAT( 473 )      ! yRAOOH+MACO3=MACO3

      PROD( xACROLEIN        ) =   0.480 * RXRAT( 513 )      ! BUTADIENE13+OH=...
     &                          +  0.115 * RXRAT( 516 )      ! BUTADIENE13+O3P=...
     &                          +  0.036 * RXRAT( 537 )      ! OLE1+OH=0.036*xACROLEIN+...
     &                          +  0.124 * RXRAT( 632 )      ! PROPENE+CL=0.124*xACROLEIN+...
     &                          +  0.457 * RXRAT( 633 )      ! BUTADIENE13+CL=...
     &                          +  0.215 * RXRAT( 648 )      ! OLE1+CL=0.215*xACROLEIN+...
      LOSS( xACROLEIN        ) =           RXRAT( 494 )      ! xACROLEIN+NO=NO+ACROLEIN
     &                          +          RXRAT( 495 )      ! xACROLEIN+HO2=HO2
     &                          +          RXRAT( 496 )      ! xACROLEIN+NO3=NO3+ACROLEIN
     &                          +          RXRAT( 497 )      ! xACROLEIN+MEO2=MEO2+...
     &                          +          RXRAT( 498 )      ! xACROLEIN+RO2C=RO2C+...
     &                          +          RXRAT( 499 )      ! xACROLEIN+RO2XC=RO2XC+...
     &                          +          RXRAT( 500 )      ! xACROLEIN+MECO3=MECO3+ACROLEIN
     &                          +          RXRAT( 501 )      ! xACROLEIN+RCO3=RCO3+ACROLEIN
     &                          +          RXRAT( 502 )      ! xACROLEIN+BZCO3=BZCO3+ACROLEIN
     &                          +          RXRAT( 503 )      ! xACROLEIN+MACO3=MACO3+ACROLEIN

      PROD( ETHENE           ) =   0.0
      LOSS( ETHENE           ) =           RXRAT( 505 )      ! ETHENE+OH=xHO2+RO2C+...
     &                          +          RXRAT( 506 )      ! ETHENE+O3=0.16*HO2+0.16*OH+...
     &                          +          RXRAT( 507 )      ! ETHENE+NO3=xHO2+RO2C+xRCHO+...
     &                          +          RXRAT( 508 )      ! ETHENE+O3P=0.8*HO2+...
     &                          +          RXRAT( 631 )      ! ETHENE+CL=xHO2+2*RO2C+...

      PROD( PROPENE          ) =   0.0
      LOSS( PROPENE          ) =           RXRAT( 509 )      ! PROPENE+OH=0.984*xHO2+...
     &                          +          RXRAT( 510 )      ! PROPENE+O3=0.165*HO2+...
     &                          +          RXRAT( 511 )      ! PROPENE+NO3=0.949*xHO2+...
     &                          +          RXRAT( 512 )      ! PROPENE+O3P=0.45*RCHO+0.55*MEK
     &                          +          RXRAT( 632 )      ! PROPENE+CL=0.124*HCL+...

      PROD( BUTADIENE13      ) =   0.0
      LOSS( BUTADIENE13      ) =           RXRAT( 513 )      ! BUTADIENE13+OH=0.951*xHO2+...
     &                          +          RXRAT( 514 )      ! BUTADIENE13+O3=0.08*HO2+...
     &                          +          RXRAT( 515 )      ! BUTADIENE13+NO3=0.815*xHO2+...
     &                          +          RXRAT( 516 )      ! BUTADIENE13+O3P=0.25*HO2+...
     &                          +          RXRAT( 633 )      ! BUTADIENE13+CL=0.39*xHO2+...

      PROD( ISOPRENE         ) =   0.0
      LOSS( ISOPRENE         ) =           RXRAT( 517 )      ! ISOPRENE+O3=0.066*HO2+...
     &                          +          RXRAT( 518 )      ! ISOPRENE+O3P=0.25*MEO2+...
     &                          +          RXRAT( 634 )      ! ISOPRENE+CL=0.15*HCL+...
     &                          +          RXRAT( 675 )      ! ISOPRENE+OH=ISOPO2+ISOPRXN
     &                          +          RXRAT( 685 )      ! ISOPRENE+NO3=NISOPO2

      PROD( APIN             ) =   0.0
      LOSS( APIN             ) =           RXRAT( 519 )      ! APIN+OH=0.799*xHO2+...
     &                          +          RXRAT( 520 )      ! APIN+O3=0.009*HO2+...
     &                          +          RXRAT( 521 )      ! APIN+NO3=0.056*xHO2+...
     &                          +          RXRAT( 522 )      ! APIN+O3P=PRD2+TRPRXN
     &                          +          RXRAT( 635 )      ! APIN+CL=0.548*HCL+...

      PROD( TRPRXN           ) =           RXRAT( 519 )      ! APIN+OH=TRPRXN+0.004*xRCO3+...
     &                          +          RXRAT( 520 )      ! APIN+O3=TRPRXN+0.102*xHO2+...
     &                          +          RXRAT( 521 )      ! APIN+NO3=TRPRXN+0.643*xNO2+...
     &                          +          RXRAT( 522 )      ! APIN+O3P=TRPRXN+PRD2
     &                          +          RXRAT( 547 )      ! TERP+OH=TRPRXN+0.064*xRCO3+...
     &                          +          RXRAT( 548 )      ! TERP+O3=TRPRXN+0.046*xHO2+...
     &                          +          RXRAT( 549 )      ! TERP+NO3=TRPRXN+0.287*xNO2+...
     &                          +          RXRAT( 550 )      ! TERP+O3P=TRPRXN+0.763*PRD2+...
      LOSS( TRPRXN           ) =   0.0

      PROD( ACETYLENE        ) =   0.0
      LOSS( ACETYLENE        ) =           RXRAT( 523 )      ! ACETYLENE+OH=0.3*HO2+...
     &                          +          RXRAT( 524 )      ! ACETYLENE+O3=1.5*HO2+...
     &                          +          RXRAT( 636 )      ! ACETYLENE+CL=HO2+CO

      PROD( BENZENE          ) =   0.0
      LOSS( BENZENE          ) =           RXRAT( 525 )      ! BENZENE+OH=0.57*HO2+...

      PROD( BENZRO2          ) =           RXRAT( 525 )      ! BENZENE+OH=BENZRO2+...
      LOSS( BENZRO2          ) =           RXRAT( 654 )      ! BENZRO2+NO=NO+BNZNRXN
     &                          +          RXRAT( 655 )      ! BENZRO2+HO2=HO2+BNZHRXN

      PROD( TOLUENE          ) =   0.0
      LOSS( TOLUENE          ) =           RXRAT( 526 )      ! TOLUENE+OH=0.181*HO2+...
     &                          +          RXRAT( 637 )      ! TOLUENE+CL=0.894*xHO2+...

      PROD( TOLRO2           ) =           RXRAT( 526 )      ! TOLUENE+OH=TOLRO2+...
     &                          +          RXRAT( 545 )      ! ARO1+OH=TOLRO2+0.566*xHO2+...
      LOSS( TOLRO2           ) =           RXRAT( 658 )      ! TOLRO2+NO=NO+TOLNRXN
     &                          +          RXRAT( 659 )      ! TOLRO2+HO2=HO2+TOLHRXN

      PROD( MXYL             ) =   0.0
      LOSS( MXYL             ) =           RXRAT( 527 )      ! MXYL+OH=0.159*HO2+...
     &                          +          RXRAT( 638 )      ! MXYL+CL=0.864*xHO2+...

      PROD( XYLRO2           ) =           RXRAT( 527 )      ! MXYL+OH=XYLRO2+0.52*xHO2+...
     &                          +          RXRAT( 528 )      ! OXYL+OH=XYLRO2+0.554*xHO2+...
     &                          +          RXRAT( 529 )      ! PXYL+OH=XYLRO2+0.487*xHO2+...
     &                          +          RXRAT( 530 )      ! TRIMETH_BENZ124+OH=XYLRO2+...
     &                          +          RXRAT( 546 )      ! ARO2+OH=XYLRO2+0.617*xHO2+...
      LOSS( XYLRO2           ) =           RXRAT( 656 )      ! XYLRO2+NO=NO+XYLNRXN
     &                          +          RXRAT( 657 )      ! XYLRO2+HO2=HO2+XYLHRXN

      PROD( OXYL             ) =   0.0
      LOSS( OXYL             ) =           RXRAT( 528 )      ! OXYL+OH=0.161*HO2+...
     &                          +          RXRAT( 639 )      ! OXYL+CL=0.864*xHO2+...

      PROD( PXYL             ) =   0.0
      LOSS( PXYL             ) =           RXRAT( 529 )      ! PXYL+OH=0.159*HO2+...
     &                          +          RXRAT( 640 )      ! PXYL+CL=0.864*xHO2+...

      PROD( TRIMETH_BENZ124  ) =   0.0
      LOSS( TRIMETH_BENZ124  ) =           RXRAT( 530 )      ! TRIMETH_BENZ124+OH=...
     &                          +          RXRAT( 641 )      ! TRIMETH_BENZ124+CL=...

      PROD( ETOH             ) =   0.0
      LOSS( ETOH             ) =           RXRAT( 531 )      ! ETOH+OH=0.95*HO2+0.05*xHO2+...
     &                          +          RXRAT( 642 )      ! ETOH+CL=HCL+0.688*HO2+...

      PROD( ALK1             ) =   0.0
      LOSS( ALK1             ) =           RXRAT( 532 )      ! ALK1+OH=xHO2+RO2C+xCCHO+yROOH
     &                          +          RXRAT( 643 )      ! ALK1+CL=HCL+xHO2+RO2C+...

      PROD( ALK2             ) =   0.0
      LOSS( ALK2             ) =           RXRAT( 533 )      ! ALK2+OH=0.965*xHO2+...
     &                          +          RXRAT( 644 )      ! ALK2+CL=HCL+0.97*xHO2+...

      PROD( ALK3             ) =   0.0
      LOSS( ALK3             ) =           RXRAT( 534 )      ! ALK3+OH=0.695*xHO2+...
     &                          +          RXRAT( 645 )      ! ALK3+CL=HCL+0.835*xHO2+...

      PROD( ALK4             ) =   0.0
      LOSS( ALK4             ) =           RXRAT( 535 )      ! ALK4+OH=0.83*xHO2+...
     &                          +          RXRAT( 646 )      ! ALK4+CL=HCL+0.827*xHO2+...

      PROD( ALK5             ) =   0.0
      LOSS( ALK5             ) =           RXRAT( 536 )      ! ALK5+OH=0.647*xHO2+...
     &                          +          RXRAT( 647 )      ! ALK5+CL=HCL+0.647*xHO2+...

      PROD( ALK5RXN          ) =           RXRAT( 536 )      ! ALK5+OH=ALK5RXN+1.605*RO2C+...
      LOSS( ALK5RXN          ) =   0.0

      PROD( OLE1             ) =   0.0
      LOSS( OLE1             ) =           RXRAT( 537 )      ! OLE1+OH=0.871*xHO2+...
     &                          +          RXRAT( 538 )      ! OLE1+O3=0.095*HO2+...
     &                          +          RXRAT( 539 )      ! OLE1+NO3=0.772*xHO2+...
     &                          +          RXRAT( 540 )      ! OLE1+O3P=0.45*RCHO+...
     &                          +          RXRAT( 648 )      ! OLE1+CL=0.384*HCL+...

      PROD( OLE2             ) =   0.0
      LOSS( OLE2             ) =           RXRAT( 541 )      ! OLE2+OH=0.912*xHO2+...
     &                          +          RXRAT( 542 )      ! OLE2+O3=0.094*HO2+...
     &                          +          RXRAT( 543 )      ! OLE2+NO3=0.4*xHO2+...
     &                          +          RXRAT( 544 )      ! OLE2+O3P=0.079*RCHO+...
     &                          +          RXRAT( 649 )      ! OLE2+CL=0.279*HCL+...

      PROD( ARO1             ) =   0.0
      LOSS( ARO1             ) =           RXRAT( 545 )      ! ARO1+OH=0.123*HO2+...
     &                          +          RXRAT( 650 )      ! ARO1+CL=0.84*xHO2+...

      PROD( ARO2             ) =   0.043 * RXRAT( 676 )      ! ISOPO2+NO=0.043*ARO2+...
     &                          +  0.020 * RXRAT( 678 )      ! ISOPO2+MEO2=0.02*ARO2+...
     &                          +  0.020 * RXRAT( 679 )      ! ISOPO2+RO2C=0.02*ARO2+...
     &                          +  0.050 * RXRAT( 680 )      ! ISOPO2+ISOPO2=0.05*ARO2+...
     &                          +  0.050 * RXRAT( 681 )      ! ISOPO2+MECO3=0.05*ARO2+CO2+...
     &                          +  0.050 * RXRAT( 786 )      ! ISOPOOH=0.05*ARO2+0.91*HO2+...
      LOSS( ARO2             ) =           RXRAT( 546 )      ! ARO2+OH=0.077*HO2+...
     &                          +          RXRAT( 651 )      ! ARO2+CL=0.828*xHO2+...

      PROD( TERP             ) =   0.0
      LOSS( TERP             ) =           RXRAT( 547 )      ! TERP+OH=0.734*xHO2+...
     &                          +          RXRAT( 548 )      ! TERP+O3=0.078*HO2+...
     &                          +          RXRAT( 549 )      ! TERP+NO3=0.227*xHO2+...
     &                          +          RXRAT( 550 )      ! TERP+O3P=0.237*RCHO+...
     &                          +          RXRAT( 652 )      ! TERP+CL=0.548*HCL+...

      PROD( SESQ             ) =   0.0
      LOSS( SESQ             ) =           RXRAT( 551 )      ! SESQ+OH=0.734*xHO2+...
     &                          +          RXRAT( 552 )      ! SESQ+O3=0.078*HO2+...
     &                          +          RXRAT( 553 )      ! SESQ+NO3=0.227*xHO2+...
     &                          +          RXRAT( 554 )      ! SESQ+O3P=0.237*RCHO+...
     &                          +          RXRAT( 653 )      ! SESQ+CL=0.252*xHO2+...

      PROD( SESQRXN          ) =           RXRAT( 551 )      ! SESQ+OH=SESQRXN+...
     &                          +          RXRAT( 552 )      ! SESQ+O3=SESQRXN+0.046*xHO2+...
     &                          +          RXRAT( 553 )      ! SESQ+NO3=SESQRXN+...
     &                          +          RXRAT( 554 )      ! SESQ+O3P=SESQRXN+...
      LOSS( SESQRXN          ) =   0.0

      PROD( CL2              ) =           RXRAT( 571 )      ! CL+CLONO2=CL2+NO3
     &                          +  0.290 * RXRAT( 574 )      ! CLO+CLO=0.29*CL2+1.42*CL
      LOSS( CL2              ) =           RXRAT( 555 )      ! CL2=2*CL

      PROD( CL               ) =   2.000 * RXRAT( 555 )      ! CL2=2*CL
     &                          +          RXRAT( 557 )      ! CLNO=CL+NO
     &                          +          RXRAT( 560 )      ! CLONO=CL+NO2
     &                          +          RXRAT( 561 )      ! CLNO2=CL+NO2
     &                          +          RXRAT( 566 )      ! CLO+NO=CL+NO2
     &                          +          RXRAT( 569 )      ! CLONO2=CL+NO3
     &                          +          RXRAT( 573 )      ! HOCL=CL+OH
     &                          +  1.420 * RXRAT( 574 )      ! CLO+CLO=1.42*CL+0.29*CL2
     &                          +          RXRAT( 575 )      ! OH+HCL=CL
     &                          +          RXRAT( 600 )      ! xCL+NO=CL+NO
     &                          +          RXRAT( 602 )      ! xCL+NO3=CL+NO3
     &                          +  0.500 * RXRAT( 603 )      ! xCL+MEO2=0.5*CL+MEO2
     &                          +  0.500 * RXRAT( 604 )      ! xCL+RO2C=0.5*CL+RO2C
     &                          +  0.500 * RXRAT( 605 )      ! xCL+RO2XC=0.5*CL+RO2XC
     &                          +          RXRAT( 606 )      ! xCL+MECO3=CL+MECO3
     &                          +          RXRAT( 607 )      ! xCL+RCO3=CL+RCO3
     &                          +          RXRAT( 608 )      ! xCL+BZCO3=CL+BZCO3
     &                          +          RXRAT( 609 )      ! xCL+MACO3=CL+MACO3
      LOSS( CL               ) =           RXRAT( 556 )      ! CL+NO=CLNO
     &                          +          RXRAT( 558 )      ! CL+NO2=CLONO
     &                          +          RXRAT( 559 )      ! CL+NO2=CLNO2
     &                          +          RXRAT( 562 )      ! CL+HO2=HCL
     &                          +          RXRAT( 563 )      ! CL+HO2=CLO+OH
     &                          +          RXRAT( 564 )      ! CL+O3=CLO
     &                          +          RXRAT( 565 )      ! CL+NO3=CLO+NO2
     &                          +          RXRAT( 571 )      ! CL+CLONO2=CL2+NO3
     &                          +          RXRAT( 576 )      ! CL=HCL+HO2
     &                          +          RXRAT( 577 )      ! CL+HCHO=HCL+HO2+CO
     &                          +          RXRAT( 578 )      ! CL+CCHO=HCL+MECO3
     &                          +          RXRAT( 579 )      ! CL+MEOH=HCL+HCHO+HO2
     &                          +          RXRAT( 580 )      ! CL+RCHO=HCL+0.9*RCO3+...
     &                          +          RXRAT( 581 )      ! CL+ACETONE=HCL+RO2C+xHCHO+...
     &                          +          RXRAT( 582 )      ! CL+MEK=HCL+0.975*RO2C+...
     &                          +          RXRAT( 583 )      ! CL+RNO3=HCL+0.038*NO2+...
     &                          +          RXRAT( 584 )      ! CL+PRD2=HCL+0.314*HO2+...
     &                          +          RXRAT( 585 )      ! CL+GLY=HCL+0.63*HO2+...
     &                          +          RXRAT( 586 )      ! CL+MGLY=HCL+CO+MECO3
     &                          +          RXRAT( 587 )      ! CL+CRES=HCL+xHO2+xBALD+yR6OOH
     &                          +          RXRAT( 588 )      ! CL+BALD=HCL+BZCO3
     &                          +          RXRAT( 589 )      ! CL+ROOH=HCL+0.414*OH+...
     &                          +          RXRAT( 590 )      ! CL+R6OOH=HCL+0.145*OH+...
     &                          +          RXRAT( 591 )      ! CL+RAOOH=0.404*HCL+...
     &                          +          RXRAT( 592 )      ! CL+ACROLEIN=0.484*xHO2+...
     &                          +          RXRAT( 593 )      ! CL+MACR=0.25*HCL+...
     &                          +          RXRAT( 594 )      ! CL+MVK=1.283*RO2C+...
     &                          +          RXRAT( 595 )      ! CL+IPRD=0.401*HCL+...
     &                          +          RXRAT( 598 )      ! CL+CLCCHO=HCL+RCO3
     &                          +          RXRAT( 630 )      ! CL=HCL+MEO2
     &                          +          RXRAT( 631 )      ! CL+ETHENE=xHO2+2*RO2C+...
     &                          +          RXRAT( 632 )      ! CL+PROPENE=0.124*HCL+...
     &                          +          RXRAT( 633 )      ! CL+BUTADIENE13=0.39*xHO2+...
     &                          +          RXRAT( 634 )      ! CL+ISOPRENE=0.15*HCL+...
     &                          +          RXRAT( 635 )      ! CL+APIN=0.548*HCL+...
     &                          +          RXRAT( 636 )      ! CL+ACETYLENE=HO2+CO
     &                          +          RXRAT( 637 )      ! CL+TOLUENE=0.894*xHO2+...
     &                          +          RXRAT( 638 )      ! CL+MXYL=0.864*xHO2+...
     &                          +          RXRAT( 639 )      ! CL+OXYL=0.864*xHO2+...
     &                          +          RXRAT( 640 )      ! CL+PXYL=0.864*xHO2+...
     &                          +          RXRAT( 641 )      ! CL+TRIMETH_BENZ124=...
     &                          +          RXRAT( 642 )      ! CL+ETOH=HCL+0.688*HO2+...
     &                          +          RXRAT( 643 )      ! CL+ALK1=HCL+xHO2+RO2C+...
     &                          +          RXRAT( 644 )      ! CL+ALK2=HCL+0.97*xHO2+...
     &                          +          RXRAT( 645 )      ! CL+ALK3=HCL+0.835*xHO2+...
     &                          +          RXRAT( 646 )      ! CL+ALK4=HCL+0.827*xHO2+...
     &                          +          RXRAT( 647 )      ! CL+ALK5=HCL+0.647*xHO2+...
     &                          +          RXRAT( 648 )      ! CL+OLE1=0.384*HCL+...
     &                          +          RXRAT( 649 )      ! CL+OLE2=0.279*HCL+...
     &                          +          RXRAT( 650 )      ! CL+ARO1=0.84*xHO2+...
     &                          +          RXRAT( 651 )      ! CL+ARO2=0.828*xHO2+...
     &                          +          RXRAT( 652 )      ! CL+TERP=0.548*HCL+...
     &                          +          RXRAT( 653 )      ! CL+SESQ=0.252*xHO2+...

      PROD( CLNO             ) =           RXRAT( 556 )      ! CL+NO=CLNO
      LOSS( CLNO             ) =           RXRAT( 557 )      ! CLNO=CL+NO

      PROD( CLONO            ) =           RXRAT( 558 )      ! CL+NO2=CLONO
      LOSS( CLONO            ) =           RXRAT( 560 )      ! CLONO=CL+NO2

      PROD( CLNO2            ) =           RXRAT( 559 )      ! CL+NO2=CLNO2
      LOSS( CLNO2            ) =           RXRAT( 561 )      ! CLNO2=CL+NO2

      PROD( HCL              ) =           RXRAT( 562 )      ! CL+HO2=HCL
     &                          +          RXRAT( 576 )      ! CL=HCL+HO2
     &                          +          RXRAT( 577 )      ! HCHO+CL=HCL+HO2+CO
     &                          +          RXRAT( 578 )      ! CCHO+CL=HCL+MECO3
     &                          +          RXRAT( 579 )      ! MEOH+CL=HCL+HCHO+HO2
     &                          +          RXRAT( 580 )      ! RCHO+CL=HCL+0.9*RCO3+...
     &                          +          RXRAT( 581 )      ! ACETONE+CL=HCL+RO2C+xHCHO+...
     &                          +          RXRAT( 582 )      ! MEK+CL=HCL+0.975*RO2C+...
     &                          +          RXRAT( 583 )      ! RNO3+CL=HCL+0.038*NO2+...
     &                          +          RXRAT( 584 )      ! PRD2+CL=HCL+0.314*HO2+...
     &                          +          RXRAT( 585 )      ! GLY+CL=HCL+0.63*HO2+...
     &                          +          RXRAT( 586 )      ! MGLY+CL=HCL+CO+MECO3
     &                          +          RXRAT( 587 )      ! CRES+CL=HCL+xHO2+xBALD+yR6OOH
     &                          +          RXRAT( 588 )      ! BALD+CL=HCL+BZCO3
     &                          +          RXRAT( 589 )      ! ROOH+CL=HCL+0.414*OH+...
     &                          +          RXRAT( 590 )      ! R6OOH+CL=HCL+0.145*OH+...
     &                          +  0.404 * RXRAT( 591 )      ! RAOOH+CL=0.404*HCL+...
     &                          +  0.216 * RXRAT( 592 )      ! ACROLEIN+CL=0.216*HCL+...
     &                          +  0.250 * RXRAT( 593 )      ! MACR+CL=0.25*HCL+...
     &                          +  0.401 * RXRAT( 595 )      ! IPRD+CL=0.401*HCL+...
     &                          +          RXRAT( 598 )      ! CLCCHO+CL=HCL+RCO3
     &                          +          RXRAT( 630 )      ! CL=HCL+MEO2
     &                          +  0.124 * RXRAT( 632 )      ! PROPENE+CL=0.124*HCL+...
     &                          +  0.150 * RXRAT( 634 )      ! ISOPRENE+CL=0.15*HCL+...
     &                          +  0.548 * RXRAT( 635 )      ! APIN+CL=0.548*HCL+...
     &                          +          RXRAT( 642 )      ! ETOH+CL=HCL+0.688*HO2+...
     &                          +          RXRAT( 643 )      ! ALK1+CL=HCL+xHO2+RO2C+...
     &                          +          RXRAT( 644 )      ! ALK2+CL=HCL+0.97*xHO2+...
     &                          +          RXRAT( 645 )      ! ALK3+CL=HCL+0.835*xHO2+...
     &                          +          RXRAT( 646 )      ! ALK4+CL=HCL+0.827*xHO2+...
     &                          +          RXRAT( 647 )      ! ALK5+CL=HCL+0.647*xHO2+...
     &                          +  0.384 * RXRAT( 648 )      ! OLE1+CL=0.384*HCL+...
     &                          +  0.279 * RXRAT( 649 )      ! OLE2+CL=0.279*HCL+...
     &                          +  0.548 * RXRAT( 652 )      ! TERP+CL=0.548*HCL+...
     &                          +  0.548 * RXRAT( 653 )      ! SESQ+CL=0.548*HCL+...
      LOSS( HCL              ) =           RXRAT( 575 )      ! HCL+OH=CL

      PROD( CLO              ) =           RXRAT( 563 )      ! CL+HO2=CLO+OH
     &                          +          RXRAT( 564 )      ! CL+O3=CLO
     &                          +          RXRAT( 565 )      ! CL+NO3=CLO+NO2
     &                          +          RXRAT( 568 )      ! CLONO2=CLO+NO2
     &                          +          RXRAT( 570 )      ! CLONO2=CLO+NO2
      LOSS( CLO              ) =           RXRAT( 566 )      ! CLO+NO=CL+NO2
     &                          +          RXRAT( 567 )      ! CLO+NO2=CLONO2
     &                          +          RXRAT( 572 )      ! CLO+HO2=HOCL
     &                          +  2.000 * RXRAT( 574 )      ! CLO+CLO=0.29*CL2+1.42*CL

      PROD( CLONO2           ) =           RXRAT( 567 )      ! CLO+NO2=CLONO2
      LOSS( CLONO2           ) =           RXRAT( 568 )      ! CLONO2=CLO+NO2
     &                          +          RXRAT( 569 )      ! CLONO2=CL+NO3
     &                          +          RXRAT( 570 )      ! CLONO2=CLO+NO2
     &                          +          RXRAT( 571 )      ! CLONO2+CL=CL2+NO3

      PROD( HOCL             ) =           RXRAT( 572 )      ! CLO+HO2=HOCL
      LOSS( HOCL             ) =           RXRAT( 573 )      ! HOCL=OH+CL

      PROD( xCL              ) =   0.274 * RXRAT( 592 )      ! ACROLEIN+CL=0.274*xCL+...
     &                          +          RXRAT( 596 )      ! CLCCHO=xCL+CO+RO2C+HO2+...
     &                          +          RXRAT( 599 )      ! CLACET=xCL+RO2C+MECO3+...
     &                          +  0.541 * RXRAT( 633 )      ! BUTADIENE13+CL=0.541*xCL+...
     &                          +  0.177 * RXRAT( 634 )      ! ISOPRENE+CL=0.177*xCL+...
     &                          +  0.068 * RXRAT( 635 )      ! APIN+CL=0.068*xCL+...
     &                          +  0.442 * RXRAT( 649 )      ! OLE2+CL=0.442*xCL+...
     &                          +  0.068 * RXRAT( 652 )      ! TERP+CL=0.068*xCL+...
     &                          +  0.068 * RXRAT( 653 )      ! SESQ+CL=0.068*xCL+...
      LOSS( xCL              ) =           RXRAT( 600 )      ! xCL+NO=NO+CL
     &                          +          RXRAT( 601 )      ! xCL+HO2=HO2
     &                          +          RXRAT( 602 )      ! xCL+NO3=NO3+CL
     &                          +          RXRAT( 603 )      ! xCL+MEO2=MEO2+0.5*CL
     &                          +          RXRAT( 604 )      ! xCL+RO2C=RO2C+0.5*CL
     &                          +          RXRAT( 605 )      ! xCL+RO2XC=RO2XC+0.5*CL
     &                          +          RXRAT( 606 )      ! xCL+MECO3=MECO3+CL
     &                          +          RXRAT( 607 )      ! xCL+RCO3=RCO3+CL
     &                          +          RXRAT( 608 )      ! xCL+BZCO3=BZCO3+CL
     &                          +          RXRAT( 609 )      ! xCL+MACO3=MACO3+CL

      PROD( xCLCCHO          ) =   0.484 * RXRAT( 592 )      ! ACROLEIN+CL=0.484*xCLCCHO+...
     &                          +  0.180 * RXRAT( 593 )      ! MACR+CL=0.18*xCLCCHO+...
     &                          +  0.947 * RXRAT( 594 )      ! MVK+CL=0.947*xCLCCHO+...
     &                          +  0.140 * RXRAT( 595 )      ! IPRD+CL=0.14*xCLCCHO+...
     &                          +  0.306 * RXRAT( 632 )      ! PROPENE+CL=0.306*xCLCCHO+...
     &                          +  0.067 * RXRAT( 634 )      ! ISOPRENE+CL=0.067*xCLCCHO+...
     &                          +  0.109 * RXRAT( 635 )      ! APIN+CL=0.109*xCLCCHO+...
     &                          +  0.192 * RXRAT( 648 )      ! OLE1+CL=0.192*xCLCCHO+...
     &                          +  0.013 * RXRAT( 649 )      ! OLE2+CL=0.013*xCLCCHO+...
     &                          +  0.109 * RXRAT( 652 )      ! TERP+CL=0.109*xCLCCHO+...
     &                          +  0.109 * RXRAT( 653 )      ! SESQ+CL=0.109*xCLCCHO+...
      LOSS( xCLCCHO          ) =           RXRAT( 610 )      ! xCLCCHO+NO=NO+CLCCHO
     &                          +          RXRAT( 611 )      ! xCLCCHO+HO2=HO2
     &                          +          RXRAT( 612 )      ! xCLCCHO+NO3=NO3+CLCCHO
     &                          +          RXRAT( 613 )      ! xCLCCHO+MEO2=MEO2+0.5*CLCCHO
     &                          +          RXRAT( 614 )      ! xCLCCHO+RO2C=RO2C+0.5*CLCCHO
     &                          +          RXRAT( 615 )      ! xCLCCHO+RO2XC=RO2XC+0.5*CLCCHO
     &                          +          RXRAT( 616 )      ! xCLCCHO+MECO3=MECO3+CLCCHO
     &                          +          RXRAT( 617 )      ! xCLCCHO+RCO3=RCO3+CLCCHO
     &                          +          RXRAT( 618 )      ! xCLCCHO+BZCO3=BZCO3+CLCCHO
     &                          +          RXRAT( 619 )      ! xCLCCHO+MACO3=MACO3+CLCCHO

      PROD( xCLACET          ) =   0.541 * RXRAT( 593 )      ! MACR+CL=0.541*xCLACET+...
     &                          +  0.420 * RXRAT( 595 )      ! IPRD+CL=0.42*xCLACET+...
     &                          +  0.540 * RXRAT( 632 )      ! PROPENE+CL=0.54*xCLACET+...
     &                          +  0.337 * RXRAT( 648 )      ! OLE1+CL=0.337*xCLACET+...
     &                          +  0.191 * RXRAT( 649 )      ! OLE2+CL=0.191*xCLACET+...
      LOSS( xCLACET          ) =           RXRAT( 620 )      ! xCLACET+NO=NO+CLACET
     &                          +          RXRAT( 621 )      ! xCLACET+HO2=HO2
     &                          +          RXRAT( 622 )      ! xCLACET+NO3=NO3+CLACET
     &                          +          RXRAT( 623 )      ! xCLACET+MEO2=MEO2+0.5*CLACET
     &                          +          RXRAT( 624 )      ! xCLACET+RO2C=RO2C+0.5*CLACET
     &                          +          RXRAT( 625 )      ! xCLACET+RO2XC=RO2XC+0.5*CLACET
     &                          +          RXRAT( 626 )      ! xCLACET+MECO3=MECO3+CLACET
     &                          +          RXRAT( 627 )      ! xCLACET+RCO3=RCO3+CLACET
     &                          +          RXRAT( 628 )      ! xCLACET+BZCO3=BZCO3+CLACET
     &                          +          RXRAT( 629 )      ! xCLACET+MACO3=MACO3+CLACET

      PROD( CLCCHO           ) =           RXRAT( 610 )      ! xCLCCHO+NO=CLCCHO+NO
     &                          +          RXRAT( 612 )      ! xCLCCHO+NO3=CLCCHO+NO3
     &                          +  0.500 * RXRAT( 613 )      ! xCLCCHO+MEO2=0.5*CLCCHO+MEO2
     &                          +  0.500 * RXRAT( 614 )      ! xCLCCHO+RO2C=0.5*CLCCHO+RO2C
     &                          +  0.500 * RXRAT( 615 )      ! xCLCCHO+RO2XC=0.5*CLCCHO+RO2XC
     &                          +          RXRAT( 616 )      ! xCLCCHO+MECO3=CLCCHO+MECO3
     &                          +          RXRAT( 617 )      ! xCLCCHO+RCO3=CLCCHO+RCO3
     &                          +          RXRAT( 618 )      ! xCLCCHO+BZCO3=CLCCHO+BZCO3
     &                          +          RXRAT( 619 )      ! xCLCCHO+MACO3=CLCCHO+MACO3
      LOSS( CLCCHO           ) =           RXRAT( 596 )      ! CLCCHO=HO2+CO+RO2C+xCL+...
     &                          +          RXRAT( 597 )      ! CLCCHO+OH=RCO3
     &                          +          RXRAT( 598 )      ! CLCCHO+CL=HCL+RCO3

      PROD( CLACET           ) =           RXRAT( 620 )      ! xCLACET+NO=CLACET+NO
     &                          +          RXRAT( 622 )      ! xCLACET+NO3=CLACET+NO3
     &                          +  0.500 * RXRAT( 623 )      ! xCLACET+MEO2=0.5*CLACET+MEO2
     &                          +  0.500 * RXRAT( 624 )      ! xCLACET+RO2C=0.5*CLACET+RO2C
     &                          +  0.500 * RXRAT( 625 )      ! xCLACET+RO2XC=0.5*CLACET+RO2XC
     &                          +          RXRAT( 626 )      ! xCLACET+MECO3=CLACET+MECO3
     &                          +          RXRAT( 627 )      ! xCLACET+RCO3=CLACET+RCO3
     &                          +          RXRAT( 628 )      ! xCLACET+BZCO3=CLACET+BZCO3
     &                          +          RXRAT( 629 )      ! xCLACET+MACO3=CLACET+MACO3
      LOSS( CLACET           ) =           RXRAT( 599 )      ! CLACET=MECO3+RO2C+xCL+...

      PROD( CLCHO            ) =           RXRAT( 631 )      ! ETHENE+CL=CLCHO+2*RO2C+...
      LOSS( CLCHO            ) =   0.0

      PROD( BNZNRXN          ) =           RXRAT( 654 )      ! BENZRO2+NO=BNZNRXN+NO
      LOSS( BNZNRXN          ) =   0.0

      PROD( BNZHRXN          ) =           RXRAT( 655 )      ! BENZRO2+HO2=BNZHRXN+HO2
      LOSS( BNZHRXN          ) =   0.0

      PROD( XYLNRXN          ) =           RXRAT( 656 )      ! XYLRO2+NO=XYLNRXN+NO
      LOSS( XYLNRXN          ) =   0.0

      PROD( XYLHRXN          ) =           RXRAT( 657 )      ! XYLRO2+HO2=XYLHRXN+HO2
      LOSS( XYLHRXN          ) =   0.0

      PROD( TOLNRXN          ) =           RXRAT( 658 )      ! TOLRO2+NO=TOLNRXN+NO
      LOSS( TOLNRXN          ) =   0.0

      PROD( TOLHRXN          ) =           RXRAT( 659 )      ! TOLRO2+HO2=TOLHRXN+HO2
      LOSS( TOLHRXN          ) =   0.0

      PROD( HCHO_PRIMARY     ) =   0.0
      LOSS( HCHO_PRIMARY     ) =           RXRAT( 660 )      ! HCHO_PRIMARY=
     &                          +          RXRAT( 661 )      ! HCHO_PRIMARY=
     &                          +          RXRAT( 662 )      ! HCHO_PRIMARY+OH=OH
     &                          +          RXRAT( 663 )      ! HCHO_PRIMARY+NO3=NO3
     &                          +          RXRAT( 664 )      ! HCHO_PRIMARY+CL=CL

      PROD( CCHO_PRIMARY     ) =   0.0
      LOSS( CCHO_PRIMARY     ) =           RXRAT( 665 )      ! CCHO_PRIMARY+OH=OH
     &                          +          RXRAT( 666 )      ! CCHO_PRIMARY=
     &                          +          RXRAT( 667 )      ! CCHO_PRIMARY+NO3=NO3
     &                          +          RXRAT( 668 )      ! CCHO_PRIMARY+CL=CL

      PROD( ACROLEIN_PRIMARY ) =   0.0
      LOSS( ACROLEIN_PRIMARY ) =           RXRAT( 669 )      ! ACROLEIN_PRIMARY+OH=OH
     &                          +          RXRAT( 670 )      ! ACROLEIN_PRIMARY+O3=O3
     &                          +          RXRAT( 671 )      ! ACROLEIN_PRIMARY+NO3=NO3
     &                          +          RXRAT( 672 )      ! ACROLEIN_PRIMARY+O3P=O3P
     &                          +          RXRAT( 673 )      ! ACROLEIN_PRIMARY=
     &                          +          RXRAT( 674 )      ! ACROLEIN_PRIMARY+CL=CL

      PROD( ISOPO2           ) =           RXRAT( 675 )      ! ISOPRENE+OH=ISOPO2+ISOPRXN
     &                          +  0.387 * RXRAT( 779 )      ! ISOPOOH+OH=0.387*ISOPO2+...
      LOSS( ISOPO2           ) =           RXRAT( 676 )      ! ISOPO2+NO=0.4*MVK+...
     &                          +          RXRAT( 677 )      ! ISOPO2+HO2=0.88*ISOPOOH+...
     &                          +          RXRAT( 678 )      ! ISOPO2+MEO2=0.45*HO2+...
     &                          +          RXRAT( 679 )      ! ISOPO2+RO2C=0.45*HO2+...
     &                          +  2.000 * RXRAT( 680 )      ! ISOPO2+ISOPO2=0.91*HO2+...
     &                          +          RXRAT( 681 )      ! ISOPO2+MECO3=MEO2+CO2+...
     &                          +          RXRAT( 682 )      ! ISOPO2=HO2+HPALD

      PROD( ISOPRXN          ) =           RXRAT( 675 )      ! ISOPRENE+OH=ISOPRXN+ISOPO2
      LOSS( ISOPRXN          ) =   0.0

      PROD( ISOPND           ) =   0.070 * RXRAT( 676 )      ! ISOPO2+NO=0.07*ISOPND+...
     &                          +  0.250 * RXRAT( 689 )      ! NISOPO2+MEO2=0.25*ISOPND+...
     &                          +  0.250 * RXRAT( 690 )      ! NISOPO2+RO2C=0.25*ISOPND+...
     &                          +  0.500 * RXRAT( 691 )      ! NISOPO2+NISOPO2=0.5*ISOPND+...
      LOSS( ISOPND           ) =           RXRAT( 700 )      ! ISOPND+OH=ISOPNOOD
     &                          +          RXRAT( 706 )      ! ISOPND+O3=0.36*ETHLN+...

      PROD( ISOPNB           ) =   0.047 * RXRAT( 676 )      ! ISOPO2+NO=0.047*ISOPNB+...
      LOSS( ISOPNB           ) =           RXRAT( 707 )      ! ISOPNB+OH=ISOPNOOB
     &                          +          RXRAT( 713 )      ! ISOPNB+O3=0.12*MVKN+...

      PROD( HC5              ) =   0.100 * RXRAT( 676 )      ! ISOPO2+NO=0.1*HC5+...
     &                          +  0.060 * RXRAT( 678 )      ! ISOPO2+MEO2=0.06*HC5+...
     &                          +  0.060 * RXRAT( 679 )      ! ISOPO2+RO2C=0.06*HC5+...
     &                          +  0.110 * RXRAT( 680 )      ! ISOPO2+ISOPO2=0.11*HC5+...
     &                          +  0.110 * RXRAT( 681 )      ! ISOPO2+MECO3=0.11*HC5+CO2+...
     &                          +  0.230 * RXRAT( 686 )      ! NISOPO2+NO3=0.23*HC5+...
     &                          +  0.230 * RXRAT( 687 )      ! NISOPO2+NO=0.23*HC5+...
     &                          +  0.115 * RXRAT( 689 )      ! NISOPO2+MEO2=0.115*HC5+...
     &                          +  0.115 * RXRAT( 690 )      ! NISOPO2+RO2C=0.115*HC5+...
     &                          +  0.230 * RXRAT( 691 )      ! NISOPO2+NISOPO2=0.23*HC5+...
     &                          +  0.230 * RXRAT( 692 )      ! NISOPO2+MECO3=0.23*HC5+CO2+...
     &                          +  0.613 * RXRAT( 779 )      ! ISOPOOH+OH=0.613*HC5+...
     &                          +  0.110 * RXRAT( 786 )      ! ISOPOOH=0.11*HC5+0.91*HO2+...
      LOSS( HC5              ) =           RXRAT( 693 )      ! HC5+OH=HC5OO
     &                          +          RXRAT( 699 )      ! HC5+O3=0.5*MGLY+0.35*GLY+...

      PROD( DIBOO            ) =   0.080 * RXRAT( 676 )      ! ISOPO2+NO=0.08*DIBOO+...
     &                          +  0.050 * RXRAT( 678 )      ! ISOPO2+MEO2=0.05*DIBOO+...
     &                          +  0.050 * RXRAT( 679 )      ! ISOPO2+RO2C=0.05*DIBOO+...
     &                          +  0.090 * RXRAT( 680 )      ! ISOPO2+ISOPO2=0.09*DIBOO+...
     &                          +  0.090 * RXRAT( 681 )      ! ISOPO2+MECO3=0.09*DIBOO+...
     &                          +  0.090 * RXRAT( 786 )      ! ISOPOOH=0.09*DIBOO+...
      LOSS( DIBOO            ) =           RXRAT( 735 )      ! DIBOO+NO=NO2+HO2+...
     &                          +          RXRAT( 736 )      ! DIBOO+HO2=R6OOH
     &                          +          RXRAT( 737 )      ! DIBOO+MEO2=0.5*HO2+...
     &                          +          RXRAT( 738 )      ! DIBOO+RO2C=0.5*HO2+...
     &                          +          RXRAT( 739 )      ! DIBOO+MECO3=HO2+...

      PROD( ISOPOOH          ) =   0.880 * RXRAT( 677 )      ! ISOPO2+HO2=0.88*ISOPOOH+...
      LOSS( ISOPOOH          ) =           RXRAT( 778 )      ! ISOPOOH+OH=IEPOX+OH
     &                          +          RXRAT( 779 )      ! ISOPOOH+OH=0.387*ISOPO2+...
     &                          +          RXRAT( 786 )      ! ISOPOOH=OH+0.91*HO2+...

      PROD( HPALD            ) =           RXRAT( 682 )      ! ISOPO2=HPALD+HO2
      LOSS( HPALD            ) =           RXRAT( 683 )      ! HPALD=OH+HO2+0.5*HACET+...
     &                          +          RXRAT( 684 )      ! HPALD+OH=OH+PRD2

      PROD( HACET            ) =   0.500 * RXRAT( 683 )      ! HPALD=0.5*HACET+HO2+OH+...
     &                          +  0.216 * RXRAT( 694 )      ! HC5OO+NO=0.216*HACET+...
     &                          +  0.108 * RXRAT( 696 )      ! HC5OO+MEO2=0.108*HACET+...
     &                          +  0.108 * RXRAT( 697 )      ! HC5OO+RO2C=0.108*HACET+...
     &                          +  0.216 * RXRAT( 698 )      ! HC5OO+MECO3=0.216*HACET+...
     &                          +  0.150 * RXRAT( 699 )      ! HC5+O3=0.15*HACET+0.35*GLY+...
     &                          +  0.440 * RXRAT( 701 )      ! ISOPNOOD+NO=0.44*HACET+...
     &                          +  0.220 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.22*HACET+...
     &                          +  0.220 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.22*HACET+...
     &                          +  0.440 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.44*HACET+...
     &                          +  0.160 * RXRAT( 706 )      ! ISOPND+O3=0.16*HACET+...
     &                          +  0.600 * RXRAT( 708 )      ! ISOPNOOB+NO=0.6*HACET+...
     &                          +  0.300 * RXRAT( 710 )      ! ISOPNOOB+MEO2=0.3*HACET+...
     &                          +  0.300 * RXRAT( 711 )      ! ISOPNOOB+RO2C=0.3*HACET+...
     &                          +  0.600 * RXRAT( 712 )      ! ISOPNOOB+MECO3=0.6*HACET+...
     &                          +  0.040 * RXRAT( 713 )      ! ISOPNB+O3=0.04*HACET+...
     &                          +  0.480 * RXRAT( 735 )      ! DIBOO+NO=0.48*HACET+HO2+...
     &                          +  0.240 * RXRAT( 737 )      ! DIBOO+MEO2=0.24*HACET+...
     &                          +  0.240 * RXRAT( 738 )      ! DIBOO+RO2C=0.24*HACET+...
     &                          +  0.480 * RXRAT( 739 )      ! DIBOO+MECO3=0.48*HACET+...
     &                          +  0.720 * RXRAT( 747 )      ! MACROO+NO=0.72*HACET+...
     &                          +  0.424 * RXRAT( 749 )      ! MACROO+MEO2=0.424*HACET+...
     &                          +  0.424 * RXRAT( 750 )      ! MACROO+RO2C=0.424*HACET+...
     &                          +  0.850 * RXRAT( 751 )      ! MACROO+MECO3=0.85*HACET+...
     &                          +          RXRAT( 762 )      ! MAPAN+OH=HACET+CO+NO2
     &                          +  0.850 * RXRAT( 774 )      ! MACRN+OH=0.85*HACET+...
     &                          +          RXRAT( 775 )      ! MACRN=HACET+NO2+CO+HO2
     &                          +  0.500 * RXRAT( 776 )      ! DHMOB+OH=0.5*HACET+0.5*HO2+...
     &                          +  0.725 * RXRAT( 781 )      ! IEPOXOO+HO2=0.725*HACET+...
     &                          +  0.725 * RXRAT( 782 )      ! IEPOXOO+NO=0.725*HACET+...
     &                          +  0.363 * RXRAT( 783 )      ! IEPOXOO+MEO2=0.363*HACET+...
     &                          +  0.363 * RXRAT( 784 )      ! IEPOXOO+RO2C=0.363*HACET+...
     &                          +  0.725 * RXRAT( 785 )      ! IEPOXOO+MECO3=0.725*HACET+...
      LOSS( HACET            ) =           RXRAT( 764 )      ! HACET+OH=0.75*MGLY+...
     &                          +          RXRAT( 765 )      ! HACET=HO2+MECO3+HCHO

      PROD( NISOPO2          ) =           RXRAT( 685 )      ! ISOPRENE+NO3=NISOPO2
     &                          +  0.300 * RXRAT( 789 )      ! NISOPOOH+OH=0.3*NISOPO2+...
      LOSS( NISOPO2          ) =           RXRAT( 686 )      ! NISOPO2+NO3=0.7*NIT1+...
     &                          +          RXRAT( 687 )      ! NISOPO2+NO=0.7*NIT1+...
     &                          +          RXRAT( 688 )      ! NISOPO2+HO2=NISOPOOH
     &                          +          RXRAT( 689 )      ! NISOPO2+MEO2=0.35*NIT1+...
     &                          +          RXRAT( 690 )      ! NISOPO2+RO2C=0.35*NIT1+...
     &                          +  2.000 * RXRAT( 691 )      ! NISOPO2+NISOPO2=0.7*NIT1+...
     &                          +          RXRAT( 692 )      ! NISOPO2+MECO3=MEO2+CO2+...

      PROD( NIT1             ) =   0.700 * RXRAT( 686 )      ! NISOPO2+NO3=0.7*NIT1+...
     &                          +  0.700 * RXRAT( 687 )      ! NISOPO2+NO=0.7*NIT1+...
     &                          +  0.600 * RXRAT( 689 )      ! NISOPO2+MEO2=0.25*NIT1+...
     &                          +  0.600 * RXRAT( 690 )      ! NISOPO2+RO2C=0.25*NIT1+...
     &                          +  1.200 * RXRAT( 691 )      ! NISOPO2+NISOPO2=0.5*NIT1+...
     &                          +  0.700 * RXRAT( 692 )      ! NISOPO2+MECO3=0.7*NIT1+CO2+...
     &                          +  0.700 * RXRAT( 789 )      ! NISOPOOH+OH=0.7*NIT1+...
      LOSS( NIT1             ) =           RXRAT( 714 )      ! NIT1+NO3=0.6*NIT1NO3OOA+...
     &                          +          RXRAT( 728 )      ! NIT1+O3=0.3*PROPNNB+...
     &                          +          RXRAT( 729 )      ! NIT1+OH=0.345*NIT1NO3OOA+...

      PROD( NISOPOOH         ) =           RXRAT( 688 )      ! NISOPO2+HO2=NISOPOOH
      LOSS( NISOPOOH         ) =           RXRAT( 788 )      ! NISOPOOH+OH=RNO3I+OH
     &                          +          RXRAT( 789 )      ! NISOPOOH+OH=0.3*NISOPO2+...

      PROD( HC5OO            ) =           RXRAT( 693 )      ! HC5+OH=HC5OO
      LOSS( HC5OO            ) =           RXRAT( 694 )      ! HC5OO+NO=NO2+0.234*HOCCHO+...
     &                          +          RXRAT( 695 )      ! HC5OO+HO2=R6OOH
     &                          +          RXRAT( 696 )      ! HC5OO+MEO2=0.117*HOCCHO+...
     &                          +          RXRAT( 697 )      ! HC5OO+RO2C=0.117*HOCCHO+...
     &                          +          RXRAT( 698 )      ! HC5OO+MECO3=MEO2+CO2+...

      PROD( DHMOB            ) =   0.290 * RXRAT( 694 )      ! HC5OO+NO=0.29*DHMOB+...
     &                          +  0.145 * RXRAT( 696 )      ! HC5OO+MEO2=0.145*DHMOB+...
     &                          +  0.145 * RXRAT( 697 )      ! HC5OO+RO2C=0.145*DHMOB+...
     &                          +  0.290 * RXRAT( 698 )      ! HC5OO+MECO3=0.29*DHMOB+CO2+...
      LOSS( DHMOB            ) =           RXRAT( 776 )      ! DHMOB+OH=1.5*CO+0.5*HO2+...

      PROD( ISOPNOOD         ) =           RXRAT( 700 )      ! ISOPND+OH=ISOPNOOD
      LOSS( ISOPNOOD         ) =           RXRAT( 701 )      ! ISOPNOOD+NO=0.34*PRD2+...
     &                          +          RXRAT( 702 )      ! ISOPNOOD+HO2=RNO3I
     &                          +          RXRAT( 703 )      ! ISOPNOOD+MEO2=0.17*PRD2+...
     &                          +          RXRAT( 704 )      ! ISOPNOOD+RO2C=0.17*PRD2+...
     &                          +          RXRAT( 705 )      ! ISOPNOOD+MECO3=MEO2+CO2+...

      PROD( PROPNN           ) =   0.150 * RXRAT( 701 )      ! ISOPNOOD+NO=0.15*PROPNN+...
     &                          +  0.075 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.075*PROPNN+...
     &                          +  0.075 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.075*PROPNN+...
     &                          +  0.150 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.15*PROPNN+...
     &                          +  0.290 * RXRAT( 706 )      ! ISOPND+O3=0.29*PROPNN+...
      LOSS( PROPNN           ) =           RXRAT( 768 )      ! PROPNN+OH=MGLY+NO2
     &                          +          RXRAT( 770 )      ! PROPNN=MECO3+HCHO+NO2

      PROD( MVKN             ) =   0.070 * RXRAT( 701 )      ! ISOPNOOD+NO=0.07*MVKN+...
     &                          +  0.035 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.035*MVKN+...
     &                          +  0.035 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.035*MVKN+...
     &                          +  0.070 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.07*MVKN+...
     &                          +  0.140 * RXRAT( 708 )      ! ISOPNOOB+NO=0.14*MVKN+...
     &                          +  0.070 * RXRAT( 710 )      ! ISOPNOOB+MEO2=0.07*MVKN+...
     &                          +  0.070 * RXRAT( 711 )      ! ISOPNOOB+RO2C=0.07*MVKN+...
     &                          +  0.140 * RXRAT( 712 )      ! ISOPNOOB+MECO3=0.14*MVKN+...
     &                          +  0.120 * RXRAT( 713 )      ! ISOPNB+O3=0.12*MVKN+...
     &                          +  0.110 * RXRAT( 741 )      ! MVKOO+NO=0.11*MVKN+...
      LOSS( MVKN             ) =           RXRAT( 772 )      ! MVKN+OH=0.65*HCOOH+...
     &                          +          RXRAT( 773 )      ! MVKN=MECO3+NO2+HOCCHO

      PROD( ETHLN            ) =   0.130 * RXRAT( 701 )      ! ISOPNOOD+NO=0.13*ETHLN+...
     &                          +  0.065 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.065*ETHLN+...
     &                          +  0.065 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.065*ETHLN+...
     &                          +  0.130 * RXRAT( 705 )      ! ISOPNOOD+MECO3=0.13*ETHLN+...
     &                          +  0.360 * RXRAT( 706 )      ! ISOPND+O3=0.36*ETHLN+...
      LOSS( ETHLN            ) =           RXRAT( 766 )      ! ETHLN+OH=HCHO+CO2+NO2
     &                          +          RXRAT( 767 )      ! ETHLN=NO2+HCHO+HO2+CO

      PROD( RNO3I            ) =           RXRAT( 702 )      ! ISOPNOOD+HO2=RNO3I
     &                          +  0.500 * RXRAT( 703 )      ! ISOPNOOD+MEO2=0.5*RNO3I+...
     &                          +  0.500 * RXRAT( 704 )      ! ISOPNOOD+RO2C=0.5*RNO3I+...
     &                          +          RXRAT( 709 )      ! ISOPNOOB+HO2=RNO3I
     &                          +  0.500 * RXRAT( 710 )      ! ISOPNOOB+MEO2=0.5*RNO3I+...
     &                          +  0.500 * RXRAT( 711 )      ! ISOPNOOB+RO2C=0.5*RNO3I+...
     &                          +  0.280 * RXRAT( 713 )      ! ISOPNB+O3=0.28*RNO3I+...
     &                          +  0.060 * RXRAT( 723 )      ! NIT1NO3OOB+NO=0.06*RNO3I+...
     &                          +          RXRAT( 724 )      ! NIT1NO3OOB+HO2=RNO3I
     &                          +  0.300 * RXRAT( 725 )      ! NIT1NO3OOB+RO2C=0.3*RNO3I+...
     &                          +  0.300 * RXRAT( 726 )      ! NIT1NO3OOB+MEO2=0.3*RNO3I+...
     &                          +  0.081 * RXRAT( 730 )      ! NIT1OHOO+NO=0.015*RNO3I+...
     &                          +  0.311 * RXRAT( 732 )      ! NIT1OHOO+RO2C=0.011*RNO3I+...
     &                          +  0.311 * RXRAT( 733 )      ! NIT1OHOO+MEO2=0.011*RNO3I+...
     &                          +  0.016 * RXRAT( 734 )      ! NIT1OHOO+MECO3=0.016*RNO3I+...
     &                          +          RXRAT( 788 )      ! NISOPOOH+OH=RNO3I+OH
      LOSS( RNO3I            ) =           RXRAT( 787 )      ! RNO3I+OH=NO2+HO2+PRD2

      PROD( ISOPNOOB         ) =           RXRAT( 707 )      ! ISOPNB+OH=ISOPNOOB
      LOSS( ISOPNOOB         ) =           RXRAT( 708 )      ! ISOPNOOB+NO=0.6*HOCCHO+...
     &                          +          RXRAT( 709 )      ! ISOPNOOB+HO2=RNO3I
     &                          +          RXRAT( 710 )      ! ISOPNOOB+MEO2=0.3*HOCCHO+...
     &                          +          RXRAT( 711 )      ! ISOPNOOB+RO2C=0.3*HOCCHO+...
     &                          +          RXRAT( 712 )      ! ISOPNOOB+MECO3=MEO2+CO2+...

      PROD( MACRN            ) =   0.260 * RXRAT( 708 )      ! ISOPNOOB+NO=0.26*MACRN+...
     &                          +  0.130 * RXRAT( 710 )      ! ISOPNOOB+MEO2=0.13*MACRN+...
     &                          +  0.130 * RXRAT( 711 )      ! ISOPNOOB+RO2C=0.13*MACRN+...
     &                          +  0.260 * RXRAT( 712 )      ! ISOPNOOB+MECO3=0.26*MACRN+...
     &                          +  0.320 * RXRAT( 713 )      ! ISOPNB+O3=0.32*MACRN+...
     &                          +  0.150 * RXRAT( 747 )      ! MACROO+NO=0.15*MACRN+...
      LOSS( MACRN            ) =           RXRAT( 774 )      ! MACRN+OH=0.08*CCOOH+...
     &                          +          RXRAT( 775 )      ! MACRN=HACET+NO2+CO+HO2

      PROD( NIT1NO3OOA       ) =   0.600 * RXRAT( 714 )      ! NIT1+NO3=0.6*NIT1NO3OOA+...
     &                          +  0.345 * RXRAT( 729 )      ! NIT1+OH=0.345*NIT1NO3OOA+...
      LOSS( NIT1NO3OOA       ) =           RXRAT( 715 )      ! NIT1NO3OOA+NO3=NO2+PROPNNB+...
     &                          +          RXRAT( 716 )      ! NIT1NO3OOA+NO=NO2+PROPNNB+...
     &                          +          RXRAT( 717 )      ! NIT1NO3OOA+NO2=MAPAN
     &                          +          RXRAT( 718 )      ! NIT1NO3OOA+HO2=0.75*RCOOOH+...
     &                          +          RXRAT( 719 )      ! NIT1NO3OOA+RO2C=PROPNNB+CO+...
     &                          +          RXRAT( 720 )      ! NIT1NO3OOA+MEO2=PROPNNB+CO+...
     &                          +          RXRAT( 721 )      ! NIT1NO3OOA+MECO3=MEO2+CO2+...

      PROD( NIT1NO3OOB       ) =   0.400 * RXRAT( 714 )      ! NIT1+NO3=0.4*NIT1NO3OOB+...
      LOSS( NIT1NO3OOB       ) =           RXRAT( 722 )      ! NIT1NO3OOB+NO3=PROPNNB+GLY+...
     &                          +          RXRAT( 723 )      ! NIT1NO3OOB+NO=0.94*PROPNNB+...
     &                          +          RXRAT( 724 )      ! NIT1NO3OOB+HO2=RNO3I
     &                          +          RXRAT( 725 )      ! NIT1NO3OOB+RO2C=...
     &                          +          RXRAT( 726 )      ! NIT1NO3OOB+MEO2=...
     &                          +          RXRAT( 727 )      ! NIT1NO3OOB+MECO3=MEO2+CO2+...

      PROD( PROPNNB          ) =           RXRAT( 715 )      ! NIT1NO3OOA+NO3=PROPNNB+NO2+...
     &                          +          RXRAT( 716 )      ! NIT1NO3OOA+NO=PROPNNB+NO2+...
     &                          +          RXRAT( 719 )      ! NIT1NO3OOA+RO2C=PROPNNB+CO+...
     &                          +          RXRAT( 720 )      ! NIT1NO3OOA+MEO2=PROPNNB+CO+...
     &                          +          RXRAT( 721 )      ! NIT1NO3OOA+MECO3=PROPNNB+...
     &                          +          RXRAT( 722 )      ! NIT1NO3OOB+NO3=PROPNNB+GLY+...
     &                          +  0.940 * RXRAT( 723 )      ! NIT1NO3OOB+NO=0.94*PROPNNB+...
     &                          +  0.700 * RXRAT( 725 )      ! NIT1NO3OOB+RO2C=...
     &                          +  0.700 * RXRAT( 726 )      ! NIT1NO3OOB+MEO2=...
     &                          +          RXRAT( 727 )      ! NIT1NO3OOB+MECO3=PROPNNB+...
     &                          +  0.300 * RXRAT( 728 )      ! NIT1+O3=0.3*PROPNNB+...
     &                          +  0.919 * RXRAT( 730 )      ! NIT1OHOO+NO=0.919*PROPNNB+...
     &                          +  0.689 * RXRAT( 732 )      ! NIT1OHOO+RO2C=...
     &                          +  0.689 * RXRAT( 733 )      ! NIT1OHOO+MEO2=...
     &                          +  0.984 * RXRAT( 734 )      ! NIT1OHOO+MECO3=...
      LOSS( PROPNNB          ) =           RXRAT( 769 )      ! PROPNNB+OH=MGLY+NO2
     &                          +          RXRAT( 771 )      ! PROPNNB=MECO3+HCHO+NO2

      PROD( NIT1OHOO         ) =   0.655 * RXRAT( 729 )      ! NIT1+OH=0.655*NIT1OHOO+...
      LOSS( NIT1OHOO         ) =           RXRAT( 730 )      ! NIT1OHOO+NO=0.919*PROPNNB+...
     &                          +          RXRAT( 731 )      ! NIT1OHOO+HO2=R6OOH
     &                          +          RXRAT( 732 )      ! NIT1OHOO+RO2C=...
     &                          +          RXRAT( 733 )      ! NIT1OHOO+MEO2=...
     &                          +          RXRAT( 734 )      ! NIT1OHOO+MECO3=MEO2+CO2+...

      PROD( MVKOO            ) =           RXRAT( 740 )      ! MVK+OH=MVKOO
      LOSS( MVKOO            ) =           RXRAT( 741 )      ! MVKOO+NO=0.625*HOCCHO+...
     &                          +          RXRAT( 742 )      ! MVKOO+HO2=ROOH
     &                          +          RXRAT( 743 )      ! MVKOO+MEO2=0.35*HOCCHO+...
     &                          +          RXRAT( 744 )      ! MVKOO+RO2C=0.35*HOCCHO+...
     &                          +          RXRAT( 745 )      ! MVKOO+MECO3=MEO2+CO2+...

      PROD( MACROO           ) =   0.470 * RXRAT( 746 )      ! MACR+OH=0.47*MACROO+0.53*MACO3
      LOSS( MACROO           ) =           RXRAT( 747 )      ! MACROO+NO=0.85*NO2+...
     &                          +          RXRAT( 748 )      ! MACROO+HO2=ROOH
     &                          +          RXRAT( 749 )      ! MACROO+MEO2=0.5*HO2+...
     &                          +          RXRAT( 750 )      ! MACROO+RO2C=0.5*HO2+...
     &                          +          RXRAT( 751 )      ! MACROO+MECO3=MEO2+CO2+HO2+...

      PROD( PYRUACD          ) =   0.350 * RXRAT( 772 )      ! MVKN+OH=0.35*PYRUACD+...
      LOSS( PYRUACD          ) =           RXRAT( 777 )      ! PYRUACD=CCHO+CO2

      PROD( IEPOX            ) =           RXRAT( 778 )      ! ISOPOOH+OH=IEPOX+OH
      LOSS( IEPOX            ) =           RXRAT( 780 )      ! IEPOX+OH=IEPOXOO

      PROD( IEPOXOO          ) =           RXRAT( 780 )      ! IEPOX+OH=IEPOXOO
      LOSS( IEPOXOO          ) =           RXRAT( 781 )      ! IEPOXOO+HO2=0.725*HACET+...
     &                          +          RXRAT( 782 )      ! IEPOXOO+NO=0.725*HACET+...
     &                          +          RXRAT( 783 )      ! IEPOXOO+MEO2=0.363*HACET+...
     &                          +          RXRAT( 784 )      ! IEPOXOO+RO2C=0.363*HACET+...
     &                          +          RXRAT( 785 )      ! IEPOXOO+MECO3=0.725*HACET+...

      RETURN

      END
