
!------------------------------------------------------------------------!
!  The Community Multiscale Air Quality (CMAQ) system software is in     !
!  continuous development by various groups and is based on information  !
!  from these groups: Federal Government employees, contractors working  !
!  within a United States Government contract, and non-Federal sources   !
!  including research institutions.  These groups give the Government    !
!  permission to use, prepare derivative works of, and distribute copies !
!  of their work in the CMAQ system to the public and to permit others   !
!  to do so.  The United States Environmental Protection Agency          !
!  therefore grants similar permission to use the CMAQ system software,  !
!  but users are requested to provide copies of derivative works or      !
!  products designed to operate in the CMAQ system to the United States  !
!  Government without restrictions as to use by others.  Software        !
!  that is used with the CMAQ system but distributed under the GNU       !
!  General Public License or the GNU Lesser General Public License is    !
!  subject to their copyright restrictions.                              !
!------------------------------------------------------------------------!

C RCS file, release, date & time of last delta, author, state, [and locker]
C $Header$

C what(1) key, module and SID; SCCS file; date and time of last delta:
C %W% %P% %G% %U%

      SUBROUTINE HRPRODLOSS

C***********************************************************************
C
C  FUNCTION: To compute the production and loss rates for EBI species
C
C  PRECONDITIONS: For the SAPRC07TB family of mechanisms
C
C  RETURN VALUES: None
C
C  KEY SUBROUTINES/FUNCTIONS CALLED: None
C
C  REVISION HISTORY: Created by EBI solver program, June 1, 2012
C
C***********************************************************************
      USE HRDATA

      IMPLICIT NONE


C.....INCLUDES: NONE


C.....ARGUMENTS: NONE


C.....PARAMETERS: NONE


C.....EXTERNAL FUNCTIONS: NONE


C.....SAVED VARIABLES: NONE
      CHARACTER( 16 ), SAVE  ::  PNAME = 'HRPRODLOSS'   ! Program name


C.....LOCAL VARIABLES:

      INTEGER ESP

C***********************************************************************


      PROD( HNO3             ) =   2.000 * RXRAT(  13 )      ! N2O5=2*HNO3
     &                          +  2.000 * RXRAT(  14 )      ! N2O5=2*HNO3
     &                          +          RXRAT(  25 )      ! OH+NO2=HNO3
     &                          +  0.200 * RXRAT(  39 )      ! NO3+HO2=0.2*HNO3+0.8*NO2+...
     &                          +          RXRAT( 139 )      ! HCHO+NO3=HNO3+HO2+CO
     &                          +          RXRAT( 142 )      ! CCHO+NO3=HNO3+MECO3
     &                          +          RXRAT( 145 )      ! RCHO+NO3=HNO3+RCO3
     &                          +          RXRAT( 165 )      ! GLY+NO3=HNO3+0.63*HO2+...
     &                          +          RXRAT( 168 )      ! MGLY+NO3=HNO3+CO+MECO3
     &                          +          RXRAT( 171 )      ! CRES+NO3=HNO3+BZO
     &                          +          RXRAT( 177 )      ! BALD+NO3=HNO3+BZCO3
     &                          +  0.500 * RXRAT( 188 )      ! MACR+NO3=0.5*HNO3+0.5*RO2C+...
     &                          +  0.150 * RXRAT( 197 )      ! IPRD+NO3=0.15*HNO3+...
     &                          +          RXRAT( 205 )      ! HOCCHO+NO3=HNO3+MECO3
     &                          +  0.967 * RXRAT( 208 )      ! ACROLEIN+NO3=0.967*HNO3+...
      LOSS( HNO3             ) =           RXRAT(  27 )      ! HNO3+OH=NO3
     &                          +          RXRAT(  28 )      ! HNO3=OH+NO2

      PROD( CO               ) =           RXRAT( 134 )      ! xCO=CO
     &                          +          RXRAT( 136 )      ! HCHO=CO+2*HO2
     &                          +          RXRAT( 137 )      ! HCHO=CO
     &                          +          RXRAT( 138 )      ! HCHO+OH=CO+HO2
     &                          +          RXRAT( 139 )      ! HCHO+NO3=CO+HO2+HNO3
     &                          +          RXRAT( 141 )      ! CCHO=CO+HO2+MEO2
     &                          +          RXRAT( 144 )      ! RCHO=CO+xHO2+yROOH+xCCHO+...
     &                          +  0.380 * RXRAT( 147 )      ! ACETONE=0.38*CO+1.38*MEO2+...
     &                          +  2.000 * RXRAT( 162 )      ! GLY=2*CO+2*HO2
     &                          +          RXRAT( 163 )      ! GLY=CO+HCHO
     &                          +  1.260 * RXRAT( 164 )      ! GLY+OH=1.26*CO+0.63*HO2+...
     &                          +  1.260 * RXRAT( 165 )      ! GLY+NO3=1.26*CO+0.63*HO2+...
     &                          +          RXRAT( 166 )      ! MGLY=CO+HO2+MECO3
     &                          +          RXRAT( 167 )      ! MGLY+OH=CO+MECO3
     &                          +          RXRAT( 168 )      ! MGLY+NO3=CO+HNO3+MECO3
     &                          +  0.522 * RXRAT( 179 )      ! AFG1+O3=0.522*CO+0.522*HO2+...
     &                          +  0.695 * RXRAT( 180 )      ! AFG1=0.695*CO+0.173*MEO2+...
     &                          +  0.522 * RXRAT( 182 )      ! AFG2+O3=0.522*CO+0.522*HO2+...
     &                          +  0.580 * RXRAT( 185 )      ! AFG3+O3=0.58*CO+0.554*HO2+...
     &                          +  0.450 * RXRAT( 187 )      ! MACR+O3=0.45*CO+0.108*HO2+...
     &                          +  0.670 * RXRAT( 190 )      ! MACR=0.67*CO+0.67*HO2+...
     &                          +  0.475 * RXRAT( 192 )      ! MVK+O3=0.475*CO+0.064*HO2+...
     &                          +  0.600 * RXRAT( 194 )      ! MVK=0.6*CO+0.4*MEO2+...
     &                          +  0.498 * RXRAT( 196 )      ! IPRD+O3=0.498*CO+0.4*HO2+...
     &                          +  1.233 * RXRAT( 198 )      ! IPRD=1.233*CO+0.467*MECO3+...
     &                          +          RXRAT( 204 )      ! HOCCHO=CO+2*HO2+HCHO
     &                          +  1.005 * RXRAT( 207 )      ! ACROLEIN+O3=1.005*CO+...
     &                          +  1.188 * RXRAT( 210 )      ! ACROLEIN=1.188*CO+0.178*OH+...
     &                          +  0.510 * RXRAT( 267 )      ! ETHENE+O3=0.51*CO+0.16*OH+...
     &                          +  0.510 * RXRAT( 269 )      ! ETHENE+O3P=0.51*CO+...
     &                          +  0.525 * RXRAT( 271 )      ! PROPENE+O3=0.525*CO+...
     &                          +  0.255 * RXRAT( 275 )      ! BUTADIENE13+O3=0.255*CO+...
     &                          +  0.275 * RXRAT( 279 )      ! ISOPRENE+O3=0.275*CO+...
     &                          +  0.029 * RXRAT( 283 )      ! APIN+O3=0.029*CO+...
     &                          +  0.300 * RXRAT( 286 )      ! ACETYLENE+OH=0.3*CO+0.7*OH+...
     &                          +  1.500 * RXRAT( 287 )      ! ACETYLENE+O3=1.5*CO+0.5*OH+...
     &                          +  0.303 * RXRAT( 301 )      ! OLE1+O3=0.303*CO+...
     &                          +  0.299 * RXRAT( 305 )      ! OLE2+O3=0.299*CO+...
     &                          +  0.249 * RXRAT( 311 )      ! TERP+O3=0.249*CO+...
     &                          +  0.249 * RXRAT( 315 )      ! SESQ+O3=0.249*CO+...
     &                          +          RXRAT( 340 )      ! HCHO+CL=CO+HO2+HCL
     &                          +  1.260 * RXRAT( 348 )      ! GLY+CL=1.26*CO+0.63*HO2+...
     &                          +          RXRAT( 349 )      ! MGLY+CL=CO+HCL+MECO3
     &                          +          RXRAT( 359 )      ! CLCCHO=CO+HO2+RO2C+xCL+...
     &                          +          RXRAT( 375 )      ! ACETYLENE+CL=CO+HO2
      LOSS( CO               ) =           RXRAT(  29 )      ! CO+OH=HO2+CO2

      PROD( CO2              ) =           RXRAT(  29 )      ! OH+CO=CO2+HO2
     &                          +  0.400 * RXRAT(  69 )      ! PAN=0.4*CO2+0.6*NO2+...
     &                          +          RXRAT(  70 )      ! MECO3+NO=CO2+MEO2+NO2
     &                          +          RXRAT(  72 )      ! MECO3+NO3=CO2+MEO2+NO2
     &                          +  0.900 * RXRAT(  73 )      ! MECO3+MEO2=0.9*CO2+...
     &                          +          RXRAT(  74 )      ! MECO3+RO2C=CO2+MEO2
     &                          +          RXRAT(  75 )      ! MECO3+RO2XC=CO2+MEO2
     &                          +  2.000 * RXRAT(  76 )      ! MECO3+MECO3=2*CO2+2*MEO2
     &                          +  0.400 * RXRAT(  79 )      ! PAN2=0.4*CO2+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=CO2+RO2C+xHO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=CO2+RO2C+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=CO2+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=CO2+xHO2+xCCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=CO2+xHO2+xCCHO+...
     &                          +  2.000 * RXRAT(  86 )      ! RCO3+MECO3=2*CO2+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*CO2+2*xHO2+...
     &                          +  0.400 * RXRAT(  90 )      ! PBZN=0.4*CO2+0.6*NO2+...
     &                          +          RXRAT(  91 )      ! BZCO3+NO=CO2+NO2+BZO+RO2C
     &                          +          RXRAT(  93 )      ! BZCO3+NO3=CO2+NO2+BZO+RO2C
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=CO2+HO2+RO2C+...
     &                          +          RXRAT(  95 )      ! BZCO3+RO2C=CO2+BZO+RO2C
     &                          +          RXRAT(  96 )      ! BZCO3+RO2XC=CO2+BZO+RO2C
     &                          +  2.000 * RXRAT(  97 )      ! BZCO3+MECO3=2*CO2+MEO2+BZO+...
     &                          +  2.000 * RXRAT(  98 )      ! BZCO3+RCO3=2*CO2+RO2C+xHO2+...
     &                          +  2.000 * RXRAT(  99 )      ! BZCO3+BZCO3=2*CO2+2*RO2C+2*BZO
     &                          +  0.400 * RXRAT( 102 )      ! MAPAN=0.4*CO2+0.6*NO2+...
     &                          +          RXRAT( 103 )      ! MACO3+NO=CO2+NO2+HCHO+MECO3
     &                          +          RXRAT( 105 )      ! MACO3+NO3=CO2+NO2+HCHO+MECO3
     &                          +          RXRAT( 106 )      ! MACO3+MEO2=CO2+HO2+2*HCHO+...
     &                          +          RXRAT( 107 )      ! MACO3+RO2C=CO2+HCHO+MECO3
     &                          +          RXRAT( 108 )      ! MACO3+RO2XC=CO2+HCHO+MECO3
     &                          +  2.000 * RXRAT( 109 )      ! MACO3+MECO3=2*CO2+MEO2+...
     &                          +  2.000 * RXRAT( 110 )      ! MACO3+RCO3=2*CO2+MECO3+...
     &                          +  2.000 * RXRAT( 111 )      ! MACO3+BZCO3=2*CO2+MECO3+...
     &                          +  2.000 * RXRAT( 112 )      ! MACO3+MACO3=2*CO2+2*MECO3+...
     &                          +          RXRAT( 151 )      ! HCOOH+OH=CO2+HO2
     &                          +  0.509 * RXRAT( 152 )      ! CCOOH+OH=0.509*CO2+...
     &                          +  0.143 * RXRAT( 153 )      ! RCOOH+OH=0.143*CO2+xHO2+...
     &                          +  0.174 * RXRAT( 179 )      ! AFG1+O3=0.174*CO2+...
     &                          +  0.174 * RXRAT( 182 )      ! AFG2+O3=0.174*CO2+...
     &                          +  0.190 * RXRAT( 185 )      ! AFG3+O3=0.19*CO2+0.554*HO2+...
     &                          +  0.117 * RXRAT( 187 )      ! MACR+O3=0.117*CO2+...
     &                          +  0.124 * RXRAT( 192 )      ! MVK+O3=0.124*CO2+0.064*HO2+...
     &                          +  0.140 * RXRAT( 196 )      ! IPRD+O3=0.14*CO2+0.4*HO2+...
     &                          +  0.310 * RXRAT( 207 )      ! ACROLEIN+O3=0.31*CO2+...
     &                          +  0.102 * RXRAT( 210 )      ! ACROLEIN=0.102*CO2+...
     &                          +  0.020 * RXRAT( 211 )      ! CCOOOH+OH=0.02*CO2+...
     &                          +          RXRAT( 212 )      ! CCOOOH=CO2+MEO2+OH
     &                          +  0.110 * RXRAT( 213 )      ! RCOOOH+OH=0.11*CO2+...
     &                          +          RXRAT( 214 )      ! RCOOOH=CO2+xCCHO+yROOH+xHO2+OH
     &                          +  0.120 * RXRAT( 267 )      ! ETHENE+O3=0.12*CO2+0.16*OH+...
     &                          +  0.215 * RXRAT( 271 )      ! PROPENE+O3=0.215*CO2+...
     &                          +  0.185 * RXRAT( 275 )      ! BUTADIENE13+O3=0.185*CO2+...
     &                          +  0.122 * RXRAT( 279 )      ! ISOPRENE+O3=0.122*CO2+...
     &                          +  0.017 * RXRAT( 283 )      ! APIN+O3=0.017*CO2+...
     &                          +  0.500 * RXRAT( 287 )      ! ACETYLENE+O3=0.5*CO2+...
     &                          +  0.088 * RXRAT( 301 )      ! OLE1+O3=0.088*CO2+...
     &                          +  0.161 * RXRAT( 305 )      ! OLE2+O3=0.161*CO2+...
     &                          +  0.063 * RXRAT( 311 )      ! TERP+O3=0.063*CO2+...
     &                          +  0.063 * RXRAT( 315 )      ! SESQ+O3=0.063*CO2+...
      LOSS( CO2              ) =   0.0

      PROD( HO2H             ) =           RXRAT(  37 )      ! HO2+HO2=HO2H
     &                          +          RXRAT(  38 )      ! HO2+HO2=HO2H
      LOSS( HO2H             ) =           RXRAT(  41 )      ! HO2H=2*OH
     &                          +          RXRAT(  42 )      ! HO2H+OH=HO2

      PROD( SO2              ) =   0.0
      LOSS( SO2              ) =           RXRAT(  44 )      ! SO2+OH=HO2+SULF+SULRXN

      PROD( SULF             ) =           RXRAT(  44 )      ! OH+SO2=SULF+HO2+SULRXN
      LOSS( SULF             ) =   0.0

      PROD( SULRXN           ) =           RXRAT(  44 )      ! OH+SO2=SULRXN+SULF+HO2
      LOSS( SULRXN           ) =   0.0

      PROD( NO2EX            ) =           RXRAT(  46 )      ! NO2=NO2EX
      LOSS( NO2EX            ) =           RXRAT(  47 )      ! NO2EX=NO2
     &                          +          RXRAT(  48 )      ! NO2EX=NO2
     &                          +          RXRAT(  49 )      ! NO2EX=OH+HONO

      PROD( MEO2             ) =   0.400 * RXRAT(  69 )      ! PAN=0.4*MEO2+0.6*NO2+...
     &                          +          RXRAT(  70 )      ! MECO3+NO=MEO2+CO2+NO2
     &                          +          RXRAT(  72 )      ! MECO3+NO3=MEO2+CO2+NO2
     &                          +          RXRAT(  74 )      ! MECO3+RO2C=MEO2+CO2
     &                          +          RXRAT(  75 )      ! MECO3+RO2XC=MEO2+CO2
     &                          +  2.000 * RXRAT(  76 )      ! MECO3+MECO3=2*MEO2+2*CO2
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=MEO2+2*CO2+RO2C+...
     &                          +          RXRAT(  97 )      ! BZCO3+MECO3=MEO2+2*CO2+BZO+...
     &                          +          RXRAT( 109 )      ! MACO3+MECO3=MEO2+2*CO2+...
     &                          +          RXRAT( 114 )      ! TBUO=MEO2+ACETONE
     &                          +          RXRAT( 124 )      ! xMEO2=MEO2
     &                          +          RXRAT( 141 )      ! CCHO=MEO2+HO2+CO
     &                          +  1.380 * RXRAT( 147 )      ! ACETONE=1.38*MEO2+...
     &                          +  0.509 * RXRAT( 152 )      ! CCOOH+OH=0.509*MEO2+...
     &                          +  0.700 * RXRAT( 154 )      ! COOH+OH=0.7*MEO2+0.3*OH+...
     &                          +  0.173 * RXRAT( 180 )      ! AFG1=0.173*MEO2+1.023*HO2+...
     &                          +  0.400 * RXRAT( 194 )      ! MVK=0.4*MEO2+0.6*CO+...
     &                          +  0.234 * RXRAT( 210 )      ! ACROLEIN=0.234*MEO2+...
     &                          +          RXRAT( 212 )      ! CCOOOH=MEO2+CO2+OH
     &                          +          RXRAT( 265 )      ! OH=MEO2
     &                          +  0.510 * RXRAT( 269 )      ! ETHENE+O3P=0.51*MEO2+...
     &                          +  0.355 * RXRAT( 271 )      ! PROPENE+O3=0.355*MEO2+...
     &                          +  0.250 * RXRAT( 281 )      ! ISOPRENE+O3P=0.25*MEO2+...
     &                          +  0.307 * RXRAT( 305 )      ! OLE2+O3=0.307*MEO2+...
     &                          +          RXRAT( 369 )      ! CL=MEO2+HCL
      LOSS( MEO2             ) =           RXRAT(  50 )      ! MEO2+NO=NO2+HCHO+HO2
     &                          +          RXRAT(  51 )      ! MEO2+HO2=COOH
     &                          +          RXRAT(  52 )      ! MEO2+HO2=HCHO
     &                          +          RXRAT(  53 )      ! MEO2+NO3=HCHO+HO2+NO2
     &                          +  2.000 * RXRAT(  54 )      ! MEO2+MEO2=MEOH+HCHO
     &                          +  2.000 * RXRAT(  55 )      ! MEO2+MEO2=2*HCHO+2*HO2
     &                          +          RXRAT(  59 )      ! MEO2+RO2C=0.5*HO2+...
     &                          +          RXRAT(  64 )      ! MEO2+RO2XC=0.5*HO2+...
     &                          +  0.100 * RXRAT(  73 )      ! MEO2+MECO3=0.9*MEO2+...
     &                          +          RXRAT(  83 )      ! MEO2+RCO3=HCHO+HO2+RO2C+...
     &                          +          RXRAT(  94 )      ! MEO2+BZCO3=HCHO+HO2+RO2C+...
     &                          +          RXRAT( 106 )      ! MEO2+MACO3=2*HCHO+HO2+CO2+...

      PROD( HCHO             ) =           RXRAT(  50 )      ! MEO2+NO=HCHO+NO2+HO2
     &                          +          RXRAT(  52 )      ! MEO2+HO2=HCHO
     &                          +          RXRAT(  53 )      ! MEO2+NO3=HCHO+HO2+NO2
     &                          +          RXRAT(  54 )      ! MEO2+MEO2=HCHO+MEOH
     &                          +  2.000 * RXRAT(  55 )      ! MEO2+MEO2=2*HCHO+2*HO2
     &                          +  0.750 * RXRAT(  59 )      ! RO2C+MEO2=0.75*HCHO+...
     &                          +  0.750 * RXRAT(  64 )      ! RO2XC+MEO2=0.75*HCHO+...
     &                          +          RXRAT(  73 )      ! MECO3+MEO2=0.1*HCHO+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=HCHO+HO2+RO2C+...
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=HCHO+HO2+RO2C+...
     &                          +  0.400 * RXRAT( 102 )      ! MAPAN=0.4*HCHO+0.6*NO2+...
     &                          +          RXRAT( 103 )      ! MACO3+NO=HCHO+CO2+NO2+MECO3
     &                          +          RXRAT( 105 )      ! MACO3+NO3=HCHO+CO2+NO2+MECO3
     &                          +  2.000 * RXRAT( 106 )      ! MACO3+MEO2=2*HCHO+HO2+CO2+...
     &                          +          RXRAT( 107 )      ! MACO3+RO2C=HCHO+CO2+MECO3
     &                          +          RXRAT( 108 )      ! MACO3+RO2XC=HCHO+CO2+MECO3
     &                          +          RXRAT( 109 )      ! MACO3+MECO3=HCHO+MEO2+...
     &                          +          RXRAT( 110 )      ! MACO3+RCO3=HCHO+MECO3+RO2C+...
     &                          +          RXRAT( 111 )      ! MACO3+BZCO3=HCHO+MECO3+BZO+...
     &                          +  2.000 * RXRAT( 112 )      ! MACO3+MACO3=2*HCHO+2*MECO3+...
     &                          +          RXRAT( 150 )      ! MEOH+OH=HCHO+HO2
     &                          +  0.300 * RXRAT( 154 )      ! COOH+OH=0.3*HCHO+0.3*OH+...
     &                          +          RXRAT( 155 )      ! COOH=HCHO+HO2+OH
     &                          +          RXRAT( 163 )      ! GLY=HCHO+CO
     &                          +  0.100 * RXRAT( 187 )      ! MACR+O3=0.1*HCHO+0.108*HO2+...
     &                          +  0.340 * RXRAT( 190 )      ! MACR=0.34*HCHO+0.67*HO2+...
     &                          +  0.050 * RXRAT( 192 )      ! MVK+O3=0.05*HCHO+0.064*HO2+...
     &                          +  0.124 * RXRAT( 196 )      ! IPRD+O3=0.124*HCHO+0.4*HO2+...
     &                          +  0.300 * RXRAT( 198 )      ! IPRD=0.3*HCHO+0.467*MECO3+...
     &                          +  0.002 * RXRAT( 199 )      ! PRD2+OH=0.002*HCHO+...
     &                          +  0.074 * RXRAT( 202 )      ! RNO3=0.074*HCHO+0.554*xHO2+...
     &                          +          RXRAT( 204 )      ! HOCCHO=HCHO+2*HO2+CO
     &                          +  0.500 * RXRAT( 207 )      ! ACROLEIN+O3=0.5*HCHO+...
     &                          +  0.340 * RXRAT( 210 )      ! ACROLEIN=0.34*HCHO+...
     &                          +          RXRAT( 215 )      ! xHCHO=HCHO
     &                          +          RXRAT( 267 )      ! ETHENE+O3=HCHO+0.16*OH+...
     &                          +  0.500 * RXRAT( 271 )      ! PROPENE+O3=0.5*HCHO+...
     &                          +  0.500 * RXRAT( 275 )      ! BUTADIENE13+O3=0.5*HCHO+...
     &                          +  0.400 * RXRAT( 279 )      ! ISOPRENE+O3=0.4*HCHO+...
     &                          +  0.500 * RXRAT( 301 )      ! OLE1+O3=0.5*HCHO+...
     &                          +  0.131 * RXRAT( 305 )      ! OLE2+O3=0.131*HCHO+...
     &                          +  0.127 * RXRAT( 311 )      ! TERP+O3=0.127*HCHO+...
     &                          +  0.127 * RXRAT( 315 )      ! SESQ+O3=0.127*HCHO+...
     &                          +          RXRAT( 342 )      ! MEOH+CL=HCHO+HCL+HO2
      LOSS( HCHO             ) =           RXRAT( 136 )      ! HCHO=2*HO2+CO
     &                          +          RXRAT( 137 )      ! HCHO=CO
     &                          +          RXRAT( 138 )      ! HCHO+OH=HO2+CO
     &                          +          RXRAT( 139 )      ! HCHO+NO3=HNO3+HO2+CO
     &                          +          RXRAT( 340 )      ! HCHO+CL=HCL+HO2+CO

      PROD( COOH             ) =           RXRAT(  51 )      ! MEO2+HO2=COOH
      LOSS( COOH             ) =           RXRAT( 154 )      ! COOH+OH=0.3*HCHO+0.3*OH+...
     &                          +          RXRAT( 155 )      ! COOH=HCHO+HO2+OH

      PROD( MEOH             ) =           RXRAT(  54 )      ! MEO2+MEO2=MEOH+HCHO
     &                          +  0.250 * RXRAT(  59 )      ! RO2C+MEO2=0.25*MEOH+...
     &                          +  0.250 * RXRAT(  64 )      ! RO2XC+MEO2=0.25*MEOH+...
      LOSS( MEOH             ) =           RXRAT( 150 )      ! MEOH+OH=HCHO+HO2
     &                          +          RXRAT( 342 )      ! MEOH+CL=HCL+HCHO+HO2

      PROD( RO2C             ) =   0.400 * RXRAT(  79 )      ! PAN2=0.4*RO2C+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=RO2C+NO2+xHO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=RO2C+NO2+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=RO2C+HO2+HCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=RO2C+xHO2+xCCHO+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=RO2C+MEO2+2*CO2+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*RO2C+2*xHO2+...
     &                          +  0.400 * RXRAT(  90 )      ! PBZN=0.4*RO2C+0.6*NO2+...
     &                          +          RXRAT(  91 )      ! BZCO3+NO=RO2C+CO2+BZO+NO2
     &                          +          RXRAT(  93 )      ! BZCO3+NO3=RO2C+CO2+BZO+NO2
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=RO2C+HO2+HCHO+...
     &                          +          RXRAT(  96 )      ! BZCO3+RO2XC=RO2C+BZO+CO2
     &                          +          RXRAT(  97 )      ! BZCO3+MECO3=RO2C+MEO2+BZO+...
     &                          +  2.000 * RXRAT(  98 )      ! BZCO3+RCO3=RO2C+2*CO2+xHO2+...
     &                          +  2.000 * RXRAT(  99 )      ! BZCO3+BZCO3=2*RO2C+2*BZO+2*CO2
     &                          +          RXRAT( 110 )      ! MACO3+RCO3=RO2C+MECO3+HCHO+...
     &                          +          RXRAT( 111 )      ! MACO3+BZCO3=RO2C+MECO3+BZO+...
     &                          +          RXRAT( 117 )      ! BZO=RO2C+CRES+xHO2
     &                          +  0.035 * RXRAT( 143 )      ! RCHO+OH=0.035*RO2C+...
     &                          +          RXRAT( 144 )      ! RCHO=RO2C+xHO2+yROOH+xCCHO+...
     &                          +          RXRAT( 146 )      ! ACETONE+OH=RO2C+xMECO3+...
     &                          +  0.967 * RXRAT( 148 )      ! MEK+OH=0.967*RO2C+...
     &                          +          RXRAT( 149 )      ! MEK=RO2C+MECO3+xHO2+xCCHO+...
     &                          +  0.491 * RXRAT( 152 )      ! CCOOH+OH=0.491*RO2C+...
     &                          +          RXRAT( 153 )      ! RCOOH+OH=RO2C+xHO2+...
     &                          +  0.251 * RXRAT( 156 )      ! ROOH+OH=0.251*RO2C+...
     &                          +  0.222 * RXRAT( 158 )      ! R6OOH+OH=0.222*RO2C+...
     &                          +  0.782 * RXRAT( 159 )      ! R6OOH=0.782*RO2C+0.142*HO2+...
     &                          +  0.589 * RXRAT( 160 )      ! RAOOH+OH=0.589*RO2C+...
     &                          +  0.800 * RXRAT( 170 )      ! CRES+OH=0.8*RO2C+0.2*BZO+...
     &                          +  0.723 * RXRAT( 178 )      ! AFG1+OH=0.723*RO2C+...
     &                          +  0.652 * RXRAT( 179 )      ! AFG1+O3=0.652*RO2C+...
     &                          +  0.723 * RXRAT( 181 )      ! AFG2+OH=0.723*RO2C+...
     &                          +  0.652 * RXRAT( 182 )      ! AFG2+O3=0.652*RO2C+...
     &                          +  0.733 * RXRAT( 184 )      ! AFG3+OH=0.733*RO2C+...
     &                          +  0.258 * RXRAT( 185 )      ! AFG3+O3=0.258*RO2C+...
     &                          +  0.500 * RXRAT( 186 )      ! MACR+OH=0.5*RO2C+0.5*MACO3+...
     &                          +  0.100 * RXRAT( 187 )      ! MACR+O3=0.1*RO2C+0.108*HO2+...
     &                          +  0.500 * RXRAT( 188 )      ! MACR+NO3=0.5*RO2C+...
     &                          +  0.330 * RXRAT( 190 )      ! MACR=0.33*RO2C+0.67*HO2+...
     &                          +  0.975 * RXRAT( 191 )      ! MVK+OH=0.975*RO2C+...
     &                          +  0.050 * RXRAT( 192 )      ! MVK+O3=0.05*RO2C+0.064*HO2+...
     &                          +  0.670 * RXRAT( 195 )      ! IPRD+OH=0.67*RO2C+...
     &                          +  0.048 * RXRAT( 196 )      ! IPRD+O3=0.048*RO2C+0.4*HO2+...
     &                          +  0.799 * RXRAT( 197 )      ! IPRD+NO3=0.799*RO2C+...
     &                          +  0.473 * RXRAT( 199 )      ! PRD2+OH=0.473*RO2C+...
     &                          +  1.590 * RXRAT( 200 )      ! PRD2=1.59*RO2C+0.4*MECO3+...
     &                          +  0.976 * RXRAT( 201 )      ! RNO3+OH=0.976*RO2C+...
     &                          +  0.721 * RXRAT( 202 )      ! RNO3=0.721*RO2C+0.554*xHO2+...
     &                          +  0.250 * RXRAT( 206 )      ! ACROLEIN+OH=0.25*RO2C+...
     &                          +  0.031 * RXRAT( 208 )      ! ACROLEIN+NO3=0.031*RO2C+...
     &                          +  0.020 * RXRAT( 211 )      ! CCOOOH+OH=0.02*RO2C+...
     &                          +  0.194 * RXRAT( 213 )      ! RCOOOH+OH=0.194*RO2C+...
     &                          +          RXRAT( 266 )      ! ETHENE+OH=RO2C+xHO2+...
     &                          +          RXRAT( 268 )      ! ETHENE+NO3=RO2C+xHO2+xRCHO+...
     &                          +  0.290 * RXRAT( 269 )      ! ETHENE+O3P=0.29*RO2C+...
     &                          +  0.984 * RXRAT( 270 )      ! PROPENE+OH=0.984*RO2C+...
     &                          +  0.949 * RXRAT( 272 )      ! PROPENE+NO3=0.949*RO2C+...
     &                          +  1.189 * RXRAT( 274 )      ! BUTADIENE13+OH=1.189*RO2C+...
     &                          +  1.055 * RXRAT( 276 )      ! BUTADIENE13+NO3=1.055*RO2C+...
     &                          +  0.235 * RXRAT( 277 )      ! BUTADIENE13+O3P=0.235*RO2C+...
     &                          +  0.986 * RXRAT( 278 )      ! ISOPRENE+OH=0.986*RO2C+...
     &                          +  0.192 * RXRAT( 279 )      ! ISOPRENE+O3=0.192*RO2C+...
     &                          +  0.936 * RXRAT( 280 )      ! ISOPRENE+NO3=0.936*RO2C+...
     &                          +  0.240 * RXRAT( 281 )      ! ISOPRENE+O3P=0.24*RO2C+...
     &                          +  1.042 * RXRAT( 282 )      ! APIN+OH=1.042*RO2C+...
     &                          +  1.511 * RXRAT( 283 )      ! APIN+O3=1.511*RO2C+...
     &                          +  1.050 * RXRAT( 284 )      ! APIN+NO3=1.05*RO2C+...
     &                          +  0.290 * RXRAT( 288 )      ! BENZENE+OH=0.29*RO2C+...
     &                          +  0.454 * RXRAT( 289 )      ! TOLUENE+OH=0.454*RO2C+...
     &                          +  0.520 * RXRAT( 290 )      ! MXYL+OH=0.52*RO2C+...
     &                          +  0.554 * RXRAT( 291 )      ! OXYL+OH=0.554*RO2C+...
     &                          +  0.487 * RXRAT( 292 )      ! PXYL+OH=0.487*RO2C+...
     &                          +  0.627 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.050 * RXRAT( 294 )      ! ETOH+OH=0.05*RO2C+...
     &                          +          RXRAT( 295 )      ! ALK1+OH=RO2C+xHO2+xCCHO+yROOH
     &                          +  0.965 * RXRAT( 296 )      ! ALK2+OH=0.965*RO2C+...
     &                          +  1.253 * RXRAT( 297 )      ! ALK3+OH=1.253*RO2C+...
     &                          +  1.763 * RXRAT( 298 )      ! ALK4+OH=1.763*RO2C+...
     &                          +  1.605 * RXRAT( 299 )      ! ALK5+OH=1.605*RO2C+...
     &                          +  1.202 * RXRAT( 300 )      ! OLE1+OH=1.202*RO2C+...
     &                          +  0.090 * RXRAT( 301 )      ! OLE1+O3=0.09*RO2C+...
     &                          +  1.463 * RXRAT( 302 )      ! OLE1+NO3=1.463*RO2C+...
     &                          +  0.953 * RXRAT( 304 )      ! OLE2+OH=0.953*RO2C+...
     &                          +  0.212 * RXRAT( 305 )      ! OLE2+O3=0.212*RO2C+...
     &                          +  1.193 * RXRAT( 306 )      ! OLE2+NO3=1.193*RO2C+...
     &                          +  0.566 * RXRAT( 308 )      ! ARO1+OH=0.566*RO2C+...
     &                          +  0.617 * RXRAT( 309 )      ! ARO2+OH=0.617*RO2C+...
     &                          +  1.211 * RXRAT( 310 )      ! TERP+OH=1.211*RO2C+...
     &                          +  0.490 * RXRAT( 311 )      ! TERP+O3=0.49*RO2C+...
     &                          +  1.786 * RXRAT( 312 )      ! TERP+NO3=1.786*RO2C+...
     &                          +  1.211 * RXRAT( 314 )      ! SESQ+OH=1.211*RO2C+...
     &                          +  0.490 * RXRAT( 315 )      ! SESQ+O3=0.49*RO2C+...
     &                          +  1.786 * RXRAT( 316 )      ! SESQ+NO3=1.786*RO2C+...
     &                          +  0.100 * RXRAT( 343 )      ! RCHO+CL=0.1*RO2C+0.9*RCO3+...
     &                          +          RXRAT( 344 )      ! ACETONE+CL=RO2C+HCL+xHCHO+...
     &                          +  0.975 * RXRAT( 345 )      ! MEK+CL=0.975*RO2C+HCL+...
     &                          +  1.282 * RXRAT( 346 )      ! RNO3+CL=1.282*RO2C+...
     &                          +  0.680 * RXRAT( 347 )      ! PRD2+CL=0.68*RO2C+...
     &                          +  0.588 * RXRAT( 352 )      ! ROOH+CL=0.588*RO2C+...
     &                          +  1.078 * RXRAT( 353 )      ! R6OOH+CL=1.078*RO2C+...
     &                          +  0.589 * RXRAT( 354 )      ! RAOOH+CL=0.589*RO2C+...
     &                          +  1.032 * RXRAT( 355 )      ! ACROLEIN+CL=1.032*RO2C+...
     &                          +  0.802 * RXRAT( 356 )      ! MACR+CL=0.802*RO2C+...
     &                          +  1.283 * RXRAT( 357 )      ! MVK+CL=1.283*RO2C+...
     &                          +  0.730 * RXRAT( 358 )      ! IPRD+CL=0.73*RO2C+...
     &                          +          RXRAT( 359 )      ! CLCCHO=RO2C+CO+HO2+xCL+...
     &                          +          RXRAT( 362 )      ! CLACET=RO2C+MECO3+xCL+...
     &                          +  2.000 * RXRAT( 370 )      ! ETHENE+CL=2*RO2C+xHO2+...
     &                          +  0.971 * RXRAT( 371 )      ! PROPENE+CL=0.971*RO2C+...
     &                          +  1.884 * RXRAT( 372 )      ! BUTADIENE13+CL=1.884*RO2C+...
     &                          +  1.168 * RXRAT( 373 )      ! ISOPRENE+CL=1.168*RO2C+...
     &                          +  2.258 * RXRAT( 374 )      ! APIN+CL=2.258*RO2C+...
     &                          +  0.894 * RXRAT( 376 )      ! TOLUENE+CL=0.894*RO2C+...
     &                          +  0.864 * RXRAT( 377 )      ! MXYL+CL=0.864*RO2C+...
     &                          +  0.864 * RXRAT( 378 )      ! OXYL+CL=0.864*RO2C+...
     &                          +  0.864 * RXRAT( 379 )      ! PXYL+CL=0.864*RO2C+...
     &                          +  0.838 * RXRAT( 380 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.312 * RXRAT( 381 )      ! ETOH+CL=0.312*RO2C+...
     &                          +          RXRAT( 382 )      ! ALK1+CL=RO2C+xHO2+HCL+...
     &                          +  0.970 * RXRAT( 383 )      ! ALK2+CL=0.97*RO2C+...
     &                          +  1.361 * RXRAT( 384 )      ! ALK3+CL=1.361*RO2C+...
     &                          +  1.737 * RXRAT( 385 )      ! ALK4+CL=1.737*RO2C+...
     &                          +  1.541 * RXRAT( 386 )      ! ALK5+CL=1.541*RO2C+...
     &                          +  1.608 * RXRAT( 387 )      ! OLE1+CL=1.608*RO2C+...
     &                          +  1.492 * RXRAT( 388 )      ! OLE2+CL=1.492*RO2C+...
     &                          +  0.840 * RXRAT( 389 )      ! ARO1+CL=0.84*RO2C+...
     &                          +  0.828 * RXRAT( 390 )      ! ARO2+CL=0.828*RO2C+...
     &                          +  2.258 * RXRAT( 391 )      ! TERP+CL=2.258*RO2C+...
     &                          +  2.258 * RXRAT( 392 )      ! SESQ+CL=2.258*RO2C+...
      LOSS( RO2C             ) =           RXRAT(  56 )      ! RO2C+NO=NO2
     &                          +          RXRAT(  57 )      ! RO2C+HO2=
     &                          +          RXRAT(  58 )      ! RO2C+NO3=NO2
     &                          +          RXRAT(  59 )      ! RO2C+MEO2=0.5*HO2+...
     &                          +  2.000 * RXRAT(  60 )      ! RO2C+RO2C=
     &                          +          RXRAT(  65 )      ! RO2C+RO2XC=
     &                          +          RXRAT(  74 )      ! RO2C+MECO3=MEO2+CO2
     &                          +          RXRAT( 107 )      ! RO2C+MACO3=CO2+HCHO+MECO3

      PROD( RO2XC            ) =   0.039 * RXRAT( 148 )      ! MEK+OH=0.039*RO2XC+...
     &                          +  0.004 * RXRAT( 156 )      ! ROOH+OH=0.004*RO2XC+...
     &                          +  0.029 * RXRAT( 158 )      ! R6OOH+OH=0.029*RO2XC+...
     &                          +  0.077 * RXRAT( 159 )      ! R6OOH=0.077*RO2XC+...
     &                          +  0.124 * RXRAT( 160 )      ! RAOOH+OH=0.124*RO2XC+...
     &                          +  0.060 * RXRAT( 178 )      ! AFG1+OH=0.06*RO2XC+...
     &                          +  0.060 * RXRAT( 181 )      ! AFG2+OH=0.06*RO2XC+...
     &                          +  0.117 * RXRAT( 184 )      ! AFG3+OH=0.117*RO2XC+...
     &                          +  0.007 * RXRAT( 185 )      ! AFG3+O3=0.007*RO2XC+...
     &                          +  0.025 * RXRAT( 191 )      ! MVK+OH=0.025*RO2XC+...
     &                          +  0.041 * RXRAT( 195 )      ! IPRD+OH=0.041*RO2XC+...
     &                          +  0.051 * RXRAT( 197 )      ! IPRD+NO3=0.051*RO2XC+...
     &                          +  0.071 * RXRAT( 199 )      ! PRD2+OH=0.071*RO2XC+...
     &                          +  0.087 * RXRAT( 200 )      ! PRD2=0.087*RO2XC+0.4*MECO3+...
     &                          +  0.175 * RXRAT( 201 )      ! RNO3+OH=0.175*RO2XC+...
     &                          +  0.102 * RXRAT( 202 )      ! RNO3=0.102*RO2XC+...
     &                          +  0.002 * RXRAT( 208 )      ! ACROLEIN+NO3=0.002*RO2XC+...
     &                          +  0.016 * RXRAT( 270 )      ! PROPENE+OH=0.016*RO2XC+...
     &                          +  0.051 * RXRAT( 272 )      ! PROPENE+NO3=0.051*RO2XC+...
     &                          +  0.049 * RXRAT( 274 )      ! BUTADIENE13+OH=0.049*RO2XC+...
     &                          +  0.065 * RXRAT( 276 )      ! BUTADIENE13+NO3=...
     &                          +  0.015 * RXRAT( 277 )      ! BUTADIENE13+O3P=...
     &                          +  0.093 * RXRAT( 278 )      ! ISOPRENE+OH=0.093*RO2XC+...
     &                          +  0.008 * RXRAT( 279 )      ! ISOPRENE+O3=0.008*RO2XC+...
     &                          +  0.064 * RXRAT( 280 )      ! ISOPRENE+NO3=0.064*RO2XC+...
     &                          +  0.010 * RXRAT( 281 )      ! ISOPRENE+O3P=0.01*RO2XC+...
     &                          +  0.197 * RXRAT( 282 )      ! APIN+OH=0.197*RO2XC+...
     &                          +  0.337 * RXRAT( 283 )      ! APIN+O3=0.337*RO2XC+...
     &                          +  0.293 * RXRAT( 284 )      ! APIN+NO3=0.293*RO2XC+...
     &                          +  0.024 * RXRAT( 288 )      ! BENZENE+OH=0.024*RO2XC+...
     &                          +  0.054 * RXRAT( 289 )      ! TOLUENE+OH=0.054*RO2XC+...
     &                          +  0.082 * RXRAT( 290 )      ! MXYL+OH=0.082*RO2XC+...
     &                          +  0.087 * RXRAT( 291 )      ! OXYL+OH=0.087*RO2XC+...
     &                          +  0.076 * RXRAT( 292 )      ! PXYL+OH=0.076*RO2XC+...
     &                          +  0.121 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.035 * RXRAT( 296 )      ! ALK2+OH=0.035*RO2XC+...
     &                          +  0.070 * RXRAT( 297 )      ! ALK3+OH=0.07*RO2XC+...
     &                          +  0.149 * RXRAT( 298 )      ! ALK4+OH=0.149*RO2XC+...
     &                          +  0.353 * RXRAT( 299 )      ! ALK5+OH=0.353*RO2XC+...
     &                          +  0.128 * RXRAT( 300 )      ! OLE1+OH=0.128*RO2XC+...
     &                          +  0.005 * RXRAT( 301 )      ! OLE1+O3=0.005*RO2XC+...
     &                          +  0.228 * RXRAT( 302 )      ! OLE1+NO3=0.228*RO2XC+...
     &                          +  0.088 * RXRAT( 304 )      ! OLE2+OH=0.088*RO2XC+...
     &                          +  0.003 * RXRAT( 305 )      ! OLE2+O3=0.003*RO2XC+...
     &                          +  0.140 * RXRAT( 306 )      ! OLE2+NO3=0.14*RO2XC+...
     &                          +  0.110 * RXRAT( 308 )      ! ARO1+OH=0.11*RO2XC+...
     &                          +  0.128 * RXRAT( 309 )      ! ARO2+OH=0.128*RO2XC+...
     &                          +  0.201 * RXRAT( 310 )      ! TERP+OH=0.201*RO2XC+...
     &                          +  0.121 * RXRAT( 311 )      ! TERP+O3=0.121*RO2XC+...
     &                          +  0.460 * RXRAT( 312 )      ! TERP+NO3=0.46*RO2XC+...
     &                          +  0.201 * RXRAT( 314 )      ! SESQ+OH=0.201*RO2XC+...
     &                          +  0.121 * RXRAT( 315 )      ! SESQ+O3=0.121*RO2XC+...
     &                          +  0.460 * RXRAT( 316 )      ! SESQ+NO3=0.46*RO2XC+...
     &                          +  0.039 * RXRAT( 345 )      ! MEK+CL=0.039*RO2XC+...
     &                          +  0.202 * RXRAT( 346 )      ! RNO3+CL=0.202*RO2XC+...
     &                          +  0.116 * RXRAT( 347 )      ! PRD2+CL=0.116*RO2XC+...
     &                          +  0.117 * RXRAT( 353 )      ! R6OOH+CL=0.117*RO2XC+...
     &                          +  0.124 * RXRAT( 354 )      ! RAOOH+CL=0.124*RO2XC+...
     &                          +  0.026 * RXRAT( 355 )      ! ACROLEIN+CL=0.026*RO2XC+...
     &                          +  0.033 * RXRAT( 356 )      ! MACR+CL=0.033*RO2XC+...
     &                          +  0.053 * RXRAT( 357 )      ! MVK+CL=0.053*RO2XC+...
     &                          +  0.051 * RXRAT( 358 )      ! IPRD+CL=0.051*RO2XC+...
     &                          +  0.029 * RXRAT( 371 )      ! PROPENE+CL=0.029*RO2XC+...
     &                          +  0.069 * RXRAT( 372 )      ! BUTADIENE13+CL=0.069*RO2XC+...
     &                          +  0.085 * RXRAT( 373 )      ! ISOPRENE+CL=0.085*RO2XC+...
     &                          +  0.582 * RXRAT( 374 )      ! APIN+CL=0.582*RO2XC+...
     &                          +  0.106 * RXRAT( 376 )      ! TOLUENE+CL=0.106*RO2XC+...
     &                          +  0.136 * RXRAT( 377 )      ! MXYL+CL=0.136*RO2XC+...
     &                          +  0.136 * RXRAT( 378 )      ! OXYL+CL=0.136*RO2XC+...
     &                          +  0.136 * RXRAT( 379 )      ! PXYL+CL=0.136*RO2XC+...
     &                          +  0.162 * RXRAT( 380 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.030 * RXRAT( 383 )      ! ALK2+CL=0.03*RO2XC+...
     &                          +  0.070 * RXRAT( 384 )      ! ALK3+CL=0.07*RO2XC+...
     &                          +  0.165 * RXRAT( 385 )      ! ALK4+CL=0.165*RO2XC+...
     &                          +  0.352 * RXRAT( 386 )      ! ALK5+CL=0.352*RO2XC+...
     &                          +  0.127 * RXRAT( 387 )      ! OLE1+CL=0.127*RO2XC+...
     &                          +  0.106 * RXRAT( 388 )      ! OLE2+CL=0.106*RO2XC+...
     &                          +  0.160 * RXRAT( 389 )      ! ARO1+CL=0.16*RO2XC+...
     &                          +  0.172 * RXRAT( 390 )      ! ARO2+CL=0.172*RO2XC+...
     &                          +  0.582 * RXRAT( 391 )      ! TERP+CL=0.582*RO2XC+...
     &                          +  0.582 * RXRAT( 392 )      ! SESQ+CL=0.582*RO2XC+...
      LOSS( RO2XC            ) =           RXRAT(  61 )      ! RO2XC+NO=
     &                          +          RXRAT(  62 )      ! RO2XC+HO2=
     &                          +          RXRAT(  63 )      ! RO2XC+NO3=NO2
     &                          +          RXRAT(  64 )      ! RO2XC+MEO2=0.5*HO2+...
     &                          +          RXRAT(  65 )      ! RO2XC+RO2C=
     &                          +  2.000 * RXRAT(  66 )      ! RO2XC+RO2XC=
     &                          +          RXRAT(  75 )      ! RO2XC+MECO3=MEO2+CO2
     &                          +          RXRAT(  85 )      ! RO2XC+RCO3=RO2C+xHO2+xCCHO+...
     &                          +          RXRAT(  96 )      ! RO2XC+BZCO3=RO2C+BZO+CO2
     &                          +          RXRAT( 108 )      ! RO2XC+MACO3=CO2+HCHO+MECO3

      PROD( CCOOOH           ) =   0.700 * RXRAT(  71 )      ! MECO3+HO2=0.7*CCOOOH+...
      LOSS( CCOOOH           ) =           RXRAT( 211 )      ! CCOOOH+OH=0.98*MECO3+...
     &                          +          RXRAT( 212 )      ! CCOOOH=MEO2+CO2+OH

      PROD( CCOOH            ) =   0.300 * RXRAT(  71 )      ! MECO3+HO2=0.3*CCOOH+...
     &                          +  0.100 * RXRAT(  73 )      ! MECO3+MEO2=0.1*CCOOH+...
     &                          +  0.050 * RXRAT( 210 )      ! ACROLEIN=0.05*CCOOH+...
     &                          +  0.075 * RXRAT( 271 )      ! PROPENE+O3=0.075*CCOOH+...
     &                          +  0.065 * RXRAT( 305 )      ! OLE2+O3=0.065*CCOOH+...
      LOSS( CCOOH            ) =           RXRAT( 152 )      ! CCOOH+OH=0.509*MEO2+...

      PROD( RCO3             ) =           RXRAT(  78 )      ! PAN2=RCO3+NO2
     &                          +  0.600 * RXRAT(  79 )      ! PAN2=0.6*RCO3+0.6*NO2+...
     &                          +          RXRAT( 128 )      ! xRCO3=RCO3
     &                          +  0.965 * RXRAT( 143 )      ! RCHO+OH=0.965*RCO3+...
     &                          +          RXRAT( 145 )      ! RCHO+NO3=RCO3+HNO3
     &                          +  0.370 * RXRAT( 164 )      ! GLY+OH=0.37*RCO3+1.26*CO+...
     &                          +  0.370 * RXRAT( 165 )      ! GLY+NO3=0.37*RCO3+0.63*HO2+...
     &                          +  0.300 * RXRAT( 198 )      ! IPRD=0.3*RCO3+0.467*MECO3+...
     &                          +  0.600 * RXRAT( 200 )      ! PRD2=0.6*RCO3+0.4*MECO3+...
     &                          +  0.806 * RXRAT( 213 )      ! RCOOOH+OH=0.806*RCO3+...
     &                          +  0.900 * RXRAT( 343 )      ! RCHO+CL=0.9*RCO3+HCL+...
     &                          +  0.370 * RXRAT( 348 )      ! GLY+CL=0.37*RCO3+0.63*HO2+...
     &                          +          RXRAT( 360 )      ! CLCCHO+OH=RCO3
     &                          +          RXRAT( 361 )      ! CLCCHO+CL=RCO3+HCL
      LOSS( RCO3             ) =           RXRAT(  77 )      ! RCO3+NO2=PAN2
     &                          +          RXRAT(  80 )      ! RCO3+NO=NO2+RO2C+xHO2+...
     &                          +          RXRAT(  81 )      ! RCO3+HO2=0.75*RCOOOH+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=NO2+RO2C+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=HCHO+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=RO2C+xHO2+xCCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=RO2C+xHO2+xCCHO+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=2*CO2+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*RO2C+2*xHO2+...
     &                          +          RXRAT(  98 )      ! RCO3+BZCO3=2*CO2+RO2C+xHO2+...
     &                          +          RXRAT( 110 )      ! RCO3+MACO3=HCHO+MECO3+RO2C+...

      PROD( PAN2             ) =           RXRAT(  77 )      ! RCO3+NO2=PAN2
      LOSS( PAN2             ) =           RXRAT(  78 )      ! PAN2=RCO3+NO2
     &                          +          RXRAT(  79 )      ! PAN2=0.6*RCO3+0.6*NO2+...

      PROD( xHO2             ) =   0.400 * RXRAT(  79 )      ! PAN2=0.4*xHO2+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=xHO2+RO2C+NO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=xHO2+RO2C+NO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=xHO2+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=xHO2+RO2C+xCCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=xHO2+RO2C+xCCHO+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=xHO2+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*xHO2+2*RO2C+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=xHO2+RO2C+2*CO2+...
     &                          +          RXRAT( 110 )      ! MACO3+RCO3=xHO2+MECO3+RO2C+...
     &                          +          RXRAT( 117 )      ! BZO=xHO2+RO2C+CRES
     &                          +  0.035 * RXRAT( 143 )      ! RCHO+OH=0.035*xHO2+...
     &                          +          RXRAT( 144 )      ! RCHO=xHO2+RO2C+yROOH+xCCHO+...
     &                          +  0.376 * RXRAT( 148 )      ! MEK+OH=0.376*xHO2+...
     &                          +          RXRAT( 149 )      ! MEK=xHO2+RO2C+MECO3+xCCHO+...
     &                          +  0.491 * RXRAT( 152 )      ! CCOOH+OH=0.491*xHO2+...
     &                          +          RXRAT( 153 )      ! RCOOH+OH=xHO2+RO2C+...
     &                          +  0.239 * RXRAT( 156 )      ! ROOH+OH=0.239*xHO2+...
     &                          +  0.090 * RXRAT( 158 )      ! R6OOH+OH=0.09*xHO2+...
     &                          +  0.782 * RXRAT( 159 )      ! R6OOH=0.782*xHO2+0.142*HO2+...
     &                          +  0.565 * RXRAT( 160 )      ! RAOOH+OH=0.565*xHO2+...
     &                          +  0.800 * RXRAT( 170 )      ! CRES+OH=0.8*xHO2+0.8*RO2C+...
     &                          +  0.521 * RXRAT( 178 )      ! AFG1+OH=0.521*xHO2+...
     &                          +  0.521 * RXRAT( 181 )      ! AFG2+OH=0.521*xHO2+...
     &                          +  0.561 * RXRAT( 184 )      ! AFG3+OH=0.561*xHO2+...
     &                          +  0.095 * RXRAT( 185 )      ! AFG3+O3=0.095*xHO2+...
     &                          +  0.500 * RXRAT( 186 )      ! MACR+OH=0.5*xHO2+0.5*RO2C+...
     &                          +  0.500 * RXRAT( 188 )      ! MACR+NO3=0.5*xHO2+0.5*RO2C+...
     &                          +  0.300 * RXRAT( 191 )      ! MVK+OH=0.3*xHO2+...
     &                          +  0.050 * RXRAT( 192 )      ! MVK+O3=0.05*xHO2+0.064*HO2+...
     &                          +  0.670 * RXRAT( 195 )      ! IPRD+OH=0.67*xHO2+...
     &                          +  0.799 * RXRAT( 197 )      ! IPRD+NO3=0.799*xHO2+...
     &                          +  0.379 * RXRAT( 199 )      ! PRD2+OH=0.379*xHO2+...
     &                          +  0.913 * RXRAT( 200 )      ! PRD2=0.913*xHO2+0.4*MECO3+...
     &                          +  0.305 * RXRAT( 201 )      ! RNO3+OH=0.305*xHO2+...
     &                          +  0.554 * RXRAT( 202 )      ! RNO3=0.554*xHO2+0.344*HO2+...
     &                          +  0.250 * RXRAT( 206 )      ! ACROLEIN+OH=0.25*xHO2+...
     &                          +  0.031 * RXRAT( 208 )      ! ACROLEIN+NO3=0.031*xHO2+...
     &                          +  0.084 * RXRAT( 213 )      ! RCOOOH+OH=0.084*xHO2+...
     &                          +          RXRAT( 214 )      ! RCOOOH=xHO2+xCCHO+yROOH+CO2+OH
     &                          +          RXRAT( 266 )      ! ETHENE+OH=xHO2+RO2C+...
     &                          +          RXRAT( 268 )      ! ETHENE+NO3=xHO2+RO2C+xRCHO+...
     &                          +  0.290 * RXRAT( 269 )      ! ETHENE+O3P=0.29*xHO2+...
     &                          +  0.984 * RXRAT( 270 )      ! PROPENE+OH=0.984*xHO2+...
     &                          +  0.949 * RXRAT( 272 )      ! PROPENE+NO3=0.949*xHO2+...
     &                          +  0.951 * RXRAT( 274 )      ! BUTADIENE13+OH=0.951*xHO2+...
     &                          +  0.815 * RXRAT( 276 )      ! BUTADIENE13+NO3=0.815*xHO2+...
     &                          +  0.117 * RXRAT( 277 )      ! BUTADIENE13+O3P=0.117*xHO2+...
     &                          +  0.907 * RXRAT( 278 )      ! ISOPRENE+OH=0.907*xHO2+...
     &                          +  0.749 * RXRAT( 280 )      ! ISOPRENE+NO3=0.749*xHO2+...
     &                          +  0.799 * RXRAT( 282 )      ! APIN+OH=0.799*xHO2+...
     &                          +  0.102 * RXRAT( 283 )      ! APIN+O3=0.102*xHO2+...
     &                          +  0.056 * RXRAT( 284 )      ! APIN+NO3=0.056*xHO2+...
     &                          +  0.290 * RXRAT( 288 )      ! BENZENE+OH=0.29*xHO2+...
     &                          +  0.454 * RXRAT( 289 )      ! TOLUENE+OH=0.454*xHO2+...
     &                          +  0.520 * RXRAT( 290 )      ! MXYL+OH=0.52*xHO2+...
     &                          +  0.554 * RXRAT( 291 )      ! OXYL+OH=0.554*xHO2+...
     &                          +  0.487 * RXRAT( 292 )      ! PXYL+OH=0.487*xHO2+...
     &                          +  0.627 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.050 * RXRAT( 294 )      ! ETOH+OH=0.05*xHO2+0.95*HO2+...
     &                          +          RXRAT( 295 )      ! ALK1+OH=xHO2+RO2C+xCCHO+yROOH
     &                          +  0.965 * RXRAT( 296 )      ! ALK2+OH=0.965*xHO2+...
     &                          +  0.695 * RXRAT( 297 )      ! ALK3+OH=0.695*xHO2+...
     &                          +  0.830 * RXRAT( 298 )      ! ALK4+OH=0.83*xHO2+...
     &                          +  0.647 * RXRAT( 299 )      ! ALK5+OH=0.647*xHO2+...
     &                          +  0.871 * RXRAT( 300 )      ! OLE1+OH=0.871*xHO2+...
     &                          +  0.057 * RXRAT( 301 )      ! OLE1+O3=0.057*xHO2+...
     &                          +  0.772 * RXRAT( 302 )      ! OLE1+NO3=0.772*xHO2+...
     &                          +  0.912 * RXRAT( 304 )      ! OLE2+OH=0.912*xHO2+...
     &                          +  0.041 * RXRAT( 305 )      ! OLE2+O3=0.041*xHO2+...
     &                          +  0.400 * RXRAT( 306 )      ! OLE2+NO3=0.4*xHO2+...
     &                          +  0.566 * RXRAT( 308 )      ! ARO1+OH=0.566*xHO2+...
     &                          +  0.617 * RXRAT( 309 )      ! ARO2+OH=0.617*xHO2+...
     &                          +  0.734 * RXRAT( 310 )      ! TERP+OH=0.734*xHO2+...
     &                          +  0.046 * RXRAT( 311 )      ! TERP+O3=0.046*xHO2+...
     &                          +  0.227 * RXRAT( 312 )      ! TERP+NO3=0.227*xHO2+...
     &                          +  0.734 * RXRAT( 314 )      ! SESQ+OH=0.734*xHO2+...
     &                          +  0.046 * RXRAT( 315 )      ! SESQ+O3=0.046*xHO2+...
     &                          +  0.227 * RXRAT( 316 )      ! SESQ+NO3=0.227*xHO2+...
     &                          +  0.100 * RXRAT( 343 )      ! RCHO+CL=0.1*xHO2+0.9*RCO3+...
     &                          +  0.840 * RXRAT( 345 )      ! MEK+CL=0.84*xHO2+...
     &                          +  0.547 * RXRAT( 346 )      ! RNO3+CL=0.547*xHO2+...
     &                          +  0.541 * RXRAT( 347 )      ! PRD2+CL=0.541*xHO2+...
     &                          +          RXRAT( 350 )      ! CRES+CL=xHO2+HCL+xBALD+yR6OOH
     &                          +  0.482 * RXRAT( 352 )      ! ROOH+CL=0.482*xHO2+...
     &                          +  0.237 * RXRAT( 353 )      ! R6OOH+CL=0.237*xHO2+...
     &                          +  0.565 * RXRAT( 354 )      ! RAOOH+CL=0.565*xHO2+...
     &                          +  0.484 * RXRAT( 355 )      ! ACROLEIN+CL=0.484*xHO2+...
     &                          +  0.802 * RXRAT( 356 )      ! MACR+CL=0.802*xHO2+...
     &                          +  0.322 * RXRAT( 357 )      ! MVK+CL=0.322*xHO2+...
     &                          +  0.712 * RXRAT( 358 )      ! IPRD+CL=0.712*xHO2+...
     &                          +          RXRAT( 370 )      ! ETHENE+CL=xHO2+2*RO2C+...
     &                          +  0.971 * RXRAT( 371 )      ! PROPENE+CL=0.971*xHO2+...
     &                          +  0.390 * RXRAT( 372 )      ! BUTADIENE13+CL=0.39*xHO2+...
     &                          +  0.738 * RXRAT( 373 )      ! ISOPRENE+CL=0.738*xHO2+...
     &                          +  0.252 * RXRAT( 374 )      ! APIN+CL=0.252*xHO2+...
     &                          +  0.894 * RXRAT( 376 )      ! TOLUENE+CL=0.894*xHO2+...
     &                          +  0.864 * RXRAT( 377 )      ! MXYL+CL=0.864*xHO2+...
     &                          +  0.864 * RXRAT( 378 )      ! OXYL+CL=0.864*xHO2+...
     &                          +  0.864 * RXRAT( 379 )      ! PXYL+CL=0.864*xHO2+...
     &                          +  0.838 * RXRAT( 380 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.312 * RXRAT( 381 )      ! ETOH+CL=0.312*xHO2+...
     &                          +          RXRAT( 382 )      ! ALK1+CL=xHO2+HCL+RO2C+...
     &                          +  0.970 * RXRAT( 383 )      ! ALK2+CL=0.97*xHO2+HCL+...
     &                          +  0.835 * RXRAT( 384 )      ! ALK3+CL=0.835*xHO2+HCL+...
     &                          +  0.827 * RXRAT( 385 )      ! ALK4+CL=0.827*xHO2+HCL+...
     &                          +  0.647 * RXRAT( 386 )      ! ALK5+CL=0.647*xHO2+HCL+...
     &                          +  0.873 * RXRAT( 387 )      ! OLE1+CL=0.873*xHO2+...
     &                          +  0.450 * RXRAT( 388 )      ! OLE2+CL=0.45*xHO2+...
     &                          +  0.840 * RXRAT( 389 )      ! ARO1+CL=0.84*xHO2+...
     &                          +  0.828 * RXRAT( 390 )      ! ARO2+CL=0.828*xHO2+...
     &                          +  0.252 * RXRAT( 391 )      ! TERP+CL=0.252*xHO2+...
     &                          +  0.252 * RXRAT( 392 )      ! SESQ+CL=0.252*xHO2+...
      LOSS( xHO2             ) =           RXRAT( 118 )      ! xHO2=HO2
     &                          +          RXRAT( 119 )      ! xHO2=

      PROD( yROOH            ) =   0.400 * RXRAT(  79 )      ! PAN2=0.4*yROOH+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=yROOH+RO2C+xHO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=yROOH+RO2C+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=yROOH+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=yROOH+xHO2+xCCHO+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=yROOH+xHO2+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=yROOH+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*yROOH+2*xHO2+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=yROOH+RO2C+xHO2+...
     &                          +          RXRAT( 110 )      ! MACO3+RCO3=yROOH+MECO3+...
     &                          +  0.035 * RXRAT( 143 )      ! RCHO+OH=0.035*yROOH+...
     &                          +          RXRAT( 144 )      ! RCHO=yROOH+xHO2+RO2C+xCCHO+...
     &                          +          RXRAT( 146 )      ! ACETONE+OH=yROOH+xMECO3+...
     &                          +          RXRAT( 148 )      ! MEK+OH=yROOH+0.039*RO2XC+...
     &                          +          RXRAT( 149 )      ! MEK=yROOH+RO2C+xHO2+xCCHO+...
     &                          +  0.491 * RXRAT( 152 )      ! CCOOH+OH=0.491*yROOH+...
     &                          +          RXRAT( 153 )      ! RCOOH+OH=yROOH+xHO2+...
     &                          +  0.256 * RXRAT( 156 )      ! ROOH+OH=0.256*yROOH+...
     &                          +  0.160 * RXRAT( 158 )      ! R6OOH+OH=0.16*yROOH+...
     &                          +  0.500 * RXRAT( 186 )      ! MACR+OH=0.5*yROOH+0.5*RO2C+...
     &                          +  0.100 * RXRAT( 187 )      ! MACR+O3=0.1*yROOH+...
     &                          +  0.500 * RXRAT( 188 )      ! MACR+NO3=0.5*yROOH+...
     &                          +  0.330 * RXRAT( 190 )      ! MACR=0.33*yROOH+0.67*HO2+...
     &                          +          RXRAT( 191 )      ! MVK+OH=yROOH+0.025*RO2XC+...
     &                          +  0.050 * RXRAT( 192 )      ! MVK+O3=0.05*yROOH+...
     &                          +  0.157 * RXRAT( 201 )      ! RNO3+OH=0.157*yROOH+...
     &                          +  0.066 * RXRAT( 202 )      ! RNO3=0.066*yROOH+...
     &                          +  0.250 * RXRAT( 206 )      ! ACROLEIN+OH=0.25*yROOH+...
     &                          +  0.033 * RXRAT( 208 )      ! ACROLEIN+NO3=0.033*yROOH+...
     &                          +  0.020 * RXRAT( 211 )      ! CCOOOH+OH=0.02*yROOH+...
     &                          +  0.194 * RXRAT( 213 )      ! RCOOOH+OH=0.194*yROOH+...
     &                          +          RXRAT( 214 )      ! RCOOOH=yROOH+xCCHO+xHO2+CO2+OH
     &                          +          RXRAT( 266 )      ! ETHENE+OH=yROOH+RO2C+...
     &                          +          RXRAT( 268 )      ! ETHENE+NO3=yROOH+RO2C+...
     &                          +  0.290 * RXRAT( 269 )      ! ETHENE+O3P=0.29*yROOH+...
     &                          +          RXRAT( 270 )      ! PROPENE+OH=yROOH+...
     &                          +          RXRAT( 272 )      ! PROPENE+NO3=yROOH+...
     &                          +          RXRAT( 274 )      ! BUTADIENE13+OH=yROOH+...
     &                          +          RXRAT( 276 )      ! BUTADIENE13+NO3=yROOH+...
     &                          +  0.250 * RXRAT( 277 )      ! BUTADIENE13+O3P=0.25*yROOH+...
     &                          +  0.050 * RXRAT( 294 )      ! ETOH+OH=0.05*yROOH+...
     &                          +          RXRAT( 295 )      ! ALK1+OH=yROOH+RO2C+xCCHO+xHO2
     &                          +          RXRAT( 296 )      ! ALK2+OH=yROOH+0.965*RO2C+...
     &                          +  0.983 * RXRAT( 297 )      ! ALK3+OH=0.983*yROOH+...
     &                          +  0.169 * RXRAT( 300 )      ! OLE1+OH=0.169*yROOH+...
     &                          +  0.011 * RXRAT( 301 )      ! OLE1+O3=0.011*yROOH+...
     &                          +  0.169 * RXRAT( 302 )      ! OLE1+NO3=0.169*yROOH+...
     &                          +  0.319 * RXRAT( 304 )      ! OLE2+OH=0.319*yROOH+...
     &                          +  0.073 * RXRAT( 305 )      ! OLE2+O3=0.073*yROOH+...
     &                          +  0.319 * RXRAT( 306 )      ! OLE2+NO3=0.319*yROOH+...
     &                          +  0.100 * RXRAT( 343 )      ! RCHO+CL=0.1*yROOH+0.9*RCO3+...
     &                          +          RXRAT( 344 )      ! ACETONE+CL=yROOH+RO2C+...
     &                          +          RXRAT( 345 )      ! MEK+CL=yROOH+0.975*RO2C+...
     &                          +  0.586 * RXRAT( 352 )      ! ROOH+CL=0.586*yROOH+...
     &                          +  0.784 * RXRAT( 355 )      ! ACROLEIN+CL=0.784*yROOH+...
     &                          +  0.835 * RXRAT( 356 )      ! MACR+CL=0.835*yROOH+...
     &                          +          RXRAT( 357 )      ! MVK+CL=yROOH+0.053*RO2XC+...
     &                          +          RXRAT( 359 )      ! CLCCHO=yROOH+CO+RO2C+xCL+...
     &                          +          RXRAT( 362 )      ! CLACET=yROOH+RO2C+xCL+...
     &                          +          RXRAT( 371 )      ! PROPENE+CL=yROOH+...
     &                          +          RXRAT( 372 )      ! BUTADIENE13+CL=yROOH+...
     &                          +  0.312 * RXRAT( 381 )      ! ETOH+CL=0.312*yROOH+...
     &                          +          RXRAT( 382 )      ! ALK1+CL=yROOH+xHO2+RO2C+...
     &                          +          RXRAT( 383 )      ! ALK2+CL=yROOH+0.97*xHO2+...
     &                          +  0.983 * RXRAT( 384 )      ! ALK3+CL=0.983*yROOH+...
     &                          +  0.169 * RXRAT( 387 )      ! OLE1+CL=0.169*yROOH+...
     &                          +  0.319 * RXRAT( 388 )      ! OLE2+CL=0.319*yROOH+...
      LOSS( yROOH            ) =           RXRAT( 252 )      ! yROOH=ROOH
     &                          +          RXRAT( 253 )      ! yROOH=MEK
     &                          +          RXRAT( 254 )      ! yROOH=

      PROD( xCCHO            ) =   0.400 * RXRAT(  79 )      ! PAN2=0.4*xCCHO+0.6*NO2+...
     &                          +          RXRAT(  80 )      ! RCO3+NO=xCCHO+RO2C+xHO2+...
     &                          +          RXRAT(  82 )      ! RCO3+NO3=xCCHO+RO2C+xHO2+...
     &                          +          RXRAT(  83 )      ! RCO3+MEO2=xCCHO+HO2+RO2C+...
     &                          +          RXRAT(  84 )      ! RCO3+RO2C=xCCHO+xHO2+RO2C+...
     &                          +          RXRAT(  85 )      ! RCO3+RO2XC=xCCHO+xHO2+RO2C+...
     &                          +          RXRAT(  86 )      ! RCO3+MECO3=xCCHO+MEO2+RO2C+...
     &                          +  2.000 * RXRAT(  87 )      ! RCO3+RCO3=2*xCCHO+2*xHO2+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=xCCHO+RO2C+xHO2+...
     &                          +          RXRAT( 110 )      ! MACO3+RCO3=xCCHO+MECO3+...
     &                          +  0.035 * RXRAT( 143 )      ! RCHO+OH=0.035*xCCHO+...
     &                          +          RXRAT( 144 )      ! RCHO=xCCHO+xHO2+yROOH+RO2C+...
     &                          +  0.504 * RXRAT( 148 )      ! MEK+OH=0.504*xCCHO+...
     &                          +          RXRAT( 149 )      ! MEK=xCCHO+RO2C+xHO2+MECO3+...
     &                          +  0.142 * RXRAT( 153 )      ! RCOOH+OH=0.142*xCCHO+xHO2+...
     &                          +  0.012 * RXRAT( 156 )      ! ROOH+OH=0.012*xCCHO+...
     &                          +  0.020 * RXRAT( 158 )      ! R6OOH+OH=0.02*xCCHO+...
     &                          +  0.026 * RXRAT( 159 )      ! R6OOH=0.026*xCCHO+...
     &                          +  0.083 * RXRAT( 199 )      ! PRD2+OH=0.083*xCCHO+...
     &                          +  0.163 * RXRAT( 200 )      ! PRD2=0.163*xCCHO+0.4*MECO3+...
     &                          +  0.429 * RXRAT( 201 )      ! RNO3+OH=0.429*xCCHO+...
     &                          +  0.230 * RXRAT( 202 )      ! RNO3=0.23*xCCHO+0.554*xHO2+...
     &                          +  0.167 * RXRAT( 206 )      ! ACROLEIN+OH=0.167*xCCHO+...
     &                          +  0.110 * RXRAT( 213 )      ! RCOOOH+OH=0.11*xCCHO+...
     &                          +          RXRAT( 214 )      ! RCOOOH=xCCHO+xHO2+yROOH+CO2+OH
     &                          +  0.984 * RXRAT( 270 )      ! PROPENE+OH=0.984*xCCHO+...
     &                          +          RXRAT( 295 )      ! ALK1+OH=xCCHO+RO2C+xHO2+yROOH
     &                          +  0.445 * RXRAT( 297 )      ! ALK3+OH=0.445*xCCHO+...
     &                          +  0.438 * RXRAT( 298 )      ! ALK4+OH=0.438*xCCHO+...
     &                          +  0.106 * RXRAT( 299 )      ! ALK5+OH=0.106*xCCHO+...
     &                          +  0.010 * RXRAT( 300 )      ! OLE1+OH=0.01*xCCHO+...
     &                          +  0.011 * RXRAT( 301 )      ! OLE1+O3=0.011*xCCHO+...
     &                          +  0.013 * RXRAT( 302 )      ! OLE1+NO3=0.013*xCCHO+...
     &                          +  0.835 * RXRAT( 304 )      ! OLE2+OH=0.835*xCCHO+...
     &                          +  0.071 * RXRAT( 305 )      ! OLE2+O3=0.071*xCCHO+...
     &                          +  0.579 * RXRAT( 306 )      ! OLE2+NO3=0.579*xCCHO+...
     &                          +  0.002 * RXRAT( 316 )      ! SESQ+NO3=0.002*xCCHO+...
     &                          +  0.100 * RXRAT( 343 )      ! RCHO+CL=0.1*xCCHO+0.9*RCO3+...
     &                          +  0.070 * RXRAT( 345 )      ! MEK+CL=0.07*xCCHO+...
     &                          +  0.300 * RXRAT( 346 )      ! RNO3+CL=0.3*xCCHO+...
     &                          +  0.109 * RXRAT( 347 )      ! PRD2+CL=0.109*xCCHO+...
     &                          +  0.104 * RXRAT( 352 )      ! ROOH+CL=0.104*xCCHO+...
     &                          +  0.186 * RXRAT( 353 )      ! R6OOH+CL=0.186*xCCHO+...
     &                          +          RXRAT( 382 )      ! ALK1+CL=xCCHO+xHO2+RO2C+...
     &                          +  0.340 * RXRAT( 384 )      ! ALK3+CL=0.34*xCCHO+...
     &                          +  0.287 * RXRAT( 385 )      ! ALK4+CL=0.287*xCCHO+...
     &                          +  0.080 * RXRAT( 386 )      ! ALK5+CL=0.08*xCCHO+...
     &                          +  0.206 * RXRAT( 387 )      ! OLE1+CL=0.206*xCCHO+...
     &                          +  0.383 * RXRAT( 388 )      ! OLE2+CL=0.383*xCCHO+...
      LOSS( xCCHO            ) =           RXRAT( 217 )      ! xCCHO=CCHO
     &                          +          RXRAT( 218 )      ! xCCHO=

      PROD( RCOOOH           ) =   0.750 * RXRAT(  81 )      ! RCO3+HO2=0.75*RCOOOH+...
     &                          +  0.750 * RXRAT(  92 )      ! BZCO3+HO2=0.75*RCOOOH+...
     &                          +  0.750 * RXRAT( 104 )      ! MACO3+HO2=0.75*RCOOOH+...
      LOSS( RCOOOH           ) =           RXRAT( 213 )      ! RCOOOH+OH=0.806*RCO3+...
     &                          +          RXRAT( 214 )      ! RCOOOH=xHO2+xCCHO+yROOH+CO2+OH

      PROD( RCOOH            ) =   0.250 * RXRAT(  81 )      ! RCO3+HO2=0.25*RCOOH+...
     &                          +  0.250 * RXRAT(  92 )      ! BZCO3+HO2=0.25*RCOOH+...
     &                          +  0.250 * RXRAT( 104 )      ! MACO3+HO2=0.25*RCOOH+...
     &                          +  0.372 * RXRAT( 196 )      ! IPRD+O3=0.372*RCOOH+...
     &                          +  0.159 * RXRAT( 301 )      ! OLE1+O3=0.159*RCOOH+...
     &                          +  0.235 * RXRAT( 305 )      ! OLE2+O3=0.235*RCOOH+...
     &                          +  0.068 * RXRAT( 311 )      ! TERP+O3=0.068*RCOOH+...
     &                          +  0.068 * RXRAT( 315 )      ! SESQ+O3=0.068*RCOOH+...
      LOSS( RCOOH            ) =           RXRAT( 153 )      ! RCOOH+OH=RO2C+xHO2+...

      PROD( BZCO3            ) =           RXRAT(  89 )      ! PBZN=BZCO3+NO2
     &                          +  0.600 * RXRAT(  90 )      ! PBZN=0.6*BZCO3+0.6*NO2+...
     &                          +          RXRAT( 175 )      ! BALD+OH=BZCO3
     &                          +          RXRAT( 177 )      ! BALD+NO3=BZCO3+HNO3
     &                          +          RXRAT( 351 )      ! BALD+CL=BZCO3+HCL
      LOSS( BZCO3            ) =           RXRAT(  88 )      ! BZCO3+NO2=PBZN
     &                          +          RXRAT(  91 )      ! BZCO3+NO=NO2+CO2+BZO+RO2C
     &                          +          RXRAT(  92 )      ! BZCO3+HO2=0.75*RCOOOH+...
     &                          +          RXRAT(  93 )      ! BZCO3+NO3=NO2+CO2+BZO+RO2C
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=HCHO+HO2+RO2C+...
     &                          +          RXRAT(  95 )      ! BZCO3+RO2C=RO2C+BZO+CO2
     &                          +          RXRAT(  96 )      ! BZCO3+RO2XC=RO2C+BZO+CO2
     &                          +          RXRAT(  97 )      ! BZCO3+MECO3=2*CO2+MEO2+BZO+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=2*CO2+RO2C+xHO2+...
     &                          +  2.000 * RXRAT(  99 )      ! BZCO3+BZCO3=2*BZO+2*RO2C+2*CO2
     &                          +          RXRAT( 111 )      ! BZCO3+MACO3=HCHO+MECO3+BZO+...

      PROD( PBZN             ) =           RXRAT(  88 )      ! BZCO3+NO2=PBZN
      LOSS( PBZN             ) =           RXRAT(  89 )      ! PBZN=BZCO3+NO2
     &                          +          RXRAT(  90 )      ! PBZN=0.6*BZCO3+0.6*NO2+...

      PROD( BZO              ) =   0.400 * RXRAT(  90 )      ! PBZN=0.4*BZO+0.6*NO2+...
     &                          +          RXRAT(  91 )      ! BZCO3+NO=BZO+CO2+NO2+RO2C
     &                          +          RXRAT(  93 )      ! BZCO3+NO3=BZO+CO2+NO2+RO2C
     &                          +          RXRAT(  94 )      ! BZCO3+MEO2=BZO+HO2+RO2C+...
     &                          +          RXRAT(  95 )      ! BZCO3+RO2C=BZO+RO2C+CO2
     &                          +          RXRAT(  96 )      ! BZCO3+RO2XC=BZO+RO2C+CO2
     &                          +          RXRAT(  97 )      ! BZCO3+MECO3=BZO+MEO2+2*CO2+...
     &                          +          RXRAT(  98 )      ! BZCO3+RCO3=BZO+RO2C+xHO2+...
     &                          +  2.000 * RXRAT(  99 )      ! BZCO3+BZCO3=2*BZO+2*RO2C+2*CO2
     &                          +          RXRAT( 111 )      ! MACO3+BZCO3=BZO+MECO3+HCHO+...
     &                          +  0.200 * RXRAT( 170 )      ! CRES+OH=0.2*BZO+0.8*RO2C+...
     &                          +          RXRAT( 171 )      ! CRES+NO3=BZO+HNO3
     &                          +          RXRAT( 172 )      ! NPHE+OH=BZO
      LOSS( BZO              ) =           RXRAT( 115 )      ! BZO+NO2=NPHE
     &                          +          RXRAT( 116 )      ! BZO+HO2=CRES
     &                          +          RXRAT( 117 )      ! BZO=CRES+RO2C+xHO2

      PROD( MACO3            ) =           RXRAT( 101 )      ! MAPAN=MACO3+NO2
     &                          +  0.600 * RXRAT( 102 )      ! MAPAN=0.6*MACO3+0.6*NO2+...
     &                          +          RXRAT( 130 )      ! xMACO3=MACO3
     &                          +  0.217 * RXRAT( 178 )      ! AFG1+OH=0.217*MACO3+...
     &                          +  0.500 * RXRAT( 180 )      ! AFG1=0.5*MACO3+0.173*MEO2+...
     &                          +  0.217 * RXRAT( 181 )      ! AFG2+OH=0.217*MACO3+...
     &                          +  0.206 * RXRAT( 184 )      ! AFG3+OH=0.206*MACO3+...
     &                          +  0.500 * RXRAT( 186 )      ! MACR+OH=0.5*MACO3+0.5*RO2C+...
     &                          +  0.500 * RXRAT( 188 )      ! MACR+NO3=0.5*MACO3+...
     &                          +  0.330 * RXRAT( 190 )      ! MACR=0.33*MACO3+0.67*HO2+...
     &                          +  0.400 * RXRAT( 194 )      ! MVK=0.4*MACO3+0.6*CO+...
     &                          +  0.289 * RXRAT( 195 )      ! IPRD+OH=0.289*MACO3+...
     &                          +  0.150 * RXRAT( 197 )      ! IPRD+NO3=0.15*MACO3+...
     &                          +  0.750 * RXRAT( 206 )      ! ACROLEIN+OH=0.75*MACO3+...
     &                          +  0.967 * RXRAT( 208 )      ! ACROLEIN+NO3=0.967*MACO3+...
     &                          +  0.330 * RXRAT( 210 )      ! ACROLEIN=0.33*MACO3+...
     &                          +  0.216 * RXRAT( 355 )      ! ACROLEIN+CL=0.216*MACO3+...
     &                          +  0.165 * RXRAT( 356 )      ! MACR+CL=0.165*MACO3+...
     &                          +  0.154 * RXRAT( 358 )      ! IPRD+CL=0.154*MACO3+...
      LOSS( MACO3            ) =           RXRAT( 100 )      ! MACO3+NO2=MAPAN
     &                          +          RXRAT( 103 )      ! MACO3+NO=NO2+CO2+HCHO+MECO3
     &                          +          RXRAT( 104 )      ! MACO3+HO2=0.75*RCOOOH+...
     &                          +          RXRAT( 105 )      ! MACO3+NO3=NO2+CO2+HCHO+MECO3
     &                          +          RXRAT( 106 )      ! MACO3+MEO2=2*HCHO+HO2+CO2+...
     &                          +          RXRAT( 107 )      ! MACO3+RO2C=CO2+HCHO+MECO3
     &                          +          RXRAT( 108 )      ! MACO3+RO2XC=CO2+HCHO+MECO3
     &                          +          RXRAT( 109 )      ! MACO3+MECO3=2*CO2+MEO2+...
     &                          +          RXRAT( 110 )      ! MACO3+RCO3=HCHO+MECO3+RO2C+...
     &                          +          RXRAT( 111 )      ! MACO3+BZCO3=HCHO+MECO3+BZO+...
     &                          +  2.000 * RXRAT( 112 )      ! MACO3+MACO3=2*HCHO+2*MECO3+...

      PROD( MAPAN            ) =           RXRAT( 100 )      ! MACO3+NO2=MAPAN
      LOSS( MAPAN            ) =           RXRAT( 101 )      ! MAPAN=MACO3+NO2
     &                          +          RXRAT( 102 )      ! MAPAN=0.6*MACO3+0.6*NO2+...

      PROD( TBUO             ) =           RXRAT( 132 )      ! xTBUO=TBUO
      LOSS( TBUO             ) =           RXRAT( 113 )      ! TBUO+NO2=RNO3
     &                          +          RXRAT( 114 )      ! TBUO=ACETONE+MEO2

      PROD( RNO3             ) =           RXRAT( 113 )      ! TBUO+NO2=RNO3
     &                          +          RXRAT( 247 )      ! xRNO3=RNO3
     &                          +          RXRAT( 249 )      ! zRNO3=RNO3
      LOSS( RNO3             ) =   0.811 * RXRAT( 201 )      ! RNO3+OH=0.189*RNO3+...
     &                          +          RXRAT( 202 )      ! RNO3=0.344*HO2+0.554*xHO2+...
     &                          +  0.945 * RXRAT( 346 )      ! RNO3+CL=0.055*RNO3+...

      PROD( ACETONE          ) =           RXRAT( 114 )      ! TBUO=ACETONE+MEO2
     &                          +          RXRAT( 221 )      ! xACETONE=ACETONE
     &                          +  0.051 * RXRAT( 305 )      ! OLE2+O3=0.051*ACETONE+...
      LOSS( ACETONE          ) =           RXRAT( 146 )      ! ACETONE+OH=RO2C+xMECO3+...
     &                          +          RXRAT( 147 )      ! ACETONE=0.62*MECO3+...
     &                          +          RXRAT( 344 )      ! ACETONE+CL=HCL+RO2C+xHCHO+...

      PROD( NPHE             ) =           RXRAT( 115 )      ! BZO+NO2=NPHE
      LOSS( NPHE             ) =           RXRAT( 172 )      ! NPHE+OH=BZO
     &                          +          RXRAT( 173 )      ! NPHE=HONO
     &                          +          RXRAT( 174 )      ! NPHE=

      PROD( CRES             ) =           RXRAT( 116 )      ! BZO+HO2=CRES
     &                          +          RXRAT( 117 )      ! BZO=CRES+RO2C+xHO2
     &                          +  0.570 * RXRAT( 288 )      ! BENZENE+OH=0.57*CRES+...
     &                          +  0.181 * RXRAT( 289 )      ! TOLUENE+OH=0.181*CRES+...
     &                          +  0.159 * RXRAT( 290 )      ! MXYL+OH=0.159*CRES+...
     &                          +  0.161 * RXRAT( 291 )      ! OXYL+OH=0.161*CRES+...
     &                          +  0.159 * RXRAT( 292 )      ! PXYL+OH=0.159*CRES+...
     &                          +  0.022 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.123 * RXRAT( 308 )      ! ARO1+OH=0.123*CRES+...
     &                          +  0.077 * RXRAT( 309 )      ! ARO2+OH=0.077*CRES+...
      LOSS( CRES             ) =           RXRAT( 170 )      ! CRES+OH=0.2*BZO+0.8*RO2C+...
     &                          +          RXRAT( 171 )      ! CRES+NO3=HNO3+BZO
     &                          +          RXRAT( 350 )      ! CRES+CL=HCL+xHO2+xBALD+yR6OOH

      PROD( xOH              ) =   0.012 * RXRAT( 156 )      ! ROOH+OH=0.012*xOH+...
     &                          +  0.041 * RXRAT( 158 )      ! R6OOH+OH=0.041*xOH+...
     &                          +  0.024 * RXRAT( 160 )      ! RAOOH+OH=0.024*xOH+...
     &                          +  0.020 * RXRAT( 211 )      ! CCOOOH+OH=0.02*xOH+...
     &                          +  0.110 * RXRAT( 213 )      ! RCOOOH+OH=0.11*xOH+...
     &                          +  0.104 * RXRAT( 352 )      ! ROOH+CL=0.104*xOH+0.414*OH+...
     &                          +  0.502 * RXRAT( 353 )      ! R6OOH+CL=0.502*xOH+...
     &                          +  0.024 * RXRAT( 354 )      ! RAOOH+CL=0.024*xOH+...
      LOSS( xOH              ) =           RXRAT( 120 )      ! xOH=OH
     &                          +          RXRAT( 121 )      ! xOH=

      PROD( xNO2             ) =   0.313 * RXRAT( 201 )      ! RNO3+OH=0.313*xNO2+...
     &                          +  0.120 * RXRAT( 276 )      ! BUTADIENE13+NO3=0.12*xNO2+...
     &                          +  0.187 * RXRAT( 280 )      ! ISOPRENE+NO3=0.187*xNO2+...
     &                          +  0.643 * RXRAT( 284 )      ! APIN+NO3=0.643*xNO2+...
     &                          +  0.426 * RXRAT( 306 )      ! OLE2+NO3=0.426*xNO2+...
     &                          +  0.287 * RXRAT( 312 )      ! TERP+NO3=0.287*xNO2+...
     &                          +  0.287 * RXRAT( 316 )      ! SESQ+NO3=0.287*xNO2+...
     &                          +  0.159 * RXRAT( 346 )      ! RNO3+CL=0.159*xNO2+...
      LOSS( xNO2             ) =           RXRAT( 122 )      ! xNO2=NO2
     &                          +          RXRAT( 123 )      ! xNO2=

      PROD( xMEO2            ) =   0.010 * RXRAT( 298 )      ! ALK4+OH=0.01*xMEO2+...
     &                          +  0.001 * RXRAT( 300 )      ! OLE1+OH=0.001*xMEO2+...
     &                          +  0.035 * RXRAT( 306 )      ! OLE2+NO3=0.035*xMEO2+...
     &                          +  0.003 * RXRAT( 385 )      ! ALK4+CL=0.003*xMEO2+...
     &                          +  0.001 * RXRAT( 388 )      ! OLE2+CL=0.001*xMEO2+...
      LOSS( xMEO2            ) =           RXRAT( 124 )      ! xMEO2=MEO2
     &                          +          RXRAT( 125 )      ! xMEO2=

      PROD( xMECO3           ) =           RXRAT( 146 )      ! ACETONE+OH=xMECO3+RO2C+...
     &                          +  0.510 * RXRAT( 148 )      ! MEK+OH=0.51*xMECO3+...
     &                          +  0.201 * RXRAT( 178 )      ! AFG1+OH=0.201*xMECO3+...
     &                          +  0.201 * RXRAT( 181 )      ! AFG2+OH=0.201*xMECO3+...
     &                          +  0.117 * RXRAT( 184 )      ! AFG3+OH=0.117*xMECO3+...
     &                          +  0.330 * RXRAT( 190 )      ! MACR=0.33*xMECO3+0.67*HO2+...
     &                          +  0.675 * RXRAT( 191 )      ! MVK+OH=0.675*xMECO3+...
     &                          +  0.029 * RXRAT( 199 )      ! PRD2+OH=0.029*xMECO3+...
     &                          +  0.001 * RXRAT( 283 )      ! APIN+O3=0.001*xMECO3+...
     &                          +  0.011 * RXRAT( 298 )      ! ALK4+OH=0.011*xMECO3+...
     &                          +  0.156 * RXRAT( 305 )      ! OLE2+O3=0.156*xMECO3+...
     &                          +  0.202 * RXRAT( 311 )      ! TERP+O3=0.202*xMECO3+...
     &                          +  0.202 * RXRAT( 315 )      ! SESQ+O3=0.202*xMECO3+...
     &                          +          RXRAT( 344 )      ! ACETONE+CL=xMECO3+RO2C+...
     &                          +  0.085 * RXRAT( 345 )      ! MEK+CL=0.085*xMECO3+...
     &                          +  0.007 * RXRAT( 347 )      ! PRD2+CL=0.007*xMECO3+...
     &                          +  0.625 * RXRAT( 357 )      ! MVK+CL=0.625*xMECO3+...
     &                          +  0.034 * RXRAT( 374 )      ! APIN+CL=0.034*xMECO3+...
     &                          +  0.004 * RXRAT( 385 )      ! ALK4+CL=0.004*xMECO3+...
     &                          +  0.034 * RXRAT( 391 )      ! TERP+CL=0.034*xMECO3+...
     &                          +  0.034 * RXRAT( 392 )      ! SESQ+CL=0.034*xMECO3+...
      LOSS( xMECO3           ) =           RXRAT( 126 )      ! xMECO3=MECO3
     &                          +          RXRAT( 127 )      ! xMECO3=

      PROD( xRCO3            ) =   0.074 * RXRAT( 148 )      ! MEK+OH=0.074*xRCO3+...
     &                          +  0.652 * RXRAT( 179 )      ! AFG1+O3=0.652*xRCO3+...
     &                          +  0.652 * RXRAT( 182 )      ! AFG2+O3=0.652*xRCO3+...
     &                          +  0.163 * RXRAT( 185 )      ! AFG3+O3=0.163*xRCO3+...
     &                          +  0.100 * RXRAT( 187 )      ! MACR+O3=0.1*xRCO3+...
     &                          +  0.050 * RXRAT( 192 )      ! MVK+O3=0.05*xRCO3+...
     &                          +  0.048 * RXRAT( 196 )      ! IPRD+O3=0.048*xRCO3+...
     &                          +  0.049 * RXRAT( 199 )      ! PRD2+OH=0.049*xRCO3+...
     &                          +  0.004 * RXRAT( 282 )      ! APIN+OH=0.004*xRCO3+...
     &                          +  0.297 * RXRAT( 283 )      ! APIN+O3=0.297*xRCO3+...
     &                          +  0.007 * RXRAT( 284 )      ! APIN+NO3=0.007*xRCO3+...
     &                          +  0.008 * RXRAT( 305 )      ! OLE2+O3=0.008*xRCO3+...
     &                          +  0.064 * RXRAT( 310 )      ! TERP+OH=0.064*xRCO3+...
     &                          +  0.059 * RXRAT( 311 )      ! TERP+O3=0.059*xRCO3+...
     &                          +  0.026 * RXRAT( 312 )      ! TERP+NO3=0.026*xRCO3+...
     &                          +  0.064 * RXRAT( 314 )      ! SESQ+OH=0.064*xRCO3+...
     &                          +  0.059 * RXRAT( 315 )      ! SESQ+O3=0.059*xRCO3+...
     &                          +  0.026 * RXRAT( 316 )      ! SESQ+NO3=0.026*xRCO3+...
     &                          +  0.036 * RXRAT( 345 )      ! MEK+CL=0.036*xRCO3+...
     &                          +  0.022 * RXRAT( 347 )      ! PRD2+CL=0.022*xRCO3+...
     &                          +  0.050 * RXRAT( 374 )      ! APIN+CL=0.05*xRCO3+...
     &                          +  0.050 * RXRAT( 391 )      ! TERP+CL=0.05*xRCO3+...
     &                          +  0.050 * RXRAT( 392 )      ! SESQ+CL=0.05*xRCO3+...
      LOSS( xRCO3            ) =           RXRAT( 128 )      ! xRCO3=RCO3
     &                          +          RXRAT( 129 )      ! xRCO3=

      PROD( xMACO3           ) =   0.118 * RXRAT( 277 )      ! BUTADIENE13+O3P=...
     &                          +  0.192 * RXRAT( 279 )      ! ISOPRENE+O3=0.192*xMACO3+...
     &                          +  0.240 * RXRAT( 281 )      ! ISOPRENE+O3P=0.24*xMACO3+...
     &                          +  0.016 * RXRAT( 374 )      ! APIN+CL=0.016*xMACO3+...
     &                          +  0.016 * RXRAT( 391 )      ! TERP+CL=0.016*xMACO3+...
     &                          +  0.016 * RXRAT( 392 )      ! SESQ+CL=0.016*xMACO3+...
      LOSS( xMACO3           ) =           RXRAT( 130 )      ! xMACO3=MACO3
     &                          +          RXRAT( 131 )      ! xMACO3=

      PROD( xTBUO            ) =   0.236 * RXRAT( 297 )      ! ALK3+OH=0.236*xTBUO+...
     &                          +  0.094 * RXRAT( 384 )      ! ALK3+CL=0.094*xTBUO+...
      LOSS( xTBUO            ) =           RXRAT( 132 )      ! xTBUO=TBUO
     &                          +          RXRAT( 133 )      ! xTBUO=

      PROD( xCO              ) =   0.035 * RXRAT( 143 )      ! RCHO+OH=0.035*xCO+...
     &                          +  0.334 * RXRAT( 178 )      ! AFG1+OH=0.334*xCO+...
     &                          +  0.334 * RXRAT( 181 )      ! AFG2+OH=0.334*xCO+...
     &                          +  0.114 * RXRAT( 184 )      ! AFG3+OH=0.114*xCO+...
     &                          +  0.416 * RXRAT( 186 )      ! MACR+OH=0.416*xCO+0.5*RO2C+...
     &                          +  0.500 * RXRAT( 188 )      ! MACR+NO3=0.5*xCO+0.5*RO2C+...
     &                          +  0.336 * RXRAT( 195 )      ! IPRD+OH=0.336*xCO+...
     &                          +  0.572 * RXRAT( 197 )      ! IPRD+NO3=0.572*xCO+...
     &                          +  0.167 * RXRAT( 206 )      ! ACROLEIN+OH=0.167*xCO+...
     &                          +  0.031 * RXRAT( 208 )      ! ACROLEIN+NO3=0.031*xCO+...
     &                          +  0.278 * RXRAT( 269 )      ! ETHENE+O3P=0.278*xCO+...
     &                          +  0.115 * RXRAT( 277 )      ! BUTADIENE13+O3P=0.115*xCO+...
     &                          +  0.002 * RXRAT( 282 )      ! APIN+OH=0.002*xCO+...
     &                          +  0.051 * RXRAT( 283 )      ! APIN+O3=0.051*xCO+...
     &                          +  0.005 * RXRAT( 284 )      ! APIN+NO3=0.005*xCO+...
     &                          +  0.002 * RXRAT( 298 )      ! ALK4+OH=0.002*xCO+...
     &                          +  0.001 * RXRAT( 310 )      ! TERP+OH=0.001*xCO+...
     &                          +  0.012 * RXRAT( 312 )      ! TERP+NO3=0.012*xCO+...
     &                          +  0.001 * RXRAT( 314 )      ! SESQ+OH=0.001*xCO+...
     &                          +  0.012 * RXRAT( 316 )      ! SESQ+NO3=0.012*xCO+...
     &                          +  0.100 * RXRAT( 343 )      ! RCHO+CL=0.1*xCO+0.9*RCO3+...
     &                          +  0.484 * RXRAT( 355 )      ! ACROLEIN+CL=0.484*xCO+...
     &                          +  0.541 * RXRAT( 356 )      ! MACR+CL=0.541*xCO+...
     &                          +  0.498 * RXRAT( 358 )      ! IPRD+CL=0.498*xCO+...
     &                          +  0.035 * RXRAT( 374 )      ! APIN+CL=0.035*xCO+...
     &                          +  0.003 * RXRAT( 385 )      ! ALK4+CL=0.003*xCO+...
     &                          +  0.035 * RXRAT( 391 )      ! TERP+CL=0.035*xCO+...
     &                          +  0.035 * RXRAT( 392 )      ! SESQ+CL=0.035*xCO+...
      LOSS( xCO              ) =           RXRAT( 134 )      ! xCO=CO
     &                          +          RXRAT( 135 )      ! xCO=

      PROD( CCHO             ) =   0.001 * RXRAT( 199 )      ! PRD2+OH=0.001*CCHO+...
     &                          +  0.214 * RXRAT( 202 )      ! RNO3=0.214*CCHO+0.554*xHO2+...
     &                          +          RXRAT( 217 )      ! xCCHO=CCHO
     &                          +  0.100 * RXRAT( 269 )      ! ETHENE+O3P=0.1*CCHO+...
     &                          +  0.500 * RXRAT( 271 )      ! PROPENE+O3=0.5*CCHO+...
     &                          +  0.950 * RXRAT( 294 )      ! ETOH+OH=0.95*CCHO+...
     &                          +  0.453 * RXRAT( 305 )      ! OLE2+O3=0.453*CCHO+...
     &                          +  0.688 * RXRAT( 381 )      ! ETOH+CL=0.688*CCHO+...
      LOSS( CCHO             ) =           RXRAT( 140 )      ! CCHO+OH=MECO3
     &                          +          RXRAT( 141 )      ! CCHO=CO+HO2+MEO2
     &                          +          RXRAT( 142 )      ! CCHO+NO3=HNO3+MECO3
     &                          +          RXRAT( 341 )      ! CCHO+CL=HCL+MECO3

      PROD( RCHO             ) =   0.744 * RXRAT( 156 )      ! ROOH+OH=0.744*RCHO+...
     &                          +          RXRAT( 157 )      ! ROOH=RCHO+HO2+OH
     &                          +  0.085 * RXRAT( 159 )      ! R6OOH=0.085*RCHO+0.142*HO2+...
     &                          +          RXRAT( 189 )      ! MACR+O3P=RCHO
     &                          +  0.450 * RXRAT( 193 )      ! MVK+O3P=0.45*RCHO+0.55*MEK
     &                          +  0.143 * RXRAT( 199 )      ! PRD2+OH=0.143*RCHO+...
     &                          +  0.001 * RXRAT( 201 )      ! RNO3+OH=0.001*RCHO+...
     &                          +  0.074 * RXRAT( 202 )      ! RNO3=0.074*RCHO+0.554*xHO2+...
     &                          +          RXRAT( 209 )      ! ACROLEIN+O3P=RCHO
     &                          +          RXRAT( 219 )      ! xRCHO=RCHO
     &                          +  0.450 * RXRAT( 273 )      ! PROPENE+O3P=0.45*RCHO+0.55*MEK
     &                          +  0.500 * RXRAT( 301 )      ! OLE1+O3=0.5*RCHO+...
     &                          +  0.450 * RXRAT( 303 )      ! OLE1+O3P=0.45*RCHO+...
     &                          +  0.333 * RXRAT( 305 )      ! OLE2+O3=0.333*RCHO+...
     &                          +  0.079 * RXRAT( 307 )      ! OLE2+O3P=0.079*RCHO+...
     &                          +  0.237 * RXRAT( 313 )      ! TERP+O3P=0.237*RCHO+...
     &                          +  0.237 * RXRAT( 317 )      ! SESQ+O3P=0.237*RCHO+...
     &                          +  0.009 * RXRAT( 346 )      ! RNO3+CL=0.009*RCHO+...
     &                          +  0.198 * RXRAT( 347 )      ! PRD2+CL=0.198*RCHO+...
     &                          +  0.414 * RXRAT( 352 )      ! ROOH+CL=0.414*RCHO+...
      LOSS( RCHO             ) =           RXRAT( 143 )      ! RCHO+OH=0.965*RCO3+...
     &                          +          RXRAT( 144 )      ! RCHO=RO2C+xHO2+yROOH+xCCHO+...
     &                          +          RXRAT( 145 )      ! RCHO+NO3=HNO3+RCO3
     &                          +          RXRAT( 343 )      ! RCHO+CL=HCL+0.9*RCO3+...

      PROD( xHCHO            ) =           RXRAT( 146 )      ! ACETONE+OH=xHCHO+xMECO3+...
     &                          +  0.088 * RXRAT( 148 )      ! MEK+OH=0.088*xHCHO+...
     &                          +  0.012 * RXRAT( 156 )      ! ROOH+OH=0.012*xHCHO+...
     &                          +  0.652 * RXRAT( 179 )      ! AFG1+O3=0.652*xHCHO+...
     &                          +  0.652 * RXRAT( 182 )      ! AFG2+O3=0.652*xHCHO+...
     &                          +  0.163 * RXRAT( 185 )      ! AFG3+O3=0.163*xHCHO+...
     &                          +  0.084 * RXRAT( 186 )      ! MACR+OH=0.084*xHCHO+...
     &                          +  0.100 * RXRAT( 187 )      ! MACR+O3=0.1*xHCHO+...
     &                          +  0.330 * RXRAT( 190 )      ! MACR=0.33*xHCHO+0.67*HO2+...
     &                          +  0.300 * RXRAT( 191 )      ! MVK+OH=0.3*xHCHO+...
     &                          +  0.050 * RXRAT( 192 )      ! MVK+O3=0.05*xHCHO+...
     &                          +  0.055 * RXRAT( 195 )      ! IPRD+OH=0.055*xHCHO+...
     &                          +  0.001 * RXRAT( 196 )      ! IPRD+O3=0.001*xHCHO+...
     &                          +  0.227 * RXRAT( 197 )      ! IPRD+NO3=0.227*xHCHO+...
     &                          +  0.211 * RXRAT( 199 )      ! PRD2+OH=0.211*xHCHO+...
     &                          +  0.303 * RXRAT( 200 )      ! PRD2=0.303*xHCHO+0.4*MECO3+...
     &                          +  0.011 * RXRAT( 201 )      ! RNO3+OH=0.011*xHCHO+...
     &                          +  0.061 * RXRAT( 202 )      ! RNO3=0.061*xHCHO+...
     &                          +  0.083 * RXRAT( 206 )      ! ACROLEIN+OH=0.083*xHCHO+...
     &                          +  0.020 * RXRAT( 211 )      ! CCOOOH+OH=0.02*xHCHO+...
     &                          +  1.610 * RXRAT( 266 )      ! ETHENE+OH=1.61*xHCHO+RO2C+...
     &                          +  0.278 * RXRAT( 269 )      ! ETHENE+O3P=0.278*xHCHO+...
     &                          +  0.984 * RXRAT( 270 )      ! PROPENE+OH=0.984*xHCHO+...
     &                          +  0.708 * RXRAT( 274 )      ! BUTADIENE13+OH=0.708*xHCHO+...
     &                          +  0.115 * RXRAT( 276 )      ! BUTADIENE13+NO3=...
     &                          +  0.624 * RXRAT( 278 )      ! ISOPRENE+OH=0.624*xHCHO+...
     &                          +  0.192 * RXRAT( 279 )      ! ISOPRENE+O3=0.192*xHCHO+...
     &                          +  0.240 * RXRAT( 281 )      ! ISOPRENE+O3P=0.24*xHCHO+...
     &                          +  0.022 * RXRAT( 282 )      ! APIN+OH=0.022*xHCHO+...
     &                          +  0.344 * RXRAT( 283 )      ! APIN+O3=0.344*xHCHO+...
     &                          +  0.007 * RXRAT( 284 )      ! APIN+NO3=0.007*xHCHO+...
     &                          +  0.081 * RXRAT( 294 )      ! ETOH+OH=0.081*xHCHO+...
     &                          +  0.026 * RXRAT( 297 )      ! ALK3+OH=0.026*xHCHO+...
     &                          +  0.029 * RXRAT( 298 )      ! ALK4+OH=0.029*xHCHO+...
     &                          +  0.040 * RXRAT( 299 )      ! ALK5+OH=0.04*xHCHO+...
     &                          +  0.582 * RXRAT( 300 )      ! OLE1+OH=0.582*xHCHO+...
     &                          +  0.179 * RXRAT( 304 )      ! OLE2+OH=0.179*xHCHO+...
     &                          +  0.114 * RXRAT( 305 )      ! OLE2+O3=0.114*xHCHO+...
     &                          +  0.072 * RXRAT( 306 )      ! OLE2+NO3=0.072*xHCHO+...
     &                          +  0.411 * RXRAT( 310 )      ! TERP+OH=0.411*xHCHO+...
     &                          +  0.033 * RXRAT( 311 )      ! TERP+O3=0.033*xHCHO+...
     &                          +  0.023 * RXRAT( 312 )      ! TERP+NO3=0.023*xHCHO+...
     &                          +  0.411 * RXRAT( 314 )      ! SESQ+OH=0.411*xHCHO+...
     &                          +  0.033 * RXRAT( 315 )      ! SESQ+O3=0.033*xHCHO+...
     &                          +  0.023 * RXRAT( 316 )      ! SESQ+NO3=0.023*xHCHO+...
     &                          +          RXRAT( 344 )      ! ACETONE+CL=xHCHO+RO2C+HCL+...
     &                          +  0.065 * RXRAT( 345 )      ! MEK+CL=0.065*xHCHO+...
     &                          +  0.045 * RXRAT( 346 )      ! RNO3+CL=0.045*xHCHO+...
     &                          +  0.237 * RXRAT( 347 )      ! PRD2+CL=0.237*xHCHO+...
     &                          +  0.106 * RXRAT( 352 )      ! ROOH+CL=0.106*xHCHO+...
     &                          +  0.274 * RXRAT( 355 )      ! ACROLEIN+CL=0.274*xHCHO+...
     &                          +  0.195 * RXRAT( 358 )      ! IPRD+CL=0.195*xHCHO+...
     &                          +          RXRAT( 359 )      ! CLCCHO=xHCHO+CO+RO2C+xCL+...
     &                          +          RXRAT( 362 )      ! CLACET=xHCHO+RO2C+xCL+...
     &                          +          RXRAT( 370 )      ! ETHENE+CL=xHCHO+2*RO2C+...
     &                          +  0.863 * RXRAT( 372 )      ! BUTADIENE13+CL=0.863*xHCHO+...
     &                          +  0.275 * RXRAT( 373 )      ! ISOPRENE+CL=0.275*xHCHO+...
     &                          +  0.158 * RXRAT( 374 )      ! APIN+CL=0.158*xHCHO+...
     &                          +  0.503 * RXRAT( 381 )      ! ETOH+CL=0.503*xHCHO+...
     &                          +  0.078 * RXRAT( 384 )      ! ALK3+CL=0.078*xHCHO+...
     &                          +  0.034 * RXRAT( 385 )      ! ALK4+CL=0.034*xHCHO+...
     &                          +  0.022 * RXRAT( 386 )      ! ALK5+CL=0.022*xHCHO+...
     &                          +  0.036 * RXRAT( 387 )      ! OLE1+CL=0.036*xHCHO+...
     &                          +  0.190 * RXRAT( 388 )      ! OLE2+CL=0.19*xHCHO+...
     &                          +  0.158 * RXRAT( 391 )      ! TERP+CL=0.158*xHCHO+...
     &                          +  0.158 * RXRAT( 392 )      ! SESQ+CL=0.158*xHCHO+...
      LOSS( xHCHO            ) =           RXRAT( 215 )      ! xHCHO=HCHO
     &                          +          RXRAT( 216 )      ! xHCHO=

      PROD( MEK              ) =   0.550 * RXRAT( 193 )      ! MVK+O3P=0.55*MEK+0.45*RCHO
     &                          +  0.210 * RXRAT( 196 )      ! IPRD+O3=0.21*MEK+0.4*HO2+...
     &                          +  0.233 * RXRAT( 198 )      ! IPRD=0.233*MEK+0.467*MECO3+...
     &                          +  0.010 * RXRAT( 201 )      ! RNO3+OH=0.01*MEK+...
     &                          +  0.124 * RXRAT( 202 )      ! RNO3=0.124*MEK+0.554*xHO2+...
     &                          +          RXRAT( 223 )      ! xMEK=MEK
     &                          +          RXRAT( 253 )      ! yROOH=MEK
     &                          +  0.550 * RXRAT( 273 )      ! PROPENE+O3P=0.55*MEK+0.45*RCHO
     &                          +  0.008 * RXRAT( 283 )      ! APIN+O3=0.008*MEK+...
     &                          +  0.009 * RXRAT( 301 )      ! OLE1+O3=0.009*MEK+...
     &                          +  0.390 * RXRAT( 303 )      ! OLE1+O3P=0.39*MEK+...
     &                          +  0.033 * RXRAT( 305 )      ! OLE2+O3=0.033*MEK+...
     &                          +  0.751 * RXRAT( 307 )      ! OLE2+O3P=0.751*MEK+...
     &                          +  0.002 * RXRAT( 311 )      ! TERP+O3=0.002*MEK+...
     &                          +  0.002 * RXRAT( 315 )      ! SESQ+O3=0.002*MEK+...
     &                          +  0.018 * RXRAT( 346 )      ! RNO3+CL=0.018*MEK+...
      LOSS( MEK              ) =           RXRAT( 148 )      ! MEK+OH=0.967*RO2C+...
     &                          +          RXRAT( 149 )      ! MEK=MECO3+RO2C+xHO2+xCCHO+...
     &                          +          RXRAT( 345 )      ! MEK+CL=HCL+0.975*RO2C+...

      PROD( zRNO3            ) =   0.039 * RXRAT( 148 )      ! MEK+OH=0.039*zRNO3+...
     &                          +  0.004 * RXRAT( 156 )      ! ROOH+OH=0.004*zRNO3+...
     &                          +  0.029 * RXRAT( 158 )      ! R6OOH+OH=0.029*zRNO3+...
     &                          +  0.077 * RXRAT( 159 )      ! R6OOH=0.077*zRNO3+...
     &                          +  0.124 * RXRAT( 160 )      ! RAOOH+OH=0.124*zRNO3+...
     &                          +  0.060 * RXRAT( 178 )      ! AFG1+OH=0.06*zRNO3+...
     &                          +  0.060 * RXRAT( 181 )      ! AFG2+OH=0.06*zRNO3+...
     &                          +  0.117 * RXRAT( 184 )      ! AFG3+OH=0.117*zRNO3+...
     &                          +  0.007 * RXRAT( 185 )      ! AFG3+O3=0.007*zRNO3+...
     &                          +  0.025 * RXRAT( 191 )      ! MVK+OH=0.025*zRNO3+...
     &                          +  0.041 * RXRAT( 195 )      ! IPRD+OH=0.041*zRNO3+...
     &                          +  0.051 * RXRAT( 197 )      ! IPRD+NO3=0.051*zRNO3+...
     &                          +  0.071 * RXRAT( 199 )      ! PRD2+OH=0.071*zRNO3+...
     &                          +  0.087 * RXRAT( 200 )      ! PRD2=0.087*zRNO3+0.4*MECO3+...
     &                          +  0.175 * RXRAT( 201 )      ! RNO3+OH=0.175*zRNO3+...
     &                          +  0.102 * RXRAT( 202 )      ! RNO3=0.102*zRNO3+...
     &                          +  0.002 * RXRAT( 208 )      ! ACROLEIN+NO3=0.002*zRNO3+...
     &                          +  0.016 * RXRAT( 270 )      ! PROPENE+OH=0.016*zRNO3+...
     &                          +  0.051 * RXRAT( 272 )      ! PROPENE+NO3=0.051*zRNO3+...
     &                          +  0.049 * RXRAT( 274 )      ! BUTADIENE13+OH=0.049*zRNO3+...
     &                          +  0.065 * RXRAT( 276 )      ! BUTADIENE13+NO3=...
     &                          +  0.015 * RXRAT( 277 )      ! BUTADIENE13+O3P=...
     &                          +  0.093 * RXRAT( 278 )      ! ISOPRENE+OH=0.093*zRNO3+...
     &                          +  0.008 * RXRAT( 279 )      ! ISOPRENE+O3=0.008*zRNO3+...
     &                          +  0.064 * RXRAT( 280 )      ! ISOPRENE+NO3=0.064*zRNO3+...
     &                          +  0.010 * RXRAT( 281 )      ! ISOPRENE+O3P=0.01*zRNO3+...
     &                          +  0.197 * RXRAT( 282 )      ! APIN+OH=0.197*zRNO3+...
     &                          +  0.337 * RXRAT( 283 )      ! APIN+O3=0.337*zRNO3+...
     &                          +  0.293 * RXRAT( 284 )      ! APIN+NO3=0.293*zRNO3+...
     &                          +  0.024 * RXRAT( 288 )      ! BENZENE+OH=0.024*zRNO3+...
     &                          +  0.054 * RXRAT( 289 )      ! TOLUENE+OH=0.054*zRNO3+...
     &                          +  0.082 * RXRAT( 290 )      ! MXYL+OH=0.082*zRNO3+...
     &                          +  0.087 * RXRAT( 291 )      ! OXYL+OH=0.087*zRNO3+...
     &                          +  0.076 * RXRAT( 292 )      ! PXYL+OH=0.076*zRNO3+...
     &                          +  0.121 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.035 * RXRAT( 296 )      ! ALK2+OH=0.035*zRNO3+...
     &                          +  0.070 * RXRAT( 297 )      ! ALK3+OH=0.07*zRNO3+...
     &                          +  0.149 * RXRAT( 298 )      ! ALK4+OH=0.149*zRNO3+...
     &                          +  0.353 * RXRAT( 299 )      ! ALK5+OH=0.353*zRNO3+...
     &                          +  0.128 * RXRAT( 300 )      ! OLE1+OH=0.128*zRNO3+...
     &                          +  0.005 * RXRAT( 301 )      ! OLE1+O3=0.005*zRNO3+...
     &                          +  0.228 * RXRAT( 302 )      ! OLE1+NO3=0.228*zRNO3+...
     &                          +  0.088 * RXRAT( 304 )      ! OLE2+OH=0.088*zRNO3+...
     &                          +  0.003 * RXRAT( 305 )      ! OLE2+O3=0.003*zRNO3+...
     &                          +  0.140 * RXRAT( 306 )      ! OLE2+NO3=0.14*zRNO3+...
     &                          +  0.110 * RXRAT( 308 )      ! ARO1+OH=0.11*zRNO3+...
     &                          +  0.128 * RXRAT( 309 )      ! ARO2+OH=0.128*zRNO3+...
     &                          +  0.201 * RXRAT( 310 )      ! TERP+OH=0.201*zRNO3+...
     &                          +  0.121 * RXRAT( 311 )      ! TERP+O3=0.121*zRNO3+...
     &                          +  0.460 * RXRAT( 312 )      ! TERP+NO3=0.46*zRNO3+...
     &                          +  0.201 * RXRAT( 314 )      ! SESQ+OH=0.201*zRNO3+...
     &                          +  0.121 * RXRAT( 315 )      ! SESQ+O3=0.121*zRNO3+...
     &                          +  0.460 * RXRAT( 316 )      ! SESQ+NO3=0.46*zRNO3+...
     &                          +  0.039 * RXRAT( 345 )      ! MEK+CL=0.039*zRNO3+...
     &                          +  0.202 * RXRAT( 346 )      ! RNO3+CL=0.202*zRNO3+...
     &                          +  0.116 * RXRAT( 347 )      ! PRD2+CL=0.116*zRNO3+...
     &                          +  0.117 * RXRAT( 353 )      ! R6OOH+CL=0.117*zRNO3+...
     &                          +  0.124 * RXRAT( 354 )      ! RAOOH+CL=0.124*zRNO3+...
     &                          +  0.026 * RXRAT( 355 )      ! ACROLEIN+CL=0.026*zRNO3+...
     &                          +  0.033 * RXRAT( 356 )      ! MACR+CL=0.033*zRNO3+...
     &                          +  0.053 * RXRAT( 357 )      ! MVK+CL=0.053*zRNO3+...
     &                          +  0.051 * RXRAT( 358 )      ! IPRD+CL=0.051*zRNO3+...
     &                          +  0.029 * RXRAT( 371 )      ! PROPENE+CL=0.029*zRNO3+...
     &                          +  0.069 * RXRAT( 372 )      ! BUTADIENE13+CL=0.069*zRNO3+...
     &                          +  0.085 * RXRAT( 373 )      ! ISOPRENE+CL=0.085*zRNO3+...
     &                          +  0.582 * RXRAT( 374 )      ! APIN+CL=0.582*zRNO3+...
     &                          +  0.106 * RXRAT( 376 )      ! TOLUENE+CL=0.106*zRNO3+...
     &                          +  0.136 * RXRAT( 377 )      ! MXYL+CL=0.136*zRNO3+...
     &                          +  0.136 * RXRAT( 378 )      ! OXYL+CL=0.136*zRNO3+...
     &                          +  0.136 * RXRAT( 379 )      ! PXYL+CL=0.136*zRNO3+...
     &                          +  0.162 * RXRAT( 380 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.030 * RXRAT( 383 )      ! ALK2+CL=0.03*zRNO3+...
     &                          +  0.070 * RXRAT( 384 )      ! ALK3+CL=0.07*zRNO3+...
     &                          +  0.165 * RXRAT( 385 )      ! ALK4+CL=0.165*zRNO3+...
     &                          +  0.352 * RXRAT( 386 )      ! ALK5+CL=0.352*zRNO3+...
     &                          +  0.127 * RXRAT( 387 )      ! OLE1+CL=0.127*zRNO3+...
     &                          +  0.106 * RXRAT( 388 )      ! OLE2+CL=0.106*zRNO3+...
     &                          +  0.160 * RXRAT( 389 )      ! ARO1+CL=0.16*zRNO3+...
     &                          +  0.172 * RXRAT( 390 )      ! ARO2+CL=0.172*zRNO3+...
     &                          +  0.582 * RXRAT( 391 )      ! TERP+CL=0.582*zRNO3+...
     &                          +  0.582 * RXRAT( 392 )      ! SESQ+CL=0.582*zRNO3+...
      LOSS( zRNO3            ) =           RXRAT( 249 )      ! zRNO3=RNO3
     &                          +          RXRAT( 250 )      ! zRNO3=PRD2+HO2
     &                          +          RXRAT( 251 )      ! zRNO3=

      PROD( xRCHO            ) =   0.376 * RXRAT( 148 )      ! MEK+OH=0.376*xRCHO+...
     &                          +  0.400 * RXRAT( 153 )      ! RCOOH+OH=0.4*xRCHO+xHO2+...
     &                          +  0.205 * RXRAT( 156 )      ! ROOH+OH=0.205*xRCHO+...
     &                          +  0.075 * RXRAT( 158 )      ! R6OOH+OH=0.075*xRCHO+...
     &                          +  0.058 * RXRAT( 159 )      ! R6OOH=0.058*xRCHO+...
     &                          +  0.448 * RXRAT( 160 )      ! RAOOH+OH=0.448*xRCHO+...
     &                          +  0.407 * RXRAT( 178 )      ! AFG1+OH=0.407*xRCHO+...
     &                          +  0.407 * RXRAT( 181 )      ! AFG2+OH=0.407*xRCHO+...
     &                          +  0.013 * RXRAT( 195 )      ! IPRD+OH=0.013*xRCHO+...
     &                          +  0.218 * RXRAT( 197 )      ! IPRD+NO3=0.218*xRCHO+...
     &                          +  0.402 * RXRAT( 199 )      ! PRD2+OH=0.402*xRCHO+...
     &                          +  0.780 * RXRAT( 200 )      ! PRD2=0.78*xRCHO+0.4*MECO3+...
     &                          +  0.036 * RXRAT( 201 )      ! RNO3+OH=0.036*xRCHO+...
     &                          +  0.063 * RXRAT( 202 )      ! RNO3=0.063*xRCHO+...
     &                          +  0.084 * RXRAT( 213 )      ! RCOOOH+OH=0.084*xRCHO+...
     &                          +          RXRAT( 268 )      ! ETHENE+NO3=xRCHO+RO2C+xHO2+...
     &                          +  0.776 * RXRAT( 282 )      ! APIN+OH=0.776*xRCHO+...
     &                          +  0.240 * RXRAT( 283 )      ! APIN+O3=0.24*xRCHO+...
     &                          +  0.684 * RXRAT( 284 )      ! APIN+NO3=0.684*xRCHO+...
     &                          +  0.261 * RXRAT( 296 )      ! ALK2+OH=0.261*xRCHO+...
     &                          +  0.122 * RXRAT( 297 )      ! ALK3+OH=0.122*xRCHO+...
     &                          +  0.236 * RXRAT( 298 )      ! ALK4+OH=0.236*xRCHO+...
     &                          +  0.209 * RXRAT( 299 )      ! ALK5+OH=0.209*xRCHO+...
     &                          +  0.666 * RXRAT( 300 )      ! OLE1+OH=0.666*xRCHO+...
     &                          +  0.044 * RXRAT( 301 )      ! OLE1+O3=0.044*xRCHO+...
     &                          +  0.003 * RXRAT( 302 )      ! OLE1+NO3=0.003*xRCHO+...
     &                          +  0.510 * RXRAT( 304 )      ! OLE2+OH=0.51*xRCHO+...
     &                          +  0.019 * RXRAT( 305 )      ! OLE2+O3=0.019*xRCHO+...
     &                          +  0.163 * RXRAT( 306 )      ! OLE2+NO3=0.163*xRCHO+...
     &                          +  0.385 * RXRAT( 310 )      ! TERP+OH=0.385*xRCHO+...
     &                          +  0.208 * RXRAT( 311 )      ! TERP+O3=0.208*xRCHO+...
     &                          +  0.403 * RXRAT( 312 )      ! TERP+NO3=0.403*xRCHO+...
     &                          +  0.385 * RXRAT( 314 )      ! SESQ+OH=0.385*xRCHO+...
     &                          +  0.208 * RXRAT( 315 )      ! SESQ+O3=0.208*xRCHO+...
     &                          +  0.403 * RXRAT( 316 )      ! SESQ+NO3=0.403*xRCHO+...
     &                          +  0.840 * RXRAT( 345 )      ! MEK+CL=0.84*xRCHO+...
     &                          +  0.020 * RXRAT( 346 )      ! RNO3+CL=0.02*xRCHO+...
     &                          +  0.591 * RXRAT( 347 )      ! PRD2+CL=0.591*xRCHO+...
     &                          +  0.197 * RXRAT( 352 )      ! ROOH+CL=0.197*xRCHO+...
     &                          +  0.676 * RXRAT( 353 )      ! R6OOH+CL=0.676*xRCHO+...
     &                          +  0.448 * RXRAT( 354 )      ! RAOOH+CL=0.448*xRCHO+...
     &                          +  0.185 * RXRAT( 374 )      ! APIN+CL=0.185*xRCHO+...
     &                          +  0.482 * RXRAT( 383 )      ! ALK2+CL=0.482*xRCHO+...
     &                          +  0.343 * RXRAT( 384 )      ! ALK3+CL=0.343*xRCHO+...
     &                          +  0.412 * RXRAT( 385 )      ! ALK4+CL=0.412*xRCHO+...
     &                          +  0.258 * RXRAT( 386 )      ! ALK5+CL=0.258*xRCHO+...
     &                          +  0.072 * RXRAT( 387 )      ! OLE1+CL=0.072*xRCHO+...
     &                          +  0.317 * RXRAT( 388 )      ! OLE2+CL=0.317*xRCHO+...
     &                          +  0.185 * RXRAT( 391 )      ! TERP+CL=0.185*xRCHO+...
     &                          +  0.185 * RXRAT( 392 )      ! SESQ+CL=0.185*xRCHO+...
      LOSS( xRCHO            ) =           RXRAT( 219 )      ! xRCHO=RCHO
     &                          +          RXRAT( 220 )      ! xRCHO=

      PROD( HCOOH            ) =   0.333 * RXRAT( 187 )      ! MACR+O3=0.333*HCOOH+...
     &                          +  0.351 * RXRAT( 192 )      ! MVK+O3=0.351*HCOOH+...
     &                          +  0.100 * RXRAT( 196 )      ! IPRD+O3=0.1*HCOOH+0.4*HO2+...
     &                          +  0.185 * RXRAT( 207 )      ! ACROLEIN+O3=0.185*HCOOH+...
     &                          +  0.370 * RXRAT( 267 )      ! ETHENE+O3=0.37*HCOOH+...
     &                          +  0.185 * RXRAT( 271 )      ! PROPENE+O3=0.185*HCOOH+...
     &                          +  0.185 * RXRAT( 275 )      ! BUTADIENE13+O3=0.185*HCOOH+...
     &                          +  0.204 * RXRAT( 279 )      ! ISOPRENE+O3=0.204*HCOOH+...
     &                          +  0.300 * RXRAT( 286 )      ! ACETYLENE+OH=0.3*HCOOH+...
     &                          +  0.185 * RXRAT( 301 )      ! OLE1+O3=0.185*HCOOH+...
     &                          +  0.024 * RXRAT( 305 )      ! OLE2+O3=0.024*HCOOH+...
     &                          +  0.172 * RXRAT( 311 )      ! TERP+O3=0.172*HCOOH+...
     &                          +  0.172 * RXRAT( 315 )      ! SESQ+O3=0.172*HCOOH+...
      LOSS( HCOOH            ) =           RXRAT( 151 )      ! HCOOH+OH=HO2+CO2

      PROD( xMGLY            ) =   0.491 * RXRAT( 152 )      ! CCOOH+OH=0.491*xMGLY+...
     &                          +  0.252 * RXRAT( 160 )      ! RAOOH+OH=0.252*xMGLY+...
     &                          +  0.250 * RXRAT( 170 )      ! CRES+OH=0.25*xMGLY+...
     &                          +  0.267 * RXRAT( 178 )      ! AFG1+OH=0.267*xMGLY+...
     &                          +  0.267 * RXRAT( 181 )      ! AFG2+OH=0.267*xMGLY+...
     &                          +  0.153 * RXRAT( 184 )      ! AFG3+OH=0.153*xMGLY+...
     &                          +  0.095 * RXRAT( 185 )      ! AFG3+O3=0.095*xMGLY+...
     &                          +  0.084 * RXRAT( 186 )      ! MACR+OH=0.084*xMGLY+...
     &                          +  0.300 * RXRAT( 191 )      ! MVK+OH=0.3*xMGLY+...
     &                          +  0.174 * RXRAT( 195 )      ! IPRD+OH=0.174*xMGLY+...
     &                          +  0.008 * RXRAT( 197 )      ! IPRD+NO3=0.008*xMGLY+...
     &                          +  0.020 * RXRAT( 282 )      ! APIN+OH=0.02*xMGLY+...
     &                          +  0.002 * RXRAT( 284 )      ! APIN+NO3=0.002*xMGLY+...
     &                          +  0.151 * RXRAT( 289 )      ! TOLUENE+OH=0.151*xMGLY+...
     &                          +  0.380 * RXRAT( 290 )      ! MXYL+OH=0.38*xMGLY+...
     &                          +  0.238 * RXRAT( 291 )      ! OXYL+OH=0.238*xMGLY+...
     &                          +  0.112 * RXRAT( 292 )      ! PXYL+OH=0.112*xMGLY+...
     &                          +  0.405 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.100 * RXRAT( 308 )      ! ARO1+OH=0.1*xMGLY+...
     &                          +  0.312 * RXRAT( 309 )      ! ARO2+OH=0.312*xMGLY+...
     &                          +  0.003 * RXRAT( 310 )      ! TERP+OH=0.003*xMGLY+...
     &                          +  0.003 * RXRAT( 311 )      ! TERP+O3=0.003*xMGLY+...
     &                          +  0.003 * RXRAT( 314 )      ! SESQ+OH=0.003*xMGLY+...
     &                          +  0.003 * RXRAT( 315 )      ! SESQ+O3=0.003*xMGLY+...
     &                          +  0.252 * RXRAT( 354 )      ! RAOOH+CL=0.252*xMGLY+...
     &                          +  0.017 * RXRAT( 358 )      ! IPRD+CL=0.017*xMGLY+...
      LOSS( xMGLY            ) =           RXRAT( 229 )      ! xMGLY=MGLY
     &                          +          RXRAT( 230 )      ! xMGLY=

      PROD( xBACL            ) =   0.457 * RXRAT( 153 )      ! RCOOH+OH=0.457*xBACL+xHO2+...
     &                          +  0.019 * RXRAT( 184 )      ! AFG3+OH=0.019*xBACL+...
     &                          +  0.023 * RXRAT( 282 )      ! APIN+OH=0.023*xBACL+...
     &                          +  0.081 * RXRAT( 283 )      ! APIN+O3=0.081*xBACL+...
     &                          +  0.185 * RXRAT( 291 )      ! OXYL+OH=0.185*xBACL+...
     &                          +  0.112 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.134 * RXRAT( 309 )      ! ARO2+OH=0.134*xBACL+...
     &                          +  0.009 * RXRAT( 310 )      ! TERP+OH=0.009*xBACL+...
     &                          +  0.039 * RXRAT( 311 )      ! TERP+O3=0.039*xBACL+...
     &                          +  0.009 * RXRAT( 314 )      ! SESQ+OH=0.009*xBACL+...
     &                          +  0.039 * RXRAT( 315 )      ! SESQ+O3=0.039*xBACL+...
     &                          +  0.003 * RXRAT( 374 )      ! APIN+CL=0.003*xBACL+...
     &                          +  0.003 * RXRAT( 391 )      ! TERP+CL=0.003*xBACL+...
     &                          +  0.003 * RXRAT( 392 )      ! SESQ+CL=0.003*xBACL+...
      LOSS( xBACL            ) =           RXRAT( 231 )      ! xBACL=BACL
     &                          +          RXRAT( 232 )      ! xBACL=

      PROD( ROOH             ) =           RXRAT( 252 )      ! yROOH=ROOH
      LOSS( ROOH             ) =           RXRAT( 156 )      ! ROOH+OH=0.744*OH+...
     &                          +          RXRAT( 157 )      ! ROOH=RCHO+HO2+OH
     &                          +          RXRAT( 352 )      ! ROOH+CL=HCL+0.414*OH+...

      PROD( xPROD2           ) =   0.034 * RXRAT( 156 )      ! ROOH+OH=0.034*xPROD2+...
     &                          +  0.084 * RXRAT( 158 )      ! R6OOH+OH=0.084*xPROD2+...
     &                          +  0.698 * RXRAT( 159 )      ! R6OOH=0.698*xPROD2+...
     &                          +  0.332 * RXRAT( 195 )      ! IPRD+OH=0.332*xPROD2+...
     &                          +  0.007 * RXRAT( 199 )      ! PRD2+OH=0.007*xPROD2+...
     &                          +  0.031 * RXRAT( 201 )      ! RNO3+OH=0.031*xPROD2+...
     &                          +  0.261 * RXRAT( 202 )      ! RNO3=0.261*xPROD2+...
     &                          +  0.146 * RXRAT( 298 )      ! ALK4+OH=0.146*xPROD2+...
     &                          +  0.407 * RXRAT( 299 )      ! ALK5+OH=0.407*xPROD2+...
     &                          +  0.168 * RXRAT( 300 )      ! OLE1+OH=0.168*xPROD2+...
     &                          +  0.023 * RXRAT( 304 )      ! OLE2+OH=0.023*xPROD2+...
     &                          +  0.309 * RXRAT( 308 )      ! ARO1+OH=0.309*xPROD2+...
     &                          +  0.057 * RXRAT( 309 )      ! ARO2+OH=0.057*xPROD2+...
     &                          +  0.409 * RXRAT( 310 )      ! TERP+OH=0.409*xPROD2+...
     &                          +  0.409 * RXRAT( 314 )      ! SESQ+OH=0.409*xPROD2+...
     &                          +  0.046 * RXRAT( 346 )      ! RNO3+CL=0.046*xPROD2+...
     &                          +  0.040 * RXRAT( 347 )      ! PRD2+CL=0.04*xPROD2+...
     &                          +  0.280 * RXRAT( 353 )      ! R6OOH+CL=0.28*xPROD2+...
     &                          +  0.130 * RXRAT( 385 )      ! ALK4+CL=0.13*xPROD2+...
     &                          +  0.378 * RXRAT( 386 )      ! ALK5+CL=0.378*xPROD2+...
     &                          +  0.840 * RXRAT( 389 )      ! ARO1+CL=0.84*xPROD2+...
     &                          +  0.359 * RXRAT( 390 )      ! ARO2+CL=0.359*xPROD2+...
      LOSS( xPROD2           ) =           RXRAT( 225 )      ! xPROD2=PRD2
     &                          +          RXRAT( 226 )      ! xPROD2=

      PROD( R6OOH            ) =           RXRAT( 255 )      ! yR6OOH=R6OOH
      LOSS( R6OOH            ) =           RXRAT( 158 )      ! R6OOH+OH=0.84*OH+...
     &                          +          RXRAT( 159 )      ! R6OOH=OH+0.142*HO2+...
     &                          +          RXRAT( 353 )      ! R6OOH+CL=HCL+0.145*OH+...

      PROD( PRD2             ) =   0.840 * RXRAT( 158 )      ! R6OOH+OH=0.84*PRD2+...
     &                          +  0.142 * RXRAT( 159 )      ! R6OOH=0.142*PRD2+0.142*HO2+...
     &                          +  0.074 * RXRAT( 160 )      ! RAOOH+OH=0.074*PRD2+...
     &                          +          RXRAT( 183 )      ! AFG2=PRD2
     &                          +  0.600 * RXRAT( 194 )      ! MVK=0.6*PRD2+0.6*CO+...
     &                          +  0.008 * RXRAT( 201 )      ! RNO3+OH=0.008*PRD2+...
     &                          +  0.190 * RXRAT( 202 )      ! RNO3=0.19*PRD2+0.554*xHO2+...
     &                          +          RXRAT( 225 )      ! xPROD2=PRD2
     &                          +          RXRAT( 250 )      ! zRNO3=PRD2+HO2
     &                          +          RXRAT( 256 )      ! yR6OOH=PRD2
     &                          +          RXRAT( 259 )      ! yRAOOH=PRD2
     &                          +  0.125 * RXRAT( 275 )      ! BUTADIENE13+O3=0.125*PRD2+...
     &                          +  0.750 * RXRAT( 277 )      ! BUTADIENE13+O3P=0.75*PRD2+...
     &                          +  0.100 * RXRAT( 279 )      ! ISOPRENE+O3=0.1*PRD2+...
     &                          +  0.750 * RXRAT( 281 )      ! ISOPRENE+O3P=0.75*PRD2+...
     &                          +  0.255 * RXRAT( 283 )      ! APIN+O3=0.255*PRD2+...
     &                          +          RXRAT( 285 )      ! APIN+O3P=PRD2+TRPRXN
     &                          +  0.268 * RXRAT( 301 )      ! OLE1+O3=0.268*PRD2+...
     &                          +  0.160 * RXRAT( 303 )      ! OLE1+O3P=0.16*PRD2+...
     &                          +  0.037 * RXRAT( 305 )      ! OLE2+O3=0.037*PRD2+...
     &                          +  0.170 * RXRAT( 307 )      ! OLE2+O3P=0.17*PRD2+...
     &                          +  0.502 * RXRAT( 311 )      ! TERP+O3=0.502*PRD2+...
     &                          +  0.763 * RXRAT( 313 )      ! TERP+O3P=0.763*PRD2+...
     &                          +  0.502 * RXRAT( 315 )      ! SESQ+O3=0.502*PRD2+...
     &                          +  0.763 * RXRAT( 317 )      ! SESQ+O3P=0.763*PRD2+...
     &                          +  0.012 * RXRAT( 346 )      ! RNO3+CL=0.012*PRD2+...
     &                          +  0.145 * RXRAT( 353 )      ! R6OOH+CL=0.145*PRD2+...
     &                          +  0.074 * RXRAT( 354 )      ! RAOOH+CL=0.074*PRD2+...
      LOSS( PRD2             ) =   0.671 * RXRAT( 199 )      ! PRD2+OH=0.329*PRD2+...
     &                          +          RXRAT( 200 )      ! PRD2=0.913*xHO2+0.4*MECO3+...
     &                          +  0.884 * RXRAT( 347 )      ! PRD2+CL=0.116*PRD2+...

      PROD( yR6OOH           ) =   0.858 * RXRAT( 159 )      ! R6OOH=0.858*yR6OOH+...
     &                          +  0.713 * RXRAT( 160 )      ! RAOOH+OH=0.713*yR6OOH+...
     &                          +  0.800 * RXRAT( 170 )      ! CRES+OH=0.8*yR6OOH+...
     &                          +  0.783 * RXRAT( 178 )      ! AFG1+OH=0.783*yR6OOH+...
     &                          +  0.652 * RXRAT( 179 )      ! AFG1+O3=0.652*yR6OOH+...
     &                          +  0.783 * RXRAT( 181 )      ! AFG2+OH=0.783*yR6OOH+...
     &                          +  0.652 * RXRAT( 182 )      ! AFG2+O3=0.652*yR6OOH+...
     &                          +  0.794 * RXRAT( 184 )      ! AFG3+OH=0.794*yR6OOH+...
     &                          +  0.264 * RXRAT( 185 )      ! AFG3+O3=0.264*yR6OOH+...
     &                          +  0.711 * RXRAT( 195 )      ! IPRD+OH=0.711*yR6OOH+...
     &                          +  0.048 * RXRAT( 196 )      ! IPRD+O3=0.048*yR6OOH+...
     &                          +  0.850 * RXRAT( 197 )      ! IPRD+NO3=0.85*yR6OOH+...
     &                          +  0.528 * RXRAT( 199 )      ! PRD2+OH=0.528*yR6OOH+...
     &                          +          RXRAT( 200 )      ! PRD2=yR6OOH+0.4*MECO3+...
     &                          +  0.636 * RXRAT( 201 )      ! RNO3+OH=0.636*yR6OOH+...
     &                          +  0.591 * RXRAT( 202 )      ! RNO3=0.591*yR6OOH+...
     &                          +          RXRAT( 278 )      ! ISOPRENE+OH=yR6OOH+...
     &                          +  0.200 * RXRAT( 279 )      ! ISOPRENE+O3=0.2*yR6OOH+...
     &                          +          RXRAT( 280 )      ! ISOPRENE+NO3=yR6OOH+...
     &                          +  0.250 * RXRAT( 281 )      ! ISOPRENE+O3P=0.25*yR6OOH+...
     &                          +          RXRAT( 282 )      ! APIN+OH=yR6OOH+0.004*xRCO3+...
     &                          +  0.737 * RXRAT( 283 )      ! APIN+O3=0.737*yR6OOH+...
     &                          +          RXRAT( 284 )      ! APIN+NO3=yR6OOH+0.643*xNO2+...
     &                          +  0.073 * RXRAT( 289 )      ! TOLUENE+OH=0.073*yR6OOH+...
     &                          +  0.047 * RXRAT( 290 )      ! MXYL+OH=0.047*yR6OOH+...
     &                          +  0.055 * RXRAT( 291 )      ! OXYL+OH=0.055*yR6OOH+...
     &                          +  0.102 * RXRAT( 292 )      ! PXYL+OH=0.102*yR6OOH+...
     &                          +  0.043 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.017 * RXRAT( 297 )      ! ALK3+OH=0.017*yR6OOH+...
     &                          +          RXRAT( 298 )      ! ALK4+OH=yR6OOH+0.01*xMEO2+...
     &                          +          RXRAT( 299 )      ! ALK5+OH=yR6OOH+1.605*RO2C+...
     &                          +  0.831 * RXRAT( 300 )      ! OLE1+OH=0.831*yR6OOH+...
     &                          +  0.052 * RXRAT( 301 )      ! OLE1+O3=0.052*yR6OOH+...
     &                          +  0.831 * RXRAT( 302 )      ! OLE1+NO3=0.831*yR6OOH+...
     &                          +  0.681 * RXRAT( 304 )      ! OLE2+OH=0.681*yR6OOH+...
     &                          +  0.136 * RXRAT( 305 )      ! OLE2+O3=0.136*yR6OOH+...
     &                          +  0.681 * RXRAT( 306 )      ! OLE2+NO3=0.681*yR6OOH+...
     &                          +  0.369 * RXRAT( 308 )      ! ARO1+OH=0.369*yR6OOH+...
     &                          +  0.101 * RXRAT( 309 )      ! ARO2+OH=0.101*yR6OOH+...
     &                          +          RXRAT( 310 )      ! TERP+OH=yR6OOH+0.064*xRCO3+...
     &                          +  0.428 * RXRAT( 311 )      ! TERP+O3=0.428*yR6OOH+...
     &                          +          RXRAT( 312 )      ! TERP+NO3=yR6OOH+0.287*xNO2+...
     &                          +          RXRAT( 314 )      ! SESQ+OH=yR6OOH+0.064*xRCO3+...
     &                          +  0.428 * RXRAT( 315 )      ! SESQ+O3=0.428*yR6OOH+...
     &                          +          RXRAT( 316 )      ! SESQ+NO3=yR6OOH+0.287*xNO2+...
     &                          +  0.908 * RXRAT( 346 )      ! RNO3+CL=0.908*yR6OOH+...
     &                          +  0.686 * RXRAT( 347 )      ! PRD2+CL=0.686*yR6OOH+...
     &                          +          RXRAT( 350 )      ! CRES+CL=yR6OOH+xHO2+xBALD+HCL
     &                          +  0.855 * RXRAT( 353 )      ! R6OOH+CL=0.855*yR6OOH+...
     &                          +  0.713 * RXRAT( 354 )      ! RAOOH+CL=0.713*yR6OOH+...
     &                          +  0.762 * RXRAT( 358 )      ! IPRD+CL=0.762*yR6OOH+...
     &                          +          RXRAT( 373 )      ! ISOPRENE+CL=yR6OOH+...
     &                          +          RXRAT( 374 )      ! APIN+CL=yR6OOH+0.252*xHO2+...
     &                          +  0.017 * RXRAT( 384 )      ! ALK3+CL=0.017*yR6OOH+...
     &                          +          RXRAT( 385 )      ! ALK4+CL=yR6OOH+0.827*xHO2+...
     &                          +          RXRAT( 386 )      ! ALK5+CL=yR6OOH+0.647*xHO2+...
     &                          +  0.831 * RXRAT( 387 )      ! OLE1+CL=0.831*yR6OOH+...
     &                          +  0.681 * RXRAT( 388 )      ! OLE2+CL=0.681*yR6OOH+...
     &                          +          RXRAT( 391 )      ! TERP+CL=yR6OOH+0.252*xHO2+...
     &                          +          RXRAT( 392 )      ! SESQ+CL=yR6OOH+0.068*xCL+...
      LOSS( yR6OOH           ) =           RXRAT( 255 )      ! yR6OOH=R6OOH
     &                          +          RXRAT( 256 )      ! yR6OOH=PRD2
     &                          +          RXRAT( 257 )      ! yR6OOH=

      PROD( RAOOH            ) =           RXRAT( 258 )      ! yRAOOH=RAOOH
      LOSS( RAOOH            ) =           RXRAT( 160 )      ! RAOOH+OH=0.139*OH+...
     &                          +          RXRAT( 161 )      ! RAOOH=OH+HO2+0.5*GLY+...
     &                          +          RXRAT( 354 )      ! RAOOH+CL=0.404*HCL+...

      PROD( MGLY             ) =   0.147 * RXRAT( 160 )      ! RAOOH+OH=0.147*MGLY+...
     &                          +  0.500 * RXRAT( 161 )      ! RAOOH=0.5*MGLY+HO2+0.5*GLY+...
     &                          +  0.568 * RXRAT( 179 )      ! AFG1+O3=0.568*MGLY+...
     &                          +  0.305 * RXRAT( 180 )      ! AFG1=0.305*MGLY+0.173*MEO2+...
     &                          +  0.568 * RXRAT( 182 )      ! AFG2+O3=0.568*MGLY+...
     &                          +  0.184 * RXRAT( 185 )      ! AFG3+O3=0.184*MGLY+...
     &                          +  0.900 * RXRAT( 187 )      ! MACR+O3=0.9*MGLY+0.108*HO2+...
     &                          +  0.950 * RXRAT( 192 )      ! MVK+O3=0.95*MGLY+0.064*HO2+...
     &                          +  0.742 * RXRAT( 196 )      ! IPRD+O3=0.742*MGLY+0.4*HO2+...
     &                          +          RXRAT( 229 )      ! xMGLY=MGLY
     &                          +  0.147 * RXRAT( 354 )      ! RAOOH+CL=0.147*MGLY+...
      LOSS( MGLY             ) =           RXRAT( 166 )      ! MGLY=HO2+CO+MECO3
     &                          +          RXRAT( 167 )      ! MGLY+OH=CO+MECO3
     &                          +          RXRAT( 168 )      ! MGLY+NO3=HNO3+CO+MECO3
     &                          +          RXRAT( 349 )      ! MGLY+CL=HCL+CO+MECO3

      PROD( IPRD             ) =   0.139 * RXRAT( 160 )      ! RAOOH+OH=0.139*IPRD+...
     &                          +  0.003 * RXRAT( 185 )      ! AFG3+O3=0.003*IPRD+...
     &                          +          RXRAT( 245 )      ! xIPRD=IPRD
     &                          +  0.150 * RXRAT( 279 )      ! ISOPRENE+O3=0.15*IPRD+...
     &                          +  0.139 * RXRAT( 354 )      ! RAOOH+CL=0.139*IPRD+...
      LOSS( IPRD             ) =           RXRAT( 195 )      ! IPRD+OH=0.289*MACO3+...
     &                          +          RXRAT( 196 )      ! IPRD+O3=0.285*OH+0.4*HO2+...
     &                          +          RXRAT( 197 )      ! IPRD+NO3=0.15*MACO3+...
     &                          +          RXRAT( 198 )      ! IPRD=1.233*HO2+0.467*MECO3+...
     &                          +          RXRAT( 358 )      ! IPRD+CL=0.401*HCL+...

      PROD( xGLY             ) =   0.026 * RXRAT( 160 )      ! RAOOH+OH=0.026*xGLY+...
     &                          +  0.107 * RXRAT( 178 )      ! AFG1+OH=0.107*xGLY+...
     &                          +  0.107 * RXRAT( 181 )      ! AFG2+OH=0.107*xGLY+...
     &                          +  0.274 * RXRAT( 184 )      ! AFG3+OH=0.274*xGLY+...
     &                          +  0.150 * RXRAT( 195 )      ! IPRD+OH=0.15*xGLY+...
     &                          +  0.083 * RXRAT( 206 )      ! ACROLEIN+OH=0.083*xGLY+...
     &                          +  0.012 * RXRAT( 269 )      ! ETHENE+O3P=0.012*xGLY+...
     &                          +  0.002 * RXRAT( 283 )      ! APIN+O3=0.002*xGLY+...
     &                          +  0.290 * RXRAT( 288 )      ! BENZENE+OH=0.29*xGLY+...
     &                          +  0.238 * RXRAT( 289 )      ! TOLUENE+OH=0.238*xGLY+...
     &                          +  0.100 * RXRAT( 290 )      ! MXYL+OH=0.1*xGLY+0.52*xHO2+...
     &                          +  0.084 * RXRAT( 291 )      ! OXYL+OH=0.084*xGLY+...
     &                          +  0.286 * RXRAT( 292 )      ! PXYL+OH=0.286*xGLY+...
     &                          +  0.074 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.158 * RXRAT( 308 )      ! ARO1+OH=0.158*xGLY+...
     &                          +  0.088 * RXRAT( 309 )      ! ARO2+OH=0.088*xGLY+...
     &                          +  0.026 * RXRAT( 354 )      ! RAOOH+CL=0.026*xGLY+...
     &                          +  0.274 * RXRAT( 355 )      ! ACROLEIN+CL=0.274*xGLY+...
     &                          +  0.007 * RXRAT( 374 )      ! APIN+CL=0.007*xGLY+...
     &                          +  0.007 * RXRAT( 391 )      ! TERP+CL=0.007*xGLY+...
     &                          +  0.007 * RXRAT( 392 )      ! SESQ+CL=0.007*xGLY+...
      LOSS( xGLY             ) =           RXRAT( 227 )      ! xGLY=GLY
     &                          +          RXRAT( 228 )      ! xGLY=

      PROD( xMEK             ) =   0.030 * RXRAT( 160 )      ! RAOOH+OH=0.03*xMEK+...
     &                          +  0.129 * RXRAT( 178 )      ! AFG1+OH=0.129*xMEK+...
     &                          +  0.129 * RXRAT( 181 )      ! AFG2+OH=0.129*xMEK+...
     &                          +  0.416 * RXRAT( 186 )      ! MACR+OH=0.416*xMEK+...
     &                          +  0.150 * RXRAT( 195 )      ! IPRD+OH=0.15*xMEK+...
     &                          +  0.115 * RXRAT( 199 )      ! PRD2+OH=0.115*xMEK+...
     &                          +  0.170 * RXRAT( 201 )      ! RNO3+OH=0.17*xMEK+...
     &                          +  0.083 * RXRAT( 202 )      ! RNO3=0.083*xMEK+0.554*xHO2+...
     &                          +  0.332 * RXRAT( 297 )      ! ALK3+OH=0.332*xMEK+...
     &                          +  0.106 * RXRAT( 298 )      ! ALK4+OH=0.106*xMEK+...
     &                          +  0.086 * RXRAT( 299 )      ! ALK5+OH=0.086*xMEK+...
     &                          +  0.080 * RXRAT( 304 )      ! OLE2+OH=0.08*xMEK+...
     &                          +  0.001 * RXRAT( 305 )      ! OLE2+O3=0.001*xMEK+...
     &                          +  0.002 * RXRAT( 306 )      ! OLE2+NO3=0.002*xMEK+...
     &                          +  0.007 * RXRAT( 310 )      ! TERP+OH=0.007*xMEK+...
     &                          +  0.007 * RXRAT( 314 )      ! SESQ+OH=0.007*xMEK+...
     &                          +  0.041 * RXRAT( 346 )      ! RNO3+CL=0.041*xMEK+...
     &                          +  0.051 * RXRAT( 347 )      ! PRD2+CL=0.051*xMEK+...
     &                          +  0.285 * RXRAT( 352 )      ! ROOH+CL=0.285*xMEK+...
     &                          +  0.030 * RXRAT( 354 )      ! RAOOH+CL=0.03*xMEK+...
     &                          +  0.253 * RXRAT( 384 )      ! ALK3+CL=0.253*xMEK+...
     &                          +  0.076 * RXRAT( 385 )      ! ALK4+CL=0.076*xMEK+...
     &                          +  0.041 * RXRAT( 386 )      ! ALK5+CL=0.041*xMEK+...
     &                          +  0.042 * RXRAT( 388 )      ! OLE2+CL=0.042*xMEK+...
      LOSS( xMEK             ) =           RXRAT( 223 )      ! xMEK=MEK
     &                          +          RXRAT( 224 )      ! xMEK=

      PROD( xAFG1            ) =   0.073 * RXRAT( 160 )      ! RAOOH+OH=0.073*xAFG1+...
     &                          +  0.195 * RXRAT( 184 )      ! AFG3+OH=0.195*xAFG1+...
     &                          +  0.001 * RXRAT( 277 )      ! BUTADIENE13+O3P=...
     &                          +  0.029 * RXRAT( 288 )      ! BENZENE+OH=0.029*xAFG1+...
     &                          +  0.195 * RXRAT( 289 )      ! TOLUENE+OH=0.195*xAFG1+...
     &                          +  0.336 * RXRAT( 290 )      ! MXYL+OH=0.336*xAFG1+...
     &                          +  0.253 * RXRAT( 291 )      ! OXYL+OH=0.253*xAFG1+...
     &                          +  0.045 * RXRAT( 292 )      ! PXYL+OH=0.045*xAFG1+...
     &                          +  0.088 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.072 * RXRAT( 308 )      ! ARO1+OH=0.072*xAFG1+...
     &                          +  0.221 * RXRAT( 309 )      ! ARO2+OH=0.221*xAFG1+...
     &                          +  0.073 * RXRAT( 354 )      ! RAOOH+CL=0.073*xAFG1+...
     &                          +  0.009 * RXRAT( 358 )      ! IPRD+CL=0.009*xAFG1+...
     &                          +  0.006 * RXRAT( 374 )      ! APIN+CL=0.006*xAFG1+...
     &                          +  0.006 * RXRAT( 391 )      ! TERP+CL=0.006*xAFG1+...
     &                          +  0.006 * RXRAT( 392 )      ! SESQ+CL=0.006*xAFG1+...
      LOSS( xAFG1            ) =           RXRAT( 235 )      ! xAFG1=AFG1
     &                          +          RXRAT( 236 )      ! xAFG1=

      PROD( xAFG2            ) =   0.073 * RXRAT( 160 )      ! RAOOH+OH=0.073*xAFG2+...
     &                          +  0.195 * RXRAT( 184 )      ! AFG3+OH=0.195*xAFG2+...
     &                          +  0.001 * RXRAT( 277 )      ! BUTADIENE13+O3P=...
     &                          +  0.261 * RXRAT( 288 )      ! BENZENE+OH=0.261*xAFG2+...
     &                          +  0.195 * RXRAT( 289 )      ! TOLUENE+OH=0.195*xAFG2+...
     &                          +  0.144 * RXRAT( 290 )      ! MXYL+OH=0.144*xAFG2+...
     &                          +  0.253 * RXRAT( 291 )      ! OXYL+OH=0.253*xAFG2+...
     &                          +  0.067 * RXRAT( 292 )      ! PXYL+OH=0.067*xAFG2+...
     &                          +  0.352 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.185 * RXRAT( 308 )      ! ARO1+OH=0.185*xAFG2+...
     &                          +  0.247 * RXRAT( 309 )      ! ARO2+OH=0.247*xAFG2+...
     &                          +  0.073 * RXRAT( 354 )      ! RAOOH+CL=0.073*xAFG2+...
     &                          +  0.009 * RXRAT( 358 )      ! IPRD+CL=0.009*xAFG2+...
     &                          +  0.006 * RXRAT( 374 )      ! APIN+CL=0.006*xAFG2+...
     &                          +  0.006 * RXRAT( 391 )      ! TERP+CL=0.006*xAFG2+...
     &                          +  0.006 * RXRAT( 392 )      ! SESQ+CL=0.006*xAFG2+...
      LOSS( xAFG2            ) =           RXRAT( 237 )      ! xAFG2=AFG2
     &                          +          RXRAT( 238 )      ! xAFG2=

      PROD( GLY              ) =   0.500 * RXRAT( 161 )      ! RAOOH=0.5*GLY+HO2+OH+...
     &                          +  0.432 * RXRAT( 179 )      ! AFG1+O3=0.432*GLY+...
     &                          +  0.195 * RXRAT( 180 )      ! AFG1=0.195*GLY+0.173*MEO2+...
     &                          +  0.432 * RXRAT( 182 )      ! AFG2+O3=0.432*GLY+...
     &                          +  0.366 * RXRAT( 185 )      ! AFG3+O3=0.366*GLY+...
     &                          +  0.023 * RXRAT( 196 )      ! IPRD+O3=0.023*GLY+0.4*HO2+...
     &                          +  0.500 * RXRAT( 207 )      ! ACROLEIN+O3=0.5*GLY+...
     &                          +          RXRAT( 227 )      ! xGLY=GLY
     &                          +  0.700 * RXRAT( 286 )      ! ACETYLENE+OH=0.7*GLY+...
      LOSS( GLY              ) =           RXRAT( 162 )      ! GLY=2*CO+2*HO2
     &                          +          RXRAT( 163 )      ! GLY=HCHO+CO
     &                          +          RXRAT( 164 )      ! GLY+OH=0.63*HO2+1.26*CO+...
     &                          +          RXRAT( 165 )      ! GLY+NO3=HNO3+0.63*HO2+...
     &                          +          RXRAT( 348 )      ! GLY+CL=HCL+0.63*HO2+...

      PROD( AFG1             ) =   0.500 * RXRAT( 161 )      ! RAOOH=0.5*AFG1+HO2+0.5*GLY+...
     &                          +  0.350 * RXRAT( 185 )      ! AFG3+O3=0.35*AFG1+...
     &                          +          RXRAT( 235 )      ! xAFG1=AFG1
     &                          +  0.042 * RXRAT( 358 )      ! IPRD+CL=0.042*AFG1+...
      LOSS( AFG1             ) =           RXRAT( 178 )      ! AFG1+OH=0.217*MACO3+...
     &                          +          RXRAT( 179 )      ! AFG1+O3=0.826*OH+0.522*HO2+...
     &                          +          RXRAT( 180 )      ! AFG1=1.023*HO2+0.173*MEO2+...

      PROD( AFG2             ) =   0.500 * RXRAT( 161 )      ! RAOOH=0.5*AFG2+HO2+0.5*GLY+...
     &                          +  0.350 * RXRAT( 185 )      ! AFG3+O3=0.35*AFG2+...
     &                          +          RXRAT( 237 )      ! xAFG2=AFG2
     &                          +  0.042 * RXRAT( 358 )      ! IPRD+CL=0.042*AFG2+...
      LOSS( AFG2             ) =           RXRAT( 181 )      ! AFG2+OH=0.217*MACO3+...
     &                          +          RXRAT( 182 )      ! AFG2+O3=0.826*OH+0.522*HO2+...
     &                          +          RXRAT( 183 )      ! AFG2=PRD2

      PROD( BACL             ) =           RXRAT( 231 )      ! xBACL=BACL
      LOSS( BACL             ) =           RXRAT( 169 )      ! BACL=2*MECO3

      PROD( BALD             ) =           RXRAT( 233 )      ! xBALD=BALD
      LOSS( BALD             ) =           RXRAT( 175 )      ! BALD+OH=BZCO3
     &                          +          RXRAT( 176 )      ! BALD=
     &                          +          RXRAT( 177 )      ! BALD+NO3=HNO3+BZCO3
     &                          +          RXRAT( 351 )      ! BALD+CL=HCL+BZCO3

      PROD( AFG3             ) =           RXRAT( 239 )      ! xAFG3=AFG3
     &                          +  0.116 * RXRAT( 288 )      ! BENZENE+OH=0.116*AFG3+...
     &                          +  0.312 * RXRAT( 289 )      ! TOLUENE+OH=0.312*AFG3+...
     &                          +  0.239 * RXRAT( 290 )      ! MXYL+OH=0.239*AFG3+...
     &                          +  0.198 * RXRAT( 291 )      ! OXYL+OH=0.198*AFG3+...
     &                          +  0.278 * RXRAT( 292 )      ! PXYL+OH=0.278*AFG3+...
     &                          +  0.230 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.202 * RXRAT( 308 )      ! ARO1+OH=0.202*AFG3+...
     &                          +  0.178 * RXRAT( 309 )      ! ARO2+OH=0.178*AFG3+...
      LOSS( AFG3             ) =           RXRAT( 184 )      ! AFG3+OH=0.206*MACO3+...
     &                          +  0.861 * RXRAT( 185 )      ! AFG3+O3=0.139*AFG3+...

      PROD( xIPRD            ) =   0.231 * RXRAT( 184 )      ! AFG3+OH=0.231*xIPRD+...
     &                          +  0.471 * RXRAT( 274 )      ! BUTADIENE13+OH=0.471*xIPRD+...
     &                          +  0.120 * RXRAT( 276 )      ! BUTADIENE13+NO3=0.12*xIPRD+...
     &                          +  0.357 * RXRAT( 278 )      ! ISOPRENE+OH=0.357*xIPRD+...
     &                          +  0.936 * RXRAT( 280 )      ! ISOPRENE+NO3=0.936*xIPRD+...
     &                          +  0.009 * RXRAT( 300 )      ! OLE1+OH=0.009*xIPRD+...
     &                          +  0.012 * RXRAT( 304 )      ! OLE2+OH=0.012*xIPRD+...
     &                          +  0.002 * RXRAT( 310 )      ! TERP+OH=0.002*xIPRD+...
     &                          +  0.001 * RXRAT( 311 )      ! TERP+O3=0.001*xIPRD+...
     &                          +  0.004 * RXRAT( 312 )      ! TERP+NO3=0.004*xIPRD+...
     &                          +  0.002 * RXRAT( 314 )      ! SESQ+OH=0.002*xIPRD+...
     &                          +  0.001 * RXRAT( 315 )      ! SESQ+O3=0.001*xIPRD+...
     &                          +  0.004 * RXRAT( 316 )      ! SESQ+NO3=0.004*xIPRD+...
     &                          +  0.082 * RXRAT( 356 )      ! MACR+CL=0.082*xIPRD+...
     &                          +  0.115 * RXRAT( 358 )      ! IPRD+CL=0.115*xIPRD+...
     &                          +  0.473 * RXRAT( 372 )      ! BUTADIENE13+CL=0.473*xIPRD+...
     &                          +  0.671 * RXRAT( 373 )      ! ISOPRENE+CL=0.671*xIPRD+...
     &                          +  0.158 * RXRAT( 374 )      ! APIN+CL=0.158*xIPRD+...
     &                          +  0.038 * RXRAT( 387 )      ! OLE1+CL=0.038*xIPRD+...
     &                          +  0.161 * RXRAT( 388 )      ! OLE2+CL=0.161*xIPRD+...
     &                          +  0.158 * RXRAT( 391 )      ! TERP+CL=0.158*xIPRD+...
     &                          +  0.158 * RXRAT( 392 )      ! SESQ+CL=0.158*xIPRD+...
      LOSS( xIPRD            ) =           RXRAT( 245 )      ! xIPRD=IPRD
     &                          +          RXRAT( 246 )      ! xIPRD=

      PROD( MACR             ) =   0.003 * RXRAT( 185 )      ! AFG3+O3=0.003*MACR+...
     &                          +          RXRAT( 241 )      ! xMACR=MACR
     &                          +  0.390 * RXRAT( 279 )      ! ISOPRENE+O3=0.39*MACR+...
      LOSS( MACR             ) =           RXRAT( 186 )      ! MACR+OH=0.5*MACO3+0.5*RO2C+...
     &                          +          RXRAT( 187 )      ! MACR+O3=0.208*OH+0.108*HO2+...
     &                          +          RXRAT( 188 )      ! MACR+NO3=0.5*MACO3+...
     &                          +          RXRAT( 189 )      ! MACR+O3P=RCHO
     &                          +          RXRAT( 190 )      ! MACR=0.33*OH+0.67*HO2+...
     &                          +          RXRAT( 356 )      ! MACR+CL=0.25*HCL+...

      PROD( MVK              ) =   0.004 * RXRAT( 185 )      ! AFG3+O3=0.004*MVK+...
     &                          +          RXRAT( 243 )      ! xMVK=MVK
     &                          +  0.375 * RXRAT( 275 )      ! BUTADIENE13+O3=0.375*MVK+...
     &                          +  0.160 * RXRAT( 279 )      ! ISOPRENE+O3=0.16*MVK+...
      LOSS( MVK              ) =           RXRAT( 191 )      ! MVK+OH=0.975*RO2C+...
     &                          +          RXRAT( 192 )      ! MVK+O3=0.164*OH+0.064*HO2+...
     &                          +          RXRAT( 193 )      ! MVK+O3P=0.45*RCHO+0.55*MEK
     &                          +          RXRAT( 194 )      ! MVK=0.4*MEO2+0.6*CO+...
     &                          +          RXRAT( 357 )      ! MVK+CL=1.283*RO2C+...

      PROD( xHOCCHO          ) =   0.675 * RXRAT( 191 )      ! MVK+OH=0.675*xHOCCHO+...
     &                          +  0.129 * RXRAT( 195 )      ! IPRD+OH=0.129*xHOCCHO+...
     &                          +  0.047 * RXRAT( 196 )      ! IPRD+O3=0.047*xHOCCHO+...
     &                          +  0.195 * RXRAT( 266 )      ! ETHENE+OH=0.195*xHOCCHO+...
     &                          +  0.010 * RXRAT( 294 )      ! ETOH+OH=0.01*xHOCCHO+...
     &                          +  0.007 * RXRAT( 300 )      ! OLE1+OH=0.007*xHOCCHO+...
     &                          +  0.002 * RXRAT( 312 )      ! TERP+NO3=0.002*xHOCCHO+...
     &                          +  0.061 * RXRAT( 381 )      ! ETOH+CL=0.061*xHOCCHO+...
      LOSS( xHOCCHO          ) =           RXRAT( 261 )      ! xHOCCHO=HOCCHO
     &                          +          RXRAT( 262 )      ! xHOCCHO=

      PROD( xRNO3            ) =   0.572 * RXRAT( 197 )      ! IPRD+NO3=0.572*xRNO3+...
     &                          +  0.305 * RXRAT( 201 )      ! RNO3+OH=0.305*xRNO3+...
     &                          +  0.031 * RXRAT( 208 )      ! ACROLEIN+NO3=0.031*xRNO3+...
     &                          +  0.355 * RXRAT( 276 )      ! BUTADIENE13+NO3=...
     &                          +  0.056 * RXRAT( 284 )      ! APIN+NO3=0.056*xRNO3+...
     &                          +  0.774 * RXRAT( 302 )      ! OLE1+NO3=0.774*xRNO3+...
     &                          +  0.320 * RXRAT( 306 )      ! OLE2+NO3=0.32*xRNO3+...
     &                          +  0.228 * RXRAT( 312 )      ! TERP+NO3=0.228*xRNO3+...
     &                          +  0.228 * RXRAT( 316 )      ! SESQ+NO3=0.228*xRNO3+...
     &                          +  0.547 * RXRAT( 346 )      ! RNO3+CL=0.547*xRNO3+...
      LOSS( xRNO3            ) =           RXRAT( 247 )      ! xRNO3=RNO3
     &                          +          RXRAT( 248 )      ! xRNO3=

      PROD( HOCCHO           ) =   0.467 * RXRAT( 198 )      ! IPRD=0.467*HOCCHO+...
     &                          +          RXRAT( 261 )      ! xHOCCHO=HOCCHO
      LOSS( HOCCHO           ) =           RXRAT( 203 )      ! HOCCHO+OH=MECO3
     &                          +          RXRAT( 204 )      ! HOCCHO=CO+2*HO2+HCHO
     &                          +          RXRAT( 205 )      ! HOCCHO+NO3=HNO3+MECO3

      PROD( xACETONE         ) =   0.004 * RXRAT( 201 )      ! RNO3+OH=0.004*xACETONE+...
     &                          +  0.008 * RXRAT( 202 )      ! RNO3=0.008*xACETONE+...
     &                          +  0.034 * RXRAT( 282 )      ! APIN+OH=0.034*xACETONE+...
     &                          +  0.345 * RXRAT( 283 )      ! APIN+O3=0.345*xACETONE+...
     &                          +  0.069 * RXRAT( 284 )      ! APIN+NO3=0.069*xACETONE+...
     &                          +  0.704 * RXRAT( 296 )      ! ALK2+OH=0.704*xACETONE+...
     &                          +  0.024 * RXRAT( 297 )      ! ALK3+OH=0.024*xACETONE+...
     &                          +  0.426 * RXRAT( 298 )      ! ALK4+OH=0.426*xACETONE+...
     &                          +  0.071 * RXRAT( 299 )      ! ALK5+OH=0.071*xACETONE+...
     &                          +  0.007 * RXRAT( 300 )      ! OLE1+OH=0.007*xACETONE+...
     &                          +  0.003 * RXRAT( 301 )      ! OLE1+O3=0.003*xACETONE+...
     &                          +  0.034 * RXRAT( 302 )      ! OLE1+NO3=0.034*xACETONE+...
     &                          +  0.144 * RXRAT( 304 )      ! OLE2+OH=0.144*xACETONE+...
     &                          +  0.116 * RXRAT( 306 )      ! OLE2+NO3=0.116*xACETONE+...
     &                          +  0.037 * RXRAT( 310 )      ! TERP+OH=0.037*xACETONE+...
     &                          +  0.057 * RXRAT( 311 )      ! TERP+O3=0.057*xACETONE+...
     &                          +  0.239 * RXRAT( 312 )      ! TERP+NO3=0.239*xACETONE+...
     &                          +  0.037 * RXRAT( 314 )      ! SESQ+OH=0.037*xACETONE+...
     &                          +  0.057 * RXRAT( 315 )      ! SESQ+O3=0.057*xACETONE+...
     &                          +  0.239 * RXRAT( 316 )      ! SESQ+NO3=0.239*xACETONE+...
     &                          +  0.003 * RXRAT( 346 )      ! RNO3+CL=0.003*xACETONE+...
     &                          +  0.274 * RXRAT( 374 )      ! APIN+CL=0.274*xACETONE+...
     &                          +  0.488 * RXRAT( 383 )      ! ALK2+CL=0.488*xACETONE+...
     &                          +  0.075 * RXRAT( 384 )      ! ALK3+CL=0.075*xACETONE+...
     &                          +  0.247 * RXRAT( 385 )      ! ALK4+CL=0.247*xACETONE+...
     &                          +  0.044 * RXRAT( 386 )      ! ALK5+CL=0.044*xACETONE+...
     &                          +  0.086 * RXRAT( 388 )      ! OLE2+CL=0.086*xACETONE+...
     &                          +  0.274 * RXRAT( 391 )      ! TERP+CL=0.274*xACETONE+...
     &                          +  0.274 * RXRAT( 392 )      ! SESQ+CL=0.274*xACETONE+...
      LOSS( xACETONE         ) =           RXRAT( 221 )      ! xACETONE=ACETONE
     &                          +          RXRAT( 222 )      ! xACETONE=

      PROD( ACROLEIN         ) =           RXRAT( 263 )      ! xACROLEIN=ACROLEIN
     &                          +  0.500 * RXRAT( 275 )      ! BUTADIENE13+O3=...
      LOSS( ACROLEIN         ) =           RXRAT( 206 )      ! ACROLEIN+OH=0.25*xHO2+...
     &                          +          RXRAT( 207 )      ! ACROLEIN+O3=0.83*HO2+...
     &                          +          RXRAT( 208 )      ! ACROLEIN+NO3=0.031*xHO2+...
     &                          +          RXRAT( 209 )      ! ACROLEIN+O3P=RCHO
     &                          +          RXRAT( 210 )      ! ACROLEIN=1.066*HO2+...
     &                          +          RXRAT( 355 )      ! ACROLEIN+CL=0.484*xHO2+...

      PROD( xBALD            ) =   0.065 * RXRAT( 289 )      ! TOLUENE+OH=0.065*xBALD+...
     &                          +  0.041 * RXRAT( 290 )      ! MXYL+OH=0.041*xBALD+...
     &                          +  0.047 * RXRAT( 291 )      ! OXYL+OH=0.047*xBALD+...
     &                          +  0.088 * RXRAT( 292 )      ! PXYL+OH=0.088*xBALD+...
     &                          +  0.036 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.026 * RXRAT( 309 )      ! ARO2+OH=0.026*xBALD+...
     &                          +          RXRAT( 350 )      ! CRES+CL=xBALD+xHO2+HCL+yR6OOH
     &                          +  0.894 * RXRAT( 376 )      ! TOLUENE+CL=0.894*xBALD+...
     &                          +  0.864 * RXRAT( 377 )      ! MXYL+CL=0.864*xBALD+...
     &                          +  0.864 * RXRAT( 378 )      ! OXYL+CL=0.864*xBALD+...
     &                          +  0.864 * RXRAT( 379 )      ! PXYL+CL=0.864*xBALD+...
     &                          +  0.838 * RXRAT( 380 )      ! TRIMETH_BENZ124+CL=...
     &                          +  0.469 * RXRAT( 390 )      ! ARO2+CL=0.469*xBALD+...
      LOSS( xBALD            ) =           RXRAT( 233 )      ! xBALD=BALD
     &                          +          RXRAT( 234 )      ! xBALD=

      PROD( xAFG3            ) =   0.286 * RXRAT( 292 )      ! PXYL+OH=0.286*xAFG3+...
     &                          +  0.151 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +  0.068 * RXRAT( 309 )      ! ARO2+OH=0.068*xAFG3+...
     &                          +  0.001 * RXRAT( 374 )      ! APIN+CL=0.001*xAFG3+...
     &                          +  0.001 * RXRAT( 391 )      ! TERP+CL=0.001*xAFG3+...
     &                          +  0.001 * RXRAT( 392 )      ! SESQ+CL=0.001*xAFG3+...
      LOSS( xAFG3            ) =           RXRAT( 239 )      ! xAFG3=AFG3
     &                          +          RXRAT( 240 )      ! xAFG3=

      PROD( xMACR            ) =   0.230 * RXRAT( 278 )      ! ISOPRENE+OH=0.23*xMACR+...
     &                          +  0.001 * RXRAT( 300 )      ! OLE1+OH=0.001*xMACR+...
     &                          +  0.002 * RXRAT( 311 )      ! TERP+O3=0.002*xMACR+...
     &                          +  0.005 * RXRAT( 312 )      ! TERP+NO3=0.005*xMACR+...
     &                          +  0.002 * RXRAT( 315 )      ! SESQ+O3=0.002*xMACR+...
     &                          +  0.005 * RXRAT( 316 )      ! SESQ+NO3=0.005*xMACR+...
     &                          +  0.025 * RXRAT( 388 )      ! OLE2+CL=0.025*xMACR+...
      LOSS( xMACR            ) =           RXRAT( 241 )      ! xMACR=MACR
     &                          +          RXRAT( 242 )      ! xMACR=

      PROD( xMVK             ) =   0.460 * RXRAT( 276 )      ! BUTADIENE13+NO3=0.46*xMVK+...
     &                          +  0.320 * RXRAT( 278 )      ! ISOPRENE+OH=0.32*xMVK+...
     &                          +  0.012 * RXRAT( 300 )      ! OLE1+OH=0.012*xMVK+...
     &                          +  0.002 * RXRAT( 304 )      ! OLE2+OH=0.002*xMVK+...
     &                          +  0.003 * RXRAT( 310 )      ! TERP+OH=0.003*xMVK+...
     &                          +  0.001 * RXRAT( 312 )      ! TERP+NO3=0.001*xMVK+...
     &                          +  0.003 * RXRAT( 314 )      ! SESQ+OH=0.003*xMVK+...
     &                          +  0.001 * RXRAT( 316 )      ! SESQ+NO3=0.001*xMVK+...
     &                          +  0.177 * RXRAT( 373 )      ! ISOPRENE+CL=0.177*xMVK+...
     &                          +  0.003 * RXRAT( 374 )      ! APIN+CL=0.003*xMVK+...
     &                          +  0.019 * RXRAT( 387 )      ! OLE1+CL=0.019*xMVK+...
     &                          +  0.058 * RXRAT( 388 )      ! OLE2+CL=0.058*xMVK+...
     &                          +  0.003 * RXRAT( 391 )      ! TERP+CL=0.003*xMVK+...
     &                          +  0.003 * RXRAT( 392 )      ! SESQ+CL=0.003*xMVK+...
      LOSS( xMVK             ) =           RXRAT( 243 )      ! xMVK=MVK
     &                          +          RXRAT( 244 )      ! xMVK=

      PROD( yRAOOH           ) =   0.314 * RXRAT( 288 )      ! BENZENE+OH=0.314*yRAOOH+...
     &                          +  0.435 * RXRAT( 289 )      ! TOLUENE+OH=0.435*yRAOOH+...
     &                          +  0.555 * RXRAT( 290 )      ! MXYL+OH=0.555*yRAOOH+...
     &                          +  0.586 * RXRAT( 291 )      ! OXYL+OH=0.586*yRAOOH+...
     &                          +  0.461 * RXRAT( 292 )      ! PXYL+OH=0.461*yRAOOH+...
     &                          +  0.705 * RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
      LOSS( yRAOOH           ) =           RXRAT( 258 )      ! yRAOOH=RAOOH
     &                          +          RXRAT( 259 )      ! yRAOOH=PRD2
     &                          +          RXRAT( 260 )      ! yRAOOH=

      PROD( xACROLEIN        ) =   0.480 * RXRAT( 274 )      ! BUTADIENE13+OH=...
     &                          +  0.115 * RXRAT( 277 )      ! BUTADIENE13+O3P=...
     &                          +  0.036 * RXRAT( 300 )      ! OLE1+OH=0.036*xACROLEIN+...
     &                          +  0.124 * RXRAT( 371 )      ! PROPENE+CL=0.124*xACROLEIN+...
     &                          +  0.457 * RXRAT( 372 )      ! BUTADIENE13+CL=...
     &                          +  0.215 * RXRAT( 387 )      ! OLE1+CL=0.215*xACROLEIN+...
      LOSS( xACROLEIN        ) =           RXRAT( 263 )      ! xACROLEIN=ACROLEIN
     &                          +          RXRAT( 264 )      ! xACROLEIN=

      PROD( ETHENE           ) =   0.0
      LOSS( ETHENE           ) =           RXRAT( 266 )      ! ETHENE+OH=xHO2+RO2C+...
     &                          +          RXRAT( 267 )      ! ETHENE+O3=0.16*HO2+0.16*OH+...
     &                          +          RXRAT( 268 )      ! ETHENE+NO3=xHO2+RO2C+xRCHO+...
     &                          +          RXRAT( 269 )      ! ETHENE+O3P=0.8*HO2+...
     &                          +          RXRAT( 370 )      ! ETHENE+CL=xHO2+2*RO2C+...

      PROD( PROPENE          ) =   0.0
      LOSS( PROPENE          ) =           RXRAT( 270 )      ! PROPENE+OH=0.984*xHO2+...
     &                          +          RXRAT( 271 )      ! PROPENE+O3=0.165*HO2+...
     &                          +          RXRAT( 272 )      ! PROPENE+NO3=0.949*xHO2+...
     &                          +          RXRAT( 273 )      ! PROPENE+O3P=0.45*RCHO+0.55*MEK
     &                          +          RXRAT( 371 )      ! PROPENE+CL=0.124*HCL+...

      PROD( BUTADIENE13      ) =   0.0
      LOSS( BUTADIENE13      ) =           RXRAT( 274 )      ! BUTADIENE13+OH=0.951*xHO2+...
     &                          +          RXRAT( 275 )      ! BUTADIENE13+O3=0.08*HO2+...
     &                          +          RXRAT( 276 )      ! BUTADIENE13+NO3=0.815*xHO2+...
     &                          +          RXRAT( 277 )      ! BUTADIENE13+O3P=0.25*HO2+...
     &                          +          RXRAT( 372 )      ! BUTADIENE13+CL=0.39*xHO2+...

      PROD( ISOPRENE         ) =   0.0
      LOSS( ISOPRENE         ) =           RXRAT( 278 )      ! ISOPRENE+OH=0.907*xHO2+...
     &                          +          RXRAT( 279 )      ! ISOPRENE+O3=0.066*HO2+...
     &                          +          RXRAT( 280 )      ! ISOPRENE+NO3=0.749*xHO2+...
     &                          +          RXRAT( 281 )      ! ISOPRENE+O3P=0.25*MEO2+...
     &                          +          RXRAT( 373 )      ! ISOPRENE+CL=0.15*HCL+...

      PROD( ISOPRXN          ) =           RXRAT( 278 )      ! ISOPRENE+OH=ISOPRXN+...
      LOSS( ISOPRXN          ) =   0.0

      PROD( APIN             ) =   0.0
      LOSS( APIN             ) =           RXRAT( 282 )      ! APIN+OH=0.799*xHO2+...
     &                          +          RXRAT( 283 )      ! APIN+O3=0.009*HO2+...
     &                          +          RXRAT( 284 )      ! APIN+NO3=0.056*xHO2+...
     &                          +          RXRAT( 285 )      ! APIN+O3P=PRD2+TRPRXN
     &                          +          RXRAT( 374 )      ! APIN+CL=0.548*HCL+...

      PROD( TRPRXN           ) =           RXRAT( 282 )      ! APIN+OH=TRPRXN+0.004*xRCO3+...
     &                          +          RXRAT( 283 )      ! APIN+O3=TRPRXN+0.102*xHO2+...
     &                          +          RXRAT( 284 )      ! APIN+NO3=TRPRXN+0.643*xNO2+...
     &                          +          RXRAT( 285 )      ! APIN+O3P=TRPRXN+PRD2
     &                          +          RXRAT( 310 )      ! TERP+OH=TRPRXN+0.064*xRCO3+...
     &                          +          RXRAT( 311 )      ! TERP+O3=TRPRXN+0.046*xHO2+...
     &                          +          RXRAT( 312 )      ! TERP+NO3=TRPRXN+0.287*xNO2+...
     &                          +          RXRAT( 313 )      ! TERP+O3P=TRPRXN+0.763*PRD2+...
      LOSS( TRPRXN           ) =   0.0

      PROD( ACETYLENE        ) =   0.0
      LOSS( ACETYLENE        ) =           RXRAT( 286 )      ! ACETYLENE+OH=0.3*HO2+...
     &                          +          RXRAT( 287 )      ! ACETYLENE+O3=1.5*HO2+...
     &                          +          RXRAT( 375 )      ! ACETYLENE+CL=HO2+CO

      PROD( BENZENE          ) =   0.0
      LOSS( BENZENE          ) =           RXRAT( 288 )      ! BENZENE+OH=0.57*HO2+...

      PROD( BENZRO2          ) =           RXRAT( 288 )      ! BENZENE+OH=BENZRO2+...
      LOSS( BENZRO2          ) =           RXRAT( 393 )      ! BENZRO2+NO=NO+BNZNRXN
     &                          +          RXRAT( 394 )      ! BENZRO2+HO2=HO2+BNZHRXN

      PROD( TOLUENE          ) =   0.0
      LOSS( TOLUENE          ) =           RXRAT( 289 )      ! TOLUENE+OH=0.181*HO2+...
     &                          +          RXRAT( 376 )      ! TOLUENE+CL=0.894*xHO2+...

      PROD( TOLRO2           ) =           RXRAT( 289 )      ! TOLUENE+OH=TOLRO2+...
     &                          +          RXRAT( 308 )      ! ARO1+OH=TOLRO2+0.566*xHO2+...
      LOSS( TOLRO2           ) =           RXRAT( 397 )      ! TOLRO2+NO=NO+TOLNRXN
     &                          +          RXRAT( 398 )      ! TOLRO2+HO2=HO2+TOLHRXN

      PROD( MXYL             ) =   0.0
      LOSS( MXYL             ) =           RXRAT( 290 )      ! MXYL+OH=0.159*HO2+...
     &                          +          RXRAT( 377 )      ! MXYL+CL=0.864*xHO2+...

      PROD( XYLRO2           ) =           RXRAT( 290 )      ! MXYL+OH=XYLRO2+0.52*xHO2+...
     &                          +          RXRAT( 291 )      ! OXYL+OH=XYLRO2+0.554*xHO2+...
     &                          +          RXRAT( 292 )      ! PXYL+OH=XYLRO2+0.487*xHO2+...
     &                          +          RXRAT( 293 )      ! TRIMETH_BENZ124+OH=XYLRO2+...
     &                          +          RXRAT( 309 )      ! ARO2+OH=XYLRO2+0.617*xHO2+...
      LOSS( XYLRO2           ) =           RXRAT( 395 )      ! XYLRO2+NO=NO+XYLNRXN
     &                          +          RXRAT( 396 )      ! XYLRO2+HO2=HO2+XYLHRXN

      PROD( OXYL             ) =   0.0
      LOSS( OXYL             ) =           RXRAT( 291 )      ! OXYL+OH=0.161*HO2+...
     &                          +          RXRAT( 378 )      ! OXYL+CL=0.864*xHO2+...

      PROD( PXYL             ) =   0.0
      LOSS( PXYL             ) =           RXRAT( 292 )      ! PXYL+OH=0.159*HO2+...
     &                          +          RXRAT( 379 )      ! PXYL+CL=0.864*xHO2+...

      PROD( TRIMETH_BENZ124  ) =   0.0
      LOSS( TRIMETH_BENZ124  ) =           RXRAT( 293 )      ! TRIMETH_BENZ124+OH=...
     &                          +          RXRAT( 380 )      ! TRIMETH_BENZ124+CL=...

      PROD( ETOH             ) =   0.0
      LOSS( ETOH             ) =           RXRAT( 294 )      ! ETOH+OH=0.95*HO2+0.05*xHO2+...
     &                          +          RXRAT( 381 )      ! ETOH+CL=HCL+0.688*HO2+...

      PROD( ALK1             ) =   0.0
      LOSS( ALK1             ) =           RXRAT( 295 )      ! ALK1+OH=xHO2+RO2C+xCCHO+yROOH
     &                          +          RXRAT( 382 )      ! ALK1+CL=HCL+xHO2+RO2C+...

      PROD( ALK2             ) =   0.0
      LOSS( ALK2             ) =           RXRAT( 296 )      ! ALK2+OH=0.965*xHO2+...
     &                          +          RXRAT( 383 )      ! ALK2+CL=HCL+0.97*xHO2+...

      PROD( ALK3             ) =   0.0
      LOSS( ALK3             ) =           RXRAT( 297 )      ! ALK3+OH=0.695*xHO2+...
     &                          +          RXRAT( 384 )      ! ALK3+CL=HCL+0.835*xHO2+...

      PROD( ALK4             ) =   0.0
      LOSS( ALK4             ) =           RXRAT( 298 )      ! ALK4+OH=0.83*xHO2+...
     &                          +          RXRAT( 385 )      ! ALK4+CL=HCL+0.827*xHO2+...

      PROD( ALK5             ) =   0.0
      LOSS( ALK5             ) =           RXRAT( 299 )      ! ALK5+OH=0.647*xHO2+...
     &                          +          RXRAT( 386 )      ! ALK5+CL=HCL+0.647*xHO2+...

      PROD( ALK5RXN          ) =           RXRAT( 299 )      ! ALK5+OH=ALK5RXN+1.605*RO2C+...
      LOSS( ALK5RXN          ) =   0.0

      PROD( OLE1             ) =   0.0
      LOSS( OLE1             ) =           RXRAT( 300 )      ! OLE1+OH=0.871*xHO2+...
     &                          +          RXRAT( 301 )      ! OLE1+O3=0.095*HO2+...
     &                          +          RXRAT( 302 )      ! OLE1+NO3=0.772*xHO2+...
     &                          +          RXRAT( 303 )      ! OLE1+O3P=0.45*RCHO+...
     &                          +          RXRAT( 387 )      ! OLE1+CL=0.384*HCL+...

      PROD( OLE2             ) =   0.0
      LOSS( OLE2             ) =           RXRAT( 304 )      ! OLE2+OH=0.912*xHO2+...
     &                          +          RXRAT( 305 )      ! OLE2+O3=0.094*HO2+...
     &                          +          RXRAT( 306 )      ! OLE2+NO3=0.4*xHO2+...
     &                          +          RXRAT( 307 )      ! OLE2+O3P=0.079*RCHO+...
     &                          +          RXRAT( 388 )      ! OLE2+CL=0.279*HCL+...

      PROD( ARO1             ) =   0.0
      LOSS( ARO1             ) =           RXRAT( 308 )      ! ARO1+OH=0.123*HO2+...
     &                          +          RXRAT( 389 )      ! ARO1+CL=0.84*xHO2+...

      PROD( ARO2             ) =   0.0
      LOSS( ARO2             ) =           RXRAT( 309 )      ! ARO2+OH=0.077*HO2+...
     &                          +          RXRAT( 390 )      ! ARO2+CL=0.828*xHO2+...

      PROD( TERP             ) =   0.0
      LOSS( TERP             ) =           RXRAT( 310 )      ! TERP+OH=0.734*xHO2+...
     &                          +          RXRAT( 311 )      ! TERP+O3=0.078*HO2+...
     &                          +          RXRAT( 312 )      ! TERP+NO3=0.227*xHO2+...
     &                          +          RXRAT( 313 )      ! TERP+O3P=0.237*RCHO+...
     &                          +          RXRAT( 391 )      ! TERP+CL=0.548*HCL+...

      PROD( SESQ             ) =   0.0
      LOSS( SESQ             ) =           RXRAT( 314 )      ! SESQ+OH=0.734*xHO2+...
     &                          +          RXRAT( 315 )      ! SESQ+O3=0.078*HO2+...
     &                          +          RXRAT( 316 )      ! SESQ+NO3=0.227*xHO2+...
     &                          +          RXRAT( 317 )      ! SESQ+O3P=0.237*RCHO+...
     &                          +          RXRAT( 392 )      ! SESQ+CL=0.252*xHO2+...

      PROD( SESQRXN          ) =           RXRAT( 314 )      ! SESQ+OH=SESQRXN+...
     &                          +          RXRAT( 315 )      ! SESQ+O3=SESQRXN+0.046*xHO2+...
     &                          +          RXRAT( 316 )      ! SESQ+NO3=SESQRXN+...
     &                          +          RXRAT( 317 )      ! SESQ+O3P=SESQRXN+...
      LOSS( SESQRXN          ) =   0.0

      PROD( CL2              ) =           RXRAT( 334 )      ! CL+CLONO2=CL2+NO3
     &                          +  0.290 * RXRAT( 337 )      ! CLO+CLO=0.29*CL2+1.42*CL
      LOSS( CL2              ) =           RXRAT( 318 )      ! CL2=2*CL

      PROD( CL               ) =   2.000 * RXRAT( 318 )      ! CL2=2*CL
     &                          +          RXRAT( 320 )      ! CLNO=CL+NO
     &                          +          RXRAT( 323 )      ! CLONO=CL+NO2
     &                          +          RXRAT( 324 )      ! CLNO2=CL+NO2
     &                          +          RXRAT( 329 )      ! CLO+NO=CL+NO2
     &                          +          RXRAT( 332 )      ! CLONO2=CL+NO3
     &                          +          RXRAT( 336 )      ! HOCL=CL+OH
     &                          +  1.420 * RXRAT( 337 )      ! CLO+CLO=1.42*CL+0.29*CL2
     &                          +          RXRAT( 338 )      ! OH+HCL=CL
     &                          +          RXRAT( 363 )      ! xCL=CL
      LOSS( CL               ) =           RXRAT( 319 )      ! CL+NO=CLNO
     &                          +          RXRAT( 321 )      ! CL+NO2=CLONO
     &                          +          RXRAT( 322 )      ! CL+NO2=CLNO2
     &                          +          RXRAT( 325 )      ! CL+HO2=HCL
     &                          +          RXRAT( 326 )      ! CL+HO2=CLO+OH
     &                          +          RXRAT( 327 )      ! CL+O3=CLO
     &                          +          RXRAT( 328 )      ! CL+NO3=CLO+NO2
     &                          +          RXRAT( 334 )      ! CL+CLONO2=CL2+NO3
     &                          +          RXRAT( 339 )      ! CL=HCL+HO2
     &                          +          RXRAT( 340 )      ! CL+HCHO=HCL+HO2+CO
     &                          +          RXRAT( 341 )      ! CL+CCHO=HCL+MECO3
     &                          +          RXRAT( 342 )      ! CL+MEOH=HCL+HCHO+HO2
     &                          +          RXRAT( 343 )      ! CL+RCHO=HCL+0.9*RCO3+...
     &                          +          RXRAT( 344 )      ! CL+ACETONE=HCL+RO2C+xHCHO+...
     &                          +          RXRAT( 345 )      ! CL+MEK=HCL+0.975*RO2C+...
     &                          +          RXRAT( 346 )      ! CL+RNO3=HCL+0.038*NO2+...
     &                          +          RXRAT( 347 )      ! CL+PRD2=HCL+0.314*HO2+...
     &                          +          RXRAT( 348 )      ! CL+GLY=HCL+0.63*HO2+...
     &                          +          RXRAT( 349 )      ! CL+MGLY=HCL+CO+MECO3
     &                          +          RXRAT( 350 )      ! CL+CRES=HCL+xHO2+xBALD+yR6OOH
     &                          +          RXRAT( 351 )      ! CL+BALD=HCL+BZCO3
     &                          +          RXRAT( 352 )      ! CL+ROOH=HCL+0.414*OH+...
     &                          +          RXRAT( 353 )      ! CL+R6OOH=HCL+0.145*OH+...
     &                          +          RXRAT( 354 )      ! CL+RAOOH=0.404*HCL+...
     &                          +          RXRAT( 355 )      ! CL+ACROLEIN=0.484*xHO2+...
     &                          +          RXRAT( 356 )      ! CL+MACR=0.25*HCL+...
     &                          +          RXRAT( 357 )      ! CL+MVK=1.283*RO2C+...
     &                          +          RXRAT( 358 )      ! CL+IPRD=0.401*HCL+...
     &                          +          RXRAT( 361 )      ! CL+CLCCHO=HCL+RCO3
     &                          +          RXRAT( 369 )      ! CL=HCL+MEO2
     &                          +          RXRAT( 370 )      ! CL+ETHENE=xHO2+2*RO2C+...
     &                          +          RXRAT( 371 )      ! CL+PROPENE=0.124*HCL+...
     &                          +          RXRAT( 372 )      ! CL+BUTADIENE13=0.39*xHO2+...
     &                          +          RXRAT( 373 )      ! CL+ISOPRENE=0.15*HCL+...
     &                          +          RXRAT( 374 )      ! CL+APIN=0.548*HCL+...
     &                          +          RXRAT( 375 )      ! CL+ACETYLENE=HO2+CO
     &                          +          RXRAT( 376 )      ! CL+TOLUENE=0.894*xHO2+...
     &                          +          RXRAT( 377 )      ! CL+MXYL=0.864*xHO2+...
     &                          +          RXRAT( 378 )      ! CL+OXYL=0.864*xHO2+...
     &                          +          RXRAT( 379 )      ! CL+PXYL=0.864*xHO2+...
     &                          +          RXRAT( 380 )      ! CL+TRIMETH_BENZ124=...
     &                          +          RXRAT( 381 )      ! CL+ETOH=HCL+0.688*HO2+...
     &                          +          RXRAT( 382 )      ! CL+ALK1=HCL+xHO2+RO2C+...
     &                          +          RXRAT( 383 )      ! CL+ALK2=HCL+0.97*xHO2+...
     &                          +          RXRAT( 384 )      ! CL+ALK3=HCL+0.835*xHO2+...
     &                          +          RXRAT( 385 )      ! CL+ALK4=HCL+0.827*xHO2+...
     &                          +          RXRAT( 386 )      ! CL+ALK5=HCL+0.647*xHO2+...
     &                          +          RXRAT( 387 )      ! CL+OLE1=0.384*HCL+...
     &                          +          RXRAT( 388 )      ! CL+OLE2=0.279*HCL+...
     &                          +          RXRAT( 389 )      ! CL+ARO1=0.84*xHO2+...
     &                          +          RXRAT( 390 )      ! CL+ARO2=0.828*xHO2+...
     &                          +          RXRAT( 391 )      ! CL+TERP=0.548*HCL+...
     &                          +          RXRAT( 392 )      ! CL+SESQ=0.252*xHO2+...

      PROD( CLNO             ) =           RXRAT( 319 )      ! CL+NO=CLNO
      LOSS( CLNO             ) =           RXRAT( 320 )      ! CLNO=CL+NO

      PROD( CLONO            ) =           RXRAT( 321 )      ! CL+NO2=CLONO
      LOSS( CLONO            ) =           RXRAT( 323 )      ! CLONO=CL+NO2

      PROD( CLNO2            ) =           RXRAT( 322 )      ! CL+NO2=CLNO2
      LOSS( CLNO2            ) =           RXRAT( 324 )      ! CLNO2=CL+NO2

      PROD( HCL              ) =           RXRAT( 325 )      ! CL+HO2=HCL
     &                          +          RXRAT( 339 )      ! CL=HCL+HO2
     &                          +          RXRAT( 340 )      ! HCHO+CL=HCL+HO2+CO
     &                          +          RXRAT( 341 )      ! CCHO+CL=HCL+MECO3
     &                          +          RXRAT( 342 )      ! MEOH+CL=HCL+HCHO+HO2
     &                          +          RXRAT( 343 )      ! RCHO+CL=HCL+0.9*RCO3+...
     &                          +          RXRAT( 344 )      ! ACETONE+CL=HCL+RO2C+xHCHO+...
     &                          +          RXRAT( 345 )      ! MEK+CL=HCL+0.975*RO2C+...
     &                          +          RXRAT( 346 )      ! RNO3+CL=HCL+0.038*NO2+...
     &                          +          RXRAT( 347 )      ! PRD2+CL=HCL+0.314*HO2+...
     &                          +          RXRAT( 348 )      ! GLY+CL=HCL+0.63*HO2+...
     &                          +          RXRAT( 349 )      ! MGLY+CL=HCL+CO+MECO3
     &                          +          RXRAT( 350 )      ! CRES+CL=HCL+xHO2+xBALD+yR6OOH
     &                          +          RXRAT( 351 )      ! BALD+CL=HCL+BZCO3
     &                          +          RXRAT( 352 )      ! ROOH+CL=HCL+0.414*OH+...
     &                          +          RXRAT( 353 )      ! R6OOH+CL=HCL+0.145*OH+...
     &                          +  0.404 * RXRAT( 354 )      ! RAOOH+CL=0.404*HCL+...
     &                          +  0.216 * RXRAT( 355 )      ! ACROLEIN+CL=0.216*HCL+...
     &                          +  0.250 * RXRAT( 356 )      ! MACR+CL=0.25*HCL+...
     &                          +  0.401 * RXRAT( 358 )      ! IPRD+CL=0.401*HCL+...
     &                          +          RXRAT( 361 )      ! CLCCHO+CL=HCL+RCO3
     &                          +          RXRAT( 369 )      ! CL=HCL+MEO2
     &                          +  0.124 * RXRAT( 371 )      ! PROPENE+CL=0.124*HCL+...
     &                          +  0.150 * RXRAT( 373 )      ! ISOPRENE+CL=0.15*HCL+...
     &                          +  0.548 * RXRAT( 374 )      ! APIN+CL=0.548*HCL+...
     &                          +          RXRAT( 381 )      ! ETOH+CL=HCL+0.688*HO2+...
     &                          +          RXRAT( 382 )      ! ALK1+CL=HCL+xHO2+RO2C+...
     &                          +          RXRAT( 383 )      ! ALK2+CL=HCL+0.97*xHO2+...
     &                          +          RXRAT( 384 )      ! ALK3+CL=HCL+0.835*xHO2+...
     &                          +          RXRAT( 385 )      ! ALK4+CL=HCL+0.827*xHO2+...
     &                          +          RXRAT( 386 )      ! ALK5+CL=HCL+0.647*xHO2+...
     &                          +  0.384 * RXRAT( 387 )      ! OLE1+CL=0.384*HCL+...
     &                          +  0.279 * RXRAT( 388 )      ! OLE2+CL=0.279*HCL+...
     &                          +  0.548 * RXRAT( 391 )      ! TERP+CL=0.548*HCL+...
     &                          +  0.548 * RXRAT( 392 )      ! SESQ+CL=0.548*HCL+...
      LOSS( HCL              ) =           RXRAT( 338 )      ! HCL+OH=CL

      PROD( CLO              ) =           RXRAT( 326 )      ! CL+HO2=CLO+OH
     &                          +          RXRAT( 327 )      ! CL+O3=CLO
     &                          +          RXRAT( 328 )      ! CL+NO3=CLO+NO2
     &                          +          RXRAT( 331 )      ! CLONO2=CLO+NO2
     &                          +          RXRAT( 333 )      ! CLONO2=CLO+NO2
      LOSS( CLO              ) =           RXRAT( 329 )      ! CLO+NO=CL+NO2
     &                          +          RXRAT( 330 )      ! CLO+NO2=CLONO2
     &                          +          RXRAT( 335 )      ! CLO+HO2=HOCL
     &                          +  2.000 * RXRAT( 337 )      ! CLO+CLO=0.29*CL2+1.42*CL

      PROD( CLONO2           ) =           RXRAT( 330 )      ! CLO+NO2=CLONO2
      LOSS( CLONO2           ) =           RXRAT( 331 )      ! CLONO2=CLO+NO2
     &                          +          RXRAT( 332 )      ! CLONO2=CL+NO3
     &                          +          RXRAT( 333 )      ! CLONO2=CLO+NO2
     &                          +          RXRAT( 334 )      ! CLONO2+CL=CL2+NO3

      PROD( HOCL             ) =           RXRAT( 335 )      ! CLO+HO2=HOCL
      LOSS( HOCL             ) =           RXRAT( 336 )      ! HOCL=OH+CL

      PROD( xCL              ) =   0.274 * RXRAT( 355 )      ! ACROLEIN+CL=0.274*xCL+...
     &                          +          RXRAT( 359 )      ! CLCCHO=xCL+CO+RO2C+HO2+...
     &                          +          RXRAT( 362 )      ! CLACET=xCL+RO2C+MECO3+...
     &                          +  0.541 * RXRAT( 372 )      ! BUTADIENE13+CL=0.541*xCL+...
     &                          +  0.177 * RXRAT( 373 )      ! ISOPRENE+CL=0.177*xCL+...
     &                          +  0.068 * RXRAT( 374 )      ! APIN+CL=0.068*xCL+...
     &                          +  0.442 * RXRAT( 388 )      ! OLE2+CL=0.442*xCL+...
     &                          +  0.068 * RXRAT( 391 )      ! TERP+CL=0.068*xCL+...
     &                          +  0.068 * RXRAT( 392 )      ! SESQ+CL=0.068*xCL+...
      LOSS( xCL              ) =           RXRAT( 363 )      ! xCL=CL
     &                          +          RXRAT( 364 )      ! xCL=

      PROD( xCLCCHO          ) =   0.484 * RXRAT( 355 )      ! ACROLEIN+CL=0.484*xCLCCHO+...
     &                          +  0.180 * RXRAT( 356 )      ! MACR+CL=0.18*xCLCCHO+...
     &                          +  0.947 * RXRAT( 357 )      ! MVK+CL=0.947*xCLCCHO+...
     &                          +  0.140 * RXRAT( 358 )      ! IPRD+CL=0.14*xCLCCHO+...
     &                          +  0.306 * RXRAT( 371 )      ! PROPENE+CL=0.306*xCLCCHO+...
     &                          +  0.067 * RXRAT( 373 )      ! ISOPRENE+CL=0.067*xCLCCHO+...
     &                          +  0.109 * RXRAT( 374 )      ! APIN+CL=0.109*xCLCCHO+...
     &                          +  0.192 * RXRAT( 387 )      ! OLE1+CL=0.192*xCLCCHO+...
     &                          +  0.013 * RXRAT( 388 )      ! OLE2+CL=0.013*xCLCCHO+...
     &                          +  0.109 * RXRAT( 391 )      ! TERP+CL=0.109*xCLCCHO+...
     &                          +  0.109 * RXRAT( 392 )      ! SESQ+CL=0.109*xCLCCHO+...
      LOSS( xCLCCHO          ) =           RXRAT( 365 )      ! xCLCCHO=CLCCHO
     &                          +          RXRAT( 366 )      ! xCLCCHO=

      PROD( xCLACET          ) =   0.541 * RXRAT( 356 )      ! MACR+CL=0.541*xCLACET+...
     &                          +  0.420 * RXRAT( 358 )      ! IPRD+CL=0.42*xCLACET+...
     &                          +  0.540 * RXRAT( 371 )      ! PROPENE+CL=0.54*xCLACET+...
     &                          +  0.337 * RXRAT( 387 )      ! OLE1+CL=0.337*xCLACET+...
     &                          +  0.191 * RXRAT( 388 )      ! OLE2+CL=0.191*xCLACET+...
      LOSS( xCLACET          ) =           RXRAT( 367 )      ! xCLACET=CLACET
     &                          +          RXRAT( 368 )      ! xCLACET=

      PROD( CLCCHO           ) =           RXRAT( 365 )      ! xCLCCHO=CLCCHO
      LOSS( CLCCHO           ) =           RXRAT( 359 )      ! CLCCHO=HO2+CO+RO2C+xCL+...
     &                          +          RXRAT( 360 )      ! CLCCHO+OH=RCO3
     &                          +          RXRAT( 361 )      ! CLCCHO+CL=HCL+RCO3

      PROD( CLACET           ) =           RXRAT( 367 )      ! xCLACET=CLACET
      LOSS( CLACET           ) =           RXRAT( 362 )      ! CLACET=MECO3+RO2C+xCL+...

      PROD( CLCHO            ) =           RXRAT( 370 )      ! ETHENE+CL=CLCHO+2*RO2C+...
      LOSS( CLCHO            ) =   0.0

      PROD( BNZNRXN          ) =           RXRAT( 393 )      ! BENZRO2+NO=BNZNRXN+NO
      LOSS( BNZNRXN          ) =   0.0

      PROD( BNZHRXN          ) =           RXRAT( 394 )      ! BENZRO2+HO2=BNZHRXN+HO2
      LOSS( BNZHRXN          ) =   0.0

      PROD( XYLNRXN          ) =           RXRAT( 395 )      ! XYLRO2+NO=XYLNRXN+NO
      LOSS( XYLNRXN          ) =   0.0

      PROD( XYLHRXN          ) =           RXRAT( 396 )      ! XYLRO2+HO2=XYLHRXN+HO2
      LOSS( XYLHRXN          ) =   0.0

      PROD( TOLNRXN          ) =           RXRAT( 397 )      ! TOLRO2+NO=TOLNRXN+NO
      LOSS( TOLNRXN          ) =   0.0

      PROD( TOLHRXN          ) =           RXRAT( 398 )      ! TOLRO2+HO2=TOLHRXN+HO2
      LOSS( TOLHRXN          ) =   0.0

      PROD( HCHO_PRIMARY     ) =   0.0
      LOSS( HCHO_PRIMARY     ) =           RXRAT( 399 )      ! HCHO_PRIMARY=
     &                          +          RXRAT( 400 )      ! HCHO_PRIMARY=
     &                          +          RXRAT( 401 )      ! HCHO_PRIMARY+OH=OH
     &                          +          RXRAT( 402 )      ! HCHO_PRIMARY+NO3=NO3
     &                          +          RXRAT( 403 )      ! HCHO_PRIMARY+CL=CL

      PROD( CCHO_PRIMARY     ) =   0.0
      LOSS( CCHO_PRIMARY     ) =           RXRAT( 404 )      ! CCHO_PRIMARY+OH=OH
     &                          +          RXRAT( 405 )      ! CCHO_PRIMARY=
     &                          +          RXRAT( 406 )      ! CCHO_PRIMARY+NO3=NO3
     &                          +          RXRAT( 407 )      ! CCHO_PRIMARY+CL=CL

      PROD( ACROLEIN_PRIMARY ) =   0.0
      LOSS( ACROLEIN_PRIMARY ) =           RXRAT( 408 )      ! ACROLEIN_PRIMARY+OH=OH
     &                          +          RXRAT( 409 )      ! ACROLEIN_PRIMARY+O3=O3
     &                          +          RXRAT( 410 )      ! ACROLEIN_PRIMARY+NO3=NO3
     &                          +          RXRAT( 411 )      ! ACROLEIN_PRIMARY+O3P=O3P
     &                          +          RXRAT( 412 )      ! ACROLEIN_PRIMARY=
     &                          +          RXRAT( 413 )      ! ACROLEIN_PRIMARY+CL=CL

      RETURN

      END
